/*global window, XMLHttpRequest, console */
/**
 * Modul do zapisywania bledow JS'owy po stronie serwera. W przpypadku blednej
 * odpowiedzi zapisu, bledy zapisywane sa w localstorage.
 * 
 * @param {window Object} win
 * @returns {undefined}
 */
(function (win) {
    'use strict';
    function ErrorReporting() {
        throw new TypeError('Invalid constructor');
    }
    ErrorReporting.prototype = (function () {
        var userAgent = win.navigator.userAgent,
            screen = win.screen || {},
            pageDate = new Date(),
            pageTimestamp = pageDate.getTime(),
            navigator = win.navigator,
            document = win.document,
            er = win.errorReporting || {},
            errorReportingUrl = er.url,
            reportingEnabled = er.enabled || false,
            SAME_ERROR_LIMIT = 1,
            loggedErrors = [];
        return {
            /**
             * @type {Boolean}
             */
            enabled: reportingEnabled,
            /**
             * @type {Boolean}
             */
            ajaxErrorEnabled: false,
            /**
             * Funkcja inicjalizujaca
             * @returns {undefined}
             */
            init: function () {
                this.addListeners();
            },
            /**
             * Dodanie listenerow
             * @returns {undefined}
             */
            addListeners: function () {
                var ua = window.navigator.userAgent,
                    msie = ua.indexOf('MSIE ');

                if (msie > 0 || !!navigator.userAgent.match(/Trident.*rv\:11\./)) {
                    // specjalny wyjatek dla IE bo jest ...
                    win.onerror = function (message, file, line, col, error) {
                        error = error || {};
                        error.message = message;
                        error.filename = file;
                        error.lineno = line;
                        error.colno = col;
                        error.stack = (error.stack || error.stacktrace || '').toString();
                        this.handleError(error);
                    }.bind(this);
                } else {
                    // w przypadku innych przegladarek
                    win.addEventListener('error', this.handleError.bind(this), true);
                }
                
            },
            /**
             * Zdarzenie obslugi bledow
             * @param {Object} error
             * @returns {Boolean}
             */
            handleError: function (error) {
                error = error || {};
                var errorObject,
                    errorObjectString,
                    message = error.message || error.description || error.error || '',
                    url = error.filename || error.fileName || '',
                    line = error.lineno || error.lineNumber || error.number,
                    column = error.colno || error.columnNumber,
                    stack = (error.stack || error.stacktrace || '').toString(),
                    tagName = error.target ? (error.target.tagName || '-') : 'window',
                    tagSrc = error.target ? error.target.src || error.target.href || '' : '';

                if (tagName) {
                    stack += "\nTAG_NAME: " + tagName;
                }
                if (tagSrc) {
                    stack += "\nTAG_SRC: " + tagSrc;
                }

                if (tagName === 'SCRIPT' && tagSrc && tagSrc.indexOf) {
                    // logujemy bledy o niezaladowaniu skryptow tylko z naszych domen
                    if (tagSrc.indexOf('cloudfront') === -1 && tagSrc.indexOf('clickmeeting') === -1 && tagSrc.indexOf('clickwebinar') === -1) {
                        return;
                    }
                } else if (tagName === 'IMG' || tagName === 'OBJECT') {
                    // nie logujemy bledow o obrazkach i obiektach
                    return;
                }
                
                if (this.enabled === false) {
                    return false;
                }
                if (this.canReadAndLogError(message) === false) {
                    return false;
                }
                loggedErrors.push(message);
                errorObject = this.prepareErrorObject(message, url, line, column, stack, tagName, tagSrc);
                try {
                    errorObjectString = JSON.stringify([errorObject]);
                    this.send(errorObjectString);
                    return false;
                } catch (e) {
                    return false;
                }
                return false;
            },
            /**
             * Metoda raportujaca bledy ajaxowe
             * 
             * @param {jQuery ajax error object} error
             * @returns {undefined}
             */
            handleAjaxError: function (error) {
                if (!(error && typeof error === 'object')) {
                    error = {};
                }
                if (!this.ajaxErrorEnabled) {
                    return;
                }
                var errorMessage = {
                    responseText: error.responseText,
                    statusText: error.statusText,
                    status: error.status
                };
                try {
                    errorMessage = JSON.stringify(errorMessage);
                } catch (e) {
                    errorMessage = '';
                }
                if (!errorMessage) {
                    return;
                }
                this.handleError(errorMessage);
            },
            /**
             * 
             * @param {String} message
             * @param {String} url
             * @param {Number} line
             * @param {Number} column
             * @param {String} stack
             * @param {String} tagName
             * @param {String} tagSrc
             * @returns {Object}
             */
            prepareErrorObject: function (message, url, line, column, stack, tagName, tagSrc) {
                var date = new Date(),
                    timestamp = date.getTime(),
                    plugins = this.getPlugins(),
                    errorObject = {
                        message: message,
                        url: url,
                        line: line,
                        column: column,
                        user_agent: userAgent,
                        page_date: pageDate,
                        page_timestamp: pageTimestamp,
                        date: date,
                        timestamp: timestamp,
                        browser_language: navigator.language,
                        screen_width: screen.width,
                        screen_height: screen.height,
                        screen_avail_width: screen.availWidth,
                        screen_avail_height: screen.availHeight,
                        screen_color_depth: screen.colorDepth,
                        screen_pixel_depth: screen.pixelDepth,
                        www_url: encodeURIComponent(win.location.href),
                        referrer: encodeURIComponent(document.referrer),
                        plugins: plugins,
                        stack: stack,
                        tag_name: tagName,
                        tag_src: tagSrc
                    };
                return errorObject;
            },
            /**
             * Funkcja wysylajaca bledy JS'owe do backendu
             * 
             * @param {String} errorsToSend
             * @returns {Boolean}
             */
            send: function (errorsToSend) {
                var req;
                if (this.enabled === false) {
                    return false;
                }
                if (!errorReportingUrl) {
                    return false;
                }
                req = new XMLHttpRequest();
                req.open('POST', errorReportingUrl, true);
                req.onreadystatechange = function () {
                    if (req.readyState === 4) {
                        if (req.status === 200) {
                            this.onSendSuccess();
                        }
                    }
                }.bind(this);
                if (errorsToSend) {
                    req.send('errors=' + errorsToSend);
                }
            },
            /**
             * Callback wywolywany po otrzymaniu odpowiedzi
             * 
             * @returns {undefined}
             */
            onSendSuccess: function () {
                return null;
            },
            /**
             * Wlaczenie zapisywania bledow
             * @returns {undefined}
             */
            enable: function () {
                this.enabled = true;
            },
            /**
             * Wylaczenie zapisywania bledow
             * @returns {undefined}
             */
            disable: function () {
                this.enabled = false;
            },
            /**
             * Sprawdza czy mozna odczytac informacje o bledzie
             * @param {String} errorMessage
             * @returns {Boolean}
             */
            canReadAndLogError: function (errorMessage) {
                errorMessage = errorMessage || '';
                if (errorMessage.indexOf('Location.toString') > -1) {
                    return false;
                }
                if (errorMessage.indexOf('Error loading script') > -1) {
                    return false;
                }
                if (errorMessage.indexOf('Script error.') > -1) {
                    return false;
                }
                if (this.sameErrorLimitReached(errorMessage)) {
                    return false;
                }
                return true;
            },
            /**
             * Sprawdza czy limit bledow tego samego typu zostal przekroczony
             * 
             * @param {String} errorMessage
             * @returns {Boolean}
             */
            sameErrorLimitReached: function (errorMessage) {
                var count = this.getErrorCountForMessage(errorMessage);
                if (count >= SAME_ERROR_LIMIT) {
                    return true;
                }
                return false;
            },
            /**
             * Sprawdza liczbe zalogowany bledow dla danego message'a
             * 
             * @param {String} errorMessage
             * @returns {Number}
             */
            getErrorCountForMessage: function (errorMessage) {
                var count = 0;
                loggedErrors.forEach(function (errMsg) {
                    if (errMsg === errorMessage) {
                        count += 1;
                    }
                });
                return count;
            },
            /**
             * Pobiera pluginy
             * 
             * @returns {Array}
             */
            getPlugins: function () {
                var plugins = navigator.plugins,
                    pluginId,
                    plugin,
                    pluginData = [],
                    pData;
                try {
                    for (pluginId in plugins) {
                        if (plugins.hasOwnProperty(pluginId)) {
                            plugin = plugins[pluginId];
                            pData = {
                                name: plugin.name,
                                filename: plugin.filename,
                                description: plugin.description
                            };
                            pluginData.push(pData);
                        }
                    }
                } catch (e) {
                    pluginData = [];
                }
                pluginData = JSON.stringify(pluginData);
                return pluginData;
            }
        };
    }());
    var errorReporting = Object.create(ErrorReporting.prototype);
    errorReporting.init();
    win.reportError = errorReporting.handleError.bind(errorReporting);
    win.reportAjaxError = errorReporting.handleAjaxError.bind(errorReporting);
    window.errorReporting = errorReporting;
}(window));
