;
/*!
 * mustache.js - Logic-less {{mustache}} templates with JavaScript
 * http://github.com/janl/mustache.js
 */

/*global define: false*/

(function (global, factory) {
	if (typeof exports === "object" && exports) {
		factory(exports); // CommonJS
	} else if (typeof define === "function" && define.amd) {
		define(factory({})); // AMD
	} else {
		global.Mustache = factory({}); // <script>
	}
}(this, function (mustache) {

	var Object_toString = Object.prototype.toString;
	var isArray = Array.isArray || function (object) {
		return Object_toString.call(object) === '[object Array]';
	};

	function isFunction(object) {
		return typeof object === 'function';
	}

	function escapeRegExp(string) {
		return string.replace(/[\-\[\]{}()*+?.,\\\^$|#\s]/g, "\\$&");
	}

	// Workaround for https://issues.apache.org/jira/browse/COUCHDB-577
	// See https://github.com/janl/mustache.js/issues/189
	var RegExp_test = RegExp.prototype.test;
	function testRegExp(re, string) {
		return RegExp_test.call(re, string);
	}

	var nonSpaceRe = /\S/;
	function isWhitespace(string) {
		return !testRegExp(nonSpaceRe, string);
	}

	var entityMap = {
		"&": "&amp;",
		"<": "&lt;",
		">": "&gt;",
		'"': '&quot;',
		"'": '&#39;',
		"/": '&#x2F;'
	};

	function escapeHtml(string) {
		return String(string).replace(/[&<>"'\/]/g, function (s) {
			return entityMap[s];
		});
	}

	var whiteRe = /\s*/;
	var spaceRe = /\s+/;
	var equalsRe = /\s*=/;
	var curlyRe = /\s*\}/;
	var tagRe = /#|\^|\/|>|\{|&|=|!/;

	/**
	 * Breaks up the given `template` string into a tree of tokens. If the `tags`
	 * argument is given here it must be an array with two string values: the
	 * opening and closing tags used in the template (e.g. [ "<%", "%>" ]). Of
	 * course, the default is to use mustaches (i.e. mustache.tags).
	 *
	 * A token is an array with at least 4 elements. The first element is the
	 * mustache symbol that was used inside the tag, e.g. "#" or "&". If the tag
	 * did not contain a symbol (i.e. {{myValue}}) this element is "name". For
	 * all text that appears outside a symbol this element is "text".
	 *
	 * The second element of a token is its "value". For mustache tags this is
	 * whatever else was inside the tag besides the opening symbol. For text tokens
	 * this is the text itself.
	 *
	 * The third and fourth elements of the token are the start and end indices,
	 * respectively, of the token in the original template.
	 *
	 * Tokens that are the root node of a subtree contain two more elements: 1) an
	 * array of tokens in the subtree and 2) the index in the original template at
	 * which the closing tag for that section begins.
	 */
	function parseTemplate(template, tags) {
		if (!template)
			return [];

		var sections = [];     // Stack to hold section tokens
		var tokens = [];       // Buffer to hold the tokens
		var spaces = [];       // Indices of whitespace tokens on the current line
		var hasTag = false;    // Is there a {{tag}} on the current line?
		var nonSpace = false;  // Is there a non-space char on the current line?

		// Strips all whitespace tokens array for the current line
		// if there was a {{#tag}} on it and otherwise only space.
		function stripSpace() {
			if (hasTag && !nonSpace) {
				while (spaces.length)
					delete tokens[spaces.pop()];
			} else {
				spaces = [];
			}

			hasTag = false;
			nonSpace = false;
		}

		var openingTagRe, closingTagRe, closingCurlyRe;
		function compileTags(tags) {
			if (typeof tags === 'string')
				tags = tags.split(spaceRe, 2);

			if (!isArray(tags) || tags.length !== 2)
				throw new Error('Invalid tags: ' + tags);

			openingTagRe = new RegExp(escapeRegExp(tags[0]) + '\\s*');
			closingTagRe = new RegExp('\\s*' + escapeRegExp(tags[1]));
			closingCurlyRe = new RegExp('\\s*' + escapeRegExp('}' + tags[1]));
		}

		compileTags(tags || mustache.tags);

		var scanner = new Scanner(template);

		var start, type, value, chr, token, openSection;
		while (!scanner.eos()) {
			start = scanner.pos;

			// Match any text between tags.
			value = scanner.scanUntil(openingTagRe);

			if (value) {
				for (var i = 0, valueLength = value.length; i < valueLength; ++i) {
					chr = value.charAt(i);

					if (isWhitespace(chr)) {
						spaces.push(tokens.length);
					} else {
						nonSpace = true;
					}

					tokens.push([ 'text', chr, start, start + 1 ]);
					start += 1;

					// Check for whitespace on the current line.
					if (chr === '\n')
						stripSpace();
				}
			}

			// Match the opening tag.
			if (!scanner.scan(openingTagRe))
				break;

			hasTag = true;

			// Get the tag type.
			type = scanner.scan(tagRe) || 'name';
			scanner.scan(whiteRe);

			// Get the tag value.
			if (type === '=') {
				value = scanner.scanUntil(equalsRe);
				scanner.scan(equalsRe);
				scanner.scanUntil(closingTagRe);
			} else if (type === '{') {
				value = scanner.scanUntil(closingCurlyRe);
				scanner.scan(curlyRe);
				scanner.scanUntil(closingTagRe);
				type = '&';
			} else {
				value = scanner.scanUntil(closingTagRe);
			}

			// Match the closing tag.
			if (!scanner.scan(closingTagRe))
				throw new Error('Unclosed tag at ' + scanner.pos);

			token = [ type, value, start, scanner.pos ];
			tokens.push(token);

			if (type === '#' || type === '^') {
				sections.push(token);
			} else if (type === '/') {
				// Check section nesting.
				openSection = sections.pop();

				if (!openSection)
					throw new Error('Unopened section "' + value + '" at ' + start);

				if (openSection[1] !== value)
					throw new Error('Unclosed section "' + openSection[1] + '" at ' + start);
			} else if (type === 'name' || type === '{' || type === '&') {
				nonSpace = true;
			} else if (type === '=') {
				// Set the tags for the next time around.
				compileTags(value);
			}
		}

		// Make sure there are no open sections when we're done.
		openSection = sections.pop();

		if (openSection)
			throw new Error('Unclosed section "' + openSection[1] + '" at ' + scanner.pos);

		return nestTokens(squashTokens(tokens));
	}

	/**
	 * Combines the values of consecutive text tokens in the given `tokens` array
	 * to a single token.
	 */
	function squashTokens(tokens) {
		var squashedTokens = [];

		var token, lastToken;
		for (var i = 0, numTokens = tokens.length; i < numTokens; ++i) {
			token = tokens[i];

			if (token) {
				if (token[0] === 'text' && lastToken && lastToken[0] === 'text') {
					lastToken[1] += token[1];
					lastToken[3] = token[3];
				} else {
					squashedTokens.push(token);
					lastToken = token;
				}
			}
		}

		return squashedTokens;
	}

	/**
	 * Forms the given array of `tokens` into a nested tree structure where
	 * tokens that represent a section have two additional items: 1) an array of
	 * all tokens that appear in that section and 2) the index in the original
	 * template that represents the end of that section.
	 */
	function nestTokens(tokens) {
		var nestedTokens = [];
		var collector = nestedTokens;
		var sections = [];

		var token, section;
		for (var i = 0, numTokens = tokens.length; i < numTokens; ++i) {
			token = tokens[i];

			switch (token[0]) {
				case '#':
				case '^':
					collector.push(token);
					sections.push(token);
					collector = token[4] = [];
					break;
				case '/':
					section = sections.pop();
					section[5] = token[2];
					collector = sections.length > 0 ? sections[sections.length - 1][4] : nestedTokens;
					break;
				default:
					collector.push(token);
			}
		}

		return nestedTokens;
	}

	/**
	 * A simple string scanner that is used by the template parser to find
	 * tokens in template strings.
	 */
	function Scanner(string) {
		this.string = string;
		this.tail = string;
		this.pos = 0;
	}

	/**
	 * Returns `true` if the tail is empty (end of string).
	 */
	Scanner.prototype.eos = function () {
		return this.tail === "";
	};

	/**
	 * Tries to match the given regular expression at the current position.
	 * Returns the matched text if it can match, the empty string otherwise.
	 */
	Scanner.prototype.scan = function (re) {
		var match = this.tail.match(re);

		if (!match || match.index !== 0)
			return '';

		var string = match[0];

		this.tail = this.tail.substring(string.length);
		this.pos += string.length;

		return string;
	};

	/**
	 * Skips all text until the given regular expression can be matched. Returns
	 * the skipped string, which is the entire tail if no match can be made.
	 */
	Scanner.prototype.scanUntil = function (re) {
		var index = this.tail.search(re), match;

		switch (index) {
			case -1:
				match = this.tail;
				this.tail = "";
				break;
			case 0:
				match = "";
				break;
			default:
				match = this.tail.substring(0, index);
				this.tail = this.tail.substring(index);
		}

		this.pos += match.length;

		return match;
	};

	/**
	 * Represents a rendering context by wrapping a view object and
	 * maintaining a reference to the parent context.
	 */
	function Context(view, parentContext) {
		this.view = view == null ? {} : view;
		this.cache = { '.': this.view };
		this.parent = parentContext;
	}

	/**
	 * Creates a new context using the given view with this context
	 * as the parent.
	 */
	Context.prototype.push = function (view) {
		return new Context(view, this);
	};

	/**
	 * Returns the value of the given name in this context, traversing
	 * up the context hierarchy if the value is absent in this context's view.
	 */
	Context.prototype.lookup = function (name) {
		var cache = this.cache;

		var value;
		if (name in cache) {
			value = cache[name];
		} else {
			var context = this, names, index;

			while (context) {
				if (name.indexOf('.') > 0) {
					value = context.view;
					names = name.split('.');
					index = 0;

					while (value != null && index < names.length)
						value = value[names[index++]];
				} else {
					value = context.view[name];
				}

				if (value != null)
					break;

				context = context.parent;
			}

			cache[name] = value;
		}

		if (isFunction(value))
			value = value.call(this.view);

		return value;
	};

	/**
	 * A Writer knows how to take a stream of tokens and render them to a
	 * string, given a context. It also maintains a cache of templates to
	 * avoid the need to parse the same template twice.
	 */
	function Writer() {
		this.cache = {};
	}

	/**
	 * Clears all cached templates in this writer.
	 */
	Writer.prototype.clearCache = function () {
		this.cache = {};
	};

	/**
	 * Parses and caches the given `template` and returns the array of tokens
	 * that is generated from the parse.
	 */
	Writer.prototype.parse = function (template, tags) {
		var cache = this.cache;
		var tokens = cache[template];

		if (tokens == null)
			tokens = cache[template] = parseTemplate(template, tags);

		return tokens;
	};

	/**
	 * High-level method that is used to render the given `template` with
	 * the given `view`.
	 *
	 * The optional `partials` argument may be an object that contains the
	 * names and templates of partials that are used in the template. It may
	 * also be a function that is used to load partial templates on the fly
	 * that takes a single argument: the name of the partial.
	 */
	Writer.prototype.render = function (template, view, partials) {
		var tokens = this.parse(template);
		var context = (view instanceof Context) ? view : new Context(view);
		return this.renderTokens(tokens, context, partials, template);
	};

	/**
	 * Low-level method that renders the given array of `tokens` using
	 * the given `context` and `partials`.
	 *
	 * Note: The `originalTemplate` is only ever used to extract the portion
	 * of the original template that was contained in a higher-order section.
	 * If the template doesn't use higher-order sections, this argument may
	 * be omitted.
	 */
	Writer.prototype.renderTokens = function (tokens, context, partials, originalTemplate) {
		var buffer = '';

		// This function is used to render an arbitrary template
		// in the current context by higher-order sections.
		var self = this;
		function subRender(template) {
			return self.render(template, context, partials);
		}

		var token, value;
		for (var i = 0, numTokens = tokens.length; i < numTokens; ++i) {
			token = tokens[i];

			switch (token[0]) {
				case '#':
					value = context.lookup(token[1]);

					if (!value)
						continue;

					if (isArray(value)) {
						for (var j = 0, valueLength = value.length; j < valueLength; ++j) {
							buffer += this.renderTokens(token[4], context.push(value[j]), partials, originalTemplate);
						}
					} else if (typeof value === 'object' || typeof value === 'string') {
						buffer += this.renderTokens(token[4], context.push(value), partials, originalTemplate);
					} else if (isFunction(value)) {
						if (typeof originalTemplate !== 'string')
							throw new Error('Cannot use higher-order sections without the original template');

						// Extract the portion of the original template that the section contains.
						value = value.call(context.view, originalTemplate.slice(token[3], token[5]), subRender);

						if (value != null)
							buffer += value;
					} else {
						buffer += this.renderTokens(token[4], context, partials, originalTemplate);
					}

					break;
				case '^':
					value = context.lookup(token[1]);

					// Use JavaScript's definition of falsy. Include empty arrays.
					// See https://github.com/janl/mustache.js/issues/186
					if (!value || (isArray(value) && value.length === 0))
						buffer += this.renderTokens(token[4], context, partials, originalTemplate);

					break;
				case '>':
					if (!partials)
						continue;

					value = isFunction(partials) ? partials(token[1]) : partials[token[1]];

					if (value != null)
						buffer += this.renderTokens(this.parse(value), context, partials, value);

					break;
				case '&':
					value = context.lookup(token[1]);

					if (value != null)
						buffer += value;

					break;
				case 'name':
					value = context.lookup(token[1]);

					if (value != null)
						buffer += mustache.escape(value);

					break;
				case 'text':
					buffer += token[1];
					break;
			}
		}

		return buffer;
	};

	mustache.name = "mustache.js";
	mustache.version = "0.8.1";
	mustache.tags = [ "{{", "}}" ];

	// All high-level mustache.* functions use this writer.
	var defaultWriter = new Writer();

	/**
	 * Clears all cached templates in the default writer.
	 */
	mustache.clearCache = function () {
		return defaultWriter.clearCache();
	};

	/**
	 * Parses and caches the given template in the default writer and returns the
	 * array of tokens it contains. Doing this ahead of time avoids the need to
	 * parse templates on the fly as they are rendered.
	 */
	mustache.parse = function (template, tags) {
		return defaultWriter.parse(template, tags);
	};

	/**
	 * Renders the `template` with the given `view` and `partials` using the
	 * default writer.
	 */
	mustache.render = function (template, view, partials) {
		return defaultWriter.render(template, view, partials);
	};

	// This is here for backwards compatibility with 0.4.x.
	mustache.to_html = function (template, view, partials, send) {
		var result = mustache.render(template, view, partials);

		if (isFunction(send)) {
			send(result);
		} else {
			return result;
		}
	};

	// Export the escaping function so that the user may override it.
	// See https://github.com/janl/mustache.js/issues/244
	mustache.escape = escapeHtml;

	// Export these mainly for testing, but also for advanced usage.
	mustache.Scanner = Scanner;
	mustache.Context = Context;
	mustache.Writer = Writer;

	return mustache;

}));;
/*! Copyright (c) 2011 Brandon Aaron (http://brandonaaron.net)
 * Licensed under the MIT License (LICENSE.txt).
 *
 * Thanks to: http://adomas.org/javascript-mouse-wheel/ for some pointers.
 * Thanks to: Mathias Bank(http://www.mathias-bank.de) for a scope bug fix.
 * Thanks to: Seamus Leahy for adding deltaX and deltaY
 *
 * Version: 3.0.6
 * 
 * Requires: 1.2.2+
 */
(function(a){function d(b){var c=b||window.event,d=[].slice.call(arguments,1),e=0,f=!0,g=0,h=0;return b=a.event.fix(c),b.type="mousewheel",c.wheelDelta&&(e=c.wheelDelta/120),c.detail&&(e=-c.detail/3),h=e,c.axis!==undefined&&c.axis===c.HORIZONTAL_AXIS&&(h=0,g=-1*e),c.wheelDeltaY!==undefined&&(h=c.wheelDeltaY/120),c.wheelDeltaX!==undefined&&(g=-1*c.wheelDeltaX/120),d.unshift(b,e,g,h),(a.event.dispatch||a.event.handle).apply(this,d)}var b=["DOMMouseScroll","mousewheel"];if(a.event.fixHooks)for(var c=b.length;c;)a.event.fixHooks[b[--c]]=a.event.mouseHooks;a.event.special.mousewheel={setup:function(){if(this.addEventListener)for(var a=b.length;a;)this.addEventListener(b[--a],d,!1);else this.onmousewheel=d},teardown:function(){if(this.removeEventListener)for(var a=b.length;a;)this.removeEventListener(b[--a],d,!1);else this.onmousewheel=null}},a.fn.extend({mousewheel:function(a){return a?this.bind("mousewheel",a):this.trigger("mousewheel")},unmousewheel:function(a){return this.unbind("mousewheel",a)}})})(jQuery)
;
/*
 == malihu jquery custom scrollbars plugin ==
 version: 2.8.3
 author: malihu (http://manos.malihu.gr)
 plugin home: http://manos.malihu.gr/jquery-custom-content-scroller
 */

/*
 Copyright 2010-2013 Manos Malihutsakis

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with this program.  If not, see http://www.gnu.org/licenses/lgpl.html.
 */
(function($){
	/*plugin script*/
	var methods={
			init:function(options){
				var defaults={
						set_width:false, /*optional element width: boolean, pixels, percentage*/
						set_height:false, /*optional element height: boolean, pixels, percentage*/
						horizontalScroll:false, /*scroll horizontally: boolean*/
						scrollInertia:950, /*scrolling inertia: integer (milliseconds)*/
						mouseWheel:true, /*mousewheel support: boolean*/
						mouseWheelPixels:"auto", /*mousewheel pixels amount: integer, "auto"*/
						autoDraggerLength:true, /*auto-adjust scrollbar dragger length: boolean*/
						autoHideScrollbar:false, /*auto-hide scrollbar when idle*/
						alwaysShowScrollbar:false, /*always show scrollbar even when there's nothing to scroll (disables autoHideScrollbar): boolean*/
						snapAmount:null, /* optional element always snaps to a multiple of this number in pixels */
						snapOffset:0, /* when snapping, snap with this number in pixels as an offset */
						scrollButtons:{ /*scroll buttons*/
							enable:false, /*scroll buttons support: boolean*/
							scrollType:"continuous", /*scroll buttons scrolling type: "continuous", "pixels"*/
							scrollSpeed:"auto", /*scroll buttons continuous scrolling speed: integer, "auto"*/
							scrollAmount:40 /*scroll buttons pixels scroll amount: integer (pixels)*/
						},
						advanced:{
							updateOnBrowserResize:true, /*update scrollbars on browser resize (for layouts based on percentages): boolean*/
							updateOnContentResize:false, /*auto-update scrollbars on content resize (for dynamic content): boolean*/
							autoExpandHorizontalScroll:false, /*auto-expand width for horizontal scrolling: boolean*/
							autoScrollOnFocus:true, /*auto-scroll on focused elements: boolean*/
							normalizeMouseWheelDelta:false /*normalize mouse-wheel delta (-1/1)*/
						},
						contentTouchScroll:true, /*scrolling by touch-swipe content: boolean*/
						callbacks:{
							onScrollStart:function(){}, /*user custom callback function on scroll start event*/
							onScroll:function(){}, /*user custom callback function on scroll event*/
							onTotalScroll:function(){}, /*user custom callback function on scroll end reached event*/
							onTotalScrollBack:function(){}, /*user custom callback function on scroll begin reached event*/
							onTotalScrollOffset:0, /*scroll end reached offset: integer (pixels)*/
							onTotalScrollBackOffset:0, /*scroll begin reached offset: integer (pixels)*/
							whileScrolling:function(){} /*user custom callback function on scrolling event*/
						},
						theme:"light" /*"light", "dark", "light-2", "dark-2", "light-thick", "dark-thick", "light-thin", "dark-thin"*/
					},
					options=$.extend(true,defaults,options);
				return this.each(function(){
					var $this=$(this);
					/*set element width/height, create markup for custom scrollbars, add classes*/
					if(options.set_width){
						$this.css("width",options.set_width);
					}
					if(options.set_height){
						$this.css("height",options.set_height);
					}
					if(!$(document).data("mCustomScrollbar-index")){
						$(document).data("mCustomScrollbar-index","1");
					}else{
						var mCustomScrollbarIndex=parseInt($(document).data("mCustomScrollbar-index"));
						$(document).data("mCustomScrollbar-index",mCustomScrollbarIndex+1);
					}
					$this.wrapInner("<div class='mCustomScrollBox"+" mCS-"+options.theme+"' id='mCSB_"+$(document).data("mCustomScrollbar-index")+"' style='position:relative; height:100%; overflow:hidden; max-width:100%;' />").addClass("mCustomScrollbar _mCS_"+$(document).data("mCustomScrollbar-index"));
					var mCustomScrollBox=$this.children(".mCustomScrollBox");
					if(options.horizontalScroll){
						mCustomScrollBox.addClass("mCSB_horizontal").wrapInner("<div class='mCSB_h_wrapper' style='position:relative; left:0; width:999999px;' />");
						var mCSB_h_wrapper=mCustomScrollBox.children(".mCSB_h_wrapper");
						mCSB_h_wrapper.wrapInner("<div class='mCSB_container' style='position:absolute; left:0;' />").children(".mCSB_container").css({"width":mCSB_h_wrapper.children().outerWidth(),"position":"relative"}).unwrap();
					}else{
						mCustomScrollBox.wrapInner("<div class='mCSB_container' style='position:relative; top:0;' />");
					}
					var mCSB_container=mCustomScrollBox.children(".mCSB_container");
					if($.support.touch){
						mCSB_container.addClass("mCS_touch");
					}
					mCSB_container.after("<div class='mCSB_scrollTools' style='position:absolute;'><div class='mCSB_draggerContainer'><div class='mCSB_dragger' style='position:absolute;' oncontextmenu='return false;'><div class='mCSB_dragger_bar' style='position:relative;'></div></div><div class='mCSB_draggerRail'></div></div></div>");
					var mCSB_scrollTools=mCustomScrollBox.children(".mCSB_scrollTools"),
						mCSB_draggerContainer=mCSB_scrollTools.children(".mCSB_draggerContainer"),
						mCSB_dragger=mCSB_draggerContainer.children(".mCSB_dragger");
					if(options.horizontalScroll){
						mCSB_dragger.data("minDraggerWidth",mCSB_dragger.width());
					}else{
						mCSB_dragger.data("minDraggerHeight",mCSB_dragger.height());
					}
					if(options.scrollButtons.enable){
						if(options.horizontalScroll){
							mCSB_scrollTools.prepend("<a class='mCSB_buttonLeft' oncontextmenu='return false;'></a>").append("<a class='mCSB_buttonRight' oncontextmenu='return false;'></a>");
						}else{
							mCSB_scrollTools.prepend("<a class='mCSB_buttonUp' oncontextmenu='return false;'></a>").append("<a class='mCSB_buttonDown' oncontextmenu='return false;'></a>");
						}
					}
					/*mCustomScrollBox scrollTop and scrollLeft is always 0 to prevent browser focus scrolling*/
					mCustomScrollBox.bind("scroll",function(){
						if(!$this.is(".mCS_disabled")){ /*native focus scrolling for disabled scrollbars*/
							mCustomScrollBox.scrollTop(0).scrollLeft(0);
						}
					});
					/*store options, global vars/states, intervals*/
					$this.data({
						/*init state*/
						"mCS_Init":true,
						/*instance index*/
						"mCustomScrollbarIndex":$(document).data("mCustomScrollbar-index"),
						/*option parameters*/
						"horizontalScroll":options.horizontalScroll,
						"scrollInertia":options.scrollInertia,
						"scrollEasing":"mcsEaseOut",
						"mouseWheel":options.mouseWheel,
						"mouseWheelPixels":options.mouseWheelPixels,
						"autoDraggerLength":options.autoDraggerLength,
						"autoHideScrollbar":options.autoHideScrollbar,
						"alwaysShowScrollbar":options.alwaysShowScrollbar,
						"snapAmount":options.snapAmount,
						"snapOffset":options.snapOffset,
						"scrollButtons_enable":options.scrollButtons.enable,
						"scrollButtons_scrollType":options.scrollButtons.scrollType,
						"scrollButtons_scrollSpeed":options.scrollButtons.scrollSpeed,
						"scrollButtons_scrollAmount":options.scrollButtons.scrollAmount,
						"autoExpandHorizontalScroll":options.advanced.autoExpandHorizontalScroll,
						"autoScrollOnFocus":options.advanced.autoScrollOnFocus,
						"normalizeMouseWheelDelta":options.advanced.normalizeMouseWheelDelta,
						"contentTouchScroll":options.contentTouchScroll,
						"onScrollStart_Callback":options.callbacks.onScrollStart,
						"onScroll_Callback":options.callbacks.onScroll,
						"onTotalScroll_Callback":options.callbacks.onTotalScroll,
						"onTotalScrollBack_Callback":options.callbacks.onTotalScrollBack,
						"onTotalScroll_Offset":options.callbacks.onTotalScrollOffset,
						"onTotalScrollBack_Offset":options.callbacks.onTotalScrollBackOffset,
						"whileScrolling_Callback":options.callbacks.whileScrolling,
						/*events binding state*/
						"bindEvent_scrollbar_drag":false,
						"bindEvent_content_touch":false,
						"bindEvent_scrollbar_click":false,
						"bindEvent_mousewheel":false,
						"bindEvent_buttonsContinuous_y":false,
						"bindEvent_buttonsContinuous_x":false,
						"bindEvent_buttonsPixels_y":false,
						"bindEvent_buttonsPixels_x":false,
						"bindEvent_focusin":false,
						"bindEvent_autoHideScrollbar":false,
						/*buttons intervals*/
						"mCSB_buttonScrollRight":false,
						"mCSB_buttonScrollLeft":false,
						"mCSB_buttonScrollDown":false,
						"mCSB_buttonScrollUp":false
					});
					/*max-width/max-height*/
					if(options.horizontalScroll){
						if($this.css("max-width")!=="none"){
							if(!options.advanced.updateOnContentResize){ /*needs updateOnContentResize*/
								options.advanced.updateOnContentResize=true;
							}
						}
					}else{
						if($this.css("max-height")!=="none"){
							var percentage=false,maxHeight=parseInt($this.css("max-height"));
							if($this.css("max-height").indexOf("%")>=0){
								percentage=maxHeight,
									maxHeight=$this.parent().height()*percentage/100;
							}
							$this.css("overflow","hidden");
							mCustomScrollBox.css("max-height",maxHeight);
						}
					}
					$this.mCustomScrollbar("update");
					/*window resize fn (for layouts based on percentages)*/
					if(options.advanced.updateOnBrowserResize){
						var mCSB_resizeTimeout,currWinWidth=$(window).width(),currWinHeight=$(window).height();
						$(window).bind("resize."+$this.data("mCustomScrollbarIndex"),function(){
							if(mCSB_resizeTimeout){
								clearTimeout(mCSB_resizeTimeout);
							}
							mCSB_resizeTimeout=setTimeout(function(){
								if(!$this.is(".mCS_disabled") && !$this.is(".mCS_destroyed")){
									var winWidth=$(window).width(),winHeight=$(window).height();
									if(currWinWidth!==winWidth || currWinHeight!==winHeight){ /*ie8 fix*/
										if($this.css("max-height")!=="none" && percentage){
											mCustomScrollBox.css("max-height",$this.parent().height()*percentage/100);
										}
										$this.mCustomScrollbar("update");
										currWinWidth=winWidth; currWinHeight=winHeight;
									}
								}
							},150);
						});
					}
					/*content resize fn (for dynamically generated content)*/
					if(options.advanced.updateOnContentResize){
						var mCSB_onContentResize;
						if(options.horizontalScroll){
							var mCSB_containerOldSize=mCSB_container.outerWidth();
						}else{
							var mCSB_containerOldSize=mCSB_container.outerHeight();
						}
						mCSB_onContentResize=setInterval(function(){
							if(options.horizontalScroll){
								if(options.advanced.autoExpandHorizontalScroll){
									mCSB_container.css({"position":"absolute","width":"auto"}).wrap("<div class='mCSB_h_wrapper' style='position:relative; left:0; width:999999px;' />").css({"width":mCSB_container.outerWidth(),"position":"relative"}).unwrap();
								}
								var mCSB_containerNewSize=mCSB_container.outerWidth();
							}else{
								var mCSB_containerNewSize=mCSB_container.outerHeight();
							}
							if(mCSB_containerNewSize!=mCSB_containerOldSize){
								$this.mCustomScrollbar("update");
								mCSB_containerOldSize=mCSB_containerNewSize;
							}
						},300);
					}
				});
			},
			update:function(){
				var $this=$(this),
					mCustomScrollBox=$this.children(".mCustomScrollBox"),
					mCSB_container=mCustomScrollBox.children(".mCSB_container");
				mCSB_container.removeClass("mCS_no_scrollbar");
				$this.removeClass("mCS_disabled mCS_destroyed");
				mCustomScrollBox.scrollTop(0).scrollLeft(0); /*reset scrollTop/scrollLeft to prevent browser focus scrolling*/
				var mCSB_scrollTools=mCustomScrollBox.children(".mCSB_scrollTools"),
					mCSB_draggerContainer=mCSB_scrollTools.children(".mCSB_draggerContainer"),
					mCSB_dragger=mCSB_draggerContainer.children(".mCSB_dragger");
				if($this.data("horizontalScroll")){
					var mCSB_buttonLeft=mCSB_scrollTools.children(".mCSB_buttonLeft"),
						mCSB_buttonRight=mCSB_scrollTools.children(".mCSB_buttonRight"),
						mCustomScrollBoxW=mCustomScrollBox.width();
					if($this.data("autoExpandHorizontalScroll")){
						mCSB_container.css({"position":"absolute","width":"auto"}).wrap("<div class='mCSB_h_wrapper' style='position:relative; left:0; width:999999px;' />").css({"width":mCSB_container.outerWidth(),"position":"relative"}).unwrap();
					}
					var mCSB_containerW=mCSB_container.outerWidth();
				}else{
					var mCSB_buttonUp=mCSB_scrollTools.children(".mCSB_buttonUp"),
						mCSB_buttonDown=mCSB_scrollTools.children(".mCSB_buttonDown"),
						mCustomScrollBoxH=mCustomScrollBox.height(),
						mCSB_containerH=mCSB_container.outerHeight();
				}
				if(mCSB_containerH>mCustomScrollBoxH && !$this.data("horizontalScroll")){ /*content needs vertical scrolling*/
					mCSB_scrollTools.css("display","block");
					var mCSB_draggerContainerH=mCSB_draggerContainer.height();
					/*auto adjust scrollbar dragger length analogous to content*/
					if($this.data("autoDraggerLength")){
						var draggerH=Math.round(mCustomScrollBoxH/mCSB_containerH*mCSB_draggerContainerH),
							minDraggerH=mCSB_dragger.data("minDraggerHeight");
						if(draggerH<=minDraggerH){ /*min dragger height*/
							mCSB_dragger.css({"height":minDraggerH});
						}else if(draggerH>=mCSB_draggerContainerH-10){ /*max dragger height*/
							var mCSB_draggerContainerMaxH=mCSB_draggerContainerH-10;
							mCSB_dragger.css({"height":mCSB_draggerContainerMaxH});
						}else{
							mCSB_dragger.css({"height":draggerH});
						}
						mCSB_dragger.children(".mCSB_dragger_bar").css({"line-height":mCSB_dragger.height()+"px"});
					}
					var mCSB_draggerH=mCSB_dragger.height(),
					/*calculate and store scroll amount, add scrolling*/
						scrollAmount=(mCSB_containerH-mCustomScrollBoxH)/(mCSB_draggerContainerH-mCSB_draggerH);
					$this.data("scrollAmount",scrollAmount).mCustomScrollbar("scrolling",mCustomScrollBox,mCSB_container,mCSB_draggerContainer,mCSB_dragger,mCSB_buttonUp,mCSB_buttonDown,mCSB_buttonLeft,mCSB_buttonRight);
					/*scroll*/
					var mCSB_containerP=Math.abs(mCSB_container.position().top);
					$this.mCustomScrollbar("scrollTo",mCSB_containerP,{scrollInertia:0,trigger:"internal"});
				}else if(mCSB_containerW>mCustomScrollBoxW && $this.data("horizontalScroll")){ /*content needs horizontal scrolling*/
					mCSB_scrollTools.css("display","block");
					var mCSB_draggerContainerW=mCSB_draggerContainer.width();
					/*auto adjust scrollbar dragger length analogous to content*/
					if($this.data("autoDraggerLength")){
						var draggerW=Math.round(mCustomScrollBoxW/mCSB_containerW*mCSB_draggerContainerW),
							minDraggerW=mCSB_dragger.data("minDraggerWidth");
						if(draggerW<=minDraggerW){ /*min dragger height*/
							mCSB_dragger.css({"width":minDraggerW});
						}else if(draggerW>=mCSB_draggerContainerW-10){ /*max dragger height*/
							var mCSB_draggerContainerMaxW=mCSB_draggerContainerW-10;
							mCSB_dragger.css({"width":mCSB_draggerContainerMaxW});
						}else{
							mCSB_dragger.css({"width":draggerW});
						}
					}
					var mCSB_draggerW=mCSB_dragger.width(),
					/*calculate and store scroll amount, add scrolling*/
						scrollAmount=(mCSB_containerW-mCustomScrollBoxW)/(mCSB_draggerContainerW-mCSB_draggerW);
					$this.data("scrollAmount",scrollAmount).mCustomScrollbar("scrolling",mCustomScrollBox,mCSB_container,mCSB_draggerContainer,mCSB_dragger,mCSB_buttonUp,mCSB_buttonDown,mCSB_buttonLeft,mCSB_buttonRight);
					/*scroll*/
					var mCSB_containerP=Math.abs(mCSB_container.position().left);
					$this.mCustomScrollbar("scrollTo",mCSB_containerP,{scrollInertia:0,trigger:"internal"});
				}else{ /*content does not need scrolling*/
					/*unbind events, reset content position, hide scrollbars, remove classes*/
					mCustomScrollBox.unbind("mousewheel focusin");
					if($this.data("horizontalScroll")){
						mCSB_dragger.add(mCSB_container).css("left",0);
					}else{
						mCSB_dragger.add(mCSB_container).css("top",0);
					}
					if ($this.data("alwaysShowScrollbar")) {
						if(!$this.data("horizontalScroll")){ /*vertical scrolling*/
							mCSB_dragger.css({"height":mCSB_draggerContainer.height()});
						}else if($this.data("horizontalScroll")){ /*horizontal scrolling*/
							mCSB_dragger.css({"width":mCSB_draggerContainer.width()});
						}
					} else {
						mCSB_scrollTools.css("display","none");
						mCSB_container.addClass("mCS_no_scrollbar");
					}
					$this.data({"bindEvent_mousewheel":false,"bindEvent_focusin":false});
				}
			},
			scrolling:function(mCustomScrollBox,mCSB_container,mCSB_draggerContainer,mCSB_dragger,mCSB_buttonUp,mCSB_buttonDown,mCSB_buttonLeft,mCSB_buttonRight){
				var $this=$(this);
				/*scrollbar drag scrolling*/
				if(!$this.data("bindEvent_scrollbar_drag")){
					var mCSB_draggerDragY,mCSB_draggerDragX,
						mCSB_dragger_downEvent,mCSB_dragger_moveEvent,mCSB_dragger_upEvent;
					if($.support.pointer){ /*pointer*/
						mCSB_dragger_downEvent="pointerdown";
						mCSB_dragger_moveEvent="pointermove";
						mCSB_dragger_upEvent="pointerup";
					}else if($.support.msPointer){ /*MSPointer*/
						mCSB_dragger_downEvent="MSPointerDown";
						mCSB_dragger_moveEvent="MSPointerMove";
						mCSB_dragger_upEvent="MSPointerUp";
					}
					if($.support.pointer || $.support.msPointer){ /*pointer, MSPointer*/
						mCSB_dragger.bind(mCSB_dragger_downEvent,function(e){
							e.preventDefault();
							$this.data({"on_drag":true}); mCSB_dragger.addClass("mCSB_dragger_onDrag");
							var elem=$(this),
								elemOffset=elem.offset(),
								x=e.originalEvent.pageX-elemOffset.left,
								y=e.originalEvent.pageY-elemOffset.top;
							if(x<elem.width() && x>0 && y<elem.height() && y>0){
								mCSB_draggerDragY=y;
								mCSB_draggerDragX=x;
							}
						});
						$(document).bind(mCSB_dragger_moveEvent+"."+$this.data("mCustomScrollbarIndex"),function(e){
							e.preventDefault();
							if($this.data("on_drag")){
								var elem=mCSB_dragger,
									elemOffset=elem.offset(),
									x=e.originalEvent.pageX-elemOffset.left,
									y=e.originalEvent.pageY-elemOffset.top;
								scrollbarDrag(mCSB_draggerDragY,mCSB_draggerDragX,y,x);
							}
						}).bind(mCSB_dragger_upEvent+"."+$this.data("mCustomScrollbarIndex"),function(e){
								$this.data({"on_drag":false}); mCSB_dragger.removeClass("mCSB_dragger_onDrag");
							});
					}else{ /*mouse/touch*/
						mCSB_dragger.bind("mousedown touchstart",function(e){
							e.preventDefault(); e.stopImmediatePropagation();
							var        elem=$(this),elemOffset=elem.offset(),x,y;
							if(e.type==="touchstart"){
								var touch=e.originalEvent.touches[0] || e.originalEvent.changedTouches[0];
								x=touch.pageX-elemOffset.left; y=touch.pageY-elemOffset.top;
							}else{
								$this.data({"on_drag":true}); mCSB_dragger.addClass("mCSB_dragger_onDrag");
								x=e.pageX-elemOffset.left; y=e.pageY-elemOffset.top;
							}
							if(x<elem.width() && x>0 && y<elem.height() && y>0){
								mCSB_draggerDragY=y; mCSB_draggerDragX=x;
							}
						}).bind("touchmove",function(e){
								e.preventDefault(); e.stopImmediatePropagation();
								var touch=e.originalEvent.touches[0] || e.originalEvent.changedTouches[0],
									elem=$(this),
									elemOffset=elem.offset(),
									x=touch.pageX-elemOffset.left,
									y=touch.pageY-elemOffset.top;
								scrollbarDrag(mCSB_draggerDragY,mCSB_draggerDragX,y,x);
							});
						$(document).bind("mousemove."+$this.data("mCustomScrollbarIndex"),function(e){
							if($this.data("on_drag")){
								var elem=mCSB_dragger,
									elemOffset=elem.offset(),
									x=e.pageX-elemOffset.left,
									y=e.pageY-elemOffset.top;
								scrollbarDrag(mCSB_draggerDragY,mCSB_draggerDragX,y,x);
							}
						}).bind("mouseup."+$this.data("mCustomScrollbarIndex"),function(e){
								$this.data({"on_drag":false}); mCSB_dragger.removeClass("mCSB_dragger_onDrag");
							});
					}
					$this.data({"bindEvent_scrollbar_drag":true});
				}
				function scrollbarDrag(mCSB_draggerDragY,mCSB_draggerDragX,y,x){
					if($this.data("horizontalScroll")){
						$this.mCustomScrollbar("scrollTo",(mCSB_dragger.position().left-(mCSB_draggerDragX))+x,{moveDragger:true,trigger:"internal"});
					}else{
						$this.mCustomScrollbar("scrollTo",(mCSB_dragger.position().top-(mCSB_draggerDragY))+y,{moveDragger:true,trigger:"internal"});
					}
				}
				/*content touch-drag*/
				if($.support.touch && $this.data("contentTouchScroll")){
					if(!$this.data("bindEvent_content_touch")){
						var touch,
							elem,elemOffset,y,x,mCSB_containerTouchY,mCSB_containerTouchX, mCSB_isMoveEvent;
						mCSB_container.bind("touchstart",function(e){
							/* Abbiamo commentato lo stop Propagation che viene eseguito in un secondo momento sul touchmove */
							/*e.stopImmediatePropagation();*/
							touch=e.originalEvent.touches[0] || e.originalEvent.changedTouches[0];
							elem=$(this);
							elemOffset=elem.offset();
							x=touch.pageX-elemOffset.left;
							y=touch.pageY-elemOffset.top;
							mCSB_containerTouchY=y;
							mCSB_containerTouchX=x;
							mCSB_isMoveEvent = 0;
						});
						mCSB_container.bind("touchmove",function(e){
							e.preventDefault(); e.stopImmediatePropagation();
							touch=e.originalEvent.touches[0] || e.originalEvent.changedTouches[0];
							elem=$(this).parent();
							elemOffset=elem.offset();
							x=touch.pageX-elemOffset.left;
							y=touch.pageY-elemOffset.top;
							if($this.data("horizontalScroll")){
								$this.mCustomScrollbar("scrollTo",mCSB_containerTouchX-x,{trigger:"internal"});
							}else{
								$this.mCustomScrollbar("scrollTo",mCSB_containerTouchY-y,{trigger:"internal"});
							}
							mCSB_isMoveEvent = 1;
						});
						mCSB_container.bind("touchend",function(e){
							if ( mCSB_isMoveEvent ) {
								e.preventDefault();
								e.stopImmediatePropagation();
							}
						});
					}
				}
				/*dragger rail click scrolling*/
				if(!$this.data("bindEvent_scrollbar_click")){
					mCSB_draggerContainer.bind("click",function(e){
						var scrollToPos=(e.pageY-mCSB_draggerContainer.offset().top)*$this.data("scrollAmount"),target=$(e.target);
						if($this.data("horizontalScroll")){
							scrollToPos=(e.pageX-mCSB_draggerContainer.offset().left)*$this.data("scrollAmount");
						}
						if(target.hasClass("mCSB_draggerContainer") || target.hasClass("mCSB_draggerRail")){
							$this.mCustomScrollbar("scrollTo",scrollToPos,{trigger:"internal",scrollEasing:"draggerRailEase"});
						}
					});
					$this.data({"bindEvent_scrollbar_click":true});
				}
				/*mousewheel scrolling*/
				if($this.data("mouseWheel")){
					if(!$this.data("bindEvent_mousewheel")){
						mCustomScrollBox.bind("mousewheel",function(e,delta){
							var scrollTo,mouseWheelPixels=$this.data("mouseWheelPixels"),absPos=Math.abs(mCSB_container.position().top),
								draggerPos=mCSB_dragger.position().top,limit=mCSB_draggerContainer.height()-mCSB_dragger.height();
							if($this.data("normalizeMouseWheelDelta")){
								if(delta<0){delta=-1;}else{delta=1;}
							}
							if(mouseWheelPixels==="auto"){
								mouseWheelPixels=100+Math.round($this.data("scrollAmount")/2);
							}
							if($this.data("horizontalScroll")){
								draggerPos=mCSB_dragger.position().left;
								limit=mCSB_draggerContainer.width()-mCSB_dragger.width();
								absPos=Math.abs(mCSB_container.position().left);
							}
							if((delta>0 && draggerPos!==0) || (delta<0 && draggerPos!==limit)){e.preventDefault(); e.stopImmediatePropagation();}
							scrollTo=absPos-(delta*mouseWheelPixels);
							$this.mCustomScrollbar("scrollTo",scrollTo,{trigger:"internal"});
						});
						$this.data({"bindEvent_mousewheel":true});
					}
				}
				/*buttons scrolling*/
				if($this.data("scrollButtons_enable")){
					if($this.data("scrollButtons_scrollType")==="pixels"){ /*scroll by pixels*/
						if($this.data("horizontalScroll")){
							mCSB_buttonRight.add(mCSB_buttonLeft).unbind("mousedown touchstart MSPointerDown pointerdown mouseup MSPointerUp pointerup mouseout MSPointerOut pointerout touchend",mCSB_buttonRight_stop,mCSB_buttonLeft_stop);
							$this.data({"bindEvent_buttonsContinuous_x":false});
							if(!$this.data("bindEvent_buttonsPixels_x")){
								/*scroll right*/
								mCSB_buttonRight.bind("click",function(e){
									e.preventDefault();
									PixelsScrollTo(Math.abs(mCSB_container.position().left)+$this.data("scrollButtons_scrollAmount"));
								});
								/*scroll left*/
								mCSB_buttonLeft.bind("click",function(e){
									e.preventDefault();
									PixelsScrollTo(Math.abs(mCSB_container.position().left)-$this.data("scrollButtons_scrollAmount"));
								});
								$this.data({"bindEvent_buttonsPixels_x":true});
							}
						}else{
							mCSB_buttonDown.add(mCSB_buttonUp).unbind("mousedown touchstart MSPointerDown pointerdown mouseup MSPointerUp pointerup mouseout MSPointerOut pointerout touchend",mCSB_buttonRight_stop,mCSB_buttonLeft_stop);
							$this.data({"bindEvent_buttonsContinuous_y":false});
							if(!$this.data("bindEvent_buttonsPixels_y")){
								/*scroll down*/
								mCSB_buttonDown.bind("click",function(e){
									e.preventDefault();
									PixelsScrollTo(Math.abs(mCSB_container.position().top)+$this.data("scrollButtons_scrollAmount"));
								});
								/*scroll up*/
								mCSB_buttonUp.bind("click",function(e){
									e.preventDefault();
									PixelsScrollTo(Math.abs(mCSB_container.position().top)-$this.data("scrollButtons_scrollAmount"));
								});
								$this.data({"bindEvent_buttonsPixels_y":true});
							}
						}
						function PixelsScrollTo(to){
							if(!mCSB_dragger.data("preventAction")){
								mCSB_dragger.data("preventAction",true);
								$this.mCustomScrollbar("scrollTo",to,{trigger:"internal"});
							}
						}
					}else{ /*continuous scrolling*/
						if($this.data("horizontalScroll")){
							mCSB_buttonRight.add(mCSB_buttonLeft).unbind("click");
							$this.data({"bindEvent_buttonsPixels_x":false});
							if(!$this.data("bindEvent_buttonsContinuous_x")){
								/*scroll right*/
								mCSB_buttonRight.bind("mousedown touchstart MSPointerDown pointerdown",function(e){
									e.preventDefault();
									var scrollButtonsSpeed=ScrollButtonsSpeed();
									$this.data({"mCSB_buttonScrollRight":setInterval(function(){
										$this.mCustomScrollbar("scrollTo",Math.abs(mCSB_container.position().left)+scrollButtonsSpeed,{trigger:"internal",scrollEasing:"easeOutCirc"});
									},17)});
								});
								var mCSB_buttonRight_stop=function(e){
									e.preventDefault(); clearInterval($this.data("mCSB_buttonScrollRight"));
								}
								mCSB_buttonRight.bind("mouseup touchend MSPointerUp pointerup mouseout MSPointerOut pointerout",mCSB_buttonRight_stop);
								/*scroll left*/
								mCSB_buttonLeft.bind("mousedown touchstart MSPointerDown pointerdown",function(e){
									e.preventDefault();
									var scrollButtonsSpeed=ScrollButtonsSpeed();
									$this.data({"mCSB_buttonScrollLeft":setInterval(function(){
										$this.mCustomScrollbar("scrollTo",Math.abs(mCSB_container.position().left)-scrollButtonsSpeed,{trigger:"internal",scrollEasing:"easeOutCirc"});
									},17)});
								});
								var mCSB_buttonLeft_stop=function(e){
									e.preventDefault(); clearInterval($this.data("mCSB_buttonScrollLeft"));
								}
								mCSB_buttonLeft.bind("mouseup touchend MSPointerUp pointerup mouseout MSPointerOut pointerout",mCSB_buttonLeft_stop);
								$this.data({"bindEvent_buttonsContinuous_x":true});
							}
						}else{
							mCSB_buttonDown.add(mCSB_buttonUp).unbind("click");
							$this.data({"bindEvent_buttonsPixels_y":false});
							if(!$this.data("bindEvent_buttonsContinuous_y")){
								/*scroll down*/
								mCSB_buttonDown.bind("mousedown touchstart MSPointerDown pointerdown",function(e){
									e.preventDefault();
									var scrollButtonsSpeed=ScrollButtonsSpeed();
									$this.data({"mCSB_buttonScrollDown":setInterval(function(){
										$this.mCustomScrollbar("scrollTo",Math.abs(mCSB_container.position().top)+scrollButtonsSpeed,{trigger:"internal",scrollEasing:"easeOutCirc"});
									},17)});
								});
								var mCSB_buttonDown_stop=function(e){
									e.preventDefault(); clearInterval($this.data("mCSB_buttonScrollDown"));
								}
								mCSB_buttonDown.bind("mouseup touchend MSPointerUp pointerup mouseout MSPointerOut pointerout",mCSB_buttonDown_stop);
								/*scroll up*/
								mCSB_buttonUp.bind("mousedown touchstart MSPointerDown pointerdown",function(e){
									e.preventDefault();
									var scrollButtonsSpeed=ScrollButtonsSpeed();
									$this.data({"mCSB_buttonScrollUp":setInterval(function(){
										$this.mCustomScrollbar("scrollTo",Math.abs(mCSB_container.position().top)-scrollButtonsSpeed,{trigger:"internal",scrollEasing:"easeOutCirc"});
									},17)});
								});
								var mCSB_buttonUp_stop=function(e){
									e.preventDefault(); clearInterval($this.data("mCSB_buttonScrollUp"));
								}
								mCSB_buttonUp.bind("mouseup touchend MSPointerUp pointerup mouseout MSPointerOut pointerout",mCSB_buttonUp_stop);
								$this.data({"bindEvent_buttonsContinuous_y":true});
							}
						}
						function ScrollButtonsSpeed(){
							var speed=$this.data("scrollButtons_scrollSpeed");
							if($this.data("scrollButtons_scrollSpeed")==="auto"){
								speed=Math.round(($this.data("scrollInertia")+100)/40);
							}
							return speed;
						}
					}
				}
				/*scrolling on element focus (e.g. via TAB key)*/
				if($this.data("autoScrollOnFocus")){
					if(!$this.data("bindEvent_focusin")){
						mCustomScrollBox.bind("focusin",function(){
							mCustomScrollBox.scrollTop(0).scrollLeft(0);
							var focusedElem=$(document.activeElement);
							if(focusedElem.is("input,textarea,select,button,a[tabindex],area,object")){
								var mCSB_containerPos=mCSB_container.position().top,
									focusedElemPos=focusedElem.position().top,
									visibleLimit=mCustomScrollBox.height()-focusedElem.outerHeight();
								if($this.data("horizontalScroll")){
									mCSB_containerPos=mCSB_container.position().left;
									focusedElemPos=focusedElem.position().left;
									visibleLimit=mCustomScrollBox.width()-focusedElem.outerWidth();
								}
								if(mCSB_containerPos+focusedElemPos<0 || mCSB_containerPos+focusedElemPos>visibleLimit){
									$this.mCustomScrollbar("scrollTo",focusedElemPos,{trigger:"internal"});
								}
							}
						});
						$this.data({"bindEvent_focusin":true});
					}
				}
				/*auto-hide scrollbar*/
				if($this.data("autoHideScrollbar") && !$this.data("alwaysShowScrollbar")){
					if(!$this.data("bindEvent_autoHideScrollbar")){
						mCustomScrollBox.bind("mouseenter",function(e){
							mCustomScrollBox.addClass("mCS-mouse-over");
							functions.showScrollbar.call(mCustomScrollBox.children(".mCSB_scrollTools"));
						}).bind("mouseleave touchend",function(e){
								mCustomScrollBox.removeClass("mCS-mouse-over");
								if(e.type==="mouseleave"){functions.hideScrollbar.call(mCustomScrollBox.children(".mCSB_scrollTools"));}
							});
						$this.data({"bindEvent_autoHideScrollbar":true});
					}
				}
			},
			scrollTo:function(scrollTo,options){
				var $this=$(this),
					defaults={
						moveDragger:false,
						trigger:"external",
						callbacks:true,
						scrollInertia:$this.data("scrollInertia"),
						scrollEasing:$this.data("scrollEasing")
					},
					options=$.extend(defaults,options),
					draggerScrollTo,
					mCustomScrollBox=$this.children(".mCustomScrollBox"),
					mCSB_container=mCustomScrollBox.children(".mCSB_container"),
					mCSB_scrollTools=mCustomScrollBox.children(".mCSB_scrollTools"),
					mCSB_draggerContainer=mCSB_scrollTools.children(".mCSB_draggerContainer"),
					mCSB_dragger=mCSB_draggerContainer.children(".mCSB_dragger"),
					contentSpeed=draggerSpeed=options.scrollInertia,
					scrollBeginning,scrollBeginningOffset,totalScroll,totalScrollOffset;
				if(!mCSB_container.hasClass("mCS_no_scrollbar")){
					$this.data({"mCS_trigger":options.trigger});
					if($this.data("mCS_Init")){options.callbacks=false;}
					if(scrollTo || scrollTo===0){
						if(typeof(scrollTo)==="number"){ /*if integer, scroll by number of pixels*/
							if(options.moveDragger){ /*scroll dragger*/
								draggerScrollTo=scrollTo;
								if($this.data("horizontalScroll")){
									scrollTo=mCSB_dragger.position().left*$this.data("scrollAmount");
								}else{
									scrollTo=mCSB_dragger.position().top*$this.data("scrollAmount");
								}
								draggerSpeed=0;
							}else{ /*scroll content by default*/
								draggerScrollTo=scrollTo/$this.data("scrollAmount");
							}
						}else if(typeof(scrollTo)==="string"){ /*if string, scroll by element position*/
							var target;
							if(scrollTo==="top"){ /*scroll to top*/
								target=0;
							}else if(scrollTo==="bottom" && !$this.data("horizontalScroll")){ /*scroll to bottom*/
								target=mCSB_container.outerHeight()-mCustomScrollBox.height();
							}else if(scrollTo==="left"){ /*scroll to left*/
								target=0;
							}else if(scrollTo==="right" && $this.data("horizontalScroll")){ /*scroll to right*/
								target=mCSB_container.outerWidth()-mCustomScrollBox.width();
							}else if(scrollTo==="first"){ /*scroll to first element position*/
								target=$this.find(".mCSB_container").find(":first");
							}else if(scrollTo==="last"){ /*scroll to last element position*/
								target=$this.find(".mCSB_container").find(":last");
							}else{ /*scroll to element position*/
								target=$this.find(scrollTo);
							}
							if(target.length===1){ /*if such unique element exists, scroll to it*/
								if($this.data("horizontalScroll")){
									scrollTo=target.position().left;
								}else{
									scrollTo=target.position().top;
								}
								draggerScrollTo=scrollTo/$this.data("scrollAmount");
							}else{
								draggerScrollTo=scrollTo=target;
							}
						}
						/*scroll to*/
						if($this.data("horizontalScroll")){
							if($this.data("onTotalScrollBack_Offset")){ /*scroll beginning offset*/
								scrollBeginningOffset=-$this.data("onTotalScrollBack_Offset");
							}
							if($this.data("onTotalScroll_Offset")){ /*total scroll offset*/
								totalScrollOffset=mCustomScrollBox.width()-mCSB_container.outerWidth()+$this.data("onTotalScroll_Offset");
							}
							if(draggerScrollTo<0){ /*scroll start position*/
								draggerScrollTo=scrollTo=0; clearInterval($this.data("mCSB_buttonScrollLeft"));
								if(!scrollBeginningOffset){scrollBeginning=true;}
							}else if(draggerScrollTo>=mCSB_draggerContainer.width()-mCSB_dragger.width()){ /*scroll end position*/
								draggerScrollTo=mCSB_draggerContainer.width()-mCSB_dragger.width();
								scrollTo=mCustomScrollBox.width()-mCSB_container.outerWidth(); clearInterval($this.data("mCSB_buttonScrollRight"));
								if(!totalScrollOffset){totalScroll=true;}
							}else{scrollTo=-scrollTo;}
							var snapAmount = $this.data("snapAmount");
							if (snapAmount) {
								scrollTo = Math.round(scrollTo / snapAmount) * snapAmount - $this.data("snapOffset");
							}
							/*scrolling animation*/
							functions.mTweenAxis.call(this,mCSB_dragger[0],"left",Math.round(draggerScrollTo),draggerSpeed,options.scrollEasing);
							functions.mTweenAxis.call(this,mCSB_container[0],"left",Math.round(scrollTo),contentSpeed,options.scrollEasing,{
								onStart:function(){
									if(options.callbacks && !$this.data("mCS_tweenRunning")){callbacks("onScrollStart");}
									if($this.data("autoHideScrollbar") && !$this.data("alwaysShowScrollbar")){functions.showScrollbar.call(mCSB_scrollTools);}
								},
								onUpdate:function(){
									if(options.callbacks){callbacks("whileScrolling");}
								},
								onComplete:function(){
									if(options.callbacks){
										callbacks("onScroll");
										if(scrollBeginning || (scrollBeginningOffset && mCSB_container.position().left>=scrollBeginningOffset)){callbacks("onTotalScrollBack");}
										if(totalScroll || (totalScrollOffset && mCSB_container.position().left<=totalScrollOffset)){callbacks("onTotalScroll");}
									}
									mCSB_dragger.data("preventAction",false); $this.data("mCS_tweenRunning",false);
									if($this.data("autoHideScrollbar") && !$this.data("alwaysShowScrollbar")){if(!mCustomScrollBox.hasClass("mCS-mouse-over")){functions.hideScrollbar.call(mCSB_scrollTools);}}
								}
							});
						}else{
							if($this.data("onTotalScrollBack_Offset")){ /*scroll beginning offset*/
								scrollBeginningOffset=-$this.data("onTotalScrollBack_Offset");
							}
							if($this.data("onTotalScroll_Offset")){ /*total scroll offset*/
								totalScrollOffset=mCustomScrollBox.height()-mCSB_container.outerHeight()+$this.data("onTotalScroll_Offset");
							}
							if(draggerScrollTo<0){ /*scroll start position*/
								draggerScrollTo=scrollTo=0; clearInterval($this.data("mCSB_buttonScrollUp"));
								if(!scrollBeginningOffset){scrollBeginning=true;}
							}else if(draggerScrollTo>=mCSB_draggerContainer.height()-mCSB_dragger.height()){ /*scroll end position*/
								draggerScrollTo=mCSB_draggerContainer.height()-mCSB_dragger.height();
								scrollTo=mCustomScrollBox.height()-mCSB_container.outerHeight(); clearInterval($this.data("mCSB_buttonScrollDown"));
								if(!totalScrollOffset){totalScroll=true;}
							}else{scrollTo=-scrollTo;}
							var snapAmount = $this.data("snapAmount");
							if (snapAmount) {
								scrollTo = Math.round(scrollTo / snapAmount) * snapAmount - $this.data("snapOffset");
							}
							/*scrolling animation*/
							functions.mTweenAxis.call(this,mCSB_dragger[0],"top",Math.round(draggerScrollTo),draggerSpeed,options.scrollEasing);
							functions.mTweenAxis.call(this,mCSB_container[0],"top",Math.round(scrollTo),contentSpeed,options.scrollEasing,{
								onStart:function(){
									if(options.callbacks && !$this.data("mCS_tweenRunning")){callbacks("onScrollStart");}
									if($this.data("autoHideScrollbar") && !$this.data("alwaysShowScrollbar")){functions.showScrollbar.call(mCSB_scrollTools);}
								},
								onUpdate:function(){
									if(options.callbacks){callbacks("whileScrolling");}
								},
								onComplete:function(){
									if(options.callbacks){
										callbacks("onScroll");
										if(scrollBeginning || (scrollBeginningOffset && mCSB_container.position().top>=scrollBeginningOffset)){callbacks("onTotalScrollBack");}
										if(totalScroll || (totalScrollOffset && mCSB_container.position().top<=totalScrollOffset)){callbacks("onTotalScroll");}
									}
									mCSB_dragger.data("preventAction",false); $this.data("mCS_tweenRunning",false);
									if($this.data("autoHideScrollbar") && !$this.data("alwaysShowScrollbar")){if(!mCustomScrollBox.hasClass("mCS-mouse-over")){functions.hideScrollbar.call(mCSB_scrollTools);}}
								}
							});
						}
						if($this.data("mCS_Init")){$this.data({"mCS_Init":false});}
					}
				}
				/*callbacks*/
				function callbacks(cb){
					if ($this.data("mCustomScrollbarIndex")) {
						this.mcs = {
							top: mCSB_container.position().top, left: mCSB_container.position().left,
							draggerTop: mCSB_dragger.position().top, draggerLeft: mCSB_dragger.position().left,
							topPct: Math.round((100 * Math.abs(mCSB_container.position().top)) / Math.abs(mCSB_container.outerHeight() - mCustomScrollBox.height())),
							leftPct: Math.round((100 * Math.abs(mCSB_container.position().left)) / Math.abs(mCSB_container.outerWidth() - mCustomScrollBox.width()))
						};
						switch (cb) {
							/*start scrolling callback*/
							case "onScrollStart":
								$this.data("mCS_tweenRunning", true).data("onScrollStart_Callback").call($this, this.mcs);
								break;
							case "whileScrolling":
								$this.data("whileScrolling_Callback").call($this, this.mcs);
								break;
							case "onScroll":
								$this.data("onScroll_Callback").call($this, this.mcs);
								break;
							case "onTotalScrollBack":
								$this.data("onTotalScrollBack_Callback").call($this, this.mcs);
								break;
							case "onTotalScroll":
								$this.data("onTotalScroll_Callback").call($this, this.mcs);
								break;
						}
					}
				}
			},
			stop:function(){
				var $this=$(this),
					mCSB_container=$this.children().children(".mCSB_container"),
					mCSB_dragger=$this.children().children().children().children(".mCSB_dragger");
				functions.mTweenAxisStop.call(this,mCSB_container[0]);
				functions.mTweenAxisStop.call(this,mCSB_dragger[0]);
			},
			disable:function(resetScroll){
				var $this=$(this),
					mCustomScrollBox=$this.children(".mCustomScrollBox"),
					mCSB_container=mCustomScrollBox.children(".mCSB_container"),
					mCSB_scrollTools=mCustomScrollBox.children(".mCSB_scrollTools"),
					mCSB_dragger=mCSB_scrollTools.children().children(".mCSB_dragger");
				mCustomScrollBox.unbind("mousewheel focusin mouseenter mouseleave touchend");
				mCSB_container.unbind("touchstart touchmove")
				if(resetScroll){
					if($this.data("horizontalScroll")){
						mCSB_dragger.add(mCSB_container).css("left",0);
					}else{
						mCSB_dragger.add(mCSB_container).css("top",0);
					}
				}
				mCSB_scrollTools.css("display","none");
				mCSB_container.addClass("mCS_no_scrollbar");
				$this.data({"bindEvent_mousewheel":false,"bindEvent_focusin":false,"bindEvent_content_touch":false,"bindEvent_autoHideScrollbar":false}).addClass("mCS_disabled");
			},
			destroy:function(){
				var $this=$(this);
				$this.removeClass("mCustomScrollbar _mCS_"+$this.data("mCustomScrollbarIndex")).addClass("mCS_destroyed").children().children(".mCSB_container").unwrap().children().unwrap().siblings(".mCSB_scrollTools").remove();
				$(document).unbind("mousemove."+$this.data("mCustomScrollbarIndex")+" mouseup."+$this.data("mCustomScrollbarIndex")+" MSPointerMove."+$this.data("mCustomScrollbarIndex")+" MSPointerUp."+$this.data("mCustomScrollbarIndex"));
				$(window).unbind("resize."+$this.data("mCustomScrollbarIndex"));
			}
		},
		functions={
			/*hide/show scrollbar*/
			showScrollbar:function(){
				this.stop().animate({opacity:1},"fast");
			},
			hideScrollbar:function(){
				this.stop().animate({opacity:0},"fast");
			},
			/*js animation tween*/
			mTweenAxis:function(el,prop,to,duration,easing,callbacks){
				var callbacks=callbacks || {},
					onStart=callbacks.onStart || function(){},onUpdate=callbacks.onUpdate || function(){},onComplete=callbacks.onComplete || function(){};
				var startTime=_getTime(),_delay,progress=0,from=el.offsetTop,elStyle=el.style;
				if(prop==="left"){from=el.offsetLeft;}
				var diff=to-from;
				_cancelTween();
				_startTween();
				function _getTime(){
					if(window.performance && window.performance.now){
						return window.performance.now();
					}else{
						if(window.performance && window.performance.webkitNow){
							return window.performance.webkitNow();
						}else{
							if(Date.now){return Date.now();}else{return new Date().getTime();}
						}
					}
				}
				function _step(){
					if(!progress){onStart.call();}
					progress=_getTime()-startTime;
					_tween();
					if(progress>=el._time){
						el._time=(progress>el._time) ? progress+_delay-(progress- el._time) : progress+_delay-1;
						if(el._time<progress+1){el._time=progress+1;}
					}
					if(el._time<duration){el._id=_request(_step);}else{onComplete.call();}
				}
				function _tween(){
					if(duration>0){
						el.currVal=_ease(el._time,from,diff,duration,easing);
						elStyle[prop]=Math.round(el.currVal)+"px";
					}else{
						elStyle[prop]=to+"px";
					}
					onUpdate.call();
				}
				function _startTween(){
					_delay=1000/60;
					el._time=progress+_delay;
					_request=(!window.requestAnimationFrame) ? function(f){_tween(); return setTimeout(f,0.01);} : window.requestAnimationFrame;
					el._id=_request(_step);
				}
				function _cancelTween(){
					if(el._id==null){return;}
					if(!window.requestAnimationFrame){clearTimeout(el._id);
					}else{window.cancelAnimationFrame(el._id);}
					el._id=null;
				}
				function _ease(t,b,c,d,type){
					switch(type){
						case "linear":
							return c*t/d + b;
							break;
						case "easeOutQuad":
							t /= d; return -c * t*(t-2) + b;
							break;
						case "easeInOutQuad":
							t /= d/2;
							if (t < 1) return c/2*t*t + b;
							t--;
							return -c/2 * (t*(t-2) - 1) + b;
							break;
						case "easeOutCubic":
							t /= d; t--; return c*(t*t*t + 1) + b;
							break;
						case "easeOutQuart":
							t /= d; t--; return -c * (t*t*t*t - 1) + b;
							break;
						case "easeOutQuint":
							t /= d; t--; return c*(t*t*t*t*t + 1) + b;
							break;
						case "easeOutCirc":
							t /= d; t--; return c * Math.sqrt(1 - t*t) + b;
							break;
						case "easeOutSine":
							return c * Math.sin(t/d * (Math.PI/2)) + b;
							break;
						case "easeOutExpo":
							return c * ( -Math.pow( 2, -10 * t/d ) + 1 ) + b;
							break;
						case "mcsEaseOut":
							var ts=(t/=d)*t,tc=ts*t;
							return b+c*(0.499999999999997*tc*ts + -2.5*ts*ts + 5.5*tc + -6.5*ts + 4*t);
							break;
						case "draggerRailEase":
							t /= d/2;
							if (t < 1) return c/2*t*t*t + b;
							t -= 2;
							return c/2*(t*t*t + 2) + b;
							break;
					}
				}
			},
			/*stop js animation tweens*/
			mTweenAxisStop:function(el){
				if(el._id==null){return;}
				if(!window.requestAnimationFrame){clearTimeout(el._id);
				}else{window.cancelAnimationFrame(el._id);}
				el._id=null;
			},
			/*detect requestAnimationFrame and polyfill*/
			rafPolyfill:function(){
				var pfx=["ms","moz","webkit","o"],i=pfx.length;
				while(--i > -1 && !window.requestAnimationFrame){
					window.requestAnimationFrame=window[pfx[i]+"RequestAnimationFrame"];
					window.cancelAnimationFrame=window[pfx[i]+"CancelAnimationFrame"] || window[pfx[i]+"CancelRequestAnimationFrame"];
				}
			}
		}
	/*detect features*/
	functions.rafPolyfill.call(); /*requestAnimationFrame*/
	$.support.touch=!!('ontouchstart' in window); /*touch*/
	$.support.pointer=window.navigator.pointerEnabled; /*pointer support*/
	$.support.msPointer=window.navigator.msPointerEnabled; /*MSPointer support*/
	/*plugin dependencies*/
	var _dlp=("https:"==document.location.protocol) ? "https:" : "http:";
	$.event.special.mousewheel || document.write('<script src="'+_dlp+'//cdnjs.cloudflare.com/ajax/libs/jquery-mousewheel/3.0.6/jquery.mousewheel.min.js"><\/script>');
	/*plugin fn*/
	$.fn.mCustomScrollbar=function(method){
		if(methods[method]){
			return methods[method].apply(this,Array.prototype.slice.call(arguments,1));
		}else if(typeof method==="object" || !method){
			return methods.init.apply(this,arguments);
		}else{
			$.error("Method "+method+" does not exist");
		}
	};
})(jQuery);;
/*Script per gestione box Overlay (pseudo LightBox)*/
if (!document.getElementById('boxOverlayContainer'))
	document.write("<div id=\"boxOverlayContainer\"></div>");

var boxOverlay = function(idBox,theme,options){
	this.idBox = idBox;	
	this.header = null;
	this.footer = null;
	this.footer_cont = null;
	this.hasFooter = false;
	this.content = null;
	this.closeEvents = null;
	this.showEvents = null;
	this.divBoxOverlay = null;
	this.doubleBorder = false;
	this.divBoxOverlayIframe = null;
	this.boxWrapper = null;
	this.closeButton = null;
	this.bottomBorder = null;
	this.scroll = null;
	this.destroyOnClose = true;
	this.box = null;
	this.theme = theme ? theme.charAt(0).toUpperCase() + theme.substr(1).toLowerCase() : "";
	this.blockScroll = false;
	this.fixedPosition = true;
	this.stop_event_bubbling = false;
	this.stop_event_bubbling_added = false;
	this.fixedCoords = {};
	this.isOpen = false;
	this.firstShow = true;
	this.onlyColoumnCenter = false;
	this.whiteHeader = false;
	this.manageConflicts = true;
	this.zIndex = null;
	this.closeLabel = "Chiudi";
	this.closeLabelTranslate = {"it": "Chiudi", "en": "Close", "es": "Cerrar", "fr": "Fermer", "de": "Schließen", "pt": "Fechar", "gr": "Κλείνω", "ru": "Закрыть"};
	
	if (typeof(options) != "undefined" && typeof(options.lang) != "undefined"){
		this.lang = options.lang;
	} else {
		this.lang = this.getPageLang();
	}

	if (this.lang){
		this.closeLabel = this.closeLabelTranslate[this.lang] ? this.closeLabelTranslate[this.lang] : "Chiudi";
	}

	/**
	 * Se true indica che la chiusura del boxOverlay sara' impedita. Default a false
	 */
	this.avoidClosing = (typeof(options) != 'undefined' && typeof(options.avoidClosing) != "undefined") ? options.avoidClosing : false;
	this.hideCloseBtn = (typeof(options) != 'undefined' && typeof(options.hideCloseBtn) != "undefined") ? options.hideCloseBtn : false;

	this.exitOnEsc(this); 
	
	this._hideOnlyBox = false;
	
	addEvent(window,"unload",this.destroy.bind(this))
	
	if (typeof options!="undefined"){
		if (typeof options.onlyColoumnCenter != "undefined" && options.onlyColoumnCenter)
			this.onlyColoumnCenter = true;
			
		if (typeof options.whiteHeader != "undefined" && options.whiteHeader)
			this.whiteHeader = true;
	}
	
	this.init();
	this.isOpen = false;
	
	return this;
};

boxOverlay.prototype.getPageLang = function(){
	if (document.getElementsByTagName('html')[0].getAttribute('lang')){
		return document.getElementsByTagName('html')[0].getAttribute('lang');
	}
}

boxOverlay.prototype.init = function(){
	this.appendCss();
	this.createBox();
	this.initVars();
}

boxOverlay.prototype.initVars = function(){
	if (!this.idBox){
		throw("oggetto istanziato in maniera non corretta. Utilizzare  new boxOverlay('<idBox>')");
		return;
	}
	
	/* gestione closeButton */
	if(!this.avoidClosing){
		this.closeButton = document.getElementById("close_"+this.idBox);
		if (this.closeButton)
			this.closeButton.onclick = this.close.bind(this);
	}
	this.title = document.getElementById(this.idBox+"_title");
	this.footer = document.getElementById(this.idBox+"_footer");
	this.bottomBorder = document.getElementById(this.idBox+"_bottom_border") ? document.getElementById(this.idBox+"_bottom_border") : null;
	this.footer_cont = document.getElementById(this.idBox+"_footer_cont");
	this.divBoxOverlayIframe = document.getElementById("boxOverlayIFrame"+(this.theme ? "_"+this.theme : "")+(this.idBox ? "_"+this.idBox : ""));
	
	this.content = document.getElementById(this.idBox+"_content");
	this.box = document.getElementById(this.idBox);
	
	
	var ovShadow = getElementsByClass("divBoxOverlayShadow"+(this.theme ? "_"+this.theme : ""));
	if (ovShadow.length > 0) {
		this.divBoxOverlay = ovShadow[0];
	}

    /*
    if (this.divBoxOverlay)
        addEvent(this.divBoxOverlay,"click",this.close.bind(this));
    */
    _ovShOnBoxRes = this.reShowOnResize;
    this.addEvent(window,"resize", _ovShOnBoxRes.bind(this));
	
	//if (this.blockScroll){
		addEvent(window,"scroll",this.catchScroll.bind(this))
	//}
}

boxOverlay.prototype.close = function(){
	if (this.closeEvents && !this.avoidClosing){
		for (var i=0; i<this.closeEvents.length; i++){
			this.closeEvents[i]();
		}	
	}
    if(!this.avoidClosing){
		if (!this._hideOnlyBox && this.hide)
			this.hide();
		else
        if (this.hideOnlyBox)
			this.hideOnlyBox();

		if (this.destroyOnClose){
			this.destroy();
		}
	}
}

boxOverlay.prototype.createBox = function(){
	if (document.getElementById(this.idBox)){
		document.getElementById(this.idBox).parentNode.removeChild(document.getElementById(this.idBox));
		//throw("overlayBox con nome '" + this.idBox +"' esistente");
		//return;
	}
	if (!this.idBox){
		throw("oggetto istanziato in maniera non corretta. Utilizzare  new boxOverlay('<idBox>')");
		return;
	}
	var divBoxOverlayShadowClassSelector = "divBoxOverlayShadow"+(this.theme ? "_"+this.theme : "");
	var ovShadow = getElementsByClass(divBoxOverlayShadowClassSelector);

	if (!ovShadow.length){
		var html = "<div class=\""+divBoxOverlayShadowClassSelector+" divBoxOverlay"+(this.theme ? "_"+this.theme : "")+"\"></div>";
		var oDiv = document.createElement("div");
		oDiv.innerHTML = html;
		this.divBoxOverlay = oDiv;
		document.getElementById("boxOverlayContainer").appendChild(oDiv);
		
	}else{
		this.divBoxOverlay = ovShadow[0];
	}

	if (!document.getElementById("boxOverlayIFrame"+(this.theme ? "_"+this.theme : "")+(this.idBox ? "_"+this.idBox : ""))){
		var iframeHTML = "<iframe id=\"boxOverlayIFrame"+(this.theme ? "_"+this.theme : "")+(this.idBox ? "_"+this.idBox : "")+"\" class=\"boxOverlayIFrame"+(this.theme ? "_"+this.theme : "")+"\" src=\"about:blank\" style=\"z-index:101;display:none;border:none;\" border=\"none\" scrolling=\"no\"  frameborder=\"0\" vspace=\"0\" hspace=\"0\" marginheight=\"0\" marginwidth=\"0\"></iframe>";
		var oDiv = document.createElement("div");
		oDiv.innerHTML = iframeHTML;
		document.getElementById("boxOverlayContainer").appendChild(oDiv);
	}
	html = this.getBox();

	this.boxWrapper = document.createElement("div");
	this.boxWrapper.id = "boxOverlayWrap_"+this.idBox;
	this.boxWrapper.innerHTML = html;
	document.getElementById("boxOverlayContainer").appendChild(this.boxWrapper);

}

boxOverlay.prototype.appendCss = function(){
	if (!document.getElementById('overlayCss_'+this.theme)){
		var headID = document.getElementsByTagName("head")[0];
		var cssNode = document.createElement('link');
		cssNode.type = 'text/css';
		cssNode.rel = 'stylesheet';
		cssNode.id = 'overlayCss_'+this.theme;
		cssNode.href = '/includes/overlayBox/boxOverlay'+(this.theme ? this.theme : "")+'.css' + (typeof __gvs_EXTERNAL_LIBRARY_VERSION != 'undefined' ? __gvs_EXTERNAL_LIBRARY_VERSION : '');
		cssNode.media = 'screen';
		headID.appendChild(cssNode);
	}
}

boxOverlay.prototype.catchScroll = function(){
	if (this.blockScroll && this.scroll)
		Page.scroll(this.scroll);
	}

boxOverlay.prototype.show = function(){
	if (!this.idBox || !this.box){
		throw("box non presente");
		return;
	}
	if (typeof reshow == "undefined") {
		reshow = false;
	}
	if (this.stop_event_bubbling && !this.stop_event_bubbling_added){
		this.stop_event_bubbling_added = true;
		addEvent( this.box , "click",  stop_event_bubbling) 
		addEvent( this.divBoxOverlay , "click",  stop_event_bubbling)
	}
	
	if (this.hasFooter){
		this.footer_cont.style.display="";
	}else{
		if (document.getElementById(this.idBox+"_pop_content") && this.firstShow){
			document.getElementById(this.idBox+"_pop_content").className += " ov_white_border";
		}		
	}

	if (this.firstShow){
		//addEvent(window,"load",this.show.bind(this))
		this.firstShow = false;
	}

	if (this.doubleBorder)
		document.getElementById(this.idBox+"_doubledBorder").style.padding="10px";
		
	var pWidth = document.body.clientWidth ? document.body.clientWidth : document.documentElement.clientWidth;
	var pHeight = document.body.clientHeight ? document.body.clientHeight : document.documentElement.clientHeight;
	
	var sWidth = window.innerWidth ? window.innerWidth : (document.documentElement.clientWidth ? document.documentElement.clientWidth : pWidth);
	var sHeight = window.innerHeight ? window.innerHeight : (document.documentElement.clientHeight ? document.documentElement.clientHeight : pHeight);
    
	this.box.style.position = "absolute";
	
	var divWidth = MEgetWidth(this.box);
	var divHeight = MEgetHeight(this.box);
	

	//Se la risoluzioni è ugale o più piccola della dimensione del box
    if ( sWidth <= divWidth || sHeight <= divHeight ) {		
        this.fixedPosition = false;
    }
	this.divBoxOverlay.style.position =  "fixed";
	var info = this._getBrowser();
	if (info && info["name"].search(/explorer/i) && info["version"] < 9) {
		if (parseInt(pHeight) == parseInt(sHeight)) {
			this.fixedPosition = false;
			this.divBoxOverlay.style.position = "absolute";
			this.divBoxOverlay.style.zIndex = 101;
		}
	}
	
	if (this.zIndex)
		this.box.style.zIndex = this.zIndex;
	
	if (this.fixedPosition)
		this.box.style.position = "fixed";

	var IE6 = false /*@cc_on || @_jscript_version < 5.7 @*/;
	if (IE6)
		this.box.style.position = "absolute";
	
	var scrolly = 0;
	if (!this.fixedPosition)
		scrolly = (document.all)?document.documentElement.scrollTop:window.pageYOffset;
	this.scroll=Page.scroll();

	this.divBoxOverlay.style.top =  "0px";
	/*if (pHeight > sHeight)
		this.divBoxOverlay.style.height = pHeight + "px";
	else
	*/
		this.divBoxOverlay.style.height = sHeight + "px";
	
	/*if (parseInt(sWidth) <= 1024){
		this.divBoxOverlay.style.width = "993px";
	}else{
	*/
		this.divBoxOverlay.style.width = "100%";
	//}

	this.divBoxOverlay.style.display="block";

	var top;
	if (!this.fixedCoords.top){
      top = scrolly + (sHeight/2 - divHeight/2);
      /*if (top<175) top = 175;*/
	}else
		top = scrolly + this.fixedCoords.top;
	
	if (sHeight<=divHeight)
		top = scrolly + 5;
		
	top+="px";

	var left
	if (!this.fixedCoords.left){
		left = (sWidth/2 - divWidth/2);
	}else{
		left = this.fixedCoords.left;
	}
	left+="px";
	
	var iFrame = this.divBoxOverlayIframe;
	if (this.fixedPosition)
		iFrame.style.position = "fixed";
	else	
		iFrame.style.position = "absolute";
	var IE6 = false /*@cc_on || @_jscript_version < 5.7 @*/;
	if (IE6)
		iFrame.style.position = "absolute";
	
	iFrame.style.height = divHeight + "px";
	iFrame.style.width = divWidth + "px";
	iFrame.style.left = left;
	iFrame.style.top = top;
	var IE6 = false /*@cc_on || @_jscript_version < 5.7 @*/;
	if (IE6)
		iFrame.style.display="inline";
	else
		iFrame.style.display="none";
	
	document.getElementById(this.idBox).style.top = top;
	document.getElementById(this.idBox).style.left = left;

	if (this.manageConflicts)
		this.manageConflictComponents('hide');

	if (document.getElementById(this.idBox).style.display!="block") {
		this.isOpen = true;
		document.getElementById(this.idBox).style.visibility="visible";
		document.getElementById(this.idBox).style.display="block";
	}

	try{
		if (this.showEvents){
			for (var i in this.showEvents){
				this.showEvents[i]();
			}
		}
	}catch(e){
	
	}
}

/* Nascondo i frame perchè i contentuti flash esterni non settati con wmode transparent vengono al di sopra del box overlay */
boxOverlay.prototype.manageConflictComponents = function(action){
	var Iframes = document.getElementsByTagName('iFrame');
	switch (action){
		case 'hide':
			var IntIframes = _getElementsByTagName('iFrame',document.getElementById(this.idBox));
			for (i=0; i< Iframes.length;i++){
				var skipConflict = Iframes[i].getAttribute('skipOverlayManageConflict');
				
				if (!skipConflict && Iframes[i]!=this.divBoxOverlayIframe && !in_array(Iframes[i],IntIframes)){										
					Iframes[i].style.visibility = 'hidden';
				}
			}
			break;
			
		case 'show':
			var IntIframes = _getElementsByTagName('iFrame',document.getElementById(this.idBox));
			for (i=0; i< Iframes.length;i++){
				var skipConflict = Iframes[i].getAttribute('skipOverlayManageConflict');
				
				if (!skipConflict && Iframes[i]!=this.divBoxOverlayIframe && !in_array(Iframes[i],IntIframes)){					
					Iframes[i].style.visibility = 'visible';
				}
			}
			break;
	}
}

boxOverlay.prototype.checkExitBtn = function(e){
		var kC  = (window.event) ?    // MSIE or Firefox?
	           window.event.keyCode : (e.keyCode ? e.keyCode : e.which);
		var Esc = (window.event) ?   
	          27 : e.DOM_VK_ESCAPE // MSIE : Firefox  
		if(kC==Esc){
			if (this.close)
			  this.close();
		}
   }
boxOverlay.prototype._reShowOnResize = function(){
	if (document.getElementById(this.idBox).style.display=="block"){
		this.show();
	}
}

boxOverlay.prototype.reShowOnResize = function(){
	if (typeof this.resizeTimeout != "undefined")
		clearTimeout(this.resizeTimeout);
	try{
		this.resizeTimeout = setTimeout(this._reShowOnResize.bind(this),100);
	}	catch(e){
	
	}
}

   
boxOverlay.prototype.hideOnlyBox = function(){
	
	if (!this.idBox || !document.getElementById(this.idBox)){
		throw("box non presente");
		return;
	}
	this.box.style.display="none";
	this.isOpen = false;
	
}
   
boxOverlay.prototype.hide = function()
{
		
		var idBox;
		var closeEvent;
		//removeEvent(document,"keyup", this.checkExitBtn.bind(this));
		this.scroll = null;
	
		idBox = this.idBox;
		closeEvent = this.closeEvent ? this.closeEvent : null;
		
		if (!idBox || !document.getElementById(idBox)){
			throw("box non presente");
			return;
		}

		this.divBoxOverlay.style.display="none";
		this.divBoxOverlayIframe.style.display="none";
		if (this.manageConflicts)
			this.manageConflictComponents('show');
		this.box.style.display="none";
		this.isOpen = false;
		
		/*if (this.stop_event_bubbling){
			setTimeout(function(){
					try{
						removeEvent( this.box , "click",  stop_event_bubbling) 
						removeEvent( this.divBoxOverlay , "click",  stop_event_bubbling)
					}catch(e){
						
					}
				}.bind(this),1000);
		}*/
	}

boxOverlay.prototype.setTitle = function(headerContent){
	if (!this.idBox || !this.box){
		throw("box non presente");
		return;
	}
	this.title.innerHTML = headerContent; 
	
}
boxOverlay.prototype.setFooter = function(footerContent){
	
	if (!this.idBox || !this.box){
		throw("box non presente");
		return;
	}
	if (typeof footerContent == "undefined" || !footerContent){
		this.hasFooter = false;
	}else{
		this.hasFooter = true;
	}
	this.footer.innerHTML = footerContent;
}

boxOverlay.prototype.hideFooter = function(){
	if (!this.idBox || !this.box){
		throw("box non presente");
		return;
	}
	if(!jq.isEmpty(this.footer)){
		jq(this.footer).css("display","none");
		jq(this.footer).parent().css("display","none");
	}
}

boxOverlay.prototype.setContent = function(bodyContent){
	if (!this.idBox || !this.box){
		throw("box non presente");
		return;
	}
	this.content.innerHTML = bodyContent;
	if (this.isOpen){
		this.hide();
        this.show();
    }
}

boxOverlay.prototype.getContent = function(){
	
	return this.content.innerHTML; 
}
boxOverlay.prototype.addEventOnClose = function(func){

	if (!this.closeEvents)
		this.closeEvents = new Array();
	if(!this.avoidClosing)
		this.closeEvents.push(func);
}

boxOverlay.prototype.addEventOnShow = function(func, idHandler){
	
	if (!this.showEvents)
		this.showEvents = new Array();
	if (typeof idHandler != 'undefined')
		this.showEvents[idHandler] = func;
	else
		this.showEvents.push(func);
}

boxOverlay.prototype.resetEventsOnShow = function(){
	this.showEvents = new Array();
}

boxOverlay.prototype.removeEventOnClose = function(func){
	if (this.closeEvents){
		for (var i=0; i<this.closeEvents.length; i++){
			if (this.closeEvents[i] == func){
				this.closeEvents[i] = function(){};
			}
		}	
	}
}

boxOverlay.prototype.resetEventsOnClose = function(){
	if (this.closeEvents){
		this.closeEvents = new Array();
	}
}

boxOverlay.prototype.removeEventOnShow = function(idHandler){
	if (this.showEvents){
		for (var i in this.showEvents){
			if (i == idHandler){
				this.showEvents[i] = function(){};
			}
		}	
	}
}


boxOverlay.prototype.destroy = function(){
	
	try{
		if (!this.idBox || !this.box){
			throw("box non presente");
			return;
		}
		this.box.parentNode.removeChild(this.box);
		removeEvent(window,"unload",this.destroy.bind(this));
		this.purgeDomObj(this.boxWrapper);
		this.cleanVars();
	}catch(e){
		//alert(e);
	}
}

boxOverlay.prototype.exitOnEsc = function(obj){
	this.addEvent(document,"keyup", this.checkExitBtn.bind(this));
}

boxOverlay.prototype.addEvent = function(obj,ev,fn){
	if(obj.addEventListener) {
		// metodo w3c
		obj.addEventListener(ev, fn, false);
	} else if(obj.attachEvent) {
		// metodo IE
		obj.attachEvent('on'+ev, fn);
	} else {
		// se i suddetti metodi non sono applicabili
		// se esiste gia' una funzione richiamata da quel gestore evento
		if(typeof(obj['on'+ev])=='function'){
			// salvo in variabile la funzione gia' associata al gestore
			var f=obj['on'+ev];
			// setto per quel gestore una nuova funzione 
			// che comprende la vecchia e la nuova
			obj['on'+ev]=function(){if(f)f();fn()}
		}
		// altrimenti setto la funzione per il gestore
		else obj['on'+ev]=fn;
	}
}

boxOverlay.prototype.getPosition = function(){
	var position=new Object();
	position.top = this.box.style.top;
	position.left = this.box.style.left;
	return position;
}

boxOverlay.prototype.setPosition = function(obj){
	
	this.fixedCoords = {};

	if (typeof obj != "object" || (!obj.top && !obj.left)){
		return;
	}
	
	if (!obj.top && !obj.left)
		return;
	
	if (obj.top)
		this.fixedCoords.top = parseInt(obj.top);
		
	if (obj.left)
		this.fixedCoords.left = parseInt(obj.left);
	
}

boxOverlay.prototype.getBox = function(){
	var html = "";
	
	switch (this.theme){
		case "Grey" :
			return this.greyBox();
			break;
		case "Grey2" :
			return this.greyBox2();
			break;
		default :
			return this.defaultBox();
	}
	
}

boxOverlay.prototype.defaultBox = function(){
var html = "<div style=\"display:none\" id=\""+this.idBox+"\" class=\"boxOverlay"+(this.theme ? "_"+this.theme : "")+"\">"+
"	<table  cellspacing=\"0\" cellpadding=\"0\" border=\"0\" >"+
"		<tr>"+
"			<td class=\"pop_topleft \"></td>"+
"			<td class=\"pop_borderTop\"></td>"+
"			<td class=\"pop_topright\"></td>"+
"		</tr>"+
"		<tr>"+
"			<td class=\"pop_borderLateral\"></td>"+
"			<td class=\"pop_content\" style=\"\">"+
"				<div id=\""+this.idBox+"_doubledBorder\" class=\"doubleBorder\">"+
"				<div class=\"boxHeader\">"+
"					<div id=\""+this.idBox+"_title\" class=\"title\">"+
"					</div>";

if(!this.avoidClosing && !this.hideCloseBtn){
html += "			<a id=\"close_"+this.idBox+"\" class=\"closeButton\">"+ this.closeLabel + "&nbsp;</a>";
}
html += "			<div style=\"clear:both\"></div>"+
"				</div>"+


"					<div id=\""+this.idBox+"_content\" class=\"boxBody\">"+
"					</div>"+

		
"				</div>"+
"				<div id=\""+this.idBox+"_footer_cont\" class=\"boxFooter\" style=\"display:none\">"+
"					<div id=\""+this.idBox+"_footer\">"+
"					</div>"+
"					<div style=\"clear:both\"></div>"+
"			</div>"+
			"</td>"+
"			<td class=\"pop_borderLateral\"></td>"+
"		</tr>"+
"		<tr>"+
"			<td class=\"pop_bottomleft\"></td>"+
"			<td class=\"pop_borderTop\"></td>"+
"			<td class=\"pop_bottomright\"></td>"+
"		</tr>"+
"</table>"+
"</div>";

return html;
}

boxOverlay.prototype.greyBox = function(){

var bgColor = this.whiteHeader ? 'style="background: none repeat scroll 0 0 #FFFFFF;"' : '';
var tlClass = this.whiteHeader ? 'tlWhite' : '';
var trClass = this.whiteHeader ? 'trWhite' : '';

var html = "<div style=\"display:none\" id=\""+this.idBox+"\" class=\"boxOverlay"+(this.theme ? "_"+this.theme : "")+"\">"+
"	<table  cellspacing=\"0\" cellpadding=\"0\" border=\"0\" >"+
"		<tr class=\"pop_headerCont\">"+
"			<td class=\"\" style=\"\" >"+
"				<div class=\"boxHeader\" "+bgColor+">"+
"					<div class=\"pop_centerTitleContent\" "+bgColor+">"+
"						<div id=\""+this.idBox+"_title\" class=\"title\"></div>";
if(!this.avoidClosing && !this.hideCloseBtn){
html += "				<a id=\"close_"+this.idBox+"\" class=\"closeButton\">"+ this.closeLabel + "&nbsp;</a>";
}
html += "				<div class='clear'></div>"+
"					</div>"+
"				</div>"+
"			</td>"+
"		</tr>"+
"		<tr class=\"content_row\">";

	if ( !this.onlyColoumnCenter ) {
		html +="<td class=\"pop_content\" id=\""+this.idBox+"_pop_content\">"+
		"			<div id=\""+this.idBox+"_content\" class=\"boxBody\">"+
		"			</div>"+
		"		</td>";
	} else {
		html +="<td class=\"pop_content\" colspan=\"3\" style=\"padding:0\" id=\""+this.idBox+"_pop_content\">"+
		"			<div id=\""+this.idBox+"_content\" class=\"boxBody\">"+
		"			</div>"+
		"		</td>";
	}
	
	html +=""+
"		</tr>"+
"		<tr>"+
"			<td>"+
"				<div class=\"boxFooter pop_footerCont\" id=\""+this.idBox+"_footer_cont\" style=\"display:none\">"+
"					<div  cellspacing=\"0\" cellpadding=\"0\" border=\"0\">"+
"						<div>"+
"							<div class=\"pop_centerTitleContent\"><div id=\""+this.idBox+"_footer\" class=\"title\"></div></div>"+
"						</div>"+
"					</div>"+
"					<div style=\"clear:both\"></div>"+
"				</div>"+
"				<div style=\"clear:both\"></div>"+
"			</td>"+	
"		</tr>"+
/*"		<tr>"+
"			<td class=\"pop_borderBottom\"><div id=\""+this.idBox+"_bottom_border\" class=\"ov_white_border\"></div></td>"+
"		</tr>"+
*/
"	</table>"+
"</div>";

return html;
}



boxOverlay.prototype.greyBox2 = function(){

var bgColor = this.whiteHeader ? 'style="background: none repeat scroll 0 0 #FFFFFF;"' : '';

var html = 	"<div style=\"display:none\" id=\""+this.idBox+"\" class=\"boxOverlay"+(this.theme ? "_"+this.theme : "")+"\">"+
			"	<div class=\"boxHeader\" "+bgColor+">";
			if(!this.avoidClosing && !this.hideCloseBtn){
				html +="<a id=\"close_"+this.idBox+"\" class=\"closeBtn\">&times;</a>";
			}
	html +=	"		<div id=\""+this.idBox+"_title\" class=\"title\"></div>"+
			"	</div>"+
			"	<div id=\""+this.idBox+"_content\" class=\"boxBody\"></div>"+
			"	<div class=\"boxFooter pop_footerCont\" id=\""+this.idBox+"_footer_cont\" style=\"display:none\">"+
			"		<div id=\""+this.idBox+"_footer\"></div>"+
			"	</div>"+
			"</div>";

return html;
}

boxOverlay.prototype.setHideOnlyBox = function(hideOnly) {
	this._hideOnlyBox = hideOnly;
}

boxOverlay.prototype.cleanVars = function(){	
	for (var i in this){
		try{
			if (this[i] && typeof this[i] == "Object"){	
				this.cleanVars(this[i]);
			}else
				this[i] = null;
		}catch(e){
			//alert(e)
		}
	}
}

boxOverlay.prototype.purgeDomObj = function(d){
//IE memory leak purge
	try{
		if (typeof d == "undefined" || !d){
		  return;
		}
		var a = d.attributes, i, l, n;
		var b = d.childNodes;
		
			for (var i in d){
				if (d[i] && typeof d[i] == "Object"){			
					d.purgeDomObj(d[i]);
				}
			}
		
		
			if (a) {
				l = a.length;
				for (i = 0; i < l; i += 1) {
					n = a[i].name;
					if (typeof d[n] === 'function') {
						d[n] = null;
					}
				}
			}
		
			if (b) {
				l = b.length;
				for (i = 0; i < l; i += 1) {
					
					if (d.childNodes[i]){
						purgeDomObj(d.childNodes[i]);
						d.removeChild(d.childNodes[i]);
					}
				}
				
			}
		d.innerHTML = "";
		d.parentNode.removeChild(d);
		d = null;
	}catch(e){
		
		//_doLog("error:"+e,'error');
	}
}


boxOverlay.prototype._getBrowser = function(){	
	try{
		var info = Array();
		var browser=navigator.appName;
		info["name"] = info[0] = browser;
    
		var b_version=navigator.appVersion;
		var version = "";

		if (browser.toLowerCase().search(/explorer/)>=0){
			var x = b_version.split(';');
			var version=parseInt(x[1].split(" ")[2]);
		}else{
			var x = b_version.split(' ');
			var version=parseInt(x[0]);
		}
		info["version"] = info[1] = version;
		
		return info;
	}catch(e){
		return null;
	}
	
}
;
var MEPopBoxFactory = function(){
    this.init();
}

if (!document.getElementById('popBoxContainerDiv'))
	document.write("<div id=\"popBoxContainerDiv\" style=\"\"></div>");

MEPopBoxFactory.prototype.init = function(){
    appendCss("/includes/popBox/popBox.css","MEPB_css");
    //addEvent(document.body,"mousemove",this.setMousePosition.bind(this));
	addEvent(document.body,"click",this.setMousePosition.bind(this));
    this.instances = new Array();
    MEPB_hide = new Array();
    MEPB_show = new Array();
	MEPB_mOut = new Array();
    
}

MEPopBoxFactory.prototype.setMousePosition = function(e){

    if (typeof e == "undefined"){
        this.mouseX = window.event.clientX +10;
        this.mouseY = window.event.clientY;
    }
    else{
        this.mouseX = e.clientX +10;
        this.mouseY = e.clientY;
    }
}

MEPopBoxFactory.prototype.getPopBox = function(id,objRef){
	
    if (typeof this.instances[id] == "undefined" || !this.instances[id]){
        this.instances[id] = new MEpopBox(id);
		this.instances[id].parentObj = this;
	}
    if (typeof objRef != "undefined"){
			this.instances[id].objRef = objRef;
    }
	return this.instances[id]
}


var MEpopBox = function(idPopBox){
    this.parentObj = null;
    this.id = null;
    this.urlPars = null;
    this.url = "/MEpopBox/php/getPopBoxBox.php";
	this.divBox = null;
	this.divBoxContent = null;
    this.thumbs = new Array();
	this.bigImage = null;
	this.hidden = true;
	this.objRef = null;
	this.donthide = false;
	this.reset ();
	this.closeEvents = new Array();
    if (typeof idPopBox == "undefined" || !idPopBox){
        this.id = null;
    }else
        this.id = idPopBox;
//    addEvent(window,"unload",this.destroy.bind(this));
    return this.init();
}

MEpopBox.prototype.reset = function() {
	this.hideClose = false;
	this.forceToRight = false;
	this.forceToLeft = false;
	this.forcePopupStyle = false;
	this.boxOffsetLeft = 5;
	this.hideOnEsc = true;
	this.arrowOffsetTop = -10;
}

MEpopBox.prototype.setObjRefFromId = function(idObj){
	this.objRef = document.getElementById(idObj);
}

MEpopBox.prototype.bodyClick = function(e){
	if (!this.hidden)
		this.hide();
}

MEpopBox.prototype.mouseClick = function(e){	
	if (window.event)
		window.event.cancelBubble = true
	else
		e.stopPropagation();
}

MEpopBox.prototype.init = function(){
    if (!this.id){
        alert("called MEPopBox without id");
        return null;
    }
	this.exitOnEsc();
	return this.createBox();
}

MEpopBox.prototype.createBox = function(){
    
	/*try{
	*/
		var html = this.getBoxHtml();
		this.divBox = document.createElement("div");
		this.divBox.id = "MEPB_DIV_"+this.id;
		this.divBox.innerHTML = html;
		document.getElementById('popBoxContainerDiv').appendChild(this.divBox);
		this.divBox.style.display = "none";
		var aCont = getElementsByClass("MEPB_center_content",this.divBox);
		this.divBoxContent = aCont[0];
	
		var aArrow = getElementsByClass("MEPB_arrow",this.divBox);
		this.divBox.arrow = aArrow[0];
		var aClose = getElementsByClass("MEPB_close",this.divBox);
		this.divBox.close = aClose[0];
		addEvent(this.divBox.close,"click",this.hide.bind(this),false);
		addEvent(this.divBox,"click",this.mouseClick.bind(this),false);
	
		//addEvent(this.divBox,"mouseover",this.mover.bind(this));
		//addEvent(this.divBox,"mouseout",this.mout.bind(this));
		return this;
		
/*	}catch(e){
        alert(e.description ? e.description : e.message);
		this.hide();
	}*/
}
MEpopBox.prototype.setBoxStyle = function(){
    this.divBox.style.width = MEgetWidth(this.divBoxContent.parentNode.parentNode) + MEgetWidth(this.divBox.arrow) + 10 + "px" ;
    if (!this.forcePopupStyle) {
    	this.divBox.style.position = "absolute";
    	this.divBox.style.zIndex = "150";
    }
}


MEpopBox.prototype.display = function(){
    
    
    this.setPosition();
    if (this.divBox.style.display != "block"){
		setTimeout(function(){this.divBox.style.display = "block";}.bind(this),0);
    }
    this.hidden = false;
    document.body.style.cursor = "";
}

MEpopBox.prototype.toggle = function(){
	if (this.hidden)
		this.show();
	else
		this.hide();
}

MEpopBox.prototype.show = function(){
	/*try{
	*/

		if (this.hideClose && this.objRef){
			this.divBox.close.style.display = "none";
		}else{
			this.divBox.close.style.display = "block";
		}
		if (this.hidden)
		    document.body.style.cursor = "wait";
		if (typeof this.toTimer != "undefined")
		    clearTimeout(this.toTimer);
		
		MEPB_show[this.id] = this._show.bind(this);
		this.showTimer = setTimeout("MEPB_show['"+this.id+"']()",0);
	/*}catch(e){
		alert(e.description ? e.description : e.message);
		this.hide();
	}*/
}

MEpopBox.prototype._show = function(){
	/*try{
	*/
		
		this.display();
		
	/*}catch(e){
		alert(e.description ? e.description : e.message);
		this.hide();
	}*/
}



MEpopBox.prototype.hide = function(){

    if (typeof this.showTimer != "undefined")
        clearTimeout(this.showTimer);
    document.body.style.cursor = "";
    MEPB_hide[this.id] = this._hide.bind(this);
    this.toTimer = setTimeout("MEPB_hide['"+this.id+"']()",0);
}

MEpopBox.prototype._hide = function(){

	for (var i=0; i<this.closeEvents.length;i++){
		if (typeof this.closeEvents[i] == "function")
			this.closeEvents[i]();
	}
	if (this.divBox != null )
	    this.divBox.style.display = "none";
	this.hidden = true;
}

MEpopBox.prototype.mover = function(){
    this.show();
}

MEpopBox.prototype.mout = function(){
    this.hide();
}

MEpopBox.prototype.getMousePosition = function(){
    var coords = new Array();
    coords["x"] = this.parentObj.mouseX;
    coords["y"] = this.parentObj.mouseY;
    return coords;
}
MEpopBox.prototype.getRefLeftPoint = function(obj){
	var pos = getPosition(obj);
	var width = MEgetWidth(obj);
	
	return parseInt(pos.x) + parseInt(width);
}

MEpopBox.prototype.getRefTopPoint = function(obj){
	var pos = getPosition(obj);
	return parseInt(pos.y);
}

MEpopBox.prototype.setClosePosition = function(){
    //this.divBox.close.style.marginLeft = cWidth - closeWidth - 20 + "px";
}
MEpopBox.prototype.setPosition = function(){
	
    this.divBox.style.display = "block";
    this.divBox.style.visibility = "hidden";
	
	this.setBoxStyle();
	
    var mouseCoords  = this.getMousePosition();
    var wHeight = window.innerHeight ? window.innerHeight : document.documentElement.clientHeight;
	var wWidth = window.innerWidth ? window.innerWidth : document.documentElement.clientWidth;
    var scrollY = (document.all)?document.documentElement.scrollTop:window.pageYOffset;
    bHeight = MEgetHeight(this.divBox);
    bWidth = MEgetWidth(this.divBox);
	cWidth = MEgetWidth(this.divBoxContent);
	closeWidth = MEgetWidth(this.divBox.close);

	this.setClosePosition();

	var boxStartLeft = 0;
	if (!this.objRef)
		boxStartLeft = mouseCoords["x"] + this.boxOffsetLeft;
	else
		boxStartLeft = this.getRefLeftPoint(this.objRef) + this.boxOffsetLeft;

	var sPositionArrow = 'left';
	// controlla se il box ha lo spazio per essere visualizzato a destra a meno che le impostazioni non lo impongano
	
	var ipoteticLeftPosition;
	if (!this.objRef)
			ipoteticboxStartLeft = mouseCoords["x"] - this.boxOffsetLeft - bWidth;
		else
			ipoteticboxStartLeft = this.getRefLeftPoint(this.objRef) - this.boxOffsetLeft - bWidth - MEgetWidth(this.objRef);
	
	if ( (!this.forceToRight && (boxStartLeft + bWidth > wWidth) && (ipoteticboxStartLeft > 0)) || this.forceToLeft) {
		
			boxStartLeft = ipoteticboxStartLeft + this.boxOffsetLeft; //gli tolgo l'offset
			sPositionArrow = 'right';
	}
	
	this.divBox.style.left = boxStartLeft + "px";
    
	var newPos;
	var clickedY;
	var iHeight;
	if (!this.objRef || (this.objRef.tagName.toLowerCase() == "select")){
		newPos = mouseCoords["y"]  - (bHeight/2);	
		clickedY = mouseCoords["y"];
	}
	else{
		
		iHeight = MEgetHeight(this.objRef);
		
		newPos = this.getRefTopPoint(this.objRef) - scrollY - bHeight/2 + iHeight/2 +5;
		clickedY = this.getRefTopPoint(this.objRef) - scrollY;
	}
	
	
    var diffPos = 0;
	while (newPos + bHeight > wHeight){
        var backupPos = newPos;
        newPos = newPos - 10;
        diffPos = Math.abs(backupPos - newPos);
    }
	
	newPos = newPos + scrollY;
	if (newPos < scrollY){
        var backupPos = newPos;
        newPos = scrollY + 10;
        diffPos = Math.abs(backupPos - newPos);
    }
	
	this.divBox.style.top = newPos + "px";
    this.setArrowPosition(clickedY - newPos + (iHeight ? iHeight/2 : 0) + scrollY + this.arrowOffsetTop, sPositionArrow);
    this.divBox.style.display = "none";
    this.divBox.style.visibility = "visible";
	
}

MEpopBox.prototype.forcePosition = function(pos){
	if (typeof pos == "undefined")
		pos = "left";
	
	if (pos == "right"){
		this.forceToLeft = false;
		this.forceToRight = true;
	}else{
		this.forceToLeft = true;
		this.forceToRight = false;
	}
}

MEpopBox.prototype.setArrowPosition = function(nTop, sPosition){

    if (sPosition == 'right') {
        
		//this.divBox.arrow.style.left = ( MEgetWidth(this.divBox) - 21 )+ "px";
		
		var contByCl = getElementsByClass("MEPB_center_content_cont",this.divBox);
		var cont = contByCl[0];
		this.divBox.arrow.style.left = ( MEgetWidth(cont)) + 9 +"px"; //9 è la larghezza dell'immagine - 2 px per posizionarla sul bordo
		
        this.divBox.arrow.style.backgroundImage = "url('/img2/popBox/arrow_right.png')";
        this.divBox.arrow.style.backgroundPosition = "-2px 0px";
    }else{
		//default
		this.divBox.arrow.style.left =  0 +"px"; //9 è la larghezza dell'immagine - 2 px per posizionarla sul bordo
		
        this.divBox.arrow.style.backgroundImage = "url('/img2/popBox/arrow.png')";
        this.divBox.arrow.style.backgroundPosition = "0px 0px";
	}
    if (typeof nTop != "undefined"){
        if (nTop < 0)
            nTop = 0;
        if (nTop > MEgetHeight(this.divBox))
            nTop = MEgetHeight(this.divBox);
        this.divBox.arrow.style.marginTop = nTop + "px";
        return;
    }
    var mouseCoords  = this.getMousePosition();
    this.divBox.arrow.style.marginTop = MEgetHeight(this.divBox)/2 + "px";
}


MEpopBox.prototype.getBoxHtml = function(nTop){
	var html = ''+
	'<div class="MEPB_PopBox">'+
		'<div class="MEPB_arrow">'+
		'</div>'+
		'<table class="MEPB_content" cellpadding="0" cellspacing="0" style="border-collapse:collapse;">'+
			'<tr>'+
				'<td class="MEPB_top MEPB_left"></td><td class="MEPB_top MEPB_center"></td><td class="MEPB_top MEPB_right"></td>'+
			'</tr>'+
			'<tr>'+
				'<td class="MEPB_center MEPB_left"></td>'+
				'<td class="MEPB_center_content_cont">'+
				'<div class="MEPB_close"><a href="javascript:">chiudi</a></div>'+
				'<div class="MEPB_center_content"></div>'+
				'</td>'+
				'<td class="MEPB_center MEPB_right"></td>'+
			'</tr>'+
			'<tr>'+
				'<td class="MEPB_bottom MEPB_left"></td><td class="MEPB_bottom MEPB_center"></td><td class="MEPB_bottom MEPB_right"></td>'+
			'</tr>'+
		'</table>'+
	'</div>';
	return html;
}

MEpopBox.prototype.exitOnEsc = function(){
    addEvent(document,"keyup", function(e){
		if (this.hideOnEsc){
			var kC  = (window.event) ?    // MSIE or Firefox?
		       window.event.keyCode : (e.keyCode ? e.keyCode : e.which);
			var Esc = (window.event) ?   
			      27 : e.DOM_VK_ESCAPE // MSIE : Firefox
			if(kC==Esc)
			   this.hide();
		}
   }.bind(this));
}

MEpopBox.prototype.setContent = function(htmlContent){
	this.divBoxContent.innerHTML = htmlContent;
	return this;
}

MEpopBox.prototype.getContentElement = function(){
	return this.divBoxContent;
}

MEpopBox.prototype.clearContent = function(){
	this.divBoxContent.innerHTML = "";
	return this;
}

MEpopBox.prototype.appendContentElement = function(oHtmlElement){
	this.divBoxContent.appendChild(oHtmlElement);
	return this;
}

MEpopBox.prototype.destroy = function(){
	this.parentObj.instances[this.divBox.id] = null;
    this.divBox.parentNode.removeChild(this.divBox);
	removeEvent(window,"unload",this.destroy.bind(this))	
	this.purgeDomObj(this.divBox);
    this.cleanVars();
}

MEpopBox.prototype.cleanVars = function(){	
	for (var i in this){
		try{
			if (this[i] && typeof this[i] == "Object"){	
				this.cleanVars(this[i]);
			}else
				this[i] = null;
		}catch(e){
			//alert(e)
		}
	}
}


MEpopBox.prototype.addEventOnClose = function(func){	
	this.closeEvents.push(func);
}

MEpopBox.prototype.removeEventOnClose = function(func){	
	
	for (var i=0; i<this.closeEvents.length;i++)
		if (this.closeEvents[i] == func)
			this.closeEvents[i] = null;
}

MEpopBox.prototype.purgeDomObj = function(d){
//IE memory leak purge
	try{
		if (typeof d == "undefined" || !d){
		  return;
		}
		var a = d.attributes, i, l, n;
		var b = d.childNodes;
		
			for (var i in d){
				if (d[i] && typeof d[i] == "Object"){			
					d.purgeDomObj(d[i]);
				}
			}
		
		
			if (a) {
				l = a.length;
				for (i = 0; i < l; i += 1) {
					n = a[i].name;
					if (typeof d[n] === 'function') {
						d[n] = null;
					}
				}
			}
		
			if (b) {
				l = b.length;
				for (i = 0; i < l; i += 1) {
					
					if (d.childNodes[i]){
						purgeDomObj(d.childNodes[i]);
						d.removeChild(d.childNodes[i]);
					}
				}
				
			}
		d.innerHTML = "";
		d = null;
	}catch(e){
		//_doLog("error:"+e,'error');
	}
}

MEpopBox.prototype.renderVisibleOnPopup = function(func){	
	this.divBox.style.position = "fixed";
	this.divBox.style.zIndex = "301";
	this.forcePopupStyle = true;
}
;
/* this file nedd functions.js to work properly*/

var expand = new Array();
var expandw = new Array();
var collapse = new Array();
var speed = 20;

var MErestrictTimer= new Array();
var MEenlargeTimer= new Array();
var transition = false;

function ME(prop){
    var first = prop.charAt(0);
    var id = prop.split(first)[1];
    if (first=='#')
    return document.getElementById(id) ? document.getElementById(id) : null;
        else if (first=='.')
            return MEgetElementsByClass(id);
return null;
}

function MEhide(id){
 var div;
 if (typeof id == "string")
	div = document.getElementById(id);
 else
	div = id;

 if (div){
	if (typeof div._initDisplay == "undefined"){
		div._initDisplay = MEgetDefaultProp(div,"display");
	}
	div.style.display='none';
 }
}

function MEshow(id){
	var div;
 if (typeof id == "string")
	div = document.getElementById(id);
 else
	div = id;
 if (div){
	if (typeof div._initDisplay == "undefined"){
		div._initDisplay = MEgetDefaultProp(div,"display");
	}
	if (div._initDisplay && div._initDisplay != "none"){
		div.style.display=div._initDisplay;
	}else{
		div.width = parseInt(MEgetWidth(div)) + "px";
		div.style.display='inline-block';
	}
 }

}

function MEtoggle(id){
	var div;
	if (typeof id == "string")
		div = document.getElementById(id);
	else
		div = id;
	div = document.getElementById(id);
    if (div)
		if (typeof div._initDisplay == "undefined"){
			div._initDisplay = MEgetDefaultProp(div,"display");
		}
		if (div.style.display=='none')
			MEshow(id);
		else
			MEhide(id);
}

function MEexpand(id,speed,height){
   var div = document.getElementById(id);
   actheight = div.style.height;

   actheight = parseInt(actheight);

   if (!speed){
		div.style.display = "block";
		div.style.height = "";
		expand[id]=false;
		return;
   }

   if (actheight > height-speed){

    div.style.height = "";
    expand[id]=false;
    div.open = true;
	div.closed = false;

   }else{
      actheight+=speed;
      div.style.height=actheight+"px";
      slide_div = setTimeout("MEexpand('"+id+"',"+speed+",'"+height+"')",0);
   }
}

function MEcollapse(id,speed,originalDivHeight){

   var div = document.getElementById(id);
   var actHeight = MEgetHeight(div);
   if (!speed){
		div.style.display = "none";
		div.style.height = "auto";
		collapse[id] = false;
		return;
   }

   var actHeight = parseInt(actHeight);
   if (actHeight <= speed){
      var pos = div.style.position
      div.style.display='none';
      div.style.visibility='visible';
      div.style.height = originalDivHeight;
      div.closed = true;
	  div.open = false;
      collapse[id] = false;
   }else{
      actHeight-=speed;
      if (actHeight<0)
		actHeight=0;
		div.style.height=actHeight+"px";
        setTimeout("MEcollapse('"+id+"',"+speed+",'"+originalDivHeight+"')",0);
   }
}


function MEgetHeight(div){
	if ( typeof div == "undefined" || !div )
		return 0;

	  var pos;
	  var vis;
	  var dis;
      pos = MEgetComputedProp(div,"position");

	  vis = MEgetComputedProp(div,"visibility");
	  dis = MEgetComputedProp(div,"display");

      if (dis && dis == "none"){
		if (pos)
			div.style.position='absolute';
		div.style.visibility='hidden';
		div.style.display='block';
	  }

	  var stopHeight = div.clientHeight ? div.clientHeight : div.offsetHeight;

      if (dis && dis == "none"){
		if (pos)
			div.style.position=pos;
		div.style.display=dis;
		div.style.visibility=vis;
	  }
      return stopHeight;
}

function MEgetWidth(div){
	if ( typeof div == "undefined" || !div )
		return 0;

	var pos;
	var vis;
	var dis;

    pos = MEgetComputedProp(div,"position");

	vis = MEgetComputedProp(div,"visibility");
	dis = MEgetComputedProp(div,"display");

	if (dis && dis == "none"){
		if (pos)
			div.style.position='absolute';
		div.style.visibility='hidden';
		div.style.display='block';
	}
     var stopWidth = div.clientWidth ? div.clientWidth : div.offsetWidth;

     if (dis && dis == "none"){
		if (pos)
			div.style.position=pos;
		div.style.display=dis;
		div.style.visibility=vis;
	  }

    return stopWidth;
}

function slideTo( id, speed, stop ) {
	var div = document.getElementById( id );
	var start = MEgetHeight(div);

	if ( start < stop ) {
		slideToDown( id, speed, stop );
	} else {
		slideToUp( id, speed , stop );
	}
}

function slideToUp( id, speed, stop ) {
	var div = document.getElementById( id );
	var actHeight = MEgetHeight(div);

	var nextHeight = actHeight - speed;
	if ( nextHeight < stop ) {
		div.style.height = stop+'px';
		return;
	}
	div.style.height = nextHeight+'px';
	setTimeout("slideToUp('"+id+"',"+speed+","+stop+")",0);
}

function slideToDown( id, speed, stop ) {
	var div = document.getElementById( id );
	var actHeight = MEgetHeight(div);

	var nextHeight = actHeight + speed;
	if ( nextHeight > stop ) {
		div.style.height = stop+'px';
		return;
	}
	div.style.height = nextHeight+'px';
	setTimeout("slideToDown('"+id+"',"+speed+","+stop+")",0);
}

function MEslideUp(id,speed){

    if (typeof MESlideUpTimer != "undefined"){
		clearTimeout(MESlideUpTimer);
    }

	if (typeof document.getElementById(id).MESlideUpStatus!="undefined" && document.getElementById(id).MESlideUpStatus == "open"){
		return;
	}

	document.getElementById(id).MESlideUpStatus="open";
    if (collapse[id]){
        setTimeout("MEslideUp('"+id+"',"+speed+")",0);
    }else{
    expand[id] = true;
		var div = document.getElementById(id);
     if (div){
      var stopHeight = MEgetHeight(div);
      div.style.height='0px';
	  div.style.visibility = "visible";
	  div.style.display = "block";
	  div.style.overflowY = "hidden";
      MEexpand(id,speed,stopHeight);
    }else{
      throw('ERROR - No id='+id+' element in SlideUp function');
   }
    }
}

function MEslideDown(id,speed){
    if (expand[id]){
        setTimeout("MEslideDown('"+id+"',"+speed+")",0);
    }else{
		if (typeof document.getElementById(id).MESlideUpStatus!="undefined" && document.getElementById(id).MESlideUpStatus == "closed"){
			//MESlideUpTimer = setTimeout("MEslideDown('"+id+"',"+speed+")",5000);
			return;
		}
		document.getElementById(id).MESlideUpStatus="closed";
			var div = document.getElementById(id);
			if (div){

				var originalDivHeight = div.style.height;
				if (div.style.display!='none'){
					collapse[id]=true;
					MEcollapse(id,speed,originalDivHeight);
				}

			}else{
				throw('ERROR - No id='+id+' element in SlideDown function');
			}

	}
}

function MEtoggleSlide(div,speed){
	var oDiv = document.getElementById(div);
	if (oDiv){
		if (oDiv.style.display!='none') MEslideDown(div,speed);
			else MEslideUp(div,speed);
	}


}


function MEresizeDivWidth(div,width,speed){
    var oDiv = document.getElementById(div);
    var divWidth = MEgetWidth(oDiv);
    if (parseInt(divWidth)>parseInt(width)) MErestrict(div,width,speed);
	else MEenlarge(div,width,speed);
}

function MEenlarge(div,targetWidth,speed){

    var oDiv = document.getElementById(div);
    var divWidth = oDiv.style.width;
    if (!divWidth) divWidth = MEgetWidth(oDiv);

    if (parseInt(divWidth)>parseInt(targetWidth)-parseInt(speed)){
	if (MEenlargeTimer)
	    clearTimeout(MEenlargeTimer[div]);
	oDiv.style.width=targetWidth+"px";
    }else{

	oDiv.style.width=(parseInt(divWidth) + parseInt(speed))+"px";
	//clearTimeout(MEenlargeTimer);
	MEenlargeTimer[div] = setTimeout("MEenlarge('"+div+"','"+targetWidth+"',"+speed+")",0);
    }
}

function MErestrict(div,targetWidth,speed){

    var oDiv = document.getElementById(div);
    var divWidth = oDiv.style.width;
    if (!divWidth) divWidth = MEgetWidth(oDiv);

    if (parseInt(targetWidth)>parseInt(divWidth)-parseInt(speed)){
	if (MErestrictTimer)
	    clearTimeout(MErestrictTimer[div]);
	oDiv.style.width=targetWidth+"px";
    }else{

	oDiv.style.width=(parseInt(divWidth)-parseInt(speed))+"px";
	//clearTimeout(MErestrictTimer);
	MErestrictTimer[div] = setTimeout("MErestrict('"+div+"','"+targetWidth+"',"+speed+")",0);
    }
}



/* Accordion Instructions
*
* Set the accordions elements in this way : <element class="myClass" id="myClass_Id">....</element> where id is unique for each Element
*
* To use the accordion set the onclick property like this :  onclick="MEaccordion(IdToExpand,speed)"
*
*/

function MEaccordion(id,speed){
   if (collapse[id] || expand[id]){
    setTimeout("MEaccordion('"+id+"',"+speed+")",0);
    return;
   }

   var div = document.getElementById(id);
   if (div.style.display=='block'){

      MEslideDown(id,speed);
   }else{

   var accClass = id.split("_")[0];
   var accDivs = getElementsByClass(accClass);
   for(i=0; i<accDivs.length; i++){
	if (accDivs[i].id != id){
		collapse[accDivs[i]] = true;
		MEslideDown(accDivs[i].id,speed);
		collapse[accDivs[i]] = false;
	}
   }

   div.style.display="block";
	MEslideUp(id,speed);
   }

}

function MEevidenzia(id,valore){

        div = document.getElementById(id);
        if (div){
        div.style.opacity = valore/10;
	div.style.filter = 'alpha(opacity=' + valore*10 + ')';
        }
}

function popUp(url){
window.open(url,'','toolbar=no,width=,height=,scrollbars=1');
}

function MEpopup(page,title,width,height){
var win;
 params = "toolbar=0,";

params += "location=0,";

params += "directories=0,";

params += "status=0,";

params += "menubar=0,";

params += "titlebar=0,";

params += "scrollbars=0,";

params += "resizable=0,";

params += "top=0,";

params += "left=50,";
 params += "width="+width+",";

params += "height="+height;
win=window.open(page,title,params);
}

function MEgetElementsByClass(searchClass,node,tag) {
	var classElements = new Array();
	if ( node == null )
		node = document;
	if ( tag == null )
		tag = '*';
	var els = node.getElementsByTagName(tag);
	var elsLen = els.length;
	var pattern = new RegExp("(^|\\\\s)"+searchClass+"(\\\\s|$)");
	for (i = 0, j = 0; i < elsLen; i++) {
		if ( pattern.test(els[i].className) ) {
			classElements[j] = els[i];
			j++;
		}
	}
	return classElements;
}

function MEmove(id,pos,speed){

   div = document.getElementById(id);
   var left = parseInt(div.style.left) ? parseInt(div.style.left) : 0;
   //div.style.position="relative";

   if (left > pos-speed){

    div.style.left = pos+"px";
    //move[id]=false;
   }else{
      left+=speed;
      div.style.left=left+"px";
      slide_div = setTimeout("MEmove('"+id+"',"+pos+",'"+speed+"')",0);
   }
}

function changeMenu_Icon(el,updateMyImg){
	if (typeof updateMyImg == "undefined")
	    updateMyImg = true;
	var menus = document.getElementsByClassName('menus');

	var className = el.className;
	var flag = className.search(/sel/);

		for (i=0;i<menus.length;i++){
			menus[i].style.backgroundImage="url(/img2/mail_template_cat.png)";
		}
		if (updateMyImg)
		    el.className="menus";
		if (flag<0 && updateMyImg){
			el.className="menus sel";
			el.style.backgroundImage="url(/img2/mail_template_cat_minus.png)";
		}

}

function MEfadeOut(el,value,speed,stopValue){

	if (typeof stopValue == "undefined"){
		stopValue = 0;
	}

	if (typeof speed=="undefined" || speed == 0) {
		speed = 0.5;
	}

	if (typeof el == "undefined" || !el){
		return;
	}

	if (typeof value=="undefined")
		value = 10;

	if (value<=stopValue){
		//if (el) el.style.visibility="hidden";
		return;
	}else{
		value=value-speed;
		evidenzia(el,value);
		el.style.visibility="visible";
		setTimeout(function(){MEfadeOut(el,value,speed,stopValue)},0);
	}
}

function MEfadeIn(el,value,speed,stopValue){
	evidenzia(el,0);
	if (typeof speed=="undefined" || speed == 0) {
		speed = 0.5;
	}
	_MEfadeIn(el,value,speed,stopValue);
}

function _MEfadeIn(el,value,speed,stopValue){

	if (typeof stopValue == "undefined")
		stopValue = 10;

	if (typeof el == "undefined" || !el){
		return;
	}

	if (typeof value=="undefined"){
		value = 0;
		el.style.visibility="hidden";
		MEshow(el.id)
	}

	if (value>stopValue){
		el.style.visibility="visible";
		MEshow(el.id)
		return;
	}else{
		value=value+speed;
		evidenzia(el,value);
		MEshow(el.id);

		el.style.visibility="visible";
		setTimeout(function(){_MEfadeIn(el,value,speed,stopValue)},0);
	}
	MEshow(el)
}

function MEgetDefaultProp(el,cssProp){
	try{

	var oEl = document.createElement(el.tagName);

	document.body.appendChild(oEl);
	var defaultProp = MEgetComputedProp(oEl,cssProp);
	oEl.parentNode.removeChild(oEl);

	return defaultProp;

	}catch(e){
		return null;
	}

	return null;
}

function MEgetComputedProp(el,cssProp){
	try{
		if (window.getComputedStyle){
			var compStyle = window.getComputedStyle(el,null);
			return compStyle.getPropertyValue(cssProp);
		}

		if (document.defaultView && document.defaultView.getComputedStyle)
			return document.defaultView.getComputedStyle(el, "")[cssProp];

		if ( el.currentStyle ){
			cssProp = cssProp.replace(/\-(\w)/g, function (strMatch, p1) {
				return p1.toUpperCase();
			});
			return el.currentStyle[cssProp];
		}

		return el.style[cssProp];
	}
	catch(e){

	}
	return null;
}

function MEslideBySide(id,speed){

    if (typeof MEslideBySideTimer != "undefined"){
		clearTimeout(MEslideBySideTimer);
    }

	document.getElementById(id).MESlideStatus="open";

    if (collapse[id]){
        setTimeout("MEslideBySide('"+id+"',"+speed+")",0);
    }else{
		expandw[id] = true;
		var div = document.getElementById(id);
		if (div){
			var stopWidth = MEgetWidth(div);
			div.style.width='0px';
			div.style.visibility = "visible";
			div.style.display = "block";
			div.style.overflowY = "hidden";
			MEexpandByWidth(id,speed,stopWidth);
		}else{
			throw('ERROR - No id='+id+' element in SlideBySide function');
		}
    }
}

function MEexpandByWidth(id,speed,width){
   var div = document.getElementById(id);
   actwidth = div.style.width;

   actwidth = parseInt(actwidth);

   if (!speed){
		div.style.display = "block";
		div.style.width = "";
		expandw[id]=false;
		return;
   }

   if (actwidth > width-speed){

    div.style.width = "";
    expandw[id]=false;
    div.open = true;
	div.closed = false;

   }else{
      actwidth+=speed;
      div.style.width=actwidth+"px";
      slide_div = setTimeout("MEexpandByWidth('"+id+"',"+speed+",'"+width+"')",0);
   }
}
;
var __debugging_doLog = true;

/* *********************************************************************************************** */

// Oggetto per interazioni MEAjax

function getMEAjaxObj(){
    var MEAjax = new Object();
    MEAjax.isUpdating = true;


    MEAjax.Request = function(method, url, callback, async )
    {
	this.url = url;
	this.sendData = url;
	this.isUpdating = true;
	this.callbackMethod = callback;
	this.request = (window.XMLHttpRequest)? new XMLHttpRequest(): new ActiveXObject("MSXML2.XMLHTTP");

	if ( typeof async == 'undefined' )
		async = true;

	this.request.onreadystatechange = function() { MEAjax.checkReadyState(); };
	if (method == "POST"){
	    var nPos = url.search(/\?/);
	    this.sendData = "";
	    if (nPos >= 0) {
    	    this.url = url.substr(0, nPos);
    	    this.sendData = url.substr(nPos + 1);

	    }

        //alert("ORIGINAL URL: " + url + "\nURL:" + this.url + "\nDATA: " + this.sendData + "\nLENGTH: " + this.sendData.length);

	    this.request.open(method, this.url, async);
	    this.request.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
	    //this.request.setRequestHeader("Content-length",this.sendData.length);
	    //this.request.setRequestHeader("Connection","close");
	}else{
	    this.request.open(method, this.url, async);
	}
	this.request.send(this.sendData);
    };


    MEAjax.checkReadyState = function(_id){
	switch(this.request.readyState)
	{
	    case 1: break;
	    case 2: break;
	    case 3: break;
	    case 4:
		this.isUpdating = false;
		this.callbackMethod(this.request);
	}
    };

    MEAjax.updateHTML = function(method, url, idElement){
	this.url = url;
	this.sendData = url;
	this.isUpdating = true;
	this.callbackMethod = function(doc){document.getElementById(idElement).innerHTML = doc.responseText;};
	this.request = (window.XMLHttpRequest)? new XMLHttpRequest(): new ActiveXObject("MSXML2.XMLHTTP");

	this.request.onreadystatechange = function() { MEAjax.checkReadyState(); };
	if (method == "POST"){
	    var nPos = url.search(/\?/);
	    this.sendData = "";
	    if (nPos >= 0) {
    	    this.url = url.substr(0, nPos);
    	    this.sendData = url.substr(nPos + 1);
	    }

        //alert("ORIGINAL URL: " + url + "\nURL:" + this.url + "\nDATA: " + this.sendData + "\nLENGTH: " + this.sendData.length);

	    this.request.open(method, this.url, true);
	    this.request.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
	    this.request.setRequestHeader("Content-length",this.sendData.length);
	    this.request.setRequestHeader("Connection","close");
	}else{
	    this.request.open(method, this.url, true);
	}
	this.request.send(this.sendData);
    };
    return MEAjax;
}
/* *********************************************************************************************** */

/*
	@obj oggetto dom
	@ev quale avento registrare (click,mouseover,mouseout....)
	@fn handler a funzione da eseguire al catch dell'evento
	@captureMethod true=capturingEventMethod, false bubblingEventMethos
*/
function addEvent(obj,ev,fn,captureMethod){

	if (typeof obj=="undefined" || !obj)
		return;

	try{
		removeEvent(obj,ev,fn);
	}catch(e){
		//no event to remove
	}

	if (typeof captureMethod == "undefined")
		captureMethod = false;

	if(obj.addEventListener){
		// metodo w3c
		obj.addEventListener(ev, fn, captureMethod);
	} else if(obj.attachEvent) {
		// metodo IE
		obj.attachEvent('on'+ev, fn);
	} else {
		// se i suddetti metodi non sono applicabili
		// se esiste gia' una funzione richiamata da quel gestore evento
		if(typeof(obj['on'+ev])=='function'){
			// salvo in variabile la funzione gia' associata al gestore
			var f=obj['on'+ev];
			// setto per quel gestore una nuova funzione
			// che comprende la vecchia e la nuova
			obj['on'+ev]=function(){if(f)f();fn();};
		}
		// altrimenti setto la funzione per il gestore
		else obj['on'+ev]=fn;
	}
}

function removeEvent(obj,ev,fn,captureMethod){
	if (typeof captureMethod == "undefined")
		captureMethod = false;
  if(obj.removeEventListener)
	obj.removeEventListener(ev,fn,captureMethod);
  else if(obj.detachEvent){
    obj.detachEvent('on'+ev,fn);
    obj['on'+ev]=null;
    obj['on'+ev]=null;
  }
}

/* *********************************************************************************************** */
//ricerca elementi in base alla classe

function getElementsByClass(searchClass,node,tag) {
	var classElements = new Array();
	if ( node == null )
		node = document;
	if ( tag == null )
		tag = '*';
	var els = node.getElementsByTagName(tag);
	var elsLen = els.length;
	var pattern = new RegExp("(^|.*[\\s]+)"+searchClass+"([\\s]+.*|$)");
	for (var i = 0, j = 0; i < elsLen; i++) {
		if ( pattern.test(els[i].className) ) {
			classElements[j] = els[i];
			j++;
		}
	}

	return classElements;
}



/*
	restituisce un array di elementi che coincidono con il prametro tagl contenuti nel nodo node
*/
function _getElementsByTagName(tag,node) {
	var classElements = new Array();
	if ( node == null )
		node = document;
	if ( tag == null )
		tag = '*';
	var els = node.getElementsByTagName(tag);

	return els;
}

/* *********************************************************************************************** */

/*
function hide(el){
    el.style.display='none';
}
function show(el){
    el.style.display='block';
}
*/

function hideClass(className){
	var els = getElementsByClass(className);
	for (i=0;i<els.length;i++)
		els[i].style.display = "none";
}
function showClass(className){
	var els = getElementsByClass(className);
	for (i=0;i<els.length;i++)
		els[i].style.display = "";
}
/* ********************************************************************************************** */

// elimina spazi prima e dopo di una stringa

function trim(stringa){
	if (typeof stringa != "number" && typeof stringa != "object")
		return stringa.replace(/^\s*/, '').replace(/\s*$/, '');
	else
		return stringa;
}

function trim2(stringa,symbol){
	if (typeof stringa == "number" || typeof stringa == "object")
		return stringa;

	if (!!symbol){
		if (typeof symbol == "string"){
			var x = new RegExp("^["+symbol+"]*","");
			var y = new RegExp("["+symbol+"]*$","");
			return stringa.replace(x, '').replace(y, '');
		}
		else
			if (typeof symbol[0] != "undefined"){
				for (var i=0;i<symbol.length;i++){
				var x = new RegExp("^"+symbol[i]+"*","ig");
				var y = new RegExp(""+symbol[i]+"*$","ig");
				return_string =  stringa.replace(x, '').replace(y, '');
		}
		return return_string;
		}
	}
	return stringa.replace(/^\s*/, '').replace(/\s*$/, '');
}

/* ********************************************************************************************** */

function evidenzia(img,valore){
	img.style.opacity = valore/10;
	img.style.filter = 'alpha(opacity=' + valore*10 + ')';
}


function showAlertTip(message,type,closeAfter,top){

	if (typeof message=="undefined" || message == ''){
		if (typeof _msg_tip!="undefined")
			_msg_tip.parentNode.removeChild(_msg_tip);
		return;
	}
	if (typeof top!="undefined")
		top = parseInt(top) + "px";
	else
		top = "10px";
	if (typeof type== "undefined"){
		type = "info";
	}

	if (typeof _msg_tip == "undefined"){
		_msg_tip = document.createElement("div");
		_msg_tip.className = "top_message_tip";
	}
	switch (type){
		case "error":
			_msg_tip.style.backgroundColor="red";
			//_msg_tip.style.backgroundImage="url(/img2/transparent_red.png)";
			break;

		default:
			_msg_tip.style.backgroundColor="green";
			//_msg_tip.style.backgroundImage="url(/img2/transparent_green.png)";

			break;
	}

	if (typeof _msg_tip_timer!="undefined")
		clearTimeout(_msg_tip_timer);

	_msg_tip.style.padding="10px 10px 10px 10px";
	_msg_tip.style.color="#FFF";
	_msg_tip.style.position="fixed";
	_msg_tip.style.top=top;

	_msg_tip.style.zIndex="999";
	_msg_tip.style.border="2px solid #AEAEAE";
	_msg_tip.innerHTML=message;
	_msg_tip.visibility="hidden";

	_msg_tip.style.fontWeight="bold";
	_msg_tip.style.fontSize="15px";
	_msg_tip.style.textAlign = "center";
	document.body.appendChild(_msg_tip);

	var sWidth = window.innerWidth ? window.innerWidth : document.documentElement.clientWidth;
	var tWidth = MEgetWidth(_msg_tip);


	_msg_tip.style.left= sWidth/2 - tWidth/2 - 4 + "px";

	MEfadeIn(_msg_tip);
	if (typeof closeAfter=="undefined" || !closeAfter){
		closeAfter = 5000;
	}
	if (closeAfter)
		_msg_tip_timer = setTimeout(function(){MEfadeOut(_msg_tip);},closeAfter);

}


function popUp(url){
	window.open(url,'','toolbar=no,scrollbars=1');
}

function popup(page, title, width, height) {
	var win = null,
		winl = '',
		wint = '';

	if (width){
		winl = (screen.width - width) / 2;
	}

	if (height) {
		wint = (screen.height - height) / 2;
	}

	params = "toolbar=0,";
	params += "location=0,";
	params += "directories=0,";
	params += "status=0,";
	params += "menubar=0,";
	params += "titlebar=0,";
	params += "scrollbars=1,";
	params += "resizable=0,";
	params += "top=" + wint + ",";
	params += "left=" + winl + ",";
	params += "width=" + width + ",";
	params += "height=" + height;

	win = window.open(page, title, params);

	return win;
}

function qsFromForm(form)
{

    var params = "";
    var length = form.elements.length;
    for( var i = 0; i < length; i++ )
    {

	element = form.elements[i];
	if (!element.name)
		continue;

	if (params == '')
	    parSep = "?";
	else
	    parSep = "&";

	if(element.tagName.toLowerCase() == 'textarea' )
        {
                params+=parSep+element.name+"="+Url.encode(element.value);

        }
        else if( element.tagName.toLowerCase() == 'input' )
        {
                if( element.type == 'text' || element.type == 'hidden' || element.type == 'password' || element.type == 'email')
                {
                        params+=parSep+element.name+"="+Url.encode(element.value);
                }
                else if( element.type == 'radio' && element.checked )
                {
                        if( !element.value )
                                params+=parSep+element.name+"="+"on";
                        else
                                params+=parSep+element.name+"="+Url.encode(element.value);

                }
                else if( element.type == 'checkbox' )
                {
                        if(element.checked)
                            params+=parSep+element.name+"="+Url.encode(element.value);
						//else
						//    params+=element.name+"="+"";

                }

        }else if (element.tagName.toLowerCase()=="select"){
			if(element.type=="select-one"){
				if (element.selectedIndex>=0)
					params += parSep+element.name+"="+Url.encode(element.options[element.selectedIndex].value);
			} else if (element.type == "select-multiple") {
				var aSelectMultipleVal = jq(element).val();

				if (typeof aSelectMultipleVal !== 'undefined' && aSelectMultipleVal != null && aSelectMultipleVal.length > 0){
					for( var j = 0; j < aSelectMultipleVal.length; j++ ){
						params += parSep+element.name+"="+Url.encode(aSelectMultipleVal[j]);
					}
				}

			}
		}
    }

    return params;

}

function emptyForm(form)
{
    var length = form.elements.length;
    for( var i = 0; i < length; i++ )
    {

    element = form.elements[i];
	if(element.tagName.toLowerCase() == 'textarea' )
        {
                element.innerHTML = "";
        }
        else if( element.tagName.toLowerCase() == 'input' )
        {
                if( element.type == 'text' || element.type == 'hidden' || element.type == 'password' || element.type == 'email')
                {
                        element.value = "";
                }
                else if( element.type == 'radio' && element.checked )
                {
                        element.checked = false;

                }
                else if( element.type == 'checkbox' )
                {
                        element.checked = false;
                }

        }else if (element.tagName.toLowerCase()=="select"){
	    if(element.type=="select-one"){
			element.value = "";
		    element.selectedIndex = 0;
		}
	}
    }
    return true;

}


function emptyField(element)
{
	if (!element)
		throw("missing required parameter for function emptyField");
	if(element.tagName.toLowerCase() == 'textarea' )
        {
                element.innerHTML = "";

        }
        else if( element.tagName.toLowerCase() == 'input' )
        {
                if( element.type == 'text' || element.type == 'hidden' || element.type == 'password' || element.type == 'email')
                {
                        element.value = "";
                }
                else if( element.type == 'radio' && element.checked )
                {
                        element.checked = false;

                }
                else if( element.type == 'checkbox' )
                {
                        element.checked = false;
                }

        }else if (element.tagName.toLowerCase()=="select"){
	    if(element.type=="select-one"){
			element.value = "";
		    element.selectedIndex = 0;
		}
	}

    return true;

}


function checkUrl(url) {

	var urlRegEx = /^(http(s)?:\/\/)([a-z0-9-]+)(\.[a-z0-9-]+)*(\:[0-9]+)*(\/.*)?$/ig;
	url = url.toLowerCase();
	if (!url.match(urlRegEx)) return false;
	 else return true;
}

function checkEmail(email) {

//var emailRegEx = /^([a-zA-Z0-9_\.\-])+\@(([a-zA-Z0-9\-])+\.)+([a-zA-Z0-9]{2,4})+$/;

var emailRegEx = /^[a-z0-9!#$%&'*+=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?$/ig;
email = email.toLowerCase();
if (!email.match(emailRegEx)) return false;
 else return true;
}
//definisco un alias per la funzione di verifica email
isEmail = checkEmail;

function getValue(name)
{
    var str=window.location.search;
    if (str.indexOf(name)!=-1){
      var pos_start=str.indexOf(name)+name.length+1;
      var pos_end=str.indexOf("&",pos_start);
      if (pos_end==-1){
	 return str.substring(pos_start);
      }else{
	 return str.substring(pos_start,pos_end);
      }
   }else{
      return null;
 }
}

function createCookie(name, value, days, hours, mins, path, domain) {
	var interval = 0;
	if(typeof days != "undefined" && days != '') {
		interval = interval + (parseInt(days) * 24 * 60 * 60);
	}
	if(typeof hours != "undefined" && hours != '') {
		interval = interval + (parseInt(hours) * 60 * 60);
	}
	if(typeof mins != "undefined" && mins != '') {
		interval = interval + (parseInt(mins) * 60);
	}
	var pPath, pDomain, addPathDomain = '', suffDomain;
	pPath = (typeof path != 'undefined' && path != '') ? path : "/";
	addPathDomain += "; path="+pPath;
	pDomain = (typeof domain != 'undefined' && domain != '') ? domain : null;
	if(null !== pDomain){
		addPathDomain += "; domain="+pDomain;
	}

	if(interval > 0) {
		interval = interval * 1000;
		var date = new Date();
		date.setTime(date.getTime() + interval);
		var expires = "; expires=" + date.toGMTString();
	}
	else {
		var expires = "";
	}
	document.cookie = name + "=" + value + expires + addPathDomain;
}

function readCookie(name) {
	var nameEQ = name + "=";
	var ca = document.cookie.split(';');
	for(var i=0;i < ca.length;i++) {
		var c = ca[i];
		while (c.charAt(0)==' ') c = c.substring(1,c.length);
		if (c.indexOf(nameEQ) == 0) return c.substring(nameEQ.length,c.length);
	}
	return null;
}

function deleteCookie(name) {
	var date = new Date();
	var expires = "; expires="+date.toGMTString();
	value = "";
	document.cookie = name+"="+value+expires+"; path=/";
}

function getCookie(c_name)
{
var i,x,y,ARRcookies=document.cookie.split(";");
for (i=0;i<ARRcookies.length;i++)
{
  x=ARRcookies[i].substr(0,ARRcookies[i].indexOf("="));
  y=ARRcookies[i].substr(ARRcookies[i].indexOf("=")+1);
  x=x.replace(/^\s+|\s+$/g,"");
  if (x==c_name)
    {
    return unescape(y);
    }
  }
}

/**
 * Metodo che verifica se i cookie sono attivi
 */
function checkCookiesEnabled() {
	createCookie( 'checkCookie', '1') ;
	if ( !readCookie("checkCookie") ) {
		return false;
	} else {
		deleteCookie( "checkCookie");
		return true;
	}
}

/************** URL UTF8 ENCODE E DECODE ************************/

var Url = {

    // public method for url encoding
    encode : function (string) {
		return encodeURIComponent(string);
        //return encodeURIComponent(this._utf8_encode(string));
    },

    // public method for url decoding
    decode : function (string) {
		return decodeURIComponent(string);
        //return this._utf8_decode(decodeURIComponent(string));
    },

	// public method for absolutizing an url
	absolute : function (url) {
		if (url.match(/^http:\/\//) || url.match(/^https:\/\//)) {
			return url;
		}
		if (url.charAt(0) != "/") {
			var path = window.location.pathname;
			if (path.charAt(path.length-1) != "/") {
				components = path.split("/");
				path = components.splice(0, components.length-1).join("/") + "/";
			}
			url = path + url;
		}
		return window.location.protocol+"//"+window.location.host+url;
	},

    // private method for UTF-8 encoding
    _utf8_encode : function (string) {
        string = string.replace(/\r\n/g,"\n");
        var utftext = "";

        for (var n = 0; n < string.length; n++) {

            var c = string.charCodeAt(n);

            if (c < 128) {
                utftext += String.fromCharCode(c);
            }
            else if((c > 127) && (c < 2048)) {
                utftext += String.fromCharCode((c >> 6) | 192);
                utftext += String.fromCharCode((c & 63) | 128);
            }
            else {
                utftext += String.fromCharCode((c >> 12) | 224);
                utftext += String.fromCharCode(((c >> 6) & 63) | 128);
                utftext += String.fromCharCode((c & 63) | 128);
            }

        }

        return utftext;
    },

    // private method for UTF-8 decoding
    _utf8_decode : function (utftext) {
        var string = "";
        var i = 0;
        var c = 0;
		var c1 = 0;
		var c2 = 0;

        while ( i < utftext.length ) {

            c = utftext.charCodeAt(i);

            if (c < 128) {
                string += String.fromCharCode(c);
                i++;
            }
            else if((c > 191) && (c < 224)) {
                c2 = utftext.charCodeAt(i+1);
                string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
                i += 2;
            }
            else {
                c2 = utftext.charCodeAt(i+1);
                c3 = utftext.charCodeAt(i+2);
                string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
                i += 3;
            }

        }

        return string;
    }

};


/* Base 64 encode/decode */

/**
*
*  Base64 encode / decode
*  http://www.webtoolkit.info/
*
**/

var Base64 = {

	// private property
	_keyStr : "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",

	// public method for encoding
	encode : function (input) {
		var output = "";
		var chr1, chr2, chr3, enc1, enc2, enc3, enc4;
		var i = 0;

		input = Base64._utf8_encode(input);

		while (i < input.length) {

			chr1 = input.charCodeAt(i++);
			chr2 = input.charCodeAt(i++);
			chr3 = input.charCodeAt(i++);

			enc1 = chr1 >> 2;
			enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
			enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
			enc4 = chr3 & 63;

			if (isNaN(chr2)) {
				enc3 = enc4 = 64;
			} else if (isNaN(chr3)) {
				enc4 = 64;
			}

			output = output +
			this._keyStr.charAt(enc1) + this._keyStr.charAt(enc2) +
			this._keyStr.charAt(enc3) + this._keyStr.charAt(enc4);

		}

		return output;
	},

	// public method for decoding
	decode : function (input) {
		var output = "";
		var chr1, chr2, chr3;
		var enc1, enc2, enc3, enc4;
		var i = 0;

		input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");

		while (i < input.length) {

			enc1 = this._keyStr.indexOf(input.charAt(i++));
			enc2 = this._keyStr.indexOf(input.charAt(i++));
			enc3 = this._keyStr.indexOf(input.charAt(i++));
			enc4 = this._keyStr.indexOf(input.charAt(i++));

			chr1 = (enc1 << 2) | (enc2 >> 4);
			chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
			chr3 = ((enc3 & 3) << 6) | enc4;

			output = output + String.fromCharCode(chr1);

			if (enc3 != 64) {
				output = output + String.fromCharCode(chr2);
			}
			if (enc4 != 64) {
				output = output + String.fromCharCode(chr3);
			}

		}

		output = Base64._utf8_decode(output);

		return output;

	},

	// private method for UTF-8 encoding
	_utf8_encode : function (string) {
		if(typeof(string) == 'undefined' || string == null)
			return "";

		string = string.replace(/\r\n/g,"\n");
		var utftext = "";

		for (var n = 0; n < string.length; n++) {

			var c = string.charCodeAt(n);

			if (c < 128) {
				utftext += String.fromCharCode(c);
			}
			else if((c > 127) && (c < 2048)) {
				utftext += String.fromCharCode((c >> 6) | 192);
				utftext += String.fromCharCode((c & 63) | 128);
			}
			else {
				utftext += String.fromCharCode((c >> 12) | 224);
				utftext += String.fromCharCode(((c >> 6) & 63) | 128);
				utftext += String.fromCharCode((c & 63) | 128);
			}

		}

		return utftext;
	},

	// private method for UTF-8 decoding
	_utf8_decode : function (utftext) {
		var string = "";
		var i = 0;
		var c = 0;
		var c1 = 0;
		var c2 = 0;

		while ( i < utftext.length ) {

			c = utftext.charCodeAt(i);

			if (c < 128) {
				string += String.fromCharCode(c);
				i++;
			}
			else if((c > 191) && (c < 224)) {
				c2 = utftext.charCodeAt(i+1);
				string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
				i += 2;
			}
			else {
				c2 = utftext.charCodeAt(i+1);
				c3 = utftext.charCodeAt(i+2);
				string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
				i += 3;
			}

		}

		return string;
	}

};

/***************  AGGIUNGI CSS RUNTIME ******************/

function appendCss(file,idCss){
    var headID = document.getElementsByTagName("head")[0];
    var cssNode = document.createElement('link');
    cssNode.type = 'text/css';
    cssNode.rel = 'stylesheet';
    if (typeof idCss !="undefined" && idCss!="")
	cssNode.id = idCss;
    else
    	idCss = file;
    cssNode.href = file + (typeof __gvs_EXTERNAL_LIBRARY_VERSION != 'undefined' ? __gvs_EXTERNAL_LIBRARY_VERSION : '');
    cssNode.media = 'screen';
    if (!document.getElementById(idCss))
		headID.appendChild(cssNode);
}

/************** RESTITUISCE INFO SUL BROWSER ********************/

function getBrowser(){
	try{
		var info = Array();
		var browser=navigator.appName;
		info["name"] = info[0] = browser;

		var b_version=navigator.appVersion;
		var version = "";

		if (browser.toLowerCase().search(/explorer/)>=0){
			var x = b_version.split(';');
			var version=parseInt(x[1].split(" ")[2]);
		}else{
			var x = b_version.split(' ');
			var version=parseInt(x[0]);
		}
		info["version"] = info[1] = version;

		return info;
	}catch(e){
		return null;
	}

}

var BrowserDetect = {
	init: function () {
		this.browser = this.searchString(this.dataBrowser) || "UNKNOW_BROWSER";
		this.version = this.searchVersion(navigator.userAgent)
			|| this.searchVersion(navigator.appVersion)
			|| "UNKNOW_VERSION";
		this.OS = this.searchString(this.dataOS) || "UNKNOW_OS";
		this.cookieEnabled = this.checkCookieEnabled(navigator.cookieEnabled);
		this.OSVersion = this.searchOSVersion(this.clientStrings);
		return this;
	},
	searchString: function (data) {
		for (var i = 0 ; i < data.length ; i++) {
			var dataStringUserAgent = data[i].ua;
			//var dataStringUserAgent = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.81 Safari/537.36";
			var dataString = data[i].string;
			var dataProp = data[i].prop;
			this.versionSearchString = data[i].versionSearch || data[i].identity;
			this.versionOSSearchString = dataStringUserAgent || dataString;  //in generale rappresenta userAgent
			if(dataString) {
				if(dataString.indexOf(data[i].subString) != -1) {
					return data[i].identity;
				}
			} else if(dataProp) {
				return data[i].identity;
			}
		}
	},
	searchVersion: function (dataString) {
		var index = dataString.indexOf(this.versionSearchString);
		if (index == -1) return;
		return parseFloat(dataString.substring(index+this.versionSearchString.length+1));
	},
	searchOSVersion: function ( client ) {
		var version = {detail: 'UNKNOW_VERSION_OS', spec: 'unknow'};
		if(typeof this.versionOSSearchString !== 'undefined' && '' != this.versionOSSearchString){
			for (var id in client) {
				var cs = client[id];
				if (cs.r.test( this.versionOSSearchString )) {
					version.detail = cs.s;
					break;
				}
			}

			if (/Windows/.test(version.detail)) {
				var regular = /Windows (.*)/.exec( version.detail );
				version.spec = (typeof regular !== 'undefined' && regular !== null) ? regular[1] : 'unknow';
			}
			switch (version.detail) {
				case 'Mac OS X':
					var regular = /Mac OS X (10([_|.][\d]+)*)/.exec( this.versionOSSearchString );
						aTemp;
					version.spec = (typeof regular !== 'undefined' && regular !== null) ? regular[1] : 'unknow';
					if ( /[\_]/.test(version.spec) ) {
						aTemp = version.spec.split('_');
					}
					if ( /[\.]/.test(version.spec) ) {
						aTemp = version.spec.split('.');
					}
					version.spec = aTemp;
					break;

				case 'Android':
					var regular = /Android[-|\s]?([\d]([.]?[\d]+)*)/.exec( this.versionOSSearchString );
					version.spec = (typeof regular !== 'undefined' && regular !== null) ? regular[1] : 'unknow';
					if ( /[\.]/.test(version.spec) ) {
						var aTemp = version.spec.split('.');
						version.spec = aTemp;
					}
					break;

				case 'iOS':
					var regular = /OS ((\d+)[_|.](\d+)[_|.]?(\d+)?)/.exec( this.versionOSSearchString),
						aTemp;
					version.spec = (typeof regular !== 'undefined' && regular !== null) ? regular[1] : 'unknow';
					if ( /[\_]/.test(version.spec) ) {
						aTemp = version.spec.split('_');
					}
					if ( /[\.]/.test(version.spec) ) {
						aTemp = version.spec.split('.');
					}
					version.spec = aTemp;
					break;

				case 'Linux':
					osVersion = /x(\d+)_(\d+)/.exec( this.versionOSSearchString );
					version.spec = (null !== osVersion && typeof osVersion !== 'undefined' && typeof(osVersion[1]) !== 'undefined' && typeof(osVersion[2]) !== 'undefined') ? 'x' + osVersion[1] + ' - ' + osVersion[2] + ' bit' : '-';
					break;
			}
		}
		return version;
	},
	checkCookieEnabled: function ( browserC ){
		// cookie
		var cookieEnabled = (browserC) ? true : false;

		if (typeof browserC == 'undefined' && !cookieEnabled) {
			document.cookie = 'testcookie=1';
			cookieEnabled = (document.cookie.indexOf('testcookie') != -1) ? true : false;
			document.cookie = "testcookie=; expires=Thu, 01 Jan 1970 00:00:00 UTC";
		}

		return cookieEnabled;
	},
	dataBrowser: [
		{
			string: navigator.userAgent,
			subString: "Chrome",
			identity: "Chrome"
		},
		{ 	string: navigator.userAgent,
			subString: "OmniWeb",
			versionSearch: "OmniWeb/",
			identity: "OmniWeb"
		},
		{
			string: navigator.vendor,
			subString: "Apple",
			identity: "Safari",
			versionSearch: "Version"
		},
		{
			prop: window.opera,
			identity: "Opera",
			versionSearch: "Version"
		},
		{
			string: navigator.vendor,
			subString: "iCab",
			identity: "iCab"
		},
		{
			string: navigator.vendor,
			subString: "KDE",
			identity: "Konqueror"
		},
		{
			string: navigator.userAgent,
			subString: "Firefox",
			identity: "Firefox"
		},
		{
			string: navigator.vendor,
			subString: "Camino",
			identity: "Camino"
		},
		{		// Netscapes(6+)
			string: navigator.userAgent,
			subString: "Netscape",
			identity: "Netscape"
		},
		{
			string: navigator.userAgent,
			subString: "MSIE",
			identity: "Explorer",
			versionSearch: "MSIE"
		},
		{
			string: navigator.userAgent,
			subString: "Gecko",
			identity: "Mozilla",
			versionSearch: "rv"
		},
		{ 		//Netscapes(4-)
			string: navigator.userAgent,
			subString: "Mozilla",
			identity: "Netscape",
			versionSearch: "Mozilla"
		}
	],
	dataOS : [
		{
			string: navigator.userAgent,
			subString: "iPad",
			identity: "iPad"
	    },
		{		// Android
			string: navigator.userAgent,
			subString: "Android",
			identity: "Android"
		},
		{
			string: navigator.userAgent,
			subString: "iPhone",
			identity: "iPhone/iPod"
	    },
		{
			string: navigator.platform,
			ua: navigator.userAgent,
			subString: "Win",
			identity: "Windows"
		},
		{
			string: navigator.platform,
			ua: navigator.userAgent,
			subString: "Mac",
			identity: "Mac"
		},
		{
			string: navigator.platform,
			ua: navigator.userAgent,
			subString: "Linux",
			identity: "Linux"
		}
	],
	clientStrings : [
		{s:'Windows 3.11', r:/Win16/},
		{s:'Windows 95', r:/(Windows 95|Win95|Windows_95)/},
		{s:'Windows ME', r:/(Win 9x 4.90|Windows ME)/},
		{s:'Windows 98', r:/(Windows 98|Win98)/},
		{s:'Windows CE', r:/Windows CE/},
		{s:'Windows 2000', r:/(Windows NT 5.0|Windows 2000)/},
		{s:'Windows XP', r:/(Windows NT 5.1|Windows XP)/},
		{s:'Windows Server 2003', r:/Windows NT 5.2/},
		{s:'Windows Vista', r:/Windows NT 6.0/},
		{s:'Windows 7', r:/(Windows 7|Windows NT 6.1)/},
		{s:'Windows 8.1', r:/(Windows 8.1|Windows NT 6.3)/},
		{s:'Windows 8', r:/(Windows 8|Windows NT 6.2)/},
		{s:'Windows NT 4.0', r:/(Windows NT 4.0|WinNT4.0|WinNT|Windows NT)/},
		{s:'Windows ME', r:/Windows ME/},
		{s:'Android', r:/Android/},
		{s:'Open BSD', r:/OpenBSD/},
		{s:'Sun OS', r:/SunOS/},
		{s:'Linux', r:/(Linux|X11)/},
		{s:'iOS', r:/(iPhone|iPad|iPod)/},
		{s:'Mac OS X', r:/Mac OS X/},
		{s:'Mac OS', r:/(MacPPC|MacIntel|Mac_PowerPC|Macintosh)/},
		{s:'QNX', r:/QNX/},
		{s:'UNIX', r:/UNIX/},
		{s:'BeOS', r:/BeOS/},
		{s:'OS/2', r:/OS\/2/},
		{s:'Search Bot', r:/(nuhk|Googlebot|Yammybot|Openbot|Slurp|MSNBot|Ask Jeeves\/Teoma|ia_archiver)/}
	]
};

/************** CENTRA UN DIV IN MEZZO ALLA PAGINA *********************/
	function MEcenterDiv(divId){
		document.getElementById(divId).style.position="absolute";

		var pWidth= document.body.clientWidth? document.body.clientWidth: document.documentElement.clientWitdh;
		var pHeight = document.body.clientHeight ? document.body.clientHeight : document.documentElement.clientHeight;

		var sWidth= window.innerWidth ? window.innerWidth: document.documentElement.clientWidth;
		var sHeight = window.innerHeight ? window.innerHeight : document.documentElement.clientHeight;

		prevVis = document.getElementById(divId).style.visibility;
		prevDisp = document.getElementById(divId).style.display;
		document.getElementById(divId).style.visibility="hidden";
		document.getElementById(divId).style.display="block";

		var divWidth= document.getElementById(divId).clientWidth;
		var divHeight = document.getElementById(divId).clientHeight;
		if (sWidth<= divWidth){
			sWidth= pWidth;
		}

		if (sHeight <= divHeight){
			sHeight = pHeight;
		}
		var scrolly = (document.all)?document.documentElement.scrollTop:window.pageYOffset;

		var top = scrolly + (sHeight/2 - divHeight/2);
	    if (top<0) top = 10;
	    top+="px";
		var left = (sWidth/2 - divWidth/2) + "px";
		document.getElementById(divId).style.top =  top;

		document.getElementById(divId).style.left = left;
		document.getElementById(divId).style.visibility=prevVis;
		document.getElementById(divId).style.display=prevDisp;
	}

/* submit di un form mediante una POST ajax */

	function submitAjaxForm(form,handleFunction,action){

				if (typeof action == "undefined"){
					action = form.action;
				}
				if (typeof form == "string")
					var form = document.getElementById(form);

				var pars = qsFromForm(form);

				if (action)
					action += pars;
				else
					action = form.action + pars;

				var ajaxObj = getMEAjaxObj();
				ajaxObj.Request("POST", action, handleFunction);
	}

/* effettua il submit del form premendo enter - l'evento deve essere associato a keypress sui campi input del form */

	function submitOnEnter(e,oForm){
		var kC  = (window.event) ?    // MSIE or Firefox?
	           window.event.keyCode : (e.keyCode ? e.keyCode : e.which);
		var returnCode = 13;
		if (kC == returnCode)
			oForm.submit();
	}

/* esegue una azione sulla pressione di invio l'evento deve essere associato a keypress sui campi input del form */
	function eventOnEnter(e,handler){
		var kC  = (window.event) ?    // MSIE or Firefox?
	           window.event.keyCode : (e.keyCode ? e.keyCode : e.which);
		var returnCode = 13;
		if (kC == returnCode)
			handler();
	}

/* Bind di funzioni per non perdere il riferimento all'oggetto originario */
/* Estende le funzioni di javascript */
if (typeof Function.bind !== 'function') {
	Function.prototype.bind = function (obj, arguments) {
		var fx = this;
		return function () {
			return fx.apply(obj, arguments);
		};
	};
}


/**
/* IMPLEMENTAZIONE DEL MERGE_SORT PER ORDINARE IN AMNIERA VELOCE UN ARRAY E POTER USARE LA RICERCA BINARIA ER SELEZIONARE GLI ELEMENTI*/
/* @metodoConfronto(a,b)  funzione utilizzata per esffettuare il confronto (restituisce 1 se a>b, -1 se a<b, 0 se a == b)
/* 							se non specificata ne usa una di default che effettua il confronto tra stringhe (_defaultMetodoConfronto)




																																		*/
//merge_sort ricorsivo

function mergeSort(items,metodoConfronto){
	if (!metodoConfronto)
		metodoConfronto = _defaultMetodoConfronto;
    if (items.length == 1) {
        return items;
    }

    var middle = Math.floor(items.length / 2),
        left    = items.slice(0, middle),
        right   = items.slice(middle);

    return merge(mergeSort(left,metodoConfronto), mergeSort(right,metodoConfronto),metodoConfronto);
}



function merge(left, right, metodoConfronto){
    var result = [];

    while (left.length > 0 && right.length > 0){

//		alert(left[0]+" - "+ right[0] +" = " + metodoConfronto(left[0],right[0]));
        if (metodoConfronto(left[0],right[0]) < 0){
            result.push(left.shift());
        } else {
            result.push(right.shift());
        }
    }
    return result.concat(left).concat(right);
}

function _defaultMetodoConfronto(left, right)
{
	if(left == right)
		return 0;
	else if(left < right)
		return -1;
	else
		return 1;
}

/**
/* IMPLEMENTAZIONE DEL MERGE_SORT PER ORDINARE IN AMNIERA VELOCE UN ARRAY E POTER USARE LA RICERCA BINARIA ER SELEZIONARE GLI ELEMENTI*/
/* @metodoConfronto(a,b)  funzione utilizzata per effettuare la ricerca Binaria in un array ordinato
/* Restituisce l'indice dell'elemento nell'array se esiste, -1 altrimenti
/* 							se non specificata ne usa una di default che effettua il confronto tra stringhe (_defaultMetodoConfronto)
															*/
function binarySearch (needle, haystack, metodoConfronto ) {
	if (typeof(haystack) === 'undefined' || !haystack.length) return -1;

	var high = haystack.length - 1;
	var low = 0;
	while (low <= high) {
		mid = parseInt((low + high) / 2);
		element = haystack[mid];
		if (metodoConfronto(element,needle)==1) {
			high = mid - 1;
		} else if (metodoConfronto(element,needle)==-1) {
			low = mid + 1;
		} else {
			return mid;
		}
	}

	return -1;
};


function getPosition(oElement,relative)
{
	var pos = {};
	pos.y = 0;
	pos.x = 0;
	while( oElement != null ){
		pos.y += oElement.offsetTop ?  oElement.offsetTop : 0;
		pos.x += oElement.offsetLeft ?  oElement.offsetLeft : 0;
		oElement = oElement.offsetParent;
        if (relative)
            break;
	}
	return pos;
}


function getMousePosition(e){
  try{
	tempX = e.pageX ? e.pageX : event.clientX + document.body.scrollLeft;
	tempY = e.pageY ? e.pageY : event.clientY + document.body.scrollTop;
	// catch possible negative values in NS4
	if (tempX < 0){tempX = 0;}
	if (tempY < 0){tempY = 0;}
	// show the position values in the form named Show
	// in the text fields named MouseX and MouseY
	var pos = {};
	pos.x = tempX;
	pos.y = tempY;
  }catch(e){
	return null;
  }
  return pos;
}

/******* PseudoAjax Image Upload ********************/


function get(theVar){
	return document.getElementById(theVar);
}

function removeElement(element){
	if (typeof element == "string")
		var element = document.getElementById(element);
    try{
	    element.parentNode.removeChild(element);
	}catch(e)
	{
		//alert(e);
	}
}

function submitImgForm(form,resultContainer){
 	var loaderImg = document.createElement("img");
 	loaderImg.setAttribute("src","/img2/loader7.gif");
 	get(resultContainer).appendChild(loaderImg);
	function doUpload(){
		removeEvent(get('iFrameImg'),"load", doUpload);
		if (typeof resultContainer!="undefined"){
			var cross = "javascript: ";
			cross += "window.parent.get('"+resultContainer+"').innerHTML = document.body.innerHTML;void(0);";
			get('iFrameImg').src  = cross;
            doUpload2();
		}
	}
    function doUpload2(){

        removeEvent(get('iFrameImg'),"load", doUpload2);

        setTimeout("removeElement('"+_uploader_iframeContainer.id+"')",1000);
    }
try{
	_uploader_iframeHTML = "<iframe name=\"iFrameImg\" id=\"iFrameImg\" width=\"0\" height=\"0\" border=\"0\" style=\"width: 0; height: 0; border: none;\";></iframe>";
	_uploader_iframeContainer = document.createElement("div");
	_uploader_iframeContainer.id = "iFrameImgContainer";
	_uploader_iframeContainer.innerHTML = _uploader_iframeHTML;
	get(resultContainer).parentNode.appendChild(_uploader_iframeContainer);
	iFrameImg = document.getElementById("iFrameImg");

	_target = form.getAttribute("target");
	_method = form.getAttribute("method");
	_enctype = form.getAttribute("enctype");
	_encoding = form.getAttribute("encoding");

	form.setAttribute("target","iFrameImg");
	form.setAttribute("method","post");
	form.setAttribute("enctype","multipart/form-data");
	form.setAttribute("encoding","multipart/form-data");
	addEvent(get('iFrameImg'),"load", doUpload);
	form.submit();
	if (!_target)
		form.setAttribute("target","");
	else
		form.setAttribute("target",_target);
	form.setAttribute("method",_method);
	form.setAttribute("enctype",_enctype);
	form.setAttribute("encoding",_encoding);
}
catch(e){
}
	return false;
}

/***************************************************/

//variabile globale per testare se si tratta di internet explorer 6
function isIE6() {
	return false /*@cc_on || @_jscript_version < 5.7 @*/;
}
function isIE(){
	var br = getBrowser();
	var br = br["name"];
	if (br.search(/Internet Explorer/) > 0)
		return true;
	else
		return false;
}

function getInternetExplorerVersion() {
	var rv = -1;
	if (navigator.appName == 'Microsoft Internet Explorer')
	{
		var ua = navigator.userAgent;
		var re  = new RegExp("MSIE ([0-9]{1,}[\.0-9]{0,})");
		if (re.exec(ua) != null)
			rv = parseFloat( RegExp.$1 );
	}
	else if (navigator.appName == 'Netscape')
	{
		var ua = navigator.userAgent;
		var re  = new RegExp("Trident/.*rv:([0-9]{1,}[\.0-9]{0,})");
		if (re.exec(ua) != null)
			rv = parseFloat( RegExp.$1 );
	}
	return rv;
}


function addJS(script_filename,id) {

    var html_doc = document.getElementsByTagName('head').item(0);
    var js = document.createElement('script');

	if (typeof id == "undefined")
		id = script_filename;

    js.setAttribute('language', 'javascript');
    js.setAttribute('type', 'text/javascript');
    js.setAttribute('src', script_filename);
	js.setAttribute('id', id);

	if (!document.getElementById(id)) {
		html_doc.appendChild(js);
	}

    return false;
}


/* FUNIONE ESPANZIONE TEXTAREA AUTO */
function espandiBox(id){
    setTimeout("_espandiBox('"+id+"')",500);
}
function _espandiBox(id){

	/* oDiv = document.getElementById(div); */
	oTxtArea=document.getElementById(id);

	if (typeof oTxtArea.isExpanding !="undefined" && oTxtArea.isExpanding == true)
		return;
	else
		oTxtArea.isExpanding = true;
	if (typeof oTxtArea.initRows == "undefined")
		oTxtArea.initRows = oTxtArea.rows;

	if (oTxtArea.rows * oTxtArea.cols < 0)
		return;

	a = oTxtArea.value.split('\n');
	b=0;
	for (x=0;x < a.length; x++) {
		if (a[x].length >= oTxtArea.cols) b+= Math.floor(a[x].length/oTxtArea.cols);
	}

	b+=a.length;

	if (b > oTxtArea.initRows)
		oTxtArea.rows = b;
	else{
		oTxtArea.rows = oTxtArea.initRows;
	}
	oTxtArea.isExpanding = false;
	oTxtArea.focus();
}


function isNumeric(input)

{
   if (typeof input == "undefined" || input == "")
	return false;

   var ValidChars = "0123456789,.";
   var IsNumber=true;
   var Char;


   for (var i = 0; i < input.length && IsNumber == true; i++)
      {
      Char = input.charAt(i);
      if (ValidChars.indexOf(Char) == -1)
         {
         IsNumber = false;
         }
      }
   return IsNumber;

}

function getFuncName(oFunction){
	if (typeof oFunction != "function" || !oFunction)
		return "main";
	var fn = oFunction.toString();

	var fname = fn.substring(fn.indexOf("function") + 8, fn.indexOf('(')) || 'anonymous';
	return fname;

}

function _raiseError(message){
	if (typeof message=="undefined")
		message = "no description";
		throw new Error('[error] '+ (getFuncName(arguments.callee.caller)) + ': '+message+"\n\n StackTrace : "+getStackTrace().join(",\n\n"));
}

function _doLog(message, type) {
	try{		
		if (typeof console!="undefined" && console[type || 'debug']){			
			console[type || 'debug'](getFuncName(arguments.callee.caller), message);
		}else{
			if (typeof console!= "undefined" && console.log)
				console.log(message);
		}
	}catch(e){
				
	}
}


//restituisce stringa con var dump
function var_dump(variable,full,maxDeep,_count) {

	var message = "";
	try{
		if (!maxDeep)
			maxDeep = 2;

		if (!full)
			full = false;

		if (!_count)
			_count = 0;

		_count = _count + 1;

		var type = typeof variable;


		tabs = "";
		tabs_1 = "";
		for (var i = 0; i< _count;i++)
			tabs= tabs+"\t";
		for (var i = 0; i< _count-1;i++)
			tabs_1= tabs_1+"\t";

		if (_count == 1)
			message+=type+"\n";

		if (type == "object"){

			if (_count>maxDeep){
				for (var name in variable){
					message = message + tabs +name+" = \""+(typeof variable[name])+"\" { .... },\n";
				}
			}else{
				for (var name in variable){
					message = message + (typeof variable[name] == "object" ? "\n"+tabs_1 : tabs) +name+" = \""+(typeof variable[name])+"\" {"+(typeof variable[name] == "object" ? "\n"+tabs_1 : "")+" "+_var_dump(variable[name],full,maxDeep,_count)+(typeof variable[name] == "object" ? "\n"+tabs_1 : "")+"},\n";
				}

			}
			message+="\n";
		}else{

			if ((type == "function" && full) || type!="function"){
				message+=variable;

			}else{
				message+="....... (set full = 1 to see the func body)";
			}
		}

	}catch(e){
		return message;
	}

	return(message);
}

function getStackTrace() {
  var callstack = [];
  var isCallstackPopulated = false;
  var i = null;
  try {
    i.dont.exist+=0; //doesn't exist- that's the point
  } catch(e) {
    if (e.stack) { //Firefox
      var lines = e.stack.split('\n');
      for (var i=0, len=lines.length; i<len; i++) {
        if (lines[i].match(/^\s*[A-Za-z0-9\-_\$]+\(/)) {
          callstack.push(lines[i]);
        }
      }
      //Remove call to printStackTrace()
      callstack.shift();
      isCallstackPopulated = true;
    }
    else if (window.opera && e.message) { //Opera
      var lines = e.message.split('\n');
      for (var i=0, len=lines.length; i<len; i++) {
        if (lines[i].match(/^\s*[A-Za-z0-9\-_\$]+\(/)) {
          var entry = lines[i];
          //Append next line also since it has the file info
          if (lines[i+1]) {
            entry += "&quot; at &quot;" + lines[i+1];
            i++;
          }
          callstack.push(entry);
        }
      }
      //Remove call to printStackTrace()
      callstack.shift();
      isCallstackPopulated = true;
    }
  }
  if (!isCallstackPopulated) { //IE and Safari
    var currentFunction = arguments.callee.caller;
    while (currentFunction) {
	  var fn = currentFunction.toString();
      var fname = fn.substring(fn.indexOf("function") + 8, fn.indexOf('(')) || 'anonymous';
      callstack.push(fname);
      currentFunction = currentFunction.caller;
    }
  }
  return callstack;
}


stringify = function (obj) {
	var t = typeof (obj);
	if (t != "object" || obj === null) {
		// simple data type
		if (t == "string") obj = '"'+obj+'"';
			return String(obj);
		}
		else {
			var n, v, json = [], arr = (obj && obj.constructor === Array);
			//voglio glia array sempre associativi (obj jSon)
			//arr = obj.constructor === Array;
			for (n in obj) {

				v = obj[n]; t = typeof(v);
				if (t == "function")
					continue;
				if (t == "string") v = '"'+v+'"';
				else if (t == "object" && v !== null) v = stringify(v);

				if (arr)
					json.push(String(v));
				else
					json.push( ('"' + n + '":') + String(v));
			}

			return (arr ? "[" : "{") + String(json) + (arr ? "]" : "}");
		}
	};

/*
	cerca un elemento (neddle) dentro ad  un array (haystack)
*/
function in_array(needle, haystack) {
    var length = haystack.length;
    for(var i = 0; i < length; i++){
        if(haystack[i] == needle) return true;
    }
    return false;
}


/* Scroll Progressivo*/


/*
 * getPageSize()
 *
 * Returns array with page width, height and window width, height
 *
 */
function getPageSize(){

 var xScroll, yScroll;

 if (window.innerHeight && window.scrollMaxY) {
     xScroll = document.body.scrollWidth;
     yScroll = window.innerHeight + window.scrollMaxY;
 } else if (document.body.scrollHeight > document.body.offsetHeight){ // all but Explorer Mac
     xScroll = document.body.scrollWidth;
     yScroll = document.body.scrollHeight;
 } else { // Explorer Mac...would also work in Explorer 6 Strict, Mozilla and Safari
     xScroll = document.body.offsetWidth;
     yScroll = document.body.offsetHeight;
 }

 var windowWidth, windowHeight;
 if (self.innerHeight) {    // all except Explorer
     windowWidth = self.innerWidth;
     windowHeight = self.innerHeight;
 } else if (document.documentElement && document.documentElement.clientHeight) { // Explorer 6 Strict Mode
     windowWidth = document.documentElement.clientWidth;
     windowHeight = document.documentElement.clientHeight;
 } else if (document.body) { // other Explorers
     windowWidth = document.body.clientWidth;
     windowHeight = document.body.clientHeight;
 }

 // for small pages with total height less then height of the viewport
 if(yScroll < windowHeight){
     pageHeight = windowHeight;
 } else {
     pageHeight = yScroll;
 }

 // for small pages with total width less then width of the viewport
 if(xScroll < windowWidth){
     pageWidth = windowWidth;
 } else {
     pageWidth = xScroll;
 }

 return [ pageWidth,pageHeight,windowWidth,windowHeight ];
}

//IE memory leak purge
function purgeDomObj(d) {
	try{
		if (typeof d == "undefined" || !d){
		  return;
		}
		var a = d.attributes, i, l, n;
		var b = d.childNodes;

			for (var i in d){
				if (d[i] && typeof d[i] == "Object"){
					d.purgeDomObj(d[i]);
				}
			}


			if (a) {
				l = a.length;
				for (i = 0; i < l; i += 1) {
					n = a[i].name;
					if (typeof d[n] === 'function') {
						d[n] = null;
					}
				}
			}

			if (b) {
				l = b.length;
				for (i = 0; i < l; i += 1) {

					if (d.childNodes[i]){
						purgeDomObj(d.childNodes[i]);
						d.removeChild(d.childNodes[i]);
					}
				}

			}
		d.innerHTML = "";
		d = null;
	}catch(e){
		//_doLog("error:"+e,'error');
	}

}

function updateVisibilita(id,field,obj){
    if (!obj || !id || field.trim()=="") return;
    var oAjax = new getMEAjaxObj();
    var value=0;
    if (obj.checked) value=1;
    var pars = "id="+id+"&field="+field+"&value="+value;
    var url = "/backoffice/updateVisibilita.php?"+pars;
    oAjax.Request('POST',url, updateVisibilitaH);
}

function updateVisibilitaH(data){
    if (data.responseText=="-1"){
	alert("Si è verificato un problema, riprovare l'operazione");
	return;
    }
}

function preloadImages(aImages,onloadF){
	var __ti_preload_imgs_idx = 0;
	var img = document.createElement("img");
	img.onload = function(){
		if (aImages.length > ++__ti_preload_imgs_idx){
			this.src = aImages[__ti_preload_imgs_idx];
		}else{
			this.onload = function(){};
			if (typeof onloadF == "function")
			onloadF();
		}
	};
	img.src = aImages[0];
}


/*confronta due array passati per argomento e ritorna true se l'array2 contiene gli stessi elementi dell'array1, anche in ordine diverso, altrimenti ritorna false*/
function fnSameElemArray(aArray1, aArray2){
    if(aArray1.length != aArray2.length)
		bSame = false;
    else{
        bSame = true;
        var iCounter = 0;
        var aSorted1 = mergeSort(aArray1);
        var aSorted2 = mergeSort(aArray2);
	for(var i=0;i<aSorted1.length;i++)
	    if(aSorted1[i] != aSorted2[i]){
		bSame = false;
		break;
	    }
    }
    return bSame;
}


/*elimina un nodo dal dom passandogli il suo id*/
function fnRemoveNodeById(sNode){
    var oNodeToRemove = document.getElementById(sNode);
	if (oNodeToRemove)
		oNodeToRemove.parentNode.removeChild(oNodeToRemove);
    return;
}

function disableAutoComplete(selector){

    if (!selector)
		return;
	var first = selector.charAt(0);
    var id =selector.split(first)[1];
    if (first=='#'){
		var inputElement = document.getElementById(id);
        if (inputElement)
			inputElement.setAttribute("autocomplete","off");
	}
    else
	if (first=='.'){
        var inputElements = document.getElementsByClass(id);
        for (i=0; inputElements[i]; i++) {
                inputElements[i].setAttribute("autocomplete","off");
        }
    }
}

function trackGAClickEvent(category,action,opt_label,opt_noninteraction){

	if (typeof opt_noninteraction == 'undefined' || !opt_noninteraction){
		opt_noninteraction = true;
	}else{
		opt_noninteraction = false;
	}

	if (typeof category == "undefined" || typeof action == "undefined" || typeof opt_label == "undefined"){
		if (typeof __debugging_doLog!="undefined" && __debugging_doLog)
			_doLog("Missing Required Field");
			return false;
	}

	if (typeof _gaq == "undefined"){
		if (typeof __debugging_doLog!="undefined" && __debugging_doLog)
			_doLog("_gaq is undefined");
	}else{
		_gaq.push(['_trackEvent', category, action, opt_label]);
		if (typeof __debugging_doLog!="undefined" && __debugging_doLog)
			_doLog("_gaq.push(['_trackEvent','"+category+"','"+ action+ "','"+opt_label+"');");
	}

	if (typeof ga == "undefined"){
		if (typeof __debugging_doLog!="undefined" && __debugging_doLog)
			_doLog("ga is undefined");
	}else{
		ga('send', 'event', category, action, opt_label, {'nonInteraction': opt_noninteraction});
		if (typeof __debugging_doLog!="undefined" && __debugging_doLog)
			_doLog("ga('send','event','"+category+"','"+ action+ "','"+opt_label+"', {'nonInteraction': "+opt_noninteraction+"'});");

	}

	return true;

}

Function.prototype.clone=function(obj){
	function F(){}
	F.prototype = obj;
	return new F();
};


String.prototype.ucwords = function(){
    return this.replace(/\w+/g, function(a){
        return a.charAt(0).toUpperCase() + a.substr(1).toLowerCase();
    });
};

if (!String.prototype.format) {
	String.prototype.format = function() {
		var str = this.toString();

		if (!arguments.length){
			return str;
		}

		var args = typeof arguments[0],
			args = (("string" == args || "number" == args) ? arguments : arguments[0]);

		for (arg in args){
			str = str.replace(RegExp("\\{" + arg + "\\}", "gi"), args[arg]);
		}

		return str;
	}
}

if (typeof Event != "undefined" && typeof Event.prototype != "undefined")
Event.prototype.stopBubbling = function(){
	try{
		this.cancelBubble = true;
		this.stopPropagation();
	}catch(e){
		_doLog(e);
	}
};

function simulateClickOnLink(oA,forceNewWindow) {
	if (typeof forceNewWindow == "undefined")
		forceNewWindow = false;
	if (typeof(oA.click) != 'undefined' && /msie/i.test(navigator.userAgent)) {
		var fakeLink = document.createElement("a");
		fakeLink.href = oA.href;
		if (forceNewWindow)
			fakeLink.target = "_blank";
		else
			fakeLink.target = oA.target;
		document.body.appendChild(fakeLink);
		fakeLink.click();
	} else {
		if ((oA.target && oA.target == "_blank") || forceNewWindow) {
			window.open(oA.href);
			setTimeout(function(){document.body.focus();},0);
		} else {
			document.location.href = oA.href;
		}
	}
}

stop_event_bubbling=function(e,hard){
	/*il parametro
		hard = true
	   fa si che su IE/Safari oltre a stoppare la propagazione dell'evento,
	   blocca anche il comportamento predefinito del browser sull'evento stesso.
	*/
	if (typeof hard == "undefined"){
		hard = false;
	}

    if (!e)
		e = window.event;
    e.cancelBubble = true;
    if (e.stopPropagation) e.stopPropagation();
	if (hard){
		if (typeof Event != "undefined" && Event.stop)
			Event.stop(e);
	}

};

var isMobile = {
    Android: function() {
        return navigator.userAgent.match(/\bAndroid\b/i);
    },
    BlackBerry: function() {
        return navigator.userAgent.match(/\bBlackBerry\b/i);
    },
    iOS: function() {
        return navigator.userAgent.match(/\b(iPhone|iPad|iPod)\b/i);
    },
    Opera: function() {
        return navigator.userAgent.match(/\bOpera Mini\b/i);
    },
    Windows: function() {
        return navigator.userAgent.match(/\bIEMobile\b/i);
    },
    any: function() {
        return (isMobile.Android() || isMobile.BlackBerry() || isMobile.iOS() || isMobile.Opera() || isMobile.Windows());
    }
};

function checkAndFormatCurrency(input,event,blurOnCheck,acceptDecimal){

		if (!input || !input.value)
			return false;
		if (typeof acceptDecimal == "undefined")
			acceptDecimal = false;

    var ev = event ? event : window.event;

        var prezzo = input.value.replace(/\./g,'');
		var iLength_init = input.value.length;
		if (!prezzo || parseInt(prezzo) <= 0){
			input.value = "";
			return false;
		}

		var caretPos;
		if (ev)
			caretPos = getCaretPosition(input);
		var length = prezzo.length;

        var prezzo_old = prezzo;

        prezzo = "";
		alreadyAddedDecSep = false;
        for (var i=0;i<length;i++){
            if ( isNumeric(prezzo_old.charAt(i))){
				if (prezzo_old.charAt(i)==","){
					if(!acceptDecimal || alreadyAddedDecSep)
						continue;
					alreadyAddedDecSep = true;
				}
                prezzo = prezzo + "" + prezzo_old.charAt(i);
            }else{
                if (i < caretPos)
                    caretPos--;
            }
        }
		var decimal = "";
		var hasDecimal = prezzo.search(/,/);
		if (hasDecimal > 0){
			var aPrezzo = prezzo.split(",");
			if (2 == aPrezzo.length){
				prezzo = aPrezzo[0];
				decimal = aPrezzo[1];
			}
		}

        length = prezzo.length;
        var newPrice = new Array();
        var newPriceIdx = 0;
        var digitCounter = 1;
		for (var i=length;i>=1;i--){
				newPrice[newPriceIdx++] = prezzo.charAt(i-1);
				if (((i-1 % 3) != 0) && ((digitCounter % 3) == 0)){
						digitCounter = 1;
						newPrice[newPriceIdx++] = ".";
				}else{
						digitCounter++;
				}
		}

		var output = "";
		for (var i=newPrice.length-1;i>=0;i--){
				output+=newPrice[i];
		}

		if (hasDecimal > 0)
		 output+=","+decimal;

		if ( output != input.value ) {
			jq( input ).change();
		}
		input.value = output;
		var iLength_post = input.value.length;

		if (iLength_post > iLength_init)
			caretPos++;

		if (ev)
			setCaretPos(input,caretPos);

		return true;
}

function getCaretPosition(oField) {

  // Initialize
  var iCaretPos = 0;

  // IE Support
  if (document.selection && jq(oField).is(':visible')) { // su IE effettuare il focus su un elemento non visibile genera un errore js che rompe la pagina

    // Set focus on the element
    oField.focus ();

    // To get cursor position, get empty selection range
    var oSel = document.selection.createRange ();

    // Move selection start to 0 position
    oSel.moveStart ('character', -oField.value.length);

    // The caret position is selection length
    iCaretPos = oSel.text.length;
  }

  // Firefox support
  else if (oField.selectionStart || oField.selectionStart == '0')
    iCaretPos = oField.selectionStart;

  // Return results
  return (iCaretPos);
}

function setCaretPos(fieldOrId,pos){

    var oField = (typeof fieldOrId == "string" || fieldOrId instanceof String) ? document.getElementById(fieldOrId) : fieldOrId;

    if(!oField){
        return false;
    }else if(oField.createTextRange){
        var textRange = oField.createTextRange();
        textRange.collapse(true);
        textRange.moveEnd('character',pos);
        textRange.moveStart('character',pos);
        textRange.select();


        return true;
    }else if(oField.setSelectionRange){

        oField.setSelectionRange(pos,pos);
        return true;
    }

    return false;
}


/* Classe per la gestione dei tracciamenti tramite immegine 1x1px*/
var ImgTracking = function(){};

ImgTracking.prototype.trackEvent = function ( actionUrl, event ) {
  var charRandom = actionUrl.indexOf( "&" )!= -1 ? '&' : '?';

  /* Simula chiamata ajax per incremento contatori */
  var imagecount = document.createElement( 'img' );
  imagecount.src = actionUrl + charRandom + Math.random();

  /* Stoppa la propagazione del click nel div contenitore*/
  stop_event_bubbling(event);
};

function trackGMapImpression(section,type){

	if (typeof section == "undefined" || !in_array(type.toLowerCase(),new Array("static","v3","v2")))
		return false;

	trackGAClickEvent(section,'mappa_google','impressions_'+type.toLowerCase());
}

function checkCurrencysPercentual(evt,value) {

	evt = (evt) ? evt : event;
	var charCode = (evt.charCode) ? evt.charCode :
		((evt.keyCode) ? evt.keyCode :
			((evt.which) ? evt.which : 0)
		);

	var carSpec = evt.keyCode;


	if (carSpec == 46 || carSpec ==37 || carSpec ==39 )
		return true;

	if (charCode == 44)
		return true;

	if (charCode > 31 && (charCode < 48 || charCode > 57 )) {
		alert("In questo campo sono ammessi solo numeri");
		return false;
	}

	var numero = charCode-48;
	var realnumero = String(value) + String(numero);


	if ((realnumero>100) || (realnumero<1) && (realnumero!='') ){
			alert("In questo campo sono ammessi solo numeri\ncompresi tra 1 e 100");
			return false;
	}

	return true;
}

/* FUNZIONE PER IL TRACCIAMENTO GOOGLE ANALYTICS*/
function trackPage(page, pageCustomVars){
	if (typeof pageCustomVars != "object")
		pageCustomVars = null;

	if (page && typeof _gaq != "undefined"){
		/* TRACCIAMENTO GOOGLE */
		if (pageCustomVars)
			trackCustomVars(pageCustomVars);

		_gaq.push(['_trackPageview',page]);

		if (typeof __debugging_doLog != "undefined" && __debugging_doLog)
			_doLog("_gaq.push(['trackPageview','"+page+"']);");

		if (typeof ga != "undefined"){
			ga('send', 'pageview');
			if (typeof __debugging_doLog != "undefined" && __debugging_doLog)
				_doLog("ga('send','pageView','"+page+"');");
		}

		return true;
	}
	return false;
}

/******************************** CONTROLLO LEVENSHTEIN *******************************************/
/**
 * Attua il confronto di Levenshtein tra due stringhe ritornando la distanza
 * @param {String} str1 Prima stringa da confrontare
 * @param {String} str2 Seconda stringa da confrontare
 * @returns {Integer} Distanza di Levenshtein tra le 2 stringhe 
 **/
function getLevenshteinDistance(str1, str2){
	if (str1 == str2) return 0;
	var str1_len = (str1.length !== 0) ? str1.length : str2.length;
	var str2_len = (str2.length !== 0) ? str2.length : str1.length;
	if(str1.length === 0) return str1_len;
	if(str2.length === 0) return str2_len;

	var split = false;
	try{split=!("0")[0];}catch(i){split=true;};
	if (split) { str1 = str1.split(''); str2 = str2.split(''); }

	var v0 = new Array(str1_len + 1), v1 = new Array(str1_len + 1);
	var str1_idx = 0, str2_idx = 0, cost = 0;
	for (str1_idx = 0; str1_idx < str1_len + 1; str1_idx++) {
		v0[str1_idx] = str1_idx;
	}
	var char_str1 = '',
	char_str2 = '';
	for (str2_idx = 1; str2_idx <= str2_len; str2_idx++) {
		v1[0] = str2_idx;
		char_str2 = str2[str2_idx - 1];

		for (str1_idx = 0; str1_idx < str1_len; str1_idx++) {
			char_str1 = str1[str1_idx];
			cost = (char_str1 == char_str2) ? 0 : 1;
			var m_min = v0[str1_idx + 1] + 1;
			var b = v1[str1_idx] + 1;
			var c = v0[str1_idx] + cost;
			m_min = (b < m_min) ? b : m_min;
			m_min = (c < m_min) ? c : m_min;
			v1[str1_idx + 1] = m_min;
		}
		var v_tmp = v0;
		v0 = v1;
		v1 = v_tmp;
	}
	return v0[str1_len];
};

/********************************* FUNZIONI PER STRINGA ********************************************/

String.prototype.capitalize = function()
{
   return this.toLowerCase().replace(/^.|\s\S/g, function(a) { return a.toUpperCase(); });
};

/******** POLYFILL/FALLBACKS *****/
/**
 * Rewrite String.contains for browser that not support this method
 */
if(!('contains' in String.prototype))
	String.prototype.contains = function(str, startIndex) { return -1 !== String.prototype.indexOf.call(this, str, startIndex); };


/**
 * Date Format php like
 * @sintax var a = new Date(); a.format('m-d-Y h:i:s');
 * @see https://github.com/jacwright/date.format
 */
(function(){Date.shortMonths="Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec".split(" ");Date.longMonths="January February March April May June July August September October November December".split(" ");Date.shortDays="Sun Mon Tue Wed Thu Fri Sat".split(" ");Date.longDays="Sunday Monday Tuesday Wednesday Thursday Friday Saturday".split(" ");var e={d:function(){return(10>this.getDate()?"0":"")+this.getDate()},D:function(){return Date.shortDays[this.getDay()]},j:function(){return this.getDate()},
	l:function(){return Date.longDays[this.getDay()]},N:function(){return this.getDay()+1},S:function(){return 1==this.getDate()%10&&11!=this.getDate()?"st":2==this.getDate()%10&&12!=this.getDate()?"nd":3==this.getDate()%10&&13!=this.getDate()?"rd":"th"},w:function(){return this.getDay()},z:function(){var a=new Date(this.getFullYear(),0,1);return Math.ceil((this-a)/864E5)},W:function(){var a=new Date(this.getFullYear(),0,1);return Math.ceil(((this-a)/864E5+a.getDay()+1)/7)},F:function(){return Date.longMonths[this.getMonth()]},
	m:function(){return(9>this.getMonth()?"0":"")+(this.getMonth()+1)},M:function(){return Date.shortMonths[this.getMonth()]},n:function(){return this.getMonth()+1},t:function(){var a=new Date;return(new Date(a.getFullYear(),a.getMonth(),0)).getDate()},L:function(){var a=this.getFullYear();return 0==a%400||0!=a%100&&0==a%4},o:function(){var a=new Date(this.valueOf());a.setDate(a.getDate()-(this.getDay()+6)%7+3);return a.getFullYear()},Y:function(){return this.getFullYear()},y:function(){return(""+this.getFullYear()).substr(2)},
	a:function(){return 12>this.getHours()?"am":"pm"},A:function(){return 12>this.getHours()?"AM":"PM"},B:function(){return Math.floor(1E3*((this.getUTCHours()+1)%24+this.getUTCMinutes()/60+this.getUTCSeconds()/3600)/24)},g:function(){return this.getHours()%12||12},G:function(){return this.getHours()},h:function(){return(10>(this.getHours()%12||12)?"0":"")+(this.getHours()%12||12)},H:function(){return(10>this.getHours()?"0":"")+this.getHours()},i:function(){return(10>this.getMinutes()?"0":"")+this.getMinutes()},
	s:function(){return(10>this.getSeconds()?"0":"")+this.getSeconds()},u:function(){var a=this.getMilliseconds();return(10>a?"00":100>a?"0":"")+a},e:function(){return"Not Yet Supported"},I:function(){for(var a=null,b=0;12>b;++b){var c=(new Date(this.getFullYear(),b,1)).getTimezoneOffset();if(null===a)a=c;else if(c<a){a=c;break}else if(c>a)break}return this.getTimezoneOffset()==a|0},O:function(){return(0>-this.getTimezoneOffset()?"-":"+")+(10>Math.abs(this.getTimezoneOffset()/60)?"0":"")+Math.abs(this.getTimezoneOffset()/
		60)+"00"},P:function(){return(0>-this.getTimezoneOffset()?"-":"+")+(10>Math.abs(this.getTimezoneOffset()/60)?"0":"")+Math.abs(this.getTimezoneOffset()/60)+":00"},T:function(){return this.toTimeString().replace(/^.+ \(?([^\)]+)\)?$/,"$1")},Z:function(){return 60*-this.getTimezoneOffset()},c:function(){return this.format("Y-m-d\\TH:i:sP")},r:function(){return this.toString()},U:function(){return this.getTime()/1E3}};Date.prototype.format=function(a){var b=this;return a.replace(/(\\?)(.)/g,function(a,f,d){return""===f&&e[d]?e[d].call(b):d})}}).call(this);

/**
 * Restituisce primo indice dell'elemento dall'array in cui la stringa str è presente, -1 altrimenti
 * @param arr
 * @param str
 * @returns {number}
 */
function hasStringInside(arr, str) {
	for (var j=0; j<arr.length; j++) {
		if(typeof(arr[j]) != 'undefined' && typeof(arr[j]) == 'string')
			if (arr[j].match(str)) return j;
	}
	return -1;
}

/**
 * Restituisce indice dell'elemento dall'array in cui la stringa str è presente negli obj figli, -1 altrimenti
 * @param arr
 * @param str
 * @returns {number}
 */
function hasStringInsideChild(arr, str) {
	for (var j=0; j<arr.length; j++) {
		for (var key in arr[j]) {
			if(key == 'name' && arr[j][key] == str)
				return j;
		}
	}
	return -1;
}

/***
 * Funzione che sostituisce tutte le accentate con la veersione non accentata del carattere stesso
 *
 * str = "\xc0\xc1\xc2\xc3\xc4\xc5\xe0\xe1\xe2\xe3\xe4\xe5\xd2\xd3\xd4\xd5\xd6\xd8\xf2\xf3\xf4\xf5\xf6\xf8\xc8\xc9\xca\xcb\xe9\xe8\xea\xeb\xc7\xe7\xcc\xcd\xce\xcf\xec\xed\xee\xef\xd9\xda\xdb\xdc\xf9\xfa\xfb\xfc\xff\xd1\xf1";
 * removeDiacritics(str) = 'AAAAAAaaaaaaOOOOOOooooooEEEEeeeeCcIIIIiiiiUUUUuuuuyNn';
 *
****/
function removeDiacritics(str) {

	var defaultDiacriticsRemovalMap = [
    {'base':'A', 'letters':/[\u0041\u24B6\uFF21\u00C0\u00C1\u00C2\u1EA6\u1EA4\u1EAA\u1EA8\u00C3\u0100\u0102\u1EB0\u1EAE\u1EB4\u1EB2\u0226\u01E0\u00C4\u01DE\u1EA2\u00C5\u01FA\u01CD\u0200\u0202\u1EA0\u1EAC\u1EB6\u1E00\u0104\u023A\u2C6F]/g},
    {'base':'AA','letters':/[\uA732]/g},
    {'base':'AE','letters':/[\u00C6\u01FC\u01E2]/g},
    {'base':'AO','letters':/[\uA734]/g},
    {'base':'AU','letters':/[\uA736]/g},
    {'base':'AV','letters':/[\uA738\uA73A]/g},
    {'base':'AY','letters':/[\uA73C]/g},
    {'base':'B', 'letters':/[\u0042\u24B7\uFF22\u1E02\u1E04\u1E06\u0243\u0182\u0181]/g},
    {'base':'C', 'letters':/[\u0043\u24B8\uFF23\u0106\u0108\u010A\u010C\u00C7\u1E08\u0187\u023B\uA73E]/g},
    {'base':'D', 'letters':/[\u0044\u24B9\uFF24\u1E0A\u010E\u1E0C\u1E10\u1E12\u1E0E\u0110\u018B\u018A\u0189\uA779]/g},
    {'base':'DZ','letters':/[\u01F1\u01C4]/g},
    {'base':'Dz','letters':/[\u01F2\u01C5]/g},
    {'base':'E', 'letters':/[\u0045\u24BA\uFF25\u00C8\u00C9\u00CA\u1EC0\u1EBE\u1EC4\u1EC2\u1EBC\u0112\u1E14\u1E16\u0114\u0116\u00CB\u1EBA\u011A\u0204\u0206\u1EB8\u1EC6\u0228\u1E1C\u0118\u1E18\u1E1A\u0190\u018E]/g},
    {'base':'F', 'letters':/[\u0046\u24BB\uFF26\u1E1E\u0191\uA77B]/g},
    {'base':'G', 'letters':/[\u0047\u24BC\uFF27\u01F4\u011C\u1E20\u011E\u0120\u01E6\u0122\u01E4\u0193\uA7A0\uA77D\uA77E]/g},
    {'base':'H', 'letters':/[\u0048\u24BD\uFF28\u0124\u1E22\u1E26\u021E\u1E24\u1E28\u1E2A\u0126\u2C67\u2C75\uA78D]/g},
    {'base':'I', 'letters':/[\u0049\u24BE\uFF29\u00CC\u00CD\u00CE\u0128\u012A\u012C\u0130\u00CF\u1E2E\u1EC8\u01CF\u0208\u020A\u1ECA\u012E\u1E2C\u0197]/g},
    {'base':'J', 'letters':/[\u004A\u24BF\uFF2A\u0134\u0248]/g},
    {'base':'K', 'letters':/[\u004B\u24C0\uFF2B\u1E30\u01E8\u1E32\u0136\u1E34\u0198\u2C69\uA740\uA742\uA744\uA7A2]/g},
    {'base':'L', 'letters':/[\u004C\u24C1\uFF2C\u013F\u0139\u013D\u1E36\u1E38\u013B\u1E3C\u1E3A\u0141\u023D\u2C62\u2C60\uA748\uA746\uA780]/g},
    {'base':'LJ','letters':/[\u01C7]/g},
    {'base':'Lj','letters':/[\u01C8]/g},
    {'base':'M', 'letters':/[\u004D\u24C2\uFF2D\u1E3E\u1E40\u1E42\u2C6E\u019C]/g},
    {'base':'N', 'letters':/[\u004E\u24C3\uFF2E\u01F8\u0143\u00D1\u1E44\u0147\u1E46\u0145\u1E4A\u1E48\u0220\u019D\uA790\uA7A4]/g},
    {'base':'NJ','letters':/[\u01CA]/g},
    {'base':'Nj','letters':/[\u01CB]/g},
    {'base':'O', 'letters':/[\u004F\u24C4\uFF2F\u00D2\u00D3\u00D4\u1ED2\u1ED0\u1ED6\u1ED4\u00D5\u1E4C\u022C\u1E4E\u014C\u1E50\u1E52\u014E\u022E\u0230\u00D6\u022A\u1ECE\u0150\u01D1\u020C\u020E\u01A0\u1EDC\u1EDA\u1EE0\u1EDE\u1EE2\u1ECC\u1ED8\u01EA\u01EC\u00D8\u01FE\u0186\u019F\uA74A\uA74C]/g},
    {'base':'OI','letters':/[\u01A2]/g},
    {'base':'OO','letters':/[\uA74E]/g},
    {'base':'OU','letters':/[\u0222]/g},
    {'base':'P', 'letters':/[\u0050\u24C5\uFF30\u1E54\u1E56\u01A4\u2C63\uA750\uA752\uA754]/g},
    {'base':'Q', 'letters':/[\u0051\u24C6\uFF31\uA756\uA758\u024A]/g},
    {'base':'R', 'letters':/[\u0052\u24C7\uFF32\u0154\u1E58\u0158\u0210\u0212\u1E5A\u1E5C\u0156\u1E5E\u024C\u2C64\uA75A\uA7A6\uA782]/g},
    {'base':'S', 'letters':/[\u0053\u24C8\uFF33\u1E9E\u015A\u1E64\u015C\u1E60\u0160\u1E66\u1E62\u1E68\u0218\u015E\u2C7E\uA7A8\uA784]/g},
    {'base':'T', 'letters':/[\u0054\u24C9\uFF34\u1E6A\u0164\u1E6C\u021A\u0162\u1E70\u1E6E\u0166\u01AC\u01AE\u023E\uA786]/g},
    {'base':'TZ','letters':/[\uA728]/g},
    {'base':'U', 'letters':/[\u0055\u24CA\uFF35\u00D9\u00DA\u00DB\u0168\u1E78\u016A\u1E7A\u016C\u00DC\u01DB\u01D7\u01D5\u01D9\u1EE6\u016E\u0170\u01D3\u0214\u0216\u01AF\u1EEA\u1EE8\u1EEE\u1EEC\u1EF0\u1EE4\u1E72\u0172\u1E76\u1E74\u0244]/g},
    {'base':'V', 'letters':/[\u0056\u24CB\uFF36\u1E7C\u1E7E\u01B2\uA75E\u0245]/g},
    {'base':'VY','letters':/[\uA760]/g},
    {'base':'W', 'letters':/[\u0057\u24CC\uFF37\u1E80\u1E82\u0174\u1E86\u1E84\u1E88\u2C72]/g},
    {'base':'X', 'letters':/[\u0058\u24CD\uFF38\u1E8A\u1E8C]/g},
    {'base':'Y', 'letters':/[\u0059\u24CE\uFF39\u1EF2\u00DD\u0176\u1EF8\u0232\u1E8E\u0178\u1EF6\u1EF4\u01B3\u024E\u1EFE]/g},
    {'base':'Z', 'letters':/[\u005A\u24CF\uFF3A\u0179\u1E90\u017B\u017D\u1E92\u1E94\u01B5\u0224\u2C7F\u2C6B\uA762]/g},
    {'base':'a', 'letters':/[\u0061\u24D0\uFF41\u1E9A\u00E0\u00E1\u00E2\u1EA7\u1EA5\u1EAB\u1EA9\u00E3\u0101\u0103\u1EB1\u1EAF\u1EB5\u1EB3\u0227\u01E1\u00E4\u01DF\u1EA3\u00E5\u01FB\u01CE\u0201\u0203\u1EA1\u1EAD\u1EB7\u1E01\u0105\u2C65\u0250]/g},
    {'base':'aa','letters':/[\uA733]/g},
    {'base':'ae','letters':/[\u00E6\u01FD\u01E3]/g},
    {'base':'ao','letters':/[\uA735]/g},
    {'base':'au','letters':/[\uA737]/g},
    {'base':'av','letters':/[\uA739\uA73B]/g},
    {'base':'ay','letters':/[\uA73D]/g},
    {'base':'b', 'letters':/[\u0062\u24D1\uFF42\u1E03\u1E05\u1E07\u0180\u0183\u0253]/g},
    {'base':'c', 'letters':/[\u0063\u24D2\uFF43\u0107\u0109\u010B\u010D\u00E7\u1E09\u0188\u023C\uA73F\u2184]/g},
    {'base':'d', 'letters':/[\u0064\u24D3\uFF44\u1E0B\u010F\u1E0D\u1E11\u1E13\u1E0F\u0111\u018C\u0256\u0257\uA77A]/g},
    {'base':'dz','letters':/[\u01F3\u01C6]/g},
    {'base':'e', 'letters':/[\u0065\u24D4\uFF45\u00E8\u00E9\u00EA\u1EC1\u1EBF\u1EC5\u1EC3\u1EBD\u0113\u1E15\u1E17\u0115\u0117\u00EB\u1EBB\u011B\u0205\u0207\u1EB9\u1EC7\u0229\u1E1D\u0119\u1E19\u1E1B\u0247\u025B\u01DD]/g},
    {'base':'f', 'letters':/[\u0066\u24D5\uFF46\u1E1F\u0192\uA77C]/g},
    {'base':'g', 'letters':/[\u0067\u24D6\uFF47\u01F5\u011D\u1E21\u011F\u0121\u01E7\u0123\u01E5\u0260\uA7A1\u1D79\uA77F]/g},
    {'base':'h', 'letters':/[\u0068\u24D7\uFF48\u0125\u1E23\u1E27\u021F\u1E25\u1E29\u1E2B\u1E96\u0127\u2C68\u2C76\u0265]/g},
    {'base':'hv','letters':/[\u0195]/g},
    {'base':'i', 'letters':/[\u0069\u24D8\uFF49\u00EC\u00ED\u00EE\u0129\u012B\u012D\u00EF\u1E2F\u1EC9\u01D0\u0209\u020B\u1ECB\u012F\u1E2D\u0268\u0131]/g},
    {'base':'j', 'letters':/[\u006A\u24D9\uFF4A\u0135\u01F0\u0249]/g},
    {'base':'k', 'letters':/[\u006B\u24DA\uFF4B\u1E31\u01E9\u1E33\u0137\u1E35\u0199\u2C6A\uA741\uA743\uA745\uA7A3]/g},
    {'base':'l', 'letters':/[\u006C\u24DB\uFF4C\u0140\u013A\u013E\u1E37\u1E39\u013C\u1E3D\u1E3B\u017F\u0142\u019A\u026B\u2C61\uA749\uA781\uA747]/g},
    {'base':'lj','letters':/[\u01C9]/g},
    {'base':'m', 'letters':/[\u006D\u24DC\uFF4D\u1E3F\u1E41\u1E43\u0271\u026F]/g},
    {'base':'n', 'letters':/[\u006E\u24DD\uFF4E\u01F9\u0144\u00F1\u1E45\u0148\u1E47\u0146\u1E4B\u1E49\u019E\u0272\u0149\uA791\uA7A5]/g},
    {'base':'nj','letters':/[\u01CC]/g},
    {'base':'o', 'letters':/[\u006F\u24DE\uFF4F\u00F2\u00F3\u00F4\u1ED3\u1ED1\u1ED7\u1ED5\u00F5\u1E4D\u022D\u1E4F\u014D\u1E51\u1E53\u014F\u022F\u0231\u00F6\u022B\u1ECF\u0151\u01D2\u020D\u020F\u01A1\u1EDD\u1EDB\u1EE1\u1EDF\u1EE3\u1ECD\u1ED9\u01EB\u01ED\u00F8\u01FF\u0254\uA74B\uA74D\u0275]/g},
    {'base':'oi','letters':/[\u01A3]/g},
    {'base':'ou','letters':/[\u0223]/g},
    {'base':'oo','letters':/[\uA74F]/g},
    {'base':'p','letters':/[\u0070\u24DF\uFF50\u1E55\u1E57\u01A5\u1D7D\uA751\uA753\uA755]/g},
    {'base':'q','letters':/[\u0071\u24E0\uFF51\u024B\uA757\uA759]/g},
    {'base':'r','letters':/[\u0072\u24E1\uFF52\u0155\u1E59\u0159\u0211\u0213\u1E5B\u1E5D\u0157\u1E5F\u024D\u027D\uA75B\uA7A7\uA783]/g},
    {'base':'s','letters':/[\u0073\u24E2\uFF53\u00DF\u015B\u1E65\u015D\u1E61\u0161\u1E67\u1E63\u1E69\u0219\u015F\u023F\uA7A9\uA785\u1E9B]/g},
    {'base':'t','letters':/[\u0074\u24E3\uFF54\u1E6B\u1E97\u0165\u1E6D\u021B\u0163\u1E71\u1E6F\u0167\u01AD\u0288\u2C66\uA787]/g},
    {'base':'tz','letters':/[\uA729]/g},
    {'base':'u','letters':/[\u0075\u24E4\uFF55\u00F9\u00FA\u00FB\u0169\u1E79\u016B\u1E7B\u016D\u00FC\u01DC\u01D8\u01D6\u01DA\u1EE7\u016F\u0171\u01D4\u0215\u0217\u01B0\u1EEB\u1EE9\u1EEF\u1EED\u1EF1\u1EE5\u1E73\u0173\u1E77\u1E75\u0289]/g},
    {'base':'v','letters':/[\u0076\u24E5\uFF56\u1E7D\u1E7F\u028B\uA75F\u028C]/g},
    {'base':'vy','letters':/[\uA761]/g},
    {'base':'w','letters':/[\u0077\u24E6\uFF57\u1E81\u1E83\u0175\u1E87\u1E85\u1E98\u1E89\u2C73]/g},
    {'base':'x','letters':/[\u0078\u24E7\uFF58\u1E8B\u1E8D]/g},
    {'base':'y','letters':/[\u0079\u24E8\uFF59\u1EF3\u00FD\u0177\u1EF9\u0233\u1E8F\u00FF\u1EF7\u1E99\u1EF5\u01B4\u024F\u1EFF]/g},
    {'base':'z','letters':/[\u007A\u24E9\uFF5A\u017A\u1E91\u017C\u017E\u1E93\u1E95\u01B6\u0225\u0240\u2C6C\uA763]/g}
];

    for(var i=0; i<defaultDiacriticsRemovalMap.length; i++) {
        str = str.replace(defaultDiacriticsRemovalMap[i].letters, defaultDiacriticsRemovalMap[i].base);
    }

    return str;
}

/**************************** FINE FUNZIONI PER STRINGA *************************************/

/*
 * Visualizza un pulsante per la cancellazione del campo input
 * @param string inputId identificativo del campo input
 */
InputCleaner = function(inputId){

   var oInputCleaner = this;

   this.input = jq('#' + inputId);
   this.button = jq('<div class="divInputClear"></div>');

   // Inserisco la div
   this.button.insertAfter(this.input);

   // Nascondo il bottone
   this.hideButton();

   // Timer
   this.timer = null;

   // Quando il mouse si trova sul campo input visualizzo il bottone
   this.input.hover(
       function (event) { // in
			if (oInputCleaner.timer){
				clearTimeout(oInputCleaner.timer);
			}
			oInputCleaner.showButton();
       },
       function (event) { // out
           oInputCleaner.timer = setTimeout(function(){
               oInputCleaner.hideButton();
           }, 2000);
       }
   );

   // Quando viene cliccato il bottone pulisco il
   this.button.click(function(event){
       oInputCleaner.cleanInput();
   });
};

/**
 * Visualizza / nasconde il bottone per la cancellazione
 */
InputCleaner.prototype.showButton = function(){
   if (this.input.val() == ''){
       this.button.hide();
   } else {
       this.button.show();
   }
};

/**
 * Nasconde il bottone per la cancellazione
 */
InputCleaner.prototype.hideButton = function(){
   this.button.hide();
};

/**
 * Pulisce il campo input
 */
InputCleaner.prototype.cleanInput = function(){
   this.input.val('');
};

/******************************************************************************/

var Page = {

	doctype : function() {
		if (typeof document.doctype == "undefined" || !document.doctype) {
			return '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">';
		}

		var node = document.doctype;
		var doctype = "<!DOCTYPE "
			+ node.name
			+ (node.publicId ? ' PUBLIC "' + node.publicId + '"' : '')
			+ (!node.publicId && node.systemId ? ' SYSTEM' : '') 
			+ (node.systemId ? ' "' + node.systemId + '"' : '')
			+ '>';
		return doctype;
	},

	snapshot : function() {
		var doctype = Page.doctype();
		var elements;
		var body;
		var head;

		if (!jq.support.htmlSerialize) {
			var div = document.createElement ("div");
			div.innerHTML = jq("body")[0].innerHTML;
			body = jq(div);
			elements = body;
		} else {
			jq("body *").each (function () {
				var n = jq(this);
				var tag = this.tagName.toLowerCase();
				if (tag == "input") {
					n.attr("checked", n.prop("checked"));
				} else if (tag == "option") {
					n.attr("selected", n.prop("selected"));
				}
			});
			body = jq("body").clone();
			head = jq("head").clone();
			elements = jq([head[0], body[0]]);
		}

		elements.find("*").each (function () {
			var n = jq(this);
			var tag = this.tagName.toLowerCase();
			if (tag == "script") {
				n.remove();
				return;
			} else if (tag == "a" || tag == "link") {
				var href = n.attr("href");
				if (href && !href.match(/^mailto:/) && !href.match(/^#/)) {
					n.attr("href", Url.absolute (href));
				}
			} else if (tag == "img") {
				var src = n.attr("src");
				if (src && !src.match(/^data:/)) {
					n.attr("src", Url.absolute (src));
				}
			}
			var bg = n.css("background-image");
			if (bg && bg.match(/^url\(/)) {
				var url;
				if (bg[4] == "'" || bg[4] == '"') {
					url = Url.absolute (bg.substr(5,bg.length-7));
				} else {
					url = Url.absolute (bg.substr(4,bg.length-5));
				}
				n.css("background-image", 'url("'+url+'")');
			}
			var opacity = n.css("opacity");
			if(0 == opacity){
				n.css("opacity",100);
			}
		});

		var headStr;
		if (!jq.support.htmlSerialize) {
			headStr = jq("head").html().replace(/<script[\s\S]*?<\/script>/gmi, '');
			headStr = headStr.replace(/href="(.+?)"/gmi, function(match, url) {
				return 'href="'+Url.absolute(url)+'"';
			});
		} else {
			headStr = head.html();
		}

		var result = doctype+"<html><head>"+headStr+"</head><body>"+body.html()+"</body></html>";
		return result;
	},

	scroll : function(scroll) {
		if (typeof scroll != "undefined") {
			window.scrollTo(scroll.x, scroll.y);
		} else {
			scrolly = (document.all) ? document.documentElement.scrollTop : window.pageYOffset;
			scrollx = (document.all) ? document.documentElement.scrollLeft : window.pageXOffset;
			return {x: scrollx, y: scrolly};
		}
	},

	/**
	 * Scroller progressivo verso elemento passato con Id
	 * @param Int/String id identificativo elemento
	 * @param Int/String marginFromTop Margine dall'offset superiore della view
	 * @returns Void
	 */
	scrollTo : function(id, marginFromTop) {	
		setTimeout(function(){
			jq('html,body').animate({scrollTop: jq(id).offset().top - marginFromTop}, 'slow');
		}, 400);
	}
};

function getBoxPopUp(metodo,box,iframe)
{
	action = "/services/checkSession.php";
	var ajaxObj = getMEAjaxObj();
	ajaxObj.Request("POST",
					action,
					function(data)
					{
						var text = data.responseText;
						var json = eval('('+text+')');
						if( json.session == true )
						{
							eval(metodo);
						}
						else
						{
							getBox = new boxOverlay('richiestaBox',"red");
							getBox.setTitle("<strong class=\"titolo_annuncio\"></strong>");
							
							if(iframe == 1)
							{
								getBox.setContent('<iframe src="/services/getPopUpBox.php?type='+box+'" width="620" height="310" frameBorder="0" marginheight="0" marginwidth="0"></iframe>');
							}
							else
							{
								action = '/services/getPopUpBox.php?type='+box;
								var ajaxObj = getMEAjaxObj();
								ajaxObj.Request("POST", action, function(data){
																getBox.setContent(data.responseText);
																}
												);
							}
							
							getBox.show();
						}
					}
	);
};

function deselectAllText() {
	setTimeout(function() { try{document.getSelection().removeAllRanges();}catch(e){} }, 0);
}

if (getValue('debug'))
	__debugging_doLog = true;


/**
 * Formatta un numero.
 * @param decimals: numero di decimali da avere
 * @param decimal_sep: carattere usato come separatore dei decimali, se omesso di defaults è ','
 * @param thousands_sep: carattere usato come separatore delle migliaia, se omesso di defaults è '.'
 * @returns {string}
 */
Number.prototype.formatNumber = function(decimals, decimal_sep, thousands_sep)
{
	var n = this,
		c = isNaN(decimals) ? 2 : Math.abs(decimals), //if decimal is zero we must take it, it means user does not want to show any decimal
		d = decimal_sep || ',',
		t = (typeof thousands_sep === 'undefined') ? '.' : thousands_sep,
		sign = (n < 0) ? '-' : '',
		i = parseInt(n = Math.abs(n).toFixed(c)) + '',
		j = ((j = i.length) > 3) ? j % 3 : 0;
	return sign + (j ? i.substr(0, j) + t : '') + i.substr(j).replace(/(\d{3})(?=\d)/g, "$1" + t) + (c ? d + Math.abs(n - i).toFixed(c).slice(2) : '');
}

function changeReviewTab(thisTab){
	if(thisTab == "videoTab"){
		var otherTab = "videoExtTab";
		var otherTabContent = "videoExtContent";
		var thisTabContent = "videoContent";
	}else{
		var otherTab = "videoTab";
		var otherTabContent = "videoContent";
		var thisTabContent = "videoExtContent";
	}
	document.getElementById(thisTab).style.fontWeight = "bold";
	document.getElementById(otherTab).style.fontWeight = "normal";
	document.getElementById(thisTabContent).style.display = "block";
	document.getElementById(otherTabContent).style.display = "none";
}

/***************** KEYCODE OBJECT WRAPPER *******************/
var Keycodes = (function () {
	var instance;
	var object = new Object();

	function createInstance() {
		object = { "BACKSPACE" : 8, "TAB" : 9, "ENTER" : 13, "SHIFT" : 16, "CTRL" : 17,"ALT" : 18, "PAUSE" : 19,
				   "CAPSLOCK" : 20,"ESC" : 27,"SPACE":32,"PAGEUP" : 33,"PAGEDOWN" : 34,"END" : 35,"HOME" : 36,
				   "LEFT" : 37, "UP" : 38, "RIGHT" : 39, "DOWN" : 40, "INSERT" : 45, "DEL":46};

		return object;
	}

	function findByType( type ){
		if(typeof(type) == 'undefined')
			return null;

		var obj = Keycodes.get();
		var upper = type.toUpperCase();

		return (typeof(obj[upper]) != 'undefined') ? obj[upper] : null;
	}

	return {
		get: function ( name ) {
			if(typeof(name) == 'undefined'){
				if (!instance) {
					instance = createInstance();
				}
				return instance;
			}else{
				return findByType( name );
			}
		}
	};
})();
/*************************************************************/

/***************** GESTIONE ANNUNCIO UTILS *******************/
var _gestioneAnnuncioUtils = function(options){
	this.options = options;
	this.enabledLog = jq.isNotUndefined(this.options.enabledLog) ? this.options.enabledLog : 0; //da passare nelle options alla utils
};
/**
 * Log wrapper per il console.log() funzionante sia su Chrome,FF,Opera,IE. Accetta argomenti per la formattazione dell'output di console come
 * da manuali delle varie versioni(FF e Chrome)
 * @private
 */
_gestioneAnnuncioUtils.prototype._log = function(){
	var aArgs = new Array();
	var args = aArgs.slice.call(arguments, 0);

	if(this.enabledLog){
		if(typeof console != 'undefined' && !jq.isEmpty(console)){
			try {
				if(args.length < 2 || (args.length == 2 && (jq.inArray(args[1], ['error','log','dir']) != -1))){
					((typeof args[1] == 'undefined') || (args[1] == 'log')) ? console.log(args[0]) :
						( (args[1] == 'dir') ? console.dir(args[0]) : console.error(args[0]) )
				}else{
					if(typeof(console.log) != 'undefined')
						console.log.apply(console, args);
				}
			} catch( e ) {
				jq.each( args, function() {
					if(typeof _doLog != 'undefined' && typeof(_doLog) === "function")
						_doLog(this);
				});
			}
		}else{
			if(args.length <= 2){
				(typeof args[1] != 'undefined' && args[1] == 'error') ? _doLog( args[0], args[1] ) : _doLog( args[0] );
			}else
				_doLog( args );
		}
	}
};
/**
 * Mostra un loader in una popup in base ai parametri passati
 * @param {box} box Popup nel quale settare il contenuto
 * @param {params} Parametri per la gestione del loading: msg, style, imgloader
 */
_gestioneAnnuncioUtils.prototype.showLoader = function(box, params){
	params = params || {};
	var _style = typeof params._style != 'undefined' ? params._style : "height: 80px;text-align: center;padding: 62px 0 61px 0;width:524px;" ;
	var _msg = typeof params.msg != 'undefined' ? params.msg : "Caricamento in corso..." ;
	var _loader = typeof params.imgloader != 'undefined' ? params.imgloader : "/img2/loader3.gif" ;

	box.setContent("<div style='" + _style + "'><h3>" + _msg + "</h3><br /><span style=\"\"><img src='" + _loader + "'></span></div>");

	return;
};
/***************** GESTIONE ANNUNCIO UTILS *******************/

/**
 * Costruisce una stringa con la data e l'ora corrente.
 * @param dateSep
 * @param dateTimeSep
 * @param timeSep
 * @returns {*}
 */

if (typeof Date != 'undefined' && (typeof Date.now == 'undefined' || !Date.now)) {
	Date.now = function() { return new Date().getTime(); }
}

function getDateTime(dateSep, dateTimeSep, timeSep)
{
	dateSep = typeof dateSep != 'undefined' ? dateSep : '-';
	dateTimeSep = typeof dateTimeSep != 'undefined' ? dateTimeSep : ' ';
	timeSep = typeof timeSep != 'undefined' ? timeSep : ':';

	var now = new Date();
	var year = now.getFullYear();
	var month = now.getMonth() + 1;
	var day = now.getDate();
	var hour = now.getHours();
	var minute = now.getMinutes();
	var second = now.getSeconds();
	if (month.toString().length == 1) {
		month = '0' + month;
	}
	if (day.toString().length == 1) {
		day = '0' + day;
	}
	if (hour.toString().length == 1) {
		hour = '0' + hour;
	}
	if (minute.toString().length == 1) {
		minute = '0' + minute;
	}
	if (second.toString().length == 1) {
		second = '0' + second;
	}
	return year + dateSep + month + dateSep + day + dateTimeSep + hour + timeSep + minute + timeSep + second;
}


/**
 *
 * Funziona che esegue i tag script contenuti in una string testo
 * Utile per esecuzione di script contentui in ripsoste html ajax
 *
 */
function scriptEval(domObjOrHtmlStr) {

    var oScript,
        div,
        scripts,
        newScript = "";

    if (typeof domObjOrHtmlStr === 'string') {
        div = document.createElement('div');
        div.innerHTML = domObjOrHtmlStr;
    } else if (typeof domObjOrHtmlStr === 'object') {
        div = domObjOrHtmlStr;
    }else{
        throw('"required parameter is missing or wrong type passed (string or DOM object needed)"');
    }

    scripts = div.getElementsByTagName('script');

    for (var i = 0; i < scripts.length; i++) {
        if (typeof scripts[i].src != 'undefined' && scripts[i].src) {
            oScript = document.createElement('script');
            oScript.src = scripts[i].src;
            document.getElementsByTagName("head")[0].appendChild(oScript);
        } else {
            newScript += ";" + scripts[i].innerHTML;
        }
    }
    if (newScript) {
        oScript = document.createElement('script');
        oScript.text = newScript;
        document.getElementsByTagName("head")[0].appendChild(oScript);
    }
};
var __SSOaccounts = new Array();
__SSOaccounts.push(new Array("facebook","oauth"));
__SSOaccounts.push(new Array("google","openid"));
//__SSOaccounts.push(new Array("google","oauth"));

var zone_selected=new Array();
var zone_selected_TI=new Array();
var localita_selected=new Array();

function _fnShSbTim() {
	setTimeout('show_search_box()',5000);
}

function force_search_box_show(){
	//assicura la visualizzazione del form anche in presenza di errori javascript nella pagina
	addEvent(window,'load',_fnShSbTim);
}

function getScrollPosition(name){
	scrolly = (document.all)?document.documentElement.scrollTop:window.pageYOffset;
	scrollx = (document.all)?document.documentElement.scrollLeft:window.pageXOffset;
	createCookie(name+'_yPos',scrolly,0,0,5);
	createCookie(name+'_xPos',scrollx,0,0,5);
}

function resetCookies(){
 createCookie('ricerca_immobili_yPos',0,0,0,5);
 createCookie('ricerca_agenzie_yPos',0,0,0,5);
 createCookie('nuove_costruzioni_yPos',0,0,0,5);
 
}

function setScrollPosition(name){
    var yPos = readCookie(name+'_yPos');
    var xPos = readCookie(name+'_xPos');
   
    if (yPos || xPos){
	window.scrollTo(xPos,yPos);
	setTimeout("resetCookies()",1000);
    }
	
}

function setHomeSection(cat){
	createCookie('home_section',cat);
}

function cambia_categoria_nc(obj){
	var id=obj.id;
	var menus = document.getElementsByClassName("nc_tab");
	for (var i=0; i<menus.length; i++){
		menus[i].setAttribute('class',"nc_tab"); 
	}
	var menu = document.getElementById(obj.id);
	menu.setAttribute('class','nc_tab selected');
	
	
	
	var subid;
	subid = "sub_"+obj.id;
	var submenus = document.getElementsByClassName("sub_idCategoria");
	for (var i=0; i<submenus.length; i++){
		submenus[i].style.visibility='hidden'; 
	}
	var submenu = document.getElementById(subid);
	submenu.style.visibility='visible';
	
}

function resettaEstero(valore){	
	
		if ((valore == 0)&&(window.document.dati.idNazione.value!='')) {

			window.document.getElementById("divEsteroCittaDirette").style.display = 'none';
			window.document.getElementById("linkEsteroDirettiCitta").style.display = 'none';
	
	
		}else{
			window.document.getElementById("divEsteroCittaDirette").style.display = 'none';
			if (!window.document.dati.idNazione.value){
			/*	window.document.getElementById("linkEsteroDirettiCitta").style.display = 'block';
				window.document.getElementById("showLinkEsteroDirettiCitta").style.display = 'block';
*/
			}
			
		}	
	
}

function validateFormHomeRes(){
	var ret_value = false;
	
	if (document.dati.idProvincia && document.dati.idProvincia.value!=''){
		document.dati.azione.value='avviaricerca';
		ret_value = true;
	}
	
	if (!ret_value)
		alert("I campi regione e provincia sono obbligatori");
	
	if (ret_value) {
		if (jq("#tipoRicerca").val() == 'mappa') {
			uncheckZone(document.dati.idProvincia.value);
		} else {
			jq("#gm_area").val('');
			jq("#gm_area").attr("disabled", true);
		}
	}
	return ret_value;
}

function roomManager(fkCategoria,catStanze){
	
	document.getElementById('idCategoria').value = catStanze;
	xajax_getAjaxTipologiaPagIndex(catStanze);
	if( document.getElementById("radio_idContratto1") )
		document.getElementById("radio_idContratto1").onclick= function(){resetForm(fkCategoria);};
	if( document.getElementById("radio_idContratto2") )
		document.getElementById("radio_idContratto2").onclick= function(){resetForm(fkCategoria);};
}

function resetForm(fkCategoria){
	
	if ( document.getElementById("radio_idContratto1") )
		document.getElementById("radio_idContratto1").onclick = null;
	if ( document.getElementById("radio_idContratto2") )
		document.getElementById("radio_idContratto2").onclick = null;
	if ( document.getElementById("idCategoria") )
		document.getElementById('idCategoria').value = fkCategoria;
	xajax_getAjaxTipologiaPagIndex(fkCategoria);
}

function checkTerreno(){
	var _oSel = document.getElementById("idTipologia");
	var sValue = _oSel.value;
	if (sValue == 28){
		document.dati.idCategoria.value = 2;
		document.dati.sottotipologia.value = 67;
	}else{
		document.dati.idCategoria.value = 1;
		document.dati.sottotipologia.value = "";
	}
}

function getProvLink(id){
	var provId = id.replace("_citta","");
	provId = provId.replace(/_\d$/,"");
	if (provId){
		var provTxt = document.getElementById(provId).alt;
		var provLink = createPathLink("Provincia di "+provTxt,null,{id:"path_provincia"});
		return provLink;
	}
}
__dontChangeSearchPath = false;
function enableTooltip(id_zona,clearPath){
	if (isMobile.any())
		return true;
	if (__dontChangeSearchPath == true){
		__dontChangeSearchPath = false;
		return;
	}
	if (document.getElementById('tooltip_map')){
		var oArea=document.getElementById(id_zona);
		titolo=oArea.getAttribute("alt");
		var pathDiv = document.getElementById("tooltip_map");
		tit = titolo;
		var zone = createPathLink(tit,function(){return false;});
		pathDiv.appendChild(zone);
		return;
	}
		
	var pathDiv = document.getElementById("searchPath");
	if (typeof clearPath != "undefined" && clearPath == true){
		if (pathDiv){
			pathDiv.innerHTML = "";
			
		}
	}
	


	var oArea=document.getElementById(id_zona);
	titolo=oArea.getAttribute("alt");
	var _lastMouseOver = createPathLink(titolo,null,{id:"path_regione"});	
	if(window.document.dati.idRegione && window.document.dati.idRegione.value && !(window.document.dati.idComune && window.document.dati.idComune.value)){
		if (document.getElementById('path_provincia')){
			pathDiv.removeChild(document.getElementById('path_provincia'));
		}
		if (document.getElementById('path_comune')){
			pathDiv.removeChild(document.getElementById('path_comune'));
		}	
		if (typeof sitemap !="undefined" && sitemap=="on"){
		    _lastMouseOver = createPathLink(Provincia+" - "+titolo,null,{id:"path_provincia"});
		}else{
		    if (oArea.id.search("citta") > 0){
				var provLink = getProvLink(oArea.id);
				pathDiv.appendChild(provLink);
				_lastMouseOver = createPathLink(titolo,null,{id:"path_comune"});
				
		    }
			else{
				_lastMouseOver = createPathLink("Provincia di "+titolo,null,{id:"path_provincia"});
			}
		}
	}else{
		if (window.document.dati.idComune && window.document.dati.idComune.value){
		if (isNaN(oArea.id)){
			if (document.getElementById('path_provincia')){
			pathDiv.removeChild(document.getElementById('path_provincia'));
		    }
		    if (document.getElementById('path_comune')){
			pathDiv.removeChild(document.getElementById('path_comune'));
		    }
		    if (oArea.id.search("citta") > 0){
			var provLink = getProvLink(oArea.id);
			pathDiv.appendChild(provLink);
			_lastMouseOver = createPathLink(titolo,null,{id:"path_comune"});
		    }else{
			if (typeof sitemap !="undefined" && sitemap=="on"){
				_lastMouseOver = createPathLink(Provincia+" - "+titolo,null,{id:"path_provincia"});
			}else{
				_lastMouseOver = createPathLink("Provincia di "+titolo,null,{id:"path_provincia"});
			}
		    }
		}else{
		    tit = titolo;
		    _lastMouseOver = createPathLink(tit,null);
		}
	    }
	}
	pathDiv.appendChild(_lastMouseOver);
}
	
function disableTooltip(e){
	if (isMobile.any())
		return true;
	__dontChangeSearchPath = false;
	if (document.getElementById('tooltip_map')){
		var pathDiv = document.getElementById("tooltip_map");
		pathDiv.innerHTML = "";
		var nomeComune = "";
		if (typeof __isInserimentoAnnuncio!= "undefined" && __isInserimentoAnnuncio) {
			nomeComune =  window.document.dati._textComune.value;

		} else if ( ( typeof _isAlertEmail != "undefined" && _isAlertEmail ) || ( typeof _isSearchModify != "undefined" && _isSearchModify ) ) {
			if (typeof TI_agComuneField != "undefined") {
				nomeComune = TI_agComuneField.getNome();
			} else {
				var idComune = document.getElementById('TI_comune');
				nomeComune = idComune.options[idComune.selectedIndex].innerHTML;
			}
		} else {                                                            			                        
            // ottengo il nome del comune dalla suggestion o dalla select
            if (typeof __jsSection != "undefined" && __jsSection == "trovacasaStatic" && window.document.getElementById("suggestComune").value != null){
                var nomeComune = window.document.getElementById("suggestComune").value;                
            } else if (window.document.dati.idComune.options) {                
				nomeComune = window.document.dati.idComune.options[window.document.dati.idComune.selectedIndex].innerHTML;
            } else if (typeof searchForm != "undefined" && searchForm) {
                nomeComune = searchForm.getNomeComune();
			}
		}
		var comune = createPathLink(nomeComune,function(){return false});
		pathDiv.appendChild(comune);
		return;
	}
	
	if (document.getElementById("searchPath"))
		writePath();
	
}

function writePath(){
	try{	
	var pathDiv = document.getElementById("searchPath");
	pathDiv.innerHTML = "";
	for (var i=0;i<path.length;i++){
	    var oa = document.createElement("a");
		oa.innerHTML = path[i].innerHTML;
		if (path[i].onclick){
			oa.onclick = path[i].onclick;
		}
		oa.id = path[i].id;
		oa.className = path[i].className;
		pathDiv.appendChild(oa);
	}
	}catch(e){
		
	}
}

function createPathLink(label,action,pars){
	if (typeof action=="undefined")
		action=null;
	var a = document.createElement("a");
	a.innerHTML = label;
	if (action)
		a.onclick = action;
	
	if (typeof pars=="object"){
		if (pars.className)
			a.className = pars.className;
		if (pars.id)
			a.id = pars.id;
	}
	
	return a;
}

function enableBack(ricerca){
	ricerca = null;
	path = new Array();
	
	if (typeof nazione == "undefined"){
	    if (typeof sitemap !="undefined" && sitemap=="on"){
		nazione = multiItalia;
	    }else{
		nazione = "Italia";
	    }
	}

	gestioneCitta();

	//path.push(createPathLink(Italia,function(){xajax_getAjaxProvincePagIndex('','IT');},{className:"first"}));
	path.push(createPathLink(nazione,function(){xajax_getAjaxProvincePagIndex('','IT');},{className:""}));
	var oDiv = document.getElementById("div_back");

	var isVacanze = typeof searchForm != "undefined" && searchForm && searchForm.isVacanze(),
		isIssetRegione = typeof window.document.dati !== 'undefined' && window.document.dati.idRegione && window.document.dati.idRegione.value!="",
		isIssetProvincia = typeof window.document.dati !== 'undefined' && window.document.dati.idProvincia && window.document.dati.idProvincia.value!="",
		isIssetComune = typeof window.document.dati !== 'undefined' && window.document.dati.idComune && window.document.dati.idComune.value!="";

	if (isVacanze) {
		path = searchForm.getBackPath();
	} else if (isIssetRegione){
		path.push(createPathLink(window.document.dati.idRegione.options[window.document.dati.idRegione.selectedIndex].innerHTML,function(){xajax_getAjaxProvincePagIndex(window.document.dati.idRegione.value,"IT");document.getElementById('linkcitta').style.display = '';},{id:"path_regione"}));
		
		if(isIssetProvincia){
		        if (typeof sitemap !="undefined" && sitemap=="on"){
			    path.push(createPathLink(Provincia+" - "+window.document.dati.idProvincia.options[window.document.dati.idProvincia.selectedIndex].innerHTML,function(){return xajax_getAjaxComuniPagIndex(window.document.dati.idProvincia.value, window.document.dati.idRegione.value);},{id:"path_provincia"}));
		        }else{
			    path.push(createPathLink("Provincia di "+window.document.dati.idProvincia.options[window.document.dati.idProvincia.selectedIndex].innerHTML,function(){return xajax_getAjaxComuniPagIndex(window.document.dati.idProvincia.value, window.document.dati.idRegione.value);},{id:"path_provincia"}));
		        }

			if (isIssetComune){
				path.push(createPathLink(window.document.dati.idComune.options[window.document.dati.idComune.selectedIndex].innerHTML,function(){return false;},{id:"path_comune"}));
				
			}
		}
	}

	writePath();
}

function gestioneCitta() {
	if (typeof searchForm != "undefined" && searchForm.isVacanze()) {
		return;
	}
	if( document.getElementById("divCittaDirette") != null) {
		var oDivCitta = document.getElementById("divCittaDirette");
		var oLinkCitta = document.getElementById("linkDirettiCitta");
		var oShowLinkCitta = document.getElementById("showLinkDirettiCitta");
		var olinksContainer = document.getElementById("linkcitta");

		if (window.document.dati.idRegione && window.document.dati.idRegione.value){
			if(window.document.dati.idProvincia && window.document.dati.idProvincia.value){
			    if (oLinkCitta) oLinkCitta.style.display = "none";
			    if (oDivCitta) oDivCitta.style.display = "none";
			    if (oShowLinkCitta) oShowLinkCitta.style.display="none";
			    if (olinksContainer) olinksContainer.style.display="none";
			}
		}else {
		    if (oShowLinkCitta) oShowLinkCitta.style.display = "none";
		    if (oLinkCitta) oLinkCitta.style.display = "block";
		    if (oDivCitta) oDivCitta.style.display = "block";
		    if (olinksContainer) olinksContainer.style.display="block";
		}
	}
}

function enableBack_NC(ricerca){
	ricerca = null;
	path = new Array();
	
	if (typeof nazione == "undefined"){
		nazione = "Estero";
	}
		
	//path.push(createPathLink(Italia,function(){xajax_getAjaxProvincePagIndex('','IT');},{className:"first"}));
	path.push(createPathLink(nazione,function(){},{className:""}));
	writePath();
}

function hideShowLinkCitta(){
	try{
	if(window.document.dati.idRegione.value){
		var newDiv = document.getElementById("showLinkDirettiCitta");
		var oDiv = document.getElementById("linkDirettiCitta");
		if (!oDiv || !newDiv)
			return;
		var childDiv = oDiv.childNodes;
		var numShowCitta = 0;
		svuota_cointainer(newDiv);
		for (var i=0; i< childDiv.length ; i++){
			if(childDiv[i].id && childDiv[i].id.indexOf("link_" + window.document.dati.idRegione.value) > -1 ){
				if(numShowCitta){
					var oSpan = document.createElement("span");
					oSpan.innerHTML +=", ";
					newDiv.appendChild(oSpan);
				}		
				var oHref = document.createElement("a");
				oHref.setAttribute("href","#");
				oHref.className="linkHome";
				oHref.setAttribute("id","show_"+ childDiv[i].id);
				oHref.innerHTML += childDiv[i].innerHTML;
				oHref.setAttribute("onclick",childDiv[i].getAttribute("onclick"));
				newDiv.appendChild(oHref);
				numShowCitta++;
			}
		}
		document.getElementById("linkDirettiCitta").style.display = "none";
		if(!numShowCitta){
			document.getElementById("divCittaDirette").style.display = "none";
			document.getElementById("showLinkDirettiCitta").style.display = "none";
			document.getElementById("linkcitta").style.display = "none";
		}else{
			if (!(window.document.dati.idComune && window.document.dati.idComune.value) && document.getElementById('ricercaAvanzata') && document.getElementById('ricercaAvanzata').style.display=='none'){
				document.getElementById("linkcitta").style.display = "block";
				document.getElementById("divCittaDirette").style.display = "block";
				document.getElementById("showLinkDirettiCitta").style.display = "block";
			}else{
				document.getElementById("linkcitta").style.display = "none";
			}
		}
	}
	}catch(e){
		_doLog(e,"error");
	}
}


function hideShowLinkEsteroCitta(){

        if (window.document.dati.idNazione.value || window.document.dati.idProvincia ) {
		
		var newDiv = document.getElementById("showLinkEsteroDirettiCitta");
		var oDiv = document.getElementById("linkEsteroDirettiCitta");
		
		var childDiv = oDiv.childNodes;
		var numShowCitta = 0;
		svuota_cointainer(newDiv);

		for (var i=0; i< childDiv.length ; i++){
			if(childDiv[i].id && childDiv[i].id.indexOf("link_" + window.document.dati.idNazione.value) > -1 ){
				if(numShowCitta){
					var oSpan = document.createElement("span");
					oSpan.innerHTML +=", ";
					newDiv.appendChild(oSpan);
				}		
				var oHref = document.createElement("a");
				oHref.setAttribute("href","#");
				oHref.setAttribute("id","show_"+ childDiv[i].id);
				oHref.innerHTML += childDiv[i].innerHTML;
				
				oHref.setAttribute("onclick",childDiv[i].getAttribute("onclick"));
				newDiv.appendChild(oHref);
				numShowCitta++;
			}
		}

		//document.getElementById("linkEsteroDirettiCitta").style.display = "none";

		if(!numShowCitta || (window.document.dati.idRegione && window.document.dati.idRegione.value) || document.getElementById("idCategoria3").className == 'selected' || document.getElementById("idCategoria4").className == 'selected' ){
			document.getElementById("divEsteroCittaDirette").style.display = "none";
			document.getElementById("showLinkEsteroDirettiCitta").style.display = "none";
		}
	}
	else
	{
		document.getElementById("divEsteroCittaDirette").style.display = "block";
		document.getElementById("linkEsteroDirettiCitta").style.display = "block";

	}
}


function manage_new_search_button(action){
	if (!document.getElementById("new_search") || __noSearchRestored)
		return;
	var btn = document.getElementById("new_search");
	switch (action){
		case "show" :
			btn.style.display = "block";
			break;
		default :btn.style.display = "none";
	}
	
}


function changeImageNaz(nazione) {
	manage_new_search_button("hide");
	var ShowItem = document.getElementById("div_nazioni");
	ShowItem.style.backgroundImage = 'url(/img2/cartine/' + nazione + '_selected.gif' + getVersion() + ')';
	return true;
}

function hideImageNaz(nazione) {
	manage_new_search_button("show");
	var ShowItem = document.getElementById("div_nazioni");
	ShowItem.style.backgroundImage = 'url(/img2/cartine/transparent.gif)';
	return true;
}
		
function changeImage(region) {
	manage_new_search_button("hide");
	var ShowItem = document.getElementById("div_regioni");
	ShowItem.style.backgroundImage = 'url(/img2/cartine/' + region + '_selected.gif' + getVersion() + ')';
	return true;
}

function hideImage(region) {
	manage_new_search_button("show");
	var ShowItem = document.getElementById("div_regioni");
	if (ShowItem)
	    ShowItem.style.backgroundImage = 'url(/img2/cartine/transparent.gif)';
	return true;
}

function changeImageProv(provincia) {
	manage_new_search_button("hide");
	var oDiv = document.getElementById("div_provincie");
	if (oDiv)
	    oDiv.style.backgroundImage = 'url(/img2/cartine/' + window.document.dati.idRegione.value + '/select_' + provincia + '.gif' + getVersion() + ')';		
}

function hideImageProv(provincia) {
	manage_new_search_button("show");
	var oDiv = document.getElementById("div_provincie");
	if (oDiv)
	    oDiv.style.backgroundImage = 'url(/img2/cartine/transparent.gif)';	
}

function changeImageZone(zona) {
	var suffix = "";
	
	if (typeof window.document.dati.idRegione != "undefined")
		var idRegione = window.document.dati.idRegione.value;

	if (typeof window.document.dati.idProvincia != "undefined")
		var idProvincia = window.document.dati.idProvincia.value;
	
	if (typeof __isTrovaImmobili!="undefined" && __isTrovaImmobili) {
		suffix = "_TI";
		if (isTIVacanze() && TI_agComuneField && TI_agComuneField.getProvincia()) {
			var provincia = TI_agComuneField.getProvincia();
			var idRegione = provincia.regione.idObject;
			var idProvincia = provincia.idObject;
		} else if ( typeof window.document.dati.regione != "undefined")
			var idRegione = window.document.dati.regione.value;
	}
	if ( typeof idRegione == "undefined" || typeof idProvincia == "undefined")
		return;
	
	//ie sulla onclick della zona ricaricando l'immagine della zona cliccata fa scattare anche l'evento mouseover
	//che va inibito sul click
	manage_new_search_button("hide");
	var oDiv = document.getElementById("div_zone"+suffix);
	
	if(oDiv) {
		oDiv.style.backgroundImage = 'url(/img2/cartine/' + idRegione + '/' + idProvincia +'/map_zona_' + zona + '.gif' + getVersion() + ')';
	}
}

function hideImageZone() {
	var suffix;
	if (typeof __isTrovaImmobili!="undefined" && __isTrovaImmobili)
		suffix = "_TI";
	else
		suffix = "";
	manage_new_search_button("show");
	if( document.getElementById("div_zone"+suffix) ) {
		var oDiv = document.getElementById("div_zone"+suffix);
		oDiv.style.backgroundImage = 'url(/img2/cartine/transparent.gif)';
	}	
}

function selectLocalita(citta, zone) {
	var new_selected=new Array();
	var flag=0;

	if (zone_selected.length != 0 ){
		for (i=0,j=0; i< zone_selected.length ;i++){
			if (zone_selected[i] != zone){
				new_selected[j] = zone_selected[i];
				document.getElementById(new_selected[j]).checked = "checked";
				j++;				
			}else {
				flag=1;
			} 
		}
		if (!flag){ 
			new_selected[zone_selected.length] = zone;
		}
		zone_selected=new_selected;
	}else { 
			zone_selected[0] = zone;
	}
	return true;
}

function selectPreselectedZone(citta) {
	if (typeof zone_selected == "undefined" || zone_selected[0] == "undefined" )
		return;
	selectZona(citta, zone_selected[0]);
	
}

function selectZona(citta, zone,type) {
	__dontChangeSearchPath = true; 
	var flag = 0;
	var IE7 = false;
	
	if (typeof window.document.dati.idRegione != "undefined")
		var idRegione = window.document.dati.idRegione.value;
	
	if (typeof __isInserimentoAnnuncio != "undefined"){
		for (i=0,j=0; i< zone_selected.length ;i++)
			if (typeof zone_selected[i] != "undefined"){
				document.getElementById(zone_selected[i]).checked = false;
			}
		document.getElementById(zone).checked = true;
		zone_selected = new Array();
	}
	
	if (typeof __isTrovaImmobili!="undefined" && __isTrovaImmobili){
		type = "TI";
		if (isTIVacanze() && TI_agComuneField && TI_agComuneField.getRegione()) {
			var idRegione = TI_agComuneField.getRegione().idObject;
		} else if ( typeof window.document.dati.regione != "undefined")
			var idRegione = window.document.dati.regione.value;
	}else
	if (typeof type=="undefined")
		type=null;
		
	if (typeof zone=="undefined"){
		flag = 1;
	}
	
	var url_map = "/img2/cartine/" + idRegione + "/" + citta + "/map_zona_";
	
	var oContainer = document.getElementById("zone_container"+ (type ? "_"+type : ""));
	var browser = getBrowser();
	if ( browser && browser[0].toLowerCase().indexOf("explorer")!=-1 && parseInt(browser[1]) < 8){
	//if ( browser && browser.indexOf("msie")!=-1){
		oContainer.style.position ="relative";
		oContainer.style.left = "-150px";
	}
	var new_selected=new Array();
	var zone_sel;
	svuota_cointainer(oContainer);
	
	if (type=="TI")
		zone_sel = zone_selected_TI;
	else
		zone_sel = zone_selected;
	
	if (zone_sel.length != 0 ){
		for (i=0,j=0; i< zone_sel.length ;i++){
			if (typeof zone_sel[i] != "undefined" && zone_sel[i] != zone){
				if (typeof __isInserimentoAnnuncio!="undefined" && __isInserimentoAnnuncio && !!zone){
					document.getElementById(zone_sel[i]).checked = false;
				}else{
					new_selected[j] = zone_sel[i];
					createDivMap(oContainer, new_selected[j], url_map);
					//document.getElementById(new_selected[j]).checked = "checked";
					j++;
				}
			} else {flag=1;} 
		}
		if (!flag){ 
			new_selected[zone_sel.length] = zone;
			createDivMap(oContainer, zone, url_map);
		}
		if (!type)
			zone_selected=new_selected;
		else
		if (type=="TI")
			zone_selected_TI=new_selected;
	}else
	if (typeof zone !="undefined"){
		if (!type){
			zone_selected[0] = zone;
		}
		else
		if (type=="TI")
			zone_selected_TI[0] = zone;
		createDivMap(oContainer, zone, url_map);	
	}
	//TODO invertire chiamate swapZona e SelectZona su alcune mappe (ordine esatto prima selectZona poi SwapZona)
	if (typeof __isInserimentoAnnuncio !="undefined" && __isInserimentoAnnuncio){
		swapZona(zone,type);
	}
	
	if (zone_selected.length > 0) {
		forceTypeSearch('zona');
	} else {
		switchOffBtn('zona');
		jq("#modifica_zona").hide();
	}
	
	return true;
}

function selectProvincia(provincia) {
	var oContainer = document.getElementById("provincie_container");
	var oDivChild = document.createElement("div");
	var oImg = document.createElement("img");
	var browser = getBrowser();
	if ( browser && browser[0].toLowerCase().indexOf("explorer")!=-1 && parseInt(browser[1]) < 8){
	//if ( browser && browser.indexOf("msie")!=-1){
		oContainer.style.position ="relative";
		oContainer.style.left = "-150px";
	}

	svuota_cointainer(oContainer);
	
	oDivChild.style.position="absolute";
	oDivChild.id = provincia;
	oImg.src ="/img2/cartine/" + window.document.dati.idRegione.value + "/select_" + provincia + ".gif"+ getVersion();
	oImg.useMap="#Map";
	oImg.border="0";
	oDivChild.appendChild(oImg);
	oContainer.appendChild(oDivChild);
	
	var oArea = document.getElementById("_map_" + provincia);
	if (oArea)
		oArea.removeAttribute("shape");
}
function selectNazione(nazione) {
	var oContainer = document.getElementById("nazioni_container");
	var oDivChild = document.createElement("div");
	var oImg = document.createElement("img");
	var browser = getBrowser();
	if ( browser && browser[0].toLowerCase().indexOf("explorer")!=-1 && parseInt(browser[1]) < 8){
	//if ( browser && browser.indexOf("msie")!=-1){
		oContainer.style.position ="relative";
		oContainer.style.left = "-150px";
	}

	svuota_cointainer(oContainer);
	
	oDivChild.style.position="absolute";
	oDivChild.id = nazione;
	oImg.src ="/img2/cartine/" + window.document.dati.idNazione.value + "_selected.gif"+ getVersion();
	oImg.useMap="#nazioni";
	oImg.border="0";
	oDivChild.appendChild(oImg);
	oContainer.appendChild(oDivChild);
	
	//var oArea = document.getElementById("_map_" + nazione);
	//oArea.removeAttribute("shape");
}

function createDivMap(oContainer, id_zona, url_map){
	if( typeof(oContainer) == 'undefined' || oContainer == null || typeof id_zona == 'undefined' || !id_zona || !parseInt(id_zona) )
		return;
	
	var oDiv = document.createElement("div");
	var oImg = document.createElement("img");
	
	oDiv.id = "map_zona_" + id_zona;
	oDiv.style.position = "absolute";
	if ( id_zona != '-1' )
		oImg.src = url_map + id_zona + ".gif"+ getVersion();
	else
		oImg.src = '/img2/cartine/transparent.gif'+ getVersion();
		
	oImg.useMap = "#Map";
	oImg.border="0";
	oDiv.appendChild(oImg);
	oContainer.appendChild(oDiv);
}

function svuota_cointainer(oContainer){
	if(oContainer) {
		while (oContainer.hasChildNodes())
			oContainer.removeChild(oContainer.firstChild);
	}
}

function svuotaDivContainer(idDiv)
{
	oDivParent = window.document.getElementById(idDiv);
	if (oDivParent)
		svuota_cointainer(oDivParent);
}


function preloadImageEstero(){
	var oMapNazioni = document.getElementById("nazioni");
	if (oMapNazioni){
		var childDiv = oMapNazioni.childNodes;
		var version = getVersion();
		for (var i=0; i< (childDiv.length - 1) ; i++){
			var imageLoad = new Image();
			if(childDiv[i].id)
				imageLoad.src = "/img2/cartine/" + childDiv[i].id.substring(5) + "_selected.gif" + version;

		}
	}
}

function preloadImageNazione(){
	if (!document.getElementById("regioni"))
		return (null);

	var oMapRegioni = document.getElementById("regioni");
	var aRegioni = oMapRegioni.getElementsByTagName('area');
	var imageLoad = new Image();
	var version = getVersion();
	if (typeof __loaded_regione_index == "undefined")
		__loaded_regione_index = 0;
	else
		__loaded_regione_index++;
		
	if (__loaded_regione_index < aRegioni.length){
		var sPathImg = "/img2/cartine/" + aRegioni[__loaded_regione_index].id.substring(5) + "_selected.gif" + version;
		imageLoad.src = sPathImg;
		imageLoad.onload = preloadImageNazione;
	}
}




function preloadImageProvincie(regione, image){
		var imageLoad = new Image();
		imageLoad.src = "/img2/cartine/" + regione + "/select_prov_" + image + ".gif"+ getVersion();
}

function preloadImageZone(comune, image){
		var imageLoad = new Image();
		imageLoad.src = "/img2/cartine/" + window.document.dati.idRegione.value + "/" + comune + "/map_zona_" + image + ".gif"+ getVersion();
}

function abilitaCitta() {
	var oDivNewCitta = window.document.getElementById("otherLinkDirettiCitta");
	var oDivCitta = window.document.getElementById("linkDirettiCitta");
	if (oDivNewCitta && oDivCitta){
		oDivAltre = window.document.getElementById("linkAltreCitta");
		if(oDivAltre)
			oDivAltre.style.display="none";
		oDivCitta.innerHTML += oDivNewCitta.innerHTML;
	}
}

String.prototype.trim = function() {
	return this.replace(/^\s+|\s+$/g,"");
};
String.prototype.ltrim = function() {
	return this.replace(/^\s+/,"");
};
String.prototype.rtrim = function() {
	return this.replace(/\s+$/,"");
};

function createAnimatedButton(className,divContainer,onclickEvent,name,value,id){
	//permitted className = bottoneCerca - bottoneTrova - bottoneInvia - bottoneAttivaServizio
	var div;
	if (typeof divContainer == "string")
		div = document.getElementById(divContainer);
	else
		div = divContainer;
	
	var btn = document.createElement('input');
	btn.className = className;
	if(typeof name!='undefined')
		btn.name=name;
	if(typeof id!='undefined')
		btn.id=id;
	
	if (onclickEvent){
		addEvent(btn,"click",onclickEvent);
		btn.type="button";
	}else{
		btn.type="submit";
		
	}
	if (value)
		btn.value = value;
	else
		btn.value = "";
	
	//devo fare il bind perchè co explorer non funziona
	btn.style.position='relative';
	btn.onmousedown = function(){this.style.top='1px';this.style.left='1px';};
	btn.onmouseup = function(){this.style.top='0px';this.style.left='0px';};
	btn.onmouseout = function(){this.style.top='0px';this.style.left='0px';};
	btn.onmouseover = function(){this.style.backgroundPosition='bottom';};
	btn.onmouseout = function(){this.style.backgroundPosition='top';};
	btn.onfocus = function(){this.blur();};
	
	div.appendChild(btn);

	addEvent(window,"unload",function(){purgeDomObj(btn);});

}

function createCss3Button(color,className,divContainer,onclickEvent,tagType,name, value){
	var btnContainer;
	if (typeof divContainer == "string")
		btnContainer = document.getElementById(divContainer);
	else
		btnContainer = divContainer;
	//creo div esterno del gradiente colore
	var btnGradient = document.createElement('div');
	btnGradient.className = "buttonCss3 "+ color;
	//creo div bordo 
	var btnBorder = document.createElement('div');
	btnBorder.className = "bordoBtnCss3 ";
	
	if(typeof tagType !='undefined' && tagType == 'span') {
		var btn = document.createElement('span');
		if (onclickEvent){
			addEvent(btn,"click",onclickEvent);
		}
		if (value)
				btn.innerHTML = value;
			else
				btn.innerHTML = "";
	} else {
		
		var btn = document.createElement('input');
		
		if(typeof name!='undefined') btn.name=name;

		if (onclickEvent){
			addEvent(btn,"click",onclickEvent);
			btn.type="button";
		}else{
			btn.type="submit";
		}
		
		if (value){
			btn.value = value;
		}else{
			btn.value = "";
		}
	}
	btn.className = className;
	
	btnContainer.appendChild(btnGradient).appendChild(btnBorder).appendChild(btn);

}

function disableAnimatedButton(className,divContainer){
	var div;
	if (typeof divContainer == "string")
		div = document.getElementById(divContainer);
	else
		div = divContainer;
	var oButtons = getElementsByClass(className,div);
	for(var i = 0; i < oButtons.length; i++)
	{
		oButtons[i].style.cursor='default';
		evidenzia(oButtons[i], 3);
	}
}

function toggleDiv(idCont,el,openedText,closedText){
    if (typeof el.isShowingDiv == undefined){
        el.isShowingDiv == false;
    }
    if (el.isShowingDiv){
        MEslideDown(idCont,0);
        el.isShowingDiv = false;
        el.innerHTML = closedText;
    }else{
        MEslideUp(idCont,0);
        el.isShowingDiv = true;
        el.innerHTML = openedText;
    }
    el.blur();
}

function closeOpenDiv(closeCont, openCont){
    MEslideUp(openCont,0);
    MEslideDown(closeCont,0);
}

function getVersion() {
	if(typeof version_map != "undefined") {
		return version_map;
	}
	return ""; 
}


function __homePageLoginAnimationForm(){
    jq("#containerAccessoHome a[data-toggle='tab']").each(function(){
		jq(this).attr("hideFocus", "true");
        jq(this).click(function(e){
            jq("#containerAccessoHome .nav-tabs li").removeClass("active");
            jq(this).parent().addClass("active");

            jq("#containerAccessoHome .tab-pane").removeClass("active").removeClass("in");
            var y = jq(jq(this).attr("related-tab")).addClass("active").fadeIn();
            var x = function(){
                y.addClass("in");
            }
            setTimeout(x,50);
            e.preventDefault();
            return false;
        })
    });
}
/**
* Funzione che crea l'HTML dei form di accesso/registrazione all'interno del boxOverlay
* Ci sono 4 contenuti possibili discriminati in base al parametro _type:
* Form di Accesso Utente (_type == "U")
* Form di Registrazione Utente (_type == "RU")
* Form di Accesso Agenzia (_type == "A")
* Form di Recupera Password (_type == "RP")
**/
function create_accediBox(_type,options){ // Attenzione! non rimuovere il CSS inline. E' necessario per il corretto e immediato centramento dell'overlayBox. Se fosse incluso nel file css il box si centrerebbe solo una volta terminato il caricamento del file css stesso.
	if (typeof _type == "undefined")
		type = "U";
	else
		type = _type;
		
	if (!in_array(type, ["A","U","RU","RP"])){
		alert("wrong type for create_accedi_box");
		return false;
	}
	preloadImages(new Array("/img2/loader6.gif"));
	selAccediBox = new Object;
	switch (type){
		case "A":
			selAccediBox.form_onsubmit = eseguiAccessoAg;
			selAccediBox.height = '264px';
			selAccediBox.loaderPos = '35%';
			selAccediBox.title = "<div style='padding: 24px 28px 0px 34px; width: 341px;'><strong>Accedi a </strong><div id=\"icona_login_agenzia\"></div></div>";
			selAccediBox.footer = "<div>Cerchi, vendi o affitti casa? <a class=\"link_blu\" style=\"font-weight:bold;\" onclick=\"selAccediBox.box.destroy();var _accediUtenteBox = create_accediBox('U',{backurl:''});_accediUtenteBox.show();this.blur();return false\">Accedi</a></div>";
			selAccediBox.registrationLink = "/agenzia/registra-nuova-agenzia.php";
			if( imm_FB != null && typeof imm_FB != 'undefined' )
				imm_FB.stopAutoLogin();
			break;
		case "RU":
			selAccediBox.form_onsubmit = eseguiRegistrazione;
			selAccediBox.height = '480px';
			selAccediBox.loaderPos = '35%';
			selAccediBox.title = "<div style='padding: 24px 26px; width: 352px;'><strong>Registrati, è gratis</strong><div style='padding: 15px 0 16px;'>Sincronizza ricerche e annunci con smartphone e tablet</div></div>";
			selAccediBox.footer = "Sei un agente immobiliare o un costruttore? <a class=\"link_blu\" style=\"font-weight:bold;\" href=\"/agenzia/registra-nuova-agenzia.php\">Registrati</a>";
			break;
		case "RP":
			selAccediBox.fromType = !jq.isEmpty(options.fromType) && in_array(options.fromType, ["A","U"]) ? options.fromType : 'U';
			selAccediBox.form_onsubmit = recuperaPassword;
			selAccediBox.height = '';
			selAccediBox.loaderPos = '35%';
			selAccediBox.title = "<div style='width: 341px;'><strong>Recupera la tua password</strong></div>";
			selAccediBox.footer = "Sei già registrato? <a class=\"link_blu\" style=\"font-weight:bold;\" onclick=\"selAccediBox.box.destroy();var _accediUtenteBox = create_accediBox( selAccediBox.fromType ,'');_accediUtenteBox.show();this.blur();return false\">Accedi</a>";
			break;
		default:
			selAccediBox.form_onsubmit = eseguiAccesso;
			selAccediBox.height = '352px';
			selAccediBox.loaderPos = '18%';
			selAccediBox.title = "<div style='padding: 24px 26px; width: 352px;'><strong>Accedi</strong><div style='padding: 15px 0 16px;'>Sincronizza ricerche e annunci con smartphone e tablet</div></div>";
			selAccediBox.footer = "Non sei registrato? <a class=\"link_blu\" style=\"font-weight:bold;\" onclick=\"selAccediBox.box.destroy();var _registraUtenteBox = create_accediBox('RU',{backurl:'', callback: addAutoCompleteDomainRU});_registraUtenteBox.show();this.blur();return false\">Registrati</a>";
			break;
	}

    selAccediBox.loginType = "";
    selAccediBox.backurl = "";
	if (typeof options == "object"){
		selAccediBox.backurl = options.backurl ? options.backurl : "";
        selAccediBox.loginType = options.loginType ? options.loginType : "";
        selAccediBox.callback = (typeof options.callback == "function") ? options.callback : "";
	}
	var ac_backurl = (typeof selAccediBox.backurl != "undefined") && (selAccediBox.backurl != "") ? selAccediBox.backurl : "";

	boxContent = '';
	var that = new Object();
	that.boxContent = boxContent;
	that.type = type;
    that.loginType = selAccediBox.loginType;
    that.callback = (typeof selAccediBox.callback == "function") ?  selAccediBox.callback : null;
		
	selAccediBox.box = new boxOverlay(type+'accediBox','grey2',{'whiteHeader':true});
	selAccediBox.box.setFooter(selAccediBox.footer);
	selAccediBox.box.setTitle(selAccediBox.title);
	selAccediBox.box.setContent('<div style="min-height:'+selAccediBox.height+';background-position: center '+selAccediBox.loaderPos+' "id="loading_div"></div>');
	
	var oAjax = new getMEAjaxObj();
	var ulrAjax = '/login.php?type='+type+'&ac_backurl='+ac_backurl+"&loginType="+selAccediBox.loginType;
	oAjax.Request( 'POST' , ulrAjax , function( response ){  writeLogin( response, this.boxContent, this.type, this.callback); }.bind( that ) );
    trackGAClickEvent("boxLoginEvents","requestLogin",type);
	return selAccediBox.box;
}

function enterSubmitCheck(e, elemento, callback) {
    var el = jq(elemento);
    var kC  = (window.event) ? window.event.keyCode : (e.keyCode ? e.keyCode : e.which); /* MSIE or Firefox? */
    if (kC != 13)
        return;
	
    var attachEvent = false;
    //3 checks undefined jquery, typeof undefined nativo, check stringa su cast di undefined di jquery
    if (el.attr("old-val") == undefined || typeof el.attr("old-val") == "undefined" || el.attr("old-val") == 'undefined'){
        attachEvent = true;
	}
	el.attr("old-val",el.val());
    if (attachEvent){
        elemento.setAttribute("onkeyup","__entSubCheck(event)");
		__entSubCheck=function(event){
			return enterSubmit(event,elemento,callback);
		}
    }

    function enterSubmit(e,elemento,callback) {
        var kC  = (window.event) ? window.event.keyCode : (e.keyCode ? e.keyCode : e.which); /* MSIE or Firefox? */
        if (kC != 13)
            return;
		
        var el = jq(elemento);
        if (el.val() == el.attr("old-val")){
            callback();
        }
        else{
            el.attr("old-val",el.val());
        }

    }
}
//recupero contenuto html del boxOverlay di Login/Registrazione 
function writeLogin( response, boxContent, type, callback) {
	boxContent+= response.responseText;
	selAccediBox.box.stop_event_bubbling = true;
	selAccediBox.box.setContent(boxContent);
	selAccediBox.initContent = selAccediBox.box.getContent();
	createAnimatedButton(type+"btnOverlayBox","overlayBoxButtonWrap");
	addPlaceHolderCompatibility(type+'-form');

    jq( '#usernameLoginInput' ).keydown( function( e ) {
        enterSubmitCheck(e,this,function(){selAccediBox.form_onsubmit( document.getElementById ( type+'-form' ) );});
    });
    trackGAClickEvent("boxLoginEvents","rendered",type);

	if (typeof callback == "function"){
		callback();
	}

	if( typeof imm_FB != 'undefined' && imm_FB != null )
		imm_FB.initListnerClickStopAutoLogin();
}

/**
* Funzione per la gestione del contenuto dei campi input del login
* @option obj sender -> Elemento chiamante la funzione
* @option string value -> Valore che l'elemento chiamante assume al momento della chiamata
* @option e string -> Stringa che determina il tipo di evento che ha chiamato la funzione
*/
function controlInputLogin( sender, value, e ) {
	if ( e == 'focus' && sender.value == value ) {
		if ( sender.type == 'text' && sender.name != "email" && sender.name != "username") {
			sender.name = 'password';
			sender = changeInputType( sender, 'password' );
		}
	sender.value = '';
	jq(sender).addClass("textInput").removeClass("placeHolder");
	setTimeout(function(){ sender.focus(); },400);
	}else if (e == 'focus' && sender.value !== value){
		jq(sender).addClass("textInput").removeClass("placeHolder");
	}
	if ( e == 'blur' && sender.value == '' ) {
	if ( sender.type == 'password' && sender.name != "email") {
		sender.name = '';
		sender = changeInputType( sender, 'text' );
	}
	sender.value = value;
	jq(sender).addClass("placeHolder").removeClass("textInput");
	} else if (e == 'blur' && sender.value !== '' && sender.value !== value){
		jq(sender).addClass("textInput").removeClass("placeHolder");
	}
} 
/**
 * Funzione che cambia il tipo ad un campi input
 * @option obj oldObject -> Riferimento all'elemento input a cui cambiare il tipo
 * @option string type -> Il tipo che si vuole assegnare al campo input ( 'text', 'password', 'hidden' )
 */
function changeInputType(oldObject, oType) {
	var LTIE7 = false;
	var browser = getBrowser();
	if ( browser && browser[0].toLowerCase().indexOf("explorer")!=-1 && parseInt(browser[1]) <= 7 ){
        LTIE7 = true;
	}
	//Bug fix creazione tag name su ie7
	if ( LTIE7 ){
		var newObject = document.createElement( '<input type="'+oType+'" placeholder="Password" name="'+oldObject.name+'" />' );
	}
	else{
		var newObject = jq('<input/>', {
		'placeholder':'Password',
		name:oldObject.name,
		type:oType
		});
		
		var newObject = newObject[0];
	}

	//Recupera tutti gli attributi del vecchio oggetto e li setta nel nuovo
	var a = oldObject.attributes;
	if ( a ) {
		l = a.length;
		for ( var i = 0; i < l; i += 1) {

			n = a[i].name;

			if ( n != 'type' && n!= 'placeholder' ) {
				if ( n == 'class' )
					newObject.className = oldObject.className;
				else if ( typeof oldObject[n] == 'function' ){
					newObject.setAttribute( n, oldObject.getAttribute( n ) );
                }
				else
					newObject.setAttribute( n, oldObject[n] );
			}
		}
	}

	oldObject.parentNode.replaceChild( newObject, oldObject );
	if (typeof(oldObject._chtAttachedEvents) == "undefined"){ /*verifica se gli eventi focus/blur sono stati già associati al nuovo oggetto newObject */
        oldObject._chtAttachedEvents = 1;		
		jq(document).on("focus","#"+newObject.id,function () {
			var el=document.getElementById(newObject.id);												    
			if (jq(this).attr('placeholder') != '' && jq(this).val() == jq(this).attr('placeholder')) {
				jq(this).val('').removeClass('placeHolder').addClass('textInput').select();
				if(el.type=="text"){
					var nuovo_campo_inputPassword=changeInputType(el,"password");
					setTimeout(function(){ nuovo_campo_inputPassword.focus(); },300);	
				}
			}
		});
        jq(document).on("blur","#"+newObject.id,function () {
			var el=document.getElementById(newObject.id);
			if (jq(this).attr('placeholder') && (jq(this).attr('placeholder') != '') && (jq(this).val() == '' || jq(this).val() == jq(this).attr('placeholder'))) {
				jq(this).val(jq(this).attr('placeholder')).addClass('placeHolder').removeClass('textInput');
				if(el.type=="password"){	
					changeInputType(el,"text");
				}
			}
		});
	}
					
	return newObject;
}



/*function checkOpenedSession(){

	if (!readCookie("IMMSESSID")){
		return false;
	}
	
	showAlertTip("<strong>Recupero sessione precedente in corso.....</strong>","info");
	
	var hostname = document.location.host;
	if (document.getElementById("accedi_box_loader"))
		document.getElementById("accedi_box_loader").style.display="";
	
	if (typeof SSOwindow != "undefined")
		SSOwindow.close();
	
	var checkurl = "http://"+hostname+"/accedi.php?action=checkSession";
	var aj = getMEAjaxObj();
	var obj = {};
	
	aj.Request("POST",checkurl,function(response){
		var url;
		
		if (parseInt(response.responseText) == 1){
			showAlertTip("<strong>Sessione recuperata con successo, eseguo Login.....</strong>","info");
			var rechargeHtml = "<div style=\"width:100%;padding:40px 0px;text-align:center\"><h3>Ripristino Sessione in corso...</h3><img src=\"/img2/loader3.gif\" alt=\"loading\"/></div>";
			var box = new boxOverlay('SSOaccediBox','grey');
			box.setTitle("<div class=\"box_header title\"><div id=\"icona_login_utente\"></div></div>");
			box.setContent(rechargeHtml);
			box.show();
			setTimeout(function(){
				window.location.href = getOnAccessRelocationUrl();
			},2000);
			if (document.getElementById("accedi_box_loader"))
				document.getElementById("accedi_box_loader").style.display="none";
			return true;
		}else{
			
			showAlertTip("<strong>La sessione non è più valida, rieseguire l'accesso.</strong>","error",2000);
		}
		if (document.getElementById("accedi_box_loader"))
			document.getElementById("accedi_box_loader").style.display="none";
		return false;
	}.bind(obj));
	
	return true;
	
}
*/
function getOnAccessRelocationUrl(url){
	
	//array contente le url per cui eseguire il redirect alla pagina specificata o in home page se non specificata
	//myRegExp.push(new Array(/<url attuale da verificare>/, "<url a cui eseguire la redirect> - [opzionale]"));

	var myRegExp = new Array();	
	myRegExp.push(new Array(/registrati\.php/));
	myRegExp.push(new Array(/singleSignOn\.php/));
    myRegExp.push(new Array(/accedi\.php/));
	myRegExp.push(new Array(/pubblica_annuncio_privato\.php/,"/inserimento_annuncio_privati.php"));
	
	var loc = url;
	
	if (typeof url=="undefined" || !url)
		loc = window.location;
	else
	if (!checkUrl(url))
		loc = "/";
	
	for (var regExp in myRegExp){		
		var re = new RegExp(myRegExp[regExp][0]);
		var m = re.exec(loc);		
		if (m != null) {
			if (typeof myRegExp[regExp][1] != undefined)
				return  myRegExp[regExp][1] != undefined ?  myRegExp[regExp][1] : "/";
		}
	}
	
	return loc;
	        
}

function SSOLogin(account, mode, options) {
	options = options || {};
	deleteCookie('autoSyncFacebook');
	__can_close_SSOpopup = true;
	options.reloadAfterClosePopup = typeof options.reloadAfterClosePopup != 'undefined' ? options.reloadAfterClosePopup : true;

	/*if (typeof account=="undefined"){
		throw("Called SSO without 'account' parameter");
		return false;
	}
	
	if (typeof mode=="undefined"){
		throw("Called SSO without 'mode' parameter");
		return false;
	}
	*/
	
	var hostname = document.location.host;
	if (typeof SSOwindow != "undefined")
		SSOwindow.close();

	if (typeof account == "undefined" || !account || typeof mode == "undefined" || !typeof mode) {
		var url = "//" + hostname + "/singleSignOn.php?action=dologin&accountType=" + account + "&mode=" + mode;
	} else {
		var url = "//" + hostname + "/singleSignOn.php?action=dologin&accountType=" + account + "&mode=" + mode;

		if (document.getElementById('persistent_flag') && document.getElementById('persistent_flag').checked) {
			url += "&persistent=1";
		}
	}
	
	var syncFacebook = false;
	var forceClose = false;
	if (typeof options == "object"){
		if (options.extraUrlPars)
			url+="&"+options.extraUrlPars;
		if (options.syncOpenGraph)
			var syncFacebook = true;
		if (options.forceClose)
			var forceClose = true;

		if (options.facebook_scope){
			url+="&sso_scope="+options.facebook_scope;
		}
	}
	
	if (document.getElementById("accedi_box_loader"))
		document.getElementById("accedi_box_loader").style.display="";
	
	SSOwindow = popup(url,"MyImmobiliare","900","500");
	if (typeof selAccediBox == "undefined" || selAccediBox.box != null){
		create_accediBox("U");
		if (!syncFacebook)
			selAccediBox.box.show();
	}
	
	selAccediBox.box.hideOnlyBox();
	selAccediBox.box.addEventOnClose(function(){
		if (SSOwindow && typeof SSOwindow != "undefined") {
			SSOwindow.close();
			if (typeof checkPopupInterval!="undefined")
				clearInterval(checkPopupInterval);
		}
	});
	if (forceClose) {
		window.location.reload();
		__can_close_SSOpopup = true;
	}

	if (SSOwindow && typeof SSOwindow != "undefined") {
		checkPopup(options);
	}

	return true;
}

function checkPopup(options) {
	options = options || {};
	checkPopupInterval = setInterval(function () {
		if (SSOwindow.closed) {
			if (__can_close_SSOpopup) {
				selAccediBox.box.close();
				if (typeof options.handler == 'function') {
					options.handler();
				}

				if (options.reloadAfterClosePopup === true) {
					var backurl = options.backurl || window.getValue('backurl');

					if (!jq.isEmpty(backurl)) {
						window.location = Url.decode(backurl);
					} else {
						window.location.reload();
					}
				}
			}
			clearInterval(checkPopupInterval);
		}
	}, 500);
}


function checkAndSSOLogin(account,mode){
	if (typeof account=="undefined"){
		throw("Called SSO without 'account' parameter");
		return false;
	}
	if (typeof account=="undefined"){
		throw("Called SSO without 'mode' parameter");
		return false;
	}
	
	
	var hostname = document.location.host;
	if (document.getElementById("accedi_box_loader"))
		document.getElementById("accedi_box_loader").style.display="";
	
	if (typeof SSOwindow != "undefined")
		SSOwindow.close();

	var url = "//" + hostname + "/singleSignOn.php?action=start&accountType=" + account + "&mode=" + mode;
	SSOwindow = popup(url,"MyImmobiliare","500","300");
	selAccediBox.box.hide();

	var checkurl = "//" + hostname + "/singleSignOn.php?action=checktoken&accountType=" + account + "&mode=" + mode;
	var aj = getMEAjaxObj();
	var obj = {};
	obj.account = account;
	obj.mode = mode;
	obj.checkOnly = checkOnly;
	aj.Request("POST",checkurl,function(response){
		var url;
		var account = this.account;
		var mode = this.mode;
		var checkOnly = this.checkOnly;
		
		if (parseInt(response.responseText) == 1){
			var rechargeHtml = "<div style=\"width:100%;padding:40px 0px;text-align:center\"><h3>Accesso in corso...</h3><img src=\"/img2/loader3.gif\" alt=\"loading\"/></div>";
			var box = new boxOverlay('SSOaccediBox','grey');
			box.setContent(rechargeHtml);
			
			window.location.reload();
			document.getElementById("accedi_box_loader").style.display="none";
			return true;
		}else{

			switch (account) {
				case "google":
					if (mode == "openid") {
						url = "//" + hostname + "/singleSignOn.php?action=start&accountType=google&mode=openid";
					} else {
						url = "//" + hostname + "/singleSignOn.php?action=start&mode=oauth&accountType=google";
					}
					break;
				case "facebook":
					url = "//" + hostname + "/singleSignOn.php?action=start&mode=oauth&accountType=facebook";
					break;
			}
		
			SSOwindow = popup(url,"MyImmobiliare","500","300");
			selAccediBox.box.hide();
		}
		if (document.getElementById("accedi_box_loader"))
			document.getElementById("accedi_box_loader").style.display="none";
		return false;
	}.bind(obj));
}

function eseguiAccesso(oForm, handlerFn){
	oForm.mode.value = "xml";
	var error = false;
	var username = jq("#usernameLoginInput");
	var password = jq("#passwordLoginInput");
	var loginFailure = jq.isEmpty( jq("#loginFailure") ) ? false : true;
	if( loginFailure ){
		jq("#loginFailure").css({"height":"0px", "margin-bottom":"0px"}).text("");
	}else{
		jq("#accediBoxError").css("display","none").text("");
	}

	username.prev().css("height","0px").text("");
	password.prev().css("height","0px").text("");
	if (!oForm.email.value || oForm.email.value=="Username"){
		username.prev().css("height","21px").text("Inserisci la tua email");
		username.addClass('input_error');
		error = true;
	}else if (!checkEmail(oForm.email.value)){
		username.prev().css("height","21px").text("Inserisci una email valida");
		username.addClass('input_error');
		error = true;
	} else {
		username.removeClass('input_error');
	}
	if (!oForm.password || !oForm.password.value || oForm.password.value=="Password"){
		password.prev().css("height","21px").text("Inserisci la tua password");
		password.addClass('input_error');
		error = true;
	} else {
		password.removeClass('input_error');
	}
	if (!error){
		if (typeof handlerFn == "undefined")
			handlerFn = doLogin;
		submitAjaxForm(oForm,handlerFn);
	}
	return false;
}

function connectAccountAndLogin(response)
{
	if (trim(response.responseText) == ""){
		window.location.reload();
	}else
	if (checkUrl(response.responseText)){
		window.location.href=response.responseText;
	}
	else{
		var loginErr = document.getElementById("accediBoxError");
		var loginTr3 = document.getElementById("accediBoxErrorTr3");
		loginErr.innerHTML = response.responseText;
		loginTr3.style.display="";
	}
}
function onLoginEvents(who){
    trackGAClickEvent("boxLoginEvents","accesso",who);
	if (document.getElementById('loaderLogin')){
		document.getElementById('loaderLogin').style.display="";
		
	}else{
		var rechargeHtml = "<div style=\"width:100%;padding:40px 0px;text-align:center\">Accesso in corso...<img src=\"/img2/loader3.gif\" alt=\"loading\"/></div>";
		
		if (typeof selAccediBox != "undefined")
			selAccediBox.box.innerHTML= rechargeHtml;
		else{
			if (document.getElementById('contentAccedi'))
				document.getElementById('contentAccedi').innerHTML= rechargeHtml;
		}
	}
}

/**
 * Questa funzione serve a personalizzare le chiamate handler in caso di successo o errore del login.
 * Basta passare un oggetto così definito: { onSuccess: function(){}, onError: function(){} } e poi eseguire la "onLogin"
 * Vengono eseguite comunque alcune funzioni standard.
 *
 * @param options
 * @returns {{onLogin: onLogin}}
 */
function setDoLoginWithHandler(options)
{
	var opt = options || {};
	var onSuccess = typeof opt.onSuccess == 'function' ? opt.onSuccess : onSuccessDefault();
	var onError = typeof opt.onError == 'function' ? opt.onError : onErrorDefault();

	/**
	 * Esegue l'accesso con i relativi handler passati come opzioni
	 * @param response
	 */
	function onLogin(response)
	{
		var trackLoginType = jq("#trackLoginFormType").val() ? trim(jq("#trackLoginFormType").val()) : "U";
		if (trim(response.responseText) == ""){
			onLoginEvents(trackLoginType);
			onSuccess();
		} else if (checkUrl(response.responseText)){
			onLoginEvents(trackLoginType);
			onSuccess(response.responseText)
		} else {
			onError(response);
		}
	}

	/**
	 * Ritorna la funzione di default da eseguire in caso di successo
	 * @returns {Function}
	 */
	function onSuccessDefault()
	{
		return function(_urlLocation)
		{
			setTimeout(function(){
				if (!_urlLocation)
					window.location.reload();
				else
					window.location.href = _urlLocation;
			}, 500);
		}
	}

	/**
	 * Ritorna la funzione di default da eseguire in caso di errore
	 * @returns {Function}
	 */
	function onErrorDefault()
	{
		return function(response)
		{
			var loginFailure = jq.isEmpty( jq("#loginFailure") ) ? false : true;
			jq("#usernameLoginInput").prev().css("height","0px");
			jq("#passwordLoginInput").prev().css("height","0px");
			jq("#passwordLoginInput,#usernameLoginInput").addClass('input_error');

			var err_width = "307px";
			if(document.getElementById("standardLogin"))
				err_width = "276px";

			/* Calcolo altezza del div che conterrà l'errore */
			var errDivAppoggio = document.createElement("div");
			jq(document.body).append(errDivAppoggio);
			jq(errDivAppoggio).css("position", "absolute").css("width", err_width).css("visibility", "hidden").css("display", "block").html(response.responseText);
			var err_height = jq(errDivAppoggio).height() + "px";
			jq(errDivAppoggio).remove();

			/* Fine calcolo altezza del div che conterrà l'errore */
			if(loginFailure){
				jq("#loginFailure").html(response.responseText).css({"height":err_height, "margin-bottom":"10px"});
			}else{
				jq("#accediBoxError").css("display", "block").css("width", err_width).html(response.responseText).css("height", err_height);
			}
		}
	}

	return {
		onLogin: onLogin
	}
}


function doLogin(response)
{
	var doLoginWithHandler = new setDoLoginWithHandler({});
	doLoginWithHandler.onLogin(response);
}


function eseguiAccessoAg(oForm){
	oForm.mode.value = "xml";

	var error = false;
	var username = jq("#usernameLoginInput");
	var password = jq("#passwordLoginInput");
	
	jq("#loginFailure").css({"height":"0px"}).text("");
	username.prev().css("height","0px").text("");
	password.prev().css("height","0px").text("");
	
	if (!oForm.email.value || oForm.email.value=="Username"){
		username.prev().css("height","21px").text("Inserisci la tua email");
		username.addClass('input_error');
		error = true;
	}else if (!checkEmail(oForm.email.value)){
		username.prev().css("height","21px").text("Inserisci una email valida");
		username.addClass('input_error');
		error = true;
	} else {
		username.removeClass('input_error');
	}

	if (!oForm.password || !oForm.password.value || oForm.password.value=="Password"){
		password.prev().css("height","21px").text("Inserisci la tua password");
		password.addClass('input_error');
		error = true;
	} else {
		password.removeClass('input_error');
	}
	if (!error){
		submitAjaxForm(oForm,doLoginAg);
	}
	return false;
}

function doLoginAg(response){
	if (isNumeric(response.responseText)){
		var target = document.getElementById("A-form").getAttribute('target');
        onLoginEvents("A");
            setTimeout(function(){
                if (selAccediBox.backurl){
                    if(target == '_parent'){
                        parent.location.href=selAccediBox.backurl;
                    }else{
                        window.location.href=selAccediBox.backurl;
                    }
                    return;
                }
                switch(parseInt(response.responseText)){
                    case 0 :
                        break;
                    case 1 :
                        if(target == '_parent'){
                            parent.location.href="/home_gestionale.php";
                        }else{
                            window.location.href="/home_gestionale.php";
                        }

                        break;
                    case 2 :
                        if(target == '_parent'){
                            parent.location.href="/amministrazione.php";
                        }else{
                            window.location.href="/amministrazione.php";
                        }

                        break;
                    default :
                        if(target == '_parent'){
                            parent.location.href="/index.php";
                        }else{
                            window.location.href="/index.php";
                        }

                        break;
                }
                return;
            },500);
    }else{
            var loginErr = jq("#loginFailure");
            jq("#passwordLoginInput,#usernameLoginInput").addClass('input_error');
			jq("#usernameLoginInput").prev().css("height","0px");
			jq("#passwordLoginInput").prev().css("height","0px");

			var err_width = "307px";
			if(document.getElementById("standardLogin"))
				err_width = "276px";

			/* Calcolo altezza del div che conterrà l'errore */

			var errDivAppoggio = document.createElement("div");
			jq(document.body).append(errDivAppoggio);
			jq(errDivAppoggio).css("position","absolute").css("width",err_width).css("visibility","hidden").css("display","block").html(response.responseText);
			var err_height = jq(errDivAppoggio).height() + "px";
			jq(errDivAppoggio).remove();

			/* Fine calcolo altezza del div che conterrà l'errore */

			loginErr.html(response.responseText).css({"height":err_height,"margin-bottom":"10px"});
    }
}

var errorMsgRegistrazione = function(oForm, type, text){
	switch(type){
		case 'nome-cognome':
			jq(oForm.nome).prev().css("height","21px").text("Inserisci nome e cognome");
			jq(oForm.nome).addClass('input_error');
			jq(oForm.cognome).addClass('input_error');
			break;
		case 'nome':
			jq(oForm.nome).prev().css("height","21px").text("Inserisci il nome");
			jq(oForm.nome).addClass('input_error');
			break;
		case 'cognome':
			jq(oForm.nome).prev().css("height","21px").text("Inserisci il cognome");
			jq(oForm.cognome).addClass('input_error');
			break;
		case 'noErrors-nomecognome':
			jq(oForm.nome).removeClass('input_error');
			jq(oForm.cognome).removeClass('input_error');
			break;
		case 'telefono':
		case 'telefonoFisso':
			jq(oForm.telefonoFisso).addClass('input_error');
			if(jq('#boxPrefTel').exist()){
				jq('#boxPrefTel').addClass('input_error');
				jq('.RU-errorPhone').css("height","21px").text("Inserisci solo caratteri numerici");
			}else{
				jq(oForm.telefonoFisso).prev().css("height","21px").text("Inserisci solo caratteri numerici");
			}
			break;
		case 'email':
			var text = (jq.isEmpty(text) ? "Inserisci una email valida" : text );
			var h_ = ((text.length) > 60) ? "40px" : "21px";
			jq(oForm.email).prev().css("height", h_).text( text );
			jq(oForm.email).addClass('input_error');
			break;
		case 'password':
			jq(oForm.password).prev().css("height","21px").text("Inserisci una password");
			jq(oForm.password).addClass('input_error');
			break;
		case 'privacy':
			jq(oForm.privacy).parent().prev('.error').css("height","21px").text('È necessario il consenso all\'informativa Privacy');
			break;
	}
}

function eseguiRegistrazione(oForm){
	var filter = /^(\+)?[\s\d()\-\/.]+$/;
	var error = false;
	var formType = jq(oForm).attr('data-type');

	cleanPlaceHolder( formType+'-form' );

	jq(".error").css("height","0px").text("");
	jq("input").removeClass('input_error');
	if(jq('#boxPrefTel').exist()){
		jq('#boxPrefTel').removeClass('input_error');
	}
	
	if ((!oForm.nome.value || oForm.nome.value=="Nome") && (!oForm.cognome.value || oForm.cognome.value=="Cognome")){
		errorMsgRegistrazione(oForm, 'nome-cognome');
		error = true;
	} else if(!oForm.nome.value || oForm.nome.value=="Nome"){
		errorMsgRegistrazione(oForm, 'nome');
		error = true;
	} else if(!oForm.cognome.value || oForm.cognome.value=="Cognome"){
		errorMsgRegistrazione(oForm, 'cognome');
		error = true;
	} else {
		errorMsgRegistrazione(oForm, 'noErrors-nomecognome');
	}
	if (oForm.telefonoFisso.value && !filter.test(oForm.telefonoFisso.value)){
		errorMsgRegistrazione(oForm, 'telefono');
		error = true;
	}
	if (!oForm.email.value || oForm.email.value=="Username" || !checkEmail(oForm.email.value)){
		errorMsgRegistrazione(oForm, 'email');
		error = true;
	}
	if (!oForm.password || !oForm.password.value || oForm.password.value=="Password"){
		errorMsgRegistrazione(oForm,'password');
		error = true;
	}
	if (!oForm.privacy.checked){
		errorMsgRegistrazione(oForm, 'privacy');
		error = true;
	}
	copy_fields( oForm );
	if (!error){
		submitAjaxForm(oForm, manageRegistrazioneUtente);
	}else{
        addPlaceHolderCompatibility(oForm.id)
    }
	return false;
}

function manageRegistrazioneUtente(response){
	var aResponse = jq.parseJSON( response.responseText );
	var formType = !jq.isEmpty( aResponse.type ) ? aResponse.type : "RU";
	if(1 == aResponse.registrazione){
		trackGAClickEvent("boxLoginEvents","accesso","RU");
		jq('#'+formType+'accediBox').css('width','450px');
		jq('#'+formType+'accediBox_content').css('padding','10px 20px');
		jq('#'+formType+'accediBox').addClass('registrazioneok');

		var contentHtml = "<div class=\"titleSuccess\"><h1>"+aResponse.msg+"</h1></div><p style=\"font-size:15px; line-height: 20px;\"><b style=\"padding: 5px 0 3px; display: block;\">Verifica la tua email</b>Clicca sul link nell'email che ti abbiamo inviato per convalidare la tua registrazione e accedere ai servizi</p><p><div id=\"btnProsegui\" onclick=\"reloadPastRegistrazione();\"></div></p>";

		selAccediBox.box.setTitle("<strong>Registrazione</strong>");
		selAccediBox.box.setContent( contentHtml );
		selAccediBox.box.setFooter("");
		selAccediBox.box.hideFooter();
		selAccediBox.box.addEventOnClose(function(){reloadPastRegistrazione()});
	}else{
		if(!jq.isEmpty(aResponse.msg)){
			var h_ = ((aResponse.msg.length) > 60) ? "40px" : "21px";
			jq('#errorMsg').css("height", h_).text( jq('<div/>').html(aResponse.msg).text() );
		}
		if(!jq.isEmpty(aResponse.formErrors)){
			var oForm = document.getElementById(formType+'-form');
			for (var k in aResponse.formErrors.errors){
				if (typeof aResponse.formErrors.errors[k] !== 'function') {
					errorMsgRegistrazione(oForm, k, jq('<div/>').html(aResponse.formErrors.errors[k]).text() );
				}
			}
		}
	}
}

function reloadPastRegistrazione(){
	var href, originalHref;
	href = originalHref = window.location.href;
	var isBackUrl = /(backurl=([\w\d\W]+))/.test(href);
	var aBackUrl = /(backurl=([\w\d\W]+))/.exec(href);
	var backUrl = (isBackUrl) ? aBackUrl[2] : '';

	if(!isBackUrl && (-1 != href.indexOf('accedi.php')))
		href = VHOST_URL_HOST + "index.php";

	href = (isBackUrl) ? VHOST_URL_HOST + backUrl : href;

	setTimeout(function(){ if(originalHref == href) window.location.reload(); else window.location.href = href; }, 1000);
}

function copy_fields( oForm ){
	oForm.password1.value = oForm.password.value;
	oForm.email1.value = oForm.email.value;
}

function addAutoCompleteDomainRU(){

	jq.cachedScript( "/includes/Im_Suggestion.js"+__gvs_EXTERNAL_LIBRARY_VERSION ).done(
		function( script, textStatus ) {
			var suggestion = new Im_Suggestion("usernameLoginInput",
				{
					selectRowOnOver       : false,
					hideOnMouseOut        : false,
					showEmptyResOnBlur    : false,
					blockOnBlur           : true,
					hideNoResultDiv       : true,
					activeTitle           : true,
					titleNoClick          : "Forse intendevi...",
					focusedElementOnSelect: null,
					minChars              : 2,
					maxResults            : 4,
					url                   : "/services/checkMailDomain.php?soglia=1000",
					onselect              : function (selectedItem) {},
					onsearch              : function (qs) {
						return checkEmail(qs) ? true : false;
					},
					autoselectFirstItem   : false,
					getResultOnFocus      : true,
					resetQsFieldOnNotSel  : false
				});

		}
	);

	if(jq('#userPhone').exist()){
		jq.cachedScript( "/includes/inserimento_annuncio_privati.js"+__gvs_EXTERNAL_LIBRARY_VERSION ).done(
			function( script, textStatus ) {
				var datiUtenteManager;
				var optDU = {
					_type					  : 'reg',
					btnIdVT_DU                : 'btnVerificaTelefonoDU',
					btnIdVMail_DU             : 'btnVerificaMailDU',
					btnResendLinkMail 		  : 'btnSendValidationLinkMail',
					activateValidazionePrivati : 1,
					txtIdPhone				  : 'userPhone',
					txtIdPhoneO				  : 'userPhone',
					rowPhoneData 			  : 'verificaTel',
					rowEmailData 			  : 'verificaMail',
					txtIdTestoVerifica 	      : 'testoVerifica',
					inputPrefix				  : 'phonePrefix',
					boxPrefixPhone			  : 'boxPrefTel',
					flag					  : '#containerFlag .flag',
					boxNations				  : 'prefixNations'
				};

				datiUtenteManager = new oDatiUtenteManager(optDU);

				if(!jq.isEmpty(datiUtenteManager) && jq('#'+optDU.boxPrefixPhone).exist()){
					datiUtenteManager.setHandlerPrefixPhone('#RU-form');
				}
			}
		);
	}

}
function recuperaPassword(oForm){

	var errorMsg = function(oForm, txt, showUsername){
		jq(oForm.email).prev().css("height","21px").text(txt);
		jq(oForm.email).addClass('input_error');
		if(showUsername)
			jq(oForm.username).addClass('input_error');
	}


	var error = false;
	jq(".error").css("height","0px").text("");
	jq("input").removeClass('input_error');
	var formType = jq(oForm).attr('data-type');

	cleanPlaceHolder(formType+'-form');

	if ((!oForm.email.value || oForm.email.value=="Email")&&(!oForm.username.value || oForm.username.value=="Username")){
		errorMsg(oForm, "Inserisci email o username", true);
		error = true;
	}else if (oForm.email.value && !checkEmail(oForm.email.value)){
		errorMsg(oForm, "Inserisci una email valida", false);
		error = true;
	}
	if (!error){
		if (document.getElementById('loaderLogin')){
			document.getElementById('loaderLogin').style.display="";
		}
		submitAjaxForm(oForm,function(response){

			jq('#'+formType+'accediBox_content').html(response.responseText);
			var isFormError = jq('#'+formType+'-form').attr('data-error');

			if(typeof isFormError == 'undefined')
				isFormError = 0;

			if(isFormError){
				/* Calcolo altezza del div che conterrà l'errore */
				var err_width = "307px";

				var errDivAppoggio = document.createElement("div");
				jq(document.body).append(errDivAppoggio);
				jq(errDivAppoggio).css("position","absolute").css("width",err_width).css("visibility","hidden").css("display","block").html(jq("#loginFailure").text());
				var err_height = (jq(errDivAppoggio).height())+10 + "px";
				//jq(errDivAppoggio).remove();

				/* Fine calcolo altezza del div che conterrà l'errore */
				jq("#loginFailure").animate({'height':err_height});
				jq("input[type='text'],input[type='email']","#"+formType+"accediBox_content").addClass('input_error');
				addPlaceHolderCompatibility(oForm.id);
				createAnimatedButton(formType+"btnOverlayBox","overlayBoxButtonWrap");
			}else{
				trackGAClickEvent("boxLoginEvents","accesso","RP");
				jq('#'+formType+'accediBox').addClass('recuperaok');
			}

		});
	}else{
        addPlaceHolderCompatibility(oForm.id)
    }
	return false;
}

function checkContratto(idContratto){
	document.getElementById('radio_idContratto'+idContratto).checked = 'checked';
	if (parseInt(idContratto) == 3){
		roomManager(fkCategoria,catStanze);
	}
}

function saveLastSearchCookie(categoria, opzioni, tipologia) {
	if (typeof opzioni == "undefined" || typeof categoria == "undefined"){
		_raiseError("called function with empty requested argument");		
	}
	try{
		var sCookie = readCookie('imm_lss1');
		var oCookie = {};

		if (sCookie){
			sCookie = Base64.decode(sCookie);
			oCookie = eval( "(" + sCookie + ")" );
		}

		if (opzioni.tipologia != null) {
			categoria = categoria + "_" + opzioni.tipologia;
		} else if (!jq.isEmpty(tipologia)) {
			opzioni.tipologia = tipologia;
		}

		oCookie[categoria] = opzioni;
		sCookie = Base64.encode(stringify(oCookie));
		createCookie("imm_lss1",sCookie,90,0,0);
	}catch(e){
		deleteCookie("imm_lss1");
		_raiseError(e);
	}
}
function cleanLastSearch(categoria){
	if (typeof categoria == "undefined"){
		_raiseError("called function with empty requested argument");		
	}
	try{
		var sCookie = readCookie('imm_lss1');
		var oCookie = new Object();
		if (sCookie){
			sCookie = Base64.decode(sCookie);
			oCookie = eval( "(" + sCookie + ")" );
		}
		oCookie[categoria] = null;
		sCookie = Base64.encode(stringify(oCookie));
		createCookie("imm_lss1",sCookie,90,0,0);
		
		manage_new_search_button("hide");
		__noSearchRestored = true;
	}catch(e){
		deleteCookie("imm_lss1");
		_raiseError(e);
	}
}

function getLastSearchCookie(categoria){
	deleteCookie("imm_lss");
	oCookie = null;
	try{
		var sCookie = readCookie('imm_lss1');
		var oCookie = null;
	
		if (sCookie){
			sCookie = Base64.decode(sCookie);
			oCookie = eval( "(" + sCookie + ")" );
		}
	}catch(e){
		deleteCookie("imm_lss1");
		_raiseError(e);
		
	}
	return oCookie;
}

function restoreLastSearchForm(categoria, tipologia, searchForm){
	__noSearchRestored = true;
	if ( typeof tipologia != 'undefined' && tipologia != null )
		categoria = categoria+"_"+tipologia;
	try{
		var oCookie = null;
		oCookie = getLastSearchCookie(categoria);
		if (oCookie && typeof oCookie == "object" && oCookie[categoria] && typeof oCookie[categoria] == "object"){
			if (document.getElementById("new_search"))
				document.getElementById("new_search").style.display = "block";

			if (typeof searchForm != "undefined" && searchForm != null && categoria == "annunci_turistici") {
				searchForm.setFromCookieData (oCookie);
			} else if (oCookie[categoria].provincia != "") {
				xajax_getAjaxProvinciaAndComuniPagIndex(oCookie[categoria].comune,oCookie[categoria].provincia, oCookie[categoria].regione, oCookie[categoria].contratto, stringify(oCookie[categoria]), categoria);
			}
			if (!jq.isEmpty(searchForm) && categoria == "annunci_aste") {
				jq(searchForm.categoria).val(oCookie["annunci_aste"].categoria);
			}
			__noSearchRestored = false;
		}else{
			show_search_box();
		}
	}catch(e){
		show_search_box();
		_raiseError(e);
	}
}

function show_search_box(){
	if (document.getElementById('box_ricerca_HP'))
		document.getElementById('box_ricerca_HP').style.display="";
	if (document.getElementById('box_ricerca_LOAD'))
		document.getElementById('box_ricerca_LOAD').style.display="none";
	if (typeof searchForm != "undefined" && searchForm) {
		searchForm.show();
	}		
}


function selectZoneHome(prov,sCookie){
		oCookie = eval("(" + sCookie + ")");
		zone_selected = new Array();
		if (oCookie.zone){
			zone_selected = new Array();
			for (var i = 0; i < oCookie.zone.length; i++){
				zone_selected.push(oCookie.zone[i]);
				swapZona(oCookie.zone[i]);
			}
			selectZona(prov);
		}
		if (oCookie.localita){
			zone_selected = new Array();
			for (var i = 0; i < oCookie.localita.length; i++){
				zone_selected.push(oCookie.localita[i]);
			}
			selectLocalita(prov);
			salvaZone('localita');
		}	
}


/*
questa funzione effettua il check se ci sono le condizioni per visualizzare la pagina in modalità estesa e quindi
eseguire la funzione specificata come parametro in caso positivo.
@fAction = funzione da eseguire in caso di test positivo
*/

function _ifWidePage(fAction){
	var __widepage = false;
	
	try{
		if (screen.width >= 1280){
			__widepage = true;
		}
	}catch(e){
		__widepage = false;
		
	}

	if (__widepage == true){
		if (typeof fAction == "function")
			fAction();
	}
	return __widepage;
}

function setPageH() {
	var nHeightToBottom = 0;
	
	var viewportHeight = jq( window ).height();
	var headerContainerHeight = jq( '#header_container' ).outerHeight( true );
	var corpoHeight = jq("#corpo").length > 0 ? jq( '#corpo' ).outerHeight( true ) : jq( '#contenuto_backoffice' ).outerHeight( true );
	var footerHeight = jq( '#footer' ).outerHeight( true ); 
	
	try {
		if ( viewportHeight && jq( '#header_container' ) && (jq( '#corpo' ) || jq('#contenuto_backoffice')) && jq( '#footer' ) )
			nHeightToBottom = ( ( viewportHeight ) - ( headerContainerHeight ) - ( corpoHeight ) - ( footerHeight ) );
		if ( nHeightToBottom > 0 ) {
			jq( '#spacerToFooter' ).css( 'height', nHeightToBottom + 'px' );
		} else {
			jq( '#spacerToFooter' ).css( 'height', '0px' );
		}
	} catch( error ) {
		;
	}
}
			
	/*		
function setPageW(){
	try{
		var cH = document.getElementById("corpo").getWidth();
		var wH = document.viewport.getWidth();
		//alert("wH:"+wH+" < cH:"+cH+" && wH:"+wH+"< 1024");
		if (wH <= cH && wH < 1000){
			document.getElementById("header_container").style.width = "1000px";
			document.getElementById("footer").style.width = "1000px";
			document.getElementById("corpo_container").style.width = "1000px";
			
		
		}else{
			document.getElementById("header_container").style.width = "auto";
			document.getElementById("footer").style.width = "auto";
			document.getElementById("corpo_container").style.width ="auto";
		
		}
	}catch(r){
	
	}
}
*/
function leoHeader()
{var bLoadLeoHdr = false;
	if (window.location.search.match('utm_source=leonardo&utm_medium=banner')) {
		bLoadLeoHdr = true;
		var dtNow = new Date();
		var dtExpires = new Date();
		dtExpires.setTime(dtNow.getTime() + 1200000 /* 20 minuti */);
		document.cookie = "leoHdr=true;expires=" + dtExpires.toGMTString() + ";path=/";
	}
	if (!bLoadLeoHdr) {
		var asCookies = document.cookie.split("; ");
		for (var nCnt = 0; nCnt < asCookies.length; nCnt++)
		{var asCookie = asCookies[nCnt].split("=");
			if (asCookie[0] == 'leoHdr' && unescape(asCookie[1]) == 'true') {
				bLoadLeoHdr = true;
				break;
			}
		}
	}
	if (bLoadLeoHdr) {
		document.write('<'+'script type="text/javascript" src="http://www.leonardo.it/script/Leonardo_Intruder.php?idEditore=157"'+'><'+'/script'+'>');
	}
}
			
function datiSoc(file,w,h) {
  /*var l = Math.floor((screen.width-w)/2);
  var t = Math.floor((screen.height-h)/2);
  window.open(file,"","width=" + w + ",height=" + h + ",top=" + t + ",left=" + l +",");
  */
	jq.ajax({
		url: file
	}).done(function(result) {

		var _datiSocmodalBox = new boxOverlay("_datiSocietariOv","grey");
		_datiSocmodalBox.destroyOnClose = true;
		_datiSocmodalBox.setTitle("Dati Societari Immobiliare.it S.p.A.");


		//_datiSocmodalBox.setContent(result);
		_datiSocmodalBox.setContent("<iframe frameborder='0' scrolling='no' marginheight='0' marginwidth='0' src='"+file+"' width='" + w + "' height='" + h + "'></iframe>");

		_datiSocmodalBox.show();
	});
}

function toggleVetrina(){
	var vetrina = document.getElementById('wrap_vetrina_ricerca_container');
	var btn = document.getElementById('toggleVetrina');
	if (vetrina.style.display == "none"){
		vetrina.style.display = "block";
		btn.innerHTML = "nascondi";
		btn.style.backgroundImage = "url(/img2/freccetta_up.png)";
	}else{
		btn.innerHTML = "mostra";
		vetrina.style.display = "none";
		btn.style.backgroundImage = "url(/img2/freccetta_down.png)";
	}
}

function signUpNews(){
	if (checkEmail(document.getElementById('newsLetter_email').value)){
		submitAjaxForm("newsletter_sign",confirmSignupNewsletter,"/signupNewsletter.php");
		document.getElementById('btn_news_letter_cont').innerHTML = "<img src=\"/img2/loader7.gif\" style=\"margin-top:4px\"/>";
	}else{
		alert("Digita una email valida");
	}
}


function confirmSignupNewsletter(res){
	signup_box = new boxOverlay('signupNewsLetter','grey');

	var splittedRes = res.responseText.split(",");
	
	var flag = typeof splittedRes[0] != "undefined" ? splittedRes[0] : "KO";
	var errcode = typeof splittedRes[1] != "undefined" ? splittedRes[1] : 0;
	var uemail = typeof splittedRes[2] != "undefined" ? splittedRes[2] : "";
	
	if (flag=="OK"){
		var response = "Iscrizione avvenuta con successo.";
		var message = "<div>Grazie per esserti iscritto alla newsletter di immobiliare.it.<br /><strong>Abbiamo inviato una mail all'indirizzo email \""+uemail+"\" contenente il link per confermare l'iscrizione</strong>.<br /><br />Riceverai le notizie più interessanti del mercato immobiliare, tutto ciò che è casa ed abitare, e infine le novità e le curiosità scovate dal nostro team editoriale, direttamente nella tua casella di posta.</div>";
	}else{
		var response = "<span style=\"margin:5px 0px;color:red\">Non è stato possibile effettuare l'iscrizione</span>";
		switch(parseInt(errcode)){
			case 1:
				var message = "<div>L'indirizzo email <strong>"+uemail+"</strong> risulta già iscritto alla newsletter di Immobiliare.it</div>";
				break;
			case 2:
				var message = "<div>L'indirizzo email inserito non è valido.</div>";
				break;
			default:
				var message = "<div>Si sono verificati degli errori durante la fase di iscrizione.<br />La preghiamo di riprovare più tardi</div>";
				break;
		}
	}
		signup_box.setContent("<div style=\"width:450px;padding:10px 0px;\"><img style=\"float:left\" src=\"/img2/icone/newsImmobiliare.png\" /><div style=\"float:right;width:350px;\"><div style=\"margin-bottom:5px;font-size:14px\"><strong style=\"margin-bottom:5px;\">"+response+"</strong></div><div class=\"clear\"></div><div style=\"font-size:12px\">"+message+"</div></div>");
		signup_box.setTitle("<strong>Newsletter Immobiliare.it </strong>&nbsp;");
		signup_box.show();
		
		/* reset form*/
		//document.getElementById('newsLetter_email').value="";
		//document.getElementById('newsLetter_email').onblur();
		document.getElementById('btn_news_letter_cont').innerHTML = "";
		createAnimatedButton("puls_freccetta","btn_news_letter_cont", signUpNews);
}


/*crea il boxettino per la conferma della cancellazione annunci pubblicati/preferiti/ricerche salvate relativa agli utenti registrati*/

/*
* sDivContainer: div a cui viene appeso il box overlay
* iWidth, iHeight: dimensioni in px del box
* iIdElToRemove: id annuncio pubblicato/id ricerca da cancellare
* sBookmark: il tipo di bookmark. pubblicati, preferiti, ricerche salvate
* sQueryString: una query string da passare alla location.
*/

function fnConfirmAction(sDivContainer, iWidth, iHeight, iIdElToRemove, sBookmark, sQueryString)
{
	if (typeof ___removeElId != "undefined")
		undoRemove(___removeElId);
	var oParentlayer = document.getElementById(sDivContainer);
	
		
	iHeight = MEgetHeight(oParentlayer);
	iWidth = MEgetWidth(oParentlayer) -2;
	
	var oConfirmLayer = document.createElement("div");
	oConfirmLayer.id="confirm_layer_"+iIdElToRemove;
	oConfirmLayer.style.backgroundColor="#aeaeae";
	oConfirmLayer.style.position="absolute";
	oConfirmLayer.style.filter="alpha(opacity = 80)";
	oConfirmLayer.style.opacity="0.8";
	
	oConfirmLayer.style.border="1px solid";
	oConfirmLayer.style.width=iWidth+"px";
	oConfirmLayer.style.height=iHeight+"px";
	oConfirmLayer.style.zIndex="140";
	
	var oConfirmControl = document.createElement("div");
	oConfirmControl.id = "controls_confirm_container_"+iIdElToRemove;
	oConfirmControl.style.filter="alpha(opacity = 100)";
	oConfirmControl.style.opacity = "1";
	oConfirmControl.style.zIndex = "150";
	oConfirmControl.style.width = iWidth+"px";
	oConfirmControl.style.height = iHeight+"px";
	
	oConfirmControl.style.position = "absolute";
	
	//dimensioni contenitore bottoni
	var iControlsDivWidth = 240; 
	var iControlsDivHeight = 35;
	
	var iControlsMargin =  Math.floor(((iHeight/2)-(iControlsDivHeight/2)))+"px "+" auto";
	
		
	var sHTMLControlContainer = "<div class=\"confirm_control\"></div>";
	
	var sHTMLButtons = "";
	if (sBookmark == 'preferiti' || sBookmark == 'ultimiAnnunci' || sBookmark == 'ultimiProgetti') {
		iControlsDivHeight = 70;
		iControlsDivWidth = 250;
		iControlsMargin =  Math.floor(((iHeight/2)-(iControlsDivHeight/2)))+"px "+" auto"
		sHTMLButtons += "<div class=\"rounded-all\" style=\"padding:5px 0px 5px 5px; background:#FFF;position:relative;opacity:1;filter:alpha(opacity = 100);z-index:150;width:"+iControlsDivWidth+"px;margin:"+iControlsMargin+"\">";
		sHTMLButtons += "<div style='float:left; font-size: 13px; margin-top:9px;margin-left:5px;'><strong>Sicuro?</strong></div>";
	} else {
		sHTMLButtons += "<div style=\"position:relative;opacity:1;filter:alpha(opacity = 100);z-index:150;width:"+iControlsDivWidth+"px;height:"+iControlsDivHeight+"px;background-image:url('/img2/fondo-box.png');margin:"+iControlsMargin+"\">";
	}
	sHTMLButtons += "<div class=\"bottoneAnnulla\" onclick=\"undoRemove('"+iIdElToRemove+"')\"></div>";
	
	switch(sBookmark){
		case('ultimiAnnunci'):
			sHTMLButtons += "<div class=\"bottoneCancella\" onclick=\"location.href='bookmarkannunci.php?showUltimiAnnunci=1&action=deleteUltimoAnnuncio&idAnnuncio=" + iIdElToRemove + "';\"></div>";
			break;
		case('ultimiProgetti'):
			sHTMLButtons += "<div class=\"bottoneCancella\" onclick=\"location.href='bookmarkannunci.php?showUltimiAnnunci=1&action=deleteUltimoAnnuncio&idProgetto=" + iIdElToRemove + "';\"></div>";
			break;
		case('pubblicati'):
			sHTMLButtons += "<div class=\"bottoneCancella\" onclick=\"location.href='pubblicaAnnunci.php?action=delete&id=" + iIdElToRemove + "';\"></div>";
			break;
		case('ricerche'):
			sHTMLButtons += "<div class=\"bottoneCancella\" onclick=\"location.href='bookmarkricerche.php?action=delete&idRicerca=" + iIdElToRemove + "';\"></div>";
			break;
		case('ultimeRicerche'):
			sHTMLButtons += "<div class=\"bottoneCancella\" onclick=\"location.href='bookmarkricerche.php?action=deleteLatest&idUltimaRicerca=" + iIdElToRemove + "';\"></div>";
			break;
		case('preferiti'):
			if (!jq.isEmpty(sQueryString)) {
				sQueryString += "&id=" + iIdElToRemove;
				sHTMLButtons += "<div class=\"bottoneCancella\" onclick=\"location.href='bookmarkannunci.php?" + sQueryString + "';\"></div>";
			} else {
				sHTMLButtons += "<div class=\"bottoneCancella\" onclick=\"location.href='bookmarkannunci.php?action=delete&id=" + iIdElToRemove + "';\"></div>";
			}
			break;
	}
	
	sHTMLButtons += "<div class=\"clear\"></div>";
	
	sHTMLButtons += "</div>";
	
	oConfirmControl.innerHTML = sHTMLButtons;
	
	oConfirmLayer.innerHTML = sHTMLControlContainer;
	
	
	oParentlayer.insertBefore(oConfirmLayer,oParentlayer.firstChild);
	
	//disattivo il rollover sul div padre
	//oConfirmLayer.parentNode.className += "_confirm_box";
	
	
	oParentlayer.insertBefore(oConfirmControl,oParentlayer.firstChild);
	
	___removeElId = iIdElToRemove;
	addEvent(document,"keyup", removeConfirmOnEsc);	
	
	return false;
}

function undoRemove(idElToRemove){
	if (typeof idElToRemove == "undefined" || !idElToRemove)
		return;
	fnRemoveNodeById("controls_confirm_container_"+idElToRemove);
	fnRemoveNodeById("confirm_layer_"+idElToRemove);
}

function removeConfirmOnEsc(e){
	
	var kC  = (window.event) ?    // MSIE or Firefox?
		   window.event.keyCode : (e.keyCode ? e.keyCode : e.which);
	var Esc = (window.event) ?   
		  27 : e.DOM_VK_ESCAPE; // MSIE : Firefox
	if(kC==Esc){
		undoRemove(___removeElId);
		removeEvent(document,"keyup", removeConfirmOnEsc);
	}
	
	
}

addEvent(window,'load',setPageH);
addEvent(window,'resize',setPageH);
//addEvent(window,'load',setPageW);
//addEvent(window,'resize',setPageW);
		
 
function createBannerAnnunciSimili(listaId) {
	var idElement = 'box_simili_top';
	var div = document.createElement("div");
	div.id = idElement;
	var parent = document.getElementById("banner_top_container");
	parent.appendChild(div);
	getBannerAnnunciSimili(idElement, listaId);
}

function getBannerAnnunciSimili(idElement, listaId){
	var ajReq = new getMEAjaxObj();
	ajReq.updateHTML("GET","/getAnnunciSimili.php?type=banner_top&section=dettaglio&ids="+listaId, idElement);
}

function togglePannelloSimili(idToggler, idElement)
{
	var oEl = document.getElementById(idElement);
	var oToggler = document.getElementById(idToggler);
	if (oEl && oToggler) {
		if (oEl.style.display=='none') {
			oEl.style.display = 'block';
			oToggler.className = 'icona_box_grigio riduci';
			oToggler.innerHTML = 'chiudi';
			oEl.parentNode.style.borderBottom = '1px none';
		} else {
			oEl.style.display = 'none';
			oToggler.className = 'icona_box_grigio espandi';
			oToggler.innerHTML = 'apri';
		}
	}
}

/**
 * Metodo che rimuove i box per le azioni dell'utente c
 * @returns {undefined}
 */
function removeUserAction() {
	jq( '#bookmarks_left' ).hide();
	jq( '#bookmarksCenter' ).hide();
	jq( '.btn_salva' ).each( function() {
		jq( this ).hide();
	});
}

var mario_i = 0;
function adattaBookmarks(forceSide)
{

	var attattaBookmarksOnScroll = function(){

		if (typeof addattaBookMarksTO != "undefined")
			clearTimeout(addattaBookMarksTO);

		addattaBookMarksTO = setTimeout(adattaBookmarks,0);

	};

	jq(window).unbind('scroll', attattaBookmarksOnScroll);

	if (typeof forceSide == "undefined")
		forceSide = null;
		
	if( typeof anteprimaPage != 'undefined' && anteprimaPage ) {
		removeUserAction();
		return true;
	}
		
    var windowWidth = window.innerWidth ? window.innerWidth : document.documentElement.clientWidth;
    var windowHeight = window.innerHeight ? window.innerHeight : document.documentElement.clientHeight;
	var divContenuto = document.getElementById('contenuto');

    var dimContenuto = MEgetWidth(divContenuto);

	var divRicercaRight = document.getElementById('ricerca_nc_right');
    if( sezioneBookmarks == 'ricerca' && divRicercaRight){
        var ricerca_nc_right= MEgetWidth(divRicercaRight);
        dimContenuto = ( parseInt(dimContenuto) + parseInt(ricerca_nc_right) ) + 20;
    }

    if ( parseInt(windowHeight) < 600 || ( parseInt(dimContenuto) + 98 ) > parseInt(windowWidth) || isIE6() || forceSide == "right") {
		if (document.getElementById('bookmarks_left'))
			document.getElementById('bookmarks_left').style.display = 'none';
		if (document.getElementById('bookmarksCenter'))
			document.getElementById('bookmarksCenter').style.display = 'block';

    } else {

		jq(window).on('scroll', attattaBookmarksOnScroll);


		if (document.getElementById('bookmarks_left'))
			document.getElementById('bookmarks_left').style.display = 'block';
		if (document.getElementById('bookmarksCenter'))
			document.getElementById('bookmarksCenter').style.display = 'none';

		if (windowWidth < 1650){
			jq('#bookmarks_left').addClass("pull-down");
			var scrolly = (document.all)?document.documentElement.scrollTop:window.pageYOffset;
			if (!!scrolly && scrolly >= 940){
				jq('#bookmarks_left').addClass("to-absolute")
			}else{
				jq('#bookmarks_left').removeClass("to-absolute")
			}

		}else{
			jq('#bookmarks_left').removeClass("pull-down");
		}


    }  
}

function slide(id, pos, speed, direction)
{
	div = document.getElementById(id);
	var left = parseInt(div.style.left) ? parseInt(div.style.left) : 0;
	
	if (direction == 'left') {
		if (left >= pos-speed){
			div.style.left = pos+"px";
		}else{
			left += speed;
			div.style.left = left+"px";
			slide_div = setTimeout("slide('"+id+"',"+pos+", "+speed+", '"+direction+"')", 0);
		}
	} else {
		if (left <= pos-speed){
			div.style.left = pos+"px";
		}else{
			left += speed;
			div.style.left = left+"px";
			slide_div = setTimeout("slide('"+id+"',"+pos+", "+speed+", '"+direction+"')", 0);
		}
	}
}

function slidePage(direction, idSlide, section, width)
{
	var speed = 30;
	var maxPage = getElementsByClass('box_'+section).length - 1;
	var lista = document.getElementById(idSlide);
	var offset = parseInt(lista.style.left) ? parseInt(lista.style.left) : 0;
	var currentPage = Math.abs(parseInt(offset / width));
	
	if (direction == 'next')
	{
		if (currentPage >= maxPage)
			return false;
		
		if (currentPage == 0)
			document.getElementById('prev_'+section).style.backgroundPosition = '0px -24px';
		
		if (currentPage+1 >= maxPage)
			document.getElementById('next_'+section).style.backgroundPosition = '0px 0px';
		
		var newPos = -1 * width * (currentPage+1);
		slide(idSlide, newPos, -speed, 'right');
	}
	else if (direction == 'prev')
	{
		if (currentPage <= 0)
			return false;
		
		if (currentPage == maxPage)
			document.getElementById('next_'+section).style.backgroundPosition = '0px -24px';

		if (currentPage-1 <= 0)
			document.getElementById('prev_'+section).style.backgroundPosition = '0px 0px';
		
		var newPos = -1 * width * (currentPage-1);
		slide(idSlide, newPos, speed, 'left');
	}
}


function clickFunction(idElement,e){
	var el = e.target ? e.target : e.srcElement;
	try{
		var elId = el.id;
		switch (elId){
			case "sito_"+idElement:
			case "lista_"+idElement:
			case "link_"+idElement:
				break;
			default:
				var oA = document.getElementById("link_"+idElement);
				if (!oA)
					return true;
				simulateClickOnLink(oA);
				break;
		}
	}catch(e){
		return true;
	}
	return false;
}

/******************************************************* CLASSE Im_ImgTracking PER IL TRACCIAMENTO ********************************************************/

var Im_ImgTracking = function(){
  this.tracker = new ImgTracking();
};

/**
 * Funzione che traccia il click sul un mostra telefono di un agenzia
 * @params string idItem -> nome dell'id o della classe su cui si vuole lavorare
 * @params string name -> da questa stringa si ricava se si vuole lavorare per id o per classe, e qualè sia il nome dell'id o della classe da recuperare
 * @params int idAgenzia -> id dell'agenzia per il tracciamento
 * @params fieldTrack -> tipo di campo telefono dell'agenzia da incrementare
 */
Im_ImgTracking.prototype.openContactAgency = function (idItem, name, idAgenzia, fieldTrack, event, opt) {
	var dataTracking = this.getTypeTracking(name);
	var managemenType = dataTracking['managemenType'];
	var nameType = dataTracking['nameType'];
	opt = opt || {};
	var link = '/stats/statsAgenzia.php?idAgenzia=' + idAgenzia + '&typeTrack=mostraTelefono&fieldTrack=' + fieldTrack;
	if (!jq.isEmpty(opt.source)) {
		link += '&source=' + opt.source;
	}

	trackGAClickEvent('ContattaInserzionistaEvents', fieldTrack, 'scopri');
	this.trackInvertVisibility(managemenType, nameType, idItem, link, event);
};

/**
 * Funzione che inverte la visibilita tra due div
 * @params string managemenType id -> cambia visibilita solo all'elemento cliccato, class -> la cambia a tutti gli elemtni con quella classe 
 * @params string nameType -> nome del div o della classe su cui lavorare
 * @params string idItem -> qualora managemenType si settato ad id allora il campo idItem accodato a nameType identifica il div univoco su cui lavorare
 * @params string actionUrl -> ulr da chiamare per tracciare l'evento
 */
Im_ImgTracking.prototype.trackInvertVisibility = function ( managemenType, nameType, idItem, actionUrl, event ) {
  if( managemenType == 'id' ) {
      document.getElementById( nameType+'_MT_'+idItem ).style.display = 'none';
      document.getElementById( nameType+'_'+idItem ).style.display = 'block';
    
  } else if( managemenType == 'class' ) {
    /* Gestione a classi apre tutti i numeri di telefono */
    var closeDiv = getElementsByClass( nameType+'_MT' );
    if ( closeDiv.length > 0 ) {
      for( var x=0; x < closeDiv.length ; x++ ) {
        closeDiv[x].style.display = 'none';
      }
    }
    var openDiv = getElementsByClass( nameType );  
    if ( openDiv.length > 0 ) {
      for( var x=0; x < openDiv.length ; x++ ) {
        openDiv[x].style.display = 'block';
      }
    } 
  } 
  this.tracker.trackEvent( actionUrl, event );
};

/**
 * Funzione che ricava se si dovra lavorare a classi o per id e il nome del relativo id o classe
 * @params string name -> da questa stringa si ricava se si vuole lavorare per id o per classe, e qualè sia il nome dell'id o della classe da recuperare
 */
Im_ImgTracking.prototype.getTypeTracking =  function ( name ) {
  var nameType = '';
  var managemenType = 'id';
  
  if( name.charAt( 0 ) == "#" ) {
    nameType =  name.split("#"); 
    nameType = nameType[1];
    
  } else if( name.charAt( 0 ) == "." ) {
    nameType =  name.split( "." ); 
    nameType = nameType[1];
    managemenType = 'class';
  
  } else {
    nameType = name;
  } 
  return {"nameType":nameType, "managemenType":managemenType};
};

im_imgTracking = new Im_ImgTracking();

/******************************************************* FINE CLASSE Im_ImgTracking PER IL TRACCIAMENTO ********************************************************/

function patchOpera() {
    /* Include il file di patch grafica se il broswer è opera */
    var userBrowser = getBrowser();
    if( userBrowser['name'].toLowerCase() == 'opera' )
        appendCss( VHOST_URL_HOST+'includes/operaPatch.css' );    
}

function changeClass( sender, myClass ) {
   if( typeof sender.toLowerCase() == "string" )
        document.getElementById(sender).className = myClass;
   else
        sender.className = myClass;
}



/************************************** Input Suggestion su Cool-out ***********************************************************/

var suggIndex= 0;
if (typeof oSuggPpFactory == "undefined" && typeof MEPopBoxFactory == 'function'){
	oSuggPpFactory = new MEPopBoxFactory();

}

/*
 * @params string options[afterMessageContent] -> Stringa HTML da inserire dopo il messaggio.
 * @params string options[boxOffsetLeft] -> Offset in pixel a sinistra del box.
 */
function showInputSuggestion(input,message,showCloseBtn,neverClose,forceSide,options,isPopup){
	var nomeClasse = '';
	var afterMessageContent = '';
	var boxOffsetLeft = null;
	var parentInputBlur = null;

	if ( typeof options != 'undefined' ) {
		if ( typeof options.nomeClasse != 'undefined' )
			nomeClasse = options.nomeClasse;
		if (typeof options.afterMessageContent != 'undefined')
			afterMessageContent = options.afterMessageContent;
		if (typeof options.boxOffsetLeft != 'undefined')
			boxOffsetLeft = options.boxOffsetLeft;
		if(typeof options.parentInputBlur != 'undefined')
			parentInputBlur = document.getElementById(options.parentInputBlur);
	}
	
	if (typeof neverClose == "undefined")
		neverClose = false;
	if (typeof forceSide == "undefined")
		forceSide = "right";
	
	if (typeof showCloseBtn == "undefined")
		showCloseBtn = false;

	var imgIconClassName = "";

	var inputClass = (typeof input.className != 'undefined') ? input.className : '';
	if ( inputClass.search(/error/)>=0)
		imgIconClassName = " err";

	if(typeof options != 'undefined' && typeof options.changeIcon != 'undefined' && options.changeIcon !== null)
		imgIconClassName += " "+options.changeIcon;
	
	var timeOpen = 1000;
	if(typeof options != 'undefined' && typeof options.openTime != 'undefined')
		timeOpen = options.openTime;
	
	var bCloseButton = (typeof options != 'undefined' && typeof options.closeButton != 'undefined') ? options.closeButton : false;

	var htmlCont = "<div class=\"popupSugg "+nomeClasse+"\">"+
		"<div class=\"icon"+imgIconClassName+"\"></div>"+
		"<div class=\"text\">"+message+"</div>"+afterMessageContent;
		
	if(bCloseButton)
		htmlCont += "<div class=\"MEPB_close buttonCloseLight\"><a href=\"javascript:\">Chiudi</a></div>";

	htmlCont += "<div class=\"clear\"></div>"+
				"</div>";
		
	var postFix = "";
	
	if (typeof showCloseBtn != "undefined" && showCloseBtn){
		postFix = "_1";
	}
	
	suggPopBox = oSuggPpFactory.getPopBox("suggPopBox"+postFix,input);
	suggPopBox.reset ();
	suggPopBox.hideOnEsc = true;
	
	if (forceSide == "left"){
		suggPopBox.forcePosition("left");
		//suggPopBox.forceToLeft = true;
		//suggPopBox.forceToRight = false;
	}
	else{
		suggPopBox.forcePosition("right");
		//suggPopBox.forceToLeft = false;
		//suggPopBox.forceToRight = true;
	}
		
		
	if (showCloseBtn && !neverClose){
		suggPopBox.hideClose = false;	
	}
	else{
		suggPopBox.hideClose = true;
	}
	
	if (neverClose){
		suggPopBox.hideOnEsc = false;
	}

	if (boxOffsetLeft != null) {
		suggPopBox.boxOffsetLeft = boxOffsetLeft;
	}

	suggPopBox.destroyOnClose = true;
	suggPopBox.setContent(htmlCont);
	suggPopBox.timeout = setTimeout(function(){suggPopBox.show();}, timeOpen);
	if (input.id == "not_valid_email"){
		if (typeof _checkValidationCookieInt == "undefined" || !_checkValidationCookieInt)
			_checkValidationCookieInt = setInterval(checkForValidatedEmail,3000);
		
		suggPopBox.addEventOnClose(function(){
			if (typeof _checkValidationCookieInt != "undefined")
				clearInterval(_checkValidationCookieInt);
				_checkValidationCookieInt = false;
			}
		);
	}	
	if (!neverClose){
		addEvent(input, "blur",function(){hideInputSugg(this);}.bind(suggPopBox));
		if(parentInputBlur)
			addEvent(parentInputBlur, "blur",function(){hideInputSugg(this);}.bind(suggPopBox));
	}

	//se è su un popup metto position: fixed e z-index > 300
	if (isPopup)
		suggPopBox.renderVisibleOnPopup();
	
	return suggPopBox;
}

function hideInputSugg(box){
	if (typeof box=="undefined" || !box)
		box = this;
		
	if (box) {
		clearTimeout (box.timeout);
		setTimeout(function(){try{box.hide();}catch(e){}},0);
	}
}

/******************* DETTAGLIO AGENZIA ************************************/
/**
 * Apertura popup orario in dettaglio agenzia
 */
function showOrarioApertura(input,message){
	var nomeClasse = " divorario";
    var imgIconClassName = " orario";
    
	var htmlCont = "<div class=\"popupSugg"+nomeClasse+"\"><div class=\"icon icon"+imgIconClassName+"\">Orari apertura</div><div class=\"text\">"+message+"</div><div class=\"clear\"></div></div>";
	
	suggPopBox = oSuggPpFactory.getPopBox("suggPopBoxOrarioAg",input);
	suggPopBox.hideOnEsc = true;
	suggPopBox.forceToRight = true;	
	suggPopBox.hideClose = false;	
	
	//suggPopBox.destroyOnClose = true;
	suggPopBox.setContent(htmlCont);
	setTimeout(function(){suggPopBox.show();},0);
	
	/*if (!neverClose)
		addEvent(input, "blur",function(){hideInputSugg(this)}.bind(suggPopBox));
*/
}
/**
 * Classe che gestisce lo switch(di visibilita') tra due elementi chiamati master e slave.
 */
var oMultimediaSwitchToggle = function( options ){
	this.plsId = (typeof(options.idpls) != 'undefined') ? options.idpls : "multimediaPlsId";
	this.elMaster = (typeof(options.containermaster) != 'undefined') ? options.containermaster : "idmaster";
	this.elSlave = (typeof(options.containerslave) != 'undefined') ? options.containerslave : "idslave";
	this.plsClass = (typeof(options.classpls) != 'undefined') ? options.classpls : ".multimediaPlsCls";
	this.containerId = (typeof(options.container) != 'undefined') ? options.container : "multimedia";
	this.multiContainer = (typeof(options.multicontainer) != 'undefined') ? options.multicontainer : "multi_container";
	this.type = (typeof(options.type) != 'undefined') ? options.type : 1;
	this.adjustHeightContainer = (typeof(options.adjustheight) != 'undefined') ? options.adjustheight : false;
	
	this._jqPlsId = jq('#'+this.plsId);
	this._jqPlsClass = jq( this.plsClass );
	this._jqContainer = jq('#'+this.containerId);
	this._jqMultiContainer = jq('#'+this.multiContainer);
	this.a_jqElement = { "master": jq('#'+this.elMaster) , "slave": jq('#'+this.elSlave) };
	this.aTxtActive = { "master":"Mostra video", "slave":"Mostra foto" };

	/* lo slave inizialmente display none */
	this.a_jqElement['slave'].css('display','none');
	/* setto data del pulsante di switch, inizialmente master e aggiorno testo pulsante */
	this.setActive('master');
	this.attachListnerEvent();
};
oMultimediaSwitchToggle.prototype.attachListnerEvent = function(){
	var oMulti = this;
	
	if(1 == this.type){
		this._jqPlsId.on("click", function(event){
			//loader
			oMulti.showLoader();
			var sOldActive = jq.data(oMulti._jqPlsId, 'active');
			//change data
			var sNewActive = (sOldActive == 'master') ? 'slave' : 'master';
			oMulti.setActive(sNewActive);
			//animation
			oMulti.a_jqElement[sOldActive].fadeToggle( 600, 'swing', function(){ 
				oMulti.a_jqElement[sNewActive].fadeToggle( 300, 'swing', oMulti.hideLoader() ) ;
			} );
		});
		//HIDE e remove type2 pulsantiera
		jq('#type2').css('display','none').remove();
	}else if(2 == this.type){
		this._jqPlsClass.on("click", { multi: oMulti }, oMulti.oneClick);
		//HIDE e remove type1 pulsantiera
		jq('#type1').css('display','none').remove();
	}
};

oMultimediaSwitchToggle.prototype.oneClick = function(event){
	var btn = event.target;
	var multiManager = event.data.multi;
	
	if( jq( btn ).hasClass('selected') )
		return false;
	
	multiManager.showLoader();

	var sNewActive = jq( btn ).attr('data-show');
	var sibling = jq( btn ).siblings();
	var sOldActive = jq( sibling ).attr('data-show');

	multiManager.setActive(sNewActive);
	jq( multiManager.plsClass+"[data-show='"+sOldActive+"']" ).removeClass('selected');	
	//animation
	multiManager.a_jqElement[sOldActive].fadeToggle( 600, 'swing', function(){ 
		multiManager.a_jqElement[sNewActive].fadeToggle( 300, 'swing', multiManager.hideLoader() ) ;
	} );
};

oMultimediaSwitchToggle.prototype.setActive = function(sActive){
	jq.data(this._jqPlsId, 'active', sActive);
	this._jqPlsId.text( this.aTxtActive[sActive] );
	
	if(2 == this.type){
		jq( this.plsClass+"[data-show='"+sActive+"']" ).addClass('selected');	
	}
};
oMultimediaSwitchToggle.prototype.hideLoader = function(){
	var oMulti = this;
	
	/* height multimedia adjust */
	if(this.adjustHeightContainer){
		var multiHeight = this._jqContainer.height();
		var paddingMulti = this._jqContainer.outerHeight(true) - multiHeight;
		var elHeight = this.a_jqElement[ jq.data(this._jqPlsId, 'active') ].height();
		var diff = Math.round( (multiHeight - elHeight) / 2 );
		(multiHeight > elHeight) ? this._jqContainer.height( multiHeight - diff) : this._jqContainer.height( elHeight + paddingMulti + diff);
	}
	
	setTimeout(function(){ oMulti._jqMultiContainer.removeClass('loader'); }, 900);
};
oMultimediaSwitchToggle.prototype.showLoader = function(){
	this._jqMultiContainer.addClass('loader');
};
/************************************************************************/
/**
* Classe che gestisce la verifica sui domini email
*/
var oEmailDomaniVerification = function( options ){
	//utility
	__DOT = '.';
	__SHARP = '#';
	__R_MOUSEIN = /^mouse(enter|over)/i;

	this.action = (typeof(options._action) != 'undefined') ? options._action : "reg";
	this.emailId = (typeof(options.emailid) != 'undefined') ? options.emailid : "emailRegistrazione";
	this.repemailId = (typeof(options.repemailid) != 'undefined') ? options.repemailid : "emailRegRepeat";
	this.btnSubmitContainer = (typeof(options.btnSubmitContId) != 'undefined') ? options.btnSubmitContId : "bottoneRegistratiContainer";
	this.aBtnSubmit = (typeof(options.btnSubmit) != 'undefined') ? options.btnSubmit : {"r":"btnRegistrati", "a": "btnAggiorna"};

	this.fieldMail = (typeof(options.fieldMail) != 'undefined') ? jq(options.fieldMail) : jq('<input type="text" maxlength="100" name="email" value="" />');
	this.fieldMail.attr('id', this.emailId);
	this.conteinerFieldMail = (typeof(options.containerMailField) != 'undefined') ? jq(__SHARP+options.containerMailField) : jq('#boxContainerMail');

	this.txtPopupFieldMail = (typeof(options.txtPopupMail) != 'undefined') ? options.txtPopupMail : "";
	this.classFieldMail = (typeof(options.clsInputMail) != 'undefined') ? options.clsInputMail : "";
	this.directionPopup = (typeof(options.dirPopup) != 'undefined') ? options.dirPopup : null;

	this._jqEmailId = jq(__SHARP+this.emailId);
	this._jqEmailRepeatId = jq(__SHARP+this.repemailId);
	this._jqSuggestionBox = jq( this.suggestionBox );
	this._jqBtnSubCont = jq(__SHARP+this.btnSubmitContainer);
	this._jqBtnSubmit = (this.action != 'update') ? jq(__SHARP+this.aBtnSubmit["r"]) : jq(__SHARP+this.aBtnSubmit["a"]);

	this.aSuggestionOptions = (typeof(options.suggestOptions) != 'undefined') ? options.suggestOptions : {"divContainerId":"resultVerificationMail","textContainerClass":"text","suggestedMailClass":"suggestedMail"};
	this.aTxtSuggestion = { "f":"Forse intendevi: ", "s":"Non hai inserito un indirizzo email valido!" };

	this.urlVerification = '/services/checkMailDomain.php';
	this.dataVerification = {"soglia": 1000,"maxResult": 5, "mail":""};

	//add mail fild to form
	this.conteinerFieldMail.append(this.fieldMail);
	if(!jq.isEmpty(this.classFieldMail))
		jq(this.fieldMail).addClass( this.classFieldMail );

	/* attachEvent */
	this.attachListnerEvent();
};
oEmailDomaniVerification.prototype.attachListnerEvent = function(){
	var oEDV = this;

	/** var delay su keyup */
	var beLate = (function(){
		var timer_ = 0;
		return function(callback, ms){
			clearTimeout (timer_);
			timer_ = setTimeout(callback, ms);
		};
	})();

	this.fieldMail.on('keyup',function() {
		beLate(function(){

			var contentMail = oEDV.fieldMail.val();
			if(isEmail(contentMail)){
				oEDV.checkCorrectEmail( false );
			}
		}, 500 );
	});


	this.fieldMail.on('blur', function (ev) {
		var containerSuggest = jq('#' + oEDV.aSuggestionOptions['divContainerId']);
		//se non ho risultati, e per caso il suggerimento è ancora aperto...
		if (jq.isEmpty(containerSuggest) || containerSuggest.is(":hidden"))
			oEDV.checkCorrectEmail(true);
	});

	this.fieldMail.on('focus', function(){
		showInputSuggestion(this, oEDV.txtPopupFieldMail,null,null, oEDV.directionPopup);
	});
};

oEmailDomaniVerification.prototype.checkCorrectEmail = function( closeSuggestDiv ){
	var oEDV = this;
	var closeSuggest = (typeof closeSuggestDiv == 'undefined') ? true : closeSuggestDiv;


	var contentMail = oEDV.fieldMail.val();
	if(isEmail(contentMail)){
		oEDV.dataVerification['mail'] = contentMail;

		var rMailVerification = jq.getJSON( oEDV.urlVerification, oEDV.dataVerification )
			.done(function( data ) {
				oEDV.manageResult( data, closeSuggest );
			})
			.fail(function() {
				_doLog("Check emailVerification class!");
			})
			.always(function() {
				// faccio partire il timeout/
			});
	}else if(!jq.isEmpty( contentMail )){
		oEDV.errorMail();
	}
};

oEmailDomaniVerification.prototype.attachListenerSuggestedMail = function(){
	var oEDV = this;

	jq('.'+this.aSuggestionOptions['suggestedMailClass']).on('click', function(){
		var mInsert = jq( this ).attr('data-email');
		var containerSuggest = jq(__SHARP+oEDV.aSuggestionOptions['divContainerId']);
		if(!jq.isEmpty(mInsert)){
			oEDV.fieldMail.val( mInsert );

			if(!jq.isEmpty(oEDV._jqEmailRepeatId)){
				oEDV._jqEmailRepeatId.val('');
				oEDV._jqEmailRepeatId.focus();
			}

			if(!jq.isEmpty( containerSuggest ) && containerSuggest.is(":visible"))
			 	containerSuggest.fadeOut(300);
		}
	});
};

oEmailDomaniVerification.prototype.manageResult = function( data, closeSuggest ){
	var closeSuggest = (typeof closeSuggest == 'undefined') ? true : closeSuggest;

	var oEDV = this;
	if(!jq.isEmpty(data) && (data.results.length > 0)){

		var txt = "<strong style='color:#CF0103;'>"+this.aTxtSuggestion['f']+"</strong>";
		var suggMail = [];

		jq.each( data.results, function( i, item ) {
			suggMail.push("<a href='#' class='"+oEDV.aSuggestionOptions['suggestedMailClass']+"' data-email='"+item.row_label+"'>"+item.row_label+"</a>");
		});

		txt = txt + suggMail.join(", ");

		jq(__SHARP+this.aSuggestionOptions['divContainerId']+' td' + __DOT + this.aSuggestionOptions['textContainerClass']).html( txt );
		jq(__SHARP+this.aSuggestionOptions['divContainerId']).fadeIn(300);

		if(closeSuggest){
			setTimeout(function(){
				oEDV.closeContainer();
			}, 8000);
		}

		this.attachListenerSuggestedMail();
	}else{
		this.closeContainer();
	}
};

oEmailDomaniVerification.prototype.errorMail = function( data ){
	var oEDV = this;

	setTimeout(function(){
		oEDV.fieldMail.val('');
	}, 500);

	if(!jq.isEmpty(this._jqEmailRepeatId))
		this._jqEmailRepeatId.val('');

	var txt = "<strong style='color:#CF0103;'>"+this.aTxtSuggestion['s']+"</strong>";
	jq(__SHARP+this.aSuggestionOptions['divContainerId']+' td'+ __DOT +this.aSuggestionOptions['textContainerClass']).html( txt );
	jq(__SHARP+this.aSuggestionOptions['divContainerId']).fadeIn(300);

	this.fieldMail.focus();

	setTimeout(function(){
		oEDV.closeContainer();
	}, 3000);
};

oEmailDomaniVerification.prototype.closeContainer = function( ){

	var containerSuggest = jq('#'+this.aSuggestionOptions['divContainerId']);
	//se non ho risultati, e per caso il suggerimento è ancora aperto...
	if(!jq.isEmpty( containerSuggest ) && containerSuggest.is(":visible"))
		containerSuggest.fadeOut(300);

};

/***********************************************************************/
function goToFormFirstError(){
	
	var errs = getElementsByClass("input_error");
	var errsLab = getElementsByClass("errore_inserimento");
	if (errs.length){
		var first_error = errs[0];
		var pos = getPosition(first_error);
		if (pos.y > 50)
			scrollTo(0,(pos.y - 250));

		for (var i=0;i<errs.length;i++){
			errs[i].label = errsLab[i];
			//addEvent(errs[i],"blur",manageErrorLabels.bind(errs[i]));
		}
		
		errs[0].focus();
	}
	
}
/************************************** Fine Input Suggestion su Cool-out ***********************************************************/


function cleanInput(oInput){
	if (oInput.value == oInput.defaultValue){
		oInput.value = "";
		oInput.style.color="#000";
	}
	oInput.onblur = function(){resetDefault(oInput);};
}

function resetDefault(oInput){
	
	if (oInput.value ==""){
		oInput.value = oInput.defaultValue;
		oInput.style.color="#888";
	}
}

/*** OPEN SPOT TV **/
function openPubblicitaTv(baseurl){
    var box = new boxOverlay("box_pubblicitatv","grey");
    
    objPubblicita = new Object;
    objPubblicita.frameVideo = baseurl+"spotTv.php";
    objPubblicita.title = "Video Spot Immobiliare.it";
    objPubblicita.content = "<div style='margin:5px'><iframe src="+objPubblicita.frameVideo+" width=\"631\" height=\"460px\" marginheight=\"0\" marginwidth=\"0\" frameborder=\"0\" scrolling=\"no\" style=\"margin-top: 0px; margin-right: 0px; margin-bottom: 0px; margin-left: 0px; display: block; \"></iframe></div>";

    box.stop_event_bubbling = true;
    box.setTitle(objPubblicita.title);
    box.setContent(objPubblicita.content);
    box.initContent = box.getContent();
    box.show();

}

function fnAutosaveAds(autosaveValue)
{
	var objAutosaveFacebook = new getMEAjaxObj();
	objAutosaveFacebook.Request("GET", "/manage_facebook_user.php?action=autosave&autosave="+autosaveValue, function(){});
}

/*---------------------------------------------------------------------------------*
 * funzione che inizializza i pulsanti per la creazione della mappa in home page.
 * --------------------------------------------------------------------------------*/

function initGoogleMapRicercaArea( section, adminLevel, options ) {
	if (typeof options == "undefined") {
		options = new Array();
	}
	
	if (typeof boxPreviewAreaMap != "undefined")
		boxPreviewAreaMap.hide();
	
	if ( typeof section == "undefined" )
		section = '';
	
	if (section == 'homepage')
		createMapOverlay('CartinaGrande');
	
	var map_options =
		{'width':900,'height':500,'zoom':15,'markerType':0,'radius':0,'showMarker':0,'drawPolygon':1,'hasPdi':0,
			'personalizedControl':0,'maximizeDimension':1,'mapControls':0,'adminLevel':adminLevel,'section':section
		};

	if (typeof options != "undefined")
		map_options = jq.extend( {}, map_options, options );

	var __createMapHandler = function() {
		createDinamicGoogleMaps('', map_options, 'Disegna l\'area d\'interesse');
	};

	if ( section == 'alertemail' ) {
		var __openMapHandler = function(opt) {
			if ( jq("#gm_area_TI").val() != '') {
				forceTypeSearch('area', '_TI');
				jq('#zone_cartina_TI').hide();
				jq('#overlay_opacity_map_TI').show();
				jq("#tipoRicerca_TI").val('mappa');
			} else {
				createDinamicGoogleMaps('', opt, 'Disegna l\'area d\'interesse');
			}
		};
		var __deletePolyHandler = function() {
				jq('#zone_cartina_TI').show();
				jq('#overlay_opacity_map_TI').hide();
				jq("#tipoRicerca_TI").val('');
				var isCittaConZona = jq("#cittaConZona").val();
				var isEnabledZone = jq("#enabledZone").val();
				var selectedZone = jq("#zoneTI_ids").val();
				if (parseInt(isCittaConZona) && parseInt(isEnabledZone) && (selectedZone == '0' || selectedZone == '')) {
					jq('#modifica_zona_TI').click();
				}
			};
		jq('#modifica_area_TI').unbind('click').click(__createMapHandler);
		jq('#overlay_opacity_map_TI').unbind('click').click(__createMapHandler);
		jq('#btn_area_TI').unbind('click').click( function(){ __openMapHandler(map_options); } );
		jq('#btn_zone_TI').unbind('click').click(__deletePolyHandler).click( function(event) { switchBtnClass(this, '_TI'); } );
	} else {
		jq('.ui-radio input[type|="radio"]').click(function(){
			if ( jq(this).hasClass('ui-icon-radio-on') )
				jq(this).attr('checked', 'checked');
			else
				jq(this).removeAttr('checked');
		});
		
		var __deletePolyHandler = function() {
				jq('#zone_cartina').show();
				jq('#area_mappa').hide();
				jq('#overlay_opacity_map').hide();
				jq("#tipoRicerca").val('');
			};
		
		jq(".ui-radio div").mouseenter( function() {
			jq(this).removeClass('ui-btn-up-c');
			jq(this).addClass('ui-btn-hover-c');
		}).mouseleave( function() {
			jq(this).removeClass('ui-btn-hover-c');
			jq(this).addClass('ui-btn-up-c');
		});
		
		var isCittaConZona = typeof jq("#infoComuneArea").val() != "undefined" && jq("#infoComuneArea").val() == 'zona';
		var isCittaConLocalita = typeof jq("#infoComuneArea").val() != "undefined" && jq("#infoComuneArea").val() == 'localita';
		var isEnabledZone = jq("#enabledZone").val();
		
		if ( section == 'homepage' || section == 'ricerca' ) {
			jq('#modifica_zona').bind('click').click(function(event) {
				event.stopPropagation();
				manageSelectArea(section, true);
			});
			
			var __openMapHandler = function() {
				
				var input = jq(this).find("input").get(0);
				var selected = input.checked;
				
				manageSelectArea(section, false);				
				if ( !selected && ( typeof jq("#gm_area").val() != "undefined" && jq("#gm_area").val() != '' ) ) {
					forceTypeSearch('area');
					jq('#overlay_opacity_map').show();
					jq("#tipoRicerca").val('mappa');
					jq('#modifica_area').show();
				} else {
					createDinamicGoogleMaps('', map_options, 'Disegna l\'area d\'interesse');
				}
				jq('#modifica_zona').hide();
			};
			
			var __deletePolyHandler = function() {
				jq('#modifica_area').hide();
				jq('#overlay_opacity_map').hide();
				jq("#tipoRicerca").val('');
				
				var input = jq(this).find("input[type=radio]").get(0);				
				var selected = input.checked;
				if (isCittaConZona) {
					var selectedZone = jq("#zone_descrizione").val();
					selectedZone = jq.trim(selectedZone);
					if (selected ||  (isEnabledZone && ( selectedZone.match(/Scegli/g) != null || selectedZone.match(/Tutte/g) != null)) ) {
						jq('#modifica_zona').click();
					} else {
						forceTypeSearch('zona');
						jq('#modifica_zona').show();
					}
				} else if (isCittaConLocalita) {
					var selectedLoc = jq("#localita_descrizione").val();
					selectedLoc = jq.trim(selectedLoc);
					if (selected || selectedLoc.match(/Scegli/g) != null || selectedLoc.match(/Tutte/g) != null) {
						jq('#modifica_zona').click();
					} else {
						forceTypeSearch('zona');
						jq('#modifica_zona').show();
					}
				} else {
					forceTypeSearch('zona');
				}
			};

			jq('#overlay_opacity_map').click(__createMapHandler);
			jq('#btn_area').bind('click').click(__openMapHandler);
			jq('#btn_zone').bind('click').click(__deletePolyHandler);
		}
		
		jq('#ricerca_area').click(__openMapHandler);
		jq('#delete_area').click(__deletePolyHandler);
		jq('#area_testo').click(__createMapHandler);
	}
}

function createMapOverlay(idContent) {
	jq('#'+idContent).append("<div id=\"overlay_opacity_map\" class='overlay_opacity_map_hp' style='display:none;'></div>");
}

function generateUrlPolygonArea(mode, area, google_key, width, height) {
	if (typeof width == "undefined")
		width = 260;
	if (typeof height == "undefined")
		height = 260;
	
	var imgUrl = '';
	switch( mode ) {
		case 'circle':
			var radius = area.radius;
			var center = area.center;
			var lat = area.center[0];
			var lng = area.center[1];
			var options = {'width':width, 'height':height, 'markerType':2, 'radius':radius, 'enableControl':0, 'enableFunction':0, 'style': 'feature:road|element:labels|visibility:off', 'fillColor':'0x005A9B44', 'fillOpacity':'44', 'strokeColor':'0x0E4E73FF', 'strokeOpacity':'FF', 'strokeWeight':2};
			var oStaticMap = new static_GMap('', lat, lng, 0, google_key, options);
			imgUrl = oStaticMap.getUrlStaticMap(false);
			break;
		case 'polygon':
			if (area.length)
				area.push(area[0]);
			var numPoints = null;
			if (typeof __gvs_MAX_STATIC_POLYGON_THUMB_PREVIEW_POINTS != "undefined" && __gvs_MAX_STATIC_POLYGON_THUMB_PREVIEW_POINTS > 0)
				numPoints = __gvs_MAX_STATIC_POLYGON_THUMB_PREVIEW_POINTS;
			var options = {'width':width, 'height':height, 'markerType':3, 'path':area, 'enableControl':0, 'enableFunction':0, 'style': 'feature:road|element:labels|visibility:off', 'fillColor':'0x005A9B44', 'fillOpacity':'44', 'strokeColor':'0x0E4E73FF', 'strokeOpacity':'FF', 'strokeWeight':2, 'numPoints':numPoints};
			var oStaticMap = new static_GMap('', 0, 0, 0, google_key, options);
			imgUrl = oStaticMap.getUrlStaticMap(false);
			break;
	}
	
	return imgUrl;
}

function trackStaticMapLoad(section, imgUrl) {
	var rand = Math.random(),
        service,
        hash,
        imgTrackPixel,
        env;

    if (window.location.hostname.search('dev\.loc') >= 0) {
        env = 'dev';
    } else if (window.location.hostname.search('staging') >= 0) {
        env = 'staging';
    } else {
        env = 'prod';
    }

	if (imgUrl && imgUrl.indexOf('maps.googleapis.com') !== -1) {
		service = 'google';
	} else {
		service = 'immobiliare';
	}

    if (section == 'trova_immobili') {
		if (service === 'google') {
			hash = '6ee693';
		} else {
			hash = '6a085c';
		}

		imgTrackPixel = '//s.ekbl.net/s.gif?m=' + env + '.sites.immobiliare.map.static.' + service + '.trova_immobili&h=' + hash + '&t=_&c=' + ((rand + '').substring(2));
	} else {
		if (service === 'google') {
			hash = 'ddb1a9';
		} else {
			hash = '5966de';
		}

		imgTrackPixel = '//s.ekbl.net/s.gif?m=' + env + '.sites.immobiliare.map.static.' + service + '.disegna_mappa&h=' + hash + '&t=_&c=' + ((rand + '').substring(2));
	}

    jQuery('<img>').attr('src', imgTrackPixel);
}

function staticMapHP(oDivContainer, mode, area, google_key, sufix, width, height) {
	if (typeof width === 'undefined') {
		width = 260;
	}
	if (typeof height === 'undefined') {
		height = 260;
	}

	if (typeof area !== 'object' || !oDivContainer) {
		return false;
	}

	if (typeof sufix === 'undefined') {
		sufix = '';
	}

	jq(oDivContainer)
		.empty()
		.append(
			jq('<img/>')
				.attr('class', 'google_map_loader')
				.attr('id', 'img_google_map_loader' + sufix)
				.attr('src', '/img2/loader6.gif')
		);
	
	var imgUrl = generateUrlPolygonArea(mode, area, google_key, width, height);

	if (imgUrl != '') {
		jq(oDivContainer).append(
			jq("<img/>")
				.attr('style', 'display: none;')
				.attr('class', 'google_map_static')
				.attr('id', 'img_google_map_statics' + sufix)
		);

		jq('#img_google_map_statics' + sufix)
			.load(function () {
				jq('#img_google_map_loader' + sufix).fadeOut('slow', function () {
					jq("#img_google_map_statics" + sufix).fadeIn('slow');
				});
			})
			.attr('src', imgUrl);

		if (typeof trackGAClickEvent === 'function') {
			trackGAClickEvent(sufix === '' ? 'ricercaAnnunciEvents' : 'trovaImmobiliEvents', 'mappa_google', 'impressions_static');
		}

		trackStaticMapLoad(sufix === '' ? 'disegna_mappa' : 'trova_immobili', imgUrl);

		return true;
	}
	
	return false;
}

function staticMapList(oDivContainer, mode, area, google_key) {
	if (typeof area !== "object")
		return false;
	
	if (!oDivContainer)
		return false;
	
	jq(oDivContainer).empty();
	
	var imgUrl = generateUrlPolygonArea(mode, area, google_key);
	
	if (imgUrl != '') {
		jq(oDivContainer).append(
			jq('<img/>').attr('id', 'img_google_map_statics').attr('src', imgUrl)
		);

		if (typeof trackGAClickEvent === 'function') {
			trackGAClickEvent('ricercaAnnunciEvents', 'mappa_google', 'impressions_static');
		}

		trackStaticMapLoad('disegna_mappa', imgUrl);

		return true;
	}
	
	return false;
}

function resetDrawingMap(scope, idProvincia, idComune) {
	if (typeof window.frames["iframe_box_mappa"] == "undefined")
		return false;
	var win = window.frames["iframe_box_mappa"].window;
	if ( typeof win.generateMap != "object" || typeof win.generateMap.drawingManager != "object" )
		return false;

	win.generateMap.drawingManager.resetMap();
	var fnForceBoundPosition = function() {
			jQuery('#preload').show();
			win.generateMap.forcePolygonPosition( scope, idProvincia, idComune, null, true );
			win.generateMap.drawingManager.switchSubmitBtn(false);
			setTimeout(function(){ jQuery('#preload', parent.document).fadeOut('fast'); }, 1500);
			boxPolygonSearch.removeEventOnShow('fnForceBoundPosition');
		};
	
	boxPolygonSearch.addEventOnShow(fnForceBoundPosition, 'fnForceBoundPosition');
	
	return true;
}

function saveLastSearchArea(categoria, jsonArea, opzioni) {
	if ( typeof opzioni != 'undefined' && opzioni.tipologia != null)
		categoria = categoria+"_"+opzioni.tipologia;
		
	var nameStorage = 'imm_area';
	jq("#gm_area").val(jsonArea);
	
	oStorage = new webStorage();
	var oStoredArea = oStorage.getJSON(nameStorage);
	if (!oStoredArea || typeof oStoredArea != "object")
		oStoredArea = new Object();

	oStoredArea[categoria] = jq.parseJSON(jsonArea);
	oStorage.set(nameStorage, stringify(oStoredArea));
}

function createPreviewAreaMap(adminLevel) {
	if (typeof adminLevel == "undefined")
		adminLevel = 3;
	
	var jGMArea = jq("#gm_area").val();
	var GMArea = jq.parseJSON(jGMArea);
	var mode = (GMArea) ? GMArea.mode : '';
	var area = (GMArea) ? GMArea.area : '';
	if (typeof FactoryPopBox == "undefined")
		FactoryPopBox = new MEPopBoxFactory();
	boxPreviewAreaMap = FactoryPopBox.getPopBox("box_preview_area_mappa", document.getElementById('modifica_area'));
	boxPreviewAreaMap.hideClose = true;
	boxPreviewAreaMap.setContent('<div id="preview_area_mappa" style="padding:10px 10px 4px;width:260px;height:260px; line-height:260px; cursor:pointer;"></div>');
	
	var map_options =
		{'width':900,'height':500,'zoom':15,'markerType':0,'radius':0,'showMarker':0,'drawPolygon':1,'hasPdi':0,
			'personalizedControl':0,'maximizeDimension':1,'mapControls':0,'adminLevel':adminLevel
		};
	var __createMapHandler = function() {
			createDinamicGoogleMaps('', map_options, 'Disegna l\'area d\'interesse');
		};
	
	var oPreviewContainer = jq("#preview_area_mappa");
	staticMapList(oPreviewContainer, mode, area, google_key);
	oPreviewContainer.mouseenter(function(){ clearTimeout(fadeOutPreviewMap); });
	oPreviewContainer.mouseleave(closePreviewAreaMap);
	oPreviewContainer.click(__createMapHandler);
}

function showPreviewAreaMap() {

	if (!document.getElementById('preview_area_mappa')) {
		createPreviewAreaMap();
	}

	boxPreviewAreaMap.objRef = document.getElementById('modifica_area');

	if (typeof fadeOutPreviewMap != "undefined") {
		clearTimeout(fadeOutPreviewMap);
	}
	if (jq("#MEPB_DIV_box_preview_area_mappa").css("display") == 'none') {
		boxPreviewAreaMap.show();
	}
}

function closePreviewAreaMap() {
	fadeOutPreviewMap = setTimeout(function() { jq("#MEPB_DIV_box_preview_area_mappa").fadeOut(400);}, 150);
}

function checkOpenPreviewMapHP(categoria) {
	var oStorage = new webStorage();
	var jsonAllDrawedArea = oStorage.get('imm_area');
	
	var allDrawedArea = jQuery.parseJSON(jsonAllDrawedArea);
	var oArea = allDrawedArea[categoria];
	
	if ( oArea && typeof oArea !== "undefined" ) {
		jq('#area_mappa').show();
		jq('#zone_cartina').hide();
		jq("#gm_area").val(stringify(oArea));
		jq("#tipoRicerca").val('mappa');
		jq('#textAreaBtn').html('Area disegnata');
		forceTypeSearch('area', '');
		var bDone = staticMapHP(document.getElementById('overlay_opacity_map'), oArea.mode, oArea.area, google_map_key);
		if (bDone)
			jq('#overlay_opacity_map').show();
	}
}

/******************************************
 * FINE GESTIONE MAPPE
 ******************************************/

/**
 * Disabilita il bottone di conferma al click sulla checkbox
 * @param check ---> HTML element
 */
function disableConferma(check) {
	if (check.checked == false)
		jq(".bottoneConferma").attr("disabled", "disabled");
	else
		jq(".bottoneConferma").removeAttr("disabled");
	
}

/**
 * Mostra gli errori di un form
 * @param (JSON) data ---> array associativo degli errori field->error @example 'nome'->'Nome obbligatorio'
 */
function showErrorContatta(data, id, disableMsg){
	//nascondo la rotella ajax
	jq("#ajax_contatta").hide();
	//scorro tutti i campi e levo la classe degli errori
	jq(".field-box,.field-check-box").each(function(i){
		jq(this).removeClass("field_error");
	});
	//scorro tutti gli elementi del JSON e assegno gli errori ai rispettivi campi
	jq.each(data, function(key, value){
		if (!disableMsg)
			jq("#showError").text(value);
		
		jq("#"+id+key).addClass("field_error")
					  .find("textarea").addClass("input_error");
	});
	//mostro il messaggio degli errori
	jq("#showError").slideDown();
}

function forceTypeSearch(type, sufix) {
	if (typeof sufix == 'undefined')
		sufix = '';
	
	var idElement = '';
	if (type == 'area') {
		idElement = '#btn_area'+ sufix;
	} else if (type == 'zona') {
		idElement = '#btn_zone'+ sufix;
	} else {
		return;
	}
	switchBtnClass(idElement, sufix);
}

function switchOffBtn(type) {
	if (type == 'area') {
		idElement = '#btn_area';
	} else if (type == 'zona') {
		idElement = '#btn_zone';
	} else {			
		jq('.ui-radio div').each(function(){
			jq(this).find('span.ui-icon').addClass('ui-icon-radio-off').removeClass('ui-icon-radio-on');
			jq(this).find('input[type|="radio"]').removeAttr('checked');
		});
		jq(this).find('span.ui-icon').addClass('ui-icon-radio-off').removeClass('ui-icon-radio-on');
		return true;
	}
	
	jq('.ui-radio div').each(function(){
		if ( jq(this).attr('id') == jq(idElement).attr('id') ) {
			jq(this).addClass('ui-radio-off').removeClass('ui-radio-on');
			jq(this).find('span.ui-icon').addClass('ui-icon-radio-off').removeClass('ui-icon-radio-on');
			jq(this).find('input[type|="radio"]').removeAttr('checked');
		}
	});
	
	return true;
}

function switchBtnClass(element, sufix) {
	if (typeof sufix == 'undefined')
		sufix = '';
	
	if ( jq(element).hasClass('ui-radio-on') )
		return;
	
	var isEnabledZone = typeof jq("#enabledZone").val() != "undefined" && parseInt(jq("#enabledZone").val()) == 1;
	if (sufix == '_TI') {
		var isCittaConZona = typeof jq("#cittaConZona").val() != "undefined" && parseInt(jq("#cittaConZona").val()) == 1;
		var isCittaConLocalita = typeof jq("#cittaConLocalita").val() != "undefined" && parseInt(jq("#cittaConLocalita").val()) == 1;
	} else {
		var isCittaConZona = typeof jq("#infoComuneArea").val() != "undefined" && jq("#infoComuneArea").val() == 'zona';
		var isCittaConLocalita = typeof jq("#infoComuneArea").val() != "undefined" && jq("#infoComuneArea").val() == 'localita';
	}
	if ( jq(element).attr('id') == 'btn_zone' + sufix) {
		jq('#modifica_area' + sufix).hide();
		jq('#btn_zone' + sufix + ' input[type|="radio"]').attr('checked', 'checked');
		jq('#btn_area' + sufix + ' input[type|="radio"]').removeAttr('checked');
		if ((sufix == '' && isCittaConLocalita) || (isCittaConZona && isEnabledZone))
			jq('#modifica_zona' + sufix).show();
		else
			jq('#modifica_zona' + sufix).hide();
	} else {
		jq('#modifica_area' + sufix).show();
		jq('#modifica_zona' + sufix).hide();
		jq('#btn_area' + sufix + ' input[type|="radio"]').attr('checked', 'checked');
		jq('#btn_zone' + sufix + ' input[type|="radio"]').removeAttr('checked');
	}
	jq('.ui-radio div').each(function(){
		if (jq(this).attr('id') == jq(element).attr('id')) {
			jq(this).removeClass('ui-radio-off').addClass('ui-radio-on');
			jq(this).find('.ui-icon').removeClass('ui-icon-radio-off').addClass('ui-icon-radio-on');
		} else {
			jq(this).addClass('ui-radio-off').removeClass('ui-radio-on');
			jq(this).find('.ui-icon').addClass('ui-icon-radio-off').removeClass('ui-icon-radio-on');
		}
	});
}


function manageSelectArea(section, show) {
	var isCittaConZona = typeof jq("#infoComuneArea").val() != "undefined" && jq("#infoComuneArea").val() == 'zona';
	var isCittaConLocalita = typeof jq("#infoComuneArea").val() != "undefined" && jq("#infoComuneArea").val() == 'localita';
	var isEnabledZone = typeof jq("#enabledZone").val() != "undefined" && parseInt(jq("#enabledZone").val()) == 1;
	
	if (!isEnabledZone)
		return;
	
	if (show) {
		if (isCittaConZona) {
			if (section == 'homepage')
				showZoneSelect(true);
			else
				showZoneSelectWithMap();
		} else if (isCittaConLocalita) {
			showLocalitaSelect(true);
		}
	} else {
		if (isEnabledZone && isCittaConZona) {
			if (section == 'homepage')
				showZoneSelect(false);
			else
				hideBoxZone();
		} else if (isCittaConLocalita) {
			showLocalitaSelect(false);
		}
	}
}

function manageFormCategoriaTipologiaStanza(){
	
	var selCategoriaStanza = jq("#selCategoriaStanza");
	var divTipologiaStanza = jq("#divTipologiaStanza");
	
	selCategoriaStanza.change(function() {
		if (selCategoriaStanza.val() >= 1){
			divTipologiaStanza.fadeIn("slow");
		} else {
			divTipologiaStanza.removeAttr('selected');
			divTipologiaStanza.find('option:first').attr('selected', 'selected');
			divTipologiaStanza.fadeOut("slow");
		}
	});
}

/********************************************************************************************************************************************************/
/********************************************************* CLASSE GESTIONE AUTO LOGIN UTENTI CON FACEBOOK *********************************************/

//Manager Chiamate
var imm_FB = null;
managerUserAutoLoginFB = function( params ) {	
	var valCookie = readCookie( "autoLoginUserFacebook" );
	if( valCookie == 1 )
 		return false;

	imm_FB = new Imm_FB( params );
	imm_FB.onLogin = function() { 
		__userLogged__ = 1; 
		imm_ManagerUpdatePage = new Imm_ManagerUpdatePage( params );
	};
}

//Inizio Classe
var Imm_FB = function( options )
{
	if( jq.isEmpty( options ) || jq.isEmpty( options.clientIdFb ) )
		return false;
	
	this.stopProcess = false;
	this.pageSection = typeof options != 'undefined' && typeof options.pageSection != 'undefined' ? options.pageSection : false;
	this.clientIdFb = options.clientIdFb;	
	this.initListnerClickStopAutoLogin();
	this.detectUserIsLoggedFb();
	this.userToken = null;
	this.runLogin = false;
}

/**
 * Metodo che avvia gli ascolatori sui bottoni per interrompere l'auto login
 * @returns {void}
 */
Imm_FB.prototype.initListnerClickStopAutoLogin = function() {
	var that = this;
	if( !that.stopProcess ) {
		this.autoLoadLoginFbWait = jq( '.autoLoadLoginFbWait' );
		this.loaderAutoLogin = jq( '#loaderAutoLogin' );		
		this.autoLoadLoginFbWait.click( function() { that.stopAutoLogin(); });		
	}
	if ( this.runLogin )
		this.autoLoadLoginFbWait.show();
}

/**
 * Metodo che effettua la chiamata a facebook per controllare se c'è un utente loggato sul social network 
 * in caso ci sia ed abbia accettato l'app di Immobiliare ne ritorna i dati
 * @returns {void}
 */
Imm_FB.prototype.detectUserIsLoggedFb = function() {
	var that = this;
	trackGAClickEvent( "boxLoginEvents", "autoLogin", "FB_Request" );
	window.fbAsyncInit = function(){
		FB.init( { appId:that.clientIdFb, status:true,  cookie:true, xfbml:true, oauth: true, version: 'v2.5' } );
		FB.getLoginStatus( function( response ) {
			if ( response.status != "unknown" ) {
				if( !that.stopProcess ) {
					that.autoLoadLoginFbWait.show();
				}
				var authResponse = FB.getAuthResponse();
				if( authResponse != null && typeof authResponse != 'undefined' && typeof authResponse['accessToken'] != 'undefined' ) {
					that.runLogin = true;
					that.userToken = FB.getAuthResponse()['accessToken'];
					FB.api('/me', function( response ) {
						if( typeof response['error'] == 'undefined' ) {
							trackGAClickEvent( "boxLoginEvents", "autoLogin", "FB_Logged" );
							that.getCodeLogin( response );
						}
					});
				} else {
					that.autoLoadLoginFbWait.hide();
				}
			} else {
				that.autoLoadLoginFbWait.hide();
			}
		});
	};

	// Load the SDK Asynchronously
	(function( d ) {
		var js, id = 'facebook-jssdk'; 
		if ( d.getElementById( id ) )
			return;		
		js = d.createElement( 'script' );
		js.id = id; 
		js.async = true;
		js.src = "//connect.facebook.net/en_US/all.js";
		d.getElementsByTagName('head')[0].appendChild( js );
	}( document ) );	
}

/**
 * Metodo che recupera il codice utile per il login di un utente
 * @param {object} response
 * @returns {void}
 */
Imm_FB.prototype.getCodeLogin = function( response ) {
	var that= this;
	var request = jq.ajax({
		url: '/autoLoginUserFb.php',
		type: "POST",
		data: {
			action : 'getCodeAutoLogin',
			userID : response['id'],
			email  : response['email'],
			token  : that.userToken			
		},
		dataType: "html"
	});
	request.done( function( code ) {
		createCookie( "autoLoginUserFacebook", 1 );		
		that.autoLoginUserToImm( code, response );
	}); 
}

/**
 * Metodo che effettua il login dell'utente
 * @param {string} code
 * @returns {void}
 */
Imm_FB.prototype.autoLoginUserToImm = function( code, userFb ) {
	var that = this;
	
	if( this.stopProcess )
		return false;
	
	trackGAClickEvent( "boxLoginEvents", "autoLogin", "IMM_Login" );
	var request = jq.ajax({
		url: '/autoLoginUserFb.php?code=' + code,
		type: "POST",		
		dataType: "html",
		data: {
			action : 'autoLoginUserFb',
			userID : userFb['id'],
			email  : userFb['email'],
			token  : that.userToken			
		}
	});
	request.done( function( response ) {			
		if( response == '1' ) {
			trackGAClickEvent( "boxLoginEvents", "autoLogin", "IMM_Done" );
			if ( typeof that.onLogin == "function" ) {
				if( that.stopProcess ) {
					that.stopProcessAutoLogin();
					return false;
				} else {
					if( typeof selAccediBox != 'undefined' && typeof selAccediBox.box != 'undefined' )
						selAccediBox.box. close();
					that.onLogin();						
				}
			}
		} else {
			that.autoLoadLoginFbWait.hide();
		}
	}); 
}

/**
 * Metodo che setta la variabile per bloccare l'auto login a true e elimina il bottone dell'auto login
 * @returns {void}
 */
Imm_FB.prototype.stopAutoLogin = function() {
	this.stopProcess = true;
	if( typeof this.autoLoadLoginFbWait != 'undefined' && this.autoLoadLoginFbWait )
		this.autoLoadLoginFbWait.hide();
}

/**
 * Metodo che stoppa l'auto login ed effettua il logout
 * @returns {undefined}
 */
Imm_FB.prototype.stopProcessAutoLogin = function() {
	var request = jq.ajax({
		url: '/logout.php',
		type: "POST",
		dataType: "html"
	});
}

/*********************************************************************************************************************************************/
/********************************************** CLASSE GESTIONE SNIPPED IMMOBILIARE.IT *******************************************************/


/**
 * Classe che gestisce la creazione degli snipped del sito effettua chiamata ajax e stampa risultati nell 'html
 * @param {string} pageSection
 * @returns {void}
 */
var Imm_ManagerUpdatePage = function( params ) {
	this.idAd = typeof params.idAd != 'undefined' ? params.idAd : null;
	this.idProject = typeof params.idProject != 'undefined' ? params.idProject : null;
	this.listAds = typeof params.listAds != 'undefined' ? params.listAds : null;
	this.backUrl = typeof params != 'undefined' && typeof params.backUrl != 'undefined' ? params.backUrl : false;
	this.pageSection = typeof params != 'undefined' && typeof params.pageSection != 'undefined' ? params.pageSection : false;
	this.contentImagesDetail = typeof params != 'undefined' && typeof params.contentImagesDetail != 'undefined' ? params.contentImagesDetail : 'dettaglio';

	switch( this.pageSection ) {
		case 'accedi' : 
			this.getAccediSectionAction();
		case 'home' : 
			this.getSnippedBox( 'getBoxUserLoggedMenu', '#containerAccessoHome' );
		break;
		case 'ricerca' : 
			this.getSnippedBox( 'getToplink_utente', '#alfBoxLinksHeader' );
			this.getSavedAdsUser();
		break;
		case 'dettaglio' : 
			this.getSnippedBox( 'getToplink_utente', '#alfBoxLinksHeader' );
			this.getIfSavedAd();
			this.autofillFormContactDetail();
			this.getLovedImages();
			this.getUserNotes();
			this.getIsLikeAd();
		break;
	}	
}

/**
 * Metodo che effettua la redirect verso la backurl settata dopo il login
 * @returns {void}
 */
Imm_ManagerUpdatePage.prototype.getAccediSectionAction = function() {	
	location.href = this.backUrl;
}

/**
 * Wrapper chiamata controllo se annuncio dettaglio è salvato
 * @returns {undefined}
 */
Imm_ManagerUpdatePage.prototype.getIfSavedAd = function ()
{
	this.checkSavedAdsUser(this.idAd, this.idProject);
}

/**
 * Metodo che concatena gli id degli annunci
 * @returns {Boolean}
 */
Imm_ManagerUpdatePage.prototype.getSavedAdsUser = function()
{
	if (this.listAds == null) {
		return false;
	}

	var idsAd = '', idsProject = '';
	for (var x = 0; x < this.listAds.length; x++) {
		if (!jq.isEmpty(this.listAds[x]['idAnnuncio'])) {
			idsAd += this.listAds[x]['idAnnuncio'] + ',';
		} else if (!jq.isEmpty(this.listAds[x]['idProgetto'])) {
			idsProject += this.listAds[x]['idProgetto'] + ',';
		}
	}
	if (idsAd != '')
		idsAd = idsAd.substr(0, (idsAd.length - 1));
	if (idsProject != '')
		idsProject = idsProject.substr(0, (idsProject.length - 1));
	this.checkSavedAdsUser(idsAd, idsProject);
}

/**
 * Metodo che effettua la chiamata ajax per determinare tra gli annunci in pagina quali sono stati salvati dall'utente
 * @returns {Boolean}
 */
Imm_ManagerUpdatePage.prototype.checkSavedAdsUser = function (idsAd, idsProject)
{
	idsProject = idsProject || '';
	var that = this;
	var request = jq.ajax({
		url: '/imm_ManagerUpdatePage.php?action=getWhichAdsIsSaved',
		type: "POST",
		dataType: "html",
		data: {
			idsAd: idsAd,
			idsProject: idsProject
		}
	});
	request.done(function (response) {
		that.activeSavedAdsBtn(jQuery.parseJSON(response));
	});
}

/**
 * Metodo che accente il link degli annunci salvati
 * @param {string} ids
 * @returns {void}
 */
Imm_ManagerUpdatePage.prototype.activeSavedAdsBtn = function (ids)
{
	if (typeof ids == 'undefined')
		return false;

	if (!jq.isEmpty(ids['ads'])) {
		for (var x = 0; x < ids['ads'].length; x++) {
			this.activeSavedBtn(ids['ads'][x], 'idAnnuncio');
		}
	}
	if (!jq.isEmpty(ids['projects'])) {
		for (var x = 0; x < ids['projects'].length; x++) {
			this.activeSavedBtn(ids['projects'][x], 'idProgetto');
		}
	}
}

/**
 * Metodo che cambia lo stato del pulsante da Salva a Salvato.
 *
 * @param id = idRecord
 * @param idParam = nome della variabile GET (idAnnuncio / idProgetto )
 */
Imm_ManagerUpdatePage.prototype.activeSavedBtn = function (id, idParam)
{
	jq('#salva_' + id).removeClass('link_salva').addClass('link_salvato');
	jq('#salva_' + id).attr('onclick', '');
	jq('#salva_' + id).attr('href', '/bookmarkannunci.php?' + idParam + '=' + id);
	if (this.pageSection == 'ricerca') {
		jq('#salva_' + id).html('<strong>Salvato</strong>');
	}
}

/**
 * Metodo che controlla se esiste un like di facebook sull'annuncio.
 */
Imm_ManagerUpdatePage.prototype.getIsLikeAd = function()
{
	var that = this;
	var request = jq.ajax({
		url: '/imm_ManagerUpdatePage.php?action=getIsLikeAd',
		type: "POST",
		dataType: "html",
		async : false,
		data: {
			idAd : that.idAd
		}
	});
	request.done( function( response ) {
		if( response == 1 )
			jq( '#fbActionLike_'+that.idAd ).removeClass( 'ico_fb_like' ).addClass( 'ico_fb_liked' );
	});
}

/**
 * Metodo che setta recupera le immagini lovvate e le accende
 * @returns {Boolean}
 */
Imm_ManagerUpdatePage.prototype.getLovedImages = function() {
	if( this.idAd == null )
		return false;
	
	var that = this;
	var request = jq.ajax({
		url: '/imm_ManagerUpdatePage.php?action=getLovedImages',
		type: "POST",		
		dataType: "html",
		async : false,
		data: {
			'idAd' : that.idAd 
		}
	});
	request.done( function( jsonResponse ) {
		fbImagesLove = new FbImagesLove( that.contentImagesDetail, true );
		fbImagesLove.setLovedImagesToAjax( jsonResponse );
	});
}

/**
 * Metodo che recupera la nota dell'utente se inserita per l'annuncio corrente
 * @returns {void}
 */
Imm_ManagerUpdatePage.prototype.getUserNotes = function() {
	var that = this;
	
	var noteOptions = { 'checkBookmarked' : 1, 'isLogged' : 1, 'section' : this.pageSection}
	var request = jq.ajax({
		url: '/imm_ManagerUpdatePage.php?action=getUserNotes',
		type: "POST",
		dataType: "html",
		async : false,
		data: {
			idAd: that.idAd,
			idProject: that.idProject
		}
	});
	request.done(function (response) {
		var idRecord = that.idAd ? that.idAd : that.idProject;
		if (response) {
			jq('#box-init-nota-' + idRecord).fadeOut('slow', function () {
				jq('#box-testo-nota-' + idRecord).html('<div class="edit-note"></div><div class="testo-nota" id="testo-nota-' + idRecord + '">' + response + '</div>').fadeIn();
				var managerNoteAd = new ManagerNoteAd(noteOptions);
				managerNoteAd.init();
			});
		}
	});
}

/**
 * Metodo che recupera i dati dell'utente
 * @returns {undefined}
 */
Imm_ManagerUpdatePage.prototype.getDataUser = function() {
	var that = this;
	var request = jq.ajax({
		url: '/imm_ManagerUpdatePage.php?action=getDataUser',
		type: "POST",		
		dataType: "html",
		async : false
	});
	request.done( function( response ) {
		that.user = jQuery.parseJSON( response );
	});
}

/**
 * Metodo che recupera i dati dell'utente e poi chiama la funzione per settarli nel form dei contatti
 * @returns {undefined}
 */
Imm_ManagerUpdatePage.prototype.autofillFormContactDetail = function( user ) {
	var that = this;	
	if( typeof user != 'undefined' ) {
		var dataUser = user;
	} else {
		this.getDataUser();
		var dataUser = this.user;
	}
	
	if( jq( '#contatta_costruttore_formUp' ).exist() ) {
		jq( '#contatta_costruttore_formUp input' ).each( function() {
			that.setFieldContact( this, dataUser );
		});
	}
	if( jq( '#contatta_costruttore_formDown' ).exist() ) {
		jq( '#contatta_costruttore_formDown input' ).each( function() {
			that.setFieldContact( this, dataUser );
		});
	}
}

/**
 * Metodo che compila i form dei contatti con i dati dell'utente corrente
 * @param {type} sender
 * @param {type} user
 * @returns {undefined}
 */
Imm_ManagerUpdatePage.prototype.setFieldContact = function( sender, user ) {
	switch( jq( sender ).attr( 'name' ) ) {
		case 'nome' : jq( sender ).val( user['nome'] );
			break;
		case 'telefono' : jq( sender ).val( user['telefono'] );
			break;
		case 'email' : jq( sender ).val( user['email'] );
			break;			
	}
}

/**
 * Metodo che recupera lo snipped del box richiesto se l'utente è loggato
 * @param {strin} action
 * @param {string|object} boxReplace
 * @returns {void}
 */
Imm_ManagerUpdatePage.prototype.getSnippedBox = function( action, boxReplace ) {
	var request = jq.ajax({
		url: '/imm_ManagerUpdatePage.php?action='+action,
		type: "POST",		
		dataType: "html"
	});
	request.done( function( boxHtml ) {
		jq( boxReplace ).fadeOut( 'slow', function() {
			jq( boxReplace ).html( boxHtml );
			jq( boxReplace ).show();
			if ( action == 'getBoxUserLoggedMenu' )
				jq( '#spotTvBannerContentIframe' ).hide();
		});				
	});		
}


sendContactJob = function () {
	cleanPlaceHolder("formCV");
    var form = jq('#formCV');

    var error = false;

    form.find('#name').removeClass('inputError');
    if (form.find('#name').val() == '') {
        error = true;
        form.find('#name').addClass('inputError');
    }

    form.find('#surname').removeClass('inputError');
    if (form.find('#surname').val() == '') {
        error = true;
        form.find('#surname').addClass('inputError');
    }

    form.find('#email').removeClass('inputError');
    if (!validateEmail(form.find('#email').val())) {
        error = true;
        form.find('#email').addClass('inputError');
    }

    form.find('#phone').removeClass('inputError');
    if (form.find('#phone').val() == '' || !(/^(\+)?[\s\d()\-\/.]+$/.test(form.find('#phone').val()))) {
        error = true;
        form.find('#phone').addClass('inputError');
    }

    form.find('#origin').removeClass('inputError');
    if (form.find('#origin').val() == '') {
        error = true;
        form.find('#origin').addClass('inputError');
    }

    form.find('#cv_wrap').removeClass('inputError');
    if (form.find('#cv').val() == '') {
        error = true;
        form.find('#cv_wrap').addClass('inputError');
    }

    if (!error){
        form.submit();
		return;
	}
	addPlaceHolderCompatibility('formCV');
}


function validateEmail( email ) { 
    var re = /^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
    return re.test(email);
}


PageSkinManager = function(pars){

	if (!(!!pars && !!pars.page)){
		throw("PageSkinManager: Manca il parametro page: {page:<section>}");
	}

	var sPars = "";
	var __skin_popup_block = false;
	for (var attr in pars){
		sPars += (sPars ? "&" : "")+attr+"="+pars[attr];
	}
	sPars = sPars ? "?"+sPars : "";
	jq.ajax({
		url: "/pageSkinManager.php"+sPars,
		context: document.body,
		type: 'POST'
	}).done(
		function(response) {

			if (response){
				var skin = jq.parseJSON(response);
				if (!!skin && !!skin.img){
					document.body.className = "pageSkin";
					addEvent(document.getElementById('header'),"click",function(e){__skin_popup_block = true;});
					addEvent(document.getElementById('corpo'),"click",function(e){__skin_popup_block = true;});
					addEvent(document.getElementById('footer'),"click",function(e){__skin_popup_block = true;});
					addEvent(document.getElementById('boxOverlayContainer'),"click",function(e){__skin_popup_block = true;});
					__bookmarkSide = "right";

					if (skin.link){
						addEvent(document.body,"click",function(){
							if (!__skin_popup_block){
								trackGAClickEvent('Page_Ads_Skin','Click_'+pars.page, skin.name);
								window.open(skin.link,'','fullscreen=yes, scrollbars=yes');
							}
							__skin_popup_block = false;
						});
						trackGAClickEvent('Page_Ads_Skin','Impressions_'+pars.page,skin.name);
					}
					if (skin.impressionCounter){
						var img = document.createElement('img');
						img.src=skin.impressionCounter;
					}
					if (skin.bkgColor){
						document.body.style.backgroundColor = skin.bkgColor;
					}
					if (skin.img){
						document.body.style.backgroundImage = "url("+skin.img+")";
					}
				}
			}
		}
	);
}

jq(document).ready(function () {

	/**
	 * Semplice box overlay di conferma
	 */
	jq(".alertConfirmSimple").on('click', function (e) {
		e.preventDefault();

		var element = jq(this);
		var boxId = typeof element.data('box-id') != 'undefined' ? element.data('box-id') : 'box_alertConfirmSimple';

		var box = new boxOverlay(boxId, "grey");

		var boxTitle = typeof element.data('box-title') != 'undefined' ? element.data('box-title') : 'Conferma azione';
		var text = typeof element.data('box-content') != 'undefined' ? element.data('box-content') : "Effettuare l'operazione richiesta?";
		var boxContent = '<div style="height:100px; width: 350px;">';
		boxContent += '<h1 style="padding-top: 10px; text-align: center;">' + text + '</h1><div class="clear"></div>';
		boxContent += '<div style="padding-top: 30px">';
		boxContent += '<button class="bottoneAnnullaBig"></button>';
		boxContent += '<button class="bottoneConferma"></button>';
		boxContent += '</div>';
		boxContent += '</div>';

		box.setTitle(boxTitle);
		box.setContent(boxContent);
		box.show();

		jq(".bottoneAnnullaBig").on('click', function (e) {
			box.hide();
		});

		jq(".bottoneConferma").on('click', function (e) {
			document.location.href = element.attr('href');
		});
		return false;
	});

});
;
function cleanTxt(){
		var txt = tinyMCE.activeEditor.getContent();
		txt = txt.replace(/<!--.*?-->/g,'');
		txt = txt.replace(/&lt;!--.*?--&gt;/g,'');
		tinyMCE.activeEditor.setContent(txt);
	}

var testo_form_dettaglio_immobile = "Il tuo messaggio: scrivi qui le tue richieste per ricevere informazioni più dettagliate";
function check_boxes(form, field, state) {
    for (i=0; i<form.length; i++){
		 if (form.elements[i].name == field)
		     form.elements[i].checked = state;
    }
}

/* Procedura per eliminazione bottone nel backoffice */
function reset_button(btn_num){
	$("imgact"+btn_num).value="del";
	$("image"+btn_num).src="";
	tinyMCE.get("testoPrinc"+btn_num.substr(2)).setContent("");
	$("preview"+btn_num.substr(2)).innerHTML="";
	if ($("anteprima_img"+btn_num.substr(2)))
		$("anteprima_img"+btn_num.substr(2)).src="";
	if ($("testoPrinc"+btn_num.substr(2)))
	$("testoPrinc"+btn_num.substr(2)).value="";
	alert("Conferma modifiche per completare la cancellazione");
}

function popupCartina(url){
	window.open(url,"popupcartina","location=no,width=600,height=400");
}

function popupPrivacy(){
	window.open("/casa/informazioni/politica-di-privacy.php","privacy","location=no,width=660,height=400,scrollbars=yes,resizable=yes,top=" + ((screen.height-400)/2) + ",left=" + ((screen.width-660)/2));
}
function popupCondizioniGenerali(){
	window.open("/casa/informazioni/condizioni-generali.php","condizioni_generali","location=no,width=600,height=400,scrollbars=yes,resizable=yes,top=" + ((screen.height-400)/2) + ",left=" + ((screen.width-600)/2));
}

function popupfoto(imgURL){
	window.open(imgURL,"popupfoto","location=no,width=730,height=600");
}

function textCounter(field, countfield, maxlimit) {
	if (field.value.length > maxlimit - 500){
		if (document.getElementById('remLenContent'))
			document.getElementById('remLenContent').style.display="block";
	}
	if (field.value.length > maxlimit)
		field.value = field.value.substring(0, maxlimit);
	else{
		 countfield.value = maxlimit - field.value.length;
		 if (countfield.value < (maxlimit/10))
		 	countfield.style.color = 'red';
		 else
		 	countfield.style.color = '#000000';
	}
	
}

function centerThumb(img, width, height){
	if (img.width)
			imgwidth = img.width;
	else
			imgwidth=width;
	if (img.height)
			imgheight=img.height;
	else
			imgheight=height;
	moveleft = Math.round((width - imgwidth) / 2);
	if (moveleft>0)
			img.style.marginLeft= moveleft + "px";
	movetop = Math.round((height - imgheight) / 2);
	if (movetop>0)
			img.style.marginTop= movetop + "px";
}
function show(tagId){
	if (document.getElementById(tagId))
		document.getElementById(tagId).style.display='';
}
function hide(tagId){
	if (document.getElementById(tagId))
		document.getElementById(tagId).style.display='none';
}

function toggleVisible(tagId){

	if ((typeof this.event !== 'undefined') && (typeof this.event.preventDefault !== 'undefined') && this.event){
		this.event.preventDefault();
	}

	if (document.getElementById(tagId)){
		visibilityStatus = document.getElementById(tagId).style.display;
		if (visibilityStatus == 'none'){
			document.getElementById(tagId).style.display = 'block';
		}else {
			document.getElementById(tagId).style.display = 'none';
		}
	}

}

//RICCARDO: due funzioni per la gestione della scritta di default nella
// form delle newsletter e il controllo se l'utente ci ha scritto dentro
function showFormText(oEl, sText) {
	var formContent = oEl.value;
	if(formContent.length == 0)
	{
		oEl.value = sText;
	}
}

function hideFormText(oEl, sText) {
	var formContent = oEl.value;
	if(formContent == sText)
	{
		oEl.value = '';
	}
}

function switchText(node, tagId, showText, hideText){
	if (document.getElementById(tagId)){
		visibilityStatus = document.getElementById(tagId).style.display;
		if (visibilityStatus=='none')
			node.innerHTML = showText;
		else
			node.innerHTML = hideText;
	}

}

function addToFavorites(url, title) {
	var sURL = window.location;
	var sText = title;

	try
	{	if (window.sidebar)
			window.sidebar.addPanel(sText, sURL, "");
		else if (window.external)
			window.external.AddFavorite(sURL, sText);
		else if (window.opera && window.print)
		{	var oA = document.createElement('a');
			oA.setAttribute('rel','sidebar');
			oA.setAttribute('href', sURL);
			oA.setAttribute('title', sText);
			oA.click();
		}
	}
	catch (e)
	{	
	}
}

function setAsHome(url){
	if (window.external){ //IE
		document.body.style.behavior='url(#default#homepage)';
		document.body.setHomePage(url);
	}
		else alert("Spiacenti, il tuo browser non supporta questa funzione");
}

function clickCounter(counterID){
	counterImg = new Image;
	counterImg.src='/img2/blank.gif?counter=' + counterID + '&amp;ts=' + (new Date()).getTime();
}

function	validateFormContatti(formObj){
	formObj.nome.value=formObj.nome.value.replace(/obbligatorio/, '');
	formObj.nome.value=formObj.nome.value.replace(/^\s+|\s+$/, '');
	formObj.telefono.value=formObj.telefono.value.replace(/obbligatorio tel. o email/, '');
	formObj.telefono.value=formObj.telefono.value.replace(/^\s+|\s+$/, '');
	formObj.email.value=formObj.email.value.replace(/obbligatorio tel. o email/, '');
	formObj.email.value=formObj.email.value.replace(/^\s+|\s+$/, '');
	
	errore ="";
	if (formObj.nome.value=="" || formObj.nome.value=="Il tuo nome")
		errore = errore + " - è obbligatorio indicare il nome\n";
    else
    if (document.getElementById('ti_utente_nome')) {
                document.getElementById('ti_utente_nome').value = formObj.nome.value;
        }
        
  if (formObj.source.value != "beez")
		if ((formObj.email.value=="" || formObj.email.value=="La tua email") && (formObj.telefono.value=="" || formObj.telefono.value=="Il tuo telefono"))
			errore = errore + " - è obbligatorio indicare almeno uno tra email e telefono\n";
  
	if (formObj.email.value!="" && formObj.email.value!="La tua email" && !checkEmail(formObj.email.value))
		errore = errore + " - indirizzo email non valido\n";
    else{
        if (document.getElementById('ti_utente_mail')) {
                document.getElementById('ti_utente_mail').value = formObj.email.value;
        }
    }
   
  if (formObj.source.value == "beez"){
		if (formObj.telefono.value=="" || formObj.telefono.value=="Il tuo telefono")
		errore = errore + " - è obbligatorio indicare il telefono\n";
		
		if (formObj.email.value=="" || formObj.email.value=="La tua email")
		errore = errore + " - è obbligatorio indicare l'email\n";
  }
	

	if (formObj.telefono.value!="" && formObj.telefono.value!="Il tuo telefono" && !/^(\+)?[\s\d()\-\/.]+$/.test(formObj.telefono.value))
		errore = errore + " - telefono non valido (non può contenere lettere, può iniziare con ' + ', può contenere numeri e ( ) / - . )\n";

    else
    if (formObj.telefono.value!="" && document.getElementById('ti_utente_telefono')) {
                document.getElementById('ti_utente_telefono').value = formObj.telefono.value;
        }
    
	if (!formObj.privacy.checked)
		errore = errore + " - conferma di aver letto la normativa sulla privacy e di acconsentire al trattamento dei dati \n";

	if (errore!=""){
		errore = "Errori nei dati:\n" + errore;
		alert(errore);
		return false;
	}else{
		return true;
	}
}
var objOmbra;
var objImage;
var objImageOver;
var objImg;

var position = "out";
var idFoto = 0;
var aperta = false;

function fotoplus(indice,imageid,op,outmin){

	//settiamo i valori per fa vedere la fotoplus
	if (op=='none' && position == "over" && idFoto != 0){
		if (!aperta || !outmin || (idFoto != indice))
		{
			position = "out";
			idFoto = 0;
			hideFotoPlus(indice,imageid);
		}
	}
	else{
		position = "over";
		idFoto = indice;
		setTimeout("showFotoPlus(" + indice + "," + imageid + ")", 1000);
	}
}

function showFotoPlus(indice,imageid){

	objOmbra = window.document.getElementById('popupOmbra_'+indice);
	objImage = window.document.getElementById('popupImage_'+indice);
	objImageOver = window.document.getElementById('popupImageOverlay_'+indice);
	objImg = window.document.getElementById('img_'+indice);

	if (position == "over" && idFoto == indice && !aperta){
		objImage.style.display='block';
		objOmbra.style.display='block';
		objImageOver.style.display='block';
		objImg.onload=initImage;
		objImg.src = __gvs_MEDIA_SERVER_IMAGE + "image/" + imageid + "/print.jpg";
		aperta = true;
		
	}
}

function hideFotoPlus(indice,imageid){

	objOmbra = window.document.getElementById('popupOmbra_'+indice);
	objImage = window.document.getElementById('popupImage_'+indice);
	objImageOver = window.document.getElementById('popupImageOverlay_'+indice);
	objImg = window.document.getElementById('img_'+indice);
	if (aperta){
		objImage.style.display='none';
		objOmbra.style.display='none';
		objImageOver.style.display='none';
		objImg.style.visibility='hidden';
		aperta = false;
	}
}

function initImage() {
	objImg.style.marginLeft = Math.round((600-objImg.width)/2) + "px";
	objImg.style.marginTop = Math.round((440-objImg.height)/2) + "px";
	setOpacity(0);
	objImg.style.visibility='visible';
	fadeIn(0);
}

function setOpacity(opacity) {
  opacity = (opacity == 100)?99.999:opacity;
  // IE/Win
  objImg.style.filter = "alpha(opacity:"+opacity+")";
  
  // Safari<1.2, Konqueror
  objImg.style.KHTMLOpacity = opacity/100;
  
  // Older Mozilla and Firefox
  objImg.style.MozOpacity = opacity/100;
  
  // Safari 1.2, newer Firefox and Mozilla, CSS3
  objImg.style.opacity = opacity/100;
}

function fadeIn(opacity) {
    if (opacity <= 100) {
      setOpacity(opacity);
      opacity += 10;
      window.setTimeout("fadeIn("+opacity+")", 50);
    }
}

function checkMessageForm(num) {
        var hasError = false;
        var cform = document.forms[ num ];
        if( cform.nome.value.length == 0 )
        {
                alert( "Il campo NOME e' obbligatorio" );
                hasError = true;
        }
        else if(!cform.telefono.value && !cform.email.value)
        {
                alert( "I campi TELEFONO ed EMAIL non possono essere entrambi vuoti" );
                hasError = true;
        }
        else if( cform.telefono.value && !cform.telefono.value.match( /^[0-9]{5,}$/ ) )
        {
                alert( "Il valore inserito nel campo TELEFONO non e' valido" );
                hasError = true;
        }
        else if( cform.email.value && !cform.email.value.match( /^[\w\-\.\+]+\@[a-zA-Z0-9\.\-]+\.[a-zA-z0-9]{2,4}$/ ) )
        {
                alert( "Il valore inserito nel campo EMAIL non e' valido" );
                hasError = true;
        }
        else if( cform.commento.value.length == 0 )
        {
                alert( "Il campo MESSAGGIO e' obbligatorio" );
                hasError = true;
        }
        else if( !cform.privacy.checked )
        {
                alert( "E' necessario autorizzare il trattamento dei dati personali" );
                hasError = true;
        }       
        if( !hasError )
                cform.submit();
}

function hideContratto(id){
	var oDivContratto = window.document.getElementById(id);
	if (oDivContratto && oDivContratto.style.display){
		if (oDivContratto.style.display != 'none'){
			oDivContratto.style.display = 'none';
			setTimeout (function() {window.document.getElementById('idContratto2').checked= 'checked'; }, 1000); 
		}
	}
}

function showContratto(id, contratto){
	if (window.document.getElementById(id) == null)
		return false;
	var oDivContratto = window.document.getElementById(id);
	if (oDivContratto && oDivContratto.style.display){
		if (oDivContratto.style.display == 'none'){
			oDivContratto.style.display = 'block';
			if (contratto && contratto != "") {
				if (window.document.getElementById('idContratto' + contratto) == null)
					return false;
				window.document.getElementById('idContratto' + contratto).checked= 'checked';
			}
		}
	}
}

function VediPassword(divId)
	{
		sDivStatus = eval("document.getElementById('pwd"+divId+"').style.display");
		if(sDivStatus == '')
			{
				sDivOp = "document.getElementById('pwd"+divId+"').style.display='none';";
				sDivOp2 = "document.getElementById('hide"+divId+"').style.display='';";
				
			}
		else
			{
				sDivOp = "document.getElementById('pwd"+divId+"').style.display='';";
				sDivOp2 = "document.getElementById('hide"+divId+"').style.display='none';";
			}
		
		eval(sDivOp);
		eval(sDivOp2);
	}
function NascondiPassword(divId)
	{
		sDivId = "document.getElementById('pwd"+divId+"').style.display='none';";
		eval(sDivId);
	}
function CaricaServizi(sUrl)
	{
		document.location.href=location.protocol+"//"+location.host+"/casa/informazioni/prestiti.php";
	}
function MostraDescrizioneSw(DivID)
	{
		sCommand = "document.getElementById('sw" + DivID + "').style.display='none'";
		eval(sCommand);
		sCommand = "document.getElementById('sw" + DivID + "_ext').style.display='block'";
		eval(sCommand);	
	}
function NascondiDescrizioneSw(DivID)
	{
		sCommand = "document.getElementById('sw" + DivID + "').style.display='block'";
		eval(sCommand);
		sCommand = "document.getElementById('sw" + DivID + "_ext').style.display='none'";
		eval (sCommand);	
	}

function switchText2(node, tagId, showText, hideText, hideTagId, heightTag){

	if (document.getElementById(tagId)){
		visibilityStatus = document.getElementById(tagId).style.display;

		if (visibilityStatus=='none'){
			node.innerHTML = showText;
			if (document.getElementById('linkcitta'))
			    document.getElementById('linkcitta').style.display='block';
		}
		else{
			node.innerHTML = hideText;
			if (document.getElementById('linkcitta'))
			    document.getElementById('linkcitta').style.display='none';
		
		}
	}
	node.blur();
}

function hideRicercaAvanzata(hideTag,hideText){
	
	if (document.getElementById(hideTag))
		document.getElementById(hideTag).style.display = 'none';
	if (document.getElementById(hideText))
		document.getElementById(hideText).style.display = 'none';
	
}



function showRicercaAvanzata(hideTag,hideText,tagId,hideTagId,heightTag){
	if (document.getElementById(tagId)){
		visibilityStatus = document.getElementById(tagId).style.display;

		if (visibilityStatus=='block'){
		    if (document.getElementById('contCitta'))
			document.getElementById('contCitta').style.display='none';
			document.getElementById('linkRa').innerHTML = '&#9658; Ricerca base';
		}
		else{
			if (visibilityStatus!='none'){
				if (document.getElementById(hideTag)){
					document.getElementById(hideTag).style.display = 'block';
					
				}
			}
			if (document.getElementById('contCitta'))
			    document.getElementById('contCitta').style.display='block';
			document.getElementById('linkRa').innerHTML = '&#9658; Ricerca avanzata';
		}
		document.getElementById(hideText).style.display = 'block';
	}

}

function isNumberKeyPressed(charCode) {
        if (charCode > 31 && (charCode < 48 || charCode > 57 )) 
        		return false;
        return true;
}
	


function checkCurrencys(evt,alertMsg) {
	evt = (evt) ? evt : window.event;
	var charCode = (evt.charCode) ? evt.charCode : 
		((evt.keyCode) ? evt.keyCode :
			((evt.which) ? evt.which : 0)
		);
	var res=isNumberKeyPressed(charCode);
	if( typeof alertMsg!="undefined" && alertMsg==true ){
		var msg="In questo campo è possibile inserire solo numeri";
		if( !res )
			alert(msg);
	}
	return res;
}

function checkCurrencysFloat(evt) {
	evt = (evt) ? evt : event;
	var charCode = (evt.charCode) ? evt.charCode : 
		((evt.keyCode) ? evt.keyCode :
			((evt.which) ? evt.which : 0)
		);
	var carSpec = evt.keyCode;

	if (carSpec == 46 || carSpec ==37 || carSpec ==39) 
		return true;

	if (charCode == 44) 
		return true;
	
	if (charCode > 31 && (charCode < 48 || charCode > 57 )) {
			alert("In questo campo sono ammessi solo numeri\nI decimali devono essere separati da virgola (es:124,58)");
			return false;
	}
	
	return true;
}


function checkCurrencysGetrix(evt,input) {
	evt = (evt) ? evt : event;
	var charCode = (evt.charCode) ? evt.charCode : 
		((evt.keyCode) ? evt.keyCode :
			((evt.which) ? evt.which : 0)
		);
	var carSpec = evt.keyCode;

	if (carSpec == 46 || carSpec ==37 || carSpec ==39 || carSpec == 8) 
		return true;

	if ( ((charCode > 31 && (charCode < 48 || charCode > 57 ) && charCode!=45) || input.value=='-1' ) || ( input.value=='-' && charCode != 49) ) 
			return false;

	return true;
}
/**
 * Metodo per la validazione lato client del campo provincia in ricerca agenzie box a lato
 */
function validateSearchAgency(){
	if(jq('#idProvincia').length > 0){
		var idp = jq('#idProvincia').val();
		if(typeof(idp) == 'undefined' || idp == '' || idp == 0){
			 alert("Il campo provincia è obbligatorio");
			 return false;
		}else{
			jq('#form_raffina_nc').submit();
		}
	}else{
		jq('#form_raffina_nc').submit();
	}
}

/**
 * Metodo che toglie il value di default di un campo input se non modificato
 */
function exChangeValue( sMeInput, sMeDiv ) {
	if (!jq( '#'+sMeDiv ) || !jq( '#'+sMeInput ) )
		return false;

	if ( jq( '#'+sMeDiv ).css( 'display' ) == 'block' || jq( '#'+sMeDiv ).css( 'display' ) == '' ) {
		jq( '#'+sMeDiv ).hide();
		jq( '#'+sMeInput ).focus();
	}
	return true;
}
 
function vDivHidden( sMeDiv, sMeInput ) {
	if ( jq( '#'+sMeDiv ).css( 'display' ) == 'none' ){
		if ( !jq( sMeInput ).val() ){
			jq( '#'+sMeDiv ).show();
		}
	}
}

function aggPrezzo(nMoltiplicatore,sDiv,nValue){
nValue=parseFloat(nValue);
nValue=nValue.toFixed(2);
	if (!document.getElementById(sDiv).innerHTML)
		return null;
	nMoltiplicatore++;
	sEuro =nValue*nMoltiplicatore;
	sEuro = sEuro.toFixed(2);
	document.getElementById(sDiv).innerHTML = " € " + sEuro.replace(".",",");
	return true;
	
	
}




/************ BORSELLINO CAMBIO STATO****************************/
function alertCambioStato(obj,pending) {
	prec = $('tipoStatoAttuale').value;
	if ( (obj.value == 2 || obj.value == 5)){
	
		if (pending>1){
			testoAlert= "ci sono " + pending + " ricariche";
		}else{
			testoAlert= "c'e' una " + pending + " ricarica";
		}

		if (!confirm("Attenzione " + testoAlert + " in sospeso. Passando dallo stato di 'prova' a 'contratto/sospeso', tutti i dati riguardanti i pagamenti del borsellino non attivi e non pagati, verrano eliminati. Premere annulla per procedere manualmente.")){
			option = $('idTipologiaAgenzia').options;
			for (var i=0;i<option.length;i++) {
				if (option[i].value==prec){ 
					option[i].selected=true;
				}
			}
		}
	}
}


function checkTrovaImmobiliForm(oForm){
        return true;
}


function ordina(criterio){
	criteria = criterio.split('-');
	document.dati.criterio.value=criteria[0];
	document.dati.ordine.value=criteria[1];
    pushSelectedZonesOnMapInForm.bind(document.getElementById('form_raffina_nc'))();
	document.dati.submit();
}

/** ex toggleTipologieDiv **/
jq(function() {
  listnerOpenTableListingDetail();
});
function listnerOpenTableListingDetail(){

	jq('a[rel="openDetailTable"]').on('click', function(event){
		var _that = this,
			id = jq(this).attr('data-id'),
			openedText = jq(this).attr('data-opentxt'),
			closedText = jq(this).attr('data-closedtxt'),
			selector = jq(this).attr('data-table-selector') + id,
			$jqBox = jq( selector);

		if ($jqBox.css('display') == 'none'){
			jq( _that ).find('span').html( openedText );
		}else{
			jq( _that ).find('span').html( closedText );
		}

		$jqBox.slideToggle( 500 );

		return false;
	});
	
}
/***/

//Gestione placeholders campi input
function addPlaceHolderCompatibility(containerID){
	var bPlaceHolderCompatibility = false;

	if(typeof containerID !== "undefined"){
		container = "#"+containerID;
	}else{
		container = document;
	}
   if(typeof(jq) != 'undefined'){
		bPlaceHolderCompatibility = jq.supportPlaceholder();
   }else{
	   var test = document.createElement('input');
	   if('placeholder' in test) bPlaceHolderCompatibility = true;
   }

   if(!bPlaceHolderCompatibility) {
 
		jq(":text,textarea,.inputEmailType,.inputPasswordType", container).bind("focus", function () {
			if (jq(this).attr('placeholder') != '' && jq(this).val() == jq(this).attr('placeholder')) {
			jq(this).val('').removeClass('placeHolder').addClass('textInput').select();
			}
		});

		jq(":text,textarea,.inputEmailType,.inputPasswordType", container).bind("blur",function () {
			if (jq(this).attr('placeholder') && (jq(this).attr('placeholder') != '') && (jq(this).val() == '' || jq(this).val() == jq(this).attr('placeholder'))) {
				jq(this).val(jq(this).attr('placeholder')).addClass('placeHolder').removeClass('textInput');
				if(jq(this).hasClass("inputPasswordType") && jq(this).attr("type")=="password"){
					var el= jq(this);
					var el = document.getElementById(el[0].id);
					changeInputType(el,"text");
				}
			}
		});

		jq(":text,textarea,.inputEmailType,.inputPasswordType", container).blur();
   }
}

function cleanPlaceHolder(idForm){
	var bPlaceHolderCompatibility = false;

	if(typeof(jq) != 'undefined'){
		bPlaceHolderCompatibility = jq.supportPlaceholder();
	}else{
		var test = document.createElement('input');
		if('placeholder' in test) bPlaceHolderCompatibility = true;
	}

	if(!bPlaceHolderCompatibility) {
		jq("input,textarea,.inputEmailType,.inputPasswordType", "#"+idForm).each(function() {
			jq(this).unbind();
			if (jq(this).attr('placeholder') != '' && jq(this).val() == jq(this).attr('placeholder')) {
				jq(this).val('').removeClass('placeHolder').addClass('textInput');
			}
		});
	}
}

function cleanPlaceHolderById( idElement ){
	jq( idElement ).unbind();
	if (jq(idElement).attr('placeholder') != '' && jq(idElement).val() == jq(idElement).attr('placeholder')) {
		jq(idElement).val('').removeClass('placeHolder').addClass('textInput');
	}
}


//Funzione per la gestione della selezione delle date nelle statistiche del backoffice
function statsDateChangeEvent(){
	setTimeout(function(){
		var dataInizio = document.getElementById('dataInizio').value,
			dataFine = document.getElementById('dataFine').value;
			
		var firstValue = dataInizio.split('/');
		var secondValue = dataFine.split('/');
		
		var firstDate = new Date();
		firstDate.setFullYear(firstValue[2],(firstValue[1] - 1 ),firstValue[0]);
		
		var secondDate=new Date();
		secondDate.setFullYear(secondValue[2],(secondValue[1] - 1 ),secondValue[0]);
		
		if (firstDate > secondDate) {
			document.getElementById('getStats').style.display = 'none';
			document.getElementById('boxEsportaCSVstats').style.display = 'none';
			alert("Attenzione! Errore nella selezione delle date!");
			return false;
		}
		
		if(dataInizio != "" && dataFine != ""){
			document.getElementById('getStats').style.display = '';
			document.getElementById('boxEsportaCSVstats').style.display = '';
		}else{
			document.getElementById('getStats').style.display = 'none';
			document.getElementById('boxEsportaCSVstats').style.display = 'none';
		}
	},500);
}

//Effettua la somma dei campi di spesa delle stanze
function updateTotalPrice()	{
	var prezzo = 0;
	var servizi = 0;
	var spese = 0;
	var spese_condominiali=0;

	if( p = parseInt( document.dati.prezzo.value ) )
		prezzo = p;

	if( sv = parseInt( document.dati.costoServizi.value ) )
		servizi = sv;

	if( sp = parseInt( document.dati.altreSpese.value ) )
		spese = sp;

	if ( sc = parseInt( document.dati.spese_condominiali.value ) )
		spese_condominiali=sc;

	document.dati.totale.value = prezzo + servizi + spese + spese_condominiali;
}

//Controlla che il campo sia popolato con un numero valido (intero positivo senza punteggiatura)
function checkNumberValue(valore, nomeCampo){

	if(valore != "" && !valore.match(/^[0-9]+$/)){
		alert("Attenzione, il campo "+nomeCampo+" non é popolato correttamente e non verrà salvato!");
		return false;
	}
	return true;
}

//Controlla che il campo sia popolato con un numero valido (valuta euro, compresa di centesimi )
function checkEuroFloatValue(valore, nomeCampo){

	if(valore != "" && !valore.match(/^[0-9]+(.[0-9]{3})*(,[0-9]{2})?$/)){
		alert("Attenzione, il campo "+nomeCampo+" non é popolato correttamente e non verrà salvato!");
		return false;
	}
	return true;
}

function createAjaxRequest() {
	if(window.XMLHttpRequest)
		return new XMLHttpRequest();
	else if(window.ActiveXObject)
		return new ActiveXObject('Microsoft.XMLHTTP');
	else
		return null;
}

function ajaxGet(url) {
	var ajaxRequest = createAjaxRequest();

	var callbacks = [];

	ajaxRequest.onreadystatechange = ajaxRequest.then = function(onSuccess, onError) {
		if(onSuccess && onSuccess.call) callbacks = [onSuccess, onError];
		if(ajaxRequest.readyState == 4) {
			if(ajaxRequest.status == 200) {
				if(callbacks[0]) callbacks[0](ajaxRequest.response);
			} else {
				if(callbacks[1]) callbacks[1](event);
			}
		}
	};

	ajaxRequest.open(
		'GET',
		url,
		true
	);
	ajaxRequest.responseType = 'arraybuffer';

	ajaxRequest.send(null);

	return ajaxRequest;
};
var __selectedOvBox; //variabile globale che tiene traccia del box aperto attualmente

function create_salvaBox(idAnnuncio, opt) {
	opt = opt || {};
	opt.idAnnuncio = idAnnuncio;
	var cookieVal = stringify(opt);
	createCookie("salvaAnnuncio", cookieVal);
	__selectedOvBox = create_accediBox("U", {loginType: "U"});
	__selectedOvBox.show();
	return __selectedOvBox;
}

function salvaAnnuncio(idAnnuncio){
	
	__blockTIAutoPopup = true;
	salvaAnnuncio_boxes[idAnnuncio].box.show();
	__selectedOvBox = salvaAnnuncio_boxes[idAnnuncio].box;
	selectedSA = salvaAnnuncio_boxes[idAnnuncio];
	return false;
}

function create_inviaBox(idAnnuncio,type){
	__blockTIAutoPopup = true;
    document.body.style.cursor = "wait";	
    var inviaAmicoBox = new boxOverlay('inviaAmicoBox_'+idAnnuncio,'grey');
    inviaAmicoBox.setTitle("<strong>Invia questo annuncio ad un amico/a</strong>");
    inviaAmicoBox.stop_event_bubbling = true;
    
    
    var ajaxObj = getMEAjaxObj();
    var url;
		if (type=="progetto"){
				url = "/invia_amico_nc.php?getForm=1&id="+idAnnuncio;
		} else {
			url = "/invia_amico.php?getForm=1&id="+idAnnuncio;
		}


    ajaxObj.Request("POST", url, function(data){
                                   inviaAmicoBox.setContent(data.responseText);
                                   inviaAmicoBoxContent = inviaAmicoBox.getContent();
                                    
												document.body.style.cursor = "default";
												inviaAmicoBox.show();
                                                });
		return inviaAmicoBox;
}

function createInviaBoxRicerca(urlRicerca,page) {
	__blockTIAutoPopup = true;
	if(urlRicerca != null && urlRicerca.length > 0) {
		urlRicerca = Url.encode(urlRicerca);
		var url = "/invia_ricerca_amico.php?getForm=1&url="+urlRicerca+"&page="+page;
		document.body.style.cursor = "wait";		
		
		var inviaRicercaAmicoBox = new boxOverlay('inviaAmicoBox_ricerca','grey');
		inviaRicercaAmicoBox.stop_event_bubbling = true;
		
		var ajaxObj = getMEAjaxObj();
        
        if( page == "home")
            var title = '<div>Segnala Immobiliare.it</div>';
        else if( page == "ricerca" || page == "risultati")
            var title = 'Invia questa ricerca ad un amico/a';
        
		ajaxObj.Request("POST", url, function(data){
								   inviaRicercaAmicoBox.setTitle(title);
                                   inviaRicercaAmicoBox.setContent(data.responseText);
                                   inviaRicercaAmicoBoxContent = inviaRicercaAmicoBox.getContent();
                                    
												document.body.style.cursor = "default";
												inviaRicercaAmicoBox.show();
                                                });
		return inviaRicercaAmicoBox;
	}
	return false;
}

function create_segnalaErroreBox(idAnnuncio,type){
    document.body.style.cursor = "wait";	
    var segnalaErroreBox = new boxOverlay('segnalaErroreBox_'+idAnnuncio,'grey');
    segnalaErroreBox.setTitle("<div><strong>Segnala un errore in questo annuncio</strong></div>");
    
    var ajaxObj = getMEAjaxObj();
		var url;		
		if (type=="progetto"){
				 url = "/segnala_errore.php?getForm=1&idProgetto="+idAnnuncio;
		}else
				url = "/segnala_errore.php?getForm=1&idAnnuncio="+idAnnuncio;

    ajaxObj.Request("POST", url, function(data){
	segnalaErroreBox.setContent(data.responseText);
	segnalaErroreBoxContent = segnalaErroreBox.getContent();

	document.body.style.cursor = "default";	                                                
	segnalaErroreBox.show();
	});

    return segnalaErroreBox;
}
function show_IkeaBox(title,content){
	if (typeof IkeaBox=="undefined"){
		 	document.body.style.cursor = "wait";	
    		IkeaBox = new boxOverlay('ikeaBox','grey');
    		IkeaBox.setTitle("<strong>Metti su casa con IKEA</strong>");
    		var ajaxObj = getMEAjaxObj();
			var url;		
			url = "ikea_box.php";
    		ajaxObj.Request("POST", url, function(data){
         	IkeaBox.setContent(data.responseText);
				document.body.style.cursor = "default";	                                                
				IkeaBox.show();
			});
	}
	else
		IkeaBox.show();
	
}

/********** immobile garantito ******/
jq(function() {
	jq('#dettaglio').on('click', '#btnOpenGarantito', function(){
		var _type = jq(this).attr('data-type');
		var _idAds = parseInt( jq(this).attr('data-ann'), 10);
		var _idPrj = parseInt( jq(this).attr('data-prj'), 10);

		var _param = {};
		_param.type = _type;
		_param.from = 'detail';
		_param.title = "<img src=\"/img2/imgo_check_small.png\" alt=\"Immobile Garantito\" style=\"padding: 0 5px 2px 0;\" /><strong>Immobile Garantito</strong>";
		_param.idannuncio = _idAds;
		_param.idprogetto = _idPrj;

		show_ImmobileGarantitoBox(_param);
	});
});

function show_ImmobileGarantitoBox( params ){
	params = params || {};

	var title = (typeof params.title == 'undefined') ? "<strong>Immobile Garantito</strong>" : params.title;
	jq('body').css('cursor', "wait");

	//APRI POPUP
	if (typeof IGBox=="undefined"){
		var IGBox = new boxOverlay('IGBox','grey');
	}

	var opt = {_params: params};
	var urlGetPopup = VHOST_URL_HOST + "services/dataImmobileGarantito.php?"+jq.param(opt)+"&type=json";

	//loader
	IGBox.setContent("<div style=\"height: 80px;text-align: center;padding: 62px 0 61px 0;width:524px;\"><h3>Attendi...</h3><br /><span style=\"\"><img src=\"/img2/loader3.gif\"></span></div>");
	IGBox.setTitle( title );
	IGBox.show();

	setTimeout(function(){
		jq.ajax({
			type: "POST",
			url: urlGetPopup,
			dataType: "json"
		}).done(function(data) {
			var outHTML = data.html;

			jq('body').css('cursor', "auto");

			IGBox.setContent(outHTML);
			IGBox.show();
		});
	}, 200);
}
/********** immobile garantito ******/

function create_contattaBox(idAnnuncio, idAgenzia, nomeAgenzia) {
	idBox = idAnnuncio ? idAnnuncio : idAgenzia;
	var __selectedOvBox = new boxOverlay('contatta_' + idBox, 'grey');
	__selectedOvBox.setTitle("<strong style=\"font-size: 1.2em;;\">Richiedi maggiori informazioni</strong>");
	__selectedOvBox.setContent("<div style=\"height:210px;text-align:center;\"><h3>Caricamento in corso...</h3><br /><span style=\"\"><img src=\"/img2/loader6.gif\"></span></div>");
	return __selectedOvBox;
}

function show_contattaBox(idAgenzia,jsonOpts){
	var idAnnuncio, isNC, nomeAgenzia;
	if (typeof jsonOpts !== 'undefined' && typeof jsonOpts == "object"){
		idAnnuncio = typeof jsonOpts.idAnnuncio != "undefined" ? jsonOpts.idAnnuncio : null;
		nomeAgenzia = typeof jsonOpts.nomeAgenzia != "undefined" ? jsonOpts.nomeAgenzia : null;
		isNC = typeof jsonOpts.isNC != "undefined" ? jsonOpts.isNC : null;
	}else{

	}

	_disableAlertPopUp = true;
	__blockTIAutoPopup = true;
	
	document.body.style.cursor = "wait";
	__selectedOvBox = new Object();	
	__selectedOvBox = create_contattaBox(idAnnuncio,idAgenzia,nomeAgenzia);
	__selectedOvBox.setContent("<div style=\"height:210px;text-align:center;\"><h3>Caricamento in corso...</h3><br /><span style=\"\"><img src=\"/img2/loader6.gif\"></span></div>");
	__selectedOvBox.show();

	var source = _mailSection;
	var tabActivated = jq('#risultati_ricerca').find('.tab_selector.act');
	if (typeof tabActivated != 'undefined') {
		switch (tabActivated.attr('id')) {
			case 'tab_btn_listaAnnunci':
				source = 'lista_annunci';
				break;
			case 'tab_btn_mapContainer':
				source = 'dettaglio_annuncio_mappa';
				break;
			case 'tab_btn_fgAnnunci':
				source = 'lista_annunci_fg';
				break;
		}
	}

	//parametri chiamata POST
	var params = {};
	params["getForm"] = 1;
	params["source"] = source;

	if (idAnnuncio)
		params["id"] = idAnnuncio;
	var url;
	
	if (idAgenzia) {
		if (!isNC)
			url = "/contatta_agenzia.php";
		else
			url = "/invia_email_nc.php";
		params["idAgenzia"] = idAgenzia;
	} else {
		url = "/contatta_utente.php";
		params["idAnnuncio"] = idAnnuncio;
	}
	
	var xhr = jq.post(url,
			params,
			function(data){
				__selectedOvBox.setContent(data);
				setContattaBoxEvent();
				document.body.style.cursor = "default";	
				_disableAlertPopUp = false;

                if (readCookie("cntct_prvcy_disc"))
                    jq("#privacy").attr("checked","checked")
			});

	xhr.done(function () {
		var $isChatEnabledForAd = jq("input[name='isChatEnabledForAd']" );

		if (idAgenzia && idAnnuncio && !jq.isEmpty($isChatEnabledForAd) && $isChatEnabledForAd.val() === '1') {
			//inizializzo la gestione della chat
			var managerChat = new ManagerChat({
				idAgenzia: idAgenzia,
				enabledLog: __gvs_IS_PRODUCTION_ENVIRONMENT ? false : true
			});
			managerChat.init();
			__selectedOvBox.addEventOnClose(function () {
				managerChat.stopPolling = true;
			});
		}
	});
}


function submitBoxForm(idForm){
	document.body.style.cursor = "wait";
	var position = idForm.search('contatto_');
	__selectedOvBox.show();
	var idBox;
	//prendo il form
	var form = document.getElementById(idForm);

	jq("input,textarea" , "#"+idForm).each(function() {
        if (jq(this).attr('placeholder') != '' && jq(this).val() == jq(this).attr('placeholder')) {
            jq(this).val('').removeClass('placeHolder').addClass('textInput');
        }
    });

	//mostro la rotellina per la chiamata
	jq("#ajax_contatta").show();
	//faccio una chiamata POST aspettandomi un JSON
	jq.post(form.action, 
				jq("#"+idForm).serialize(), 
				function(data){ 
					//è un JSON mostro gli errori
					showErrorContatta(data, "contact_box_", true);
					//setto il titolo
					__selectedOvBox.setTitle("Errore nei dati");
				}, 
				"json")
				//non è un JSON, mostro il messaggio HTML
				.error(function(data){
					__selectedOvBox.setContent(data.responseText);
					if(document.getElementById('sendOk'))
					    __selectedOvBox.setTitle("Messaggio inviato con successo");
					else
					    __selectedOvBox.setTitle("Errore nei dati");
				})
				//eseguo ultime funzioni a fine chiamata
				.complete(function(data){
					document.body.style.cursor = "default";
					retrieveTrackPage(data.responseText);
                    scriptEval(data.responseText);
				});

	addPlaceHolderCompatibility();	
	function retrieveTrackPage(page){
		var fileName;
		var myRegExp = /%%SUCCESS%%/;
		var re = new RegExp(myRegExp);
		var m = re.exec(page);
		if (m != null) {
			var pageToTrack = /%%FILE:(.*)%%/;
			re = new RegExp(pageToTrack);
			m = re.exec(page);
			
			if (m != null){
					fileName = m[1];
					if (fileName){
							trackPage(fileName);
					}
			}
		}
	}
}

/**
 * Valida il campo ed in caso mostra l'errore
 * @param (string) type --> tipo del campo @example telefono, email, ...
 * @param (HTML Element) input ---> Elemento html dell'input da validare
 * @param (string) id ---> id del campo da stilizzare
 */
function checkField(type, input, id){
	//salta il controllo sull'espressione regolare
	var skipRegExp = false;
	//in base al tipo scelgo il controllo da fare
		switch (type) {
		case "telefono":
			var filter = /^(\+)?[\s\d()\-\/.]+$/;
			break;
		case "email": 
			var filter = /^([\w-\.]+@([\w-]+\.)+[\w-]+)?$/;
			break;
		case "privacy": 
			if (jq('#'+input.id).attr("checked"))
				skipRegExp = true;
			break;
		case "nome":
			if(jq('#'+input.id).val()!="")

				skipRegExp = true;
			break;
		default:
			skipRegExp = true;		
	}
	
	//controllo se valido
	if(input.value=="" || input.value=="jq('#'+id).attr('placeholder')"){
		jq("#"+id+" .iconControl").hide().attr("src");
	}
	else if (skipRegExp || (input.value && filter && filter.test(input.value))) {
		jq("#"+id).removeClass("field_error");	//elimino la classe d'errore
		//se presente mostro l'iconcina di successo nell'input
		if (jq("#"+id+" .iconControl"))
			jq("#"+id+" .iconControl").show().attr("src", "/img2/flag.png");
	} else {
		//se presente mostro l'iconcina di errore nell'input
		if (jq("#"+id+" .iconControl") )
			jq("#"+id+" .iconControl").show().attr("src", "/img2/errore.gif");
	}
	
	//se non ci sono errori nascondo il box di errore
	if(jq("#contatta_agenzia_box .field_error").length <= 0)
		jq("#showError").slideUp();
}

/**
 * Controlla che tra uno tra mail e telefono sia compilato
 * @param (HTML Element) input ---> Elemento html dell'input da validare
 * @param (string) id ---> id dell'altro campo da controllare
 */
function checkPhoneOrEmail(input, id){
	//se uno dei due ha la classe d'errore e l'altro no li ritengo entrambi validi
	if (!jq("#contact_box_"+input.id).hasClass("field_error") && jq("#"+id).hasClass("field_error")) {
		jq("#"+id).removeClass("field_error");	
		jq("#"+id+" .iconControl").hide();
		
		//se non ci sono errori nascondo il box di errore
		if(jq("#contatta_agenzia_box .field_error").length <= 0)
			jq("#showError").slideUp();
	}
}
/**
 * Funzione che assegna lo stesse funzioni per ogni input del form nel caso di "onchange" e "onkeyup"
 */
function setContattaBoxEvent(){
		 addPlaceHolderCompatibility();
	//prendo tutti gli elementi di input e li scorro
	jq(".field-box input").each(function() {
		
		//assegno all'onchange all'onkeyup la seguente funzione
		jq(this).bind("keyup", function() {
			htmlElem = jq(this).get(0);	//prende l'elemento HTML dall'object JQuery
			//controlla il campo
			checkField(htmlElem.id, htmlElem, 'contact_box_'+htmlElem.id);
			//se telefono | email controlla se uno dei due è compilato
			if (htmlElem.id == "telefono")
				checkPhoneOrEmail(htmlElem, 'contact_box_email');
			else if (htmlElem.id == "email")
				checkPhoneOrEmail(htmlElem, 'contact_box_telefono');
		})
	})

}

function trackCustomVars(pageCustomVars){
	
	if (typeof pageCustomVars != "object")
		pageCustomVars = {};
		
	if (typeof _gaq != "undefined"){
		for (var cVar=0 ; cVar < pageCustomVars.length; cVar++){
			if (typeof pageCustomVars[cVar]=="object" && pageCustomVars[cVar].name && pageCustomVars[cVar].value && pageCustomVars[cVar].deep &&  pageCustomVars[cVar].level){
				var value = trim(pageCustomVars[cVar].value);
				var name = trim(pageCustomVars[cVar].name);
				_gaq.push(['_setCustomVar',
					pageCustomVars[cVar].deep,	// Profondità CVar (livallo categorizzazione)
					pageCustomVars[cVar].name,	// Nome variabile custom
					value,	// Valore custom var
					pageCustomVars[cVar].level	//Livello Variabile (1-Sessione, 2-utente, 3 - pagina)
				]);
				if (typeof __debugging_doLog!="undefined" && __debugging_doLog)
					_doLog("_gaq_push(['"+pageCustomVars[cVar].deep+"','"+name+"','"+value+"','"+pageCustomVars[cVar].level+"']);");

				if (typeof ga != "undefined"){
					ga('set',name, value);
					if (typeof __debugging_doLog!="undefined" && __debugging_doLog)
						_doLog("_gaq_push('"+name+"','"+value+"');");
				}
			}	
		}
	}
}

function trackClickEvent(type,cat){

	if (typeof type == "undefined")
		return false;
	
	if (typeof cat != "undefined")
		cat = trim(cat.ucwords().replace(/\s+/g,""));
	else
		cat = "";
	
	if (typeof _gaq == "undefined")
		return false;

	var category = null;
	var action = null;
	var opt_label = null;
	
	//type = [mappa, tabella, fotogallery, nuove_costruzioni, ]
	category = "ricercaAnnunciEvents";
	
	switch (type){
		
		case "visualizzazione_vetrina_ricerca":
			action = 'vetrina_ricerca';
			opt_label = 'visualizzazione_vetrina_ricerca';
			break;
			
		case "dettaglio_vetrina_ricerca":
			action = 'vetrina_ricerca';
			opt_label = 'dettaglio_annuncio_vetrina_ricerca';
			break;
	
		case "lista":
			action = 'tab_lista';
			opt_label = 'apertura-'+cat;
			break;
			
		case "dettaglio_annuncio_lista":
			action = 'tab_lista';
			opt_label = 'dettaglio_annuncio-'+cat;
			break;
			
		case "mappa":
			action = 'tab_mappa';
			opt_label = 'apertura-'+cat;
			break;
			
		case "click_annuncio_mappa":
			
			action = 'tab_mappa';
			opt_label = 'click_annuncio-'+cat;
			break;
			
		case "dettaglio_annuncio_mappa":
		
			action = 'tab_mappa';
			opt_label = 'dettaglio_annuncio-'+cat;
			break;
			
		case "tabella":
		
			action = 'tab_tabella';
			opt_label = 'apertura-'+cat;
			break;
			
		case "dettaglio_annuncio_tabella":
		
			action = 'tab_tabella';
			opt_label = 'dettaglio_annuncio-'+cat;	
			break;
			
		case "fotogallery":
		
			action = 'tab_fotogallery';
			opt_label = 'apertura-'+cat;
			break;
			
		case "dettaglio_annuncio_fotogallery":
		
			action = 'tab_fotogallery';
			opt_label = 'dettaglio_annuncio-'+cat;
			break;
			
		case "nuove_costruzioni":
		
			action = 'tab_nuove_costruzioni';
			opt_label = 'apertura-'+cat;
			break;
			
	}
	
	if (category!=null && action && opt_label){
	
		_gaq.push(['_trackEvent', category, action, opt_label]);
		if (typeof __debugging_doLog!="undefined" && __debugging_doLog)
			_doLog("_gaq.push(['_trackEvent','"+category+"','"+ action+ "','"+opt_label+"']);");

		if (typeof ga != "undefined"){
			ga('send', 'event', category, action, opt_label);
			if (typeof __debugging_doLog!="undefined" && __debugging_doLog)
				_doLog("ga('send','event','"+category+"','"+ action+ "','"+opt_label+"');");

		}
	}
	
	return true;
	
}

function trackViewedAdEvent(label){

	//tracciamento versioning algoritmo di generazione ancore, title, url (AUT)
	
	if (typeof label=="undefined" || !label)
		return false;
	
	if (typeof _gaq == "undefined")
		return false;
		
	category = "dettaglioAnnuncioEvents";
	action = 'AUT'; //acronimo per Ancora Url Title
	opt_label = label; //csv dei corrispondenti valori di versione
	
	_gaq.push(['_trackEvent', category, action, opt_label]);
	if (typeof __debugging_doLog!="undefined" && __debugging_doLog)
		_doLog("_gaq.push(['_trackEvent','"+category+"','"+ action+ "','"+opt_label+"']);");

	if (typeof ga != "undefined"){
		ga('send', 'event', category, action, opt_label);
		if (typeof __debugging_doLog!="undefined" && __debugging_doLog)
			_doLog("ga('send,'event','"+category+"','"+ action+ "','"+opt_label+"');");

	}

	return true;
	
}

function RegisterAdWordsConversion(){
	var img = document.createElement("img");
	img.style.width="1px";
	img.style.height="1px";
	img.style.border="none";
	img.src="http://www.googleadservices.com/pagead/conversion/1066595945/?label=-I_RCLPR7wEQ6ezL_AM&amp;guid=ON&amp;script=0";
	document.body.appendChild(img);
}



function thumbScroll( movement ) {
	if(-1 == jq.inArray(movement, ["up", "down"]))
		return;

	var jqImgContainer = jq('.border_box_thumb'),
		numFotoInView = 8,
		numThumb = jq('#thumbs').attr('data-numthumb');

	var imgMT = parseInt(jqImgContainer.css('margin-top'));
	imgMT = ((typeof imgMT === 'undefined') || isNaN(imgMT)) ? 0 : imgMT;
	var imgMB = parseInt(jqImgContainer.css('margin-bottom'));
	imgMB = ((typeof imgMB === 'undefined') || isNaN(imgMB)) ? 0 : imgMB;

	var heightRow = jqImgContainer.outerHeight() + imgMT + imgMB,
		heightView = (numFotoInView/2) * heightRow,
		heightMask = (numThumb/2) * heightRow,
		jqMask = jq('#thumbs-mask'),
		offTop = !jq.isEmpty( jqMask.attr('data-offset') ) ? jqMask.attr('data-offset') : 0,
		minScrollTop = -(heightMask - heightView),
		maxScrollTop = 0,
		browser = getBrowser(),
		bIsIe = browser.name.toLowerCase().search(/explorer/)>= 0,
		bVersion = browser.version;

	var _option = {nFoto: numThumb, hR: heightRow, hV: heightView, hM: heightMask, oT: Number(offTop), minST: Number(minScrollTop), maxST: Number(maxScrollTop), isIEBrowser: bIsIe, bVersion: bVersion};

	var _viewFreccie = {view: (numThumb > numFotoInView), up: (Math.abs(offTop) > 0), down: (Math.abs(_option.oT) <= heightView)};

	if('down' == movement){
		if( Math.abs(_option.oT) < Math.abs(minScrollTop) ){
			var _move = Number(offTop) - Number(heightRow);
			var _toMove = (_option._isIEBrowser && (_option.bVersion <= 6)) ? { marginTop: move } : { top: _move };

			jqMask.animate(_toMove, 200);
			jqMask.attr('data-offset', _move);
			offTop = !jq.isEmpty( jqMask.attr('data-offset') ) ? jqMask.attr('data-offset') : 0;
		}
	}
	if('up' == movement){
		if( Math.abs(_option.oT) > maxScrollTop ){
			var _move = Number(offTop) + Number(heightRow);
			var _toMove = (_option._isIEBrowser && (_option.bVersion <= 6)) ? { marginTop: move } : { top: _move };

			jqMask.animate(_toMove, 200);
			jqMask.attr('data-offset', _move);
			offTop = !jq.isEmpty( jqMask.attr('data-offset') ) ? jqMask.attr('data-offset') : 0;
		}
	}

	_viewFreccie.up = (Math.abs(offTop) > 0);
	_viewFreccie.down = !(Math.abs(offTop) >= Math.abs(minScrollTop));
	showFreccie();

	function showFreccie(){
		(_viewFreccie.up) ? jq('#vetrina_thumbs_up').css('visibility','visible') : jq('#vetrina_thumbs_up').css('visibility','hidden');
		(_viewFreccie.down) ? jq('#vetrina_thumbs_down').css('visibility','visible') : jq('#vetrina_thumbs_down').css('visibility','hidden');
	}
}

function showThumb(idOImg,imgUrl){

	var oImg = document.getElementById(idOImg);
	if (typeof idOImg == "undefined" || !oImg || typeof imgUrl == "undefined" || !oImg)
		return;
	oImg.originalSrc = oImg.src;
	oImg.src = "/img2/loader7.gif";
	oImg.src = imgUrl;
	
	addEvent(this,"mouseout",function(){
			oImg.src = oImg.originalSrc;
		
	})
}

function toggleFilter(idFilterBtn,idFilterDiv){
	var oDiv = document.getElementById(idFilterDiv);
	var oBtn = document.getElementById(idFilterBtn);
	if (oDiv.style.display == "none"){
		oDiv.style.display = "block";
		oBtn.innerHTML = "nascondi";
		oBtn.style.backgroundImage = "url(/img2/freccetta_down.png)";
	}else{
		oBtn.innerHTML = "mostra";
		oDiv.style.display = "none";
		oBtn.style.backgroundImage = "url(/img2/freccetta.png)";
	}

}    

function showPhotoplus(id,tipo,refObj){
	
	if (typeof __activeFotoplusId != "undefined"){
		
		oFpFactory.getPhotoPlus(__activeFotoplusId).hide(); 
	}
	if (typeof oFpFactory == "undefined"){
		oFpFactory = new MEPhotoPlusFactory();
	}
	pars = "idAnnuncio="+id+"&tipo="+tipo;
	oFpFactory.getPhotoPlus(id).setUrlPars(pars);
	
	if (typeof refObj!="undefined")
		oFpFactory.getPhotoPlus(id).objRefLeft = refObj;
	
	oFpFactory.getPhotoPlus(id).show();
	__activeFotoplusId = id;
	
}

function hidePhotoplus(id){
	if (typeof oFpFactory!="undefined" && oFpFactory.getPhotoPlus(id))
		oFpFactory.getPhotoPlus(id).hide(true); 
} 

//aggiunto page_section per la vetrina ricerca, ci potrebbe essere un annuncio che è presente in pagina nella lista e nella vetrina con lo stesso id
function toggle_remaining_desc(id,page_section){
	var section = (typeof page_section == "undefined") ? '' : page_section+"_";
	var desc = document.getElementById(section+"descrizione_"+id);
	var toggler = document.getElementById(section+"toggle_descrizione_"+id);
	var toggler_img = document.getElementById(section+"toggle_descrizione_img_"+id);
	var toggler_testo = document.getElementById(section+"toggle_link_testo_"+id);
	var reduceText = "riduci";
	var readMoreText = "continua a leggere";
	var lang = getPageLang();

	if (lang){
		var reduceTextTranslate = {"it": "riduci", "en": "reduce", "es": "reducir", "fr": "réduire", "de": "reduzieren", "pt": "reduzir", "gr": "μείωση", "ru": "уменьшить"};
		var reduceText = reduceTextTranslate[lang] ? reduceTextTranslate[lang] : "riduci";

		var readMoreTextTranslate = {"it": "continua a leggere", "en": "read more", "es": "leer más", "fr": "en savoir plus", "de": "mehr lesen", "pt": "leia mais", "gr": "διαβάστε περισσότερα", "ru": "читать далее"};
		var readMoreText = readMoreTextTranslate[lang] ? readMoreTextTranslate[lang] : "continua a leggere";
	}
	
	if (desc.open){
		desc.open = false;
		desc.style.height = desc.origHeight + "px";
		if (toggler_img){
			toggler_img.src = "/img2/icone/plus.png";
			toggler_img.alt="[+]";
			toggler_img.title="leggi di più";
		}else{
			toggler.innerHTML = "(continua a leggere)";
		}
		if(toggler_testo)
		    toggler_testo.innerHTML = readMoreText;
		
	}else{
		desc.open = true;
		desc.style.height = "";
		if (toggler_img){
			toggler_img.src = "/img2/icone/minus.png";
			toggler_img.alt="[-]";
			toggler_img.title="leggi di meno";
		}else{
			toggler.innerHTML = "(riduci)";
		}
		if(toggler_testo)
		    toggler_testo.innerHTML = reduceText;
	}
	
}

function getPageLang(){
	if (document.getElementsByTagName('html')[0].getAttribute('lang')){
		return document.getElementsByTagName('html')[0].getAttribute('lang');
	}
}

//aggiunto page_section per la vetrina ricerca, ci potrebbe essere un annuncio che è presente in pagina nella lista e nella vetrina con lo stesso id
function setDescProps(id,height,page_section){
    var section = (typeof page_section == "undefined") ? '' : page_section+"_";
    var desc = document.getElementById(section+"descrizione_"+id);
    var toggler = document.getElementById(section+"toggle_descrizione_"+id);
    var txtHeight = 0;
    
    if (!desc.textHeight){
	desc.style.height = height;
	desc.style.overflow= "hidden";
	var txt = desc.innerHTML;
	var oDiv = document.createElement('div');
	oDiv.innerHTML = txt;
	oDiv.style.display="none";
	desc.appendChild(oDiv);
	txtHeight = MEgetHeight(oDiv);
	desc.removeChild(oDiv);
	desc.textHeight = txtHeight;
    
    
	if (!desc.origHeight)
	    desc.origHeight = MEgetHeight(desc);
    
	if (txtHeight > desc.origHeight){
	    toggler.style.display = "inline";
	}else
	    desc.style.height = "";
    }
    
}

function stopPropagation() {
	if(window.event) {
		if(window.event.stopPropagation) {
			window.event.stopPropagation()
		} else {
			window.event.cancelBubble = true;
		}
	}
}

function preventDefault() {
	if(window.event && window.event.preventDefault) {
		window.event.stopPropagation();
	}
}

function simpleListingBookmark(el, idAnnuncio, isLogged, opt) {
	var userLogged = typeof isLogged != "undefined" && isLogged != null ? isLogged : __userLogged__,
		isSaved = jq(el).hasClass('link_salvato');

	opt = opt || {};

	stopPropagation();
	preventDefault();

	if (userLogged) {
		if (isSaved) {
			makeUnbookmarked(el);
		} else {
			makeBookmarked(el);
		}

		bookmarkWebApi({
			remove: isSaved,
			pk: idAnnuncio,
			isProgetto: opt.isProgetto,
			inVendita: opt.inVendita,
			inAffitto: opt.inAffitto
		});

	} else {
		create_salvaBox(idAnnuncio, opt);
	}
}

function makeBookmarked(el) {
	jq(el).addClass('link_salvato').removeClass('link_salva').html('Salvato');
}

function makeUnbookmarked(el) {
	jq(el).addClass('link_salva').removeClass('link_salvato').html('Salva');
}

function bookmarkWebApi(opt) {
	var params;

	if (!opt) {
		throw new TypeError('Param required');
	}

	params = {
		action: opt.remove ? 'delete' : 'add',
		id: opt.pk,
		mode: 'xml'
	};

	if (opt.isProgetto) {
		params.action += 'Progetto';
	} else {
		params.action += 'Annuncio';
	}

	if (opt.inVendita) {
		params.inVendita = 1;
	}

	if (opt.inAffitto) {
		params.inAffitto = 1;
	}

	return jq.ajax({
		method: 'GET',
		url: '/bookmarkannunci.php',
		data: params
	});
}

function bookmarkAnnuncio(idAnnuncio, isLogged, section, doFbAction, opt) {

	opt = opt || {};

	if (typeof section == "undefined" || !section)
		section = 'ricerca';
	
	if ( typeof doFbAction == "undefined" )
		doFbAction = true;
	
	if( typeof __userLogged__ == 'undefined' )
		__userLogged__ = false;
	
	var userLogged = typeof isLogged != "undefined" &&  isLogged != null ? isLogged : __userLogged__;

	if (userLogged) {
		fnSalvaAnnuncioAjax('bookmarkannunci.php', 'add', idAnnuncio, userLogged, section, doFbAction, opt);
	} else {
		create_salvaBox(idAnnuncio, opt);
	}
}


/*gestore ajax per salva annuncio*/
/*invio richiesta*/
function fnSalvaAnnuncioAjax(sUrlPage, sAction, sIdItem, isLogged, section, doFbAction, opt) {
	opt = opt || {};
	var sOptions = '';

	if (!jq.isEmpty(opt)) {
		jq.each(opt, function (index, value) {
			sOptions += '&' + index + '=' + value;
		});
	}

	savingLoading(isLogged, sIdItem, section);

	//costruisco la URL a cui inoltro la richiesta
	var sUrlRequestPage = "/" + sUrlPage + "?action=" + sAction + "&id=" + sIdItem + sOptions + "&mode=xml";

	var oAjax = new getMEAjaxObj();
	oAjax.Request('GET', sUrlRequestPage, function (data) {
		fnUpdateCounter(data, sIdItem, 0, section, doFbAction);
	});
}

/**
 * Cambia icona 'dettagli' in tipologie progetto
 * @param idLotto
 * @returns {boolean}
 */
function change_icon(idLotto) {
	var descIcon = document.getElementById("desc_icon_"+idLotto);
	if (typeof descIcon.isOpen == "undefined")
		descIcon.isOpen = false;
	if(!descIcon.isOpen) {
		descIcon.src = "/img2/icone/icona-dettagli[-].png";
		descIcon.isOpen=true;
	} else {
		descIcon.src = "/img2/icone/icona-dettagli[+].png";
		descIcon.isOpen=false;
	}
	return false;
}
/**
 * Salva un progetto tra i preferiti
 * @param idProgetto
 * @param isLogged
 * @param section
 * @param doFbAction
 * @param opt
 */
function bookmarkProgetto(idProgetto, isLogged, section, doFbAction, opt) {
	opt = opt || {};

	if (typeof section == "undefined" || !section) {
		section = 'ricerca';
	}
	if ( typeof doFbAction == "undefined" ) {
		doFbAction = true;
	}
	if( typeof __userLogged__ == 'undefined' ) {
		__userLogged__ = false;
	}
	var userLogged = typeof isLogged != "undefined" &&  isLogged != null ? isLogged : __userLogged__;
	if (userLogged) {
		fnSalvaAnnuncioAjax('bookmarkannunci.php', 'addProgetto', idProgetto, userLogged, section, doFbAction, opt);
	} else {
		opt.isProgetto = true;
		create_salvaBox(idProgetto, opt);
	}
}

function savingLoading(isLogged,aId,section){

	if (document.getElementById("vetrina_salva_"+aId))
		document.getElementById("vetrina_salva_"+aId).className = "link_salva loader_background";

	if (document.getElementById("salva_"+aId)) {
		var noborder = jq('#salva_' + aId).hasClass('noborder') ? ' noborder' : '';
		if (section == 'dettaglio') {
			document.getElementById('salva_' + aId).className = "link_saved loader_background" + noborder;
			document.getElementById('salva_' + aId).title = "Annuncio salvato nei preferiti";
		} else if ( section == 'ricerca-mappa' ) {
			document.getElementById('salva_'+aId).className = "ico-stella on";
			document.getElementById('salva_'+aId).title = "Annuncio salvato nei preferiti";
		} else {
			document.getElementById("salva_"+aId).className = "link_salva loader_background" + noborder;
		}
	}
	if (document.getElementById("ann_map_map_save_icon_"+aId))
		document.getElementById("ann_map_map_save_icon_"+aId).src = "/img2/LoadingSmall.gif";

}

/*gestione della risposta, aggiornamento contatore*/
function fnUpdateCounter(data, sIdItem, times, section, doFbAction){

	if (typeof times == "undefined")
		times = 0;
	if (typeof doFbAction == "undefined")
		doFbAction = true;
	
	var sResponse = data.responseText;

	if (sResponse == "NOT_LOGGED") {
		if (document.getElementById('salva_' + sIdItem)) {
			var oDocument = document.getElementById('salva_' + sIdItem);
			var noborder = jq('#salva_' + sIdItem).hasClass('noborder') ? ' noborder' : '';
			if (section == 'dettaglio') {
				oDocument.className = "link_saved" + noborder;
			}
			else if (section == 'ricerca-mappa') {
				oDocument.className = "ico-stella action-bookmark-annuncio";
				oDocument.title = "Memorizza questo annuncio tra i tuoi preferiti per ritrovarlo più velocemente";
			}
		}
		__selectedOvBox.show();
		return;
	} else if (sResponse == "ERROR") {
		if (document.getElementById('salva_' + sIdItem)) {
			var oDocument = document.getElementById('salva_' + sIdItem);
			var noborder = jq('#salva_' + sIdItem).hasClass('noborder') ? ' noborder' : '';
			if (section == 'dettaglio') {
				oDocument.className = "link_saved" + noborder;
			}
			else if (section == 'ricerca-mappa') {
				oDocument.className = "ico-stella action-bookmark-annuncio";
				oDocument.title = "Memorizza questo annuncio tra i tuoi preferiti per ritrovarlo più velocemente";
			}
		}
		return;
	}
	
	if(sResponse!=""){
	    _reload = true;
		try{
		var oCounter = document.getElementById('count_salvati');
		var oLinkProfilo = document.getElementById('linkProfilo');
		if ( section == 'ricerca-mappa') {
			jq('div[data-id="' + sIdItem + '"] i').addClass('on');
			jq('div[data-id="' + sIdItem + '"] span').html('Salvato');
			if (oCounter) { //se utente è loggato
				oCounter.innerHTML = parseInt(sResponse);
			}
		} else if (oLinkProfilo) { //se utente è loggato
			_reload = false;
			if (oCounter) {
				oCounter.innerHTML = parseInt(sResponse);
			}
			//tolgo il link salva annuncio e metto annuncio salvato
			if (document.getElementById('salva_'+sIdItem)){
				var noborder = jq('#salva_' + sIdItem).hasClass('noborder') ? ' noborder' : '';
				var oDocument = document.getElementById('salva_'+sIdItem);
				oDocument.href = "/bookmarkannunci.php?idAnnuncio=" + sIdItem;
				oDocument.onclick = '';
				if (section == 'dettaglio') {
					oDocument.className = "link_saved" + noborder;
					oDocument.innerHTML = "Salvato";
				} else {
					oDocument.className = "link_salvato" + noborder;
					oDocument.innerHTML = "<strong> Salvato </strong>";
				}
				if ( doFbAction )
					boxSalvaAnnuncioFacebook(sIdItem);
			}
			//aggiungo il cambio anche per l'annuncio in vetrina
			if (document.getElementById('vetrina_salva_' + sIdItem)) {
				var oDocument = document.getElementById('vetrina_salva_' + sIdItem);
				oDocument.className = "label_salvato";
				oDocument.innerHTML = "<strong> Salvato </strong>";
				oDocument.href = "/bookmarkannunci.php?idAnnuncio=" + sIdItem;
				oDocument.onclick = '';
			}
			if (document.getElementById('ann_map_salva_an_map_' + sIdItem)) {
				var oDocument = document.getElementById('ann_map_salva_an_map_' + sIdItem);
				oDocument.innerHTML = "<img src=\"/img2/icone/icona-check.png\" />Salvato";
				oDocument.href = "/bookmarkannunci.php?idAnnuncio=" + sIdItem;
				oDocument.onclick = '';
			}
			if (document.getElementById("map_save_icon_" + aId)) {
				document.getElementById("map_save_icon_" + aId).src = "/img2/icone/icona-check.png";
			}
		}else{
			window.location.reload();
		}
		
		}catch(e){
		    //alert(e);
		}
	}
}


function fnUpdateSaved(data,agArrayId){
        var sResponse = data.responseText;
        agArraySalvati = sResponse.split(',');
        for(var i=0;i<agArrayId.length;i++){
                if(document.getElementById('salva_loading_'+agArrayId[i]))
                    document.getElementById('salva_loading_'+agArrayId[i]).style.display="none";
                if(sResponse.search(agArrayId[i])>0)
                     document.getElementById('salvato_'+agArrayId[i]).style.display="inline";
                else
                     document.getElementById('salva_'+agArrayId[i]).style.display="inline";
        }
                
}

function checkIfSaveAd(isLogged, section) {
	if (!isLogged) {
		return false;
	}
	var objAnnuncio = getObjBookmarkFromCookie();
	if (!objAnnuncio) {
		return false;
	}
	var idAnnuncio = objAnnuncio.idAnnuncio;
	delete objAnnuncio.idAnnuncio;
	if (idAnnuncio) {
		if (!jq.isEmpty(objAnnuncio.isProgetto) && objAnnuncio.isProgetto ) {
			delete objAnnuncio.isProgetto;
			bookmarkProgetto(idAnnuncio, true, section, false, objAnnuncio);
		} else {
			bookmarkAnnuncio(idAnnuncio, true, section, null, objAnnuncio);
		}
		if (typeof selectOnlyAd === 'function') {
			selectOnlyAd(idAnnuncio, true);
		}
	}
	return idAnnuncio;
}

/**
 * Metodo che recupera il cookie in cui e' stato salvato l'annuncio.
 * @returns {*} = oggetto
 */
function getObjBookmarkFromCookie() {
	try {
		var _cookie = JSON.parse(readCookie("salvaAnnuncio"));
		deleteCookie("salvaAnnuncio");
		if (typeof _cookie == 'object' && typeof _cookie.idAnnuncio != 'undefined') {
			return _cookie;
		} else if (typeof _cookie == 'string') {
			var bookmarkOptions = {};
			bookmarkOptions.idAnnuncio = _cookie;
			return bookmarkOptions;
		}
	} catch(e) {
		deleteCookie("salvaAnnuncio");
		return null;
	}
}

function sendMessage(context) {
	var url;
	var invia = true;
	if(context == 'ricerca') {
		invia = controllaFormInviaAmico();
	}
        
    url = "/invia_ricerca_amico.php?invia=s&url="+document.getElementById("urlRicerca").value;
    url += "&nomeMittente="+document.getElementById("mittente").value;
    url += "&nomeDestinatario="+document.getElementById("destinatario").value;
    url += "&emailMittente="+document.getElementById("mailMittente").value;
    url += "&emailDestinatario="+document.getElementById("mailDestinatario").value;
    url += "&testo="+document.getElementById("messaggio").value;
    url += "&privacy=S";
    url += "&page="+document.getElementById("page").value;
    
    if(invia) {
		var ajaxObj = new getMEAjaxObj();
		ajaxObj.Request('POST',url,function(data){ __selectedOvBox.setContent(data.responseText);});
	}
}

function controllaFormInviaAmico() {
    var invia = true;
    if(typeof document.getElementById("destinatario") == "undefined" ||  trim(document.getElementById("destinatario").value).length == 0 ||  trim(document.getElementById("destinatario").value) == 'Campo obbligatorio') {
        campoObbligatorio("destinatario");
        invia = false;
    }
    if(typeof document.getElementById("mittente") == "undefined" ||  trim(document.getElementById("mittente").value).length == 0 ||  trim(document.getElementById("mittente").value) == 'Campo obbligatorio') {
        campoObbligatorio("mittente");
        invia = false;
    }
    if(typeof document.getElementById("mailDestinatario") == "undefined"  ||  trim(document.getElementById("mailDestinatario").value).length == 0 || !isEmail(document.getElementById("mailDestinatario").value) ||  trim(document.getElementById("mailDestinatario").value) == 'Campo obbligatorio') {
        campoObbligatorio("mailDestinatario");
        invia = false;
    }
    if(typeof document.getElementById("mailMittente") == "undefined" ||  trim(document.getElementById("mailMittente").value).length == 0 || !isEmail(document.getElementById("mailMittente").value) ||  trim(document.getElementById("mailMittente").value) == 'Campo Obbligatorio') {
        campoObbligatorio("mailMittente");
        invia = false;
    }
    if(typeof document.getElementById("privacy") == "undefined" ||  document.getElementById("privacy").checked == 0) {
        campoObbligatorio("privacy");
        invia = false;
    }
    return invia;
}

function clearObbligatorio( sender ) {
    var etichetta = 'etichetta_'+ sender.name;
	
    sender.style.borderColor = "#AAAAAA";
    document.getElementById( etichetta ).style.color = "#000000";    
}

function campoObbligatorio( campo ) {
    var etichetta = 'etichetta_'+document.getElementById( campo ).name;
    document.getElementById( campo ).style.borderColor = "#FF0000";
    document.getElementById( etichetta ).style.color = "#FF0000";
}


function setListAnnuncioEvents(aInfoAnnuncio){
	
	/* Mini map*/
	/* solo se c'è indirizzo e flagIndirizzo = segnalino (1) */
	if ( typeof __box_mappa_enabled != "undefined" && (parseFloat(aInfoAnnuncio.latitudine) || parseFloat(aInfoAnnuncio.longitudine) ) && aInfoAnnuncio.flagIndirizzo == 1){
		
		if (typeof small_map_markers == "undefined")
			small_map_markers = new Array();
		var seg = new Object();
		seg.id = aInfoAnnuncio.idAnnuncio;
		seg.lat = aInfoAnnuncio.latitudine;
		seg.lon = aInfoAnnuncio.longitudine;
		if (typeof getImmobiliareMapIcon != "undefined")
			seg.icon = getImmobiliareMapIcon();
		small_map_markers.push(seg);
	}
	
	var __adUrl = aInfoAnnuncio.url;
	var __adId = aInfoAnnuncio.idAnnuncio;
	
	addEvent(document.getElementById(aInfoAnnuncio.idAnnuncio),"mouseover",
			function(e){
				var el = e.target ? e.target : e.srcElement;
				try{
					var elId = el.id;
					var pos = elId.search("vetrina_");
					if (pos==0){
						elId = elId.substr(8);
					}
					switch (elId){
						case "toggle_descrizione_img_"+aInfoAnnuncio.idAnnuncio:
						case "salvaAd_"+aInfoAnnuncio.idAnnuncio:
						case "link_contattaAd_"+aInfoAnnuncio.idAnnuncio:
						case "link_cancellaAd_"+aInfoAnnuncio.idAnnuncio:
						case "fotoPlus_annuncio_"+aInfoAnnuncio.idAnnuncio:
							document.getElementById(aInfoAnnuncio.idAnnuncio).style.cursor="";
							break;
						default:
							document.getElementById(aInfoAnnuncio.idAnnuncio).style.cursor="pointer";
							
							break;
					}
					
					addEvent(document.getElementById(aInfoAnnuncio.idAnnuncio),"mousedown",registerBtnPress.bind(document.getElementById(aInfoAnnuncio.idAnnuncio)));
					addEvent(document.getElementById(aInfoAnnuncio.idAnnuncio),"mouseup",unregisterBtnPress.bind(document.getElementById(aInfoAnnuncio.idAnnuncio)));
				}catch(e){
					
				}
			}
		);
	
	addEvent(document.getElementById(aInfoAnnuncio.idAnnuncio),"mouseout",
		function(e){
			removeEvent(document.getElementById(aInfoAnnuncio.idAnnuncio),"mousedown",registerBtnPress.bind(document.getElementById(aInfoAnnuncio.idAnnuncio)));
			removeEvent(document.getElementById(aInfoAnnuncio.idAnnuncio),"mouseup",unregisterBtnPress.bind(document.getElementById(aInfoAnnuncio.idAnnuncio)));
		}
	);
	
	
	//addEvent(document.getElementById(idAnnuncio),"click",clickAnnuncio);
	
	/* Associazione annuncio con box mappa a sx*/
	
	if (typeof selectAd == "function"){
		addEvent(document.getElementById(aInfoAnnuncio.idAnnuncio),"mouseover",selectAd.bind(document.getElementById(aInfoAnnuncio.idAnnuncio)));
		addEvent(document.getElementById(aInfoAnnuncio.idAnnuncio),"mouseout",deselectClickedAd.bind(document.getElementById(aInfoAnnuncio.idAnnuncio)));
	}
	
}
_isCtrlPressed = false;

function registerKeyPress(e){
	__lastKeyReleased = null;
	var key = (window.event) ? window.event.keyCode : e.keyCode;
	__lastKeyPressed = key;
	if (_ctrlPressed()){
		setCtrlPressed();
	}
	
}


function registerKeyRelease(e){
	__lastKeyPressed = null;
	var key = (window.event) ? window.event.keyCode : e.keyCode;
	__lastKeyReleased = key;
	
	if (_ctrlReleased())
		unsetCtrlPressed();
}

function _ctrlPressed(){
	if (parseInt(__lastKeyPressed) == 17 || parseInt(__lastKeyPressed) == 224 || parseInt(__lastKeyPressed) == 91){ //CTRL PremostraBoxMutuouto
		return true;	
	}
	return false;
}

function _ctrlReleased(){
	if (parseInt(__lastKeyReleased) == 17 || parseInt(__lastKeyReleased) == 224 || parseInt(__lastKeyReleased) == 91){ //CTRL rilasciato
		return true;	
	}
	return false;
}

function setCtrlPressed(){
	_isCtrlPressed = true;
}
function unsetCtrlPressed(){
	_isCtrlPressed = false;
}
function isCtrlPressed(){
	return _isCtrlPressed;
}

function _isCtrl_clickSx(){
	if (typeof __lastBtnPressed != "undefined" && __lastBtnPressed == 1 && isCtrlPressed()){ //CTRL + click sinistro
		return true;	
	}
	return false;
}


function _clickSx(){
	if (typeof __lastBtnPressed != "undefined" && __lastBtnPressed == 1){ // click sinistro
		return true;	
	}
	return false;
}



function registerBtnPress(e){
	__ad_opened = false;
	var btn = (window.which) ? window.event.which : e.which;
	__lastBtnPressed = btn;
	
}

function unregisterBtnPress(e){
	
	var el = e.target ? e.target : e.srcElement;
	if (parseInt(__lastBtnPressed) == 2){
		var searchClass = "riga_annuncio";
		var pattern = new RegExp("(^|.*[\\s]+)"+searchClass+"([\\s]+.*|$)");
		while (!pattern.test(el.className) && el.parentNode){
			el = el.parentNode;
		}
		if (el && pattern.test(el.className)){
			clickAnnuncio(el.id,e);
			//el.onclick();
			__ad_opened = true;
			stop_event_bubbling(e);
		}
	}
}

function _checkNewWin(){
	forceNewWin = false;
	if (isCtrlPressed()){
		forceNewWin = true;
	}

	if (typeof __lastBtnPressed != "undefined" && parseInt(__lastBtnPressed) == 2){
		forceNewWin = true;
		__lastBtnPressed = null;
	}
	return forceNewWin;
}

	function clickAnnuncio(idAnnuncio,e){
		try{
			var el = document.getElementById(idAnnuncio);
			if (typeof e != "undefined") {
				el = e.target ? e.target : e.srcElement;
			}

			var elId = el.id;
			if (elId != 'make_blacklist_' + idAnnuncio && elId != 'remake_blacklist_' + idAnnuncio && elId != 'remove_blacklist_' + idAnnuncio) {
				stop_event_bubbling(e);
			}
			var pos = elId.search("vetrina_");
			if (pos==0){
				elId = elId.substr(8);
			}
			var clickStatus = true;
			switch (elId){
				case "toggle_descrizione_img_"+idAnnuncio:				
				case "salva_"+idAnnuncio:
				case "fbActionSave_"+idAnnuncio:
				case "facebookStatus_"+idAnnuncio:
				case "link_salvaAd_"+idAnnuncio:
				case "link_cancellaAd_"+idAnnuncio:
				case "link_contattaAd_"+idAnnuncio:
                case "toggler_"+idAnnuncio:
				case "btn_vedi_imm_asta_"+idAnnuncio:
				case "fotoPlus_annuncio_"+idAnnuncio:
				case "annuncio_new_tools_"+idAnnuncio:
					clickStatus = false;
					break;
				case "link_ad_"+idAnnuncio:					
					if ((!_isCtrl_clickSx() && !_clickSx()) || jq.browser.webkit){		
						clickStatus = false;
						break;
					}
				default:

                    if (el.className && el.className.indexOf("noClickAd") >= 0 ){
						clickStatus = false;
                        break;
                    }
                    
					var oA = document.getElementById("link_ad_"+idAnnuncio);
					
					if (oA){
						__forceNewWin = _checkNewWin();
						if (typeof __ad_opened == "undefined" || !__ad_opened){
							simulateClickOnLink(oA,__forceNewWin);
						}
					}
					
					break;
			}
			return clickStatus;
		}catch(e){
			return true;
		}
		return false;
	}
	
	function clickAgenzia(idAnnuncio,e){
	
		try{
			stop_event_bubbling(e);
			var el = document.getElementById(idAnnuncio);
		
			if (typeof e != "undefined")	
				var el = e.target ? e.target : e.srcElement;
		
			var elId = el.id;
			
			switch (elId){
				case "link_mail_"+idAnnuncio:				
				case "link_scopri_"+idAnnuncio:				
				
					break;
				case "link_ad_"+idAnnuncio:
					
					if (!_isCtrl_clickSx() && !_clickSx()){
						break;
					}
				default:
					
					var oA = document.getElementById("link_ad_"+idAnnuncio);
					
					if (oA){
						__forceNewWin = _checkNewWin();
						if (typeof __ad_opened == "undefined" || !__ad_opened){
							simulateClickOnLink(oA,__forceNewWin);
						}
					}
					
					break;
			}
		}catch(e){
			return true;
		}
		return false;
	}
    
    function clickAgenziaFranchising(idAnnuncio,e){
	
		try{
			stop_event_bubbling(e);
			var el = document.getElementById(idAnnuncio);
		
			if (typeof e != "undefined")	
				var el = e.target ? e.target : e.srcElement;
		
			var elId = el.id;
			
			switch (elId){
				case "link_ad_"+idAnnuncio:
					
					if (!_isCtrl_clickSx() && !_clickSx()){
						break;
					}
				default:
					
					var oA = document.getElementById("link_ad_"+idAnnuncio);
					
					if (oA){
						__forceNewWin = _checkNewWin();
						if (typeof __ad_opened == "undefined" || !__ad_opened){
							simulateClickOnLink(oA,__forceNewWin);
						}
					}
					
					break;
			}
		}catch(e){
			return true;
		}
		return false;
	}

function boxSalvaAnnuncioFacebook(idAnnuncio)
{
	var disabledBox = readCookie('autosaveDisabled');
	/*autosaveAds => definita in dettaglio.tpl e in ricerca.tpl*/
	if (typeof autosaveAds != "undefined" && autosaveAds) {
		oFacebookAction.doAction('save', idAnnuncio);
	} else if (disabledBox == null || !disabledBox) {
		var objBoxFacebook = new getMEAjaxObj();
		objBoxFacebook.Request("GET", "/boxPubblicaFacebook.php?content=save&idAnnuncio="+idAnnuncio, salvaAnnuncioFacebook);
		boxCondividi = new boxOverlay('save_fb', 'grey');
		boxCondividi.setTitle("<div style='font-size:14px'>Salva il tuo annuncio su facebook</div>");
	}
}

function salvaAnnuncioFacebook(data)
{
	var content = data.responseText;
	boxCondividi.setTitle("<div style='font-size:14px'>Salva il tuo annuncio su facebook</div>");
	boxCondividi.setContent(content);
	boxCondividi.show();
}

addEvent(window,"blur",function(){setTimeout(unsetCtrlPressed,500)});
addEvent(document,"keydown",registerKeyPress);
addEvent(document,"keyup",registerKeyRelease);

/**
 * Permette di digitare solo numeri e shortcut in un campo input.
 * 
 * Per usarlo nel mettere campo input "onkeydown='onlyNumberDigit(event);'"
 * @param event ---> Evento KeyPress
 */
function onlyNumberDigit(event) {
	var segnopiu = (window.event) ? 187:171;
	var keyCode = (window.event) ?    // MSIE or Firefox?
			   window.event.keyCode : (event.keyCode ? event.keyCode : event.which);
	// Allow: backspace, delete, tab, escape, and enter
	if ( keyCode == 46 || keyCode == 8 || keyCode == 9 || keyCode == 27 || keyCode == 13 || keyCode == segnopiu || 
		 // Allow: Ctrl+A
		(keyCode == 65 && event.ctrlKey === true) || 
		 // Allow: Ctrl+C
		(keyCode == 67 && event.ctrlKey === true) || 
		 // Allow: Ctrl+V
		(keyCode == 86 && event.ctrlKey === true) || 
		 // Allow: Ctrl+X
		(keyCode == 88 && event.ctrlKey === true) || 
		 // Allow: +
		(keyCode == 107) ||
		 // Allow: home, end, left, right
		(keyCode >= 35 && keyCode <= 39)) {
			 // let it happen, don't do anything
			 return;
	}
	else {
		// Ensure that it is a number and stop the keypress
		if (keyCode == 16 || (keyCode < 48 || keyCode > 57) && (keyCode < 96 || keyCode > 105 )) {
			if(event.preventDefault) 
				event.preventDefault();
			else {
				event.returnValue = false;
				return false;
			}
		}   
	}
}

// GESTIONE Bookmark
function ManagerNoteAd(options)
{
	this.options = typeof options != "undefined" ? options : new Array();
	this.isLogged = typeof this.options.isLogged != "undefined" ? this.options.isLogged : false;
	this.checkBookmarked = typeof this.options.checkBookmarked != "undefined" ? this.options.checkBookmarked : false;
	this.section = typeof this.options.section != "undefined" ? this.options.section : null;
	this.onSave = typeof this.options.onSave != "undefined" ? this.options.onSave : function () {};
	this.onDelete = typeof this.options.onDelete != "undefined" ? this.options.onDelete : function () {};
	this.boxAlwaysOpened = typeof this.options.boxAlwaysOpened != "undefined" ? this.options.boxAlwaysOpened : 1; // differenzia il comportamento tra bookmark e dettaglio
	this.minRowsTextArea = 1;
	this.maxRowsTextArea = 10;
}

ManagerNoteAd.prototype.init = function (selector) {
	var that = this;
	selector = !jq.isEmpty(selector) ? selector : '.action-note';
	jq(selector).unbind();
	jq(selector).each(function (i) {
		jq(this).attr('data-appended', 1).on('click', function () {
			var idRecord = jq(this).data('id');
			var action = jq(this).data('action');
			var isBookmarked = jq(this).data('bookmarked');
			var typeRecord = jq(this).data('type');
			that.actionNote(idRecord, action, isBookmarked, typeRecord);
		});
	});

	// controllo se esiste un cookie associato alla creazione di una nota.
	if (this.section == 'dettaglio')
		this.checkCookieNoteAd();
}

ManagerNoteAd.prototype.eventOnDelete = function (idRecord) {
	if (!this.boxAlwaysOpened) {
		jq("#nota-annuncio-" + idRecord).slideUp(500, function () {
			jq(this).removeClass('bg-white');
		});
		jq("#testo-nota-" + idRecord).html('');
	} else {
		jq("#box-testo-nota-" + idRecord).hide();
		jq("#box-init-nota-" + idRecord).show();
		jq("#testo-nota-" + idRecord).html('');
		jq("#box-modifica-nota-" + idRecord).slideUp(500, function () {
			jq("#box-nota-" + idRecord).fadeIn(300, function () {
				jq("#nota-annuncio-" + idRecord).removeClass('bg-white');
			});
		});
	}
	if (this.section == 'bookmark_annunci')
		jq("#blocco_note_" + idRecord).slideDown();

	this.onDelete();
}

ManagerNoteAd.prototype.eventOnUpdate = function (idRecord, text) {
	jq("#box-testo-nota-" + idRecord).show();
	jq("#box-init-nota-" + idRecord).hide();
	jq("#testo-nota-" + idRecord).html(text);

	jq("#overlaybox-action-note-" + idRecord).slideUp(500, function () {
		jq("#box-modifica-nota-" + idRecord).hide();
		jq("#nota-annuncio-" + idRecord).removeClass('bg-white');
		jq("#box-nota-" + idRecord).show();
	});

	this.onSave();
}

ManagerNoteAd.prototype.limitTextareaLine = function (elem, ev) {
	// se ho superato il numero di righe massimo disabilito il tasto invio
	if ((ev.keyCode || ev.which) == 13 && jq(elem).val().split('\n').length >= this.maxRowsTextArea) {
		ev.preventDefault();
		return false;
	}
}

ManagerNoteAd.prototype.eventOnEdit = function (idRecord, html, isBookmarked) {
	if (this.section == 'bookmark_annunci')
		jq("#blocco_note_" + idRecord).slideUp();

	if (!this.boxAlwaysOpened) {
		jq("#nota-annuncio-" + idRecord).slideDown(500);
	}

	var noteWithText = jq("#box-testo-nota-" + idRecord).is(":visible");
	jq("#box-modifica-nota-" + idRecord).html(html);
	jq("#nota-annuncio-" + idRecord).addClass('bg-white');

	if (noteWithText) {
		jq("#box-nota-" + idRecord).hide();
		jq("#box-modifica-nota-" + idRecord).show();
		jq("#overlaybox-action-note-" + idRecord).slideDown();
		jq("#modifica-nota-" + idRecord).focus();
	} else {
		jq("#box-nota-" + idRecord).fadeOut(300, function () {
			jq("#overlaybox-action-note-" + idRecord).show();
			jq("#box-modifica-nota-" + idRecord).slideDown();
			jq("#modifica-nota-" + idRecord).focus();
		});
	}

	// Aggiungo ai pulsanti del boxoverlay gli eventi.
	var that = this;
	jq("#box-contenitore-blocconote-" + idRecord + " .action-note").on('click', function () {
		// se i pulsanti dentro il boxoverlay sono disabilitati fermo l'evento
		if (jq(this).hasClass('disabled'))
			return;

		var action = jq(this).data('action');
		var idRecord = jq(this).data('id');
		var type = jq(this).data('type');
		that.actionNote(idRecord, action, isBookmarked, type);
	});
}

ManagerNoteAd.prototype.eventOnConfirm = function (idRecord, html) {
	this.box = new boxOverlay('notaAnnunio', 'grey');
	this.box.setTitle("<div style='font-size:14px; margin-top:3px; float:left; color:#666;'>Nota personale</div>");
	this.box.setContent(html);
	this.box.show();

	var that = this;
	jq("#conferma-nota-" + idRecord + " .action-note").on('click', function () {
		// se i pulsanti dentro il boxoverlay sono disabilitati fermo l'evento
		var action = jq(this).data('action');
		var idRecord = jq(this).data('id');
		var typeRecord = jq(this).data('type');
		that.actionNote(idRecord, action, 1, typeRecord);
		that.box.close();
	});
}

ManagerNoteAd.prototype.doAjax = function (idRecord, action, nota, isBookmarked, isFroomCookie, typeRecord) {
	if (typeof isFroomCookie == "undefined")
		isFroomCookie = 0;

	var that = this;
	jq.ajax({
		type: "POST",
		url: VHOST_URL_HOST + "bloccoNote.php",
		dataType: "xml",
		data: {
			idRecord: idRecord,
			action: action,
			nota: nota,
			type: typeRecord,
			isFroomCookie: isFroomCookie
		}
	}).done(function (mixed) {
		var action = jq(mixed).find('action').text();
		var message = jq(mixed).find('message').text();
		var html = jq(mixed).find('html').text();
		var errorCode = jq(mixed).find('errorcode').text();

		if (action == 'edit-note') {
			that.eventOnEdit(idRecord, html, isBookmarked);
		} else if (action == 'delete-note') {
			if (errorCode != '') {
				jq("#error-message-note-" + idRecord).html(message);
				jq("#error-message-note-" + idRecord).slideDown();
			} else {
				that.eventOnDelete(idRecord);
			}
		} else if (action == 'save-note') {
			if (errorCode != '') {
				jq("#error-message-note-" + idRecord).html(message);
				jq("#error-message-note-" + idRecord).slideDown();
			} else {
				if (message != '')
					that.eventOnUpdate(idRecord, message);
			}
		} else if (action == 'confirm-note') {
			that.eventOnConfirm(idRecord, html);
		}
	});
	return true;
}

// controlla la lunghezza quando viene inserita una nota
ManagerNoteAd.prototype.checkLengthNote = function (elem, idRecord, maxLength) {
	var bChecked = false;
	while (elem.rows > this.minRowsTextArea && elem.scrollHeight <= elem.offsetHeight) {
		elem.rows--;
		bChecked = true;
	}
	while (elem.rows < this.maxRowsTextArea && elem.scrollHeight > elem.offsetHeight) {
		elem.rows++;
		bChecked = true;
	}
	if (bChecked && elem.rows < this.maxRowsTextArea)
		elem.rows++;

	var len = jq(elem).val().length;
	if (len == 0) {
		jq("#overlaybox-action-note-" + idRecord + " .action-note[data-action='save-note']").addClass("disabled");
	} else {
		jq("#overlaybox-action-note-" + idRecord + " .action-note[data-action='save-note']").removeClass("disabled");
		jq("#overlaybox-action-note-" + idRecord + " .action-note[data-action='delete-note']").removeClass("disabled");
	}
	textCounter(elem, document.getElementById('len-nota-' + idRecord), maxLength);
}

ManagerNoteAd.prototype.actionNote = function (idRecord, action, isBookmarked, typeRecord)
{
	switch (action) {
		case 'save-note':
			var testoNota = typeof jq("#modifica-nota-" + idRecord).val() != 'undefined' ? jq("#modifica-nota-" + idRecord).val() : '';
			if (this.checkSession(idRecord, action, testoNota, isBookmarked, typeRecord)) {
				this.bookmarkAd(idRecord, isBookmarked, typeRecord);
				this.doAjax(idRecord, action, testoNota, isBookmarked, 0, typeRecord);
			}
			break;
		case 'delete-note':
			if (this.checkSession(idRecord, action, "", isBookmarked))
				this.doAjax(idRecord, action, "", isBookmarked, 0, typeRecord);
			break;
		case 'edit-note':
			this.doAjax(idRecord, action, "", isBookmarked, 0, typeRecord);
			break;
		case 'expand-note':
			if (this.section != 'dettaglio')
				this.expandNote(idRecord);
			break;
		case 'close-note':
			this.eventOnDelete(idRecord);
			break;
		default:
			break;
	}
}

ManagerNoteAd.prototype.createSSOBox = function (idRecord)
{
	__selectedOvBox = new boxOverlay('salvaAnnuncioBox_' + idRecord, 'grey');
	var loginOptions = {loginType: "U"}
	__selectedOvBox = create_accediBox('U', loginOptions);
	__selectedOvBox.show();
}

ManagerNoteAd.prototype.checkCookieNoteAd = function ()
{
	try {
		if (this.isLogged) {
			var oNota = jq.parseJSON(Base64.decode(readCookie("notaAnnuncio")));
			if (oNota && oNota.idRecord) {
				this.bookmarkAd(oNota.idRecord, oNota.isBookmarked, oNota.typeRecord);
				this.doAjax(oNota.idRecord, oNota.action, unescape(oNota.nota), oNota.isBookmarked, 1, oNota.typeRecord);
			}
		}
		deleteCookie("notaAnnuncio");
	} catch (e) {
		deleteCookie("notaAnnuncio");
	}
}

ManagerNoteAd.prototype.bookmarkAd = function (idRecord, isBookmarked, typeRecord)
{
	if (this.checkBookmarked && parseInt(isBookmarked) == 0) {
		if (typeRecord == 'progetto') {
			bookmarkProgetto(idRecord, this.isLogged, this.section, false)
		} else {
			bookmarkAnnuncio(idRecord, this.isLogged, this.section, false);
		}
		jq(".action-note[data-id='" + idRecord + "']").each(function () {
			jq(this).attr('data-bookmarked', 1);
		});
	}
}

ManagerNoteAd.prototype.checkSession = function (idRecord, action, testonota, isBookmarked, typeRecord)
{
	if (!this.isLogged) {
		createCookie("notaAnnuncio", Base64.encode(stringify({ idRecord: idRecord, nota: escape(testonota), action: action, isBookmarked: isBookmarked, typeRecord: typeRecord })));
		this.createSSOBox(idRecord);
		return false;
	}
	return true;
}

ManagerNoteAd.prototype.expandNote = function (idRecord)
{
	if (jq('#testo-nota-' + idRecord).hasClass('nota-aperta')) {
		jq('#testo-nota-' + idRecord).animate({ 'max-height': '16px'}, 500, function () {
			jq(this).toggleClass('nota-aperta');
		});
	} else {
		jq('#testo-nota-' + idRecord).animate({ 'max-height': '100px'}, 500, function () {
			jq(this).toggleClass('nota-aperta');
		});
	}
}

/**
 * Funzione che viene eseguita al click della riga annuncio.
 *
 * @param element = elemento su cui avviene il click.
 * @param event = evento click
 * @param selectTab = il tab selezionato.
 */
function onClickRigaAnnuncio(element, event, selectTab) {
	var jqElement = jq(element);
	var isVetrina = !jq.isEmpty(jqElement.data('isvetrina')) ? jqElement.data('isvetrina') : false;
	var cVarCategory = !jq.isEmpty(jqElement.data('category')) ? jqElement.data('category') : '';
	var trackSuffix = !jq.isEmpty(jqElement.data('tracksuffix')) ? jqElement.data('tracksuffix') : '';
	var section = !jq.isEmpty(jqElement.data('section')) ? jqElement.data('section') : '';
	var subSection = !jq.isEmpty(jqElement.data('subsection')) ? jqElement.data('subsection') : '';
	var idAnnuncio = !jq.isEmpty(jqElement.data('id')) ? jqElement.data('id') : null;
	if (!idAnnuncio) {
		return;
	}
	if (clickAnnuncio(idAnnuncio, event)) {
		if(isVetrina) {
			trackClickEvent('dettaglio_vetrina_ricerca');
		}
		switch (section) {
			case 'bookmark_annunci_salvati':
				if (subSection == 'ultimiAnnunci') {
					trackGAClickEvent('AreaUtentiEvents', 'Annunci Recenti', 'click');
				} else {
					trackGAClickEvent('AreaUtentiEvents', 'Annunci Salvati', 'click');
				}
				break;
			default:
				trackClickEvent('dettaglio_annuncio_' + selectTab, cVarCategory + trackSuffix);
				break;
		}
	}
}

/**
 * Gestione degli annunci in blacklist
 * @param options
 * @constructor
 */
function ManagerBlacklistAnnunci(options)
{
	this.options = jq.isEmpty(options) ? {} : options;
	if (typeof __userLogged__ == 'undefined') {
		__userLogged__ = false;
	}
	this.isLogged = !jq.isEmpty(this.options.isLogged) && this.options.isLogged != null ? this.options.isLogged : __userLogged__;
	this.typePageListing = jq.isEmpty(this.options.typePageListing) ? false : this.options.typePageListing;
	this.typePageDetail = jq.isEmpty(this.options.typePageDetail) ? false : this.options.typePageDetail;
	this.typePageListingMap = jq.isEmpty(this.options.typePageListingMap) ? false : this.options.typePageListingMap;
	this.section = jq.isEmpty(this.options.section) ? null : this.options.section;
	this.containerAds = !jq.isEmpty(this.options.containerAds) ? jq(this.options.containerAds) : jq('#annunciContainer');
	this.containerBlacklisted = !jq.isEmpty(this.options.containerBlacklisted) ? jq(this.options.containerBlacklisted) : jq('#container_blacklisted');
	this.counterBlacklisted = !jq.isEmpty(this.options.counterBlacklisted) ? jq(this.options.counterBlacklisted) : jq('#counter_blacklisted');
	this.counterBlacklistedMenu = !jq.isEmpty(this.options.counterBlacklistedMenu) ? jq(this.options.counterBlacklistedMenu) : jq('#counter_blacklisted_menu');
    this.counterBlacklistedMenuTop = !jq.isEmpty(this.options.counterBlacklistedMenuTop) ? jq(this.options.counterBlacklistedMenuTop) : jq('#counter_blacklisted_menu_top');
	this.counterBlacklistedToken = !jq.isEmpty(this.options.counterBlacklistedToken) ? jq(this.options.counterBlacklistedToken) : jq('#counter_blacklisted_token');
	this.counterListing = !jq.isEmpty(this.options.counterListing) ? jq(this.options.counterListing) : jq('#annListaCounter');
	this.containerPagination = !jq.isEmpty(this.options.containerPagination) ? jq(this.options.containerPagination) : jq('#footer_pag');
	this.prefixRow = "#wrapper_riga_"; //prefisso per identificare riga annuncio al quale verra' concatenato l'id
	this.prefixBtnMake = "make_blacklist_"; //prefisso per identificare l'id del bottone metti in blacklist
	this.prefixBtnRemove = "remove_blacklist_"; //prefisso per identificare il bottone rimuovi dalla blacklist
	this.prefixBtnReMake = "remake_blacklist_"; //prefisso per identificare l'id del bottone rimetti in blacklist
	this.timeSlideUp = 700;
	this.managerMarkers = jq.isEmpty(this.options.managerMarkers) ? null : this.options.managerMarkers;
	this.managerUser = jq.isEmpty(this.options.managerUser) ? null : this.options.managerUser;
	this.tokenLastSearch = jq.isEmpty(jq('#tokenLastSearch').val()) ? null : jq("#tokenLastSearch").val();
	this.criterio = jq.isEmpty(jq('#criterio').val()) ? null : jq("#criterio").val();
	this.ordine = jq.isEmpty(jq('#ordine').val()) ? null : jq("#ordine").val();
}

ManagerBlacklistAnnunci.prototype.init = function () {
	var self = this;

	this.containerAds.on('click', '.link_make_blacklist, .link_remake_blacklist, .link_make_blacklist_detail', function (e) {
		e.preventDefault();

		var id = jq(this).data('id');

		if (!self.isLogged) {
			self.createSSOBox(id);
			return false;
		}

		if (!jq(this).hasClass('clicked')) {
			jq(this).addClass('clicked');
			if (self.typePageDetail) {
				self.makeBlacklistDetail(id);
			} else if (self.typePageListing) {
				if (jq(this).hasClass('link_remake_blacklist')) {
					self.reMakeBlacklist(id);
				} else {
					self.makeBlacklist(id);
				}
			}
		}
	});

	this.containerAds.on('click', '.link_remove_blacklist, .link_remove_blacklist_detail', function (event) {
		event.preventDefault();
		if (!jq(this).hasClass('clicked')) {
			jq(this).addClass('clicked');
			var id = jq(this).data('id');
			if (self.typePageDetail) {
				self.removeBlacklistDetail(id);
			} else if (self.typePageListing) {
				self.removeBlacklist(id);
			}
		}
	});

	if (this.typePageListing && this.isLogged && this.section != 'blacklist_annunci') {
		//mostro il numero di annunci blacklistati per questa determinata ricerca
		this.showCounterBlacklistedBySearch();
	}

};

ManagerBlacklistAnnunci.prototype.createSSOBox = function (id)
{
	__selectedOvBox = new boxOverlay('blacklistBox_' + id, 'grey');
	__selectedOvBox = create_accediBox('U', {loginType: "U"});
	__selectedOvBox.show();
};

ManagerBlacklistAnnunci.prototype.getCurrentPage = function () {
	return this.containerPagination.find('a.page_num_act').text();
};

ManagerBlacklistAnnunci.prototype.makeBlacklist = function (id) {
	var self = this;

	var btnMake = jq('#' + this.prefixBtnMake + id);
	var type = btnMake.data('type');

	var data = {
		outputType: 'json',
		id: id,
		section: this.section,
		tokenLastSearch: this.tokenLastSearch,
		criterio: this.criterio,
		ordine: this.ordine
	};
	jQuery.extend(data, this.getOtherDataForMake());

	if (type == 'ad') {
		data.action = "addAnnuncio";
	} else if (type == 'pj') {
		data.action = "addProgetto";
	} else {
		return false;
	}

	btnMake.addClass('loader_background');

	this.doAjax(data).then(
		function (res, textStatus, jqXHR) {
			//onSuccess
			btnMake.removeClass('loader_background');
			self.onAdd(id, res.data);
		}, function (jqXHR, textStatus, errorThrown) {
			//onError
			btnMake.removeClass('loader_background');
		}
	);
};

ManagerBlacklistAnnunci.prototype.doAjax = function (data) {
	return jq.ajax({
		type: "POST",
		url: "/blacklist_annunci.php",
		data: data
	});
};

ManagerBlacklistAnnunci.prototype.onAdd = function (id, data) {
	var self = this;
	var row = jq(this.prefixRow + id);

	row.addClass('remove_item_animation');
	row.slideUp(this.timeSlideUp, function() {
		jq(this).remove();
	});

	//aggiungo l'annuncio successivo
	if (!jq.isEmpty(data.next)) {
		var newItem = data.next;
		if (!this.containerAds.hasClass('fgAnnunci')) {
			//se sono in modalita' fotogallery non eseffuo l'animazione
			newItem = jq(newItem).addClass('new_item_animation');
		}
		if (!jq.isEmpty(this.containerAds.find('.fgPrePagination'))) {
			this.containerAds.find('.fgPrePagination').before(newItem);
		} else {
			this.containerAds.append(newItem);
		}
	}

	//aggiorno la paginazione
	if (!jq.isEmpty(data.pagination)) {
		this.containerPagination.html(data.pagination);
	}

	//aggiorno il contatore nel footer degli annunci blacklistati
	var newCounterBlacklisted = this.numberPlusOrMinus(this.counterBlacklisted.html(), true);
	this.counterBlacklisted.html(newCounterBlacklisted);
	if (!this.containerBlacklisted.is(':visible')) {
		this.containerBlacklisted.show();
	}
	var newLabelCounterBlacklisted = newCounterBlacklisted == 1 ? 'annuncio' : 'annunci';
	jq('#label_counter_blacklisted').html(newLabelCounterBlacklisted);

	//aggiorno il contatore in testata
	this.counterListing.html(this.numberPlusOrMinus(this.counterListing.html(), false));
};

ManagerBlacklistAnnunci.prototype.numberPlusOrMinus = function (num, condition) {
	num = parseInt(num.replace('.', ""), 10);
	var operations = {
		'+': function (a, b) {
			return a + b;
		},
		'-': function (a, b) {
			return a - b;
		}
	};
	num = operations[condition ? '+' : '-'](num, 1);

	return num.formatNumber(0, ',', '.');
};

ManagerBlacklistAnnunci.prototype.removeBlacklist = function (id) {
	var self = this;

	var btnRemove = jq('#' + this.prefixBtnRemove + id);
	btnRemove.addClass('loader_background');

	var onlyNext = 1;
	var page = this.getCurrentPage();
	var numRemainingElements = this.containerAds.find('.link_remove_blacklist').length;
	if (numRemainingElements == 1 && page > 1) {
		//devo caricare tutti gli elementi della pagina precedente se esiste
		page--;
		onlyNext = 0;
	}
	var data = {
		outputType: 'json',
		section: this.section,
		action: "disableRecord",
		id: id,
		uba_id: btnRemove.data('uba_id'),
		tokenLastSearch: this.tokenLastSearch,
		pag: page,
		onlyNext: onlyNext
	};
	this.doAjax(data).then(
		function (res, textStatus, jqXHR) {
			//onSuccess
			btnRemove.removeClass('loader_background');
			self.onRemove(id, res.data);
		}, function (jqXHR, textStatus, errorThrown) {
			//onError
			btnRemove.removeClass('loader_background');
		}
	);
};

ManagerBlacklistAnnunci.prototype.onRemove = function (id, data) {
	//rimuovo la riga dalla lista
	jq(this.prefixRow + id).addClass('remove_item_animation');
	jq(this.prefixRow + id).slideUp(this.timeSlideUp, function () {
		jq(this).remove();
	});

	//aggiungo l'annuncio successivo
	if (!jq.isEmpty(data.next)) {
		var newItem = jq(data.next).addClass('new_item_animation');
		this.containerAds.append(newItem);
	}

	//aggiorno la paginazione
	if (!jq.isEmpty(data.pagination)) {
		this.containerPagination.html(data.pagination);
	}

	//aggiorno il contatore nel tab
	this.counterBlacklistedMenu.html(this.numberPlusOrMinus(this.counterBlacklistedMenu.html(), false));

	//aggiorno il contatore nel menu top
	this.counterBlacklistedMenuTop.html(this.counterBlacklistedMenu.html());

	if (!jq.isEmpty(this.counterBlacklistedToken)) {
		this.counterBlacklistedToken.html(this.numberPlusOrMinus(this.counterBlacklistedToken.html(), false));
		var newLabelCounterBlacklistedToken = this.counterBlacklistedToken.html() == 1 ? 'annuncio nascosto' : 'annunci nascosti';
		jq('#label_counter_blacklisted_token').html(newLabelCounterBlacklistedToken);
	}

};

ManagerBlacklistAnnunci.prototype.showCounterBlacklistedBySearch = function () {

	var self = this;

	var data = {
		outputType: 'json',
		action: "getCountBlacklistedBySearch",
		section: this.section,
		tokenLastSearch: this.tokenLastSearch
	};
	this.doAjax(data).then(
		function (res, textStatus, jqXHR) {
			//onSuccess
			if(res.data.count != 0){
				self.counterBlacklisted.html(res.data.count.formatNumber(0, ',', '.'));
				self.containerBlacklisted.show();
			}
		}, function (jqXHR, textStatus, errorThrown) {
			//onError
		}
	);
};

ManagerBlacklistAnnunci.prototype.makeBlacklistDetail = function (id) {
	var self = this;

	var btnMake = jq('#' + this.prefixBtnMake + id);
	var type = btnMake.data('type');

	var data = {
		outputType: 'json',
		id: id,
		section: this.section
	};
	if (type == 'ad') {
		data.action = "addAnnuncioDetail";
	} else if (type == 'pj') {
		data.action = "addProgettoDetail";
	} else {
		return false;
	}

	btnMake.addClass('loader_background');

	this.doAjax(data).then(
		function (res, textStatus, jqXHR) {
			//onSuccess
			btnMake.removeClass('loader_background');
			btnMake.hide();
			jq('#containerBoxRemoveBlacklist').html(res.data.boxNotice).show();
		}, function (jqXHR, textStatus, errorThrown) {
			//onError
			btnMake.removeClass('loader_background');
			btnMake.removeClass('clicked');
		}
	);
};

ManagerBlacklistAnnunci.prototype.removeBlacklistDetail = function (id) {
	var self = this;

	var btnRemove = jq('#' + this.prefixBtnRemove + id);
	var btnMake = jq('#' + this.prefixBtnMake + id);
	btnRemove.addClass('loader_background');

	var data = {
		outputType: 'json',
		section: this.section,
		action: "disableRecordDetail",
		id: id,
		uba_id: btnRemove.data('uba_id')
	};
	this.doAjax(data).then(
		function (res, textStatus, jqXHR) {
			//onSuccess
			btnMake.replaceWith(res.data.btnMake);
			jq('#containerBoxRemoveBlacklist').hide();
			btnMake.show();
		}, function (jqXHR, textStatus, errorThrown) {
			//onError
			btnRemove.removeClass('loader_background');
			btnRemove.removeClass('clicked');
		}
	);
};

ManagerBlacklistAnnunci.prototype.makeBlacklistMap = function (elm, opt) {
	opt = opt || {};

	var self = this;
	var btnMake = jq(elm);
	var id = btnMake.data('id');
	var row = btnMake.closest(".riga_annuncio");

	if (!self.isLogged && !self.managerUser.isLogged) {
		self.createSSOBox(id);
		return false;
	}

	if (btnMake.hasClass('clicked')) {
		return;
	}

	btnMake.addClass('clicked');
	btnMake.addClass('loader_background');

	var type = btnMake.data('type');
	var data = {
		outputType: 'json',
		id: id,
		section: this.section
	};
	if (type == 'ad') {
		data.action = "addAnnuncioMap";
	} else if (type == 'pj') {
		data.action = "addProgettoMap";
	} else {
		return false;
	}

	if (btnMake.hasClass('link_remake_blacklist_map')) {
		data.uba_id = btnMake.data('uba_id');
		data.reBlacklist = 1;
	}

	this.doAjax(data).then(
		function (res, textStatus, jqXHR) {
			//onSuccess
			var intvalPosition = setInterval(function(){
				self.managerMarkers._labelDiv.setPosition();
			}, 10);

			//rimuovo la riga dalla lista 
			if (!jq.isEmpty(opt.noSlideUp)) {
				clearInterval(intvalPosition);
				self.managerMarkers.reloadAll();
			} else {
				row.addClass('remove_item_animation');
				row.slideUp(self.timeSlideUp, function () {
					clearInterval(intvalPosition);
					self.managerMarkers.reloadAll();
				});
			}
		}, function (jqXHR, textStatus, errorThrown) {
			//onError
			btnMake.removeClass('loader_background');
			btnMake.removeClass('clicked');
		}
	);
};

ManagerBlacklistAnnunci.prototype.reMakeBlacklist = function (id) {
	var self = this;

	var btnReMake = jq('#' + this.prefixBtnReMake + id);
	var type = btnReMake.data('type');

	var data = {
		outputType: 'json',
		id: id,
		uba_id: btnReMake.data('uba_id'),
		section: this.section,
		tokenLastSearch: this.tokenLastSearch,
		criterio: this.criterio,
		ordine: this.ordine
	};
	jQuery.extend(data, this.getOtherDataForMake());
	if (type == 'ad') {
		data.action = "reAddAnnuncio";
	} else if (type == 'pj') {
		data.action = "reAddProgetto";
	} else {
		return false;
	}

	btnReMake.addClass('loader_background');

	this.doAjax(data).then(
		function (res, textStatus, jqXHR) {
			//onSuccess
			btnReMake.removeClass('loader_background');
			self.onAdd(id, res.data);
		}, function (jqXHR, textStatus, errorThrown) {
			//onError
			btnReMake.removeClass('loader_background');
		}
	);
};

ManagerBlacklistAnnunci.prototype.getOtherDataForMake = function () {
	var otherData;
	var onlyNext = 1;
	var page = this.getCurrentPage();
	var numRemainingElements = this.containerAds.find('.link_make_blacklist').length;
	numRemainingElements += this.containerAds.find('.link_remake_blacklist').length;
	if (numRemainingElements == 1 && page > 1) {
		//devo caricare tutti gli elementi della pagina precedente se esiste
		page--;
		onlyNext = 0;
	}
	return otherData = {
		pag: page,
		onlyNext: onlyNext
	};
};

function ManagerChat(opt) {
	this.opt = jq.isEmpty(opt) ? {} : opt;
	this.idAgenzia = jq.isEmpty(this.opt.idAgenzia) ? null : this.opt.idAgenzia;
	this.containerContactBtn = jq.isEmpty(this.opt.containerContactBtn) ? jq('.containerContactBtn') : jq(this.opt.containerContactBtn);
	this.containerChatBtn = jq.isEmpty(this.opt.containerChatBtn) ? jq('.containerChatBtn') : jq(this.opt.containerChatBtn);
	this.contactChatBtn = jq.isEmpty(this.opt.contactChatBtn) ? jq('.contactChatBtn') : jq(this.opt.contactChatBtn);
	this.timePolling = 15000;
	this.stopPolling = false;
	this.popUpOpened = false;
	this.wsChatStatusUrl = '/services/ws_chat.php?action=status&idAgenzia=%%ID_AGENZIA%%';
	this.enabledLog = !jq.isEmpty(this.opt.enabledLog) ? this.opt.enabledLog : false;
	this.utils = new _gestioneAnnuncioUtils({enabledLog: this.enabledLog});
}

ManagerChat.prototype.init = function () {
	var self = this;

	if (!__gvs_CHAT_ENABLED) {
		return;
	}

	if (!document.addEventListener) {
		//La chat non e' compatibile con versioni di IE inferiori alla versione 9
		//vedi: https://developer.mozilla.org/en-US/docs/Web/API/EventTarget.addEventListener
		return;
	}

	if (jq.isEmpty(this.idAgenzia)) {
		return;
	}

	this.checkChatPolling();

	//in ascolto sul click sul bottone inizia chat
	jq(this.containerContactBtn).on('click', '.contactChatBtn', function (e) {
		var $form = jq(this).closest('form'),
			$token = $form.find('input[name="token"]'),
			$tokenChat = $form.find('input[name="tokenChat"]'),
			source = $form.find('input[name="source"]').val();

		//aggiungo un time al tokenChat per rendere univoci i successivi click
		$tokenChat.val($token.val() + '-' + new Date().format('His'));

		if (isIE()) {
			//check placeholder for IE
			$form.find('input[type="text"], textarea').each(function () {
				self.clearInputIfPlaceholder(jq(this));
			});
		}

		var windowName = 'popUpChat_' + $form.find('input[name="id"]').val(),
			paramsSerializedForUrl = $form.serialize(),
			popUpUrl = __gvs_CHAT_URL_IMMOBILIARE_INDEX + '?' + paramsSerializedForUrl,
			addDebugChatUrl = '/services/ws_chat.php?action=addDebugChat';

		if (isIE()) {
			//ripristino i placeholder for IE
			$form.find('input[type="text"], textarea').each(function () {
				self.setPlaceholderToIE(jq(this));
			});
		}

		if (self.popUpOpened && !self.popUpOpened.closed) {  //checks to see if window is open
			self.popUpOpened.focus();
		} else {
			//tracciamento del click
			trackGAClickEvent('ImmobiliareChatEvents', 'open-window', source);

			//invio al ws i dati prima di inizializzare la chat per salvare i dati di debug
			jQuery.post(addDebugChatUrl, paramsSerializedForUrl)
				.done(function () {
					self.utils._log('addDebugChat: post inviata con successo');
				}).fail(function () {
					self.utils._log('addDebugChat: post fallita');
				});
			//apro la window con la live chat
			self.popUpOpened = popup(popUpUrl, windowName, 650, 650);
			self.popUpOpened.focus();
		}

		e.preventDefault(); // stop the browser's default behaviour
	});

};

ManagerChat.prototype.clearInputIfPlaceholder = function ($element) {
	if ($element.val() === $element.attr('placeholder')) {
		$element.val('');
	}
};

ManagerChat.prototype.setPlaceholderToIE = function ($element) {
	if ($element.val() === '') {
		$element.val($element.attr('placeholder'));
	}
};

ManagerChat.prototype.checkChatPolling = function () {
	var self = this,
		idAgenzia = this.idAgenzia;

	if (this.stopPolling) {
		return;
	}

	if (typeof __gvs_CHAT_DEMO_MODE !== "undefined" && __gvs_CHAT_DEMO_MODE) {
		this.utils._log('checkChatPolling: ricevuto idAgenzia: ' + idAgenzia + ' ma forzo a ' + __gvs_CHAT_DEMO_AGENCY_ID);
		idAgenzia = __gvs_CHAT_DEMO_AGENCY_ID;
	}

	jQuery.get(self.wsChatStatusUrl.replace('%%ID_AGENZIA%%', idAgenzia))
		.done(function () {
			self.containerContactBtn.addClass('hasChat');
			self.containerChatBtn.removeClass('btnDisabled');
			self.contactChatBtn.prop({"disabled": false, "title":""});
		}).fail(function (xhr, statusText) {
			if (xhr.status === 503) {
				self.stopPolling = true;
			}
			if (self.containerContactBtn.hasClass('hasChat')) {
				self.containerChatBtn.addClass('btnDisabled');
				self.contactChatBtn.prop({"disabled": true, "title":"Chat momentanemente non disponibile, l'agenzia non è online."});
			}
		})
		.always(setTimeout(function () {
			self.checkChatPolling()
		}, self.timePolling));
};;
 _lastErrMsgSetting = false;
 _isAlertEmail = false;
 _isSearchModify = false;
 _isNoGrazie = false;
 _disableAlertPopUp = false;
 __TI_servizio_1 = false;
 __TI_servizio_2 = false;
 __zone_showed = false;
 __blockTIAutoPopup = false;
 __tiReloadOnClose = false;
_searchNameManager = null;

 /* preload images */
 preloadImages(new Array("/img2/loader7.gif"));

 function resetTIFields(field){
    if (typeof field=="undefined")
        field = null;
	
    if ( field ) {
		var tagName = mapErrorFieldName( field );
		removeError( tagName, false, true );

    }else{
      if (document.getElementById('TI_servizio_1_txt'))
        document.getElementById('TI_servizio_1_txt').style.color="";

      if (document.getElementById('TI_comune_txt'))
        document.getElementById('TI_comune_txt').style.color="";

      if (document.getElementById('TI_comune'))
        document.getElementById('TI_comune').style.borderColor="";

      if (document.getElementById('zoneTI_descrizione'))
         document.getElementById('zoneTI_descrizione').style.borderColor="";

      if (document.getElementById('zoneTI_descrizione_txt'))
         document.getElementById('zoneTI_descrizione_txt').style.color="";

      if (document.getElementById('TI_email_txt'))
         document.getElementById('TI_email_txt').style.color="";

      if (document.getElementById('TI_email'))
         document.getElementById('TI_email').style.borderColor="";

      if (document.getElementById('TI_password_txt'))
         document.getElementById('TI_password_txt').style.color="";

      if (document.getElementById('TI_password'))
         document.getElementById('TI_password').style.borderColor="";

      if (document.getElementById('TI_nome_txt'))
         document.getElementById('TI_nome_txt').style.color="";

      if (document.getElementById('TI_nome'))
         document.getElementById('TI_nome').style.borderColor="";

      if (document.getElementById('TI_cognome_txt'))
         document.getElementById('TI_cognome_txt').style.color="";

      if (document.getElementById('TI_cognome'))
         document.getElementById('TI_cognome').style.borderColor="";

      if (document.getElementById('TI_privacy_txt'))
         document.getElementById('TI_privacy_txt').style.color="";

      if (document.getElementById('TI_privacy'))
         document.getElementById('TI_privacy').style.borderColor="";

      if (document.getElementById('TI_messaggio_txt'))
         document.getElementById('TI_messaggio_txt').style.color="";

      if (document.getElementById('TI_messaggio'))
         document.getElementById('TI_messaggio').style.borderColor="";
	 
      if (document.getElementById('tipologia'))
         document.getElementById('tipologia').style.borderColor="";
	 
      if (document.getElementById('tipologia_txt'))
         document.getElementById('tipologia_txt').style.color="";
	 
      if (document.getElementById('prezzoMax_txt'))
         document.getElementById('prezzoMax_txt').style.color="";
	 
      if (document.getElementById('prezzoMax'))
         document.getElementById('prezzoMax').style.borderColor="";
	 
      if (document.getElementById('supMin_txt'))
         document.getElementById('supMin_txt').style.color="";
	 
      if (document.getElementById('supMin'))
         document.getElementById('supMin').style.borderColor="";
   }
 }

 function closeAndReloadTI(){
    var html = "<div style=\"margin:0 auto;padding:20px;width:200px;\"><h3>Aggiornamento dei dati in corso...</h3><img src=\"/img2/loader3.gif\" /></div>";
    _rel_mail_alert_box = new boxOverlay('rel_mail_alert_box','Grey');
    _rel_mail_alert_box.setContent(html);
    _rel_mail_alert_box.setTitle("Aggiornamento dati");
    _rel_mail_alert_box.show();
    window.location.reload();
 }


 function getTiSource(){
   oEl = document.getElementById('TI_source');
   if (oEl)
      return oEl.value;
   return null;
 }

function checkServiceActivated(service){
   service_id = null;
   switch (service){
      case "ricevi":
         service_id = 1;
         break;
      case "invia":
         service_id = 2;
         break;
      default:
         return false;
   }

   oEl = document.getElementById('TI_servizio_'+service_id)
   if (oEl){
      if (oEl.type == "checkbox"){
         if(document.getElementById('TI_servizio_'+service_id).checked)
            return true;
      }else{
         if(document.getElementById('TI_servizio_'+service_id).value)
            return true;
      }
   }

   return false;
}

 function display_trovakasa_result(data){
    var xmlDoc = data.responseXML;
    var errorMessage;
    var risposta = xmlDoc.getElementsByTagName('risposta');
    if (risposta.length == 0){
        errorMessage = "Si è verificato un errore la preghiamo di riprovare più tardi grazie. <br />Ci scusiamo per l'inconveniente"
        showErrMsg(errorMessage);
      if (document.getElementById('trova_immobili'))
         document.getElementById('trova_immobili').innerHTML = "";
      if (document.getElementById('TILoaderCont'))
         document.getElementById('TILoaderCont').style.display = "none";
         _mail_alert_box.show();
        return;
    }

    var esito = risposta[0].getAttribute("success");
    var errors = xmlDoc.getElementsByTagName('errori');
    var pagedata = xmlDoc.getElementsByTagName('dati');
    var instructions = xmlDoc.getElementsByTagName('instructions');
	var tipo_servizio = xmlDoc.getElementsByTagName('tipo_servizio');
	var idRicerca = xmlDoc.getElementsByTagName('idRicerca');
    var ricerca_area = xmlDoc.getElementsByTagName('ricerca_area');

    if (instructions.length > 0){
        var inst = instructions[0].childNodes;

        for (var i=0;i<inst.length;i++){
            switch (inst[i].tagName){
                case "reload":
                    if(parseInt(inst[i].childNodes[0].nodeValue) == 1 && !_isSearchModify ){
                        __tiReloadOnClose = true;
                    }
                    else
                        __tiReloadOnClose = false;
                    break;
            }
        }
    }

    if (pagedata.length > 0){
        var dati = pagedata[0].childNodes;
        for (var i=0;i<dati.length;i++){
            switch (dati[i].tagName){
                case "user":
                    var userdata = dati[i].childNodes;
                    for (var i=0;i<userdata.length;i++){
                        switch (userdata[i].tagName){
                            case "nome":
                                if(document.getElementById("TI_logged_user_data"))
                                document.getElementById("TI_nome_l").value=userdata[i].childNodes[0].nodeValue;
                                break;
                            case "cognome":
                            if(document.getElementById("TI_logged_user_data"))
                                document.getElementById("TI_cognome_l").value=userdata[i].childNodes[0].nodeValue;
                                break;
                            case "telefono":
                            if(document.getElementById("TI_logged_user_data"))
                                document.getElementById("TI_telefono_l").value=userdata[i].childNodes[0].nodeValue;
                                break;
                            case "email":
                            if(document.getElementById("TI_logged_user_data")){
                                document.getElementById("TI_email_l").value=userdata[i].childNodes[0].nodeValue;
                                document.getElementById("user_email_l").innerHTML=userdata[i].childNodes[0].nodeValue;
                            }
                                break;

                        }
                    }
                    if (document.getElementById("TI_logged_user_data")){
                        document.getElementById("TI_user_data").style.display = "none";

                        document.getElementById("TI_logged_user_data").style.display = "";
                    }
                    break;
            }
        }
    }

    if (errors.length > 0){
	    document.body.style.cursor = 'default';
        var errori = errors[0].childNodes;
        errorMessage = "<span class=\"TI_err_msg_icon TI_err_title\">Verifica le seguenti informazioni: </span>";
        for (var i=0;i<errori.length;i++){
            errorMessage+="<span>"+errori[i].childNodes[0].nodeValue+"</span>";
            if (i!=errori.length-1)
                errorMessage+=", ";

			if ( __TI_servizio_2 && document.getElementById('responseBox') )
			   document.getElementById('responseBox').style.display = 'none';
		   
		    var erroreCaratteristicheImmobile = false;
		
			var tagNameError = mapErrorFieldName( errori[i].tagName );
			setErrorFieldTC( tagNameError );
			
            switch ( tagNameError ) {
			    case "erroreTentativi":
					modificaDatiAlertEmail( 6 );
					return;
				break;
			    case "zona" :
					modificaDatiAlertEmail( 5 );
					showZoneSelectForm();
					showErrMsg(errorMessage);
                    return;                
				break;
			  	case "tipologia" :
				case "prezzoMax" :
				case "supMin" :
					erroreCaratteristicheImmobile = true;                
				break;
				case "TI_comune" :					
					modificaDatiAlertEmail( 7 );
					return;
				break;			            
            }
        }
		addPlaceHolderCompatibility();
		
		if( erroreCaratteristicheImmobile )
			modificaDatiAlertEmail( 1 );
		else
			modificaDatiAlertEmail( 2 );
		
		 _lastErrMsg = errorMessage;
		 _lastErrMsgSetting = true;
		errorMessage = '<span class=\"TI_err_msg_icon\">Verifica i campi evidenziati</span>'; 
        showErrMsg(errorMessage);
		display_zone_confirm_button();
    }else{
		removeError( 'TI_username' );
        hideErrMsg();
		var form_trovaimmobili = document.getElementById('form_trovaimmobili');

		if ( ricerca_area.length > 0 && ricerca_area[0].childNodes[0].nodeValue == '1') {
			if( document.getElementById('closeAlertEmail') )
				document.getElementById('closeAlertEmail').style.display = 'block';
			
			if( document.getElementById('contentBoxTrovaImmobili') )
				document.getElementById('contentBoxTrovaImmobili').style.display = 'none';
			
			if( document.getElementById('trovaImmobiliContainer') )
				document.getElementById('trovaImmobiliContainer').style.minHeight = '50px';
		}
		if ( _isAlertEmail ) {
			//parametro che dice al php di simulare l'attivazione dei servizi al solo scopo di verificare se tutto è ok..
			//questo perchè fare il check e attivare i servizi contemporaneamnte pora a lunghi tempi di attesa per attendere
			//la risposta ddal server dopo che ha inviato tutte le mail alle agenzie interessate

			if (document.getElementById("trova_immobili") && document.getElementById("trova_immobili").isCheckRequest)
			   document.getElementById("trova_immobili").isCheckRequest.value = 1;

			//Se è la modifica ricerca chiude la finestra e fa il reload in caso contrario apre il trovaimmobili
			if ( _isSearchModify ) {
	  		   writeSearchItem();
			   return;
			} else {
			   _mail_alert_box.setTitle( "<div style='font-size:16px'>Alert Email: Salva la ricerca e ricevi gli annunci via email</div>" );
			   document.getElementById( 'idRicerca' ).value = idRicerca[0].childNodes[0].nodeValue;
			   trackPage( '/trovakasa.php?type=mailAlert' );
			   modificaDatiAlertEmail(8);
			}

		 } else {			 
			//le prossime richieste non sono di check.....
			if ( !_isAlertEmail && document.getElementById("trova_immobili") && document.getElementById("trova_immobili").isCheckRequest){
				if (_lastTypeModifica != 2){
					document.body.style.cursor = 'default';
					modificaDatiAlertEmail( 2 );
					return;
				}
			}
				document.getElementById("trova_immobili").isCheckRequest.value=0;
			//Attivazione e Invio effettivo delle email e serizio
			__TI_servizio_1 = true;
			trackPage( '/trovakasa.php?type=trova' );
			submitAjaxForm('trova_immobili',function(){});
			//form_trovaimmobili.innerHTML = create_success_box();
			if ( _isSearchModify && !_isNoGrazie ) {
				var idRicerca = document.getElementById( 'idRicerca' ).value;
				document.getElementById( 'inviaRicercaTxt_'+idRicerca ).onclick = function(){};
				document.getElementById( 'inviaRicercaTxt_'+idRicerca ).style.cursor = 'default';
				document.getElementById( 'inviaRicercaTxt_'+idRicerca ).style.color = '#318000';
				document.getElementById( 'inviaRicercaTxt_'+idRicerca ).innerHTML = 'Ricerca inviata';
			  }
			modificaDatiAlertEmail(9);
		 }
        _mail_alert_box.show();
    }
 }


/**
 * Metodo che mappa i caratteri dei campi degli errori
 * @param {string} tagName
 * @returns {String}
 */
 function mapErrorFieldName( tagName ) {
	switch( tagName ) {
		case "agComune": 
			return 'TI_agComuneField';
		break;
		case "email": 
			return 'TI_username';
		break;
		case "zona" :
		case "tipologia": 
		case "erroreTentativi":
			return tagName;
		break;
		case "prezzoMassimo": 
		case "prezzoMax":
			return 'prezzoMax';
		break;
		case "superficieMinima": 
		case "supMin":
			return 'supMin';
		break;		 
		default: 
			return 'TI_' + tagName;
		break;
	}
 }

function removeError( field ) {	
	jq( '#' + field ).removeClass(  'input_error' );
	if( document.getElementById( field + '_txt' ) )
		document.getElementById( field + '_txt' ).style.color="";
	
	if( field == "TI_username" ) {
		if( document.getElementById( field + '_ricerca_txt' ) )
			document.getElementById( field + '_ricerca_txt' ).style.color="";					
		jq( '#' + field + '_ricerca' ).removeClass(  'input_error' );
	}
	
	if( field == "TI_privacy" ) {
		if( document.getElementById( field + '_ricerca_txt' ) )
			document.getElementById( field + '_ricerca_txt' ).style.color="";					
	}	
}


/**
 * Metodo che accende o spenge gli errori del form del trovacasa e della ricerca
 * @param {string} field [ Nome del campo per cui settare o cancellare l'errore ]
 * @param {boolean} [Variabile booleana che determina de visualizzare gli errori o se rimuoverli, se true li visualizza, se false li toglie]
 * @returns {void}
 */
function setErrorFieldTC( field ) {		
	if( document.getElementById( field ) )
		jq( '#' + field ).addClass(  'input_error' );
		
	if( document.getElementById( field + '_txt' ) )
			document.getElementById( field + '_txt' ).style.color="#D10100";
		
	if( field == "TI_username" ) {
		if( document.getElementById( field + '_ricerca_txt' ) )
			document.getElementById( field + '_ricerca_txt' ).style.color="#D10100";					

		if( document.getElementById( field + '_ricerca' ) ) { 
			jq( '#' + field + '_ricerca' ).addClass(  'input_error' );
		}
	}
	
	if( field == "TI_privacy" ) {				
		if( document.getElementById( field + '_ricerca_txt' ) )
			document.getElementById( field + '_ricerca_txt' ).style.color="#D10100";					
	}
}

 function resetTrovaImmobili(){
    _mail_alert_box.close();
     if (__tiReloadOnClose)
        closeAndReloadTI();
 }

 function submitTIForm(handler, disableCheckService){
   load_zone_confirm_button();
   _lastErrMsgSetting = false;
	hideErrMsg();	

	if ( typeof disableCheckService == 'undefined' || !disableCheckService ) {
	  __TI_servizio_1 = checkServiceActivated("ricevi")
	  __TI_servizio_2 = checkServiceActivated("invia")
	}
	__TI_source = getTiSource();

	
	 if( document.getElementById('TI_servizio_2').value == 'off' )
		document.getElementById('TI_username').value = document.getElementById('TI_username_ricerca').value;

	if ( !_isAlertEmail ) {
		document.body.style.cursor = 'wait';		
	}
	
	cleanPlaceHolder( 'trova_immobili' );
		
   if (typeof handler == "function")
      submitAjaxForm('trova_immobili',handler);
   else
      submitAjaxForm('trova_immobili',display_trovakasa_result);
 }

 function hideErrMsg(){
   if (document.getElementById("TI_errors"))
    document.getElementById("TI_errors").innerHTML = "";
   if (document.getElementById("TI_err_box"))
    document.getElementById("TI_err_box").style.display = "none";
 }

 function showErrMsg($msg){
    if (document.getElementById("sendOk"))
        document.getElementById("sendOk").style.display = "none";
    if ( document.getElementById("TI_errors") )
	  document.getElementById("TI_errors").innerHTML = $msg;
    if ( document.getElementById("TI_err_box") )
	  document.getElementById("TI_err_box").style.display = "block";
 }

 function create_success_box(reload,service_proposed){
    if ( _isSearchModify && !_isNoGrazie ) {
	  var idRicerca = document.getElementById( 'idRicerca' ).value;
	  document.getElementById( 'inviaRicercaTxt_'+idRicerca ).onclick = function(){};
	  document.getElementById( 'inviaRicercaTxt_'+idRicerca ).style.cursor = 'default';
	  document.getElementById( 'inviaRicercaTxt_'+idRicerca ).style.color = '#318000';
	  document.getElementById( 'inviaRicercaTxt_'+idRicerca ).innerHTML = 'Ricerca inviata';
	}
    var res = "";

    if (typeof service_proposed == "undefined")
        service_proposed = 0;
    else
        service_proposed = parseInt(service_proposed);
    res = "<div class=\"TI_success_box\">";
            if ( !_isSearchModify && (service_proposed == 0 || service_proposed == 1 ) ){
                res+="<div class=\"TI_confirm_box_row "+(__TI_servizio_1 ? "active" : "")+"\" style=\"float:left\">"+
                    "<div style=\"display: inline-block;*display:inline;*zoom:1;\">";
                        if (__TI_servizio_1)
                            res+="<img src=\"/img2/icone/alertEmailActiveIcon.png\"/>";
                        else
                            res+="<img src=\"/img2/icone/alert-email-disattivato.png\"/>";
                    res+="</div>"+
                    "<div class=\"TI_service_res\">"+
                        "<div class=\"TI_service_name "+(__TI_servizio_1 ? "active" : "")+"\">Alert email</div>"
                        +"<div class=\"TI_service_msg\">"
                            if (__TI_servizio_1)
                                res+="Il servizio è stato attivato";
                            else
                                res+="Il servizio non è stato attivato";
                        res+="</div>";
                        res+="<div class=\"clear\"></div>";
                    res+="</div></div>";
            }
            if (service_proposed == 0 || service_proposed == 2){
                res+="<div class=\"TI_confirm_box_row "+(__TI_servizio_2 ? "active" : "")+"\" style=\"width: 520px;border-top:1px solid #DDDDDD;margin-bottom:-19px;margin-top:14px;padding-top: 11px;float:left;\">"+
                    "<div style=\"display:inline-block;*display:inline;*zoom:1\">";
                        if (__TI_servizio_2)
                            res+="<img src=\"/img2/icone/trovacasaActiveIcon.png\"/>";
                        else
                            res+="<img src=\"/img2/icone/trovacasaIcon.png\"/>";
                    res+="</div>"+
                    "<div class=\"TI_service_res\">"+
                        "<div class=\"TI_service_name "+(__TI_servizio_2 ? "active" : "")+"\">Trovaimmobili</div>"
                        +"<div class=\"TI_service_msg\">";
                            if (__TI_servizio_2)
                                //res+="Il messaggio è stato inviato alle agenzie di zona";
                                res+="La tua ricerca è stata inviata alle agenzie della zona";
                            else
                                //res+="Il messaggio non è stato inviato alle agenzie di zona";
                                res+="La tua ricerca non è stata inviata alle agenzie della zona";
                        res+="</div>";
                        res+="<div class=\"clear\"></div>";
                    res+=" </div>"+
                    "<div class=\"clear\"></div>"+
                "</div>";
            }
			   res+= "<div style=\"float: right;margin:17px 0 10px;\" id=\"TI_chiudi_btn_container\">"+
					 "<input type=\"button\" onclick=\"resetTrovaImmobili();\" value=\"\" class=\"bottoneChiudiLungo\"></div></div>";

    document.body.style.cursor = 'default';

    return res;
}

   function closeProcess() {
		__TI_servizio_1 = true;
		__TI_servizio_2 = false;
		_isNoGrazie = true;
		resetTrovaImmobili();
   }

   function writeSearchItem() {
	 var idRicerca = document.getElementById( 'idRicerca' );
	 var that = new Object();
     that.idRicerca = idRicerca.value;
	 var oAjax = new getMEAjaxObj();
	 var ulrAjax = '/bookmarkricerche.php?getDataSearchModify=1&idRicerca='+idRicerca.value;
	 oAjax.Request( 'POST' , ulrAjax , function( response){ writeMofifyResponse( response, this.idRicerca ); }.bind( that ) );
   }


   //Funzione che gestisce i vari passaggi per il processo Alert Email e Trova Immobili
   function modificaDatiAlertEmail( type, restoreError ) {
		_lastTypeModifica = type;
		document.getElementById('close_mail_alert_box').style.display = 'block';
		document.getElementById('titleAlertEmail').style.display = 'block';
		document.getElementById('alertEmailContainer').style.display = 'block';

		 // STEP DEI PARAMETRI
		if ( type == 1 ) {
			selectTabAlertEmail('#formAlertEmailContainer');
			document.getElementById('TI_err_box').style.width = 'auto';			
			hideErrMsg();
			document.getElementById('containerSelezioneZone').style.display = 'none';
			document.getElementById('formAlertEmailGeografia').style.display = 'none';
			document.getElementById('alertEmailDetail').style.display = 'none';
			document.getElementById('formAlertEmailContainer').style.display = 'block';
			document.getElementById('close_mail_alert_box').onclick = function(){ _mail_alert_box.close() };
			
			var searchFormTI = new SearchFormTI();
			searchFormTI.init();
			
		} else if ( type == 2 ) {
		 selectTabAlertEmail('#alertEmailDetail');
		  // STEP DEL RIEPILOGO
		 if ( restoreError )
			   hideErrMsg();
			document.getElementById('TI_err_box').style.width = 'auto';
			document.getElementById('formAlertEmailGeografia').style.display = 'none';
			document.getElementById('alertEmailDetail').style.display = 'block';
			document.getElementById('formAlertEmailContainer').style.display = 'none';
			document.getElementById('containerSelezioneZone').style.display = 'none';
			_mail_alert_box.setTitle("<div style='font-size:16px'>Alert Email: Salva la ricerca e ricevi gli annunci via email</div>");
			document.getElementById('close_mail_alert_box').onclick = function(){ _mail_alert_box.close() };
		} else if ( type == 3 ) {
			// SELEZIONE ZONE
			document.getElementById('TI_err_box').style.width = '635px';
			document.getElementById('containerSelezioneZone').style.display = 'block';
			document.getElementById('alertEmailContainer').style.display = 'none';
			document.getElementById('titleAlertEmail').style.display = 'none';
			document.getElementById('close_mail_alert_box').onclick = function(){ restoreSelectedZonePopup(); modificaDatiAlertEmail( 7 ); };

		} else if ( type == 4 ) {
			_mail_alert_box.setTitle( "<div style='font-size:16px'>TrovaImmobili: Invia la ricerca alle agenzie della zona</div>" );
			
			document.getElementById('titleBox').innerHTML = "Descrivi l'immobile che stai cercando, le zone e le caratteristiche e<br> invia la ricerca alle agenzie. &Egrave; gratis!";
			
			document.getElementById('responseTrovacasaRicerca').style.display = 'none';	
			document.getElementById('form_trovaimmobili').style.display = 'block';						
			
			document.getElementById('iconAlertEmail').src = '/img2/trovaImmobili/icona-1.png';
			selectTabAlertEmail('#alertEmailDetail');
			
			document.getElementById('TI_servizio_1').value = 'off';
			document.getElementById('TI_servizio_2').value = 'on';
			
			jq( '#TI_username' ).removeClass( 'placeholder' );
			
			var prefix = !_isAlertEmail ? '_TI' : '';
			if ( document.getElementById( 'TILoaderCont'+prefix ) )
				document.getElementById( 'TILoaderCont'+prefix ).style.display = "none";
			if ( document.getElementById( 'TI_btn_sub'+prefix ) ) {
				document.getElementById( 'TI_btn_sub'+prefix ).style.display = "block";
				document.getElementById( 'TI_btn_sub'+prefix ).onclick = function(){ submitTIForm();this.blur(); };
			}
			document.getElementById( 'noGrazieButton' ).style.display = "block";
			if( document.getElementById('TI_username_txt') != null )
					document.getElementById('TI_username_txt').style.color="";
				document.getElementById('TI_username').style.borderColor="";
			
			document.getElementById('boxDataUserTrovacasa').style.display = 'block';
			document.getElementById('boxDataUserRicerca').style.display = 'none';			
			document.getElementById('responseBox').style.display = 'block';			

			document.getElementById('TI_servizio_1').value = 'off';
			document.getElementById('TI_servizio_2').value = 'on';
			_isAlertEmail = false;
			
			document.getElementById('TI_privacy_ricerca_txt').innerHTML = '';
			
			if ( document.getElementById('TI_privacy') )
				document.getElementById('TI_privacy').checked = '';
			
			
			//document.getElementById('TI_usernameTrovacasa').value = document.getElementById('TI_username').value;
			//document.getElementById('TI_usernameTrovacasa').disabled = true;
			
			document.body.style.cursor = 'default';
			document.getElementById('close_mail_alert_box').onclick = function(){ _mail_alert_box.close() };
			addPlaceHolderCompatibility();
			
		} else if ( type == 5 ) {
			document.getElementById('TI_err_box').style.width = 'auto';
			document.getElementById('formAlertEmailGeografia').style.display = 'none';
			document.getElementById('alertEmailDetail').style.display = 'block';
			document.getElementById('alertEmailContainer').style.display = 'none';
			document.getElementById('titleAlertEmail').style.display = 'none';
			document.getElementById('containerSelezioneZone').style.display = 'block';
			if( document.getElementById('trovaImmobiliContainer') )
				document.getElementById('trovaImmobiliContainer').style.display = 'none';
			document.getElementById('close_mail_alert_box').style.display = 'none';

		} else if ( type == 6 ) {
			document.getElementById('alertEmailContainer').style.display = 'none';
			document.getElementById('titleAlertEmail').style.display = 'none';
			if( document.getElementById('trovaImmobiliContainer') )
				document.getElementById('trovaImmobiliContainer').style.display = 'none';

			document.getElementById('errorAttemptsContainer').style.display = 'block';
			document.getElementById('close_mail_alert_box').onclick = function(){ _mail_alert_box.close() };

		} else if (type == 7) {
			// STEP MODIFICA LUOGO
			selectTabAlertEmail('#formAlertEmailGeografia');

			document.getElementById('TI_err_box').style.width = 'auto';
			document.getElementById('formAlertEmailGeografia').style.display = 'block';
			document.getElementById('formAlertEmailContainer').style.display = 'none';
			document.getElementById('alertEmailDetail').style.display = 'none';
			document.getElementById('containerSelezioneZone').style.display = 'none';
			if( document.getElementById('trovaImmobiliContainer') )
				document.getElementById('trovaImmobiliContainer').style.display = 'none';
            addEvent(document.getElementById('textZonaBtn'),"click",function(){document.getElementById('gm_area_TI').value = "";})
            document.getElementById('close_mail_alert_box').onclick = function(){ _mail_alert_box.close() };
			managePreviewImage();
		} else if( type == 8 ) {			
			document.body.style.cursor = '';
			document.getElementById('form_trovaimmobili').style.display = 'none';			
			document.getElementById('responseTrovacasaRicerca').style.display = 'block';			
			document.getElementById('responseTrovacasaRicescaStep1').style.display = 'block';
			document.getElementById('responseTrovacasaRicescaStep2').style.display = 'none';
			document.getElementById('messageServiceTrovacasa1').style.display = 'block';
			document.getElementById('messageServiceTrovacasa2').style.display = 'none';			
			document.getElementById('responseTrovacasaIcon').src = '/img2/icone/trovacasaIcon.png';			
			
			
		} else if( type == 9 ) {			
			document.body.style.cursor = '';
			document.getElementById('form_trovaimmobili').style.display = 'none';			
			document.getElementById('responseTrovacasaRicerca').style.display = 'block';	
			document.getElementById('responseTrovacasaRicescaStep1').style.display = 'none';
			document.getElementById('responseTrovacasaRicescaStep2').style.display = 'block';
			document.getElementById('messageServiceTrovacasa2').style.display = 'block';
			document.getElementById('messageServiceTrovacasa1').style.display = 'none';	
			document.getElementById('responseTrovacasaIcon').src = '/img2/icone/trovacasaActiveIcon.png';
		} 
		
		_mail_alert_box.show();
	}

   function restoreSelectedZonePopup() {
	  var idProvincia = getIDProvincia();
	  if (!idProvincia)
		 return;

	  var totZonePopup = popupAlertZoneSelected.length;

	  var formInput = document.getElementById( 'ZoneSelectTI' ).getElementsByTagName( 'input' );
	  for( var x = 0; x < formInput.length; x++ ) {
		 if ( formInput[x].checked ) {
			selectZona( idProvincia, formInput[x].value );
		 }
		 formInput[x].checked = '';
	  }

	  for( var x = 0;  x < totZonePopup; x++ ) {
		 selectZona( idProvincia, popupAlertZoneSelected[x] );
		 swapZona( popupAlertZoneSelected[x] );
	  }
   }


   /**
    * Funzione che popola il riepilogo del primo step dell'alert email
    * popolandolo con i valori modificati o inseriti nel form dello step 2
    */
   function controllaDatiFormAlertEmail( type ) {

		var initName = document.getElementById( 'initName' ).innerHTML;
		initName = trimText(initName);

	    if (!isTIVacanze()) {
			//Cambia tipo contratto nel riepilogo
			var selection = document.getElementsByName('contratto');
			for (i=0; i<selection.length; i++){
				if (selection[i].checked == true) {
					var tipoContratto = selection[i].value;
				}
			}
			if ( document.getElementById('nomeContratto') ) {
				var nomeContratto = document.getElementById('nomeContratto');
				if ( tipoContratto == 1 ) {
					document.getElementById( 'contrattoRicerca' ).innerHTML = "<strong>Contratto:</strong>&nbsp;Vendita";
					nomeContratto.innerHTML = 'vendita';
				} else if ( tipoContratto == 2 ) {
					document.getElementById( 'contrattoRicerca' ).innerHTML = "<strong>Contratto:</strong>&nbsp;Affitto";
					nomeContratto.innerHTML = 'affitto';
				}
			}
		}

		var tipologia = document.getElementById('tipologia');
		var sottotipologia = document.getElementById('sottotipologia');
		var nomeTipologia = 'Immobili';

		if (document.getElementById('categoria_TI') && document.getElementById('categoria_TI').value == 4){
			
			var _nomeCategoria = "Stanza o posto letto";
			var _nomeTipologiaStanza = "";
			
			var selCategoriaStanzaTI = jq("#selCategoriaStanzaTI");
			var selTipologiaStanzaTI = jq("#selTipologiaStanzaTI");
			var nomeCategoria = jq("#nomeCategoria");
			var categoriaRicerca = jq("#categoriaRicerca");
			var nomeTipologiaStanza =  jq("#nomeTipologiaStanza");
			var tipologiaStanzaRicerca = jq("#tipologiaStanzaRicerca");
			
			if (selCategoriaStanzaTI.val() > 0){
				_nomeCategoria = selCategoriaStanzaTI.find(":selected").text();
			}
			
			if (selTipologiaStanzaTI.val() > 0){
				_nomeTipologiaStanza = selTipologiaStanzaTI.find(":selected").text();
			}
		
			nomeCategoria.html(_nomeCategoria);
			categoriaRicerca.html("<strong>Cetegoria:</strong>&nbsp;" + _nomeCategoria);

			nomeTipologiaStanza.html(_nomeTipologiaStanza);
			tipologiaStanzaRicerca.html("<strong>Tipo stanza:</strong>&nbsp;" + _nomeTipologiaStanza);

			if (selTipologiaStanzaTI.val() > 0){
				tipologiaStanzaRicerca.fadeIn("slow");
			} else {				
				tipologiaStanzaRicerca.fadeOut("slow");
			}
		} else {
			if ( document.getElementById('sottotipologia') && document.getElementById('nomeSottotipologia')) {
				var sottotipologia = document.getElementById('sottotipologia');
				var nomeSottotipologia = sottotipologia.options[sottotipologia.selectedIndex].innerHTML;
				document.getElementById('nomeSottotipologia').innerHTML = nomeSottotipologia;
				document.getElementById('nomeTipologia').innerHTML = '';
				document.getElementById( 'tipologiaRicerca' ).innerHTML = "<strong>Tipologia:</strong>&nbsp;" + nomeSottotipologia;
			} else {
				//Se sto nella categoria terreni
				if (tipologia.value == 28) {
					nomeTipologia = sottotipologia.options[sottotipologia.selectedIndex].innerHTML;
					if (nomeTipologia == "Tutti")
						nomeTipologia = "Terreni";
				} else if ( tipologia.value != 0 && tipologia.value != 14) {
					nomeTipologia = tipologia.options[tipologia.selectedIndex].innerHTML;
				}
				if (document.getElementById('nomeSottotipologia'))
					document.getElementById('nomeSottotipologia').innerHTML = '';
				document.getElementById('nomeTipologia').innerHTML = nomeTipologia;
				document.getElementById( 'tipologiaRicerca' ).innerHTML = "<strong>Tipologia:</strong>&nbsp;" + nomeTipologia;
			}
		}

		if ( initName.toLowerCase() == document.getElementById( 'TI_nomeRicerca' ).value.toLowerCase() ) {
			var initName = document.getElementById( 'initName' ).innerHTML;
			initName = trimText(initName);
			document.getElementById( 'TI_nomeRicerca' ).value = initName;
		}
		
		//Recupera i per la fascia prezzo
		var isPrezzo = false;
		if ( document.getElementById('prezzoMin') && document.getElementById('prezzoMax') ) {
			var prezzoMin = document.getElementById('prezzoMin').value;
			var prezzoMax = document.getElementById('prezzoMax').value;
			var prezzoRicerca = '<strong>Prezzo:</strong>';
			var prefPrezzo = prezzoMin != '' ? '&nbsp;a&nbsp;': '&nbsp;fino&nbsp;a&nbsp;';
			if ( prezzoMin != '' ) {
				prezzoRicerca += ' da '+prezzoMin+'&nbsp;&euro;';
				isPrezzo = true;
			}
			if ( prezzoMax != '' ) {
				prezzoRicerca += prefPrezzo+prezzoMax+'&nbsp;&euro;';
				isPrezzo = true;
			}
			if ( isPrezzo ) {
				document.getElementById('prezzoRicerca').innerHTML = prezzoRicerca;
			} else {
				document.getElementById('prezzoRicerca').innerHTML = '';
			}
		}

		//Recupera i valori per la fascia di superficie
		var isSuperfRicerca = false;
		if ( document.getElementById('supMin') && document.getElementById('supMax') ) {
			var supMin = document.getElementById('supMin').value;
			var supMax = document.getElementById('supMax').value;
			var superfRicerca = '';
			var contSuperfRicerca = '';
			var prefSuperf = supMin != '' ? '&nbsp;a&nbsp;': '&nbsp;fino&nbsp;a&nbsp;';
			if ( supMin != '' ) {
				contSuperfRicerca += ' da '+supMin+'&nbsp;m&sup2;';
				isSuperfRicerca = true;
			}
			if ( supMax != '' ) {
				contSuperfRicerca += prefSuperf+supMax+'&nbsp;m&sup2;';
				isSuperfRicerca = true;
			}
			if ( isSuperfRicerca ) {
				superfRicerca = '<strong>Superficie:</strong>'+contSuperfRicerca;
				document.getElementById('superficieRicerca').innerHTML = superfRicerca;
			} else {
				document.getElementById('superficieRicerca').innerHTML = '';
			}
		}

		//Recupera i valori per il numero dei locali
		var isLocMin = false;
		if ( document.getElementById('locMin') && document.getElementById('locMax') ) {
			var locMin = document.getElementById('locMin').value;
			var locMax = document.getElementById('locMax').value;
			var localiRicerca = '<strong>Locali:</strong>';
			var prefLocali = locMin ? '&nbsp;a&nbsp;' : '&nbsp;fino&nbsp;a&nbsp;';
			if ( locMin != '' ) {
				localiRicerca += ' da '+locMin;
				isLocMin = true;
			}
			if ( locMax != '' ) {
				localiRicerca += prefLocali+locMax;
				isLocMin = true;
			}
			if ( isLocMin ) {
			   document.getElementById('localiRicerca').innerHTML = localiRicerca;
			} else {
			   document.getElementById('localiRicerca').innerHTML = '';
			}
		}

		if ( document.getElementById('bagni') ) {
			var bagni = document.getElementById('bagni');
			if ( bagni.value != 0 ) {
			   document.getElementById('bagniRicerca').innerHTML = (isLocMin ? '&nbsp;|&nbsp;' : '') + '<strong>Bagni:</strong>&nbsp;'+bagni.options[bagni.selectedIndex].innerHTML;
			} else {
			   document.getElementById('bagniRicerca').innerHTML = '';
			}
		}

		isFkStato = false;
		if ( document.getElementById('fkStato') ) {
			var stato = document.getElementById('fkStato');
			if ( stato.value  != 0 ) {
			   isFkStato = true;
			   document.getElementById('statoRicerca').innerHTML = '<strong>Stato:</strong>&nbsp;'+stato.options[stato.selectedIndex].innerHTML;
			} else {
			   document.getElementById('statoRicerca').innerHTML = '';
			}
		}

		if ( document.getElementById('boxautoTI') ) {
			var postoAuto = document.getElementById('boxautoTI');
			if ( postoAuto.value  != 0 ) {
			   document.getElementById('boxPostoRicerca').innerHTML = (isFkStato ? '&nbsp;|&nbsp;' : '') + '<strong>Box/Posto auto:</strong>&nbsp;'+postoAuto.options[postoAuto.selectedIndex].innerHTML;
			} else {
			   document.getElementById('boxPostoRicerca').innerHTML = '';
			}
		}

		//Recupera i valori per il campo altri dati
		 var altriDati = '';
		 if ( document.getElementById('altriDati') ) {

			if ( document.getElementById( 'tipoProprieta' ) ) {
				var tipoPropr = document.getElementById('tipoProprieta');
				if ( tipoPropr.value != 0 && !(sottotipologia && sottotipologia.value == 113) )
					altriDati += '<strong>Tipo propriet&agrave;:</strong>&nbsp;'+
										tipoPropr.options[tipoPropr.selectedIndex].innerHTML+
										'&nbsp;| ';
			}

			if ( document.getElementById('riscaldamento') ) {
				var riscaldamento = document.getElementById('riscaldamento');
				if ( riscaldamento.value != 0 )
					altriDati += '<strong>Riscaldamento:</strong>&nbsp;'+riscaldamento.options[riscaldamento.selectedIndex].innerHTML+'&nbsp;| ';
			}

			if ( document.getElementById('terrazzoTI') ) {
				var terrazzo = document.getElementById('terrazzoTI').checked;
				if ( terrazzo )
					altriDati += '<strong>Terrazzo:</strong>&nbsp;sì&nbsp;| ';
			}

            if ( document.getElementById('balconeTI') ) {
                var flagBalcone = document.getElementById('balconeTI').checked;

                if ( flagBalcone ){
                    altriDati += '<strong>Balcone:</strong>&nbsp;sì&nbsp;| ';
                }
            }

			if ( document.getElementById('giardinoTI') ) {
				var giardino = document.getElementById('giardinoTI').checked;
				if ( giardino )
					altriDati += '<strong>Giardino:</strong>&nbsp;sì&nbsp;| ';
			}

			if ( document.getElementById('arredatoTI') ) {
				var arredato = document.getElementById('arredatoTI').checked;
				if ( arredato )
					altriDati += '<strong>Arredato</strong>:&nbsp;sì&nbsp;| ';
			}

			//IN ASTA
			if ( document.getElementById('inAstaTI') ) {
				var inAsta = document.getElementById('inAstaTI').checked;
				if ( inAsta )
					altriDati += '<strong>In asta</strong>:&nbsp;sì&nbsp;| ';
			}

			// Escludi Aste
			if (document.getElementById('noAsteTI')) {
				var noAste = document.getElementById('noAsteTI').checked;
				if (noAste) {
					altriDati += '<strong>Escludi aste</strong>:&nbsp;sì&nbsp;| ';
				}
			}

			if ( document.getElementById('animaliTI') ) {
				var animali = document.getElementById('animaliTI').checked;
				if ( animali )
					altriDati += '<strong>Animali</strong>:&nbsp;sì&nbsp;| ';
			}
			
			if ( document.getElementById('fumatoreTI') ) {
				var fumatore = document.getElementById('fumatoreTI').checked;
				if ( fumatore )
					altriDati += '<strong>Fumatori</strong>:&nbsp;sì&nbsp;| ';
			}			
			
			if ( document.getElementById('ascensoreTI') ) {
				var ascensore = document.getElementById('ascensoreTI').checked;
				if ( ascensore )
					altriDati += '<strong>Ascensore</strong>:&nbsp;sì&nbsp;| ';
			}
			
			if ( document.getElementById('giardinoTI') ) {
				var giardinoTI = document.getElementById('giardinoTI');
				if ( giardinoTI.value != 0 )
					altriDati += '<strong>Giardino:</strong>&nbsp;'+giardinoTI.options[giardinoTI.selectedIndex].innerHTML+'&nbsp;| ';
			}
			
			if ( document.getElementById('classeEnergeticaTI') ) {
				var classeEnergeticaTI = document.getElementById('classeEnergeticaTI');
				if ( classeEnergeticaTI.value != 0 )
					altriDati += '<strong>Classe energetica:</strong>&nbsp;'+classeEnergeticaTI.options[classeEnergeticaTI.selectedIndex].innerHTML+'&nbsp;| ';
			}

			 //RECUPERA I VALORI PER FASCIA PIANO
			 var jqFasciaPiano = jq('#selectFasciaPiano');
			 if (jqFasciaPiano.exist()) {
					 var aValFasciaPiano = jqFasciaPiano.val(),
					 	 aStringFasciePiano = [];

				 if ( !jq.isEmpty(aValFasciaPiano) ) {
					for(var ifp = 0;ifp < aValFasciaPiano.length; ifp++){
						aStringFasciePiano.push(jqFasciaPiano.find('option[value="'+ aValFasciaPiano[ifp] +'"]').html());
					}
					altriDati += '<strong>Piano:</strong>&nbsp;' + aStringFasciePiano.join(', ');
				 }

			 }
		
			document.getElementById('altriDati').innerHTML = trim2( altriDati, '| ' );
		 }

	   if ( false && typeof type != 'undefined' ) {
		   // viene uniformato il comportamento tra alert email e bookmark ricerche.
		   // per portalo allo stato iniziale togliere il "false"
		   submitTIForm();
	   } else {
			if( __TI_servizio_2 )
				submitTIForm();
			else
				modificaDatiAlertEmail( 2 );
	   }
   }

   function checkLocation() {
	  var idComune = document.getElementById( 'TI_comune' ).value;
	  if (isTIVacanze()) {
		  var idAreaGeografica = document.getElementById("TI_areaGeografica").value;
	  }
	  var gmAreaTI = document.getElementById('gm_area_TI').value;
	  var tipoRicercaTI = document.getElementById('tipoRicerca_TI').value;
	  var idsZone = document.getElementById('zoneTI_ids').value;
	  var isCittaConZona = document.getElementById('cittaConZona').value == 1 ? true : false;
	  var isEnabledZone = document.getElementById('enabledZone').value == 1 ? true : false;
      var nomeProvincia = document.getElementById('nomeProvincia').value;
	  var initName = document.getElementById( 'initName' ).innerHTML;
	  initName = trimText(initName);

	  if (isTIVacanze()) {
		  var agLocalitaValue = document.getElementById("TI_agComuneField").value;
		  document.getElementById('localitaRicerca').innerHTML = "<strong>Località:</strong>&nbsp;" + agLocalitaValue;
		  document.getElementById('nomeLocalitaRicerca').innerHTML = agLocalitaValue;
      } else if (document.getElementById( 'nomeComuneRicerca' )) {
		 var selectComune = document.getElementById('TI_comune');
		 var nomeComune = selectComune.options[selectComune.selectedIndex].innerHTML

		 var valueComune = selectComune.options[selectComune.selectedIndex].value;
			if (!gmAreaTI){
                if ( !valueComune)
                    nomeComune = "?";

                document.getElementById( 'congiunction_geo' ).innerHTML = " a ";
		        document.getElementById( 'nomeComuneRicerca' ).innerHTML = nomeComune;
		        document.getElementById( 'comuneRicerca' ).innerHTML = "<strong>Comune:</strong>&nbsp;" + nomeComune;
				if ( document.getElementById( 'provinciaRicerca' ) )
					document.getElementById( 'provinciaRicerca' ).innerHTML = "";
            }else{
                document.getElementById( 'congiunction_geo' ).innerHTML = " in provincia di ";
                document.getElementById( 'nomeComuneRicerca' ).innerHTML = nomeProvincia;
				if ( document.getElementById( 'provinciaRicerca' ) )
					document.getElementById( 'provinciaRicerca' ).innerHTML = "<strong>Provincia:</strong>&nbsp;" + nomeProvincia;
                document.getElementById( 'comuneRicerca' ).innerHTML = "";
            }

	  }
	  
	  if ( initName.toLowerCase() == document.getElementById( 'TI_nomeRicerca' ).value.toLowerCase() ) {
		 var initName = document.getElementById( 'initName' ).innerHTML;
		 initName = trimText(initName);
		 document.getElementById( 'TI_nomeRicerca' ).value = initName;
	  }

	   if (!isTIVacanze() && !idComune && !jq("#gm_area_TI").val()) {
		   errorMessage = "<span class=\"TI_err_msg_icon TI_err_title\">Attenzione: </span><span>Seleziona un comune oppure disegna un'area sulla mappa</span>";
		   showErrMsg( errorMessage );
		   return;
	   } else if ( (isEnabledZone && isCittaConZona) && ( ((idsZone == '' || idsZone == 0) && tipoRicercaTI == '') || (gmAreaTI == '' && tipoRicercaTI == 'mappa') ) ) {
		   if (isTIVacanze()) {
			   errorMessage = "<span class=\"TI_err_msg_icon TI_err_title\">Attenzione: </span><span>Seleziona almeno una zona</span>";
		   } else {
			   errorMessage = "<span class=\"TI_err_msg_icon TI_err_title\">Attenzione: </span><span>Seleziona almeno una zona o disegna un'area sulla mappa</span>";
		   }
		   showErrMsg( errorMessage );
		   return;
	   }

	  hideErrMsg();

	  if (isEnabledZone && isCittaConZona && tipoRicercaTI == '') {
		 var oAjax = new getMEAjaxObj();
		 var ulrAjax = '/eseguiServizioTrovakasa.php?trova_immobili=1&elencoZone='+idsZone+'&comune='+idComune;
		 oAjax.Request( 'POST' , ulrAjax , function( response ){ setNewZone( response ); loadPreviewZone(); });
	  } else {
		 if (document.getElementById('nomeNumeroZone')) {
			 if ( tipoRicercaTI == 'mappa' && gmAreaTI != '' ) {
			   loadGoogleMapStaticImage('riepilogo', true);
			   document.getElementById('nomeNumeroZone').innerHTML = '<strong>Area:</strong> disegnata su mappa';
			 } else {
			   document.getElementById('nomeNumeroZone').innerHTML = '';
			 }
		 }
		 if (document.getElementById('totZoneAlertEmail'))
			document.getElementById('totZoneAlertEmail').innerHTML = '';
		 if (document.getElementById('zoneRicerca'))
			document.getElementById('zoneRicerca').innerHTML = '';
	  }
	  document.getElementById('fkComune_TI').value = idComune;
	  if (isTIVacanze() && TI_agComuneField.getObject().type == "areaGeografica") {
		  document.getElementById('idAreaGeografica_TI').value = idAreaGeografica;
		  jq("#prev_img_confini_comune").prop('src', TI_agComuneField.getUrlCartina());
	  } else {
		  // Carico la cartina con i confini del comune.
		  if( !isEstero() )
			xajax_getAjaxComuneImage('prev_img_confini_comune', idComune);
	  }

	  if ( _lastTypeModifica == 5 ) {
			modificaDatiAlertEmail( 1, true );
	  } else {
			managePreviewImage('prev_')
			modificaDatiAlertEmail( 2 );
	  }
   }

	 function isEstero() {
		 fkNazione = document.getElementById('fkNazione').value;
		 if( fkNazione == '' || fkNazione == 'IT' )
			 return false;
		 return true;
	 }

	//Funzione che recupera il nome delle zone selezionate per stamparle nell'etichetta del riepilogo
	function settaInputZonePopup() {
	  var errorMessage = '';
	  //if ( _isAlertEmail ) {
		 var idsZone = '';
		 var totZone = 0;
		 oRadio = document.getElementById('ZoneSelectTI').getElementsByTagName('input');
		 for( var i = 0; i < oRadio.length; i++ ) {
			 if( oRadio[i].checked ) {
				 idsZone += oRadio[i].value+',';
				 if ( !in_array( oRadio[i].value, popupAlertZoneSelected ) )
					 popupAlertZoneSelected.push( oRadio[i].value );

				 totZone++;
			 }
		 }
		 var testoBtn = 'Scegli la zona';
		 if ( totZone > 0 ) {
			testoBtn = "Zone, " + totZone + (totZone == 1 ? " selezionata" : " selezionate");
		 } else {
			errorMessage = "<span class=\"TI_err_msg_icon TI_err_title\">Mancano le seguenti informazioni: </span><span>Seleziona almeno una zona</span>";
			testoBtn = "Scegli la zona";
		 }
		 document.getElementById( 'textZonaBtn' ).innerHTML = testoBtn;
		 jq("#modifica_zona_TI").show();
	  /*}else{
          submitTIForm();
      }*/

	  if ( errorMessage ) {
		 showErrMsg( errorMessage );
		 return;
	  }

	  hideErrMsg();
	  loadPreviewZone(true)
	  if ( _lastTypeModifica == 5 )
		  modificaDatiAlertEmail( 7, true );
	  else
		  modificaDatiAlertEmail( 7 );
   }

   /**
    * Funzione stampa nel riepilogo le zone ed il numero recuperati dalla chiamata ajax
    */
   function setNewZone( response ) {
	  if( document.getElementById( 'zoneRicerca' ) ) {
		 var zoneRicerca = document.getElementById( 'zoneRicerca' );
		 zoneRicerca.innerHTML = response.responseText;
		 var totNuoveZone = document.getElementById('totZoneAlertEmail').value;
		 if (totNuoveZone > 0)
			document.getElementById('nomeNumeroZone').innerHTML = '<strong>' + ( totNuoveZone == 1  ? 'Zona selezionata' : 'Zone selezionate' ) + ' ('+totNuoveZone+'):</strong>';
		 else
			document.getElementById('nomeNumeroZone').innerHTML = '';
	  }
   }

   //Funzione che resetta le zone selezionate
   function resetZone() {
		if ( document.getElementById( 'zoneTI_descrizione' ) )
			document.getElementById( 'zoneTI_descrizione' ).value = '';

		if ( document.getElementById( 'zoneTI_ids' ) )
			document.getElementById( 'zoneTI_ids' ).value = '';

		if ( document.getElementById( 'zoneRicerca' ) )
			document.getElementById( 'zoneRicerca' ).value = '';

		if ( document.getElementById( 'zoneRicerca' ) )
			document.getElementById( 'zoneRicerca' ).innerHTML = '';

		if ( document.getElementById( 'nomeNumeroZone' ) )
			document.getElementById( 'nomeNumeroZone' ).innerHTML = '';

		__zone_showed = false;
		zone_selected_TI = new Array();
	   popupAlertZoneSelected = new Array();
   }

   function resetPolygon() {
	  jq("TI_poligono").val('');
   }
   /**
    * Funzione che controlla che nella selezione zone del trovacasa siano state selezionate zone e che non siano più di 4
   */
   function closeZoneTrovaImmobili() {
	  var numZone = 0;
	  var zoneSelectTI = document.getElementById('ZoneSelectTI');
	  var idsZone =  zoneSelectTI.getElementsByTagName('input');
	  for( var x = 0; x < idsZone.length; x++ ) {
		 if ( idsZone[x].checked )
			numZone++;
	  }

	  if ( numZone > 0 && numZone < 5 ) {
		 hideErrMsg();
		 document.getElementById('TI_err_box').style.width = '435px';
		 document.getElementById('containerSelezioneZone').style.display = 'none';
		 document.getElementById('trovaImmobiliContainer').style.display = 'block';
		 document.getElementById('close_mail_alert_box').onclick = function(){ _mail_alert_box.close() };
	  } else if ( numZone == 0 ) {
		 errorMessage = "<span class=\"TI_err_msg_icon TI_err_title\">Mancano le seguenti informazioni: </span><span>Seleziona almeno una zona</span>";
		 showErrMsg( errorMessage );
	  } else if ( numZone > 4 ) {
		 errorMessage = "<span class=\"TI_err_msg_icon TI_err_title\">Mancano le seguenti informazioni: </span><span>Non è possibile selezionare più di 4 zone</span>";
		 showErrMsg( errorMessage );
	  }
   }

   function errorMessageTrovaImmabili( errorMessage ) {
	  document.getElementById('responseBox').style.display = 'none';
	  showErrMsg( errorMessage );
   }

   function startMailAlertProcess(oForm,type){
	   if(_disableAlertPopUp)
		   return false;


     _isNuoveCostruzioni = false;
	 if ( document.getElementById('categoria') )
	   _isNuoveCostruzioni = document.getElementById('categoria').value == '6' ? true :false;

	  _isAlertEmail = true;
	  appendCss("/includes/trova_immobili.css","mail_alert_css");
	  appendCss('/includes/btn_bootstrap.css','btn_bootstrap.css');
	  appendCss('/includes/tab_bootstrap.css','tab_bootstrap.css');
	  appendCss('/includes/radiobutton-ui.css','radiobutton-ui.css');

    if (typeof __ti_preloaded=="undefined"){
        preloadImages(new Array("/img2/loader3.gif","/img2/loader6.gif"));
        __ti_preloaded = true;
    }
    __TI_activeForm = oForm;
    zone_selected_TI = new Array();
    __zone_showed=false;

	popupAlertZoneSelected = new Array();
	if ( document.getElementById( 'zoneTI_ids' ) ) {
		 var iDs = trim2( document.getElementById( 'zoneTI_ids' ).value, ',' );
		 popupAlertZoneSelected = iDs.split(",");
	}


    if (typeof type == "undefined")
        if (oForm.email.value == oForm.email.getAttribute("startvalue") || oForm.email.value == ""){
         oForm.email.value = '';

        }
        else
        if (!checkEmail(oForm.email.value)){
            alert("inserisci una email valida");
            return false;
        }

    //if (typeof _mail_alert_box!="undefined" && _mail_alert_box.close && typeof _mail_alert_box.close == "function")
        //_mail_alert_box.destroy();

    hideMailAlertBottomBanner();

	if ( typeof _mail_alert_box == "undefined" || !_mail_alert_box.isOpen ) {
	  selMAiBox = {};
	  selMAiBox.box = _mail_alert_box = new boxOverlay('mail_alert_box','Grey');

	  showLoader(_mail_alert_box);
	  _mail_alert_box.setTitle("<div style='font-size:16px'>Alert Email: Salva la ricerca e ricevi gli annunci via email</div>");
	  _mail_alert_box.show();

	  var bckEmail = "";
	  if (typeof type!="undefined" && type<=3){
		  oForm.formType.value=type;
		  /*if (oForm.email)
			  oForm.email.value = "";
		  if (oForm.password)
			  oForm.email.password = "";
		*/
	  }
	  submitAjaxForm(oForm,
		  function(data){
			  _mail_alert_box.setContent(data.responseText);
			  _mail_alert_box.addEventOnClose(function(){ __isTrovaImmobili = false; _mail_alert_box.manageConflicts = true; });
			  _mail_alert_box.show();
			  _mail_alert_box.manageConflicts = false;
			  __isTrovaImmobili = true;
			  initAlertEmailArea();
			  document.getElementById('boxDataUserRicerca').style.display = 'block';
			  document.getElementById('boxDataUserTrovacasa').style.display = 'none';
			  document.getElementById('responseBox').style.display = 'none';
			  _mail_alert_box.show();
			  addPlaceHolderCompatibility();
			  _searchNameManager = new SearchNameManager();
			  _searchNameManager.addListener();
		  },
		  "/eseguiServizioTrovakasa.php"
	  );
	}
    return true;
   }

   function startSearchModify( idRicerca, type ) {
	  _isAlertEmail = true;
	  _isSearchModify = true;

	  nomeRicerca = document.getElementById( 'value_nomeRicerca_'+idRicerca ).value;
	  appendCss("/includes/trova_immobili.css","mail_alert_css");
	  appendCss('/includes/btn_bootstrap.css','btn_bootstrap.css');
	  appendCss('/includes/tab_bootstrap.css','tab_bootstrap.css');
	  appendCss('/includes/radiobutton-ui.css','radiobutton-ui.css');

	  if ( typeof __ti_preloaded=="undefined" ) {
		 preloadImages( new Array("/img2/loader3.gif","/img2/loader6.gif" ) );
		 __ti_preloaded = true;
	  }
	  zone_selected_TI = new Array();
	  __zone_showed=false;

	  hideMailAlertBottomBanner();

	  selMAiBox = {};
	  selMAiBox.box = _mail_alert_box = new boxOverlay('mail_alert_box','Grey');
	  //_mail_alert_box.fixedPosition = false;
	  showLoader( _mail_alert_box );

	  if ( nomeRicerca.toUpperCase() == nomeRicerca )
		 nomeRicerca = nomeRicerca.toLowerCase()

	  if ( nomeRicerca.length > 75 )
		 nomeRicerca = nomeRicerca.substr(0,75)+'...';

	  _mail_alert_box.setTitle( "<div style='font-size:16px;'>"+nomeRicerca+"</div>" );
	  _mail_alert_box.show();

	   var oAjax = new getMEAjaxObj();
	   var ulrAjax = '/eseguiServizioTrovakasa.php?idRicerca=' + idRicerca + '&getForm=1&trova_immobili=1&formType=1';
	   oAjax.Request('POST', ulrAjax, function (response) {
		   writeSearchModify(response, type);
		   _searchNameManager = new SearchNameManager();
		   _searchNameManager.addListener();
	   });

   }

   function writeSearchModify( data, type ) {
	  _mail_alert_box.setContent( data.responseText );
	  _mail_alert_box.addEventOnClose( function(){ __isTrovaImmobili = false; _mail_alert_box.manageConflicts = true; } );
	  popupAlertZoneSelected = new Array();
	  if ( document.getElementById( 'zoneTI_ids' ) ) {
		 var iDs = trim2( document.getElementById( 'zoneTI_ids' ).value, ',' );
		 popupAlertZoneSelected = iDs.split(",");
	  }


	  popupAlertZoneSelected = new Array();
	  if ( document.getElementById( 'zoneTI_ids' ) ) {
		 var iDs = trim2( document.getElementById( 'zoneTI_ids' ).value, ',' );
		 popupAlertZoneSelected = iDs.split(",");
	  }

	  if ( type == 1 ) {
		 modificaDatiAlertEmail( 2 );
	  } else {
		 modificaDatiAlertEmail( 4 );
		 document.getElementById("trova_immobili").isCheckRequest.value = 1;
	  }
	  _mail_alert_box.show();
	  _mail_alert_box.manageConflicts = false;
	  __isTrovaImmobili = true;
	  initAlertEmailArea();

	  /**
	   * nel caso in cui carico una ricerca salvata devo cancellare eventuali mappe aperte.
	   * boxPolygonSearch è definita in createGMapsIframe.js
	   */
	  if ( boxPolygonSearch || polygonMapLoadCompleted) {
		 polygonMapLoadCompleted = false;
		 if (typeof boxPolygonSearch.destroy === 'function')
			boxPolygonSearch.destroy();
		 boxPolygonSearch = null;
	  }

      var searchFormTI = new SearchFormTI();
      searchFormTI.init();
   }

   function showZoneSelectForm(){
	  var idProvincia = getIDProvincia();
	  if (!idProvincia)
		 return;

    if (__zone_showed){
         display_zone_confirm_button();
         return;
    }
    __zone_showed=true;
    var zoneSelect = document.getElementById( "ZoneSelectTI" );
   __TI_source = getTiSource();

	/**
	  '<input type="hidden" id="TI_servizio_1" name="ricevi" '+(__TI_servizio_1 ? 'value="on"' : '')+'>'+
	  '<input type="hidden" id="TI_servizio_2" name="invia" '+(__TI_servizio_2 ? 'value="on"' : '')+'>'+
	 **/

    var html =  '<input type="hidden" id="zoneTI_descrizione" name="zona_desc" readonly="readonly" />'+
                '<input type="hidden" id="TI_source" name="source" '+(__TI_source ? 'value="'+__TI_source+'"' : '')+'>'+
                '<div class="zone_title">'+
                '   <div class="zone_title_text">'+
                '       <strong style="font-size:14px;">Per fornirti un servizio migliore ti chiediamo di indicare le zone di tuo interesse</strong>'+
                '       <div id="tooltip_map" style="margin-left: 0px;">'+
                '       </div>'+
                '   </div>'+
                '</div>'+
                '<div>'+
                '    <div id="CartinaGrandeTI">'+
                '    </div>'+
                '    <div id="zoneSelectContainerTI" style="float:left"></div>'+
                '    <div class="clear"></div>'+
                '</div>';
    document.getElementById("TI_form_input_data").innerHTML = html;

    //_mail_alert_box.addEventOnClose(function(){TI_undoService(2)});
	_mail_alert_box.show();

	var zoneSelectCont = document.getElementById("zoneSelectContainerTI");
    zoneSelectCont.appendChild(zoneSelect);
	idComune = document.getElementById('TI_comune');
	xajax_getMapAjaxZonePagRicerca(idComune.value, idProvincia, "TI");

    var pathDiv = document.getElementById("tooltip_map");
	if ( idComune.tagName  == 'INPUT' ) {
		if (typeof TI_agComuneField != "undefined" && TI_agComuneField) {
			nomeComune = TI_agComuneField.getNome();
		} else {
			nomeComune = document.getElementById('nomeComune').value;
		}
	  var comune = createPathLink(nomeComune,function(){return false});
   } else {
	  var comune = createPathLink(idComune.options[idComune.selectedIndex].innerHTML,function(){return false});
   }

	pathDiv.appendChild(comune);
    zoneSelect.style.display = "block";

    addEvent(window,"unload",function(){zoneSelect = null})
	addEvent(window,"unload",function(){zoneSelectCont = null})
	addEvent(window,"unload",function(){pathDiv = null})

    _mail_alert_box.show();
    display_zone_confirm_button();
   }

   function display_zone_confirm_button(){
	  var prefix = !_isAlertEmail ? '_TI' : '';
      if (document.getElementById('TILoaderCont'+prefix))
        document.getElementById('TILoaderCont'+prefix).style.display = "none";
      if (document.getElementById('TI_btn_sub'+prefix)) {
         document.getElementById('TI_btn_sub'+prefix).style.display = "";
		 document.getElementById('TI_btn_sub'+prefix).onclick = function(){submitTIForm();this.blur();};
      }
   }

   function load_zone_confirm_button() {
	  var prefix = '';
	  if ( !_isAlertEmail ) {
		 prefix = '_TI';
		 //document.getElementById('noGrazieButton').style.display = "none";
	  }
      if (document.getElementById('TI_btn_sub'+prefix)) {
         document.getElementById('TI_btn_sub'+prefix).style.display = "none";
         document.getElementById('TI_btn_sub'+prefix).onclick = function(){};
      }
      if (document.getElementById('TILoaderCont'+prefix))
        document.getElementById('TILoaderCont'+prefix).style.display = "inline";
   }

   function setZoneSelected( siglaProv ){
      var sIds = document.getElementById("zoneTI_ids").value;
      var aZoneIds = sIds.split(",");
      for (i=0;i<aZoneIds.length;i++)
	  if (aZoneIds[i]!=""){
		 selectZona(siglaProv,aZoneIds[i],'TI');
		 swapZona(aZoneIds[i],"TI");
	  }
   }

   function TI_undoService(num){
        if (!num)
            return;
         if (!document.getElementById("trova_immobili"))
            return;
        if (num==1){
            __TI_servizio_1 = false;
            if (document.getElementById("trova_immobili").ricevi)
               document.getElementById("trova_immobili").ricevi.value=0;
        }
        else
        if (num==2){
            __TI_servizio_2 = false;
            if (document.getElementById("trova_immobili").invia)
               document.getElementById("trova_immobili").invia.value=0;
        }

      submitAjaxForm('trova_immobili',function(){});
      document.getElementById('form_trovaimmobili').innerHTML = create_success_box();
   }

    function showLoader(box){
        box.setContent("<div style=\"height:210px;text-align:center;\"><h3>Caricamento in corso...</h3><br /><span style=\"\"><img src=\"/img2/loader6.gif\"></span></div>");
    }


    function showMailAlertBottomBanner(){
        if (typeof __isTrovaImmobili!="undefined" && __isTrovaImmobili)
            return;
        MEslideUp("mail_alert_banner_bottom",5);
        var pushUpage = document.createElement("Div");
        pushUpage.id = "pushUpPage";
        document.body.appendChild(pushUpage);
        pushUpage.style.display = "block";
        pushUpage.style.backgroundImage="none";
        pushUpage.style.backgroundColor="#B8B8B8";
        pushUpage.style.position="relative";
        pushUpage.style.height="128px";
    }

    function hideMailAlertBottomBanner(clickedCloseButton){
        if (clickedCloseButton)
            saveCloseMaBannerCookie();
        if (document.getElementById('mail_alert_banner_bottom'))
            MEslideDown("mail_alert_banner_bottom",10);
        var pushUpPage = document.getElementById('pushUpPage') ? document.getElementById('pushUpPage') : null;
        if (pushUpPage)
            pushUpPage.parentNode.removeChild(pushUpPage);
    }

    function drawMailAlertBottomBanner(email,categoria,contratto){
        if (typeof email=="undefined")
            email="";
        if (typeof categoria=="undefined")
            categoria = null;
        if (typeof contratto=="undefined")
            contratto = 0;
        if (!getMaBannerCookie(categoria,contratto) || !categoria){
            saveMaBannerCookie(categoria,contratto);
            preloadImages(new Array("/img2/trova_immobili_banner_content_bkg.png","/img2/trova_immobili_banner_bkg.png"));
            document.write('<div class="mail_alert_box1" id="mail_alert_banner_bottom" style="display:none;">\
                     <input type="hidden" name="eventTrackName" value="mailAlertSearchBottomBanner" />\
                    <div class="mail_alert_box_bck" >\
                        <div class="mail_alert_banner_close" onclick="hideMailAlertBottomBanner(true)"></div>\
                        <div class="banner_content" id="ma_banner_content">\
                            <div class="mail_alert_email">\
                                <input type="text" name="email" '+(email ? 'value="'+email+'" disabled' : 'value="inserisci la tua email" startvalue="inserisci la tua email"')+' onclick="if (this.value==\'inserisci la tua email\') this.value=\'\'" onblur="this.value = this.value.trim();if (this.value==\'\') this.value=\'inserisci la tua email\'" />\
                            </div>\
                            <div class="bottoneAttMailAlertContainer">\
                                <input type="submit" class="bottoneAttivaServizio2" onclick="this.blur();" value=""/>\
                            </div>\
                            <div class="clear"></div>\
                        </div>\
                    </div>\
                </div>');
            if (typeof showMailAlertBottomBanner == "function")
                addEvent(window,"load",showMailAlertBottomBanner);
        }
    }

function saveMaBannerCookie(categoria,contratto){
	if (typeof categoria == "undefined"){
		_raiseError("called function with empty requested argument");
	}
	try{
		var sCookie = readCookie('maSearchBanner');
        if (typeof sCookie == "string" && sCookie=="closed"){
            return true;
        }
		var oCookie = new Object();
		if (sCookie){
			sCookie = Base64.decode(sCookie)
			oCookie = eval( "(" + sCookie + ")" );
		}
        if (!oCookie[categoria])
            oCookie[categoria] = new Array();
        oCookie[categoria][contratto] = 1;

        /* set COOKIE */
        /* ENCODED COOKIE */
		sCookie = Base64.encode(stringify(oCookie));
        createCookie("maSearchBanner",sCookie,90,0,0);

        /* ******** */

	}catch(e){
		deleteCookie("maSearchBanner");
		_raiseError(e);
	}
}


function saveCloseMaBannerCookie(){

	try{
		var sCookie = "closed";
        sCookie = Base64.encode(sCookie);
        createCookie("maSearchBanner",sCookie,90,0,0);

	}catch(e){
		deleteCookie("maSearchBanner");
		_raiseError(e);
	}
}


function getMaBannerCookie(categoria,contratto){
	oCookie = null;
	try{
		var sCookie = readCookie('maSearchBanner');
		var oCookie = null;
        var val = null;
		if (sCookie){
			sCookie = Base64.decode(sCookie);
            if (typeof sCookie == "string" && sCookie=="closed"){
                 return true;
            }
			oCookie = eval( "(" + sCookie + ")" );
		}
        if (oCookie){
            if (oCookie[categoria] && contratto){
                if(oCookie[categoria][contratto])
                    val = oCookie[categoria][contratto]
                else
                    val = null;
            }else
            if (!contratto)
                val = oCookie[categoria] ? oCookie[categoria] : null;
        }
	}catch(e){
		deleteCookie("maSearchBanner");
		_raiseError(e);
        return null;
	}
	return val;
}

function checkTrovaImmobiliForm(oForm){
        return true;
}

function sendSearchToAgency(email,nome,telefono,typeService){
   load_zone_confirm_button();
   appendCss("/includes/trova_immobili.css","mail_alert_css");
   _mail_alert_box = __selectedOvBox;
    //_mail_alert_box.fixedPosition = false;
    //showLoader(_mail_alert_box);

	if ( typeof typeService != 'undefined' && typeService == 'ricevi' ) {
		_mail_alert_box.setTitle("AlertEmail");
		__TI_servizio_1 = true;
		__TI_servizio_2 = false;
		document.getElementById( 'TI_servizio_2' ).value = '';
		document.getElementById( 'TI_servizio_1' ).value = 'on';
		var service_proposed = 1;
	} else {
		_mail_alert_box.setTitle("TrovaImmobili");
		__TI_servizio_1 = false;
		__TI_servizio_2 = true;
		document.getElementById( 'TI_servizio_2' ).value = 'on';
		document.getElementById( 'TI_servizio_1' ).value = '';
		var service_proposed = 2;
	}
    _mail_alert_box.show();


   document.getElementById('ti_utente_mail').value=email;
   document.getElementById('ti_utente_nome').value=nome;
   document.getElementById('ti_utente_telefono').value=telefono;
   document.getElementById('isCheckRequest').value='1';

   submitTIForm(function(data){
           document.getElementById('isCheckRequest').value=0;
           var xmlDoc = data.responseXML;
           var errors = xmlDoc.getElementsByTagName('errori');
           errorMessage = "";
            if (errors.length > 0){
                var errori = errors[0].childNodes;
                errorMessage = "<span class=\"TI_err_msg_icon TI_err_title\">Mancano le seguenti informazioni: </span>";
                for (var i=0;i<errori.length;i++){
					if ( errori[i].tagName == "erroreTentativi" ) {
						showErrorAttempts();
						document.body.style.cursor = 'default';
						return;
					} else {
					 errorMessage+="<span>"+errori[i].childNodes[0].nodeValue+"</span>";
					}
                    if (i!=errori.length-1)
                        errorMessage+=", ";
                }
                showErrMsg(errorMessage);
                display_zone_confirm_button();
            }else{
               _mail_alert_box.setContent(create_success_box(false,service_proposed ));
			   trackPage( '/trovakasa.php?type=mailAlert' );
                submitAjaxForm('trova_immobili',function(){});
            }
           _mail_alert_box.show();
   },true);
}

function showErrorAttempts() {
   var oAjax = new getMEAjaxObj();
   var ulrAjax = '/trovakasa.php?action=showErrorAttempts';
   oAjax.Request( 'POST' , ulrAjax , function( response){ _mail_alert_box.setContent( response.responseText );
	  _mail_alert_box.show();
	});
 }

function checkIfSaveSearch(isLogged,$page_md5){
	try{
		if (isLogged){
			var ricerca = readCookie("salvaRicerca");
			if (ricerca == $page_md5){
				startMailAlertProcess(document.getElementById('salva_ricerca_top_form'),1);
			}
		}
		deleteCookie("salvaRicerca");
	}catch(e){
		deleteCookie("salvaRicerca");
	}
}


var __iPopupMaxTimes = 3;

function checkAndShowTIPopop(canonicalUrl,tipoUtente,maxTimes){
   try{
      maxTimes = typeof maxTimes != "undefined" ? parseInt(maxTimes) : __iPopupMaxTimes;
      var  num_vis_pap_ti = 0;
      var url = "";
      var pap_ti_ck = readCookie( 'IM_PAP_TI');

      if ( pap_ti_ck) {
          pap_ti_ck = Base64.decode(pap_ti_ck);
          pap_ti_ck = eval("("+pap_ti_ck+")");
          var num_vis_pap_ti = pap_ti_ck["num"];
          url = pap_ti_ck["url"];
      }
	  if (num_vis_pap_ti < (maxTimes-1) && url != canonicalUrl ){
         num_vis_pap_ti++;
         addEvent( window, "load" , function() {
			if (__blockTIAutoPopup)
			   return;
			startMailAlertProcess( document.getElementById( 'salva_ricerca_top_form' ), tipoUtente );
			pap_ti_ck = new Object();
			pap_ti_ck.num = num_vis_pap_ti;
			pap_ti_ck.url = canonicalUrl;

			pap_ti_ck = Base64.encode(stringify(pap_ti_ck));
			pap_ti_ck = stringify(pap_ti_ck);

			createCookie( 'IM_PAP_TI', pap_ti_ck,7);
		 } );
	  }
   }catch(e){}
}

function updateContTIPopup(canonicalUrl, maxTimes){
	try{
		maxTimes = typeof maxTimes != "undefined" ? parseInt(maxTimes) : __iPopupMaxTimes;
		//aggiornamento cookie
		pap_ti_ck = new Object();
		pap_ti_ck.num = maxTimes;
		pap_ti_ck.url = canonicalUrl;
		pap_ti_ck = Base64.encode(stringify(pap_ti_ck));
		pap_ti_ck = stringify(pap_ti_ck);

		createCookie( 'IM_PAP_TI', pap_ti_ck,7);
	 }catch(e){}
}


function selectTabAlertEmail(idToggleElement) {
   jq("#alertEmailContainer .nav-tabs li").each( function() {
	  if ( jq(this).attr('data-toggle') == idToggleElement ) {
		 jq(this).addClass('active');
	  } else {
		 jq(this).removeClass('active');
	  }
   });
}
/* GESTIONE AREA POLIGONO */

function loadPreviewZone(onlyModificaLuogo) {
   onlyModificaLuogo = typeof onlyModificaLuogo == "undefined" ? false : onlyModificaLuogo;
   /* ANTEPRIMA ZONA COMUNE */
   if ( !document.getElementById( 'zoneTI_ids' ) ) {
	  return;
   }

   var iDs = trim2( document.getElementById( 'zoneTI_ids' ).value, ',' );
   var zoneSelected = iDs.split(",");

	if (isTIVacanze() && TI_agComuneField && TI_agComuneField.getRegione()) {
		var idRegione = TI_agComuneField.getRegione().idObject;
	} else if (typeof document.dati.idRegione != "undefined")
	  var idRegione = document.dati.idRegione.value;
   else if (typeof document.dati.regione != "undefined")
	  var idRegione = document.dati.regione.value;
	else
	  return;

   var idProvincia = getIDProvincia();
   if (!idProvincia)
	  return;

   if (onlyModificaLuogo) {
	  jq("#box_zone_TI").empty();
   } else {
	  jq("#prev_box_zone_TI").empty();
	  jq("#box_zone_TI").empty();
   }

   jq("#ZoneSelectTI input").each(function() {
	  for (var i = 0; i < zoneSelected.length; i++) {
		 if (zoneSelected[i] == jq(this).val()) {
			var src = '/img2/cartine/'+idRegione+'/'+idProvincia+'/map_zona_'+zoneSelected[i]+'.gif?v=2';
			if (onlyModificaLuogo) {
			   jq("#box_zone_TI").append("<img src='"+src+"' />");
			} else {
			   jq("#box_zone_TI").append("<img src='"+src+"' />");
			   jq("#prev_box_zone_TI").append("<img src='"+src+"' />");
			}

			break;
		 }
	  }
   });
}

function loadPreviewMapAlertEmail() {
   loadGoogleMapStaticImage("geografia");
   loadGoogleMapStaticImage("riepilogo")
}

function loadGoogleMapStaticImage(section, forceReload) {
   if (typeof forceReload == "undefined")
	  forceReload = false;

   var baseId = '#img_google_map_statics';

   if (section == "riepilogo") {
	  var idContainer = 'prev_overlay_opacity_map_TI';
	  var sufix = '_prev_TI';
   } else {
	  var idContainer = 'overlay_opacity_map_TI';
	  var sufix = '_TI';
   }

   baseId += sufix;
   if ( !forceReload && jq(baseId) && typeof jq(baseId).attr('src') != 'undefined' && jq(baseId).attr('src') != '' ) {
	  return true;
   }
   var jsonDrawedArea = jq("#gm_area_TI").val();
   if (jsonDrawedArea == '') {
	  return false;
   }

   var oArea = jQuery.parseJSON(jsonDrawedArea);
   var bDone = staticMapHP(document.getElementById(idContainer), oArea.mode, oArea.area, google_map_key, sufix, 150, 150);
   return bDone;
}

function initAlertEmailArea() {

	var suggestion = new Im_Suggestion("TI_username_ricerca",
		{
			selectRowOnOver       : false,
			hideOnMouseOut        : false,
			hideNoResultDiv		  : true,
			blockOnBlur			  : true,
			focusedElementOnSelect: null,
			activeTitle			  : true,
			titleNoClick		  : "Forse intendevi...",
			inOverlay		      : true,
			minChars              : 2,
			maxResults            : 4,
			url                   : "/services/checkMailDomain.php?soglia=1000",
			onselect              : function (selectedItem) {},
			onsearch              : function (qs) {
				return checkEmail(qs) ? true : false;
			},
			autoselectFirstItem   : false,
			getResultOnFocus      : true,
			resetQsFieldOnNotSel  : false
		}
	),
	jqMultiSelect = jq('.multiSel');

	if(typeof jq.fn.SumoSelect !== 'undefined' && typeof jq.fn.SumoSelect == 'function'){
		//sumoselect alertmail se attiva ricerca per piano
		jqMultiSelect.SumoSelect({
			placeholder: jqMultiSelect.attr('data-placeholder'),
			csvDispCount: 3,
			floatWidth: 500,
			okCancelInMulti: false,
			captionFormatAllSelected: 'Qualsiasi',
			captionFormat: '{0} selezionati',
			up: parseInt(jqMultiSelect.attr('data-open-up'), 10) || 0,
			classSumo: jqMultiSelect.attr('data-classSumo') || '',
			prefix: jqMultiSelect.attr('data-prefix') || ''
		});
	}

	// viene posizionato il focus sulla mail
   if (!jq("#TI_username_ricerca").attr('disabled')) {
	  jq("#TI_username_ricerca").focus();
   }

   jq(".ui-radio div").mouseenter( function() {
	  jq(this).removeClass('ui-btn-up-c');
	  jq(this).addClass('ui-btn-hover-c');
   }).mouseleave( function() {
	  jq(this).removeClass('ui-btn-hover-c');
	  jq(this).addClass('ui-btn-up-c');
   });

   jq('#alertEmailContainer .nav-tabs li a').click( function(){ jq(this).blur(); });

   jq('#modifica_zona_TI').click(function(event){
	  event.stopPropagation();
	  modificaDatiAlertEmail( 3 );
	  showZoneSelectForm();
   });

   if (document.getElementById('comune_lat')) {
	  var latitudine = document.getElementById('comune_lat').value;
	  var longitudine = document.getElementById('comune_lng').value;
	  initGoogleMapRicercaArea('alertemail', 3, {lat:latitudine, lng:longitudine, forceCenter:0, hideOnlyBox:1, mapSection:'alertemail'});
   } else {
	  initGoogleMapRicercaArea('alertemail', 3, {hideOnlyBox:1, mapSection:'alertemail'});
   }

   loadPreviewZone();
   loadPreviewMapAlertEmail();
   managePreviewImage('prev_');
   manageTIForm();
   manageVacanze();

   var searchFormTI = new SearchFormTI();
   searchFormTI.init();

	jq('#TI_nomeRicerca').on('keyup', function(e){
		if (e.keyCode == 188 || e.keyCode == 190) { // tag < e >
			var value = jq(this).val();
			var caretPos = getCaretPosition(jq(this)[0]);
			jq(this).val(value.replace(/[\<\>]/g,""));
			setCaretPos( jq(this)[0], caretPos -1);
		}
	});
}

var trovaImmobiliForm = null;
function manageTIForm() {
	trovaImmobiliForm = new SearchForm(jq("#trova_immobili")[0],
									   {fkCategoria: jq("#categoria_TI").val(),
										trovaImmobili: true});
	trovaImmobiliForm.init();
}

function isTIVacanze() {
	return (document.forms.trova_immobili && document.forms.trova_immobili.vacanze && document.forms.trova_immobili.vacanze.value) ||
		(typeof document.getElementById("TI_agComuneField") != "undefined" && document.getElementById("TI_agComuneField") != null);
}

var TI_agComuneField = null;
function manageVacanze() {
	if (!isTIVacanze()) {
		return;
	}

	if (!jq("#TI_agComuneField").length) {
		return;
	}

	var agOptions = {
		areaHiddenInput: jq("#TI_areaGeografica")[0],
		comuneHiddenInput: jq("#TI_comune")[0],
		suggestionOptions: { parentNode: jq("#TI_agComuneField").parent()[0],
							 inOverlay: true }		
	};

	TI_agComuneField = new AGComuneField ('TI_agComuneField', agOptions);
	TI_agComuneField.init();
	jq(TI_agComuneField).on('change', function () {
		resetInfoLuogoVacanze();
	});
}

function resetInfoLuogoVacanze() {
	var object = TI_agComuneField.getObject();
	if (object.type == "areaGeografica") {
		resetZone();
		jq('#modifica_zona_TI').hide();
		jq('#modifica_area_TI').hide();

		jq('#cittaConZona').val(0);

		jq('#cartina_comune').hide();
		jq('#cartina_comune_senza_zone').show();

		jq('#prev_cartina_comune').hide();
		jq('#prev_cartina_comune_senza_zone').show();

		jq('#textZonaBtn').html('Tutta la località');

		jq('#img_confini_comune').attr('src', TI_agComuneField.getUrlCartina());
	} else {
		resetInfoComune(object.idObject, object.provincia.idObject);
	}
	deselectAllText();
}

function resetInfoComune(idComune, idProvincia) {
   var isEnabledZone = document.getElementById('enabledZone').value == 1 ? true : false;
   var ricercaSuMappaAbilitata = document.getElementById('ricercaSuMappaAbilitata').value == 1 ? true : false;
   resetZone();
   if (ricercaSuMappaAbilitata)
	  xajax_getAjaxResetAreaMappa(idProvincia, idComune, 'alertemail');
   if (isEnabledZone) {
	  xajax_getMapAjaxZonePagRicerca(idComune, idProvincia);
	  xajax_getAjaxAsZoneTrovaImmobili(idComune, idProvincia);
   } else {
	  changeLocationImage(0, idComune);
   }
   jq('#modifica_zona_TI').hide();
   jq('#modifica_area_TI').hide();
}

function changeLocationImage(isCittaConZona, idComune) {
   if ( isCittaConZona ) {
	  loadPreviewZone();
	  jq('#cartina_comune').show();
	  jq('#cartina_comune_senza_zone').hide();

	  jq('#prev_cartina_comune').show();
	  jq('#prev_cartina_comune_senza_zone').hide();

	  jq('#textZonaBtn').html('Scegli la zona');
   } else {
	  // Carico la cartina con i confini del comune.
	  if( !isEstero() )
		xajax_getAjaxComuneImage('img_confini_comune', idComune);

	  jq('#cartina_comune').hide();
	  jq('#cartina_comune_senza_zone').show();

	  jq('#prev_cartina_comune').hide();
	  jq('#prev_cartina_comune_senza_zone').show();

	  jq('#textZonaBtn').html('Tutta la citt&agrave;');
   }

   forceTypeSearch('zona', '_TI');

   jq('#zone_cartina_TI').show();
   jq('#prev_zone_cartina_TI').show();

   jq('#overlay_opacity_map_TI').hide().html('');
   jq('#prev_overlay_opacity_map_TI').hide().html('');

   jq('#textAreaBtn').html('Disegna su mappa');
   jq('#gm_area_TI' ).val('');
   jq('#tipoRicerca_TI' ).val('');
}

function trimText(s) {
   s.trim();
   s = s.replace(/(<([^>]+)>)/ig,"");
   s = s.replace(/[\t\s\n]+/ig," ");
   s = s.trim();

   return s;
}

function managePreviewImage(prefix) {
   if ( typeof prefix == "undefined")
	  prefix = '';

   var tipoRicerca = jq('#tipoRicerca_TI').val();

   if (tipoRicerca == 'mappa') {
	  jq("#"+prefix+"zone_cartina_TI").hide();
	  jq("#"+prefix+"overlay_opacity_map_TI").show();
   } else {
	  jq("#tipoRicerca_TI").val('');
	  jq("#"+prefix+"zone_cartina_TI").show();
	  jq("#"+prefix+"overlay_opacity_map_TI").hide();
   }
}

function getIDProvincia() {
   var idProvincia = null;
	if (isTIVacanze() && TI_agComuneField && TI_agComuneField.getProvincia()) {
		idProvincia = TI_agComuneField.getProvincia().idObject;
	} else if (typeof document.form_alert_email != "undefined" && typeof document.form_alert_email.provincia != "undefined") {
	  idProvincia = document.form_alert_email.provincia.value;
   } else if (typeof document.dati.idProvincia != "undefined") {
	  idProvincia = document.dati.idProvincia.value;
   }

   return idProvincia;
}

var SearchFormTI = function(){
	this.speedSlide = 'fast';
}

SearchFormTI.prototype.init = function(){

	var self = this;

	this.categoria = jq('#categoria_TI');
	
	if (this.categoria.val() == 4){
		this.categoriaStanza = jq('#selCategoriaStanzaTI');
		this.tipologiaStanza = jq('#divTipologiaStanzaTI');
		this.categoriaStanza.change(function(){
			self.onCategoriaStanzaChange();
		});
		self.onCategoriaStanzaChange();
	} else {
		this.tipologia = jq('#tipologia');
		this.tipologia.change(function(){
			self.onTipologiaChange();
		});
		self.onTipologiaChange();
	}
}

SearchFormTI.prototype.onTipologiaChange = function(){
		
	var tipologiaVal = this.tipologia.val();
	var tipologiaValCustom = new Array('6', '7', '11', '12', '13');

	var tipologiaFieldsDefault = {
		divAscensoreTI:{actions:['showDiv']},
		divGiardinoTI:{actions:['showDiv']},
		divLocaliTI:{actions:['showDiv']},
		divRiscaldamentoTI:{actions:['showDiv']},
		divBoxAutoTI:{actions:['showDiv']},
		divArredatoTI:{actions:['showDiv']},
		divTerrazzoTI:{actions:['showDiv']},
		divBalconeTI:{actions:['showDiv']}
	}

	var tipologiaFields = {
		'6': jq.extend({}, tipologiaFieldsDefault, {
				ascensoreTI:{actions:['resetCheckbox']},
				divAscensoreTI:{actions:['hideDiv']},
				giardinoTI:{actions:['resetSelect']},
				divGiardinoTI:{actions:['hideDiv']},
				locMin:{actions:['resetField']},
				locMax:{actions:['resetField']},
				divLocaliTI:{actions:['hideDiv']},
				riscaldamento:{actions:['resetSelect']},
				divRiscaldamentoTI:{actions:['hideDiv']},
				boxautoTI:{actions:['resetSelect']},
				divBoxAutoTI:{actions:['hideDiv']},
				arredatoTI:{actions:['resetCheckbox']},
				divArredatoTI:{actions:['hideDiv']},
				terrazzoTI:{actions:['resetCheckbox']},
				divTerrazzoTI:{actions:['hideDiv']},
				balconeTI:{actions:['resetCheckbox']},
				divBalconeTI:{actions:['hideDiv']}
			}),
		'7': jq.extend({}, tipologiaFieldsDefault, {
				ascensoreTI:{actions:['resetCheckbox']},
				divAscensoreTI:{actions:['hideDiv']}
			}),
		'11': jq.extend({}, tipologiaFieldsDefault, {
				ascensoreTI:{actions:['resetCheckbox']},
				divAscensoreTI:{actions:['hideDiv']}
			}),
		'12': jq.extend({}, tipologiaFieldsDefault, {
				ascensoreTI:{actions:['resetCheckbox']},
				divAscensoreTI:{actions:['hideDiv']}
			}),
		'13': jq.extend({}, tipologiaFieldsDefault, {
				ascensoreTI:{actions:['resetCheckbox']},
				divAscensoreTI:{actions:['hideDiv']}
			})
	}

	if (tipologiaVal && (jq.inArray(tipologiaVal, tipologiaValCustom) > -1)){
		var fields = tipologiaFields[tipologiaVal];
	} else {
		var fields = tipologiaFieldsDefault;
	}

	for (var field in fields){
		var actions = fields[field]['actions'];
		for (var action in actions){
			this[actions[action]](field);
		}
	}
}


SearchFormTI.prototype.onCategoriaStanzaChange = function(){
	
	var categoriaStanzaVal = this.categoriaStanza.val();
	var categoriaStanzaValCustom = new Array('1', '2');

	var categoriaStanzaFieldsDefault = {}

	var categoriaStanzaFields = {
		'2': jq.extend({}, categoriaStanzaFieldsDefault, {
				divTipologiaStanzaTI:{actions:['showDiv']}
			}),
		'1': jq.extend({}, categoriaStanzaFieldsDefault, {
				selTipologiaStanzaTI:{actions:['resetSelect']},
				divTipologiaStanzaTI:{actions:['hideDiv']}
			})
	}

	if (categoriaStanzaVal && (jq.inArray(categoriaStanzaVal, categoriaStanzaValCustom) > -1)){
		var fields = categoriaStanzaFields[categoriaStanzaVal];
	} else {
		var fields = categoriaStanzaFieldsDefault;
	}

	for (var field in fields){
		var actions = fields[field]['actions'];
		for (var action in actions){
			this[actions[action]](field);
		}
	}
}

SearchFormTI.prototype.showDiv = function(id){
	var div = jq('#'+id);
	if (div.is(':visible') == false){
		div.show();
	}
}
	
SearchFormTI.prototype.hideDiv = function(id){
	var div = jq('#'+id);
	if (div.is(':visible') == true){
		div.hide();
	}
}
	
SearchFormTI.prototype.resetField = function(id){
	var field = jq('#'+id);
	field.val('');
}
	
SearchFormTI.prototype.resetSelect = function(id){
	var select = jq('#'+id);
	select.find('option:first').attr('selected','selected');
}

SearchFormTI.prototype.resetCheckbox = function(id){
	var checkbox = jq('#'+id);
	checkbox.attr('checked', false);
}

/**
 * Si occupa di processare le azioni quando si salvano le modifiche apportate al form Alert Email
 * @param opt
 */
function saveChangesAlertEmail(opt) {
	opt = opt || {};
	if (typeof opt.where != 'undefined') {
		if (opt.where == 'characteristics') {
			controllaDatiFormAlertEmail(opt.type);
		} else if (opt.where == 'location') {
			checkLocation();
		}
	}
	_searchNameManager.updateSearchName();
}

/**
 * Classe che si occupa della gestione del ricalcolo del nome ricerca
 * @constructor
 */
SearchNameManager = function () {
	this.formId = '#trova_immobili';
	this.formElm = jQuery(this.formId);
	this.nomeRicercaElm = jQuery('#TI_nomeRicerca');
	this.nomeRicercaDefaultElm = jQuery('#TI_nomeRicercaDefault');
	this.needUpdate = false;

	//Si occupa di ricalcolare il nome della ricerca se ce n'e' bisogno
	this.updateSearchName = function () {
		if (this.needUpdate && this.nomeRicercaElm.val() == this.nomeRicercaDefaultElm.val()) {
			//se ho bisogno di aggiornare e l'utente non ha toccato il nome della ricerca la aggiorno
			var that = this;
			jQuery.ajax({
				url: that.formElm.attr('action'),
				async: false,
				type: that.formElm.attr('method'),
				data: that.formElm.serialize() + '&getSearchName=1',
				dataType: 'json'
			}).done(function (response) {
				that.nomeRicercaElm.val(response.searchName);
				that.nomeRicercaDefaultElm.val(response.searchName);
			});
		}
	};

	// Aggiungo gli ascoltatoti su alcuni elementi che se cambiano imposto che devo ricalcolare il nome ricerca
	this.addListener = function () {
		var that = this;
		jQuery(this.formId).on('change', '.updateSearchName', function () {
			that.needUpdate = true;
		});
	};
};

//Posiziona il form di invio email al centro della pagina
function responseHandler(data){
	segnalazioniEcommentiBox.setContent(data.responseText);
}
function sendResponsabileMail( urlMedia ){

	//recreate snapshot immagini
	getIframeImmaginiPrivati( urlMedia );

	if (validateMailForm()){
		/*
		 - se esiste  jq('#frameImgSnapshot').val() => invia segnalazione - clearTimeout
		 - se non esiste insisto per 10 volte, alla 10 clearTimeout e invio comunque segnalazione
		 */
		var countTentativiSegnalazione = 0;
		var intervalSendSegnalazione = setInterval(function () { checkSegnalazione() }, 500);
		function checkSegnalazione() {
			var supportedPostMessage = window['postMessage'] && (typeof(window['postMessage']) != 'undefined');

			if( typeof(oActionPM) != 'undefined' && ((!jq.isEmpty(oActionPM) && 3 == oActionPM.step) && jq('#frame_immagini').exist() && supportedPostMessage)){
				if(!jq.isEmpty( jq('#frameImgSnapshot').val() ) || countTentativiSegnalazione > 9){
					inviaSegnalazione();
					countTentativiSegnalazione = 0;
				}else
					countTentativiSegnalazione++;
			}else{
				inviaSegnalazione();
			}
		}
		function inviaSegnalazione(){
			submitAjaxForm("mailSegnalaPrivatoForm",responseHandler);
			segnalazioniEcommentiBox.setContent("<div style=\"width:375px;height:300px;text-align:center;\"><h1>Invio messaggio in corso...</h1><br /><span style=\"line-height:300px;\"><img src=\"/img2/loader6.gif\"></span></div>");
			clearInterval(intervalSendSegnalazione);
		}
	}
}

function sendResponsabileAlertBorsellinoMail()
{
	var oAjax = new getMEAjaxObj();
	pars="errorBox=4";
	var url = "/inviaEmailResponsabile.php?"+pars;
	oAjax.Request('POST',url, function(){});
}

function validateMailForm()
{
	var errMsg = false;
	
	var iNome = document.getElementById('im_name');
	var iEmail = document.getElementById('im_email');
	var iMessaggio = document.getElementById('im_message');
	
	var lNome = document.getElementById('label_name');
	var lEmail = document.getElementById('label_email');
	var lMessaggio = document.getElementById('label_message');
	
	iNome.className = lNome.className = "";
	iEmail.className = lEmail.className = "";
	iMessaggio.className = lMessaggio.className = "";
	
	var iNomeV = iNome.value;
	if (iNomeV == ""){
		iNome.className = "input_error";
		lNome.className = "errore_inserimento";
		errMsg = true;
	}
	else{
		var nomeRegEx = /^[\w\s\'àèìòù]+$/;
		if (!iNomeV.match(nomeRegEx)){
			iNome.className = "input_error";
			lNome.className = "errore_inserimento";
			errMsg = true;
		}
	}
	
	var iEmailV = iEmail.value;
	if (iEmailV == ""){
		iEmail.className = "input_error";
		lEmail.className = "errore_inserimento";
		errMsg = true;
	}
	else{
		if (!isEmail(iEmailV)){
			iEmail.className = "input_error";
			lEmail.className = "errore_inserimento";
			errMsg = true;
		}
	}
	
	var iMessaggioV = iMessaggio.value;
	if (iMessaggioV == ""){
		iMessaggio.className = "input_error";
		lMessaggio.className = "errore_inserimento";
		errMsg = true;
	}
	
	if (errMsg){
		return false;
	}
	
	if (document.dati){
		var infoForm = document.getElementById('infoForm');
		if (!infoForm) {
			infoForm = document.createElement('input');
			infoForm.id = 'im_infoForm';
			infoForm.type = 'hidden';
			document.getElementById('mailSegnalaPrivatoForm').appendChild(infoForm);
		}
		infoForm.value = qsFromForm(document.dati);
	}
	return true;
}

function designResponsabileBox(flag)
{
	//Questa funziona serve solo a gestire la chiamata al box da parte del banner di commento backoffice.
	//Siccome il banner ha una vita limitata ho preferito fare un wrapper anzichè modificae il banner flash e ricaricarlo.
	if (flag)
		designMailBox(flag);
}

function designMailBox(type, urlMedia, idAnnuncio)
{
	var messaggio, oggetto, im_idAnnuncio;
	var pageSnapshot = Page.snapshot();

	// idAnnuncio exists only if we are in user publish pages and not before step 2
	if (typeof idAnnuncio === 'undefined' || idAnnuncio === null) {
		im_idAnnuncio = 'undefined';
	} else  {
		im_idAnnuncio = idAnnuncio;
	}

	switch (type){
		case 3:
			segnalazioniEcommentiBox.setTitle("<strong>Segnalazioni e suggerimenti</strong>");
			messaggio = '<h2>Che ne pensi della ricerca su mappa?</h2><span style="font-size:13px;line-height:15px;">Inserisci qui sotto suggerimenti e commenti, oppure segnala gli eventuali problemi riscontrati.<br /> Grazie per la preziosa collaborazione!</span>';
			oggetto = 'Commento per ricerca su mappa - Immobiliare.it';
			break;
		case 2:
		case 4:
			segnalazioniEcommentiBox.setTitle("<strong>Segnalazioni e suggerimenti</strong>");
			messaggio = '<h2>Che ne pensi della ricerca su mappa?</h2><span style="font-size:13px;line-height:15px;">Inserisci qui sotto suggerimenti e commenti, oppure segnala gli eventuali problemi riscontrati.<br /> Grazie per la preziosa collaborazione!</span>';
			oggetto = 'Commento per disegna su mappa - Immobiliare.it';
			break;
		default :
			segnalazioniEcommentiBox.setTitle("<strong>Segnalazioni e suggerimenti</strong>");
			messaggio = '<h2>Aiutaci a migliorare questo servizio...</h2><span style="line-height:15px;">Utilizza il form sottostante se hai dei suggerimenti da darci oppure per segnalarci le difficolt&#224; riscontrate nell\'inserire il tuo annuncio .</span>';
			oggetto = 'Commento per pubblicazione annuncio - Immobiliare.it';
			break;
	}
	segnalazioniEcommentiBox.setContent('\
	<div id="containerSegnalaPrivatoSendMail" style="width:580px;font-size:13px">\
	<div id="segnalaPrivatoSendMail">\
		<div id="mailForm">\
			<div class="clear" style="height:10px"></div>\
			<form name="mailsegnalaPrivatoForm" id="mailSegnalaPrivatoForm" action="/inviaSegnalazione.php">\
				<input name="im_subject" id="im_subject" value="'+oggetto+'" type="hidden">\
				<input name="im_idAnnuncio" id="im_idAnnuncio" value="'+im_idAnnuncio+'" type="hidden">\
				<input type=\"hidden\" name="errorBox" value="'+(type ? type : "0")+'"/>\
				<input type=\"hidden\" name=\"userData\" id=\"userData\"/>\
				<input type=\"hidden\" name=\"pageSnapshot\" id=\"pageSnapshot\"/>\
				<input type=\"hidden\" name=\"frameImgSnapshot\" id=\"frameImgSnapshot\"/>\
				<table class="form_tabled_post" style="" cellpadding="0" cellspacing="0">\
					<tr>\
						<td colspan=3>'+messaggio+'<div class="clear" style="height:10px"></div></td>\
					</tr>\
					<tr>\
						<td>\
							<table class="form_tabled_post" style="" cellpadding="0" cellspacing="0">\
								<tr>\
									<td>\
										<label id="label_name" for="im_name"><strong>Nome:</strong></label>\
										<input name="im_name" id="im_name" size="40" maxlength="255" value="" type="text" style="height:25px">\
									</td>\
								</tr>\
								<tr>\
									<td style="height:0px"></td>\
								</tr>\
								<tr>\
									<td>\
										<label id="label_email" for="im_email"><strong>Email:</strong></label>\
										<input name="im_email" id="im_email" size="40" maxlength="255" value="" type="text" style="height:25px">\
									</td>\
									<td></td>\
								</tr>\
							</table>\
						</td>\
						<td>\
							<table class="form_tabled_post" style="" cellpadding="0" cellspacing="0">\
								<tr>\
									<td rowspan="3">\
										<label id="label_message" for="im_message"><strong>Messaggio:</strong></label>\
										<textarea name="im_message" id="im_message" style="width:280px;height:85px;"></textarea>\
									</td>\
								</tr>\
							</table>\
						</td>\
					</tr>\
					<tr>\
						<td></td>\
						<td>\
							<div style="float:right;">\
								<a id="inviaButton">\
									<div style="background:url(/img2/bottoni/puls-invia.png) top center no-repeat; height:48px; width:92px;cursor:pointer;">&nbsp;</div>\
								</a>\
							</div>\
						</td>\
					</tr>\
				</table>\
			 </form>\
		</div>\
		<div id="mailResponse" style="display:none;"></div>\
	</div>\
	<div class="clear"></div>\
	</div>');

	//sendButton action
	jq('#mailSegnalaPrivatoForm').on('click', '#inviaButton', function(){
		if(typeof(urlMedia) != 'undefined')
			sendResponsabileMail( urlMedia );
		else
			sendResponsabileMail();
	});

	
	if (typeof hideInputSugg != "undefined")
	hideInputSugg();
	segnalazioniEcommentiBoxContent = segnalazioniEcommentiBox.getContent();
	segnalazioniEcommentiBox.addEventOnClose(function(){segnalazioniEcommentiBox.setContent(segnalazioniEcommentiBoxContent);});
	segnalazioniEcommentiBox.show();
	
	
	if (document.getElementById("dati")){
		var qs = qsFromForm(document.getElementById("dati"));
		qs = Url.encode(qs);
		if (document.getElementById("userData"))
			document.getElementById("userData").value=qs;
	}
	
	if (document.getElementById('form_box_responsabile')){
		var aEm_btn = getElementsByClass("email",document.getElementById('form_box_responsabile'));
		if (aEm_btn.length > 0)
			aEm_btn[0].style.display = "none";
	}

	getIframeImmaginiPrivati( urlMedia );

	jq("#pageSnapshot").val(pageSnapshot);
}

function getIframeImmaginiPrivati(urlMedia){
	/***
	 * snapshot iframeimg
	 */
	var supportedPostMessage = window['postMessage'] && (typeof(window['postMessage']) != 'undefined');
	if(typeof(oActionPM) != 'undefined' && ((!jq.isEmpty(oActionPM) && 3 == oActionPM.step) && jq('#frame_immagini').exist() && supportedPostMessage)){

		var domain = urlMedia;
		var iframeImmagini = parent.document.getElementById('frame_immagini').contentWindow;
		var message = 'getSnapshot';
		iframeImmagini.postMessage(message,domain); //send the message and target URI

		//listening back message
		addEvent(window, 'message', function(event){
			var evt = (event || window.event);
			if(evt.origin !== domain) return;
			if(!jq.isEmpty(evt.data)){
				jq('#frameImgSnapshot').val(evt.data);
			}
		}.bind(this));
	}
	/*****/
}

function showSegnalaEcontattaBox( type ) {
	type = typeof type != "undefined" ? type : 2;
	createSegnalaEcommentaBox();
	designMailBox( type );
}

function createSegnalaEcommentaBox(){
	if (typeof segnalazioniEcommentiBox == "undefined"){
		segnalazioniEcommentiBox = new boxOverlay("contatta_box","grey");
		segnalazioniEcommentiBox.destroyOnClose=false;
		segnalazioniEcommentiBox.fixedPosition = true;
	}
}

function showCheNePensiRicercaPoligono(type, options){
	options = options || {};
	if (typeof bannerContattaTimer != "undefined")
		clearTimeout(bannerContattaTimer);
	
	if (typeof type == undefined)
		type = 1;

	createSegnalaEcommentaBox();
	if (typeof _bannersegnalazioniEcommentiBox == "undefined"){
		_bannersegnalazioniEcommentiBox = document.createElement("div");
		_bannersegnalazioniEcommentiBox.onmousedown = function(){
			designMailBox(type);
			return false;
		};
		_bannersegnalazioniEcommentiBox.id = "sideContactBanner";
	}

	var browser = getBrowser();
	var isIe = browser.name.toLowerCase().search(/explorer/)>=0;
	
	var wWidth = window.innerWidth ? window.innerWidth : document.documentElement.clientWidth;
	if ( typeof options.positionFixed != 'undefined' && options.positionFixed )
		wWidth = 999999; // disabilito il controllo sulla width

	if (wWidth > 1280 && _bannersegnalazioniEcommentiBox.className != "banner_contatta_right"){ //su larghezza finestra browser > 1024
	//if (!_ifWidePage(function(){ //su risoluzione dello schermo > 1024
		_bannersegnalazioniEcommentiBox.className = "banner_contatta_right";
		document.body.appendChild(_bannersegnalazioniEcommentiBox);
		MEslideBySide("sideContactBanner",isIe ? 4 : 1);
	}
	else
	if(wWidth <= 1280 && _bannersegnalazioniEcommentiBox.className != "banner_contatta_ricerca_poligono"){
	//)){
		_bannersegnalazioniEcommentiBox.className = "banner_contatta_ricerca_poligono";
		var nodo_rss_ricerca = document.getElementById("rss_ricerca");
		nodo_rss_ricerca.parentNode.insertBefore(_bannersegnalazioniEcommentiBox, nodo_rss_ricerca);
		MEslideBySide("sideContactBanner",isIe ? 12 : 3);
		//MEfadeIn(_bannersegnalazioniEcommentiBox,1,0.1)			
	}
}

function showContattaBannerPrivati(type, urlMedia, idAnnuncio){
	
	if (typeof bannerContattaTimer !== "undefined") {
		clearTimeout(bannerContattaTimer);
	}
	
	if (typeof type === "undefined") {
		type = 1;
	}

	createSegnalaEcommentaBox();
	if (typeof _bannersegnalazioniEcommentiBox == "undefined"){
		_bannersegnalazioniEcommentiBox = document.createElement("div");
		_bannersegnalazioniEcommentiBox.onmousedown = function(){
			designMailBox(type, urlMedia, idAnnuncio);
			return false;
		};
		_bannersegnalazioniEcommentiBox.id = "sideContactBanner";
	}

/*
	// TUTTA QUESTA PARTE DI CODICE È COMMENTATA PER INUTILIZZO ORA SI UTILIZZA LA CHAT INTERNA TOCTOC (2017-03-21)

	var browser = getBrowser();
	var isIe = browser.name.toLowerCase().search(/explorer/)>=0;
	
	var wWidth = window.innerWidth ? window.innerWidth : document.documentElement.clientWidth;
	
	if (wWidth > 1024 && _bannersegnalazioniEcommentiBox.className != "banner_contatta_right"){ //su larghezza finestra browser > 1024
	//if (!_ifWidePage(function(){ //su risoluzione dello schermo > 1024
		_bannersegnalazioniEcommentiBox.className = "banner_contatta_right";
		document.body.appendChild(_bannersegnalazioniEcommentiBox);
		MEslideBySide("sideContactBanner",isIe ? 4 : 1);
	}
	else
	if(wWidth <= 1024 && _bannersegnalazioniEcommentiBox.className != "banner_contatta_box_privati_top"){
	//)){
		_bannersegnalazioniEcommentiBox.className = "banner_contatta_box_privati_top";
		document.getElementById('banner_top_container').appendChild(_bannersegnalazioniEcommentiBox);
		MEslideBySide("sideContactBanner",isIe ? 12 : 3);
		//MEfadeIn(_bannersegnalazioniEcommentiBox,1,0.1)			
	}
*/
}
;
var Im_Suggestion = function(idField,oOptions){

    this.DEFAULT_MIN_CHARS = 2;
    this.DEFAULT_MAX_HEIGHT = 200;
    this.DEFAULT_MAX_RESULTS = 10;
	this.DEFAULT_PLACEHOLDER_COLOR = "#7A7A7A";
	this.DEFAULT_TEXT_COLOR = "#000";
    this.qs = "";
	this.keyupDelay = 100;
    this.results = null;
	this._digiting = false;
    this.mouseOverEnabled = true;
    this._selectedIndex = -1;
	this._blockOnBlurEvent = false;
	this.placeHolderText = "";
    this.status = "closed";
    if (!idField)
        throw("Missing param idField");
    
    this.idField = idField;
    this.id  = "Im_Suggestion_"+this.idField;
    if (!document.getElementById(this.idField))
        throw("Element "+idField+" does not exist");
    
    this.field = document.getElementById(this.idField);
    
    disableAutoComplete("#"+this.idField);
    
    this.field.onkeypress = function(event){return this._disableEnterKey(event)}.bind(this);
    this._lastKeyPressed = null;
	
    this._setOptions(oOptions); 
    this._createComponents();
    this._attachEvents();
	addEvent(document,"mouseup",function(){ this._blockOnBlur( this.options.blockOnBlur ); }.bind(this));
	addEvent(window,"unload",function(){purgeDomObj(this)}.bind(this));
}



Im_Suggestion.prototype._disableEnterKey = function(e)
{
     var key;     
     if(window.event)
          key = window.event.keyCode; //IE
     else
        key = e.which; //firefox
	
	this._lastKeyPressed = key;
	
    return (key != 13);
}

Im_Suggestion.prototype._setOptions = function(oOptions){
    
    this.options = typeof oOptions == "object" ? oOptions : {};
    
    this.options.maxResults = (typeof this.options.maxResults != "undefined") ? this.options.maxResults : this.DEFAULT_MAX_RESULTS;
    
    this.options.selectRowOnOver = (typeof this.options.selectRowOnOver != "undefined") ? this.options.selectRowOnOver : false;
    
    this.options.hideOnMouseOut = (typeof this.options.hideOnMouseOut != "undefined") ? this.options.hideOnMouseOut : false;
	
	this.options.resetQsFieldOnNotSel = (typeof this.options.resetQsFieldOnNotSel != "undefined") ? this.options.resetQsFieldOnNotSel : false;
    
    this.options.maxHeight = (typeof this.options.maxHeight != "undefined") ? parseInt(this.options.maxHeight) : this.DEFAULT_MAX_HEIGHT;
	
	this.options.placeholderText = (typeof this.options.placeholderText != "undefined") ? this.options.placeholderText : "";
	this.options.placeholderColor = (typeof this.options.placeholderColor != "undefined") ? this.options.placeholderColor : this.DEFAULT_PLACEHOLDER_COLOR;
	
	this.options.textColor = (typeof this.options.textColor != "undefined") ? this.options.textColor : this.DEFAULT_TEXT_COLOR; 
    this.options.focusedElementOnSelect = (typeof this.options.focusedElementOnSelect != "undefined") ? this.options.focusedElementOnSelect : ""; 
	
    this.options.onmouseover = (typeof this.options.onmouseover == "function") ? this.options.onmouseover : function(){};
	
    this.options.onclick = (typeof this.options.onclick == "function") ? this.options.onclick : function(){};
    
    this.options.onselect = (typeof this.options.onselect == "function") ? this.options.onselect : function(){};
    
    this.options.onsearch = (typeof this.options.onsearch == "function") ? this.options.onsearch : function(){ return true; };
	
	this.options.onreset = (typeof this.options.onreset == "function") ? this.options.onreset : function(){};
	
	this.options.autoselectFirstItem = (typeof this.options.autoselectFirstItem != "undefined") ? this.options.autoselectFirstItem : true;
	
	this.options.showEmptyResOnBlur = (typeof this.options.showEmptyResOnBlur != "undefined") ? this.options.showEmptyResOnBlur : false;
	this.options.getResultOnFocus = (typeof this.options.getResultOnFocus != "undefined") ? this.options.getResultOnFocus : true;
	
	this.options.initValue = (typeof this.options.initValue != "undefined") ? this.options.initValue : this.field.value;
    //this.options.parentNode = (typeof this.options.parentNode != "undefined") ? this.options.parentNode : document.body;
    this.options.parentNode = (0 && this.field.parentNode) ? this.field.parentNode : document.body; //forced parentNode to the body

	this.options.hideNoResultDiv = (typeof this.options.hideNoResultDiv != "undefined") ? this.options.hideNoResultDiv : false;
	//# gestione title #
	this.options.titleNoClick = (typeof this.options.titleNoClick != "undefined") ? this.options.titleNoClick : false;
	this.options.activeTitle = (typeof this.options.activeTitle != "undefined") ? this.options.activeTitle : false;
	this.options.titleClosed = false; //inizialmente false
	this.lastQsDigited = ''; //inizialmente vuoto
	//##################

	this.options.blockOnBlur = (typeof this.options.blockOnBlur != "undefined") ? this.options.blockOnBlur : false;


	this.options.inOverlay = (typeof this.options.inOverlay != "undefined") ? this.options.inOverlay : false;
	this.qs = this.default_value = this.field.value = this.options.initValue;

    this.options.nazione = (typeof this.options.nazione != "undefined") ? this.options.nazione : false;

	this.options.textEmptyRow = (typeof this.options.textEmptyRow != "undefined") ? this.options.textEmptyRow : "Nessun risultato trovato";

}

Im_Suggestion.prototype._attachEvents = function(){

    //addEvent(this.field,"keydown",this._chooseOperation.bind(this));
	addEvent(this.field,"keydown",this._onkeydown.bind(this));
	addEvent(this.field,"keyup",this._onkeyup.bind(this));
    
    //addEvent(this.field,"click",this._getSuggestion.bind(this));
    addEvent(this.field,"blur",this._onblur.bind(this));
    addEvent(this.field,"focus",this._onfocus.bind(this));
    addEvent(this.field,"mouseout",this._onmouseout.bind(this));
	addEvent(this.field,"mouseover",this._clearHideTimeout.bind(this));
    addEvent(document,"mousemove",function(){this.mouseOverEnabled = true}.bind(this));
    addEvent(this.suggestCont,"mouseover",this._clearHideTimeout.bind(this));
    
}

Im_Suggestion.prototype._onkeydown = function(ev){
	this._digiting = true;
	this.field.style.color = this.options.textColor;
	this._chooseOperation(ev);
}

Im_Suggestion.prototype._onkeyup = function(){
	this._digiting = false;
}


Im_Suggestion.prototype._onmouseout = function(){
	
	if (this.options.hideOnMouseOut){
		this._hideDelayed.apply(this,new Array("1000"));
		this._hideDelayed.apply(this,new Array("1000"));
	}
}

Im_Suggestion.prototype._addRow= function(index){
	
}

Im_Suggestion.prototype._blockOnBlur = function(flag){
	if (typeof flag == "undefined")
		return this._blockOnBlurEvent;
	return this._blockOnBlurEvent = flag;
}

Im_Suggestion.prototype._onblur = function(){
	
	if (this._blockOnBlur()){
		return;
	}

	this._digiting = false;
	
	if (this._selectedIndex > -1 && (this.qs.toLowerCase() == this.results[this._selectedIndex].row_value.toLowerCase())){
		this._selectRow(this.results[this._selectedIndex].domDiv);
	}
	
	if (!this.options.resetQsFieldOnNotSel && this.options.placeholderText && this.field.value.trim() == ""){
		this.setPlaceHolder();
	}
	
    this._hideDelayed.apply(this,new Array("100"));
}

Im_Suggestion.prototype.setPlaceHolder = function(){
	if (!this._digiting){
		this.field.value = this.options.placeholderText;
		this.field.style.color = this.options.placeholderColor;
	}
	
}

Im_Suggestion.prototype._onfocus = function(){
	this._clearHideTimeout.bind(this);
	this._blockOnBlur(false);
	this.field.style.color = this.options.textColor;
	if (this.field.value == this.options.placeholderText){		
		this.field.value = "";
	}
	if( this.options.getResultOnFocus ) {
		if (this.field.value){			
			this._getSuggestion();			
		}
	}		
    
}



Im_Suggestion.prototype._chooseOperation = function(ev){
    
    var keys = new Array();
    
    keys[13] = "RETURN";
    keys[9] = "TAB";
    keys[27] = "ESC";
    
    
    keys[112] = "F1";
    keys[113] = "F2";
    keys[114] = "F3";
    keys[115] = "F4";
    keys[116] = "F5";
    keys[117] = "F6";
    keys[118] = "F7";
    keys[119] = "F8";
    keys[120] = "F9";
    keys[121] = "F10";
    keys[122] = "F11";
    keys[123] = "F12";
    
    keys[145] = "SCROLLOCK";
    keys[20] =  "CAPSLOCK";
    keys[144] = "NUMLOCK";
    
    
    keys[16] = "SHIFT";
    keys[19] = "PAUSE";
    keys[18] = "ALT";
    keys[17] = "CTRL";
    
    keys[33] = "PAGEUP";
    keys[34] = "PAGEDOWN";
    keys[35] = "END";
    keys[36] = "HOME";
    
    keys[37] = "ARRSX";
    keys[38] = "ARRUP";
    keys[39] = "ARRDX";
    keys[40] = "ARRDW";
    
    keys[45] = "INS";
    
    //keys[0] = "WIN";
    keys[91] = "LEFTWIN";
    keys[92] = "RIGHTWIN";
    
    var key = (window.event) ? window.event.keyCode : ev.keyCode;
	var btn = keys[key];
    switch (btn){
        case "ARRDW":
			//caretPos = this.getCaretPosition(this.field);            
            if (this.status == "closed"){
                this._getSuggestion();
            }
            else
                this._nextRow();
			//this.setCaretPosition(this.field,caretPos);
            break;
        case "ARRSX":
        case "ARRDX":
        case "PAGEUP":
        case "PAGEDOWN":
        case "HOME":
        case "SHIFT":
        case "ALT":
        case "WIN":
        case "PAUSE":
        case "CTRL":
        case "END":
        case "INS":        
        case "F1":
        case "F2":
        case "F3":
        case "F4":
        case "F5":
        case "F6":
        case "F7":
        case "F8":
        case "F9":
        case "F10":
        case "F11":
        case "F12":
        case "SCROLLOCK":
        case "CAPSLOCK":
        case "NUMLOCK":
        case "LEFTWIN":
        case "RIGHTWIN":
            break;
        case "ARRUP":
			//caretPos = this.getCaretPosition(this.field);
            this._prevRow();
			//this.setCaretPosition(this.field,caretPos);
            break;
        case "ESC":
            this.hide(this.options.resetQsFieldOnNotSel);
            this.field.blur();
            break;
		case "TAB":			
        case "RETURN":
		
            if (this.status == "closed"){
                this._getSuggestion();
            }
            else{
				if (this.results[this._selectedIndex] && this.results[this._selectedIndex])
					this._selectRow(this.results[this._selectedIndex].domDiv,true);	
            }
			if (btn != "TAB")
				this.nextFocus();
            
            break;
        default:
            this._getSuggestion();
            break;
    }
	return true;
}

Im_Suggestion.prototype.nextFocus = function(){
	
	var nextField = null;
	
	if (this.options.focusedElementOnSelect === null)
		return;
	if (typeof this.options.focusedElementOnSelect != "undefined" && this.options.focusedElementOnSelect){
		nextField = this.options.focusedElementOnSelect;
	}else{		
		var parent = this.field.parentNode;
		
		while (parent && parent.tagName.toLowerCase() != "form"){				
			parent=parent.parentNode;		
		}
		
		var els = parent.elements;
		for(i=0;i<els.length;i++){
			if (els[i] == this.field)
				break;		
		}
		while (typeof els[i++] != "undefined")
		{
			if (els[i].type!="hidden"){
				nextField = els[i];
				break;
			}
		}
		
	}
	if (nextField){
		var fn = function(){try{nextField.focus();}catch(e){}};
		setTimeout(fn,100);
	}
	
}

Im_Suggestion.prototype._createComponents = function(){
    
    appendCss("/includes/Im_Suggestion.css");
    
    this.suggestCont = document.createElement("div");
    this.suggestCont.id = "Im_suggestion_container_"+this.idField;
	this.suggestCont.className = "Im_suggestion_container "+"Im_suggestion_container_"+this.idField; 
    
    this.suggestIframe = document.createElement("iframe");
    this.suggestIframe.id = "Im_suggestion_iframe_"+this.idField;
	this.suggestIframe.className = "Im_suggestion_iframe "+"Im_suggestion_iframe_"+this.idField; 
    this.suggestIframe.src = "";
    
    this.resultCont = document.createElement("div");
    this.resultCont.id = "Im_suggestion_results_"+this.idField;
	this.resultCont.className = "Im_suggestion_results "+"Im_suggestion_results_"+this.idField; 
    this.suggestCont.appendChild(this.resultCont);
    
    this.loaderCont = document.createElement("div");
    this.loaderCont.id = "Im_suggestion_loader_"+this.idField;
	this.loaderCont.className = "Im_suggestion_loader "+"Im_suggestion_loader_"+this.idField; 
    
    this.wrapper = document.createElement("div");
	this.wrapper.id = "Im_suggestion_wrapper_"+this.idField;
    this.wrapper.className = "Im_suggestion_wrapper "+"Im_suggestion_wrapper_"+this.idField; 
	
    this.wrapper.appendChild(this.suggestIframe);
    this.wrapper.appendChild(this.loaderCont);
    this.wrapper.appendChild(this.suggestCont);

    this.options.parentNode.appendChild(this.wrapper);
    if (this.field.value.trim() == "" && this.options.placeholderText)
		this.setPlaceHolder();
	if (this.field.value == this.options.placeholderText){
		this.field.style.color = this.options.placeholderColor;
	}
	
    this.setPosition(true);
    
}

Im_Suggestion.prototype.setPosition = function(hideSugg){

	if (typeof hideSugg == "undefined")
		hideSugg = false;
    //var pos = getPosition(this.field, true);
    var pos = getPosition(this.field);
    var fW = MEgetWidth(this.field);
    var fH = MEgetHeight(this.field);
    var lH = MEgetHeight(this.loaderCont);
    var lW = MEgetWidth(this.loaderCont);
    
	var position = this.options.inOverlay ? "fixed" : "absolute";

	if (hideSugg)
		this.suggestCont.style.display = "none";
    this.suggestCont.style.width = fW + "px";
    this.suggestCont.style.position = position;
    this.suggestCont.style.zIndex = "1000";
    this.suggestCont.style.top = parseInt(pos.y) + fH + 2 +"px";
    this.suggestCont.style.left = parseInt(pos.x) + "px";
    
    this.suggestIframe.style.backgroundColor = "#FFF";
    if (hideSugg)
		this.suggestIframe.style.display = "none";
    this.suggestIframe.style.width = fW + "px";
    this.suggestIframe.style.height = parseInt(MEgetHeight(this.suggestCont)) + "px";
    this.suggestIframe.style.position = position;
    this.suggestIframe.style.zIndex = "100";
    this.suggestIframe.style.top = parseInt(pos.y) + fH + 2 +"px";
    this.suggestIframe.style.left = parseInt(pos.x) + "px";
    this.suggestIframe.style.border="none";
    
	if (hideSugg)    
		this.loaderCont.style.display = "none";
    this.loaderCont.style.position = position;
    this.loaderCont.style.top = parseInt(pos.y) + fH/2 -lH/2 +"px";
    this.loaderCont.style.left = parseInt(pos.x) + fW - lW +"px";
}

Im_Suggestion.prototype.show = function(){
    this.setPosition();
    this.status = "opened";
	if((!this.results.length && this.options.hideNoResultDiv) ||
		(this.options.activeTitle && this.options.titleClosed)
		)
		this.suggestCont.style.display = "none";
	else
		this.suggestCont.style.display = "";
    this.suggestIframe.style.display = "";
	this.wrapper.style.display = "";
}

Im_Suggestion.prototype._clearHideTimeout = function(){
    if (this.hideTimer!="undefined")
        clearTimeout(this.hideTimer);
}


Im_Suggestion.prototype.setCaretPosition = function (oField, iCaretPos) {
	
    if (!iCaretPos)
        iCaretPos = this.field.value.length;
         
    if (document.selection) { // IE
    
       var oSel = document.selection.createRange ();
       oSel.moveStart ('character', -oField.value.length);
       oSel.moveStart ('character', iCaretPos);
       oSel.moveEnd ('character', 0);
       oSel.select();
    }
    else
    if (oField.selectionStart || oField.selectionStart == '0') {// Firefox & other
       oField.selectionStart = iCaretPos;
       oField.selectionEnd = iCaretPos;
       oField.focus();
    }
}


Im_Suggestion.prototype.getCaretPosition = function(oField) {

  // Initialize
  var iCaretPos = 0;

  // IE Support
  if (document.selection) {

    // Set focus on the element
    oField.focus ();

    // To get cursor position, get empty selection range
    var oSel = document.selection.createRange ();

    // Move selection start to 0 position
    oSel.moveStart ('character', -oField.value.length);

    // The caret position is selection length
    iCaretPos = oSel.text.length;
  }

  // Firefox support
  else if (oField.selectionStart || oField.selectionStart == '0')
    iCaretPos = oField.selectionStart;

  // Return results
  return (iCaretPos);
}

Im_Suggestion.prototype._hideDelayed = function(delay){
    this._clearHideTimeout();
    this.hideTimer = setTimeout(function(){this.hide(this.options.resetQsFieldOnNotSel);}.bind(this),delay);
}

Im_Suggestion.prototype.hide = function(resetQsField){
    this.suggestCont.style.display = "none";
    this.suggestIframe.style.display = "none";
	this.wrapper.style.display = "none";

    if (this._selectedIndex>-1)
        this._unSelectDivRow(this._selectedIndex);
    this._selectedIndex = -1;
    this.status = "closed";
    if (resetQsField)
        this.resetField();
	
	if (this.field.value.trim() == this.options.placeholderText && !this._digiting)
		this.setPlaceHolder();

}

Im_Suggestion.prototype.resetField = function(){
	
    if (this.default_value && this.status == "closed"){
        this.field.value = this.default_value;
        this.field.style.color = this.options.textColor;
    }
	
	if (this.default_value.trim() == "" && this.options.placeholderText)
		this.setPlaceHolder();
		
	if (typeof this.options.onreset != "undefined")
		this.options.onsearch(this.qs);
}

Im_Suggestion.prototype.reset = function(){
    this.default_value = "";
}

Im_Suggestion.prototype.initialize = function(){
	this.field.value = "";
	this.default_value = "";
	this.qs = "";
	if (this.options.placeholderText)
		this.setPlaceHolder();
}

Im_Suggestion.prototype._showLoader = function(){
    this.setPosition();
    this.loaderCont.style.display = "";
    this.fieldFocus();
}

Im_Suggestion.prototype._hideLoader = function(){
    this.loaderCont.style.display = "none";
}

Im_Suggestion.prototype._getSuggestion = function(force){
    if (typeof force=="undefined")
        force=false;
    clearTimeout(this.suggTimer);
    this._selectedIndex = -1;
	this._overSelectedIndex = -1;
    var minChars = this.options.minChars ? this.options.minChars : this.DEFAULT_MIN_CHARS;
	
    if (!force && ((this.field.value.length+1 < minChars) || this.checkIfSelected())){
        this.hide(false);
        this._hideLoader();
        return;
    }

    if (!this.options.onsearch( this.field.value ))
		return;
    this._showLoader();
    this.suggTimer = setTimeout(this._makeSuggestionReq.bind(this),this.keyupDelay);
}


Im_Suggestion.prototype.checkIfSelected = function(url){
    
    var selText = "";
    if (window.getSelection) {      // Firefox, Opera, Google Chrome and Safari
        if (document.activeElement && 
                (document.activeElement.tagName.toLowerCase () == "textarea" || 
                 document.activeElement.tagName.toLowerCase () == "input")) 
        {
            var text = document.activeElement.value;
            selText = text.substring (document.activeElement.selectionStart, 
                                      document.activeElement.selectionEnd);
        }
        else {
            var selRange = window.getSelection ();
            selText = selRange.toString ();
        }
    }
    else {
        if (document.selection.createRange) {       // Internet Explorer
            var range = document.selection.createRange ();
            selText = range.text;
        }
    }
    
    if (selText !== "")
        return true ;
    else
        return false;
}

Im_Suggestion.prototype.setUrl = function(url){
    this.options.url = url;
}
Im_Suggestion.prototype.getUrl = function(url){
    return this.options.url;
}

Im_Suggestion.prototype._makeSuggestionReq = function(){
    var aj = getMEAjaxObj();
    var url = this.options.url;
    
    this.qs = this.field.value;
    
    if (url.search(/\?/)==-1)
        url+="?";
    else
        url+="&";
    url+="s="+Url.encode(this.qs);
	if (this.options.maxResults)
		url+="&maxRes="+this.options.maxResults;
    if (this.options.nazione)
        url+="&n="+this.options.nazione;

    aj.Request("POST",url,this._manageSuggestionResults.bind(this));
}

Im_Suggestion.prototype._manageSuggestionResults = function(result){

	this.response = {};

	//con gestione titolo attiva
	if(this.options.activeTitle){
		if(this.options.titleClosed && (this.qs != this.lastQsDigited))
			this.options.titleClosed = false;
	}

	if(!jq.isEmpty(result.responseText))
   	 	eval("this.response = "+result.responseText);
	else
		this._emptyResults();

    if (this.response["qs"] != this.qs){
		this._hideLoader();
        return;
    }
	
    this.errors = this.response["errors"];
    this.results = this.all_results = this.response["results"];
    
        
    if (this.errors.length == 0 && this.results){
        var arrayLimit = 0;
        
        if (this.options.maxResults && this.options.maxResults < this.results.length){
            arrayLimit = this.options.maxResults;
        }else
            arrayLimit = this.results.length;
        
        this.results = new Array();
        for (var i=0; i<arrayLimit;i++){
            
            this.results[i] = this.response["results"][i];
        }
        
    }
    
    this._selectedIndex = -1;
    if (this.results)
        this._createResultDiv();
         
	
	if (this.field != document.activeElement && this.results.length > 0){
		this._selectedIndex = 0;
		
		if (this.qs.toLowerCase() == this.results[this._selectedIndex].row_value.toLowerCase()){
			
			this._selectRow(this.results[this._selectedIndex].domDiv);
	
		}
		return;
	}
	
	this._hideLoader();

	if (this.field == document.activeElement)
		this.show();
    //this.fieldFocus();

	if( this.options.autoselectFirstItem ) {
		this._nextRow();
		//this._selectRow( false, true );
	}

	//con gestione titolo attiva
	if(this.options.activeTitle){
		this.lastQsDigited = this.qs;
	}
}

Im_Suggestion.prototype._emptyResults = function(){
	this.wrapper.style.display = "none";
    this.suggestCont.innerHTML = "";
}

Im_Suggestion.prototype._setResultDivProps = function(){
	
    if (this.DEFAULT_MAX_RESULTS > 0)
        return;
	
    this.suggestCont.style.height="auto";
    this.suggestCont.style.overflowY="auto";
    this.suggestCont.style.visibility = "hidden";
    this.show();
    var cH = MEgetHeight(this.suggestCont);
    
    if (cH > this.options.maxHeight){
        this.suggestCont.style.height=this.options.maxHeight+"px";
        this.suggestCont.style.overflowY="scroll";
    }else{
        this.suggestCont.style.height="auto";
        this.suggestCont.style.overflowY="auto";
    }
    this.suggestCont.style.visibility = "visible";
    
	
}

Im_Suggestion.prototype._addEmptyRow = function () {
	var oRow = document.createElement("div");
	oRow.className = "Im_suggestion_row";
	oRow.innerHTML = this.options.textEmptyRow;
	this.suggestCont.appendChild(oRow);
}

Im_Suggestion.prototype._addRow= function(index){
    
    var row = this.results[index];
    var oRow = document.createElement("div");
    
    oRow.className = "Im_suggestion_row";
    oRow.innerHTML = row.row_label;
    
    oRow.suggIndex = index;
    addEvent(oRow,"mousedown",function(){this._blockOnBlur(true)}.bind(this));

    if (typeof this.options.onclick == "function")
        addEvent(oRow,"click",function(){this.options.onclick.apply(this,new Array(row))}.bind(this));
    addEvent(oRow,"click",function(e){this._clickRow.apply(this,new Array(oRow,e))}.bind(this));
    
    addEvent(oRow,"mouseover",function(){this._onRow.apply(this,new Array(row,oRow))}.bind(this));
    if (typeof this.options.onmouseover == "function")
        addEvent(oRow,"mouseover",function(){this.options.onmouseover.apply(this,new Array(row))}.bind(this));
    
    this.suggestCont.appendChild(oRow);
    return oRow;
}

Im_Suggestion.prototype._addTitleRow = function( title, index ){
	var oSuggestion = this;

	var oRow = document.createElement("div");

	oRow.className = "Im_suggestion_row title";
	oRow.innerHTML = title+" <span class='close'>&times;</span>";
	oRow.suggIndex = index;

	jq(oRow).attr('data-type','title'); //!important
	addEvent(oRow,"click",function(e){this._clickRow.apply(this,new Array(oRow,e))}.bind(this));

	if (typeof this.options.onmouseover == "function")
		addEvent(oRow,"mouseover",function(){this.options.onmouseover.apply(this,new Array(oRow))}.bind(this));

	this.suggestCont.appendChild(oRow);
	return oRow;
}

Im_Suggestion.prototype._createResultDiv = function(){

    this._emptyResults();

    if (this.results.length == 0){
		
        if ((this.field == document.activeElement || this.options.showEmptyResOnBlur) && !this.options.hideNoResultDiv){
			this._addEmptyRow();
        }
    }
    else{

		if(this.options.activeTitle){
			if(!this.options.titleClosed && !jq.isEmpty(this.options.titleNoClick)){
				var _i = -1;
				var length = this.results.length;

				this._addTitleRow( this.options.titleNoClick, _i );
				_i = 1;
				length++;

				for (var i = _i; i < length; i++){
					this.results[i-1].domDiv = this._addRow(i-1);
				}
			}
		}else{
			for (var i = 0; i < this.results.length; i++){
				this.results[i].domDiv = this._addRow(i);
			}
		}

		this._setResultDivProps();
	}
    
}

Im_Suggestion.prototype._selectDivRow = function(oDiv){
    
    oDiv.className = "Im_suggestion_row_selected";
    var suggIndex = oDiv.suggIndex;
    oDiv.onmouseout = function(){
        this.className= "Im_suggestion_row";
    }
    this.fieldFocus();
}
    
Im_Suggestion.prototype.fieldFocus = function(){
    this.field.focus();
    //this.setCaretPosition(this.field);
}

Im_Suggestion.prototype._unSelectDivRow = function(index){
    if (index > -1)
        if (this.results && this.results.length && this.results[index])
            this.results[index].domDiv.className = "Im_suggestion_row";
}

Im_Suggestion.prototype._selectRow = function(oRow,closeSearch,automatic){
	
    if (oRow.suggIndex<0)
        return;
	
    var item = this.results[oRow.suggIndex];
    
	if( typeof automatic == 'undefined' )
		automatic = false;
	
    if ( !automatic && item.row_value)
        this.field.value = item.row_value;
    
    this.qs = this.field.value;
    this.default_value = this.qs;
    
    this.options.onselect(item);
    if (closeSearch)
        this.hide();
}

Im_Suggestion.prototype._clickRow = function(oRow,ev){

    this._selectedIndex = oRow.suggIndex;
	this._selectRow(oRow);
    this.hide();
	this.nextFocus();

	/* ONLY FOR ROW SUGGESTION TITLE */
	if(this.options.activeTitle){
		var _target = (!jq.isEmpty(ev.target)) ? ev.target.className : (!jq.isEmpty(ev.srcElement) ? ev.srcElement.className : '');
		var actionSuggMail = 'accepted';
		if((jq(oRow).attr('data-type') == 'title') && (_target == 'close')){  //pulsante di chiusura
			this.options.titleClosed = true;
			actionSuggMail = 'rejected';
		}
		if ( typeof trackGAClickEvent != 'undefined' )
			trackGAClickEvent('suggerimentoEmailEvents', 'suggerimento_dominio_email', actionSuggMail);
	}

    try{
        if (ev){
            ev.stopBubbling();
        }
    }catch(e){
        window.event.cancelBubble = true;
    }

}

Im_Suggestion.prototype._onRow = function(item,divRow){
    if (!this.mouseOverEnabled)
        return;

	this._unSelectDivRow(this._selectedIndex);	
	this._selectedIndex = -1;
	this._overSelectedIndex = divRow.suggIndex;
	if (this.options.selectRowOnOver)
        this._selectRow(divRow.suggIndex);
    
    
    this._selectDivRow(divRow);
}

Im_Suggestion.prototype._nextRow = function(){

    if (!this.results || this.results.length==0)
        return;
    this.mouseOverEnabled = false;
    if (this._selectedIndex >= this.results.length-1)
        return;
    else
        if (this._selectedIndex > -1)
            this._unSelectDivRow(this._selectedIndex);
    this.show();

    var index = ++this._selectedIndex;
    var item = this.results[index];
    
    var divRow = this.results[index].domDiv;

    if (this.options.selectRowOnOver)
        this._selectRow(divRow);

    this._selectDivRow(divRow);
}

Im_Suggestion.prototype._prevRow = function(){
    
    if (!this.results || this.results.length==0)
        return;
    this.mouseOverEnabled = false;
    if (this._selectedIndex <= 0){
        this.hide();
        return;
    }
    else
        this._unSelectDivRow(this._selectedIndex);
    
    this.show();
    
    var index = --this._selectedIndex;
    var item = this.results[index];
    
    var divRow = this.results[index].domDiv;

    if (this.options.selectRowOnOver)
        this._selectRow(divRow);
    
    this._selectDivRow(divRow);
}
;
/**
 * @name MarkerWithLabelOptions
 * @class This class represents the optional parameter passed to the {@link MarkerWithLabel} constructor.
 *  The properties available are the same as for <code>google.maps.Marker</code> with the addition
 *  of the properties listed below. To change any of these additional properties after the labeled
 *  marker has been created, call <code>google.maps.Marker.set(propertyName, propertyValue)</code>.
 *  <p>
 *  When any of these properties changes, a property changed event is fired. The names of these
 *  events are derived from the name of the property and are of the form <code>propertyname_changed</code>.
 *  For example, if the content of the label changes, a <code>labelcontent_changed</code> event
 *  is fired.
 *  <p>
 * @property {string|Node} [labelContent] The content of the label (plain text or an HTML DOM node).
 * @property {Point} [labelAnchor] By default, a label is drawn with its anchor point at (0,0) so
 *  that its top left corner is positioned at the anchor point of the associated marker. Use this
 *  property to change the anchor point of the label. For example, to center a 50px-wide label
 *  beneath a marker, specify a <code>labelAnchor</code> of <code>google.maps.Point(25, 0)</code>.
 *  (Note: x-values increase to the right and y-values increase to the top.)
 * @property {string} [labelClass] The name of the CSS class defining the styles for the label.
 *  Note that style values for <code>position</code>, <code>overflow</code>, <code>top</code>,
 *  <code>left</code>, <code>zIndex</code>, <code>display</code>, <code>marginLeft</code>, and
 *  <code>marginTop</code> are ignored; these styles are for internal use only.
 * @property {Object} [labelStyle] An object literal whose properties define specific CSS
 *  style values to be applied to the label. Style values defined here override those that may
 *  be defined in the <code>labelClass</code> style sheet. If this property is changed after the
 *  label has been created, all previously set styles (except those defined in the style sheet)
 *  are removed from the label before the new style values are applied.
 *  Note that style values for <code>position</code>, <code>overflow</code>, <code>top</code>,
 *  <code>left</code>, <code>zIndex</code>, <code>display</code>, <code>marginLeft</code>, and
 *  <code>marginTop</code> are ignored; these styles are for internal use only.
 * @property {boolean} [labelInBackground] A flag indicating whether a label that overlaps its
 *  associated marker should appear in the background (i.e., in a plane below the marker).
 *  The default is <code>false</code>, which causes the label to appear in the foreground.
 * @property {boolean} [labelVisible] A flag indicating whether the label is to be visible.
 *  The default is <code>true</code>. Note that even if <code>labelVisible</code> is
 *  <code>true</code>, the label will <i>not</i> be visible unless the associated marker is also
 *  visible (i.e., unless the marker's <code>visible</code> property is <code>true</code>).
 */
/**
 * Creates a MarkerWithLabel with the options specified in {@link MarkerWithLabelOptions}.
 * @constructor
 * @param {MarkerWithLabelOptions} [opt_options] The optional parameters.
 */


MarkerWithLabel.prototype = new google.maps.Marker();

function MarkerWithLabel(opt_options) {
	opt_options = opt_options || {};
	opt_options.labelContent = opt_options.labelContent || "";
	opt_options.labelAnchor = opt_options.labelAnchor || new google.maps.Point(0, 0);
	opt_options.labelID = opt_options.labelID || "";
	opt_options.labelClass = opt_options.labelClass || "markerLabels";
	opt_options.labelStyle = opt_options.labelStyle || {};
	opt_options.labelInBackground = opt_options.labelInBackground || false;
	opt_options.goTo = opt_options.goTo || null;

	opt_options.isMobile = opt_options.isMobile || false;
	if (typeof opt_options.labelVisible === "undefined") {
		opt_options.labelVisible = true;
	}
	if (typeof opt_options.clickable === "undefined") {
		opt_options.clickable = true;
	}
	this.label = new MarkerLabel(this); // Bind the label to the marker
	this.details = new Array();
	this.ifexist = new Object();
	// Call the parent constructor. It calls Marker.setValues to initialize, so all
	// the new parameters are conveniently saved and can be accessed with get/set.
	// Marker.set triggers a property changed event (called "propertyname_changed")
	// that the marker label listens for in order to react to state changes.
	google.maps.Marker.apply(this, arguments);
}

/**
 * Overrides the standard Marker setMap function.
 * @param {Map} theMap The map to which the marker is to be added.
 * @private
 */
MarkerWithLabel.prototype.setMap = function (theMap) {
	google.maps.Marker.prototype.setMap.apply(this, arguments);
	this.label.setMap(theMap);
	if ( !theMap ) {
		this.details = null;
		this.ifexist = null;
		this.label = null;
	}
};



/**
 * This constructor creates a label and associates it with a marker.
 * It is for the private use of the MarkerWithLabel class.
 * @constructor
 * @param {Marker} marker The marker with which the label is to be associated.
 * @private
 */

MarkerLabel.prototype = new google.maps.OverlayView();

function MarkerLabel( marker ) {
	this.marker_ = marker || {};
	this.listeners_ = null;
	this.labelDiv_ = document.createElement("div");
	this.labelDiv_.className = 'cluster-marker';
	this.labelDiv_.style.cssText = "position: absolute; overflow: hidden; display:none;";
}

/**
 * Adds the DIV representing the label to the DOM. This method is called
 * automatically when the marker's <code>setMap</code> method is called.
 * @private
 */
MarkerLabel.prototype.onAdd = function() {
	var that = this;
	var cIgnoreClick;
	// Stops all processing of an event.
	var cAbortEvent = function (e) {
		if ( e.preventDefault ) {
			e.preventDefault();
		}
		e.cancelBubble = true;
		if ( e.stopPropagation ) {
			e.stopPropagation();
		}
	};
	this.getPanes().overlayImage.appendChild( this.labelDiv_ );
	this.listeners_ = [
		google.maps.event.addListener(this.marker_, "visible_changed", function () {
			that.setVisible();
		}),
		google.maps.event.addListener(this.marker_, "labelclass_changed", function () {
			that.setStyles();
		}),
		google.maps.event.addListener(this.marker_, "labelcontent_changed", function () {
			that.setContent();
		}),
		google.maps.event.addDomListener( this.labelDiv_, "click", function (e) {
			if (that.marker_.getClickable()) {
				if ( cIgnoreClick ) { // Ignore the click reported when a label drag ends
					cIgnoreClick = false;
				} else {
					google.maps.event.trigger(that.marker_, "click", e);
					cAbortEvent( e ); // Prevent click from being passed on to map
				}
			}
		}),
		google.maps.event.addDomListener( this.labelDiv_, "dblclick", function (e) {
			if (that.marker_.getClickable()) {
				google.maps.event.trigger(that.marker_, "dblclick", e);
				cAbortEvent( e ); // Prevent map zoom when double-clicking on a label
			}
		})
	];
}

/**
 * Draws the label on the map.
 * @private
 */
MarkerLabel.prototype.draw = function() {
	this.setContent();
	this.setStyles();
}

/**
 * Removes the DIV for the label from the DOM. It also removes all event handlers.
 * This method is called automatically when the marker's <code>setMap(null)</code>
 * method is called.
 * @private
 */
MarkerLabel.prototype.onRemove = function() {
	this.labelDiv_.parentNode.removeChild( this.labelDiv_ );
	// Remove event listeners:
	for (var i = 0; i < this.listeners_.length; i++) {
		google.maps.event.removeListener(this.listeners_[i]);
		this.listeners_[i] = null;
	}
	this.listeners_ = null;
	this.marker_ = null;
	this.labelDiv_ = null;
}

/**
 * Sets the content of the label.
 * The content can be plain text or an HTML DOM node.
 * @private
 */
MarkerLabel.prototype.setContent = function () {
	var content = this.marker_.get("labelContent");
	if (typeof content.nodeType === "undefined") {
		this.labelDiv_.innerHTML = content;
	} else {
		this.labelDiv_.innerHTML = ""; // Remove current content
		this.labelDiv_.appendChild(content);
	}
}

/**
 * Sets the position of the label.
 * @private
 */
MarkerLabel.prototype.setPosition = function (yOffset) {
	var position = this.getProjection().fromLatLngToDivPixel( this.marker_.getPosition() );
	if ( typeof yOffset === "undefined" ) {
		yOffset = 0;
	}
	this.labelDiv_.style.left = Math.round(position.x) + "px";
	this.labelDiv_.style.top = Math.round(position.y - yOffset) + "px";
	this.setZIndex();
}

/**
 * Sets the position of the label.
 * @private
 */
MarkerLabel.prototype.goTo = function (_position, yOffset) {
	var position = this.getProjection().fromLatLngToDivPixel( _position );
	if ( typeof yOffset === "undefined" ) {
		yOffset = 0;
	}
	var that = this;
    var max = 801,
        min = 200,
        random =  Math.random() * (max - min) + min;

    jq(this.labelDiv_).animate(
        {left: Math.round(position.x), top: Math.round(position.y - yOffset)},
        random,
        function () {
            if (that.marker_) {
                that.marker_.setPosition(_position);
            }
        }
    );
	this.setZIndex();
}

/**
 * Sets the visibility of the label. The label is visible only if the marker itself is
 * visible (i.e., its visible property is true) and the labelVisible property is true.
 * @private
 */
MarkerLabel.prototype.setVisible = function () {
	if (this.marker_.isMobile){
		this.labelDiv_.style.display = this.marker_.getVisible() ? "block" : "none";
		return;
	}
	if ( this.marker_.getVisible() )
		jq( this.labelDiv_).fadeIn();
	else
		jq( this.labelDiv_).fadeOut();
}

/**
 * Sets the style of the label by setting the style sheet and applying
 * other specific styles requested.
 * @private
 */
MarkerLabel.prototype.setStyles = function () {
	var i, labelStyle, labelID;
	// Apply style values from the style sheet defined in the labelClass parameter:
	this.labelDiv_.className = this.marker_.get("labelClass");
	labelID = this.marker_.get("labelID");
	if ( labelID != '' )
		this.labelDiv_.id = labelID;
	// Clear existing inline style values:
	this.labelDiv_.style.cssText = "";
	// Apply style values defined in the labelStyle parameter:
	labelStyle = this.marker_.get("labelStyle");
	for (i in labelStyle) {
		if (typeof labelStyle[i] != 'undefined') {
			this.labelDiv_.style[i] = labelStyle[i];
		}
	}
	this.setMandatoryStyles();
};

/**
 * Sets the mandatory styles to the DIV representing the label.
 * This includes setting the DIV position, z-index, and visibility.
 * @private
 */
MarkerLabel.prototype.setMandatoryStyles = function () {
	this.labelDiv_.style.position = "absolute";
	this.labelDiv_.style.overflow = "hidden";

	if (!this.marker_.isMobile)
		this.labelDiv_.style.display = "none";
	// Make sure the opacity setting causes the desired effect on MSIE:
	if (typeof this.labelDiv_.style.opacity !== "undefined" && this.labelDiv_.style.opacity !== "") {
		this.labelDiv_.style.MsFilter = "\"progid:DXImageTransform.Microsoft.Alpha(opacity=" + (this.labelDiv_.style.opacity * 100) + ")\"";
		this.labelDiv_.style.filter = "alpha(opacity=" + (this.labelDiv_.style.opacity * 100) + ")";
	}
	this.setAnchor();
	this.setPosition(); // This also updates z-index, if necessary.
	this.setVisible();
}

/**
 * Sets the anchor point of the label.
 * @private
 */
MarkerLabel.prototype.setAnchor = function () {
	var anchor = this.marker_.get("labelAnchor");
	this.labelDiv_.style.marginLeft = -anchor.x + "px";
	this.labelDiv_.style.marginTop = -anchor.y + "px";
}

/**
 * Sets the z-index of the label. If the marker's z-index property has not been defined, the z-index
 * of the label is set to the vertical coordinate of the label. This is in keeping with the default
 * stacking order for Google Maps: markers to the south are in front of markers to the north.
 * @private
 */
MarkerLabel.prototype.setZIndex = function () {
	var zAdjust = (this.marker_.get("labelInBackground") ? -1 : +1);
	if (typeof this.marker_.getZIndex() === "undefined") {
		this.labelDiv_.style.zIndex = Math.abs( parseInt( parseInt(this.labelDiv_.style.top, 10) / 10, 10 ) + zAdjust );
	} else {
		this.labelDiv_.style.zIndex = Math.abs( this.marker_.getZIndex() + zAdjust );
	}
}
;
'use strict';

var BoxLabel = (function ($) {

    var BoxLabel = function (map, opt) {
        this.map_ = map;
        this.id = opt.id || '#infowindow-frame';
        this.className = opt.className || "";
        this.content = opt.content || "";
        this.position = opt.position || {};
        this.offset = opt.offset || {};
        this.events = opt.events || {};
        this.idContent = opt.idContent || '#infowindow-content';
        this.idArrow = '#infowindow-arrow';
        this.idAnchor = null;
        this.visible = false;
        this._div = $('<div id="infowindow-frame" class="infowindow-frame">' +
        '<div id="infowindow-content"></div>' +
        '<span id="infowindow-arrow"></span>' +
        '</div>');
        this.setMap(map);
    };


    /**
     * Box overlay personalizzato per la visualizzazione della lista o del dettaglio.
     * @type {google.maps.OverlayView}
     */
    BoxLabel.prototype = new google.maps.OverlayView();


    /**
     * Eventi eseguiti quando si aggiunge un overlay alla mappa
     */
    BoxLabel.prototype.onAdd = function () {
        this.getPanes().overlayImage.appendChild(this._div[0]);

        if (this.events.onAdd && typeof this.events.onAdd === 'function') {
            this.events.onAdd();
        }
    };


    /**
     * Eventi eseguiti quando si rimuove un layer
     */
    BoxLabel.prototype.onRemove = function () {
        this._div.parentNode.removeChild(this._div[0]);
    };


    /**
     * Eventi eseguiti al momento dell'inclucisione dell'html
     */
    BoxLabel.prototype.draw = function () {
        if (this.events.onDraw && typeof this.events.onDraw === 'function') {
            this.events.onDraw();
        }
    };


    /**
     * Mostra il box
     */
    BoxLabel.prototype.show = function () {
        if (this.setPosition()) {
            $(this.id).show();
            this.visible = true;
        } else {
            this.hide();
        }
    };


    /**
     * Nasconde il box
     */
    BoxLabel.prototype.hide = function () {
        $(this.id).hide();
        this.visible = false;

        if (this.events.onClose && typeof this.events.onClose === 'function') {
            this.events.onClose();
        }
    };


    /**
     * Controlla se il box è visibile o meno
     * @returns {boolean}
     */
    BoxLabel.prototype.isVisible = function () {
        return this.visible;
    };


    /**
     * Aggiunte il contenuto al box e lo riposiziona
     * @param content
     */
    BoxLabel.prototype.setContent = function (content) {
        $(this.idContent).html(content);
        this.draw();
        this.setPosition();
    };


    /**
     * Gestisce il posizionamento del box rispetto a un'ancora
     */
    BoxLabel.prototype.setPosition = function () {
        var position, offset, offsetMarker, halfWidthArrow, halfHeightArrow, top, left, offsetLeftArrow, offsetTop,
            wBoxMappa, hBoxMappa, hBoxContent, offsetContent;

        var $arrow = $(this.idArrow),
            $anchor = $(this.idAnchor),
            $content = $(this.idContent),
            $boxMapAds = $('#box-map-ads');

        // dati anchor
        position = $anchor.position();
        offset = $anchor.offset();
        offsetMarker = $anchor.outerWidth() / 2;
        halfWidthArrow = 12.5;
        halfHeightArrow = 20;

        // calcolo posizione
        if (typeof offset === 'undefined' || typeof position === 'undefined') {
            return false;
        }

        offsetTop = offset.top - $('#header').outerHeight(true);
        wBoxMappa = $boxMapAds.width();
        hBoxMappa = $boxMapAds.height();
        hBoxContent = $content.outerHeight(true);

        // logica top
        offsetContent = offsetTop * hBoxContent / hBoxMappa;

        if (offsetContent <= 30 + halfHeightArrow) {
            offsetContent = 30 + halfHeightArrow;
        } else if ((offsetContent + halfHeightArrow) >= hBoxContent) {
            offsetContent = hBoxContent - halfHeightArrow;
        }

        top = position.top - offsetContent;

        // logica left
        if ((offset.left - $('#box-listing').outerWidth(true)) > (wBoxMappa / 2)) {
            left = parseFloat(position.left) - $content.outerWidth(true) - (offsetMarker + 2 * halfWidthArrow);
            offsetLeftArrow = -(offsetMarker + 2 * halfWidthArrow) - 1;
            $arrow.addClass('arrow-dx').removeClass('arrow-sx');
        } else {
            left = parseFloat(position.left) + (offsetMarker + 2 * halfWidthArrow);
            offsetLeftArrow = offsetMarker + 1;
            $arrow.addClass('arrow-sx').removeClass('arrow-dx');
        }

        $arrow.css({top: position.top - halfHeightArrow, left: position.left + offsetLeftArrow});
        $content.css({top: top, left: left});

        return true;
    };

    return BoxLabel;
})(jQuery);


;
'use strict';

var ManagerHash = (function ($) {

    /**
     * Oggetto che si occupa della gestione dell'hash della url
     * @param opt
     * @constructor
     */
    function ManagerHash(opt) {
        this.opt = opt || {};
        this.startDelimiter = typeof this.opt.startDelimiter !== 'undefined' ? this.opt.startDelimiter : '#';
        this.delimiterK = typeof this.opt.delimiterK !== 'undefined' ? this.opt.delimiterK : '/';
        this.delimiterV = typeof this.opt.delimiterV !== 'undefined' ? this.opt.delimiterV : '_';
        this.delimiterM = typeof this.opt.delimiterM !== 'undefined' ? this.opt.delimiterM : ',';
        this.hashObj = {};
        this.pathName = window.location.pathname;
        /**
         * Gestione della hash
         */
        this.idCheckHashValue = 'checkHashValue';
        this.onBack = typeof this.opt.onBack === 'function' ? this.opt.onBack : function() {};
        this.onForward = typeof this.opt.onForward === 'function' ? this.opt.onForward : function() {};
        this.onPopState = typeof this.opt.onPopState === 'function' ? this.opt.onPopState : function() {};
    };

    /**
     * Setto uno o piu' parametri all'hash
     * Accept key and value or object of keys and values
     * @param k - key string or object with keys:values
     * @param v - value
     * @param pushOrReplace - boolean true => pushState, false => replaceState
     */
    ManagerHash.prototype.setHash = function (k, v, pushOrReplace) {
        this.convertHashStrToObj();

        if (k !== null) {
            if (typeof k === 'object') {
                for (var i in k) {
                    this.hashObj[i] = k[i];
                }
            } else {
                this.hashObj[k] = v;
            }
        }

        var hash = this.convertHashObjToStr();

        // serve per controllare se l'hash cambia per un'azione dell'utente sulla mappa o un click sulla history.
        if (typeof window.history.pushState !== 'undefined' && typeof window.history.replaceState !== 'undefined') {
            if (pushOrReplace) {
                this.pushState(hash);
            } else {
                this.replaceState(hash);
            }
        } else {
            $('#' + this.idCheckHashValue).val('#' + hash);
            window.location.hash = hash;
        }
    };

    /**
     * Torna il valore di una singola chiave se presente nell'hash
     * @param k
     * @returns {*}
     */
    ManagerHash.prototype.getHash = function (k) {
        this.convertHashStrToObj();

        return typeof this.hashObj[k] !== 'undefined' ? this.hashObj[k] : null;
    };

    /**
     * Data una chiave, cerca se e' presenta nella hash.
     * @param k
     * @returns {boolean}
     */
    ManagerHash.prototype.hasHash = function (k) {
        this.convertHashStrToObj();

        return typeof this.hashObj[k] !== 'undefined';
    };

    /**
     *
     * @param hash
     * @param state
     * @param title
     */
    ManagerHash.prototype.pushState = function (hash, state, title) {
        if (typeof window.history.pushState !== 'function') {
            return;
        }

        state = typeof state !== 'undefined' ? state : null;
        title = typeof title !== 'undefined' ? title : null;
        window.history.pushState(state, title, this.pathName + '#' + hash);
    };

    /**
     *
     * @param hash
     * @param state
     * @param title
     */
    ManagerHash.prototype.replaceState = function (hash, state, title) {
        if (typeof window.history.replaceState !== 'function') {
            return;
        }

        state = typeof state != 'undefined' ? state : null;
        title = typeof title != 'undefined' ? title : null;
        window.history.replaceState(state, title, this.pathName + '#' + hash);
    };

    /**
     * Converte la stringa hash in un oggetto
     */
    ManagerHash.prototype.convertHashStrToObj = function () {
        var _this = this;
        this.hashObj = {};

        $.each(window.location.hash.replace(this.startDelimiter, '').split(this.delimiterK), function (i, v) {
            v = v.split(_this.delimiterV);

            if (typeof v[1] !== 'undefined') {
                var key = v.shift();
                // nel caso in cui un il valore di un parametro hash (chiave+delimitatore+valore)
                // contenga il delimitatore stesso, allora considero che il primo elemento sia la chiave
                // e tutto il resto il valore.
                _this.hashObj[key] = v.join(_this.delimiterV);
            }
        });
    };

    /**
     * Converte l'oggetto hash in stringa e lo ritorna
     * @returns {string}
     */
    ManagerHash.prototype.convertHashObjToStr = function () {
        var str = '';

        for (var key in this.hashObj) {
            if (!this.hashObj[key] || this.hashObj[key] === '') {
                continue;
            }

            str += this.delimiterK + key + this.delimiterV + this.hashObj[key];
        }

        return str;
    };

    /**
     * Dato l'oggetto RefineSearch restituisco un oggetto chiave valore.
     * @param refineSearch
     * @returns {{}}
     */
    ManagerHash.prototype.getObjFromRefineSearch = function (refineSearch) {
        var obj, elements, oSerialize, name;

        obj = {};
        // mi prendo tutti gli elementi del form.
        elements = refineSearch.getFormElements();
        //serializzo il form in un oggetto CHIAVE:VALORE
        oSerialize = refineSearch.serializeObj();

        for (var k in elements) {
            name = elements[k];

            //se l'elemento ha la classe no-hash non lo devo aggiungere alla hash dell'url
            if ($(refineSearch.opt.form + ' input[name=' + name + ']').hasClass('no-hash')) {
                continue;
            }

            obj[name] = typeof oSerialize[name] != 'undefined' ? oSerialize[name] : null;
        }

        return obj;
    };

    return ManagerHash;
})(jQuery);

;
/**
 * Classe che gestisce tutti gli eventi del box raffina ricerca.
 * @param opt
 * @constructor
 */
function RefineSearch(opt) {
	this.opt = opt || {};
	this.searchType = opt.searchType || 'annuncio';
	this.lastSerialize = null;
	this.timeoutTrigger = null;
	this.asyncAjax = true;
	this.autoSubmitForm = false;
	this.formElements = null;
	//se vuoto significa che vengo dal listing e quindi non posso effettuare la modifica della categoria
	this.changeCategoryEnabled = jq.isEmpty(jq("#enabledInputSearch").val()) ? false : true;
	//bool utilizzato per capire se e' abilitato o meno la modifica del contratto
	this.changeStatusContrattoEnabled = true;
	//istanzio il gestore della hash
	this.managerHash = new ManagerHash();
	//elemento jquery dell'hidden verticale
	this.verticaleElm = jq("#verticale");

	/**
	 * Funzione che inizializza gli eventi al DOM
	 */
	this.init = function () {
		var that = this;

		// stop the enter click and submit form
		jq(this.opt.form).on('keypress', function (e) {
			if (e.keyCode == 13) {
				e.preventDefault();
			}
		});

		// trigger form change
		jq(this.opt.form + ' input, ' + this.opt.form + ' select').on('change', function () {

			var element = jq(this);
			var elementId = jq(this).attr('id');

			//se e' una select - single
			if (element.prop('type') == 'select-one') {

				if (!jq.isEmpty(element.data("ajax")) && element.data("ajax") == 1) {
					//Esegue la chiamata ajax di popolamento select ajax
					that.onChangeSelectAjax(element);
				}

				if(elementId == 'selectCategoria'){
					//Gestione dei casi speciali alla modifica della select Categoria
					that.onChangeSelectCategoria(element);
				} else if(elementId == 'selectTipologia'){
					//Gestione dei casi speciali alla modifica della select Tipologia
					that.onChangeSelectTipologia(element);
				} else if(elementId == 'selectSottotipologia'){
					//Gestione dei casi speciali alla modifica della select Sottotipologia
					that.onChangeSelectSottotipologia(element);
				} else if(elementId == 'selectTipologiaAste'){
					//Gestione dei casi speciali alla modifica della select TipologiaAste
					that.onChangeSelectTipologiaAste(element);
				} else if(elementId == 'selectTipoProprietaFake'){
					//Gestione dei casi speciali alla modifica della select TipoProprietaFake
					that.onChangeSelectTipoProprietaFake(element);
				}
			}

            //se e' una select - multipla
			if(element.prop('type') == 'select-multiple'){
                if(elementId == 'selectFasciaPiano'){
                    //Gestione dei casi speciali alla modifica della select Fascia Piano
                    //TODO: da gestire all'OCCORRENZA select multipla ricerca mappa fascia piano
                }
            }

			if (!jq.isEmpty(element.data('onchange'))) {
				//Accende/spegne tutti gli elementi che dipendono da esso.
				that.onChange(element);
			}

			//In base al numero di colonne o elementi del form visibili aggiungo o rimuove la classe 'two-columns'
			that.checkFormClassName();

			if ( !that.autoSubmitForm ) {
				return true;
			}
			if ( that.lastSerialize == that.serialize() ) {
				return true;
			}
			clearTimeout(that.timeoutTrigger);
			that.timeoutTrigger = setTimeout(function () {
				that.lastSerialize = that.serialize()
				managerMarkers.resetParamsToReloadMap(true);
			}, 300);
			return true;
		});

		// manage Styled Select
		jq(this.opt.form + ' .styled-select select')
			.on('change', function () {
				that.changeSelectLabel(this);
			})
			.on('focus', function () {
				jq(this).parent().addClass('focus');
			})
			.on('blur', function () {
				jq(this).parent().removeClass('focus');
			});

		// simulate label click
		jq(this.opt.form + ' .pure-input-label').on('click', function () {
			jq(jq(this).attr('data-for')).focus();
		});

		jq("label.pure-checkbox input[type='checkbox']").on('change', function() {
			jq(this).closest("label").toggleClass("checked", jq(this).prop('checked'));
		});

		// active button
		jq(this.opt.form + ' .btn-action').on('click', function (e) {
			e.preventDefault();
			if(jq(this).hasClass('disabled')) {
				jq(this).blur();
				return;
			}

			if (jq(this).hasClass('submit')) {
				that.submit();
				return;
			}
			if (!jq(this).hasClass('active')) {
				jq(this).parent().find('.active').removeClass('active');
				jq(this).addClass('active');
			} else {
				jq(this).removeClass('active');
			}
			if (typeof jq(this).attr('data-input') != 'undefined') {
				if (jq(this).hasClass('active'))
					jq(jq(this).attr('data-input')).val(jq(this).attr('data-value')).change();
				else
					jq(jq(this).attr('data-input')).val('').change();
			}
		});

		//si occupa di accendere e spegnere i bottoni inerenti all'elemento input
		jq(this.opt.form + ' input.btn-hidden-data-input').on('change', function () {
			var hiddenId = jq(this).attr('id');
			var hiddenVal = jq(this).val();
			jq(that.opt.form + ' .btn-action[data-input="#' + hiddenId + '"]').each(function (index) {
				jq(this).removeClass('active');
				if (jq(this).attr('data-value') == hiddenVal) {
					jq(this).addClass('active');
				}
			});
		});

		// Al click sulla label Lista torno alla pagina di ricerca standard
		jq('#btn-go-list').on('click', function () {
			that.goToStandardSearch();
		});

		//gestisco i casi specifici su eventuali elementi del form
		this.managementSpecialCases();

		//Si occupa di inizializzare la gestione dei casi speciali dei verticali
		this.initVerticale();

		//popolo il form in base all'hash nella url
		this.fillByHash();

		//si occupa della gestione se mostrare o meno la select Categoria
		this.categoryManager();
	};

	/**
	 * Esegue la chiamata ajax di popolamento select ajax degli eventuali figli
	 * @param selectElement
	 */
	this.onChangeSelectAjax = function (selectElement) {
		var element = jq(selectElement);
		if (element.attr('id') == 'selectCategoria' && element.val() == 12) {
			return;
		}
		var childs = element.data("child").split(" ");
		if (!jq.isEmpty(childs)) {
			for (var i in childs) {
				this.getResponseAjax(jq(childs[i]), element);
			}
		}
	};

	/**
	 * Metodo che si occupa di gestire tutti i casi speciali alla modifica della select Categoria
	 * @param selectCategoria
	 */
	this.onChangeSelectCategoria = function (selectCategoria) {
		var that = this;
		that.changeStatusBtnContratto(false);

		if(selectCategoria.val() == 6 || selectCategoria.val() == 12){
			//nuove costruzioni e aste
			jq('#inputContratto').val(1).change();
			that.changeStatusBtnContratto(true);
		} else if(selectCategoria.val() == 4){
			//stanze
			jq('#inputContratto').val(2).change();
			that.changeStatusBtnContratto(true);
		}
		this.changeStatusContrattoEnabled = false;

		if (that.changeCategoryEnabled) {
			if (selectCategoria.val() == 12) {
				that.setVerticaleAste();
				that.resetSelect("#selectCategoriaAste");
			} else {
				that.removeVerticaleAste();
			}
			that.verticaleAsteManager();
		}
	};

	/**
	 * Metodo che si occupa di gestire tutti i casi speciali alla modifica della select Tipologia
	 * @param selectTipologia
	 */
	this.onChangeSelectTipologia = function (selectTipologia) {
		//gestisce il cambio della label per la tipologia per Commerciale e Stanze
		var selectCategoria = jq('#selectCategoria');
		if(selectCategoria.val() == 2 || selectCategoria.val() == 4) {
			selectTipologia.attr('data-prefix', 'Gruppo');
		} else {
			selectTipologia.attr('data-prefix', 'Tipologia');
		}
	};

	/**
	 * Metodo che si occupa di gestire tutti i casi speciali alla modifica della select sottoTipologia
	 * @param selectSottotipologia
	 */
	this.onChangeSelectSottotipologia = function (selectSottotipologia) {
		var that = this;
		//Gestione dei bottoni Contratto in base alla sottotipologia selezionata
		if (this.changeStatusContrattoEnabled)
			that.changeStatusBtnContratto(false);
		if (selectSottotipologia.val() == 113) {
			//uffici condivisi
			jq('#inputContratto').val(2).change();
			that.changeStatusBtnContratto(true);
			jq('#selectTipoProprietaFake').val(9).change(); // parziale-proprieta
		}
		this.changeStatusContrattoEnabled = false;
	};

	/**
	 * Metodo che si occupa di gestire tutti i casi speciali alla modifica della select TipologiaAste
	 * @param selectTipologiaAste
	 */
	this.onChangeSelectTipologiaAste = function (selectTipologiaAste) {
		// Si occupa di gestire il caso speciale commerciale setto la tipologia Immobile di default
		var selectCategoriaAste = jq('#selectCategoriaAste');
		if (selectCategoriaAste.val() == 2) {
			if (selectTipologiaAste.val() == 63) {
				selectTipologiaAste.closest('.pure-g').addClass('hidden-row').slideUp();
			}
		} else {
			selectTipologiaAste.closest('.pure-g').removeClass('hidden-row').slideDown();
		}
	};

	/**
	 * Metodo che si occupa di gestire tutti i casi speciali alla modifica della select TipoProprietaFake
	 * @param selectTipoProprietaFake
	 */
	this.onChangeSelectTipoProprietaFake = function (selectTipoProprietaFake) {
		var aReddito = jq("#aReddito");
		var tipoProprieta = jq("#tipoProprieta");
		if (selectTipoProprietaFake.val() == 'aReddito') {
			aReddito.val(1);
			tipoProprieta.val('');
		} else {
			aReddito.val('');
			tipoProprieta.val(selectTipoProprietaFake.val());
		}
	};

	/**
	 * In questo metodo si trovano tutti i casi di gestione non generici ma specifici per il singolo elemento
	 * presente nel form
	 */
	this.managementSpecialCases = function(){
		//gestione dei minimi e massimi
		this.ifFromIsGreaterThanToShowError('#priceFrom', '#priceTo');
		this.ifFromIsGreaterThanToShowError('#supFrom', '#supTo');
		this.ifFromIsGreaterThanToShowError('#roomsFrom', '#roomsTo');
	};

	/**
	 * Se il valore di From e' piu' grande di To aggiunge un bordo rosso agli input
	 * @param idFrom
	 * @param idTo
	 */
	this.ifFromIsGreaterThanToShowError = function (idFrom, idTo) {
		var that = this;
		jq(that.opt.form + ' input' + idFrom + ', ' + that.opt.form + ' input' + idTo).on('keyup', function () {
			var elementFrom = jq(that.opt.form + ' input' + idFrom);
			var elementTo = jq(that.opt.form + ' input' + idTo);
			elementFrom.toggleClass('input_error_border', that.checkInputFromIsGreaterOfTo(idFrom, idTo));
			elementTo.toggleClass('input_error_border', that.checkInputFromIsGreaterOfTo(idFrom, idTo));
		});
	};

	/**
	 * Torna true se From e' piu' grande di To, false altrimenti
	 * @param idFrom
	 * @param idTo
	 * @returns {boolean}
	 */
	this.checkInputFromIsGreaterOfTo = function (idFrom, idTo) {
		var elementFrom = jq(this.opt.form + ' input' + idFrom),
			elementTo = jq(this.opt.form + ' input' + idTo);

		if (elementFrom.exist() && elementTo.exist() && elementFrom.val() !== '' && elementTo.val() !== '') {
			var fromVal = parseInt(elementFrom.val().replace(/\./g, '')),
				toVal = parseInt(elementTo.val().replace(/\./g, ''));

			return fromVal > toVal;
		}

		return false;
	};

	/**
	 * Se il valore di From e' piu' grande di To swap data
	 * @param idFrom
	 * @param idTo
	 */
	this.ifFromIsGreaterOfToSwapData = function (idFrom, idTo) {
		if(this.checkInputFromIsGreaterOfTo(idFrom, idTo)){
			var elementFrom = jq(this.opt.form + ' input' + idFrom);
			var elementTo = jq(this.opt.form + ' input' + idTo);
			var oldValFrom = elementFrom.val();
			var oldValTo = elementTo.val();
			elementFrom.val(oldValTo).keyup();
			elementTo.val(oldValFrom).keyup();
		}
	};

	/**
	 * Si occupa di abilitare o disabilitare i bottoni contratto in base al valore passato
	 * @param disabled bool
	 */
	this.changeStatusBtnContratto = function (disabled) {
		var that = this;
		jq(that.opt.form + ' .btn-action[data-input="#inputContratto"]').each(function () {
			jq(this).toggleClass('disabled', disabled);
		});
	};

	/**
	 * Si occupa di popolare tutti i campi di ricerca in base alla hash
	 */
	this.fillByHash = function () {

		this.asyncAjax = false;
		var that = this;
		//mi ciclo tutti gli elementi del form
		jq(this.opt.form + ' input, ' + this.opt.form + ' select').each(function () {
			var element = jq(this);
			var elementName = element.attr('name');
			if (!jq.isEmpty(elementName)) {
				var hashValue = that.managerHash.getHash(elementName),
                    aHashValue = !jq.isEmpty(hashValue) ? hashValue.split(that.managerHash.delimiterM) : null;

				if (!!hashValue) {
					if (element.prop('type') == 'select-one') {
						element.val(hashValue).change();
					} else if (element.prop('type') == 'select-multiple') {
                        element.val(aHashValue).change();
                    } else if (element.prop('type') == 'text' || element.prop('type') == 'hidden') {
						element.val(hashValue).change();
						if (element.hasClass('check-format-number')) {
							checkAndFormatCurrency(element[0]);
						}
					} else if (element.prop('type') == 'checkbox') {
						element.prop('checked', ( hashValue == 1 ? true : false )).change();
					}
				}
			}
		});
		this.asyncAjax = true;
		// Dopo aver fillato gestisco il popolamento della selectTipoProprietaFake
		if (!jq.isEmpty(jq("#aReddito").val())) {
			jq("#selectTipoProprietaFake").val("aReddito").change();
		} else if (!jq.isEmpty(jq("#tipoProprieta").val())) {
			jq("#selectTipoProprietaFake").val(jq("#tipoProprieta").val()).change();
		}
	};

	/**
	 * Si occupa nascondere la select categoria e mostrare la label opportuna nel caso in cui veniamo dal listing
	 */
	this.categoryManager = function () {
		if (!this.changeCategoryEnabled) {
			jq("#categoriaDiv").addClass('hidden-row').hide();
			jq("#categoryNameDiv").removeClass('hidden-row').show();
			if (this.isVerticaleAste()) {
				jq("#categoryName").text('Aste');
			} else {
				jq("#categoryName").text(jq("#selectCategoria").find(":selected").text());
			}
		}
	};

	/**
	 * Si occupa di inizializzare la gestione dei casi speciali dei verticali
	 */
	this.initVerticale = function () {
		if (this.managerHash.getHash("verticaleAste") == 1 || this.managerHash.getHash("verticale") == 'aste') {
			this.setVerticaleAste();
		}
		this.verticaleAsteManager();
	};

	/**
	 * Si occupa di modificare il form in base a se stiamo nella situazione verticale aste o meno
	 */
	this.verticaleAsteManager = function () {
		if (this.isVerticaleAste()) {
			this.checkAttrNameAndAddOrRemove('#selectCategoriaAste', 'add', 'idCategoria');
			this.checkAttrNameAndAddOrRemove('#selectTipologiaAste', 'add', 'idTipologia');
			this.checkAttrNameAndAddOrRemove('#selectSottotipologiaAste', 'add', 'sottotipologia');
			this.checkAttrNameAndAddOrRemove('#selectCategoria', 'remove');
			this.checkAttrNameAndAddOrRemove('#selectTipologia', 'remove');
			this.checkAttrNameAndAddOrRemove('#selectSottotipologia', 'remove');
			this.setVerticaleAste();
			if (!jq("#blockAsteDiv").is(":visible")) {
				jq("#blockAsteDiv").slideDown();
			}
			if (this.changeCategoryEnabled) {
				var selectCategoria = jq("#selectCategoria");
				if (jq.isEmpty(selectCategoria.val())) {
					selectCategoria.val(12);
					this.changeSelectLabel(selectCategoria);
				}
			}
		} else {
			this.checkAttrNameAndAddOrRemove('#selectCategoria', 'add', 'idCategoria');
			this.checkAttrNameAndAddOrRemove('#selectTipologia', 'add', 'idTipologia');
			this.checkAttrNameAndAddOrRemove('#selectSottotipologia', 'add', 'sottotipologia');
			this.checkAttrNameAndAddOrRemove('#selectCategoriaAste', 'remove');
			this.checkAttrNameAndAddOrRemove('#selectTipologiaAste', 'remove');
			this.checkAttrNameAndAddOrRemove('#selectSottotipologiaAste', 'remove');
			this.removeVerticaleAste();
			if (jq("#blockAsteDiv").is(":visible")) {
				jq("#blockAsteDiv").slideUp();
			}
		}
	};

	this.isVerticaleAste = function () {
		return this.verticaleElm.val() == 'aste';
	};

	this.setVerticaleAste = function () {
		if (jq.isEmpty(this.verticaleElm.val())) {
			this.verticaleElm.val('aste').change();
		}
	};

	this.removeVerticaleAste = function () {
		if (!jq.isEmpty(this.verticaleElm.val())) {
			this.verticaleElm.val('').change();
		}
	};

	/**
	 * In base ai parametri passati si occupa di settare l'attributo name all'elemento oppure rimuoverlo
	 * @param selector
	 * @param action
	 * @param attrName
	 */
	this.checkAttrNameAndAddOrRemove = function (selector, action, attrName) {
		var element = jq(selector);
		if (!jq.isEmpty(element)) {
			attrName = !jq.isEmpty(attrName) ? attrName : '';
			switch (action) {
				case 'add':
					if (jq.isEmpty(element.attr('name'))) {
						element.attr('name', attrName);
					}
					break;
				case 'remove':
					if (!jq.isEmpty(element.attr('name'))) {
						element.removeAttr('name');
					}
					break;
			}
		}
	};

	/**
	 * Metodo che fa il submit dei dati.
	 */
	this.submit = function () {
		this.ifFromIsGreaterOfToSwapData('#priceFrom', '#priceTo');
		this.ifFromIsGreaterOfToSwapData('#supFrom', '#supTo');
		this.ifFromIsGreaterOfToSwapData('#roomsFrom', '#roomsTo');
		if (this.lastSerialize != this.serialize()) {
			this.lastSerialize = this.serialize();
			managerMarkers.resetParamsToReloadMap(true);
		}
		actionsOnMenuHeader.closeAllMenuOpened();
	};

	/**
	 * Metodo che ritorna la serializzazione del form
	 * @returns {*}
	 */
	this.serialize = function () {
		var elementArray = this.serializeArray(),
			modifiedArray = [],
			counts = {};

		jq.each(elementArray, function(index, value) {
			if (counts[value.name]) {
				counts[value.name] += 1;
			} else {
				counts[value.name] = 1;
			}
		});

		jq.each(elementArray, function(index, value) {
			if (counts[value.name] > 1) {
				modifiedArray.push({name: value.name + "[]", value: value.value});
			} else {
				modifiedArray.push({name: value.name, value: value.value});
			}
		});

		return jq.param(modifiedArray);
	};

	/**
	 * Metodo che ritorna la serializzazione del form come oggetto
	 * @returns {*}
	 */
	this.serializeArray = function () {
		return jq(this.opt.form).serializeArray();
	};

	/**
	 * Metodo che ritorna la serializzazione del form sotto forma di oggetto CHIAVE->VALORE
	 * @returns {*}
	 */
	this.serializeObj = function () {
		var a = jq(this.opt.form).serializeArray();
		var o = {};
		for (var i in a) {
		    if(typeof o[a[i].name] !== 'undefined'){  //valori multipli da serialize
                if(typeof o[a[i].name] === 'object'){  //se gia array/object
                    o[a[i].name].push(a[i].value);
                } else {
                    o[a[i].name] = [o[a[i].name], a[i].value];
                }
            } else {
                o[a[i].name] = a[i].value;
            }
		}
		return o;
	};

	/**
	 * Data la styled-select cambia la label associata.
	 * @param elem
	 */
	this.changeSelectLabel = function (elem) {
		var $elem = jq(elem),
			prefix = $elem.attr('data-prefix'),
			textPrefix = prefix !== '' ? '<span class="txt-prefix">' + prefix + ':</span>' : '',
			textValue = '<span class="txt-value">' + $elem.find('option:selected').text() + '</span>';

		$elem.parent().find('.select-label').html(textPrefix + ' ' + textValue);
	};

	/**
	 * Effettua la chiamata ajax per aggiornare le options di una select in base al valore dell'elemento da cui dipende.
	 * @param selectElement
	 * @param selectElementParent
	 */
	this.getResponseAjax = function (selectElement, selectElementParent) {
		var obj, _value, _dataField;
		obj = jq(selectElement);
		_value = jq.isEmpty(jq(selectElementParent).val()) ? '0' : jq(selectElementParent).val();
		_dataField = obj.attr('data-field');

		obj.empty().append('<option value="">Qualsiasi</option>');
		obj.find('option:first').attr('selected', 'selected');
		if (jq.isEmpty(_value)) {
			obj.parent().find('.select-label').html(function () {
				return jq(this).attr('data-default');
			});
			obj.change();
			return;
		}

		// eccezione Per la categoria stanze
		if (selectElementParent.attr('id') == "selectTipologia" && jq("#selectCategoria").val() == 4) {
			_dataField = 'sottotipologiaStanza';
		}

		var dataAjax = {chiave_est: _value, campo: _dataField, version: 'nuoveTipologie'};
		if (selectElement.attr('id') == 'selectTipoProprietaFake') {
			//nel caso del popolamento della select Tipo Proprieta devo aggiungere la option A Reddito
			dataAjax.tipo = 'extraOptions';
		}

		jq.ajax({
			async: this.asyncAjax,
			dataType: "xml",
			type: "POST",
			url: "/getElenchiJsRicerca.php",
			data: dataAjax
		}).done(function (xml) {
			obj.parent().find('.select-label').html(function () {
				return jq(this).attr('data-default');
			});
			jq(xml).find('riga').each(function () {
				if (jq(this).text() == 'Altro')
					return;
				obj.append('<option value="' + jq(this).attr('id') + '">' + jq(this).text() + '</option>');
			});
			if (selectElementParent.attr('id') == "selectCategoriaAste" && selectElementParent.val() == 2) {
				//In caso di Categoria Commerciale nel verticale aste setto in automatico la Tipologia Immobile
				obj.val(63);
			}
			obj.change();
		});
	};

	/**
	 * Resetta un elemento in base al tipo
	 * @param elem
	 */
	this.resetInput = function (elem) {
		var element = jq(elem);
		var type = element.attr('type');
		if (type == 'radio' || type == 'checkbox') {
			element.attr('checked', false);
		} else {
			element.val('');
		}
	};

	/**
	 * Resetta una select
	 * @param elem
	 */
	this.resetSelect = function (elem) {
		var element = jq(elem);
		if (jq.isEmpty(element.val())) {
			return;
		}
		element.find('option:selected').removeAttr('selected');
		element.change();
	};

	/**
	 * Metodo che dato un elemento accende/spegne tutti gli elementi che dipendono da esso.
	 * @param elem
	 */
	this.onChange = function (elem) {
		var that = this;
		var element = jq(elem);
		var field = element.attr('data-onchange');
		var className = "hiddenfor-" + field;
		var value = element.val() == '' ? '-1' : element.val();
		jq(this.opt.form + " [data-hiddenfor-" + field + "]").not("[data-hiddenfor-" + field + "~='" + value + "']").each(function () {
			jq(this).removeClass(className);
			if (jq(this).attr('class').indexOf('hiddenfor-') < 0)
				jq(this).slideDown().removeClass('hidden-row');
		});
		jq(this.opt.form + " [data-hiddenfor-" + field + "~='" + value + "']").each(function () {
			jq(this).addClass(className).slideUp().addClass('hidden-row');
			jq(this).find('label').each(function () {
				jq(this).removeClass('checked');
			});
			jq(this).find('input').each(function () {
				that.resetInput(this);
			});
			jq(this).find('select').each(function () {
				that.resetSelect(this);
			});
		});
	};

	/**
	 * Ridirigo l'utente alla pagina di ricerca standard
	 */
	this.goToStandardSearch = function () {
		var str = '';
		var idCategoria = '';
		var oSerialize = this.serializeArray();

		if (this.isVerticaleAste()) {
			oSerialize.push({name: 'verticaleAste', value: 1});
		}

		var bFoundCriterio = false;
		oSerialize = this.managementSpecialSearch(oSerialize);

		for (var k in oSerialize) {
            if (!oSerialize.hasOwnProperty(k)) {
                continue;
            }

			var obj = oSerialize[k];
			if (obj.value == '') {
				continue;
			}
			if (obj.name == 'criterio') {
				bFoundCriterio = true;
			}

			if (obj.name == 'idMZona') {
				var zonaValues = obj.value.split(',');
				for (var zvk in zonaValues) {
					str += '&idMZona[]=' + zonaValues[zvk];
				}
			} else if (obj.name == 'idLocalita') {
				var localitaValues = obj.value.split(',');
				for (var lvk in localitaValues) {
					str += '&idLocalita[]=' + localitaValues[lvk];
				}
			} else if (obj.name == 'aFascePiano' && obj.value != '') {
                str += '&fasciaPiano[]=' + obj.value;
            } else {
				str += '&' + obj.name + "=" + obj.value;
			}

			if (obj.name == 'idAreaGeografica' && obj.value != '') {
				str += '&vacanze=1';
			}

			if (obj.name == 'idCategoria') {
				idCategoria = obj.value;
			}

			if (obj.name == 'fkMacroAreaGeografica') {
                str += '&idmacroarea=' + obj.value;
            }
        }

		if ( !bFoundCriterio ) {
			str += '&criterio=rilevanza';
		}

		if (this.searchType === 'agenzia') {
			window.location = '/ricerca_agenzie.php?avviaricerca=1' + str;
		} else if ( idCategoria == '' ) {
			// non posso effettuare il redirect alla ricerca e lo redirigo in homepage
			window.location = '/';
		} else if (idCategoria == 6) {
			window.location = '/ricerca_nc.php?avviaricerca=1' + str;
		} else {
			window.location = '/ricerca.php?avviaricerca=1' + str;
		}
	};

	this.managementSpecialSearch = function( oSerialize ) {
		var oValues = {categoria: {value:null, index:null}, tipologia: {value:null, index:null}, sottotipologia: {value:null, index:null} };
		for( var i in oSerialize ) {
			var obj = oSerialize[i];
			switch ( obj.name ) {
				case 'idCategoria':
					oValues.categoria.value = obj.value;
					oValues.categoria.index = i;
					break;
				case 'idTipologia':
					oValues.tipologia.value = obj.value;
					oValues.tipologia.index = i;
					break;
				case 'sottotipologia':
					oValues.sottotipologia.value = obj.value;
					oValues.sottotipologia.index = i;
					break;
			}
		}
		// Gestione dei terreni a => commerciale/terreni
		if ( oValues.categoria.value == 10 ) {
			if ( oValues.tipologia.value && oValues.tipologia.value != '' ) {
				if ( oValues.sottotipologia.index ) {
					oSerialize[ oValues.sottotipologia.index ].value = oValues.tipologia.value;
				} else {
					oSerialize.push( {name: 'sottotipologia', value: oValues.tipologia.value} );
				}
			}
			oSerialize[ oValues.tipologia.index ].value = 28; // tipologia terreni
			oSerialize[ oValues.categoria.index ].value = 2; // categoria commerciale
		} else if (  oValues.categoria.value == 4 ) {
			// Gestione Stanze
			if ( oValues.tipologia.value && oValues.tipologia.value != '' ) {
				oSerialize.push( {name: 'categoriaStanza', value: oValues.tipologia.value} );
				oSerialize[ oValues.tipologia.index ].value = '';
			}
			if ( oValues.sottotipologia.value && oValues.sottotipologia.value != '' ) {
				oSerialize.push( {name: 'idTipologiaStanza', value: oValues.sottotipologia.value} );
				oSerialize[ oValues.sottotipologia.index ].value = '';
			}
		}
		return oSerialize;
	};

	/**
	 * In base al numero di colonne o elementi del form visibili aggiungo o rimuove la classe 'two-columns'
	 */
	this.checkFormClassName = function () {
		var visible_elem = 0;
		var two_columns = false;
		jq(this.opt.form + ' fieldset').each(function () {
			var partial = 0;
			if (jq(this).hasClass('hidden-row')) {
				two_columns = true;
				return;
			}
			jq(this).find('.form-row').each(function () {
				partial += (!jq(this).hasClass('hidden-row')) ? 1 : 0;
			});
			if (partial <= 0) {
				jq(this).hide();
				two_columns = true;
			} else {
				jq(this).show();
			}
			visible_elem += partial;
		});
		jq(this.opt.form).toggleClass('two-columns', ( two_columns || visible_elem <= 14 ));
	};

	/**
	 * Prende gli elementi del form e restituisce un array con i loro name
	 * @returns {null|array}
	 */
	this.getFormElements = function () {
		if (!this.formElements) {
			var that = this;
			this.formElements = [];
			jq(this.opt.form + ' input, ' + this.opt.form + ' select').each(function () {
				var attrName = jq(this).attr('name');
				if (!jq.isEmpty(attrName)) {
					that.formElements.push(attrName);
				}
			});
		}
		return this.formElements;
	};

	// inizializzo la classe
	this.init();
}
;
'use strict';

var ManagerRequest = (function ($) {

    return ManagerRequest = function (params) {
        params = params || {};

        var refineSearch          = params.refineSearch || null,
            searchType            = params.searchType || 'annuncio',
            urlGetMapMarkers      = (params.ulrMarkers || '/ricerca_mappa/getMapMarkers.php') + '?search_type=' + searchType,
            urlGetListing         = (params.urlListing || '/ricerca_mappa/getListing.php') + '?search_type=' + searchType,
            urlGetDetail          = (params.urlDetail || '/ricerca_mappa/getDetail.php') + '?search_type=' + searchType,
            urlGetHtmlRowListing  = (params.urlHtmlListing || '/ricerca_mappa/getHtmlRowListing.php') + '?search_type=' + searchType,
            htmlRowListing        = '',
            timeoutRequest        = 30 * 1000, // millisecondi
            ajaxRequest           = {map: {}, listing: {}},
            orderAjaxRequest      = {map: [], listing: []},
            timeLastAjaxRequest   = {map: null, listing: null};


        /**
         *
         */
        var initialize = function () {
            $.get(urlGetHtmlRowListing, function (html) {
                htmlRowListing = html;
            });
        };


        /**
         * Effettuo la richiesta ajax x prendere gli annunci di una geohash
         *
         * @param params
         * @returns Object Deferred
         */
        var getListingAdsFromGeoHash = function (params) {
            var $deferred = $.Deferred();

            $.ajax({
                dataType: 'json',
                type: 'POST',
                data: params,
                url: urlGetMapMarkers
            })
                .done(function (data) {
                    if (!data.listing) {
                        $deferred.resolve('');
                        return;
                    }

                    var list = '';

                    for (var i = 0; i < data.listing.length; i++) {
                        list += getHtmlListing(data.listing[i]);
                    }

                    $deferred.resolve(list);
                }).fail(function () {
                    $deferred.reject();
                });

            return $deferred;
        };


        /**
         * Effettua la chiamata ajax che gestisce
         * @param dataPost
         * @returns {*}
         */
        var getMultiResponseGrid = function (dataPost) {
            var $deferred = $.Deferred();

            $.ajax({
                dataType: 'json',
                type: 'POST',
                url: urlGetMapMarkers,
                data: dataPost
            })
                .done(function (data) {
                    $deferred.resolve(data);
                })
                .fail(function () {
                    $deferred.reject();
                });

            return $deferred;
        };


        /**
         * Effettua la chiamata ajax per recupera gli annunci per il listing nel box di sinistra.
         *
         * @param requestParams
         * @returns Object Deferred
         */
        var getListingResponse = function (requestParams) {
            // controlla se ci sono altre richieste dei marker eventualmente le abortisce.
            checkListingRequest();

            var $deferred = $.Deferred(),
                requestTime = timeLastAjaxRequest.listing;

            ajaxRequest.listing[requestTime] = $.ajax({
                dataType: 'json',
                type: 'POST',
                url: urlGetListing,
                data: requestParams,
                timeout: timeoutRequest
            })
                .done(function (data) {
                    data.html = '';

                    if (data.listing && data.listing.length) {
                        for (var i = 0; i < data.listing.length; i++) {
                            data.html += getHtmlListing(data.listing[i]);
                        }
                    }

                    $deferred.resolve(data);
                }).fail(function () {
                    $deferred.reject();
                });

            return $deferred;
        };


        /**
         * Effettua la chiamata ajax per recuperare i dati di un dettaglio.
         *
         * @param id
         * @param geohash
         * @return jqXHR object
         */
        var openDetail = function (id, geohash) {
            var urlRequest,
                data = {};

            if (typeof id !== 'undefined' && id !== null) {
                urlRequest = getAjaxUrlDetail(id);

                if (refineSearch && $.isFunction(refineSearch.serializeObj)) {
                    data = refineSearch.serializeObj();
                }
            } else {
                urlRequest = getAjaxUrlListing(geohash);
            }

            return $.ajax({
                url: urlRequest,
                type: 'POST',
                data: data
            });
        };


        /**
         * Al caricamento dell'html aggiunge gli eventi riferiti al dettaglio.
         */
        var eventOnDetail = function () {
            addPlaceHolderCompatibility('contattaDettaglioUp');

            if (searchType === 'annuncio') {
                // Initialize Galleria
                Galleria.run('#galleriaMapDetail', {
                    showCounter: true,
                    showInfo: false,
                    thumbnails: "empty",
                    swipe: true,
                    clicknext: true,
                    preload: 2,
                    debug: false
                });
            }

            /* Gestisce gli eventi sul tab del dettaglio */
            $('#infowindow-frame .imm-nav-button .item').on('click', function (e) {
                var dataTab = $(this).attr('data-tab'),
                    $elem = $(dataTab);

                switch (dataTab) {
                    case '#adDetail':
                        $('#infowindow-frame .linkDetail').get(0).click();
                        break;
                    case '#adContacts':
                        $elem.siblings().removeClass('active');
                        $elem.addClass('active');
                        $(this).parent().hide();
                        $('.imm-nav-button .item[data-tab="#adPhoto"]').parent().show();
                        $('.imm-nav-button .item[data-tab="#adInfo"]').parent().show();
                        break;
                    case '#adPhoto':
                    case '#adInfo':
                        $elem.siblings().removeClass('active');
                        $elem.addClass('active');
                        $(this).parent().hide();
                        $('.imm-nav-button .item[data-tab="#adContacts"]').parent().show();
                        break;
                    default:
                        $elem.siblings().removeClass('active');
                        $elem.addClass('active');
                        break;
                }
            });

            /* Gestisce l'azione di bookmark annuncio */
            $('#infowindow-frame .action-bookmark-annuncio').on('click', function () {
                bookmarkAnnuncioSuMappa(this, {section: 'ricerca-mappa-dettaglio'});
            });
        };


        /**
         * Definisce gli eventi sulla lista all'interno dell'infowindow
         */
        var eventOnListing = function () {
            $("#infowindow-frame .riga_annuncio").on('click', function (e) {
                var $elem = $(this),
                    geohash = $elem.attr('data-gh'),
                    itemsInListing = $elem.siblings().length + 1,
                    idAd = $elem.attr('data-id');

                $(root).trigger('clickListingOnMarker', [idAd, geohash, itemsInListing, {openedFrom: 'marker'}]);
            });
        };


        /**
         * Genera la url per un dettaglio
         *
         * @param id
         * @returns {string}
         */
        var getAjaxUrlDetail = function (id) {
            return urlGetDetail
                + (urlGetDetail.search('/?') === -1 ? '?' : '&')
                + 'id=' + id;
        };


        /**
         * Genera la url di un listing
         *
         * @param idGeohash
         * @returns {string}
         */
        var getAjaxUrlListing = function (idGeohash) {
            return urlGetDetail
                + (urlGetDetail.search('/?') === -1 ? '?' : '&')
                + 'disableNazione=1&checkNewDate=1'
                + '&idGH=' + idGeohash
                + '&criterio=checkNewDate&ordine=desc'
                + (refineSearch && $.isFunction(refineSearch.serialize) ? '&' + refineSearch.serialize() : '');
        };


        /**
         * Ritorna l'html della riga di un annuncio o di una agenzia.
         *
         * @param ad = un oggetto con i dati del dettaglio
         * @returns html = l'html di un dettaglio
         */
        var getHtmlListing = function (ad) {
            if (searchType === 'agenzia') {
                return getHtmlListingAgency(ad);
            }

            return getHtmlListingAd(ad);
        };

        /**
         * Dato un oggetto con i dati dell'agenzia restituisce l'html del listing
         *
         * @param agency
         * @returns {string}
         */
        var getHtmlListingAgency = function (agency) {
            var visibilita = '',
                indirizzo = agency.ind,
                opzionePremium = agency.op ? parseInt(agency.op) : null;

            if (opzionePremium === 1) {
                visibilita = 'premium';
            } else if (opzionePremium === 2) {
                visibilita = 'top';
            }

            if ($('#idComune').exist() && $('#idComune').val() === '') {
                indirizzo += (indirizzo !== '' ? ', ' : '') + agency.cn;
            }

            return htmlRowListing
                .replace(/{%CLASSEHIDEIMAGE%}/gi, agency.fl === '' ? 'hidden' : '')
                .replace(/{%CLASSENOIMAGE%}/gi, agency.fl === '' ? 'full-width' : '')
                .replace(/{%SRCIMAGE%}/gi, agency.fl !== '' ? __gvs_MEDIA_SERVER_IMAGE + 'imagenoresize/' + agency.fl + '.jpg' : '')
                .replace(/{%VISIBILITA%}/gi, visibilita)
                .replace(/{%COMUNE%}/gi, agency.cn)
                .replace(/{%INDIRIZZO%}/gi, indirizzo)
                .replace(/{%IDGROUP%}/gi, 'GR_AG_' + agency.id)
                .replace(/{%IDGEOHASH%}/gi, agency.gh)
                .replace(/{%TITOLO%}/gi, agency.ttl);
        };

        /**
         * Dato un oggetto con i dati dell'annuncio restituisce l'html del listing
         *
         * @param ad = un oggetto con i dati del dettaglio
         * @returns html = l'html di un dettaglio
         */
        var getHtmlListingAd = function (ad) {
            var sep1, prezzo, superficie, srcimage, locali, idGr, visibilita, saved, inAffitto, inVendita,
                bkmAction, bkl_class, bkl_uba_id, bkl_type, bkl_title, immoGarantito;

            inAffitto = false;
            inVendita = false;
            immoGarantito = prezzo = locali = superficie = visibilita = '';
            saved = ad.svd ? 'ico-stella on' : 'ico-stella';

            // assegnazione valori PROGETTO
            if (ad.is == 'progetto') {
                idGr = 'GR_P_' + ad.id;
                bkmAction = ad.svd ? 'deleteProgetto' : 'addProgetto';
                bkl_type = 'pj';

                if (ad.pr1 && ad.pr2) {
                    if (ad.pr1 == ad.pr2) {
                        prezzo = '&euro; ' + ad.pr1;
                    } else {
                        prezzo = ad.pr1 + ' - ' + ad.pr2 + ' &euro;';
                    }
                }

                if (ad.su1) {
                    if (ad.su1 == ad.su2) {
                        superficie = ad.su1 + ' m&sup2;';
                    } else {
                        superficie = 'da ' + ad.su1 + ' m&sup2;';
                    }
                }

                if (ad.loc1 && ad.loc2) {
                    if (ad.loc1 === ad.loc2) {
                        locali = ad.loc1 + (ad.loc1 == 1 ? ' locale' : ' locali');
                    } else {
                        locali = ad.loc1 + ' - ' + ad.loc2 + ' locali';
                    }
                }

                sep1 = (superficie !== '' && locali !== '') ? '<span class="sep">&nbsp;•&nbsp;</span>' : '';

                if (ad.pig) {
                    immoGarantito = '<span class="immobile_garantito_circle" title="Immobile Garantito"></span>';
                }

            } else { // assegnazione valori ANNUNCI
                idGr = 'GR_A_' + ad.id;
                bkmAction = ad.svd ? 'delete' : 'add';
                bkl_type = 'ad';

                if (ad.siea) { // estensione contratto affitto agenzia (mostra affitto)
                    inAffitto = true;

                    if (ad.mpea) {
                        prezzo = '&euro; ' + ad.pea + '<span style="font-weight: normal;"> al mese</span>';
                    } else {
                        prezzo = 'Prezzo su richiesta';
                    }
                } else if (ad.ecp_ma) { //estensione contratto privati(mostra affitto)
                    inAffitto = true;

                    if (ad.ecp_pv) {
                        prezzo = "&euro; " + ad.ecp_pr + '<span style="font-weight: normal;"> al mese</span>';
                    } else {
                        prezzo = "Prezzo su richiesta";
                    }
                } else if (ad.ecp_mv) {	//estensione contratto privati(mostra vendita)
                    inVendita = true;

                    if (ad.ecp_pv) {
                        prezzo = "&euro; " + ad.ecp_pr;
                    } else {
                        prezzo = "Prezzo su richiesta";
                    }
                } else if (((ad.fp == 12 || ad.fp == 13) && typeof ad.opp_pv === 'undefined') || (typeof ad.opp_pv !== 'undefined' && ad.opp_pv == 1)) {
                    if (typeof ad.opp_pv !== 'undefined') { //annuncio privato con opzione prezzo nascosto
                        if (ad.opp_pv == 1) {
                            prezzo = "Prezzo su richiesta";
                        }
                    } else {
                        prezzo = "Prezzo su richiesta";
                    }
                } else if (ad.pr && ad.pr != '') {
                    prezzo = "&euro; " + ad.pr;

                    if (ad.fc == 2) {
                        prezzo += '<span style="font-weight: normal;"> al mese</span>';
                    }
                } else if (ad.fpn !== '') {
                    prezzo = ad.fpn;

                    if (ad.fc == 2) {
                        prezzo += '<span style="font-weight: normal;"> al mese</span>';
                    }
                }

                superficie = ad.su ? ad.su + ' m&sup2;' : '';

                if (ad.loc) {
                    locali = ad.loc;
                    locali = locali.replace('>5', 'più di 5');
                    locali += ad.loc == 1 ? ' locale' : ' locali';
                }

                sep1 = ( ad.su && ad.loc ) ? '<span class="sep">&nbsp;•&nbsp;</span>' : '';

                if (ad.aeig) {
                    immoGarantito = '<span class="immobile_garantito_circle" title="Immobile Garantito"></span>';
                }
            }

            //l'ordine è vetrina,star,top,premium,base
            if (ad.nw) {
                visibilita = 'nuovo';
            } else if (ad.vtr) {
                visibilita = 'vetrina';
            } else if (ad.star){
                visibilita = 'star';
            } else if (ad.top) {
                visibilita = 'top';
            } else if (ad.prm) {
                visibilita = 'premium';
            }

            if (ad.imm) {
                srcimage = __gvs_MEDIA_SERVER_IMAGE + 'image/' + ad.imm + '/listing.jpg';
            } else {
                srcimage = __gvs_MEDIA_SERVER + '/img2/information.gif';
            }

            if (ad.uba_id) {
                bkl_class = 'link_remake_blacklist_map';
                bkl_uba_id = ad.uba_id;
                bkl_title = 'Questo annuncio &egrave; stato aggiornato, perci&ograve; lo abbiamo automaticamente ripristinato nelle tue ricerche. Clicca per nasconderlo di nuovo';
            } else {
                bkl_class = 'link_make_blacklist_map';
                bkl_uba_id = '';
                bkl_title = 'Nascondi questo annuncio dalle tue ricerche. Potrai ripristinarlo dal tuo pannello utente.';
            }

            return htmlRowListing
                .replace(/{%SRCIMAGE%}/gi, srcimage)
                .replace(/{%IDANNUNCIO%}/gi, ad.id)
                .replace(/{%IDGROUP%}/gi, idGr)
                .replace(/{%IDGEOHASH%}/gi, ad.gh)
                .replace(/{%ISNEW%}/gi, ad.nw ? "new-ad" : '')
                .replace(/{%BOOKMARKACTION%}/gi, bkmAction)
                .replace(/{%CLASSESTELLA%}/gi, saved)
                .replace(/{%TITOLO%}/gi, ad.ttl)
                .replace(/{%VISIBILITA%}/gi, visibilita)
                .replace(/{%PREZZO%}/gi, prezzo)
                .replace(/{%SUPERFICIE%}/gi, superficie)
                .replace(/{%SEP1%}/gi, sep1)
                .replace(/{%LOCALI%}/gi, locali)
                .replace(/{%ATTR_BOOKMARK_INAFFITTO%}/gi, inAffitto ? 'data-inAffitto="1"' : '')
                .replace(/{%BKL_CLASS%}/gi, bkl_class)
                .replace(/{%BKL_UBA_ID%}/gi, bkl_uba_id)
                .replace(/{%BKL_TYPE%}/gi, bkl_type)
                .replace(/{%BKL_TITLE%}/gi, bkl_title)
                .replace(/{%IMMO_GARANTITO%}/gi, immoGarantito)
                .replace(/{%ATTR_BOOKMARK_INVENDITA%}/gi, inVendita ? 'data-inVendita="1"' : '');
        };


        /**
         * Controlla se ci sono altre richieste vecchie del listing, eventualmente le abortisce.
         */
        var checkListingRequest = function () {
            // aggiorno il nuovo starter della richiesta.
            timeLastAjaxRequest.listing = new Date().getTime();

            // aggiunto l'ultimo tempo alla lista ordinata di eventi
            orderAjaxRequest.listing.push(timeLastAjaxRequest.listing);

            // controllo tutte le richieste ajax della lista => se ne trovo alcune con un timestamp inferiore la interrompo ( ad eccezione della prima )
            if (!$.isEmpty(ajaxRequest.listing)) {

                for (var time in ajaxRequest.listing) {
                    time = parseInt(time);

                    if (typeof orderAjaxRequest.listing.indexOf !== 'undefined' && orderAjaxRequest.listing.indexOf(time) > 0 && time < timeLastAjaxRequest.listing) {
                        if (ajaxRequest.listing[time].readyState !== 4) {
                            ajaxRequest.listing[time].abort();
                        }

                        delete ajaxRequest.listing[time];
                    }
                }

            }
        };


        var root = {
            initialize: initialize,
            openDetail: openDetail,
            eventOnDetail: eventOnDetail,
            eventOnListing: eventOnListing,
            getHtmlListing: getHtmlListing,
            getListingAdsFromGeoHash: getListingAdsFromGeoHash,
            getListingResponse: getListingResponse,
            getMultiResponseGrid: getMultiResponseGrid,
            urlGetMapMarkers: urlGetMapMarkers
        };


        return root;
    };

})(jQuery);
;
var timeoutAddMarkers = null,
    timeoutCloseLabelDiv = null;

function ManagerMarkers(idContainer, options) {
    this.options = typeof options != "undefined" ? options : [];
    this.options.minZoom = options.minZoom || 5;
    this.options.maxZoom = options.maxZoom || 20;
    this.options.forceDisabledInfoWindow = true; // disabilito l'apertura di default dell'infowindow
    this.map = null;
    this.idContainer = idContainer;
    this.container = jq(this.idContainer);
    this.isMobile = typeof options.isMobile != 'undefined' && options.isMobile ? 1 : 0;
    this.geohashLen = null;
    this.bounds = null;
    this.zoom = null;
    this.lastCenter = null;
    this.markers = {};
    this.maxMarkers = this.isMobile ? 0 : 300;
    this._maxAdsOnListing = 15;
    this._minZoomForListing = 15;
    this._zoomStartJsonStream = typeof this.options.zoomStartJsonStream != 'undefined' ? this.options.zoomStartJsonStream : 15;
    this._jsonStreamEnabled = typeof this.options.jsonStreamEnabled != 'undefined' ? this.options.jsonStreamEnabled : 1;
    this.mapExt = null;
    this._labelDiv = null;
    this._stopClickMarker = false;
    this.stopOnMouseOver = false;
    this._lastHtmlListOpened = null;
    this._zoomLevels = [1, 5, 8, 10, 13, 15, 18, 30];
    this._zoomLevelsMobile = [1, 5, 8, 11, 13, 15, 18, 30];
    this._currentIdMarkerOpened = null;
    this._lastMarkerToOpen = null;
    this._lastAdToOpen = null;
    this._detailOpenedFrom = '';
    this._timeoutMouseOverListing = null;
    this.refineSearch = new RefineSearch({form: '#refine-search', searchType: this.options.searchType || 'annuncio'});
    this.managerHash = new ManagerHash({
        onBack: this.onBack(),
        onForward: this.onForward(),
        onPopState: this.onPopState()
    });

    this._enabledLog = false; // Abilita gli eventuali log presenti in managerMarkers
    this._log = new MapLog({enabled: this._enabledLog});

    this.managerRequest = new ManagerRequest({
        refineSearch: this.refineSearch,
        searchType: this.options.searchType || 'annuncio'
    });
};


ManagerMarkers.prototype.initEventManagerRequest = function () {
    var _this = this;

    jq(this.managerRequest)
        .on('clickListingOnMarker', function (event, id, geohash, itemsInListing, opt) {
            jq('#responseMapAds.listingAds').mCustomScrollbar("destroy");

            _this._lastHtmlListOpened = jq("#containerResponseMapAds").parent().html();
            _this.openDetail(id, geohash, itemsInListing, opt);
        });
};


/**
 * Funzione che inizializza tutti gli eventi riferiti alla gestione dei marker e dello zoom della mappa.
 */
ManagerMarkers.prototype.init = function () {
    google.maps.visualRefresh = false;

    // disabilita l'apertura del popup al click sui POI di google
    if (this.options.forceDisabledInfoWindow && typeof setOptEnableInfoWindow === 'function') {
        setOptEnableInfoWindow();
    }

    this.managerRequest.initialize();
    this.initEventManagerRequest();

    this.map = new Im_GMap(this.container[0], this.options);
    this.mapExt = new GMapsExtV3(this.map);
    var that = this;

    google.maps.event.addListenerOnce(this.map, 'tilesloaded', function () {
        that.addHashStatusOnDrag();
        that.zoom = that.map.getZoom();
        var optBoxLabel = {
            id: '#infowindow-frame',
            offset: {top: -60, left: -270},
            events: {
                onClose: function () {
                    managerMarkers.stopOnMouseOver = false;
                    managerMarkers._currentIdMarkerOpened = null;

                    if (that.managerHash.hasHash('idGH')) {
                        managerMarkers.managerHash.setHash({idAd: '', idGH: ''}, null, false);
                    }
                },
                onAdd: function () {
                    var _checkTouchMove = false;
                    jq('#infowindow-frame')
                        .bind('mousewheel', function (e) {
                            e.stopPropagation();
                        })
                        .on('mousedown click', function (e) {
                            e.stopPropagation();
                        })
                        .on('dblclick', function (e) {
                            e.stopPropagation();
                            e.preventDefault();
                        })
                        .on('touchstart', function () {
                            _checkTouchMove = 0;
                        })
                        .on('touchmove', function (e) {
                            _checkTouchMove = 1;
                            e.preventDefault();
                        })
                        .on('touchend', function (e) {
                            if (_checkTouchMove) {
                                e.preventDefault();
                            }
                        });
                },
                onDraw: function () {
                    jq('#infowindow-frame .trackGAClickEvent')
                        .on('click', function () {
                            trackGAClickEvent('ricercaMappaEvents', that._detailOpenedFrom, 'dettaglio');
                        });
                    jq("#infowindow-frame .close-infowindow")
                        .on('click', function () {
                            that._labelDiv.hide();
                        });
                    jq('#infowindow-frame .riga_annuncio .action-bookmark-annuncio')
                        .on('click', function (e) {
                            e.stopPropagation();
                            bookmarkAnnuncioSuMappa(this);
                        });
                    jq('#infowindow-frame .riga_annuncio')
                        .on('click', '.link_make_blacklist_map, .link_remake_blacklist_map', function (e) {
                            e.stopPropagation();
                            //mi calcolo il numero di elementi nella windows
                            var idGH = jq(this).closest(".riga_annuncio").data('gh');
                            var optBkl = {};
                            if (that.markers[idGH].labelContent == 1) {
                                optBkl.noSlideUp = true;
                            }
                            blacklistAnnunci.makeBlacklistMap(this, optBkl);
                        });
                    jq('#infowindow-frame input, #infowindow-frame textarea')
                        .on('click', function () {
                            jq(this).focus();
                        });
                }
            }
        };
        that._labelDiv = new BoxLabel(that.map, optBoxLabel);
        that._lastAdToOpen = that.managerHash.getHash('idAd');
        that._lastMarkerToOpen = that.managerHash.getHash('idGH');
    });

    google.maps.event.addListener(this.map, 'idle', function () {
        clearTimeout(timeoutAddMarkers);
        timeoutAddMarkers = setTimeout(function () {
            that.doAjax();
        }, 100);
        that._stopClickMarker = false;
    });

    google.maps.event.addListener(this.map, 'resize', function () {
        that.bounds = null;
        if (that.lastCenter) {
            that.map.setCenter(that.lastCenter);
        }
    });

    google.maps.event.addListener(this.map, 'dragstart', function () {
        that._stopClickMarker = true;
    });

    google.maps.event.addListener(this.map, 'drag', function () {
        if (that._labelDiv.isVisible()) {
            that._labelDiv.setPosition();
        }
    });

    google.maps.event.addListener(this.map, 'zoom_changed', function () {
        if (that._labelDiv) {
            that._labelDiv.hide();
        }
        that.bounds = null;
        that._currentIdMarkerOpened = null;
        var currentZoom = that.map.getZoom();
        jq("#zoom_out_box_mappa").toggleClass('disable', currentZoom == that.options.minZoom);
        jq("#zoom_in_box_mappa").toggleClass('disable', currentZoom == that.options.maxZoom);
    });

    /*-----------------------------------------------------------------*/
    /*                 GESTIONE DEGLI EVENTI SUI MARKER                */
    /*-----------------------------------------------------------------*/

    jq('#map-ads')
        .on('mouseenter', '.clusterMarker', function () {
            that.markerOnMouseOver(this);
        })
        .on('mouseleave', '.clusterMarker', function () {
            that.markerOnMouseOut();
        })
        .on('mouseenter', '#infowindow-frame', function () {
            clearTimeout(timeoutCloseLabelDiv);
        })
        .on('mouseleave', '#infowindow-frame', function () {
            that.markerOnMouseOut();
        });

    jq('#box-listing')
        .on('click', '.action-bookmark-annuncio', function (e) {
            e.stopPropagation();
            bookmarkAnnuncioSuMappa(this);
        })
        .on('click', '.link_make_blacklist_map, .link_remake_blacklist_map', function (e) {
            e.stopPropagation();
            blacklistAnnunci.makeBlacklistMap(this);
        })
        .on('click', '.riga_annuncio', function () {
            that.listingOnClick(this);
            actionsOnMenuHeader.closeAllMenuOpened();
        })
        .on('mouseenter', '.riga_annuncio', function () {
            that.listingOnMouseOver(this);
        })
        .on('mouseleave', '.riga_annuncio', function () {
            that.listingOnMouseOut();
        });

    /* eventi paginazione listing */
    jq('#box-pagination').on('click', '.go-to-page', function () {
        if (jq('#pag').val() == jq(this).attr('data-page') || jq(this).hasClass('disable')) {
            return false;
        }
        jq('#pag').val(jq(this).attr('data-page'));
        that.getListingResponse();
        that.setUrlHash();
    });
};


/**
 * Cancella tutti i marker che hanno una hashlen diversa da quella.
 * @param hashLen
 *
 * NOTA:
 * Se hashLen = 0 allora li cancello tutti.
 */
ManagerMarkers.prototype.deleteMarkers = function (hashLen) {
    for (var key in this.markers) {
        if (key.length == hashLen) {
            continue;
        }

        this.markers[key].setMap(null);
        delete this.markers[key];

        if (this._labelDiv.idAnchor == "#cluster_" + key) {
            this._labelDiv.hide();
        }
    }
};


/**
 * Dato un oggetto (o un array associativo) ne calcola la lunghezza.
 * @param obj
 * @returns {integer}
 */
function getObjLength(obj) {
    if (typeof Object.keys != "undefined") {
        return Object.keys(obj).length;
    }
    var len = obj.length ? --obj.length : -1;
    for (var k in obj) {
        len++;
    }
    return len;
};


/**
 * Prende tutti i marker della mappa e cancella i marker che sono out dal bounds.
 */
ManagerMarkers.prototype.checkVisibilityMarkers = function () {
    var objLen = getObjLength(this.markers);
    for (var key in this.markers) {
        if (this.bounds && !this.bounds.contains(this.markers[key].getPosition())) {
            if (objLen > this.maxMarkers) {
                this.markers[key].setMap(null);
                delete this.markers[key];
            } else {
                this.markers[key].setVisible(false);
            }
            // rendo invisibile la label aperta
            if (this._labelDiv.idAnchor == "#cluster_" + key) {
                this._labelDiv.hide();
            }
        }
    }
};


/**
 * Data una hashlen porta in primo piano tutti i marker che ne fanno parte.
 * @param hashLen
 */
ManagerMarkers.prototype.changeZIndexMarkers = function (hashLen) {
    for (var key in this.markers) {
        if (key.length == hashLen) {
            this.markers[key].setZIndex(600);
        }
    }
};


/**
 * Effettua lo spostamento della label del marker.
 */
ManagerMarkers.prototype.markersPanTo = function () {
    for (var key in this.markers) {
        if (this.markers[key].get('goTo')) {
            this.markers[key].label.goTo(this.markers[key].get('goTo'));
            this.markers[key].goTo = null;
        }
    }
};


/**
 * Aggiunge i marker alla mappa. Se il marker già esiste allora ne cambia solo la visibilità
 * @param idGH = idGeohash
 * @param options = array con le opzioni del marker
 * @returns {boolean}
 */
ManagerMarkers.prototype.drawMarker = function (idGH, options) {
    options = options || {};

    if (typeof options.point == "undefined") {
        return false;
    }

    var icon, that, position, goTo;

    options.goToEnabled = typeof options.goToEnabled != 'undefined' ? options.goToEnabled : true;

    // controllo se il marker e' salvato nell'array di classe
    if (typeof this.markers[idGH] !== 'undefined') {
        if (!this.markers[idGH].getVisible()) {
            this.markers[idGH].setVisible(true);
        }

        this.markers[idGH].labelContent = options.labelContent || "";

        return true;
    }

    icon = new google.maps.MarkerImage('/img2/transparent.gif');

    if (options.goToEnabled && typeof options.startPosition != "undefined") {
        goTo = options.point;
        position = options.startPosition;
    } else {
        position = options.point;
        goTo = null;
    }

    that = this;
    this.markers[idGH] = new MarkerWithLabel({
        goTo: goTo,
        position: position,
        map: this.map,
        icon: icon,
        labelAnchor: new google.maps.Point(options.pointX || 0, options.pointY || 0),
        labelContent: options.labelContent || "",
        labelID: options.labelID || "",
        labelClass: options.labelClass || "",
        isMobile: this.isMobile
    });
    this.markers[idGH].MBR = options.MBR || '';
    this.markers[idGH].count = options.count || null;
    // controllo se e' presente un click precedente allo zoom su un marker con 1 annuncio ed eventualmente apro il dettaglio.
    if (this._lastAdToOpen || (options.count == 1 && this._lastMarkerToOpen && idGH.search(this._lastMarkerToOpen) >= 0)) {
        this.openDetail(this._lastAdToOpen, idGH, options.count, {openedFrom: 'marker', needTimeout: true});
        this._lastMarkerToOpen = null;
        this._lastAdToOpen = null;
    }

    google.maps.event.addListener(this.markers[idGH], 'click', function () {
        // In questo scope "this" rappresenta il marker
        if (that._stopClickMarker) {
            that._stopClickMarker = false;
            return false;
        }

        that.stopOnMouseOver = true;
        that._lastHtmlListOpened = null;

        var numAds = parseInt(this.count),
            currentZoom = that.map.getZoom();

        that._lastMarkerToOpen = null;

        // In base ad alcune condizioni aumento lo zoom.
        if (currentZoom < that.options.maxZoom && ( numAds > that._maxAdsOnListing || currentZoom < that._minZoomForListing )) {
            if (numAds == 1) {
                that._lastMarkerToOpen = idGH;
            }
            that._labelDiv.hide();

            if (this.MBR != '') {
                var coord = this.MBR.split(','),
                    sw = new google.maps.LatLng(parseFloat(coord[0]), parseFloat(coord[2])),
                    ne = new google.maps.LatLng(parseFloat(coord[1]), parseFloat(coord[3]));

                that.map.fitBounds(new google.maps.LatLngBounds(sw, ne));
            } else {
                var position = this.getPosition();
                var zoom = numAds == 1 ? that._minZoomForListing : Math.max(10, that.getNextGeoHashZoom());

                if (currentZoom == zoom) {
                    zoom++;
                }

                that.map.setZoom(zoom);
                that.map.setCenter(position);
            }

            that.managerHash.setHash({idAd: '', idGH: ''}, null, true);
            that.addHashStatusOnDrag();

            return true;
        }

        if (that.managerHash.hasHash('idGH')) {
            that.managerHash.setHash({idAd: '', idGH: ''}, null, false);
        }

        // se il numAds > 1 allora non devo caricare niente.
        if (numAds > 1) {
            // tolgo il grey agli altri marker e lo aggiungo a dove clicco.
            jq('.clusterMarker.grey').removeClass('grey');
            jq('#cluster_' + idGH).addClass('grey');
            that.getListingAds(idGH, true);
            return true;
        }

        // cambio la classe al marker per portalo in primo piano al click.
        jq('.clusterMarker').removeClass('opened');
        this.labelClass = this.get('labelClass') + ' opened';
        google.maps.event.trigger(this, 'labelclass_changed');
        google.maps.event.trigger(this, 'labelcontent_changed');
        // se numAds = 1 allora devo aprire il dettaglio.
        that.openDetail(null, idGH, numAds, {openedFrom: 'marker'});
    });

    return true;
};


/**
 * Effettua la chiamata ajax per recuperare i dati di un dettaglio.
 * @param idAd
 * @param geohash
 * @param itemsInListing
 * @param opt
 */
ManagerMarkers.prototype.openDetail = function (idAd, geohash, itemsInListing, opt) {
    var _this;
    opt = opt || {};

    jq('.clusterMarker.grey').removeClass('grey');
    jq('#cluster_' + geohash).addClass('grey');

    if (idAd || geohash) {
        jq("#box-listing .riga_annuncio.selected").removeClass('selected');

        if (idAd) {
            jq("#box-listing").find('.riga_annuncio[data-id="' + idAd + '"]').addClass('selected');
        } else if (geohash) {
            jq("#box-listing").find('.riga_annuncio[data-gh="' + geohash + '"]').addClass('selected');
        }
    }

    this._currentIdMarkerOpened = geohash;
    this._detailOpenedFrom = 'marker'; // nel caso venga aperto dalla lista viene sovrascritto dal metodo apposito.
    this.stopOnMouseOver = true;
    this._labelDiv.idAnchor = "#cluster_" + geohash;
    this._labelDiv.setContent('<div id="containerResponseMapAds" class="loading details"></div>');
    this.managerHash.setHash({idAd: idAd, idGH: geohash}, null, false);

    _this = this;

    if (typeof opt.needTimeout !== 'undefined' && opt.needTimeout) {
        setTimeout(function () {
            _this._labelDiv.show();
        }, 100);
    } else {
        this._labelDiv.show();
    }

    var $deferred = this.managerRequest.openDetail(idAd, geohash);

    $deferred.done(function (html) {
        _this._labelDiv.setContent(html);
        _this._labelDiv.show();
        _this.eventOnDetail(itemsInListing, opt);
    });
};

/**
 * Al caricamento dell'html aggiunge gli eventi riferiti al dettaglio.
 * @param itemsInListing = quanti annunci sono raggruppati.
 */
ManagerMarkers.prototype.eventOnDetail = function (itemsInListing, opt) {
    opt = opt || {};
    this._detailOpenedFrom = opt.openedFrom || 'marker';
    trackGAClickEvent('ricercaMappaEvents', this._detailOpenedFrom, 'apertura');
    this.managerRequest.eventOnDetail();

    if (itemsInListing > 1) {
        this.eventOnGoBackDetails();
    }
};


/**
 * Aggiunge l'evento al pulsante "lista" per ricaricarne l'html nel box.
 */
ManagerMarkers.prototype.eventOnGoBackDetails = function () {
    var that = this,
        $closeElem = jq("#responseMapAds .close-infowindow");

    $closeElem.unbind('click');
    $closeElem.one('click', function () {
        that._labelDiv.setContent(that._lastHtmlListOpened);
        jq("#wrapperResponse .title").show();
        that.actionOnListing();
    });
};


/**
 * Una volta caricato l'html del listing eseguo alcune azioni.
 * @param idMarker = id del marker a cui e' associato l'infowindow
 * @param itemsInListing = quanti annunci sono raggruppati.
 */
ManagerMarkers.prototype.actionOnListing = function () {
    jq("#responseMapAds.listingAds").mCustomScrollbar({scrollInertia: 0, theme: 'dark-3'});
    this.managerRequest.eventOnListing();
};


/**
 * Calcola il livello di zoom successivo necessario a far cambiare il geohash.
 * @returns {integer} = zoom level
 */
ManagerMarkers.prototype.getNextGeoHashZoom = function () {
    var i, currentZoom;
    currentZoom = this.map.getZoom();
    var zoomLevel = this.isMobile ? this._zoomLevelsMobile : this._zoomLevels;
    for (i in zoomLevel) {
        if (currentZoom < zoomLevel[i]) {
            return zoomLevel[i] + 1;
        }
    }
    return currentZoom + 2;
};


/**
 * Metodo che viene lanciato all'evento "idle" di google maps.
 */
ManagerMarkers.prototype.doAjax = function () {
    jq('#pag').val('1');
    this.lastCenter = this.map.getCenter();
    this.getMultiResponseGrid();
    this.getListingResponse();
    this.setUrlHash();
};


/**
 * Metodo che viene lanciato per ricaricare sia la mappa che il listing
 */
ManagerMarkers.prototype.reloadAll = function () {
    this.bounds = null;
    this.geohashLen = null;
    this.getMultiResponseGrid();
    this.getListingResponse();
    this.setUrlHash();
};


/**
 * Costruisce un oggetto con i parametri base della ricerca con ajax
 * @returns {Object}
 */
ManagerMarkers.prototype.getBaseParamsQS = function () {
    var params = this.refineSearch.serializeObj();
    params.checkNewDate = 1;
    params.disableNazione = 1;
    params.isMobile = this.isMobile;
    return params;
};


/**
 * Costruisce un oggetto con i parametri da passare alla chiamata ajax della ricerca
 * @param withPage = boolean
 * @returns {Object}
 */
ManagerMarkers.prototype.getParamsQS = function (withPage) {
    var bounds, rettangoli, params, ne, sw;
    params = this.getBaseParamsQS();
    bounds = this.map.getBounds();
    // disabilitato il doppio poligono
    if (this.bounds !== null) {
        rettangoli = this.mapExt.f_rettangoli(this.bounds, bounds);
        params.minLat0 = rettangoli[0].getSouthWest().lat();
        params.maxLat0 = rettangoli[0].getNorthEast().lat();
        params.minLng0 = rettangoli[0].getSouthWest().lng();
        params.maxLng0 = rettangoli[0].getNorthEast().lng();
        params.minLat1 = rettangoli[1].getSouthWest().lat();
        params.maxLat1 = rettangoli[1].getNorthEast().lat();
        params.minLng1 = rettangoli[1].getSouthWest().lng();
        params.maxLng1 = rettangoli[1].getNorthEast().lng();
    }
    ne = bounds.getNorthEast();
    sw = bounds.getSouthWest();
    params.minLat = sw.lat();
    params.maxLat = ne.lat();
    params.minLng = sw.lng();
    params.maxLng = ne.lng();
    params.zoom = this.map.getZoom();
    if (withPage) {
        params.pag = jq("#pag").val();
    }
    // sovrascrivo l'ultimo bounds
    this.bounds = this.map.getBounds();
    // Filtro che seleziona agenzia/privato
    if (!!this.options.filterPublisher) {
        params.filterPublisher = this.options.filterPublisher;
    }
    return params;
};


/**
 * Setta l'url della pagina
 */
ManagerMarkers.prototype.setUrlHash = function () {
    var zoom = this.map.getZoom();
    var pag = (typeof jq("#pag").val() != 'undefined') ? jq("#pag").val() : '';
    var center = this.map.getCenter();
    var objRefine = this.managerHash.getObjFromRefineSearch(this.refineSearch);
    objRefine.zoom = zoom;
    objRefine.pag = pag;
    objRefine.latitudine = parseFloat(center.lat()).toFixed(5);
    objRefine.longitudine = parseFloat(center.lng()).toFixed(5);
    this.managerHash.setHash(objRefine, null, false);
};


/**
 * Costruisce un oggetto da inviarea alla chiamata ajax per i markers
 * @param idGeohash
 * @returns {Object}
 */
ManagerMarkers.prototype.getAjaxParamsGrid = function (idGeohash) {
    var params = {};

    if (typeof idGeohash !== 'undefined' && idGeohash) {
        params = this.getBaseParamsQS();
        params.onlyListing = 1;
        params.idGH = idGeohash;
    } else {
        params = this.getParamsQS(false);
    }

    return params;
};


/**
 * Costruisce un oggetto da inviarea alla chiamata ajax per il listing
 * @returns {Object}
 */
ManagerMarkers.prototype.getAjaxParamsPageListing = function () {
    this.bounds = null;
    return this.getParamsQS(true);
};


/**
 * Gestisce la chiamata ajax per disegnare i markers sulla mappa
 */
ManagerMarkers.prototype.getMultiResponseGrid = function () {
    var _this, timeoutRefresh, dataPost;

    this._log.doLog('getMultiResponseGrid', 'MultiResponseGrid', 1);
    _this = this;

    timeoutRefresh = setTimeout(function () {
        jq('#box-refresh-map').fadeIn(500);
    }, 300);

    dataPost = this.getAjaxParamsGrid(null);

    // IMPORTANTE: Nel caso di mobile per liberare memoria se cambia lo zoom elimino i markers presenti sulla mappa.
    if (this.isMobile && this.zoom != this.map.getZoom()) {
        this.deleteMarkers(0);
    }

    var $deferred = this.managerRequest.getMultiResponseGrid(dataPost);

    $deferred
        .done(function (data) {
            clearTimeout(timeoutRefresh);
            _this._log.doLog('getMultiResponseGrid', 'MultiResponseGrid', 0);
            jq('#box-refresh-map').fadeOut(500);

            if (data.map) {
                var statusGeohash, zoom, currentGeohashLen, clusterOptions;
                zoom = data.zoom; 		// prende lo zoom della chiamata.
                currentGeohashLen = data.hashlen;
                statusGeohash = _this.getStatusGeohash(currentGeohashLen);

                // se lo zoom della chiamata e' diverso da quello attuale esco
                if (zoom != _this.map.getZoom()) {
                    return;
                }

                clusterOptions = _this.getClusterOptions(data);
                _this.eventToDrawMarkers(clusterOptions, currentGeohashLen, statusGeohash);
                _this.zoom = _this.map.getZoom(); 		// salvo il nuovo livello di zoom
                _this.geohashLen = currentGeohashLen; 	// salvo il nuovo livello di geohash
                jq('#img-loader').fadeOut();
            }

            if (_this._jsonStreamEnabled && _this.map.getZoom() >= _this._zoomStartJsonStream) {
                _this.startJsonStream(dataPost);
            }
        })
        .fail(function () {
            jq('#box-refresh-map').hide();
        });
};


ManagerMarkers.prototype.startJsonStream = function (dataPost) {
    var _this = this,
        urlRequest = this.managerRequest.urlGetMapMarkers + '&' + jq.param(dataPost),
        _jStream = _jsonStreamFactory.createStream({url: urlRequest + "&onlyListing=1"});

    _jStream.done(function (json) {
        if (!json.listing || !json.listing.length) {
            return;
        }

        for (var i = 0; i < json.listing.length; i++) {
            var ad = json.listing[i],
                geohash = ad.gh,
                idDoc = ad.id;

            if (typeof _this.markers[geohash] === 'undefined') {
                continue;
            }
            if (typeof _this.markers[geohash].ifexist[idDoc] === 'undefined') {
                _this.markers[geohash].ifexist[idDoc] = 1;
                _this.markers[geohash].details.push(ad);
            }
        }
    });
};

ManagerMarkers.prototype.getListingResponse = function () {
    // controlla se ci sono altre richieste dei marker eventualmente le abortisce.
    this._log.doLog('getListingResponse', 'ListingResponse', 1);

    var _this = this,
        $boxListing = jq('#box-listing'),
        $boxRefresh = jq('#box-refresh-listing'),
        textNoAds = 'Non ci sono ' +
            (this.options.searchType === 'agenzia' ? 'agenzie' : 'annunci' ) +
            ' in quest\'area, spostati sulla mappa o modifica i filtri di ricerca.';

    $boxRefresh.show();
    $boxListing.show()
        .addClass('refresh');

    var $deferred = this.managerRequest.getListingResponse(this.getAjaxParamsPageListing());

    $deferred
        .always(function () {
            $boxListing.delay(300).removeClass('refresh');
            $boxRefresh.delay(300).hide();
        })
        .done(function (data) {
            _this._log.doLog('getListingResponse', 'ListingResponse', 0);

            if (data.count) {
                jq('#box-pagination').show();
                jq('#info-listing').html(( data.offset + 1 ) + ' - ' + ( parseInt(data.count) + parseInt(data.offset) ) + ' su <span>' + data.countTot + '</span>');
                jq('#prev-listing').attr('data-page', data.prevPage).toggleClass('disable', data.prevPage == 0);
                jq('#next-listing').attr('data-page', data.nextPage).toggleClass('disable', data.nextPage == 0);
            } else {
                jq('#box-pagination').hide();
                jq('#info-listing').html('');
            }

            if (data.html !== '') {
                $boxListing.html(data.html);
            } else {
                $boxListing.html('<div class="noAds">' + textNoAds + '</div>');
            }
        })
        .fail(function () {
            jq('#box-pagination').hide();
            jq('#info-listing').html('');
            $boxListing.html('<div class="noAds">' + textNoAds + '</div>');
        }).then(function () {
            var $container = jq('#container-listing');

            $container.mCustomScrollbar('update');
            $container.mCustomScrollbar("scrollTo", "top");
        });
};


/**
 * Data la risposta ajax dei markers genero un oggetto con le opzioni dei cluster
 * @param data = risposta ajax dei markers
 * @returns {Array}
 */
ManagerMarkers.prototype.getClusterOptions = function (data) {
    var clusterOptions, cluster, parent, flagNew, options;

    clusterOptions = {};

    for (var idGH in data.map) {
        cluster = data.map[idGH];
        parent = idGH.substr(0, this.geohashLen);
        flagNew = cluster.checkIsNew;
        options = {};
        options.count = cluster.count;
        options.MBR = cluster.MBR;
        options.point = new google.maps.LatLng(parseFloat(cluster.lat_mean), parseFloat(cluster.lon_mean));

        if (typeof this.markers[parent] !== 'undefined') {
            options.startPosition = this.markers[parent].getPosition();
        }

        options.labelContent = options.count.formatNumber(0, ',', '.');
        options.labelID = 'cluster_' + idGH;

        if (options.count < 50) {
            options.labelClass = 'clusterMarker clusterMarker1';
            options.pointX = 15;
            options.pointY = 15;
        } else if (options.count < 500) {
            options.labelClass = 'clusterMarker clusterMarker2';
            options.pointX = 17;
            options.pointY = 17;
        } else if (options.count < 2000) {
            options.labelClass = 'clusterMarker clusterMarker3';
            options.pointX = 23;
            options.pointY = 23;
        } else {
            options.labelClass = 'clusterMarker clusterMarker4';
            options.pointX = 28;
            options.pointY = 28;
        }

        clusterOptions[idGH] = options;
    }

    return clusterOptions;
};


/**
 *  Resetta le variabile una nuova ricerca su mappa.
 *  Inoltre può generare un evento per ricaricare la mappa.
 * @param needReload
 */
ManagerMarkers.prototype.resetParamsToReloadMap = function (needReload) {
    needReload = needReload || false;
    this.bounds = null;
    this.geohashLen = null;

    if (needReload) {
        google.maps.event.trigger(this.map, 'idle');
    }
};


/**
 * @param currentGeohashLen
 * @return string = status a seconda della differenza di geohash
 */
ManagerMarkers.prototype.getStatusGeohash = function (currentGeohashLen) {
    if (this.geohashLen === null) {
        return 'reset';
    }

    var diffHash, statusGeohash;

    diffHash = currentGeohashLen - this.geohashLen;

    if (diffHash == 0) {
        statusGeohash = 'same';
    } else if (diffHash > 0) {
        statusGeohash = 'zoomin';
    } else {
        statusGeohash = 'zoomout';
    }

    return statusGeohash;
};


/**
 * Date le opzioni dei diversi cluster disegna i markers sulla mappa
 * @param clusterOptions
 * @param currentGeohashLen
 * @param statusGeohash
 */
ManagerMarkers.prototype.eventToDrawMarkers = function (clusterOptions, currentGeohashLen, statusGeohash) {
    var _this = this;

    // se e' mobile sposto i markers senza effetti grafici.
    if (this.isMobile) {
        if (statusGeohash === 'zoomout' || statusGeohash === 'zoomin' || statusGeohash === 'reset') {
            this.deleteMarkers(0);
        } else if (statusGeohash === 'same') {
            _this.checkVisibilityMarkers();
        }

        for (var index in clusterOptions) {
            clusterOptions[index].goToEnabled = false;
            this.drawMarker(index, clusterOptions[index]);
        }

        return;
    }

    if (statusGeohash === 'reset') {
        this.deleteMarkers(0);

        for (var index in clusterOptions) {
            this.drawMarker(index, clusterOptions[index]);
        }
    } else if (statusGeohash === 'zoomout') {
        for (var key in _this.markers) {
            var parent = key.substr(0, currentGeohashLen);

            if (typeof clusterOptions[parent] != 'undefined') {
                this.markers[key].label.goTo(clusterOptions[parent].point);
            }
        }

        for (var index in clusterOptions) {
            this.drawMarker(index, clusterOptions[index]);
        }

        this.changeZIndexMarkers(currentGeohashLen);

        // cancello i marker con una geohashlen diversa
        setTimeout(function () {
            _this.deleteMarkers(currentGeohashLen);
        }, 600);
    } else {
        if (statusGeohash === 'zoomin') {
            this.changeZIndexMarkers(_this.geohashLen);
        }

        for (var index in clusterOptions) {
            this.drawMarker(index, clusterOptions[index]);
        }

        setTimeout(function () {
            _this.markersPanTo();
            _this.deleteMarkers(currentGeohashLen);

            if (statusGeohash === 'same') {
                _this.checkVisibilityMarkers();
            }
        }, 600);
    }
};


ManagerMarkers.prototype.listingOnMouseOver = function (obj) {
    var _this = this;

    this._timeoutMouseOverListing = setTimeout(function () {
        var $elem = jq(obj),
            idGeohash = $elem.attr('data-gh');

        jq('.clusterMarker.red').removeClass('red');
        jq('#cluster_' + idGeohash).addClass('red');

        if (!_this.stopOnMouseOver) {
            _this.getPreviewOnMap($elem.attr('data-id'), idGeohash);
        }
    }, 300);
};


ManagerMarkers.prototype.listingOnMouseOut = function () {
    clearTimeout(this._timeoutMouseOverListing);

    jq('.riga_annuncio.hover').removeClass('hover');
    jq('.clusterMarker.red').removeClass('red');

    if (!this.stopOnMouseOver) {
        this._labelDiv.hide();
        this._currentIdMarkerOpened = null;
    }
};


ManagerMarkers.prototype.listingOnClick = function (obj) {
    var $elem = jq(obj);

    if ($elem.hasClass('selected')) {
        $elem.removeClass('selected');
        this._labelDiv.hide();
        jq('.clusterMarker.grey').removeClass('grey');

        return;
    }

    this.openDetail($elem.attr('data-id'), $elem.attr('data-gh'), 1, {openedFrom: 'lista'});
    this.stopOnMouseOver = true;
};


ManagerMarkers.prototype.markerOnMouseOver = function (obj) {
    if (this.stopOnMouseOver) {
        return;
    }

    clearTimeout(timeoutCloseLabelDiv);
    var _this = this;

    this._timeoutMouseOverListing = setTimeout(function () {
        var idGeohash = jq(obj).attr('id').replace('cluster_', '');
        _this.getPreviewOnMarker(idGeohash, false);
    }, 300);
};


ManagerMarkers.prototype.markerOnMouseOut = function () {
    if (this.stopOnMouseOver) {
        return;
    }

    var _this = this;
    clearTimeout(this._timeoutMouseOverListing);

    timeoutCloseLabelDiv = setTimeout(function () {
        _this._labelDiv.hide();
        _this._currentIdMarkerOpened = null;
    }, 300);
};


ManagerMarkers.prototype.appendListOnFrame = function (idGeohash, html) {
    if (html === '') {
        return false;
    }

    this.setContentLabel(idGeohash, html, 'listingAds', 'Lista annunci');
    this.actionOnListing();

    return true;
};


ManagerMarkers.prototype.setContentLabel = function (idGeohash, html, className, boxTitle) {
    if (!this._labelDiv || typeof this._labelDiv.idAnchor === 'undefined') {
        return;
    }

    this._labelDiv.idAnchor = "#cluster_" + idGeohash;
    this._labelDiv.show();
    this._labelDiv.setContent(
        jq(
            '<div id="containerResponseMapAds" class="' + className + '">' +
            '<div id="wrapperResponse">' +
            '<div class="title" style="display:none;">' + boxTitle + '<div class="pull-right close-infowindow">&times;</div></div>' +
            '<div id="responseMapAds" class="' + className + '">' +
            html +
            '</div>' +
            '</div>' +
            '</div>')
    );
    jq('#wrapperResponse').show();
    var that = this;
    setTimeout(function () {
        that._labelDiv.setPosition();
    }, 100);
};


ManagerMarkers.prototype.getPreviewOnMap = function (idAnnuncio, idGeohash) {
    if (this._currentIdMarkerOpened == idGeohash) {
        return true;
    }
    this._currentIdMarkerOpened = idGeohash;
    var list = '';
    jq("#box-listing .riga_annuncio[data-id='" + idAnnuncio + "']").each(function () {
        list += jq(this)[0].outerHTML;
    });
    this.appendListOnFrame(idGeohash, list);
};


ManagerMarkers.prototype.getPreviewOnMarker = function (idGeohash, isFromClick) {
    if (this._currentIdMarkerOpened == idGeohash) {
        return true;
    }

    this._currentIdMarkerOpened = idGeohash;
    var list = '';
    var currentZoom = this.map.getZoom();

    if (isFromClick &&
        currentZoom >= this._zoomStartJsonStream &&
        (typeof this.markers[idGeohash].details === 'undefined' || this.markers[idGeohash].details.length <= 0)
    ) {
        // chiamata ajax SINCRONA che recupera i details di una geohash
        this.getListingAdsFromGeoHash(idGeohash);
        return;
    }

    if (typeof this.markers[idGeohash].details == 'undefined' || this.markers[idGeohash].details.length <= 0) {
        this._currentIdMarkerOpened = null;
        return;
    }

    for (var i in this.markers[idGeohash].details) {
        list += this.managerRequest.getHtmlListing(this.markers[idGeohash].details[i]);
    }

    if (list != '') {
        this.appendListOnFrame(idGeohash, list);
    }
};


ManagerMarkers.prototype.getListingAds = function (idGeohash, isFromClick) {
    isFromClick = isFromClick || false;

    if (isFromClick) {
        jq("#wrapperResponse .title").show();
    }

    this.getPreviewOnMarker(idGeohash, isFromClick);

    if (isFromClick) {
        jq("#wrapperResponse .title").show();
    }

    return true;
};


/**
 * Recupera il listing degli annunci da mostrare direttamente nel box
 * SENZA SALVARE i dettagli nel details del marker ma stampando SOLO l'html.
 * @param idGeohash
 */
ManagerMarkers.prototype.getListingAdsFromGeoHash = function (idGeohash) {
    // se e' abilitato il json stream allora non serve questa funzionalita
    if (this._jsonStreamEnabled) {
        return;
    }

    this._log.doLog('getListingAdsFromGeoHash', 'ListingAdsFromGeoHash', 1);
    this.setContentLabel(idGeohash, '', 'loading listingAds', 'Lista annunci');

    var _this = this,
        params = this.getAjaxParamsGrid(idGeohash),
        $deferred = this.managerRequest.getListingAdsFromGeoHash(params);

    $deferred
        .done(function (html) {
            _this._log.doLog('getListingAdsFromGeoHash', 'ListingAdsFromGeoHash', 0);

            if (html !== '') {
                _this.appendListOnFrame(idGeohash, html);
                jq('#wrapperResponse .title').show();
            } else {
                _this._labelDiv.hide();
            }
        }).fail(function () {
            _this._labelDiv.hide();
        });
};


/**
 * Aggiunge un evento che salva lo stato della hash quando avviene un drag.
 */
ManagerMarkers.prototype.addHashStatusOnDrag = function () {
    var that = this;
    google.maps.event.addListenerOnce(this.map, 'dragstart', function () {
        that.managerHash.pushState(window.location.hash.replace('#', ''));
    });
};


/**
 * Questo metodo prende i valori dalla hash e ricarica la mappa con questi parametri.
 */
ManagerMarkers.prototype.updateMapFromHash = function () {
    var lat, lng, zoom, currentZoom, currentCenter, center;
    lat = this.managerHash.getHash('latitudine');
    lng = this.managerHash.getHash('longitudine');
    zoom = this.managerHash.getHash('zoom');
    this.refineSearch.fillByHash();
    currentZoom = this.map.getZoom();
    currentCenter = this.map.getCenter();

    if (zoom && currentZoom != zoom) {
        this.map.setZoom(parseInt(zoom));
    }
    if (lat && lng) {
        center = new google.maps.LatLng(parseFloat(lat), parseFloat(lng));
        this.map.setCenter(center);
    }
    // se non e' stato triggerato un evento "idle" allora forzo il caricamento della mappa.
    if (currentZoom == zoom && center.lat() == currentCenter.lat() && center.lng() == currentCenter.lng()) {
        this.resetParamsToReloadMap(true);
    }
};


/**
 * Se la gestione dell'hash riconosce un onBack allora aggiorna la mappa
 * @returns {Function}
 */
ManagerMarkers.prototype.onBack = function () {
    var that = this;
    return function () {
        that.updateMapFromHash();
    }
};


/**
 * Se la gestione dell'hash riconosce un onForward allora aggiorna la mappa
 * @returns {Function}
 */
ManagerMarkers.prototype.onForward = function () {
    var that = this;
    return function () {
        that.updateMapFromHash();
    }
};


/**
 * Se la gestione degli stati riconosce una variazione dell'hash allora aggiorna la mappa
 * @returns {Function}
 */
ManagerMarkers.prototype.onPopState = function () {
    var that = this;
    return function (event) {
        that.updateMapFromHash();
    }
};
;
var managerMarkers, drawer, negativeLayer, sendResponsabileMail, __gvs_isMobile, managerUser, _bookmarkAnnuncio, actionsOnMenuHeader, eventsOnHashChange, blacklistAnnunci;
var __section__ = 'ricerca-mappa';

var minZoom = 5;
var maxZoom = 18;

jq(document).ready(function(){
	try{
		__gvs_isMobile = (typeof isMobile != "undefined") && (typeof isMobile.any == "function") ? isMobile.any() : false;
	}catch(e){
		__gvs_isMobile = false;
	}
	// Carica il tema classico della galleria
	Galleria.loadTheme(__gvs_ULR_HOST + '/includes/galleria.io/themes/map/galleria.map.js');

	managerUser = new ManageUser({page: 'ricerca-mappa'});
	managerUser.getUserInfo();
	initMapSearch();

	blacklistAnnunci = new ManagerBlacklistAnnunci({'section': __section__, 'typePageListingMap': true, 'managerMarkers': managerMarkers, 'managerUser': managerUser});

	initDomElements();
	initGMapButton();
	placeHolderCompatibility();
	initDrawer();

	/* Gestisce il "cosa ne pensi" */
	showCheNePensiRicercaPoligono(3, {positionFixed : 1});

	(function($){
		$("#box-go-list").mCustomScrollbar({scrollInertia: 0, theme: 'dark-3'});
		$("#container-listing").mCustomScrollbar({scrollInertia: 0, theme: 'dark-3'});
	})(jQuery);
});

function placeHolderCompatibility() {
	var elements = ['contInputSearch', 'box_accedi_form'];
	for( var i in elements ) {
		addPlaceHolderCompatibility(elements[i]);
	}
}

function initMapSearch() {
	var params, zoom, lat, lng, mapOptions, managerHash, needFitBound;
	managerHash = new ManagerHash();
	managerHash.convertHashStrToObj();
	params 	= managerHash.hashObj;
	needFitBound = ( typeof params.zoom == 'undefined' );
	zoom 	= parseFloat( params.zoom ) || 6;
	lat 	= parseFloat( params.latitudine ) || 42.397;
	lng 	= parseFloat( params.longitudine ) || 11.644;
	mapOptions = {
		scrollwheel : true,
		zoom: zoom,
		center: new google.maps.LatLng(lat, lng),
		mapTypeId: google.maps.MapTypeId.ROADMAP,
		streetViewControl: false,
		panControl: false,
		zoomControl: false,
		mapTypeControl: false
	};
	mapOptions.onlyNew 	= typeof params.onlyNew != 'undefined' ? params.onlyNew : '0';
	mapOptions.onlyTop 	= typeof params.onlyTop != 'undefined' ? params.onlyTop : '0';
	mapOptions.pag 			= typeof params.pag != 'undefined' ? params.pag : '1';
	mapOptions.filterPublisher = params.filterPublisher || null;
	mapOptions.maxZoom = maxZoom;
	mapOptions.minZoom = minZoom;
	mapOptions.isMobile = __gvs_isMobile;
	mapOptions.jsonStreamEnabled = __gvs_isMobile ? false : true;
	mapOptions.searchType = (window.location.pathname).search('/ricerca-mappa-agenzia') !== -1 ? 'agenzia' : 'annuncio';
	managerMarkers = new ManagerMarkers( '#map-ads', mapOptions );
	managerMarkers.init();

	// abilita le azioni nel caso in cui cambi l'hash.
	eventsOnHashChange.init(managerMarkers.managerHash);

	// abilita il movimento fluido della mappa
	var kineticScrolling = new KineticScrolling();
	kineticScrolling.setMap(managerMarkers.map);
}

function initDrawer() {
	var managerHash, params, needFitBound;

	drawer = new drawer({ parent: document });
	drawer.boxMappa = '#map-ads';
	drawer.setMap(managerMarkers.map);

	managerHash = new ManagerHash();
	managerHash.convertHashStrToObj();
	params 	= managerHash.hashObj;
	needFitBound = ( typeof params.zoom == 'undefined' );

	negativeLayer = new NegativeLayer(managerMarkers.map);
	if (__geoInfo.scope != '' && __geoInfo.code != '') {
		caricaPoligonoRicerca(__geoInfo.scope, __geoInfo.code, needFitBound)
	} else {
		MapHelper.appendLayerArea(managerMarkers.map, drawer.getSearchArea());
	}

	return;
}

/**
 * Effettua la chiamata per geolocalizzare l'indirizzo
 * @param address
 */
function resolveAddress( address ) {
	var geocoder = new google.maps.Geocoder();
	geocoder.geocode( { 'address': address, 'language': 'it'}, function(results, status) {
		if ( status == google.maps.GeocoderStatus.OK && status != google.maps.GeocoderStatus.ZERO_RESULTS ) {
			jq('#input-search').removeClass('input-error');
			var aResults = new Array();
			for ( var i = 0; i < results.length; i++ ) {
				aResults[i] = new Array();
				aResults[i]['formatted_address'] = results[i]['formatted_address'];
				aResults[i]['type'] = results[i]['types'][0];
				aResults[i]['lat'] = results[i].geometry.location.lat();
				aResults[i]['lng'] = results[i].geometry.location.lng();
			}
			managerMarkers.map.setCenter(new google.maps.LatLng(aResults[0]['lat'], aResults[0]['lng']));
			if ( aResults[0]['type'] == 'route' ) {
				managerMarkers.map.setZoom( 15 );
				jq('#input-search').val( aResults[0]['formatted_address'] );
			} else {
				managerMarkers.map.setZoom( 13 );
				jq('#input-search').val( aResults[0]['formatted_address'] );
			}
		}
		if ( status == google.maps.GeocoderStatus.ZERO_RESULTS ) {
			jq('#input-search').addClass('input-error');
		}
	});
}

/**
 * Inizializza gli eventi sui pulsanti della mappa
 */
function initGMapButton() {
	jq("#zoom_in_box_mappa").on('click', function() {
		if ( managerMarkers.map.getZoom() < maxZoom )
			managerMarkers.map.setZoom( managerMarkers.map.getZoom() + 1 );
	});
	jq("#zoom_out_box_mappa").on('click', function(){
		if ( managerMarkers.map.getZoom() > minZoom )
			managerMarkers.map.setZoom( managerMarkers.map.getZoom() - 1 );
	});
	jq(".maptype").on('click', function() {
		if ( jq(this).hasClass('active') )
			return;
		jq(".maptype").removeClass('selected');
		jq(this).addClass('selected');
		if ( jq(this).attr('data-maptype') == 'SATELLITE' )
			managerMarkers.map.setMapTypeId( google.maps.MapTypeId.SATELLITE );
		if ( jq(this).attr('data-maptype') == 'ROADMAP' )
			managerMarkers.map.setMapTypeId( google.maps.MapTypeId.ROADMAP );
	});
}
/**
 * Inizializza gli eventi sul DOM della pagina
 */
function initDomElements() {
	/* Setto l'altezza degli elementi della pagina */
	setElementsDimension();
	setElementsHeight();
	// Resize della pagina
	jq( window ).resize( function(){
		google.maps.event.trigger( managerMarkers.map, 'resize' ); // triggero l'evento on resize della mappa
		setElementsDimension();
		setElementsHeight();
	});
	/* Gestisce gli eventi sull'input del luogo */
	jq('#input-search')
		.on('keypress',function (e) {
			if (e.keyCode == 13) {
				resolveAddress(jq("#input-search").val());
				jq(this).blur();
			}
		}).on('keyup', function (e) {
			if (jq(this).val() != '')
				jq('#delete-text').show();
			else
				jq('#delete-text').hide();
		});
	jq("#delete-text").on('click', function () {
		jq('#input-search').val('').blur().focus();
		jq(this).hide();
	});
	jq('#btn-input-search').on('click', function(){
		if (jq(this).hasClass('pure-button-disabled') )
			return;
		resolveAddress(jq("#input-search").val());
		jq(this).blur();
	});

	// inizializzo gli eventi sul menu header.
	actionsOnMenuHeader.initEvents();
}

/**
 * Valida il form contatti gestendo anche la visualizzazione di eventuali errori
 * @param idForm = idJQuery del form.
 * @returns {boolean} = true => form valido
 */
function newValidateFormContatti(idForm)
{
	var formObj, errore;
	formObj = jq(idForm)[0];
	formObj.nome.value = formObj.nome.value.replace(/obbligatorio/, '');
	formObj.nome.value = formObj.nome.value.replace(/^\s+|\s+$/, '');
	formObj.telefono.value = formObj.telefono.value.replace(/obbligatorio tel. o email/, '');
	formObj.telefono.value = formObj.telefono.value.replace(/^\s+|\s+$/, '');
	formObj.email.value = formObj.email.value.replace(/obbligatorio tel. o email/, '');
	formObj.email.value = formObj.email.value.replace(/^\s+|\s+$/, '');

	errore = {};
	if (formObj.nome.value == "" || formObj.nome.value == "Il tuo nome") {
		errore.nome = "Inserisci il nome";
	} else if (!jq.isEmpty(jq('#ti_utente_nome'))) {
		jq('#ti_utente_nome').val(formObj.nome.value);
	}

	if (formObj.source.value != "beez" && (formObj.email.value == "" || formObj.email.value == "La tua email") && (formObj.telefono.value == "" || formObj.telefono.value == "Il tuo telefono"))
		errore.email = "È obbligatorio indicare almeno uno tra email e telefono";

	if (formObj.email.value != "" && formObj.email.value != "La tua email" && !checkEmail(formObj.email.value)) {
		errore.email = "Inserisci una email valida";
	} else if (!jq.isEmpty(jq('#ti_utente_mail'))) {
		jq('#ti_utente_mail').val(formObj.email.value);
	}

	if (formObj.source.value == "beez") {
		if (formObj.telefono.value == "" || formObj.telefono.value == "Il tuo telefono")
			errore.telefono = "Inserisci il telefono";

		if (formObj.email.value == "" || formObj.email.value == "La tua email")
			errore.email = "È obbligatorio indicare l'email";
	}

	if (formObj.telefono.value != "" && formObj.telefono.value != "Il tuo telefono" && !/^(\+)?[\s\d()\-\/.]+$/.test(formObj.telefono.value)) {
		errore.telefono = "Inserisci un telefono valido";
	} else if (formObj.telefono.value != "" && !jq.isEmpty(jq('#ti_utente_telefono'))) {
		jq('#ti_utente_telefono').val(formObj.telefono.value);
	}

	if (!formObj.privacy.checked) {
		errore.privacy = "È necessario il consenso all'informativa Privacy";
	}

	jq(idForm + ' input').removeClass('input-error');
	jq(idForm + ' .erroriFormContatti').removeClass('opened');
	var validForm = true;

	// visto che nome e telefono sono sulla stessa riga allora se apro un errore devo aprire anche l'altro
	// per non disallineare i form.
	if (typeof errore.nome == 'undefined' && typeof errore.telefono != 'undefined')
		errore.nome = '&nbsp;';
	if (typeof errore.telefono == 'undefined' && typeof errore.nome != 'undefined')
		errore.telefono = '&nbsp;';

	// evidenzio form non valido e mostro eventuali errori.
	if (!jq.isEmpty(errore)) {
		for (var name in errore) {
			if(errore[name] != '' && errore[name] != '&nbsp;')
				jq(idForm + ' input[name="'+name+'"]').addClass('input-error');
			jq('#errore_' + name).html(errore[name]).addClass('opened');
		}
		validForm = false;
	}
	if (typeof errore.nome != 'undefined' || typeof errore.telefono != 'undefined') {
		errore.nome = errore.telefono = '';
	}
	jq(idForm + ' .erroriFormContatti').each(function(){
		if(jq(this).hasClass('opened'))
			jq(this).slideDown();
		else
			jq(this).slideUp();
	});
	return validForm;
}


/**
 * Setta l'altezza degli elementi dinamici della mappa
 */
function setElementsHeight() {
	var heightBoxMappa = jq(window).outerHeight( true ) - jq("#header").outerHeight( true );
	jq("#box-map-ads").height( heightBoxMappa );
	jq("#box-tab-page").height( heightBoxMappa );
	jq("#imm-tab-content-menu-mappa").height( heightBoxMappa - jq("#imm-tab-menu-mappa").outerHeight(true));
}

/**
 * Dati scope e code carica il poligono dell'area geografica scelta
 * @param scope
 * @param code
 * @param fitBound = adatta l'area alla mappa
 */
function caricaPoligonoRicerca( scope, code, fitBound ) {
	var _data = { scope: scope, code: code };
	var nextZoom = null;
	if ( scope == 'comune' ) {
		nextZoom = 13;
	} else if ( scope == 'provincia' ) {
		nextZoom = 10;
	} else if ( scope == 'zona' ) {
		_data = { scope: scope, code: code, idComune: jq('#idComune').val() }
	}
	jq.ajax({
		dataType: 'json',
		url: '/services/gis/getPolygons.php',
		data: _data
	})
		.done( function( data ) {
			if ( data && typeof data[scope] != 'undefined' ) {
				negativeLayer.setNegativeLayer( data[scope] );
				var area = drawer.getSearchArea();
				if ( area )
					negativeLayer.appendNegativeLayerWithArea( area );
				else
					negativeLayer.appendNegativeLayer( fitBound );
			} else if ( nextZoom ) {
				managerMarkers.map.setZoom( nextZoom );
			}
		});
}

/**
 * Esegue il salvataggio o la cancellazione dell'annuncio tra i preferiti
 * @param elem = elemento su cui avvieni l'onclick
 * @param opt = array con opzioni
 * @return {boolean}
 */
function bookmarkAnnuncioSuMappa(elem, opt)
{
	var jElem, action, idAnnuncio, section, isLoading;
	jElem = jq(elem);

	idAnnuncio = !jq.isEmpty(jElem.data('id')) ? jElem.data('id') : null;
	action = !jq.isEmpty(jElem.attr('data-action')) ? jElem.attr('data-action') : null;
	isLoading = !jq.isEmpty(jElem.attr('data-loading')) ? jElem.attr('data-loading') : false;
	if (isLoading) {
		return false;
	}
	if (!idAnnuncio || !action) {
		throw new Error("Bookmark: id and action are required.");
		return false;
	}
	section = __section__;
	if ( typeof opt != 'undefined' ) {
		section = typeof opt.section != 'undefined' ? opt.section : __section__;
	}
	var bookmarkOptions = {};
	if(!jq.isEmpty(jElem.attr('data-inAffitto'))){
		bookmarkOptions.inAffitto = 1;
	}
	if(!jq.isEmpty(jElem.attr('data-inVendita'))){
		bookmarkOptions.inVendita = 1;
	}
	if (!managerUser.isLogged) {
		//se non loggato creo il cookie e apro il box del login
		_bookmarkAnnuncio.onLogin(idAnnuncio, bookmarkOptions);
	} else {
		_bookmarkAnnuncio.doAction(action, idAnnuncio, managerUser.isLogged, section, false, bookmarkOptions);
	}
	return true;
}

/**
 * CLASSE BOOKMARK ANNUNCIO
 */

(function($){
	/**
	 * Classe astratta bookmarkAnnuncio
	 * @returns {{doAction: _doAction}}
	 * @constructor
	 */
	var BookmarkAnnuncio = function()
	{
		/**
		 * ATTRIBUTI PUBBLICI
		 */
		this.section = null;
		this.idAd = null;
		this.action = null;

		/**
		 * ATTRIBUTI/METODI PRIVATI
		 */
		var _baseUrl = '/bookmarkannunci.php';

		/**
		 * Effettua il salvataggio dell'annuncio
		 *
		 * @param action
		 * @param idAd
		 * @param isLogged
		 * @param section
		 * @param doFbAction
		 * @param bookmarkOptions
		 * @private
		 */
		var _doAction = function(action, idAd, isLogged, section, doFbAction, bookmarkOptions)
		{
			if (!action || !idAd) {
				throw new Error("BookmarkAnnuncio: action and idAd are required.");
				return;
			}
			this.idAd = idAd;
			this.section = section;
			this.action = action;
			bookmarkOptions = bookmarkOptions || {};
			bookmarkOptions.action = this.action;
			bookmarkOptions.id = this.idAd;
			bookmarkOptions.mode = 'xml';
			this.onWaiting();
			var self = this;
			$.ajax({url: _baseUrl, data: bookmarkOptions})
				.done( function( data ) {
					switch ( data ) {
						case 'NOT_LOGGED':
							self.onLogin(self.idAd, bookmarkOptions);
						case 'ERROR':
							self.onError();
							break;
						default:
							self.onSuccess(data, doFbAction);
							break;
					}
				});
		}

		return {
			doAction: _doAction
		}
	}

	BookmarkAnnuncio.prototype._abs_onWaiting = function(){};
	BookmarkAnnuncio.prototype._abs_onSuccess = function(){};
	BookmarkAnnuncio.prototype._abs_onError = function(){};
	BookmarkAnnuncio.prototype._abs_onLogin = function(){};

	/**
	 * Classe Bookmarkannuncio su mappa
	 * @constructor
	 */
	var BookmarkAnnuncioMappa = function()
	{
		/**
		 * Le operazioni da fare prima del salvataggio
		 */
		this.onWaiting = function()
		{
			switch (this.action) {
				case 'add':
				case 'addProgetto':
					$('div[data-id="'+ this.idAd +'"] i').addClass('on');
					$('div[data-id="'+ this.idAd +'"] span').html('Salvato');
					break;
				case 'delete':
				case 'deleteProgetto':
					$('div[data-id="'+ this.idAd +'"] i').removeClass('on');
					$('div[data-id="'+ this.idAd +'"] span').html('Salva');
					break;
			}
			$('div[data-id="'+ this.idAd +'"]').attr('data-loading', 1);
		}

		/**
		 * Metodo che viene lanciato una volta che sia e' conclusa con successo l'azione.
		 * @param data
		 * @param doFbAction
		 */
		this.onSuccess = function(data, doFbAction)
		{
			switch (this.action) {
				case 'add':
				case 'addProgetto':
					$('div[data-id="'+ this.idAd +'"]').attr('data-action', this.action == 'addProgetto' ? 'deleteProgetto' : 'delete');
					$('#num-annunci-salvati').html(parseInt(data));
					if ( doFbAction ) {
						boxSalvaAnnuncioFacebook(this.idAd);
					}
					break;
				case 'delete':
				case 'deleteProgetto':
					$('div[data-id="'+ this.idAd +'"]').attr('data-action', this.action == 'deleteProgetto' ? 'addProgetto' : 'add');
					$('#num-annunci-salvati').html(parseInt(data));
					break;
			}
			$('div[data-id="'+ this.idAd +'"]').attr('data-loading', 0);
		};

		/**
		 * Metodo che viene lanciato in caso di errori nel salvataggio o cancellazione.
		 */
		this.onError = function()
		{
			switch (this.action) {
				case 'add':
				case 'addProgetto':
					$('div[data-id="'+ this.idAd +'"] i').removeClass('on');
					$('div[data-id="'+ this.idAd +'"] span').html('Salvato');
					break;
				case 'delete':
				case 'deleteProgetto':
					$('div[data-id="'+ this.idAd +'"] i').addClass('on');
					$('div[data-id="'+ this.idAd +'"] span').html('Salva');
					break;
			}
		};

		/**
		 * Metodo che viene lanciato se e' necessario effettuare il login.
		 */
		this.onLogin = function(idAnnuncio, bookmarkOptions)
		{
			bookmarkOptions = bookmarkOptions || {};
			bookmarkOptions.idAnnuncio = idAnnuncio;
			var cookieVal = stringify(bookmarkOptions);
			createCookie("salvaAnnuncio", cookieVal);
			actionsOnMenuHeader.openAccediBox();
		};
	}

	/**
	 * Estende un oggetto come fosse una classe Astratta.
	 * Solamente i metodi prototype con prefix '_abs_' della ParentClass vengono controllati.
	 * Se un metodo astratto non e' presente allora segnalo un errore.
	 * @param ParentClass
	 */
	Function.prototype.extendAbstractPattern = function (ParentClass)
	{
		var tempClass = new this();
		for(var j in ParentClass.prototype) {
			if (j.indexOf('_abs_') !== 0) {
				continue;
			}
			var fnName = j.replace('_abs_', '');
			if (!tempClass[fnName] || typeof tempClass[fnName] !== 'function')
				throw new Error("Abstract method not implemented. Need to define method " + j);
		}
		tempClass = null;
		delete tempClass;
		this.prototype = new ParentClass();
		this.prototype.constructor = this;
	}

	BookmarkAnnuncioMappa.extendAbstractPattern(BookmarkAnnuncio);
	_bookmarkAnnuncio = new BookmarkAnnuncioMappa();
})(jQuery);


/**
 * Classe che disegna un poligono/cerchio
 * @param options
 */
var drawer = function( options ) {
	this.options = options || {};
	this.map = null;
	this.mode = null;
	this.MyOverlay = null;
	this.drawPolygon = null;
	this.drawCircle = null;
	this.tooltip = null;
	this.isVisibleTooltipMouse = null;
	this.parent = this.options.parent || parent.document;
	this.sufix = '';
	this.Storage = new webStorage();
	this.boxControlsMap = '#controls_draw_map';
	this.boxMappa = '#box_mappa';
	this.storageName = 'imm_area';
	this.enabledSubmitArea = false;
	this.closedArea = false;
	this.idEventCheckSubmit = false;
	this.negativeLayer = null;
	this.negativePath = new Array();
	this.submitLayer = null;

	this._onSubmit = function() { return true; }
	this._onChange = function() { return true; }
	this._onClose = function() { return true; }
	this._onReset = function() { return true; }

	this.onSubmit = function( fn ) {
		this._onSubmit = fn;
	}

	this.submit = function() {
		this.enabledSubmitArea = true;
		this._onSubmit();
	}

	this.onChange = function( fn ) {
		this._onChange = fn;
	}

	this.onClose = function( fn ) {
		this._onClose = fn;
	}

	this.onReset = function( fn ) {
		this._onReset = fn;
	}

	this.setMap = function( map ) {
		var that = this;
		this.map = map;
		this.mode = 'polygon';
		this.geocoder = new Im_Geocode();
		jQuery("#draw_polygon").click(function() {
			that.removeStorage();
			that.setDrawingMode('polygon');
		});
		jQuery("#draw_circle").click(function() {
			that.removeStorage();
			that.setDrawingMode('circle');
		});

		this.tooltip = new tooltipManager( 'tooltip-content' );
		this.tooltip.container = this.boxMappa;
		jQuery( this.boxMappa ).prepend(
			"<div id='tooltip-mappa' class='rounded-all box-shadow tooltip-mappa' style='display:none;'>"+
				"<div id='tooltip-index' class='fleft tooltip-index'></div>"+
				"<div id='tooltip-content' class='fleft tooltip-content'></div>"+
				"</div>"
		); // tooltip per gli aiuti nel disegnare l'area.
		this.submitLayer = new SubmitLayer( this.map, { drawingManager: this } );
		this.submitLayer.appendSubmitBox();
		this.MyOverlay = new MyOverlay( this.map );
		this.MyOverlay.init();
		this.drawCircle = new drawCircle_GMap( this.map );
		this.drawPolygon = new drawPolygon_GMap( this.map );
		return true;
	}

	this.setDrawingMode = function( mode ) {
		jQuery("#box-status-bar").hide();
		switch( mode ) {
			default:
				mode = 'polygon';
			case 'circle':
			case 'polygon':

				if (this.mode == mode)
					break;

				//if (this.mode)
				this.switchSubmitBtn(false);

				this.tooltip.setMode( mode );
				var isPolygon = (mode == 'polygon');

				if (isPolygon)
					this.tooltip.setOverlay( this.drawPolygon );
				else
					this.tooltip.setOverlay( this.drawCircle );

				this.tooltip.init();
				this.enableSubmitArea(false);
				this.closedArea = false;
				this.deleteOverlay();
				this.mode = mode;
				this.initOverlay();

				jQuery("#draw_polygon").toggleClass('on', isPolygon);
				jQuery("#draw_polygon").toggleClass('off', !isPolygon);
				jQuery("#draw_circle").toggleClass('on', !isPolygon);
				jQuery("#draw_circle").toggleClass('off', isPolygon);

				if (isPolygon) {
					jQuery("#draw_circle .text").addClass('text-shadow-dark').removeClass('text-shadow-light');
					jQuery("#draw_polygon .text").addClass('text-shadow-light').removeClass('text-shadow-dark');
				} else {
					jQuery("#draw_polygon .text").addClass('text-shadow-dark').removeClass('text-shadow-light');
					jQuery("#draw_circle .text").addClass('text-shadow-light').removeClass('text-shadow-dark');
				}
				break;
		}
	}

	this.enableSubmitArea = function(bEnable) {
		this.enabledSubmitArea = bEnable;
		if ( bEnable ) {
			jQuery(".conferma_area").addClass('on');
			jQuery(".conferma_area").removeClass('off');
		} else {
			jQuery(".conferma_area").addClass('off');
			jQuery(".conferma_area").removeClass('on');
		}
	}

	this.switchSubmitBtn = function(visible) {
		this.submitLayer.setVisible( visible );
	}

	this.checkSubmitArea = function(e) {
		if (this.mode == 'polygon') {
			this.switchSubmitBtn(true);
			if ( this.drawPolygon && this.drawPolygon.polyline.getPath().length == 1 ) {
				this.layerSubmitBox.draw();
			}
			if ( !this.drawPolygon || this.drawPolygon.polyline.getPath().length < 2 || !this.drawPolygon.canDraw )
				return false;
		} else if (this.mode == 'circle') {
			this.switchSubmitBtn(false);
			if ( !this.drawCircle || !this.drawCircle.circle || !this.drawCircle.circle.getRadius() )
				return false;
		}
		if ( !this.enabledSubmitArea ) {
			this.enableSubmitArea(true);
		}
		return true;
	}

	this.saveOverlay = function() {
		if ( !this.enabledSubmitArea )
			return false;
		switch( this.mode ) {
			case 'circle':
				var oArea = this.drawCircle.getArea();
				break;
			case 'polygon':
				this.drawPolygon.closePolygon();
				var oArea = this.drawPolygon.getArea();
				break;
		}
		jQuery('#gm_area' + this.sufix, this.parent).val(stringify(oArea));
		this.saveStorage(oArea);
		return true;
	}

	this.initOverlay = function() {
		this.map.setOptions({draggableCursor:'crosshair'});
		switch( this.mode ) {
			case 'circle':
				this.drawCircle.init();
				break;
			case 'polygon':
				this.drawPolygon.init();
				break;
		}
	}

	this.deleteOverlay = function() {
		this.setVisibleTooltipMouse(false);
		switch( this.mode ) {
			case 'circle':
				this.drawCircle.removeListener();
				this.drawCircle.deleteElement();
				break;
			case 'polygon':
				this.drawPolygon.removeListener();
				this.drawPolygon.deleteElement();
				break;
		}
	}

	this.resetMap = function() {
		this.switchSubmitBtn(false);
		this.deleteOverlay();
		this.initOverlay();
		this.enableSubmitArea(false);
		this.closedArea = false;
		this.tooltip.checkAction('start');
		var that = this;
		google.maps.event.removeListener( this.idEventCheckSubmit );
		this.idEventCheckSubmit = google.maps.event.addListener(this.map, 'click',  function(e){ that.checkSubmitArea(e); });
		jQuery('#gm_area' + this.sufix, this.parent ).removeAttr( 'disabled').val('');
	}

	this.setVisibleTooltipMouse = function(visible) {
		if (visible) {
			jQuery("#tooltip-mouse-map").show();
		} else {
			jQuery("#tooltip-mouse-map").hide();
		}
		this.isVisibleTooltipMouse = visible;
	}

	/* Salva l'area nello storaga in un array con chiave la categoria */
	this.saveStorage = function( value ) {
		try {
			this.setCategory();
			var oArea = this.Storage.getJSON(this.storageName);
			if (!oArea || typeof oArea != "object")
				oArea = new Object();
			oArea[this.idCategoria] = value;
			this.Storage.set(this.storageName, stringify(oArea));
		} catch(e) {
			_doLog("Storage non supported. Private Browsing?","error");
		}
	}

	/* Cancella lo storage */
	this.removeStorage = function() {
		this.saveStorage('');
	}

	this.fitBounds = function() {
		if ( this.mode == 'circle' )
			this.map.fitBounds( this.drawCircle.circle.getBounds() );
		else if ( this.mode == 'polygon' )
			this.map.fitBounds( this.drawPolygon.polygon.getBounds() );
	}

	this.disableDrawer = function() {
		this.tooltip.close();
		jQuery( this.boxControlsMap ).fadeOut();
		this.switchSubmitBtn(false);
		this.deleteOverlay();
		jQuery('#gm_area' + this.sufix, this.parent ).attr( 'disabled');
		google.maps.event.removeListener( this.idEventCheckSubmit );
		this.map.setOptions( {draggableCursor:'', disableDoubleClickZoom: false} );
	}

	this.enableDrawer = function() {
		var that = this;
		this.initOverlay();
		this.map.setOptions( {draggableCursor:'crosshair', disableDoubleClickZoom: true} );

		// Setto gli eventi che devono avvenire in momenti particolari durante la creazione dell'area.
		this.idEventCheckSubmit = google.maps.event.addListener(this.map, 'click',  function(e){ that.checkSubmitArea(e); });
		// poligono
		this.drawPolygon.addEvent('close', function() { this.map.setOptions({draggableCursor:'auto'}); this._onClose(); }.bind(this) );
		this.drawPolygon.addEvent('addpoint', function() { this.tooltip.checkAction(); }.bind(this) );
		this.drawPolygon.addEvent('addpoint', function() { this.submitLayer.checkPosition(); }.bind(this) );
		this.drawPolygon.addEvent('polygon_changed', function() { this._onChange(); }.bind(this) );
		// cerchio
		this.drawCircle.addEvent('addcenter', function() { this.tooltip.checkAction(); this._onChange(); }.bind(this) );
		this.drawCircle.addEvent('close', function() { this.tooltip.checkAction('close'); this._onClose(); }.bind(this) );
		this.drawCircle.addEvent('radius_changed', function() { this.tooltip.checkAction('close'); this._onChange(); }.bind(this) );
		this.drawCircle.addEvent('center_changed', function() { this.tooltip.checkAction('close'); this._onChange(); }.bind(this) );
		this.drawCircle.addEvent('center_changed', function() { this.submitLayer.checkPosition(); }.bind(this) );
		// manage tooltip
		this.tooltip.setMode( this.mode );
		if ( this.mode == 'polygon' )
			this.tooltip.setOverlay( this.drawPolygon );
		else
			this.tooltip.setOverlay( this.drawCircle );
		this.loadAreaFromInput();
		this.tooltip.show();
		jQuery( this.boxControlsMap ).fadeIn();
	};

	this.loadAreaFromInput = function() {
		var area = this.getSearchArea();
		var mode = (area) ? area.mode : '';
		this.setDrawingMode( mode );
		var bDone = ( area ) ? this.setLoadedArea(area.area) : false;
		this.enableSubmitArea( bDone );
		if ( bDone ) {
			this.tooltip.checkAction('close');
			this.switchSubmitBtn(true);
		} else {
			this.tooltip.init();
		}
	};

	this.getSearchArea = function () {
		var jArea, managerHash, $gmArea;

		$gmArea = jQuery('#gm_area' + this.sufix, this.parent);
		jArea = $gmArea.val();

		if (jq.isEmpty(jArea)) {
			managerHash = new ManagerHash();
			managerHash.convertHashStrToObj();
			jArea = managerHash.hashObj.poligono || null;
			$gmArea.val(jArea);
		}

		return (jArea ? jQuery.parseJSON(jArea) : null);
	};

	this.setLoadedArea = function( area ) {
		var bDraw = false;
		switch(this.mode) {
			case 'circle':
				bDraw = this.drawCircle.setCircle(area);
				if (bDraw)
					this.drawCircle.setCircleEvents();
				break;
			case 'polygon':
				bDraw = this.drawPolygon.setPolygon(area)
				break;
		}
		return bDraw;
	}
}

function setElementsDimension() {
	if ( jq( '#header' ).width() > 1200 ) {
		jq('#user-name').show();
		jq('.actions-user').removeClass('small');
		jq('.top-refine-search').removeClass('small');
	} else {
		jq('#user-name').hide();
		jq('.actions-user').addClass('small');
		jq('.top-refine-search').addClass('small');
	}
}

/**
 * Dato un path di clipper restituisce un semplice array
 * @param paths
 * @param scale
 * @returns {Array}
 */
function paths2Array (paths, scale) {
	var _array = new Array();;
	if (!scale) scale = 1;
	for(var i = 0; i < paths.length; i++) {
		_array[i] = new Array();
		for(var j = 0; j < paths[i].length; j++){
			_array[i].push( [ paths[i][j].X / scale , paths[i][j].Y / scale ] );
		}
	}
	return _array;
}

/**
 * Dato un cerchio restituisce un poligono regolare di n lati
 * @param center = array {lat, lng}
 * @param radius = raggio espresso in KM
 * @param numPoints = numeri di lati del poligono regolare.
 * @returns {Array}
 */
function getPolygonFromCircle( center, radius, numPoints ) {
	var d2r = Math.PI / 180;
	// Convert statute Km into degrees latitude
	var circleLat = radius/111;
	var circleLng = circleLat / Math.cos(center[0] * d2r);
	var p = new Array();
	for (var i = 0; i <= numPoints; i++) {
		// Convert degrees to radians
		var theta = Math.PI * i * 2 / numPoints;
		var ps = new Array();
		ps[0] = center[0] + (circleLat * Math.sin(theta));
		ps[1] = center[1] + (circleLng * Math.cos(theta));
		p.push( ps );
	}
	return p;
}

/**
 * Classe che gestisce i log della mappa
 * @param opt = opzioni.
 * @constructor
 */
function MapLog(opt)
{
	this.opt = opt || {};
	this._doLog = typeof this.opt.enabled != 'undefined' ? this.opt.enabled : false;
	this._oTimes = new Object();

	/**
	 * @param message
	 * @param keyTime: optional abilita lo startStop e lo associa ad una chiave univoca
	 * @param startOrEnd: 1 => start, 2 => end
	 * @private
	 */
	this.doLog = function(message, keyTime, startOrEnd) {
		if (!this._doLog)
			return;
		keyTime = keyTime || false;
		if (keyTime) {
			if ( startOrEnd ) {
				this._oTimes[keyTime] = this.now();
				message = 'START: ' + message;
			} else if ( typeof this._oTimes[keyTime] != 'undefined') {
				message = 'STOP: ' + message + ' - time(ms): ' + ( this.now() - this._oTimes[keyTime] );
			}
		}
		if ( typeof _doLog == 'function')
			_doLog(message);
	};

	/**
	 * Torna in millisecondi il time attuale
	 * @returns {number}
	 */
	this.now = function() {
		return new Date().getTime();
	};
};


(function($)
{
	/**
	 * Definisce le azioni sul menu Header.
	 * @constructor
	 */
	function ActionsOnMenuHeader()
	{
		var idboxoverlayMap = '#box-overlay-map',
			idBtnRegistraUtente = '#btn-registra-utente',
			idBtnAccediUtente = '#btn-accedi-utente';

		this.accediUtenteBox;

		/**
		 * Inizializza gli eventi sui pulsanti dell'header
		 */
		this.initEvents = function()
		{
			/* Gestione pure-menu */
			$('.pure-menu > ul > li > a').click(function(e)
			{
				if ($(this).parent().has('.pure-menu-children').length) {
					e.preventDefault();
					var parent = jq(this).parent();
					if (!parent.hasClass('pure-menu-open')) {
						$('.pure-menu li.pure-menu-open').removeClass('pure-menu-open');
					}
					$(idboxoverlayMap).toggleClass('open', !parent.hasClass('pure-menu-open'));
					parent.toggleClass('pure-menu-open', !parent.hasClass('pure-menu-open'));
					return;
				}
			});

			$(idBtnRegistraUtente).click(function(e)
			{
				e.preventDefault();
				createRegistraBox();
				$(this).blur();
			});

			$(idBtnAccediUtente).click(function(e)
			{
				e.preventDefault();
				createAccediBox();
				$(this).blur();
			});

			$(idboxoverlayMap).click(function()
			{
				$('.pure-menu li.pure-menu-open').removeClass('pure-menu-open');
				$(this).removeClass('open');
			});
		}

		/**
		 * Apre il box per il login dell'utente.
		 */
		this.openAccediBox = function()
		{
			$(idBtnAccediUtente).click();
		}

		/**
		 * Chiude tutti i menu aperti dell'header.
		 */
		this.closeAllMenuOpened = function()
		{
			$(idboxoverlayMap).removeClass('open');
			$('.pure-menu > ul > li').removeClass('pure-menu-open');
		}

		/**
		 * Crea il box di accesso con gli handler relativi.
		 */
		var createAccediBox = function()
		{
			var self = this;
			this.accediUtenteBox = create_accediBox('U',{
					callback: function()
					{
						var __setDoLogin = new setDoLoginWithHandler({
							onSuccess: function()
							{
								self.accediUtenteBox.close();
								managerUser.getUserInfo();
								managerMarkers.resetParamsToReloadMap(true);
							}
						});
						$('#U-form').attr('onsubmit','').unbind('submit').bind('submit', function(e){
							e.preventDefault();
							eseguiAccesso(jq(this)[0], __setDoLogin.onLogin);
						});
					}
				}
			);
			this.accediUtenteBox.show();
		}

		/**
		 * Crea il box di registrazione dell'utente.
		 */
		var createRegistraBox = function()
		{
			this.accediUtenteBox = create_accediBox('RU',{callback: addAutoCompleteDomainRU});
			this.accediUtenteBox.show();
		}
	}

	actionsOnMenuHeader = new ActionsOnMenuHeader();
})(jQuery);


(function($){

	/**
	 * Questa classe gestisce gli eventi sulle variazioni della hash.
	 * Se il browser e' compatibile con HTML5 allora viene gestito
	 * l'evento onpopstate ( https://developer.mozilla.org/en-US/docs/Web/API/Window.onpopstate )
	 * e i suoi metodi pushState/replaceState ( https://developer.mozilla.org/en-US/docs/Web/Guide/API/DOM/Manipulating_the_browser_history )
	 * altrimenti l'evento onhashchange.
	 * @constructor
	 */
	function EventsOnHashChange()
	{
		var hashHistory, historyLength, obj;
		/**
		 * inizializza gli eventi e gli oggetti necessari.
		 * @param objDependence = questo oggetto deve implementare i metodi: onForward - onBack - onPopState
		 */
		this.init = function(objDependence)
		{
			obj = objDependence;
			if (typeof window.onpopstate != 'undefined') {
				$(window).bind("popstate", onPopState());
			} else {
				$(window).on("hashchange", detectBackOrForward());
			}
		}

		/**
		 * Questa funzione e' l'handler dell'evento on hashchange.
		 * Identifica se viene effettuato un evento di Indietro o Avanti
		 * @returns {Function}
		 */
		var detectBackOrForward = function()
		{
			if (typeof obj.onForward != 'function' || typeof obj.onBack != 'function' || typeof obj.idCheckHashValue == 'undefined') {
				return function() {};
			}
			hashHistory = [window.location.hash];
			historyLength = window.history.length;
			$('body').append( $('<input type="hidden" id="' + obj.idCheckHashValue + '" />'));
			return function()
			{
				var hash = window.location.hash,
					length = window.history.length;
				if ($('#' + obj.idCheckHashValue).val() == hash) {
					hashHistory.push(hash);
					historyLength = length;
				} else if (hashHistory.length && historyLength == length) {
					if (hashHistory[hashHistory.length - 2] == hash) {
						hashHistory = hashHistory.slice(0, -1);
						obj.onBack();
					} else {
						hashHistory.push(hash);
						$('#' + obj.idCheckHashValue).val(hash);
						obj.onForward();
					}
				} else {
					hashHistory.push(hash);
					historyLength = length;
				}
			}
		};

		/**
		 * Questa funzione e' l'handler dell'evento onpopstate.
		 * Questo evento viene triggerato o dal click sui pulsanti del browser o se si cambia la url a mano.
		 * @param event
		 * @returns {Function}
		 */
		var onPopState = function()
		{
			if (typeof obj.onPopState != 'function') {
				return function() {};
			}
			return function(event)
			{
				obj.onPopState(event);
			}
		}
	}

	// inizializza l'oggetto
	eventsOnHashChange = new EventsOnHashChange();
})(jQuery);
;
!function(t,e,i,n){var a=e.document,o=t(a),r=t(e),s=Array.prototype,c=1.35,l=!1,h=!1,u=3e4,d=!1,p=navigator.userAgent.toLowerCase(),f=e.location.hash.replace(/#\//,""),g=e.location.protocol,m=Math,v=function(){},y=function(){return!1},_=function(){var t=3,e=a.createElement("div"),i=e.getElementsByTagName("i");do e.innerHTML="<!--[if gt IE "+ ++t+"]><i></i><![endif]-->";while(i[0]);return t>4?t:a.documentMode||n}(),b=function(){return{html:a.documentElement,body:a.body,head:a.getElementsByTagName("head")[0],title:a.title}},w=e.parent!==e.self,x="data ready thumbnail loadstart loadfinish image play pause progress fullscreen_enter fullscreen_exit idle_enter idle_exit rescale lightbox_open lightbox_close lightbox_image",T=function(){var e=[];return t.each(x.split(" "),function(t,i){e.push(i),/_/.test(i)&&e.push(i.replace(/_/g,""))}),e}(),k=function(e){var i;return"object"!=typeof e?e:(t.each(e,function(n,a){/^[a-z]+_/.test(n)&&(i="",t.each(n.split("_"),function(t,e){i+=t>0?e.substr(0,1).toUpperCase()+e.substr(1):e}),e[i]=a,delete e[n])}),e)},C=function(e){return t.inArray(e,T)>-1?i[e.toUpperCase()]:e},I={youtube:{reg:/https?:\/\/(?:[a-zA_Z]{2,3}.)?(?:youtube\.com\/watch\?)((?:[\w\d\-\_\=]+&amp;(?:amp;)?)*v(?:&lt;[A-Z]+&gt;)?=([0-9a-zA-Z\-\_]+))/i,embed:function(){return"http://www.youtube.com/embed/"+this.id},getUrl:function(){return g+"//gdata.youtube.com/feeds/api/videos/"+this.id+"?v=2&alt=json-in-script&callback=?"},get_thumb:function(t){return t.entry.media$group.media$thumbnail[2].url},get_image:function(t){return t.entry.yt$hd?g+"//img.youtube.com/vi/"+this.id+"/maxresdefault.jpg":t.entry.media$group.media$thumbnail[3].url}},vimeo:{reg:/https?:\/\/(?:www\.)?(vimeo\.com)\/(?:hd#)?([0-9]+)/i,embed:function(){return"http://player.vimeo.com/video/"+this.id},getUrl:function(){return g+"//vimeo.com/api/v2/video/"+this.id+".json?callback=?"},get_thumb:function(t){return t[0].thumbnail_medium},get_image:function(t){return t[0].thumbnail_large}},dailymotion:{reg:/https?:\/\/(?:www\.)?(dailymotion\.com)\/video\/([^_]+)/,embed:function(){return g+"//www.dailymotion.com/embed/video/"+this.id},getUrl:function(){return"https://api.dailymotion.com/video/"+this.id+"?fields=thumbnail_240_url,thumbnail_720_url&callback=?"},get_thumb:function(t){return t.thumbnail_240_url},get_image:function(t){return t.thumbnail_720_url}},_inst:[]},S=function(e,i){for(var n=0;n<I._inst.length;n++)if(I._inst[n].id===i&&I._inst[n].type==e)return I._inst[n];this.type=e,this.id=i,this.readys=[],I._inst.push(this);var a=this;t.extend(this,I[e]),t.getJSON(this.getUrl(),function(e){a.data=e,t.each(a.readys,function(t,e){e(a.data)}),a.readys=[]}),this.getMedia=function(t,e,i){i=i||v;var n=this,a=function(i){e(n["get_"+t](i))};try{n.data?a(n.data):n.readys.push(a)}catch(o){i()}}},A=function(t){var e;for(var i in I)if(e=t&&I[i].reg&&t.match(I[i].reg),e&&e.length)return{id:e[2],provider:i};return!1},E={support:function(){var t=b().html;return!w&&(t.requestFullscreen||t.mozRequestFullScreen||t.webkitRequestFullScreen)}(),callback:v,enter:function(t,e,i){this.instance=t,this.callback=e||v,i=i||b().html,i.requestFullscreen?i.requestFullscreen():i.mozRequestFullScreen?i.mozRequestFullScreen():i.webkitRequestFullScreen&&i.webkitRequestFullScreen()},exit:function(t){this.callback=t||v,a.exitFullscreen?a.exitFullscreen():a.mozCancelFullScreen?a.mozCancelFullScreen():a.webkitCancelFullScreen&&a.webkitCancelFullScreen()},instance:null,listen:function(){if(this.support){var t=function(){if(E.instance){var t=E.instance._fullscreen;a.fullscreen||a.mozFullScreen||a.webkitIsFullScreen?t._enter(E.callback):t._exit(E.callback)}};a.addEventListener("fullscreenchange",t,!1),a.addEventListener("mozfullscreenchange",t,!1),a.addEventListener("webkitfullscreenchange",t,!1)}}},D=[],$=[],L=!1,P=!1,z=[],H=function(e){i.theme=e,t.each(z,function(t,e){e._initialized||e._init.call(e)}),z=[]},O=function(){return{clearTimer:function(e){t.each(i.get(),function(){this.clearTimer(e)})},addTimer:function(e){t.each(i.get(),function(){this.addTimer(e)})},array:function(t){return s.slice.call(t,0)},create:function(t,e){e=e||"div";var i=a.createElement(e);return i.className=t,i},removeFromArray:function(e,i){return t.each(e,function(t,n){return n==i?(e.splice(t,1),!1):void 0}),e},getScriptPath:function(e){e=e||t("script:last").attr("src");var i=e.split("/");return 1==i.length?"":(i.pop(),i.join("/")+"/")},animate:function(){var n,o,r,s,c,l,h,u=function(t){var i,n="transition WebkitTransition MozTransition OTransition".split(" ");if(e.opera)return!1;for(i=0;n[i];i++)if("undefined"!=typeof t[n[i]])return n[i];return!1}((a.body||a.documentElement).style),d={MozTransition:"transitionend",OTransition:"oTransitionEnd",WebkitTransition:"webkitTransitionEnd",transition:"transitionend"}[u],p={_default:[.25,.1,.25,1],galleria:[.645,.045,.355,1],galleriaIn:[.55,.085,.68,.53],galleriaOut:[.25,.46,.45,.94],ease:[.25,0,.25,1],linear:[.25,.25,.75,.75],"ease-in":[.42,0,1,1],"ease-out":[0,0,.58,1],"ease-in-out":[.42,0,.58,1]},f=function(e,i,n){var a={};n=n||"transition",t.each("webkit moz ms o".split(" "),function(){a["-"+this+"-"+n]=i}),e.css(a)},g=function(t){f(t,"none","transition"),i.WEBKIT&&i.TOUCH&&(f(t,"translate3d(0,0,0)","transform"),t.data("revert")&&(t.css(t.data("revert")),t.data("revert",null)))};return function(a,m,y){return y=t.extend({duration:400,complete:v,stop:!1},y),a=t(a),y.duration?u?(y.stop&&(a.off(d),g(a)),n=!1,t.each(m,function(t,e){h=a.css(t),O.parseValue(h)!=O.parseValue(e)&&(n=!0),a.css(t,h)}),n?(o=[],r=y.easing in p?p[y.easing]:p._default,s=" "+y.duration+"ms cubic-bezier("+r.join(",")+")",void e.setTimeout(function(e,n,a,r){return function(){e.one(n,function(t){return function(){g(t),y.complete.call(t[0])}}(e)),i.WEBKIT&&i.TOUCH&&(c={},l=[0,0,0],t.each(["left","top"],function(t,i){i in a&&(l[t]=O.parseValue(a[i])-O.parseValue(e.css(i))+"px",c[i]=a[i],delete a[i])}),(l[0]||l[1])&&(e.data("revert",c),o.push("-webkit-transform"+r),f(e,"translate3d("+l.join(",")+")","transform"))),t.each(a,function(t){o.push(t+r)}),f(e,o.join(",")),e.css(a)}}(a,d,m,s),2)):void e.setTimeout(function(){y.complete.call(a[0])},y.duration)):void a.animate(m,y):(a.css(m),void y.complete.call(a[0]))}}(),removeAlpha:function(t){if(t instanceof jQuery&&(t=t[0]),9>_&&t){var e=t.style,i=t.currentStyle,n=i&&i.filter||e.filter||"";/alpha/.test(n)&&(e.filter=n.replace(/alpha\([^)]*\)/i,""))}},forceStyles:function(e,i){e=t(e),e.attr("style")&&e.data("styles",e.attr("style")).removeAttr("style"),e.css(i)},revertStyles:function(){t.each(O.array(arguments),function(e,i){i=t(i),i.removeAttr("style"),i.attr("style",""),i.data("styles")&&i.attr("style",i.data("styles")).data("styles",null)})},moveOut:function(t){O.forceStyles(t,{position:"absolute",left:-1e4})},moveIn:function(){O.revertStyles.apply(O,O.array(arguments))},hide:function(e,i,n){n=n||v;var a=t(e);e=a[0],a.data("opacity")||a.data("opacity",a.css("opacity"));var o={opacity:0};if(i){var r=9>_&&e?function(){O.removeAlpha(e),e.style.visibility="hidden",n.call(e)}:n;O.animate(e,o,{duration:i,complete:r,stop:!0})}else 9>_&&e?(O.removeAlpha(e),e.style.visibility="hidden"):a.css(o)},show:function(e,i,n){n=n||v;var a=t(e);e=a[0];var o=parseFloat(a.data("opacity"))||1,r={opacity:o};if(i){9>_&&(a.css("opacity",0),e.style.visibility="visible");var s=9>_&&e?function(){1==r.opacity&&O.removeAlpha(e),n.call(e)}:n;O.animate(e,r,{duration:i,complete:s,stop:!0})}else 9>_&&1==r.opacity&&e?(O.removeAlpha(e),e.style.visibility="visible"):a.css(r)},wait:function(n){i._waiters=i._waiters||[],n=t.extend({until:y,success:v,error:function(){i.raise("Could not complete wait function.")},timeout:3e3},n);var a,o,r,s=O.timestamp(),c=function(){return o=O.timestamp(),a=o-s,O.removeFromArray(i._waiters,r),n.until(a)?(n.success(),!1):"number"==typeof n.timeout&&o>=s+n.timeout?(n.error(),!1):void i._waiters.push(r=e.setTimeout(c,10))};i._waiters.push(r=e.setTimeout(c,10))},toggleQuality:function(t,e){7!==_&&8!==_||!t||"IMG"!=t.nodeName.toUpperCase()||("undefined"==typeof e&&(e="nearest-neighbor"===t.style.msInterpolationMode),t.style.msInterpolationMode=e?"bicubic":"nearest-neighbor")},insertStyleTag:function(e,i){if(!i||!t("#"+i).length){var n=a.createElement("style");if(i&&(n.id=i),b().head.appendChild(n),n.styleSheet)n.styleSheet.cssText=e;else{var o=a.createTextNode(e);n.appendChild(o)}}},loadScript:function(e,i){var n=!1,a=t("<script>").attr({src:e,async:!0}).get(0);a.onload=a.onreadystatechange=function(){n||this.readyState&&"loaded"!==this.readyState&&"complete"!==this.readyState||(n=!0,a.onload=a.onreadystatechange=null,"function"==typeof i&&i.call(this,this))},b().head.appendChild(a)},parseValue:function(t){if("number"==typeof t)return t;if("string"==typeof t){var e=t.match(/\-?\d|\./g);return e&&e.constructor===Array?1*e.join(""):0}return 0},timestamp:function(){return(new Date).getTime()},loadCSS:function(e,o,r){var s,c;if(t("link[rel=stylesheet]").each(function(){return new RegExp(e).test(this.href)?(s=this,!1):void 0}),"function"==typeof o&&(r=o,o=n),r=r||v,s)return r.call(s,s),s;if(c=a.styleSheets.length,t("#"+o).length)t("#"+o).attr("href",e),c--;else{s=t("<link>").attr({rel:"stylesheet",href:e,id:o}).get(0);var l=t('link[rel="stylesheet"], style');if(l.length?l.get(0).parentNode.insertBefore(s,l[0]):b().head.appendChild(s),_&&c>=31)return void i.raise("You have reached the browser stylesheet limit (31)",h)}if("function"==typeof r){var u=t("<s>").attr("id","galleria-loader").hide().appendTo(b().body);O.wait({until:function(){return 1==u.height()},success:function(){u.remove(),r.call(s,s)},error:function(){u.remove(),i.raise("Theme CSS could not load after 20 sec. "+(i.QUIRK?"Your browser is in Quirks Mode, please add a correct doctype.":"Please download the latest theme at http://galleria.io/customer/."),h)},timeout:5e3})}return s}}}(),F=function(e){var i=".galleria-videoicon{width:60px;height:60px;position:absolute;top:50%;left:50%;z-index:1;margin:-30px 0 0 -30px;cursor:pointer;background:#000;background:rgba(0,0,0,.8);border-radius:3px;-webkit-transition:all 150ms}.galleria-videoicon i{width:0px;height:0px;border-style:solid;border-width:10px 0 10px 16px;display:block;border-color:transparent transparent transparent #ffffff;margin:20px 0 0 22px}.galleria-image:hover .galleria-videoicon{background:#000}";return O.insertStyleTag(i,"galleria-videoicon"),t(O.create("galleria-videoicon")).html("<i></i>").appendTo(e).click(function(){t(this).siblings("img").mouseup()})},N=function(){var e=function(e,i,n,a){var o=this.getOptions("easing"),r=this.getStageWidth(),s={left:r*(e.rewind?-1:1)},c={left:0};n?(s.opacity=0,c.opacity=1):s.opacity=1,t(e.next).css(s),O.animate(e.next,c,{duration:e.speed,complete:function(t){return function(){i(),t.css({left:0})}}(t(e.next).add(e.prev)),queue:!1,easing:o}),a&&(e.rewind=!e.rewind),e.prev&&(s={left:0},c={left:r*(e.rewind?1:-1)},n&&(s.opacity=1,c.opacity=0),t(e.prev).css(s),O.animate(e.prev,c,{duration:e.speed,queue:!1,easing:o,complete:function(){t(this).css("opacity",0)}}))};return{active:!1,init:function(t,e,i){N.effects.hasOwnProperty(t)&&N.effects[t].call(this,e,i)},effects:{fade:function(e,i){t(e.next).css({opacity:0,left:0}),O.animate(e.next,{opacity:1},{duration:e.speed,complete:i}),e.prev&&(t(e.prev).css("opacity",1).show(),O.animate(e.prev,{opacity:0},{duration:e.speed}))},flash:function(e,i){t(e.next).css({opacity:0,left:0}),e.prev?O.animate(e.prev,{opacity:0},{duration:e.speed/2,complete:function(){O.animate(e.next,{opacity:1},{duration:e.speed,complete:i})}}):O.animate(e.next,{opacity:1},{duration:e.speed,complete:i})},pulse:function(e,i){e.prev&&t(e.prev).hide(),t(e.next).css({opacity:0,left:0}).show(),O.animate(e.next,{opacity:1},{duration:e.speed,complete:i})},slide:function(){e.apply(this,O.array(arguments))},fadeslide:function(){e.apply(this,O.array(arguments).concat([!0]))},doorslide:function(){e.apply(this,O.array(arguments).concat([!1,!0]))}}}}();E.listen(),t.event.special["click:fast"]={propagate:!0,add:function(e){this.propagate;i.TOUCH?t(this).on("touchstart.fast",function(i){var n,a,o=i.originalEvent,r=0;1==o.touches.length&&(n=o.touches[0].pageX,a=o.touches[0].pageY,t(this).on("touchmove.fast",function(t){var e=t.originalEvent.touches;1==e.length&&(r=m.max(m.abs(n-e[0].pageX),m.abs(a-e[0].pageY)))}),t(this).on("touchend.fast",function(){return r>4?t(this).off("touchend.fast touchmove.fast"):(e.handler.call(this,i),void t(this).off("touchend.fast touchmove.fast"))}))}):t(this).on("click.fast",e.handler)},remove:function(e){i.TOUCH?t(this).off("touchstart.fast touchmove.fast touchend.fast"):t(this).off("click.fast",e.handler)}},r.on("orientationchange",function(){t(this).resize()}),i=function(){var s=this;this._options={},this._playing=!1,this._playtime=5e3,this._active=null,this._queue={length:0},this._data=[],this._dom={},this._thumbnails=[],this._layers=[],this._initialized=!1,this._firstrun=!1,this._stageWidth=0,this._stageHeight=0,this._target=n,this._binds=[],this._id=parseInt(1e4*m.random(),10);var c="container stage images image-nav image-nav-left image-nav-right info info-text info-title info-description thumbnails thumbnails-list thumbnails-container thumb-nav-left thumb-nav-right loader counter tooltip",l="current total";t.each(c.split(" "),function(t,e){s._dom[e]=O.create("galleria-"+e)}),t.each(l.split(" "),function(t,e){s._dom[e]=O.create("galleria-"+e,"span")});var h=this._keyboard={keys:{UP:38,DOWN:40,LEFT:37,RIGHT:39,RETURN:13,ESCAPE:27,BACKSPACE:8,SPACE:32},map:{},bound:!1,press:function(t){var e=t.keyCode||t.which;e in h.map&&"function"==typeof h.map[e]&&h.map[e].call(s,t)},attach:function(t){var e,i;for(e in t)t.hasOwnProperty(e)&&(i=e.toUpperCase(),i in h.keys?h.map[h.keys[i]]=t[e]:h.map[i]=t[e]);h.bound||(h.bound=!0,o.on("keydown",h.press))},detach:function(){h.bound=!1,h.map={},o.off("keydown",h.press)}},u=this._controls={0:n,1:n,active:0,swap:function(){u.active=u.active?0:1},getActive:function(){return s._options.swipe?u.slides[s._active]:u[u.active]},getNext:function(){return s._options.swipe?u.slides[s.getNext(s._active)]:u[1-u.active]},slides:[],frames:[],layers:[]},d=this._carousel={next:s.$("thumb-nav-right"),prev:s.$("thumb-nav-left"),width:0,current:0,max:0,hooks:[],update:function(){var e=0,i=0,n=[0];t.each(s._thumbnails,function(a,o){if(o.ready){e+=o.outerWidth||t(o.container).outerWidth(!0);var r=t(o.container).width();e+=r-m.floor(r),n[a+1]=e,i=m.max(i,o.outerHeight||t(o.container).outerHeight(!0))}}),s.$("thumbnails").css({width:e,height:i}),d.max=e,d.hooks=n,d.width=s.$("thumbnails-list").width(),d.setClasses(),s.$("thumbnails-container").toggleClass("galleria-carousel",e>d.width),d.width=s.$("thumbnails-list").width()},bindControls:function(){var t;d.next.on("click:fast",function(e){if(e.preventDefault(),"auto"===s._options.carouselSteps){for(t=d.current;t<d.hooks.length;t++)if(d.hooks[t]-d.hooks[d.current]>d.width){d.set(t-2);break}}else d.set(d.current+s._options.carouselSteps)}),d.prev.on("click:fast",function(e){if(e.preventDefault(),"auto"===s._options.carouselSteps)for(t=d.current;t>=0;t--){if(d.hooks[d.current]-d.hooks[t]>d.width){d.set(t+2);break}if(0===t){d.set(0);break}}else d.set(d.current-s._options.carouselSteps)})},set:function(t){for(t=m.max(t,0);d.hooks[t-1]+d.width>=d.max&&t>=0;)t--;d.current=t,d.animate()},getLast:function(t){return(t||d.current)-1},follow:function(t){if(0===t||t===d.hooks.length-2)return void d.set(t);for(var e=d.current;d.hooks[e]-d.hooks[d.current]<d.width&&e<=d.hooks.length;)e++;t-1<d.current?d.set(t-1):t+2>e&&d.set(t-e+d.current+2)},setClasses:function(){d.prev.toggleClass("disabled",!d.current),d.next.toggleClass("disabled",d.hooks[d.current]+d.width>=d.max)},animate:function(){d.setClasses();var e=-1*d.hooks[d.current];isNaN(e)||(s.$("thumbnails").css("left",function(){return t(this).css("left")}),O.animate(s.get("thumbnails"),{left:e},{duration:s._options.carouselSpeed,easing:s._options.easing,queue:!1}))}},f=this._tooltip={initialized:!1,open:!1,timer:"tooltip"+s._id,swapTimer:"swap"+s._id,init:function(){f.initialized=!0;var t=".galleria-tooltip{padding:3px 8px;max-width:50%;background:#ffe;color:#000;z-index:3;position:absolute;font-size:11px;line-height:1.3;opacity:0;box-shadow:0 0 2px rgba(0,0,0,.4);-moz-box-shadow:0 0 2px rgba(0,0,0,.4);-webkit-box-shadow:0 0 2px rgba(0,0,0,.4);}";O.insertStyleTag(t,"galleria-tooltip"),s.$("tooltip").css({opacity:.8,visibility:"visible",display:"none"})},move:function(t){var e=s.getMousePosition(t).x,i=s.getMousePosition(t).y,n=s.$("tooltip"),a=e,o=i,r=n.outerHeight(!0)+1,c=n.outerWidth(!0),l=r+15,h=s.$("container").width()-c-2,u=s.$("container").height()-r-2;isNaN(a)||isNaN(o)||(a+=10,o-=r+8,a=m.max(0,m.min(h,a)),o=m.max(0,m.min(u,o)),l>i&&(o=l),n.css({left:a,top:o}))},bind:function(e,n){if(!i.TOUCH){f.initialized||f.init();var a=function(){s.$("container").off("mousemove",f.move),s.clearTimer(f.timer),s.$("tooltip").stop().animate({opacity:0},200,function(){s.$("tooltip").hide(),s.addTimer(f.swapTimer,function(){f.open=!1},1e3)})},o=function(e,i){f.define(e,i),t(e).hover(function(){s.clearTimer(f.swapTimer),s.$("container").off("mousemove",f.move).on("mousemove",f.move).trigger("mousemove"),f.show(e),s.addTimer(f.timer,function(){s.$("tooltip").stop().show().animate({opacity:1}),f.open=!0},f.open?0:500)},a).click(a)};"string"==typeof n?o(e in s._dom?s.get(e):e,n):t.each(e,function(t,e){o(s.get(t),e)})}},show:function(i){i=t(i in s._dom?s.get(i):i);var n=i.data("tt"),a=function(t){e.setTimeout(function(t){return function(){f.move(t)}}(t),10),i.off("mouseup",a)};n="function"==typeof n?n():n,n&&(s.$("tooltip").html(n.replace(/\s/,"&#160;")),i.on("mouseup",a))},define:function(e,i){if("function"!=typeof i){var n=i;i=function(){return n}}e=t(e in s._dom?s.get(e):e).data("tt",i),f.show(e)}},g=this._fullscreen={scrolled:0,crop:n,active:!1,prev:t(),beforeEnter:function(t){t()},beforeExit:function(t){t()},keymap:s._keyboard.map,parseCallback:function(e,i){return N.active?function(){"function"==typeof e&&e.call(s);var n=s._controls.getActive(),a=s._controls.getNext();s._scaleImage(a),s._scaleImage(n),i&&s._options.trueFullscreen&&t(n.container).add(a.container).trigger("transitionend")}:e},enter:function(t){g.beforeEnter(function(){t=g.parseCallback(t,!0),s._options.trueFullscreen&&E.support?(g.active=!0,O.forceStyles(s.get("container"),{width:"100%",height:"100%"}),s.rescale(),i.MAC?i.SAFARI&&/version\/[1-5]/.test(p)?(s.$("stage").css("opacity",0),e.setTimeout(function(){g.scale(),s.$("stage").css("opacity",1)},4)):(s.$("container").css("opacity",0).addClass("fullscreen"),e.setTimeout(function(){g.scale(),s.$("container").css("opacity",1)},50)):s.$("container").addClass("fullscreen"),r.resize(g.scale),E.enter(s,t,s.get("container"))):(g.scrolled=r.scrollTop(),i.TOUCH||e.scrollTo(0,0),g._enter(t))})},_enter:function(o){g.active=!0,w&&(g.iframe=function(){var n,o=a.referrer,r=a.createElement("a"),s=e.location;return r.href=o,r.protocol!=s.protocol||r.hostname!=s.hostname||r.port!=s.port?(i.raise("Parent fullscreen not available. Iframe protocol, domains and ports must match."),!1):(g.pd=e.parent.document,t(g.pd).find("iframe").each(function(){var t=this.contentDocument||this.contentWindow.document;return t===a?(n=this,!1):void 0}),n)}()),O.hide(s.getActiveImage()),w&&g.iframe&&(g.iframe.scrolled=t(e.parent).scrollTop(),e.parent.scrollTo(0,0));var c=s.getData(),l=s._options,h=!s._options.trueFullscreen||!E.support,u={height:"100%",overflow:"hidden",margin:0,padding:0};if(h&&(s.$("container").addClass("fullscreen"),g.prev=s.$("container").prev(),g.prev.length||(g.parent=s.$("container").parent()),s.$("container").appendTo("body"),O.forceStyles(s.get("container"),{position:i.TOUCH?"absolute":"fixed",top:0,left:0,width:"100%",height:"100%",zIndex:1e4}),O.forceStyles(b().html,u),O.forceStyles(b().body,u)),w&&g.iframe&&(O.forceStyles(g.pd.documentElement,u),O.forceStyles(g.pd.body,u),O.forceStyles(g.iframe,t.extend(u,{width:"100%",height:"100%",top:0,left:0,position:"fixed",zIndex:1e4,border:"none"}))),g.keymap=t.extend({},s._keyboard.map),s.attachKeyboard({escape:s.exitFullscreen,right:s.next,left:s.prev}),g.crop=l.imageCrop,l.fullscreenCrop!=n&&(l.imageCrop=l.fullscreenCrop),c&&c.big&&c.image!==c.big){var d=new i.Picture,p=d.isCached(c.big),f=s.getIndex(),m=s._thumbnails[f];s.trigger({type:i.LOADSTART,cached:p,rewind:!1,index:f,imageTarget:s.getActiveImage(),thumbTarget:m,galleriaData:c}),d.load(c.big,function(e){s._scaleImage(e,{complete:function(e){s.trigger({type:i.LOADFINISH,cached:p,index:f,rewind:!1,imageTarget:e.image,thumbTarget:m});var n=s._controls.getActive().image;n&&t(n).width(e.image.width).height(e.image.height).attr("style",t(e.image).attr("style")).attr("src",e.image.src)}})});var v=s.getNext(f),y=new i.Picture,_=s.getData(v);y.preload(s.isFullscreen()&&_.big?_.big:_.image)}s.rescale(function(){s.addTimer(!1,function(){h&&O.show(s.getActiveImage()),"function"==typeof o&&o.call(s),s.rescale()},100),s.trigger(i.FULLSCREEN_ENTER)}),h?r.resize(g.scale):O.show(s.getActiveImage())},scale:function(){s.rescale()},exit:function(t){g.beforeExit(function(){t=g.parseCallback(t),s._options.trueFullscreen&&E.support?E.exit(t):g._exit(t)})},_exit:function(t){g.active=!1;var n=!s._options.trueFullscreen||!E.support,a=s.$("container").removeClass("fullscreen");if(g.parent?g.parent.prepend(a):a.insertAfter(g.prev),n){O.hide(s.getActiveImage()),O.revertStyles(s.get("container"),b().html,b().body),i.TOUCH||e.scrollTo(0,g.scrolled);var o=s._controls.frames[s._controls.active];o&&o.image&&(o.image.src=o.image.src)}w&&g.iframe&&(O.revertStyles(g.pd.documentElement,g.pd.body,g.iframe),g.iframe.scrolled&&e.parent.scrollTo(0,g.iframe.scrolled)),s.detachKeyboard(),s.attachKeyboard(g.keymap),s._options.imageCrop=g.crop;var c=s.getData().big,l=s._controls.getActive().image;!s.getData().iframe&&l&&c&&c==l.src&&e.setTimeout(function(t){return function(){l.src=t}}(s.getData().image),1),s.rescale(function(){s.addTimer(!1,function(){n&&O.show(s.getActiveImage()),"function"==typeof t&&t.call(s),r.trigger("resize")},50),s.trigger(i.FULLSCREEN_EXIT)}),r.off("resize",g.scale)}},v=this._idle={trunk:[],bound:!1,active:!1,add:function(e,n,a,o){if(e&&!i.TOUCH){v.bound||v.addEvent(),e=t(e),"boolean"==typeof a&&(o=a,a={}),a=a||{};var r,s={};for(r in n)n.hasOwnProperty(r)&&(s[r]=e.css(r));e.data("idle",{from:t.extend(s,a),to:n,complete:!0,busy:!1}),o?e.css(n):v.addTimer(),v.trunk.push(e)}},remove:function(e){e=t(e),t.each(v.trunk,function(t,i){i&&i.length&&!i.not(e).length&&(e.css(e.data("idle").from),v.trunk.splice(t,1))}),v.trunk.length||(v.removeEvent(),s.clearTimer(v.timer))},addEvent:function(){v.bound=!0,s.$("container").on("mousemove click",v.showAll),"hover"==s._options.idleMode&&s.$("container").on("mouseleave",v.hide)},removeEvent:function(){v.bound=!1,s.$("container").on("mousemove click",v.showAll),"hover"==s._options.idleMode&&s.$("container").off("mouseleave",v.hide)},addTimer:function(){"hover"!=s._options.idleMode&&s.addTimer("idle",function(){v.hide()},s._options.idleTime)},hide:function(){if(s._options.idleMode&&s.getIndex()!==!1){s.trigger(i.IDLE_ENTER);var e=v.trunk.length;t.each(v.trunk,function(t,i){var n=i.data("idle");n&&(i.data("idle").complete=!1,O.animate(i,n.to,{duration:s._options.idleSpeed,complete:function(){t==e-1&&(v.active=!1)}}))})}},showAll:function(){s.clearTimer("idle"),t.each(v.trunk,function(t,e){v.show(e)})},show:function(e){var n=e.data("idle");v.active&&(n.busy||n.complete)||(n.busy=!0,s.trigger(i.IDLE_EXIT),s.clearTimer("idle"),O.animate(e,n.from,{duration:s._options.idleSpeed/2,complete:function(){v.active=!0,t(e).data("idle").busy=!1,t(e).data("idle").complete=!0}})),v.addTimer()}},y=this._lightbox={width:0,height:0,initialized:!1,active:null,image:null,elems:{},keymap:!1,init:function(){if(!y.initialized){y.initialized=!0;var e="overlay box content shadow title info close prevholder prev nextholder next counter image",n={},a=s._options,o="",r="position:absolute;",c="lightbox-",l={overlay:"position:fixed;display:none;opacity:"+a.overlayOpacity+";filter:alpha(opacity="+100*a.overlayOpacity+");top:0;left:0;width:100%;height:100%;background:"+a.overlayBackground+";z-index:99990",box:"position:fixed;display:none;width:400px;height:400px;top:50%;left:50%;margin-top:-200px;margin-left:-200px;z-index:99991",shadow:r+"background:#000;width:100%;height:100%;",content:r+"background-color:#fff;top:10px;left:10px;right:10px;bottom:10px;overflow:hidden",info:r+"bottom:10px;left:10px;right:10px;color:#444;font:11px/13px arial,sans-serif;height:13px",close:r+"top:10px;right:10px;height:20px;width:20px;background:#fff;text-align:center;cursor:pointer;color:#444;font:16px/22px arial,sans-serif;z-index:99999",image:r+"top:10px;left:10px;right:10px;bottom:30px;overflow:hidden;display:block;",prevholder:r+"width:50%;top:0;bottom:40px;cursor:pointer;",nextholder:r+"width:50%;top:0;bottom:40px;right:-1px;cursor:pointer;",prev:r+"top:50%;margin-top:-20px;height:40px;width:30px;background:#fff;left:20px;display:none;text-align:center;color:#000;font:bold 16px/36px arial,sans-serif",next:r+"top:50%;margin-top:-20px;height:40px;width:30px;background:#fff;right:20px;left:auto;display:none;font:bold 16px/36px arial,sans-serif;text-align:center;color:#000",title:"float:left",counter:"float:right;margin-left:8px;"},h=function(e){return e.hover(function(){t(this).css("color","#bbb")},function(){t(this).css("color","#444")})},u={},d="";d=_>7?9>_?"background:#000;filter:alpha(opacity=0);":"background:rgba(0,0,0,0);":"z-index:99999",l.nextholder+=d,l.prevholder+=d,t.each(l,function(t,e){o+=".galleria-"+c+t+"{"+e+"}"}),o+=".galleria-"+c+"box.iframe .galleria-"+c+"prevholder,.galleria-"+c+"box.iframe .galleria-"+c+"nextholder{width:100px;height:100px;top:50%;margin-top:-70px}",O.insertStyleTag(o,"galleria-lightbox"),t.each(e.split(" "),function(t,e){s.addElement("lightbox-"+e),n[e]=y.elems[e]=s.get("lightbox-"+e)}),y.image=new i.Picture,t.each({box:"shadow content close prevholder nextholder",info:"title counter",content:"info image",prevholder:"prev",nextholder:"next"},function(e,i){var n=[];t.each(i.split(" "),function(t,e){n.push(c+e)}),u[c+e]=n}),s.append(u),t(n.image).append(y.image.container),t(b().body).append(n.overlay,n.box),h(t(n.close).on("click:fast",y.hide).html("&#215;")),t.each(["Prev","Next"],function(e,a){var o=t(n[a.toLowerCase()]).html(/v/.test(a)?"&#8249;&#160;":"&#160;&#8250;"),r=t(n[a.toLowerCase()+"holder"]);return r.on("click:fast",function(){y["show"+a]()}),8>_||i.TOUCH?void o.show():void r.hover(function(){o.show()},function(){o.stop().fadeOut(200)})}),t(n.overlay).on("click:fast",y.hide),i.IPAD&&(s._options.lightboxTransitionSpeed=0)}},rescale:function(e){var n=m.min(r.width()-40,y.width),a=m.min(r.height()-60,y.height),o=m.min(n/y.width,a/y.height),c=m.round(y.width*o)+40,l=m.round(y.height*o)+60,h={width:c,height:l,"margin-top":-1*m.ceil(l/2),"margin-left":-1*m.ceil(c/2)};e?t(y.elems.box).css(h):t(y.elems.box).animate(h,{duration:s._options.lightboxTransitionSpeed,easing:s._options.easing,complete:function(){var e=y.image,n=s._options.lightboxFadeSpeed;s.trigger({type:i.LIGHTBOX_IMAGE,imageTarget:e.image}),t(e.container).show(),t(e.image).animate({opacity:1},n),O.show(y.elems.info,n)}})},hide:function(){y.image.image=null,r.off("resize",y.rescale),t(y.elems.box).hide().find("iframe").remove(),O.hide(y.elems.info),s.detachKeyboard(),s.attachKeyboard(y.keymap),y.keymap=!1,O.hide(y.elems.overlay,200,function(){t(this).hide().css("opacity",s._options.overlayOpacity),s.trigger(i.LIGHTBOX_CLOSE)})},showNext:function(){y.show(s.getNext(y.active))},showPrev:function(){y.show(s.getPrev(y.active))},show:function(n){y.active=n="number"==typeof n?n:s.getIndex()||0,y.initialized||y.init(),s.trigger(i.LIGHTBOX_OPEN),y.keymap||(y.keymap=t.extend({},s._keyboard.map),s.attachKeyboard({escape:y.hide,right:y.showNext,left:y.showPrev})),r.off("resize",y.rescale);var a,o,c,l=s.getData(n),h=s.getDataLength(),u=s.getNext(n);O.hide(y.elems.info);try{for(c=s._options.preload;c>0;c--)o=new i.Picture,a=s.getData(u),o.preload(a.big?a.big:a.image),u=s.getNext(u)}catch(d){}y.image.isIframe=l.iframe&&!l.image,t(y.elems.box).toggleClass("iframe",y.image.isIframe),t(y.image.container).find(".galleria-videoicon").remove(),y.image.load(l.big||l.image||l.iframe,function(i){if(i.isIframe){var a=t(e).width(),o=t(e).height();if(i.video&&s._options.maxVideoSize){var c=m.min(s._options.maxVideoSize/a,s._options.maxVideoSize/o);1>c&&(a*=c,o*=c)}y.width=a,y.height=o}else y.width=i.original.width,y.height=i.original.height;if(t(i.image).css({width:i.isIframe?"100%":"100.1%",height:i.isIframe?"100%":"100.1%",top:0,bottom:0,zIndex:99998,opacity:0,visibility:"visible"}).parent().height("100%"),y.elems.title.innerHTML=l.title||"",y.elems.counter.innerHTML=n+1+" / "+h,r.resize(y.rescale),y.rescale(),l.image&&l.iframe){if(t(y.elems.box).addClass("iframe"),l.video){var u=F(i.container).hide();e.setTimeout(function(){u.fadeIn(200)},200)}t(i.image).css("cursor","pointer").mouseup(function(e,i){return function(n){t(y.image.container).find(".galleria-videoicon").remove(),n.preventDefault(),i.isIframe=!0,i.load(e.iframe+(e.video?"&autoplay=1":""),{width:"100%",height:8>_?t(y.image.container).height():"100%"})}}(l,i))}}),t(y.elems.overlay).show().css("visibility","visible"),t(y.elems.box).show()}},x=this._timer={trunk:{},add:function(t,i,n,a){if(t=t||(new Date).getTime(),a=a||!1,this.clear(t),a){var o=i;i=function(){o(),x.add(t,i,n)}}this.trunk[t]=e.setTimeout(i,n)},clear:function(t){var i,n=function(t){e.clearTimeout(this.trunk[t]),delete this.trunk[t]};if(t&&t in this.trunk)n.call(this,t);else if("undefined"==typeof t)for(i in this.trunk)this.trunk.hasOwnProperty(i)&&n.call(this,i)}};return this},i.prototype={constructor:i,init:function(e,a){return a=k(a),this._original={target:e,options:a,data:null},this._target=this._dom.target=e.nodeName?e:t(e).get(0),this._original.html=this._target.innerHTML,$.push(this),this._target?(this._options={autoplay:!1,carousel:!0,carouselFollow:!0,carouselSpeed:400,carouselSteps:"auto",clicknext:!1,dailymotion:{foreground:"%23EEEEEE",highlight:"%235BCEC5",background:"%23222222",logo:0,hideInfos:1},dataConfig:function(){return{}},dataSelector:"img",dataSort:!1,dataSource:this._target,debug:n,dummy:n,easing:"galleria",extend:function(){},fullscreenCrop:n,fullscreenDoubleTap:!0,fullscreenTransition:n,height:0,idleMode:!0,idleTime:3e3,idleSpeed:200,imageCrop:!1,imageMargin:0,imagePan:!1,imagePanSmoothness:12,imagePosition:"50%",imageTimeout:n,initialTransition:n,keepSource:!1,layerFollow:!0,lightbox:!1,lightboxFadeSpeed:200,lightboxTransitionSpeed:200,linkSourceImages:!0,maxScaleRatio:n,maxVideoSize:n,minScaleRatio:n,overlayOpacity:.85,overlayBackground:"#0b0b0b",pauseOnInteraction:!0,popupLinks:!1,preload:2,queue:!0,responsive:!0,show:0,showInfo:!0,showCounter:!0,showImagenav:!0,swipe:"auto",thumbCrop:!0,thumbEventType:"click:fast",thumbMargin:0,thumbQuality:"auto",thumbDisplayOrder:!0,thumbPosition:"50%",thumbnails:!0,touchTransition:n,transition:"fade",transitionInitial:n,transitionSpeed:400,trueFullscreen:!0,useCanvas:!1,variation:"",videoPoster:!0,vimeo:{title:0,byline:0,portrait:0,color:"aaaaaa"},wait:5e3,width:"auto",youtube:{modestbranding:1,autohide:1,color:"white",hd:1,rel:0,showinfo:0}},this._options.initialTransition=this._options.initialTransition||this._options.transitionInitial,a&&a.debug===!1&&(l=!1),a&&"number"==typeof a.imageTimeout&&(u=a.imageTimeout),a&&"string"==typeof a.dummy&&(d=a.dummy),t(this._target).children().hide(),i.QUIRK&&i.raise("Your page is in Quirks mode, Galleria may not render correctly. Please validate your HTML and add a correct doctype."),"object"==typeof i.theme?this._init():z.push(this),this):void i.raise("Target not found",h)},_init:function(){var o=this,s=this._options;if(this._initialized)return i.raise("Init failed: Gallery instance already initialized."),this;
	if(this._initialized=!0,!i.theme)return i.raise("Init failed: No theme found.",h),this;if(t.extend(!0,s,i.theme.defaults,this._original.options,i.configure.options),s.swipe=function(t){return"enforced"==t?!0:t===!1||"disabled"==t?!1:!!i.TOUCH}(s.swipe),s.swipe&&(s.clicknext=!1,s.imagePan=!1),function(t){return"getContext"in t?void(P=P||{elem:t,context:t.getContext("2d"),cache:{},length:0}):void(t=null)}(a.createElement("canvas")),this.bind(i.DATA,function(){e.screen&&e.screen.width&&Array.prototype.forEach&&this._data.forEach(function(t){var i="devicePixelRatio"in e?e.devicePixelRatio:1,n=m.max(e.screen.width,e.screen.height);1024>n*i&&(t.big=t.image)}),this._original.data=this._data,this.get("total").innerHTML=this.getDataLength();var t=this.$("container");o._options.height<2&&(o._userRatio=o._ratio=o._options.height);var n={width:0,height:0},a=function(){return o.$("stage").height()};O.wait({until:function(){return n=o._getWH(),t.width(n.width).height(n.height),a()&&n.width&&n.height>50},success:function(){o._width=n.width,o._height=n.height,o._ratio=o._ratio||n.height/n.width,i.WEBKIT?e.setTimeout(function(){o._run()},1):o._run()},error:function(){a()?i.raise("Could not extract sufficient width/height of the gallery container. Traced measures: width:"+n.width+"px, height: "+n.height+"px.",h):i.raise("Could not extract a stage height from the CSS. Traced height: "+a()+"px.",h)},timeout:"number"==typeof this._options.wait?this._options.wait:!1})}),this.append({"info-text":["info-title","info-description"],info:["info-text"],"image-nav":["image-nav-right","image-nav-left"],stage:["images","loader","counter","image-nav"],"thumbnails-list":["thumbnails"],"thumbnails-container":["thumb-nav-left","thumbnails-list","thumb-nav-right"],container:["stage","thumbnails-container","info","tooltip"]}),O.hide(this.$("counter").append(this.get("current"),a.createTextNode(" / "),this.get("total"))),this.setCounter("&#8211;"),O.hide(o.get("tooltip")),this.$("container").addClass((i.TOUCH?"touch":"notouch")+" "+this._options.variation),this._options.swipe||t.each(new Array(2),function(e){var n=new i.Picture;t(n.container).css({position:"absolute",top:0,left:0}).prepend(o._layers[e]=t(O.create("galleria-layer")).css({position:"absolute",top:0,left:0,right:0,bottom:0,zIndex:2})[0]),o.$("images").append(n.container),o._controls[e]=n;var a=new i.Picture;a.isIframe=!0,t(a.container).attr("class","galleria-frame").css({position:"absolute",top:0,left:0,zIndex:4,background:"#000",display:"none"}).appendTo(n.container),o._controls.frames[e]=a}),this.$("images").css({position:"relative",top:0,left:0,width:"100%",height:"100%"}),s.swipe&&(this.$("images").css({position:"absolute",top:0,left:0,width:0,height:"100%"}),this.finger=new i.Finger(this.get("stage"),{onchange:function(t){o.pause().show(t)},oncomplete:function(e){var i=m.max(0,m.min(parseInt(e,10),o.getDataLength()-1)),n=o.getData(i);t(o._thumbnails[i].container).addClass("active").siblings(".active").removeClass("active"),n&&(o.$("images").find("iframe").remove(),o.$("images").find(".galleria-frame").css("opacity",0).hide(),o._options.carousel&&o._options.carouselFollow&&o._carousel.follow(i))}}),this.bind(i.RESCALE,function(){this.finger.setup()}),this.$("stage").on("click",function(){var i=o.getData();if(i){if(i.iframe){o.isPlaying()&&o.pause();var a=o._controls.frames[o._active],r=o._stageWidth,s=o._stageHeight;if(t(a.container).find("iframe").length)return;return t(a.container).css({width:r,height:s,opacity:0}).show().animate({opacity:1},200),void e.setTimeout(function(){a.load(i.iframe+(i.video?"&autoplay=1":""),{width:r,height:s},function(t){o.$("container").addClass("videoplay"),t.scale({width:o._stageWidth,height:o._stageHeight,iframelimit:i.video?o._options.maxVideoSize:n})})},100)}if(i.link)if(o._options.popupLinks){e.open(i.link,"_blank")}else e.location.href=i.link;else;}}),this.bind(i.IMAGE,function(e){o.setCounter(e.index),o.setInfo(e.index);var i=this.getNext(),n=this.getPrev(),a=[n,i];a.push(this.getNext(i),this.getPrev(n),o._controls.slides.length-1);var r=[];t.each(a,function(e,i){-1==t.inArray(i,r)&&r.push(i)}),t.each(r,function(e,i){var n=o.getData(i),a=o._controls.slides[i],r=o.isFullscreen()&&n.big?n.big:n.image||n.iframe;n.iframe&&!n.image&&(a.isIframe=!0),a.ready||o._controls.slides[i].load(r,function(e){e.isIframe||t(e.image).css("visibility","hidden"),o._scaleImage(e,{complete:function(e){e.isIframe||t(e.image).css({opacity:0,visibility:"visible"}).animate({opacity:1},200)}})})})})),this.$("thumbnails, thumbnails-list").css({overflow:"hidden",position:"relative"}),this.$("image-nav-right, image-nav-left").on("click:fast",function(){s.pauseOnInteraction&&o.pause();var t=/right/.test(this.className)?"next":"prev";o[t]()}).on("click",function(t){t.preventDefault(),(s.clicknext||s.swipe)&&t.stopPropagation()}),t.each(["info","counter","image-nav"],function(t,e){s["show"+e.substr(0,1).toUpperCase()+e.substr(1).replace(/-/,"")]===!1&&O.moveOut(o.get(e.toLowerCase()))}),this.load(),s.keepSource||_||(this._target.innerHTML=""),this.get("errors")&&this.appendChild("target","errors"),this.appendChild("target","container"),s.carousel){var c=0,l=s.show;this.bind(i.THUMBNAIL,function(){this.updateCarousel(),++c==this.getDataLength()&&"number"==typeof l&&l>0&&this._carousel.follow(l)})}return s.responsive&&r.on("resize",function(){o.isFullscreen()||o.resize()}),s.fullscreenDoubleTap&&this.$("stage").on("touchstart",function(){var t,e,i,n,a,r,s=function(t){return t.originalEvent.touches?t.originalEvent.touches[0]:t};return o.$("stage").on("touchmove",function(){t=0}),function(c){if(!/(-left|-right)/.test(c.target.className)){if(r=O.timestamp(),e=s(c).pageX,i=s(c).pageY,c.originalEvent.touches.length<2&&300>r-t&&20>e-n&&20>i-a)return o.toggleFullscreen(),void c.preventDefault();t=r,n=e,a=i}}}()),t.each(i.on.binds,function(e,i){-1==t.inArray(i.hash,o._binds)&&o.bind(i.type,i.callback)}),this},addTimer:function(){return this._timer.add.apply(this._timer,O.array(arguments)),this},clearTimer:function(){return this._timer.clear.apply(this._timer,O.array(arguments)),this},_getWH:function(){var e,i=this.$("container"),n=this.$("target"),a=this,o={};return t.each(["width","height"],function(t,r){a._options[r]&&"number"==typeof a._options[r]?o[r]=a._options[r]:(e=[O.parseValue(i.css(r)),O.parseValue(n.css(r)),i[r](),n[r]()],a["_"+r]||e.splice(e.length,O.parseValue(i.css("min-"+r)),O.parseValue(n.css("min-"+r))),o[r]=m.max.apply(m,e))}),a._userRatio&&(o.height=o.width*a._userRatio),o},_createThumbnails:function(n){this.get("total").innerHTML=this.getDataLength();var o,r,s,c,l=this,h=this._options,u=n?this._data.length-n.length:0,d=u,p=[],f=0,g=8>_?"http://upload.wikimedia.org/wikipedia/commons/c/c0/Blank.gif":"data:image/gif;base64,R0lGODlhAQABAPABAP///wAAACH5BAEKAAAALAAAAAABAAEAAAICRAEAOw%3D%3D",m=function(){var t=l.$("thumbnails").find(".active");return t.length?t.find("img").attr("src"):!1}(),v="string"==typeof h.thumbnails?h.thumbnails.toLowerCase():null,y=function(t){return a.defaultView&&a.defaultView.getComputedStyle?a.defaultView.getComputedStyle(r.container,null)[t]:c.css(t)},b=function(e,n,a){return function(){t(a).append(e),l.trigger({type:i.THUMBNAIL,thumbTarget:e,index:n,galleriaData:l.getData(n)})}},w=function(e){h.pauseOnInteraction&&l.pause();var i=t(e.currentTarget).data("index");l.getIndex()!==i&&l.show(i),e.preventDefault()},x=function(e,n){t(e.container).css("visibility","visible"),l.trigger({type:i.THUMBNAIL,thumbTarget:e.image,index:e.data.order,galleriaData:l.getData(e.data.order)}),"function"==typeof n&&n.call(l,e)},T=function(e,i){e.scale({width:e.data.width,height:e.data.height,crop:h.thumbCrop,margin:h.thumbMargin,canvas:h.useCanvas,position:h.thumbPosition,complete:function(e){{var n,a,o=["left","top"],r=["Width","Height"];l.getData(e.index)}t.each(r,function(i,r){n=r.toLowerCase(),(h.thumbCrop!==!0||h.thumbCrop===n)&&(a={},a[n]=e[n],t(e.container).css(a),a={},a[o[i]]=0,t(e.image).css(a)),e["outer"+r]=t(e.container)["outer"+r](!0)}),O.toggleQuality(e.image,h.thumbQuality===!0||"auto"===h.thumbQuality&&e.original.width<3*e.width),h.thumbDisplayOrder&&!e.lazy?t.each(p,function(t,e){return t===f&&e.ready&&!e.displayed?(f++,e.displayed=!0,void x(e,i)):void 0}):x(e,i)}})};for(n||(this._thumbnails=[],this.$("thumbnails").empty());this._data[u];u++)s=this._data[u],o=s.thumb||s.image,h.thumbnails!==!0&&"lazy"!=v||!s.thumb&&!s.image?s.iframe||"empty"===v||"numbers"===v?(r={container:O.create("galleria-image"),image:O.create("img","span"),ready:!0,data:{order:u}},"numbers"===v&&t(r.image).text(u+1),s.iframe&&t(r.image).addClass("iframe"),this.$("thumbnails").append(r.container),e.setTimeout(b(r.image,u,r.container),50+20*u)):r={container:null,image:null}:(r=new i.Picture(u),r.index=u,r.displayed=!1,r.lazy=!1,r.video=!1,this.$("thumbnails").append(r.container),c=t(r.container),c.css("visibility","hidden"),r.data={width:O.parseValue(y("width")),height:O.parseValue(y("height")),order:u,src:o},c.css(h.thumbCrop!==!0?{width:"auto",height:"auto"}:{width:r.data.width,height:r.data.height}),"lazy"==v?(c.addClass("lazy"),r.lazy=!0,r.load(g,{height:r.data.height,width:r.data.width})):r.load(o,T),"all"===h.preload&&r.preload(s.image)),t(r.container).add(h.keepSource&&h.linkSourceImages?s.original:null).data("index",u).on(h.thumbEventType,w).data("thumbload",T),m===o&&t(r.container).addClass("active"),this._thumbnails.push(r);return p=this._thumbnails.slice(d),this},lazyLoad:function(e,i){var n=e.constructor==Array?e:[e],a=this,o=0;return t.each(n,function(e,r){if(!(r>a._thumbnails.length-1)){var s=a._thumbnails[r],c=s.data,l=function(){++o==n.length&&"function"==typeof i&&i.call(a)},h=t(s.container).data("thumbload");s.video?h.call(a,s,l):s.load(c.src,function(t){h.call(a,t,l)})}}),this},lazyLoadChunks:function(t,i){var n=this.getDataLength(),a=0,o=0,r=[],s=[],c=this;for(i=i||0;n>a;a++)s.push(a),(++o==t||a==n-1)&&(r.push(s),o=0,s=[]);var l=function(t){var n=r.shift();n&&e.setTimeout(function(){c.lazyLoad(n,function(){l(!0)})},i&&t?i:0)};return l(!1),this},_run:function(){var a=this;a._createThumbnails(),O.wait({timeout:1e4,until:function(){return i.OPERA&&a.$("stage").css("display","inline-block"),a._stageWidth=a.$("stage").width(),a._stageHeight=a.$("stage").height(),a._stageWidth&&a._stageHeight>50},success:function(){if(D.push(a),a._options.swipe){var o=a.$("images").width(a.getDataLength()*a._stageWidth);t.each(new Array(a.getDataLength()),function(e){var n=new i.Picture,r=a.getData(e);t(n.container).css({position:"absolute",top:0,left:a._stageWidth*e}).prepend(a._layers[e]=t(O.create("galleria-layer")).css({position:"absolute",top:0,left:0,right:0,bottom:0,zIndex:2})[0]).appendTo(o),r.video&&F(n.container),a._controls.slides.push(n);var s=new i.Picture;s.isIframe=!0,t(s.container).attr("class","galleria-frame").css({position:"absolute",top:0,left:0,zIndex:4,background:"#000",display:"none"}).appendTo(n.container),a._controls.frames.push(s)}),a.finger.setup()}return O.show(a.get("counter")),a._options.carousel&&a._carousel.bindControls(),a._options.autoplay&&(a.pause(),"number"==typeof a._options.autoplay&&(a._playtime=a._options.autoplay),a._playing=!0),a._firstrun?(a._options.autoplay&&a.trigger(i.PLAY),void("number"==typeof a._options.show&&a.show(a._options.show))):(a._firstrun=!0,i.History&&i.History.change(function(t){isNaN(t)?e.history.go(-1):a.show(t,n,!0)}),a.trigger(i.READY),i.theme.init.call(a,a._options),t.each(i.ready.callbacks,function(t,e){"function"==typeof e&&e.call(a,a._options)}),a._options.extend.call(a,a._options),/^[0-9]{1,4}$/.test(f)&&i.History?a.show(f,n,!0):a._data[a._options.show]&&a.show(a._options.show),void(a._options.autoplay&&a.trigger(i.PLAY)))},error:function(){i.raise("Stage width or height is too small to show the gallery. Traced measures: width:"+a._stageWidth+"px, height: "+a._stageHeight+"px.",h)}})},load:function(e,n,a){var o=this,r=this._options;return this._data=[],this._thumbnails=[],this.$("thumbnails").empty(),"function"==typeof n&&(a=n,n=null),e=e||r.dataSource,n=n||r.dataSelector,a=a||r.dataConfig,t.isPlainObject(e)&&(e=[e]),t.isArray(e)?this.validate(e)?this._data=e:i.raise("Load failed: JSON Array not valid."):(n+=",.video,.iframe",t(e).find(n).each(function(e,i){i=t(i);var n={},r=i.parent(),s=r.attr("href"),c=r.attr("rel");s&&("IMG"==i[0].nodeName||i.hasClass("video"))&&A(s)?n.video=s:s&&i.hasClass("iframe")?n.iframe=s:n.image=n.big=s,c&&(n.big=c),t.each("big title description link layer image".split(" "),function(t,e){i.data(e)&&(n[e]=i.data(e).toString())}),n.big||(n.big=n.image),o._data.push(t.extend({title:i.attr("title")||"",thumb:i.attr("src"),image:i.attr("src"),big:i.attr("src"),description:i.attr("alt")||"",link:i.attr("longdesc"),original:i.get(0)},n,a(i)))})),"function"==typeof r.dataSort?s.sort.call(this._data,r.dataSort):"random"==r.dataSort&&this._data.sort(function(){return m.round(m.random())-.5}),this.getDataLength()&&this._parseData(function(){this.trigger(i.DATA)}),this},_parseData:function(e){var i,a=this,o=!1,r=function(){var i=!0;t.each(a._data,function(t,e){return e.loading?(i=!1,!1):void 0}),i&&!o&&(o=!0,e.call(a))};return t.each(this._data,function(e,o){if(i=a._data[e],"thumb"in o==!1&&(i.thumb=o.image),o.big||(i.big=o.image),"video"in o){var s=A(o.video);s&&(i.iframe=new S(s.provider,s.id).embed()+function(){if("object"==typeof a._options[s.provider]){var e="?",i=[];return t.each(a._options[s.provider],function(t,e){i.push(t+"="+e)}),"youtube"==s.provider&&(i=["wmode=opaque"].concat(i)),e+i.join("&")}return""}(),i.thumb&&i.image||t.each(["thumb","image"],function(t,e){if("image"==e&&!a._options.videoPoster)return void(i.image=n);var o=new S(s.provider,s.id);i[e]||(i.loading=!0,o.getMedia(e,function(t,e){return function(i){t[e]=i,"image"!=e||t.big||(t.big=t.image),delete t.loading,r()}}(i,e)))}))}}),r(),this},destroy:function(){return this.$("target").data("galleria",null),this.$("container").off("galleria"),this.get("target").innerHTML=this._original.html,this.clearTimer(),O.removeFromArray($,this),O.removeFromArray(D,this),i._waiters.length&&t.each(i._waiters,function(t,i){i&&e.clearTimeout(i)}),this},splice:function(){var t=this,i=O.array(arguments);return e.setTimeout(function(){s.splice.apply(t._data,i),t._parseData(function(){t._createThumbnails()})},2),t},push:function(){var t=this,i=O.array(arguments);return 1==i.length&&i[0].constructor==Array&&(i=i[0]),e.setTimeout(function(){s.push.apply(t._data,i),t._parseData(function(){t._createThumbnails(i)})},2),t},_getActive:function(){return this._controls.getActive()},validate:function(){return!0},bind:function(t,e){return t=C(t),this.$("container").on(t,this.proxy(e)),this},unbind:function(t){return t=C(t),this.$("container").off(t),this},trigger:function(e){return e="object"==typeof e?t.extend(e,{scope:this}):{type:C(e),scope:this},this.$("container").trigger(e),this},addIdleState:function(){return this._idle.add.apply(this._idle,O.array(arguments)),this},removeIdleState:function(){return this._idle.remove.apply(this._idle,O.array(arguments)),this},enterIdleMode:function(){return this._idle.hide(),this},exitIdleMode:function(){return this._idle.showAll(),this},enterFullscreen:function(){return this._fullscreen.enter.apply(this,O.array(arguments)),this},exitFullscreen:function(){return this._fullscreen.exit.apply(this,O.array(arguments)),this},toggleFullscreen:function(){return this._fullscreen[this.isFullscreen()?"exit":"enter"].apply(this,O.array(arguments)),this},bindTooltip:function(){return this._tooltip.bind.apply(this._tooltip,O.array(arguments)),this},defineTooltip:function(){return this._tooltip.define.apply(this._tooltip,O.array(arguments)),this},refreshTooltip:function(){return this._tooltip.show.apply(this._tooltip,O.array(arguments)),this},openLightbox:function(){return this._lightbox.show.apply(this._lightbox,O.array(arguments)),this},closeLightbox:function(){return this._lightbox.hide.apply(this._lightbox,O.array(arguments)),this},hasVariation:function(e){return t.inArray(e,this._options.variation.split(/\s+/))>-1},getActiveImage:function(){var t=this._getActive();return t?t.image:n},getActiveThumb:function(){return this._thumbnails[this._active].image||n},getMousePosition:function(t){return{x:t.pageX-this.$("container").offset().left,y:t.pageY-this.$("container").offset().top}},addPan:function(e){if(this._options.imageCrop!==!1){e=t(e||this.getActiveImage());var i=this,n=e.width()/2,a=e.height()/2,o=parseInt(e.css("left"),10),r=parseInt(e.css("top"),10),s=o||0,c=r||0,l=0,h=0,u=!1,d=O.timestamp(),p=0,f=0,g=function(t,i,n){if(t>0&&(f=m.round(m.max(-1*t,m.min(0,i))),p!==f))if(p=f,8===_)e.parent()["scroll"+n](-1*f);else{var a={};a[n.toLowerCase()]=f,e.css(a)}},v=function(t){O.timestamp()-d<50||(u=!0,n=i.getMousePosition(t).x,a=i.getMousePosition(t).y)},y=function(){u&&(l=e.width()-i._stageWidth,h=e.height()-i._stageHeight,o=n/i._stageWidth*l*-1,r=a/i._stageHeight*h*-1,s+=(o-s)/i._options.imagePanSmoothness,c+=(r-c)/i._options.imagePanSmoothness,g(h,c,"Top"),g(l,s,"Left"))};return 8===_&&(e.parent().scrollTop(-1*c).scrollLeft(-1*s),e.css({top:0,left:0})),this.$("stage").off("mousemove",v).on("mousemove",v),this.addTimer("pan"+i._id,y,50,!0),this}},proxy:function(t,e){return"function"!=typeof t?v:(e=e||this,function(){return t.apply(e,O.array(arguments))})},removePan:function(){return this.$("stage").off("mousemove"),this.clearTimer("pan"+this._id),this},addElement:function(){var e=this._dom;return t.each(O.array(arguments),function(t,i){e[i]=O.create("galleria-"+i)}),this},attachKeyboard:function(){return this._keyboard.attach.apply(this._keyboard,O.array(arguments)),this},detachKeyboard:function(){return this._keyboard.detach.apply(this._keyboard,O.array(arguments)),this},appendChild:function(t,e){return this.$(t).append(this.get(e)||e),this},prependChild:function(t,e){return this.$(t).prepend(this.get(e)||e),this},remove:function(){return this.$(O.array(arguments).join(",")).remove(),this},append:function(t){var e,i;for(e in t)if(t.hasOwnProperty(e))if(t[e].constructor===Array)for(i=0;t[e][i];i++)this.appendChild(e,t[e][i]);else this.appendChild(e,t[e]);return this},_scaleImage:function(e,i){if(e=e||this._controls.getActive()){var n,a=function(e){t(e.container).children(":first").css({top:m.max(0,O.parseValue(e.image.style.top)),left:m.max(0,O.parseValue(e.image.style.left)),width:O.parseValue(e.image.width),height:O.parseValue(e.image.height)})};return i=t.extend({width:this._stageWidth,height:this._stageHeight,crop:this._options.imageCrop,max:this._options.maxScaleRatio,min:this._options.minScaleRatio,margin:this._options.imageMargin,position:this._options.imagePosition,iframelimit:this._options.maxVideoSize},i),this._options.layerFollow&&this._options.imageCrop!==!0?"function"==typeof i.complete?(n=i.complete,i.complete=function(){n.call(e,e),a(e)}):i.complete=a:t(e.container).children(":first").css({top:0,left:0}),e.scale(i),this}},updateCarousel:function(){return this._carousel.update(),this},resize:function(e,i){"function"==typeof e&&(i=e,e=n),e=t.extend({width:0,height:0},e);var a=this,o=this.$("container");return t.each(e,function(t,i){i||(o[t]("auto"),e[t]=a._getWH()[t])}),t.each(e,function(t,e){o[t](e)}),this.rescale(i)},rescale:function(e,a,o){var r=this;"function"==typeof e&&(o=e,e=n);var s=function(){r._stageWidth=e||r.$("stage").width(),r._stageHeight=a||r.$("stage").height(),r._options.swipe?(t.each(r._controls.slides,function(e,i){r._scaleImage(i),t(i.container).css("left",r._stageWidth*e)}),r.$("images").css("width",r._stageWidth*r.getDataLength())):r._scaleImage(),r._options.carousel&&r.updateCarousel();var n=r._controls.frames[r._controls.active];n&&r._controls.frames[r._controls.active].scale({width:r._stageWidth,height:r._stageHeight,iframelimit:r._options.maxVideoSize}),r.trigger(i.RESCALE),"function"==typeof o&&o.call(r)};return s.call(r),this},refreshImage:function(){return this._scaleImage(),this._options.imagePan&&this.addPan(),this},_preload:function(){if(this._options.preload){var t,e,n,a=this.getNext();try{for(e=this._options.preload;e>0;e--)t=new i.Picture,n=this.getData(a),t.preload(this.isFullscreen()&&n.big?n.big:n.image),a=this.getNext(a)}catch(o){}}},show:function(n,a,o){var r=this._options.swipe;if(r||!(this._queue.length>3||n===!1||!this._options.queue&&this._queue.stalled)){if(n=m.max(0,m.min(parseInt(n,10),this.getDataLength()-1)),a="undefined"!=typeof a?!!a:n<this.getIndex(),o=o||!1,!o&&i.History)return void i.History.set(n.toString());if(this.finger&&n!==this._active&&(this.finger.to=-(n*this.finger.width),this.finger.index=n),this._active=n,r){var c=this.getData(n),l=this;if(!c)return;var h=this.isFullscreen()&&c.big?c.big:c.image||c.iframe,u=this._controls.slides[n],d=u.isCached(h),p=this._thumbnails[n],f={cached:d,index:n,rewind:a,imageTarget:u.image,thumbTarget:p.image,galleriaData:c};this.trigger(t.extend(f,{type:i.LOADSTART})),l.$("container").removeClass("videoplay");var g=function(){l._layers[n].innerHTML=l.getData().layer||"",l.trigger(t.extend(f,{type:i.LOADFINISH})),l._playCheck()};l._preload(),e.setTimeout(function(){u.ready&&t(u.image).attr("src")==h?(l.trigger(t.extend(f,{type:i.IMAGE})),g()):(c.iframe&&!c.image&&(u.isIframe=!0),u.load(h,function(e){l._scaleImage(e,g).trigger(t.extend(f,{type:i.IMAGE})),g()}))},100)}else s.push.call(this._queue,{index:n,rewind:a}),this._queue.stalled||this._show();return this}},_show:function(){var a=this,o=this._queue[0],r=this.getData(o.index);if(r){var c=this.isFullscreen()&&r.big?r.big:r.image||r.iframe,l=this._controls.getActive(),h=this._controls.getNext(),u=h.isCached(c),d=this._thumbnails[o.index],p=function(){t(h.image).trigger("mouseup")};a.$("container").toggleClass("iframe",!!r.isIframe).removeClass("videoplay");var f=function(o,r,c,l,h){return function(){var u;N.active=!1,O.toggleQuality(r.image,a._options.imageQuality),a._layers[a._controls.active].innerHTML="",t(c.container).css({zIndex:0,opacity:0}).show(),t(c.container).find("iframe, .galleria-videoicon").remove(),t(a._controls.frames[a._controls.active].container).hide(),t(r.container).css({zIndex:1,left:0,top:0}).show(),a._controls.swap(),a._options.imagePan&&a.addPan(r.image),(o.iframe&&o.image||o.link||a._options.lightbox||a._options.clicknext)&&t(r.image).css({cursor:"pointer"}).on("mouseup",function(r){if(!("number"==typeof r.which&&r.which>1)){if(o.iframe){a.isPlaying()&&a.pause();var s=a._controls.frames[a._controls.active],c=a._stageWidth,l=a._stageHeight;return t(s.container).css({width:c,height:l,opacity:0}).show().animate({opacity:1},200),void e.setTimeout(function(){s.load(o.iframe+(o.video?"&autoplay=1":""),{width:c,height:l},function(t){a.$("container").addClass("videoplay"),t.scale({width:a._stageWidth,height:a._stageHeight,iframelimit:o.video?a._options.maxVideoSize:n})})},100)}return a._options.clicknext&&!i.TOUCH?(a._options.pauseOnInteraction&&a.pause(),void a.next()):o.link?void(a._options.popupLinks?u=e.open(o.link,"_blank"):e.location.href=o.link):void(a._options.lightbox&&a.openLightbox())}}),a._playCheck(),a.trigger({type:i.IMAGE,index:l.index,imageTarget:r.image,thumbTarget:h.image,galleriaData:o}),s.shift.call(a._queue),a._queue.stalled=!1,a._queue.length&&a._show()}}(r,h,l,o,d);this._options.carousel&&this._options.carouselFollow&&this._carousel.follow(o.index),a._preload(),O.show(h.container),h.isIframe=r.iframe&&!r.image,t(a._thumbnails[o.index].container).addClass("active").siblings(".active").removeClass("active"),a.trigger({type:i.LOADSTART,cached:u,index:o.index,rewind:o.rewind,imageTarget:h.image,thumbTarget:d.image,galleriaData:r}),a._queue.stalled=!0,h.load(c,function(e){var s=t(a._layers[1-a._controls.active]).html(r.layer||"").hide();a._scaleImage(e,{complete:function(e){"image"in l&&O.toggleQuality(l.image,!1),O.toggleQuality(e.image,!1),a.removePan(),a.setInfo(o.index),a.setCounter(o.index),r.layer&&(s.show(),(r.iframe&&r.image||r.link||a._options.lightbox||a._options.clicknext)&&s.css("cursor","pointer").off("mouseup").mouseup(p)),r.video&&r.image&&F(e.container);var c=a._options.transition;if(t.each({initial:null===l.image,touch:i.TOUCH,fullscreen:a.isFullscreen()},function(t,e){return e&&a._options[t+"Transition"]!==n?(c=a._options[t+"Transition"],!1):void 0}),c in N.effects==!1)f();else{var h={prev:l.container,next:e.container,rewind:o.rewind,speed:a._options.transitionSpeed||400};N.active=!0,N.init.call(a,c,h,f)}a.trigger({type:i.LOADFINISH,cached:u,index:o.index,rewind:o.rewind,imageTarget:e.image,thumbTarget:a._thumbnails[o.index].image,galleriaData:a.getData(o.index)})}})})}},getNext:function(t){return t="number"==typeof t?t:this.getIndex(),t===this.getDataLength()-1?0:t+1},getPrev:function(t){return t="number"==typeof t?t:this.getIndex(),0===t?this.getDataLength()-1:t-1},next:function(){return this.getDataLength()>1&&this.show(this.getNext(),!1),this},prev:function(){return this.getDataLength()>1&&this.show(this.getPrev(),!0),this},get:function(t){return t in this._dom?this._dom[t]:null},getData:function(t){return t in this._data?this._data[t]:this._data[this._active]},getDataLength:function(){return this._data.length},getIndex:function(){return"number"==typeof this._active?this._active:!1},getStageHeight:function(){return this._stageHeight},getStageWidth:function(){return this._stageWidth},getOptions:function(t){return"undefined"==typeof t?this._options:this._options[t]},setOptions:function(e,i){return"object"==typeof e?t.extend(this._options,e):this._options[e]=i,this},play:function(t){return this._playing=!0,this._playtime=t||this._playtime,this._playCheck(),this.trigger(i.PLAY),this},pause:function(){return this._playing=!1,this.trigger(i.PAUSE),this},playToggle:function(t){return this._playing?this.pause():this.play(t)},isPlaying:function(){return this._playing},isFullscreen:function(){return this._fullscreen.active},_playCheck:function(){var t=this,e=0,n=20,a=O.timestamp(),o="play"+this._id;if(this._playing){this.clearTimer(o);var r=function(){return e=O.timestamp()-a,e>=t._playtime&&t._playing?(t.clearTimer(o),void t.next()):void(t._playing&&(t.trigger({type:i.PROGRESS,percent:m.ceil(e/t._playtime*100),seconds:m.floor(e/1e3),milliseconds:e}),t.addTimer(o,r,n)))};t.addTimer(o,r,n)}},setPlaytime:function(t){return this._playtime=t,this},setIndex:function(t){return this._active=t,this},setCounter:function(t){if("number"==typeof t?t++:"undefined"==typeof t&&(t=this.getIndex()+1),this.get("current").innerHTML=t,_){var e=this.$("counter"),i=e.css("opacity");1===parseInt(i,10)?O.removeAlpha(e[0]):this.$("counter").css("opacity",i)}return this},setInfo:function(e){var i=this,n=this.getData(e);return t.each(["title","description"],function(t,e){var a=i.$("info-"+e);n[e]?a[n[e].length?"show":"hide"]().html(n[e]):a.empty().hide()}),this},hasInfo:function(t){var e,i="title description".split(" ");for(e=0;i[e];e++)if(this.getData(t)[i[e]])return!0;return!1},jQuery:function(e){var i=this,n=[];t.each(e.split(","),function(e,a){a=t.trim(a),i.get(a)&&n.push(a)});var a=t(i.get(n.shift()));return t.each(n,function(t,e){a=a.add(i.get(e))}),a},$:function(){return this.jQuery.apply(this,O.array(arguments))}},t.each(T,function(t,e){var n=/_/.test(e)?e.replace(/_/g,""):e;i[e.toUpperCase()]="galleria."+n}),t.extend(i,{IE9:9===_,IE8:8===_,IE7:7===_,IE6:6===_,IE:_,WEBKIT:/webkit/.test(p),CHROME:/chrome/.test(p),SAFARI:/safari/.test(p)&&!/chrome/.test(p),QUIRK:_&&a.compatMode&&"BackCompat"===a.compatMode,MAC:/mac/.test(navigator.platform.toLowerCase()),OPERA:!!e.opera,IPHONE:/iphone/.test(p),IPAD:/ipad/.test(p),ANDROID:/android/.test(p),TOUCH:"ontouchstart"in a}),i.addTheme=function(n){n.name||i.raise("No theme name specified"),n.defaults="object"!=typeof n.defaults?{}:k(n.defaults);var a,o=!1;return"string"==typeof n.css?(t("link").each(function(t,e){return a=new RegExp(n.css),a.test(e.href)?(o=!0,H(n),!1):void 0}),o||t(function(){var r=0,s=function(){t("script").each(function(t,i){a=new RegExp("galleria\\."+n.name.toLowerCase()+"\\."),a.test(i.src)&&(o=i.src.replace(/[^\/]*$/,"")+n.css,e.setTimeout(function(){O.loadCSS(o,"galleria-theme",function(){H(n)})},1))}),o||(r++>5?i.raise("No theme CSS loaded"):e.setTimeout(s,500))};s()})):H(n),n},i.loadTheme=function(n){if(!t("script").filter(function(){return t(this).attr("src")==n}).length){var a,o=!1;return t(e).load(function(){o||(a=e.setTimeout(function(){o||i.theme||i.raise("Galleria had problems loading theme at "+n+". Please check theme path or load manually.",h)},2e4))}),i.unloadTheme(),O.loadScript(n,function(){o=!0,e.clearTimeout(a)}),i}},i.unloadTheme=function(){return"object"==typeof i.theme&&(t("script").each(function(e,n){new RegExp("galleria\\."+i.theme.name+"\\.").test(n.src)&&t(n).remove()}),i.theme=n),i},i.get=function(t){return $[t]?$[t]:"number"!=typeof t?$:void i.raise("Gallery index "+t+" not found")},i.configure=function(e,n){var a={};return"string"==typeof e&&n?(a[e]=n,e=a):t.extend(a,e),i.configure.options=a,t.each(i.get(),function(t,e){e.setOptions(a)}),i},i.configure.options={},i.on=function(e,n){if(e){n=n||v;var a=e+n.toString().replace(/\s/g,"")+O.timestamp();return t.each(i.get(),function(t,i){i._binds.push(a),i.bind(e,n)}),i.on.binds.push({type:e,callback:n,hash:a}),i}},i.on.binds=[],i.run=function(e,n){return t.isFunction(n)&&(n={extend:n}),t(e||"#galleria").galleria(n),i},i.addTransition=function(t,e){return N.effects[t]=e,i},i.utils=O,i.log=function(){var i=O.array(arguments);if(!("console"in e&&"log"in e.console))return e.alert(i.join("<br>"));try{return e.console.log.apply(e.console,i)}catch(n){t.each(i,function(){e.console.log(this)})}},i.ready=function(e){return"function"!=typeof e?i:(t.each(D,function(t,i){e.call(i,i._options)}),i.ready.callbacks.push(e),i)},i.ready.callbacks=[],i.raise=function(e,i){var n=i?"Fatal error":"Error",a={color:"#fff",position:"absolute",top:0,left:0,zIndex:1e5},o=function(e){var o='<div style="padding:4px;margin:0 0 2px;background:#'+(i?"811":"222")+';">'+(i?"<strong>"+n+": </strong>":"")+e+"</div>";t.each($,function(){var t=this.$("errors"),e=this.$("target");t.length||(e.css("position","relative"),t=this.addElement("errors").appendChild("target","errors").$("errors").css(a)),t.append(o)}),$.length||t("<div>").css(t.extend(a,{position:"fixed"})).append(o).appendTo(b().body)};if(l){if(o(e),i)throw new Error(n+": "+e)}else if(i){if(L)return;L=!0,i=!1,o("Gallery could not load.")}},i.version=c,i.requires=function(t,e){return e=e||"You need to upgrade Galleria to version "+t+" to use one or more components.",i.version<t&&i.raise(e,h),i},i.Picture=function(e){this.id=e||null,this.image=null,this.container=O.create("galleria-image"),t(this.container).css({overflow:"hidden",position:"relative"}),this.original={width:0,height:0},this.ready=!1,this.isIframe=!1},i.Picture.prototype={cache:{},show:function(){O.show(this.image)},hide:function(){O.moveOut(this.image)},clear:function(){this.image=null},isCached:function(t){return!!this.cache[t]},preload:function(e){t(new Image).load(function(t,e){return function(){e[t]=t}}(e,this.cache)).attr("src",e)},load:function(n,a,o){if("function"==typeof a&&(o=a,a=null),this.isIframe){var r="if"+(new Date).getTime(),s=this.image=t("<iframe>",{src:n,frameborder:0,id:r,allowfullscreen:!0,css:{visibility:"hidden"}})[0];return a&&t(s).css(a),t(this.container).find("iframe,img").remove(),this.container.appendChild(this.image),t("#"+r).load(function(i,n){return function(){e.setTimeout(function(){t(i.image).css("visibility","visible"),"function"==typeof n&&n.call(i,i)},10)}}(this,o)),this.container}this.image=new Image,i.IE8&&t(this.image).css("filter","inherit");var c=!1,l=!1,h=t(this.container),u=t(this.image),p=function(){c?d?t(this).attr("src",d):i.raise("Image not found: "+n):(c=!0,e.setTimeout(function(t,e){return function(){t.attr("src",e+(e.indexOf("?")>-1?"&":"?")+O.timestamp())}}(t(this),n),50))},f=function(n,o,r){return function(){var s=function(){t(this).off("load"),n.original=a||{height:this.height,width:this.width},i.HAS3D&&(this.style.MozTransform=this.style.webkitTransform="translate3d(0,0,0)"),h.append(this),n.cache[r]=r,"function"==typeof o&&e.setTimeout(function(){o.call(n,n)
},1)};this.width&&this.height?s.call(this):!function(e){O.wait({until:function(){return e.width&&e.height},success:function(){s.call(e)},error:function(){l?i.raise("Could not extract width/height from image: "+e.src+". Traced measures: width:"+e.width+"px, height: "+e.height+"px."):(t(new Image).load(f).attr("src",e.src),l=!0)},timeout:100})}(this)}}(this,o,n);return h.find("iframe,img").remove(),u.css("display","block"),O.hide(this.image),t.each("minWidth minHeight maxWidth maxHeight".split(" "),function(t,e){u.css(e,/min/.test(e)?"0":"none")}),u.load(f).on("error",p).attr("src",n),this.container},scale:function(e){var a=this;if(e=t.extend({width:0,height:0,min:n,max:n,margin:0,complete:v,position:"center",crop:!1,canvas:!1,iframelimit:n},e),this.isIframe){var o,r,s=e.width,c=e.height;if(e.iframelimit){var l=m.min(e.iframelimit/s,e.iframelimit/c);1>l?(o=s*l,r=c*l,t(this.image).css({top:c/2-r/2,left:s/2-o/2,position:"absolute"})):t(this.image).css({top:0,left:0})}t(this.image).width(o||s).height(r||c).removeAttr("width").removeAttr("height"),t(this.container).width(s).height(c),e.complete.call(a,a);try{this.image.contentWindow&&t(this.image.contentWindow).trigger("resize")}catch(h){}return this.container}if(!this.image)return this.container;var u,d,p,f=t(a.container);return O.wait({until:function(){return u=e.width||f.width()||O.parseValue(f.css("width")),d=e.height||f.height()||O.parseValue(f.css("height")),u&&d},success:function(){var i=(u-2*e.margin)/a.original.width,n=(d-2*e.margin)/a.original.height,o=m.min(i,n),r=m.max(i,n),s={"true":r,width:i,height:n,"false":o,landscape:a.original.width>a.original.height?r:o,portrait:a.original.width<a.original.height?r:o},c=s[e.crop.toString()],l="";e.max&&(c=m.min(e.max,c)),e.min&&(c=m.max(e.min,c)),t.each(["width","height"],function(e,i){t(a.image)[i](a[i]=a.image[i]=m.round(a.original[i]*c))}),t(a.container).width(u).height(d),e.canvas&&P&&(P.elem.width=a.width,P.elem.height=a.height,l=a.image.src+":"+a.width+"x"+a.height,a.image.src=P.cache[l]||function(t){P.context.drawImage(a.image,0,0,a.original.width*c,a.original.height*c);try{return p=P.elem.toDataURL(),P.length+=p.length,P.cache[t]=p,p}catch(e){return a.image.src}}(l));var h={},f={},g=function(e,i,n){var o=0;if(/\%/.test(e)){var r=parseInt(e,10)/100,s=a.image[i]||t(a.image)[i]();o=m.ceil(-1*s*r+n*r)}else o=O.parseValue(e);return o},v={top:{top:0},left:{left:0},right:{left:"100%"},bottom:{top:"100%"}};t.each(e.position.toLowerCase().split(" "),function(t,e){"center"===e&&(e="50%"),h[t?"top":"left"]=e}),t.each(h,function(e,i){v.hasOwnProperty(i)&&t.extend(f,v[i])}),h=h.top?t.extend(h,f):f,h=t.extend({top:"50%",left:"50%"},h),t(a.image).css({position:"absolute",top:g(h.top,"height",d),left:g(h.left,"width",u)}),a.show(),a.ready=!0,e.complete.call(a,a)},error:function(){i.raise("Could not scale image: "+a.image.src)},timeout:1e3}),this}},t.extend(t.easing,{galleria:function(t,e,i,n,a){return(e/=a/2)<1?n/2*e*e*e+i:n/2*((e-=2)*e*e+2)+i},galleriaIn:function(t,e,i,n,a){return n*(e/=a)*e+i},galleriaOut:function(t,e,i,n,a){return-n*(e/=a)*(e-2)+i}}),i.Finger=function(){var n=(m.abs,i.HAS3D=function(){var e,i,n=a.createElement("p"),o=["webkit","O","ms","Moz",""],r=0,s="transform";for(b().html.insertBefore(n,null);o[r];r++)i=o[r]?o[r]+"Transform":s,void 0!==n.style[i]&&(n.style[i]="translate3d(1px,1px,1px)",e=t(n).css(o[r]?"-"+o[r].toLowerCase()+"-"+s:s));return b().html.removeChild(n),void 0!==e&&e.length>0&&"none"!==e}()),r=function(){var t="RequestAnimationFrame";return e.requestAnimationFrame||e["webkit"+t]||e["moz"+t]||e["o"+t]||e["ms"+t]||function(t){e.setTimeout(t,1e3/60)}}(),s=function(i,a){if(this.config={start:0,duration:500,onchange:function(){},oncomplete:function(){},easing:function(t,e,i,n,a){return-n*((e=e/a-1)*e*e*e-1)+i}},this.easeout=function(t,e,i,n,a){return n*((e=e/a-1)*e*e*e*e+1)+i},i.children.length){var o=this;t.extend(this.config,a),this.elem=i,this.child=i.children[0],this.to=this.pos=0,this.touching=!1,this.start={},this.index=this.config.start,this.anim=0,this.easing=this.config.easing,n||(this.child.style.position="absolute",this.elem.style.position="relative"),t.each(["ontouchstart","ontouchmove","ontouchend","setup"],function(t,e){o[e]=function(t){return function(){t.apply(o,arguments)}}(o[e])}),this.setX=function(){var t=o.child.style;return n?void(t.MozTransform=t.webkitTransform="translate3d("+o.pos+"px,0,0)"):void(t.left=o.pos+"px")},t(i).on("touchstart",this.ontouchstart),t(e).on("resize",this.setup),t(e).on("orientationchange",this.setup),this.setup(),function s(){r(s),o.loop.call(o)}()}};return s.prototype={constructor:s,setup:function(){this.width=t(this.elem).width(),this.length=m.ceil(t(this.child).width()/this.width),0!==this.index&&(this.index=m.max(0,m.min(this.index,this.length-1)),this.pos=this.to=-this.width*this.index)},setPosition:function(t){this.pos=t,this.to=t},ontouchstart:function(t){var e=t.originalEvent.touches;this.start={pageX:e[0].pageX,pageY:e[0].pageY,time:+new Date},this.isScrolling=null,this.touching=!0,this.deltaX=0,o.on("touchmove",this.ontouchmove),o.on("touchend",this.ontouchend)},ontouchmove:function(t){var e=t.originalEvent.touches;e&&e.length>1||t.scale&&1!==t.scale||(this.deltaX=e[0].pageX-this.start.pageX,null===this.isScrolling&&(this.isScrolling=!!(this.isScrolling||m.abs(this.deltaX)<m.abs(e[0].pageY-this.start.pageY))),this.isScrolling||(t.preventDefault(),this.deltaX/=!this.index&&this.deltaX>0||this.index==this.length-1&&this.deltaX<0?m.abs(this.deltaX)/this.width+1.8:1,this.to=this.deltaX-this.index*this.width),t.stopPropagation())},ontouchend:function(){this.touching=!1;var t=+new Date-this.start.time<250&&m.abs(this.deltaX)>40||m.abs(this.deltaX)>this.width/2,e=!this.index&&this.deltaX>0||this.index==this.length-1&&this.deltaX<0;this.isScrolling||this.show(this.index+(t&&!e?this.deltaX<0?1:-1:0)),o.off("touchmove",this.ontouchmove),o.off("touchend",this.ontouchend)},show:function(t){t!=this.index?this.config.onchange.call(this,t):this.to=-(t*this.width)},moveTo:function(t){t!=this.index&&(this.pos=this.to=-(t*this.width),this.index=t)},loop:function(){var t=this.to-this.pos,e=1;if(this.width&&t&&(e=m.max(.5,m.min(1.5,m.abs(t/this.width)))),this.touching||m.abs(t)<=1)this.pos=this.to,t=0,this.anim&&!this.touching&&this.config.oncomplete(this.index),this.anim=0,this.easing=this.config.easing;else{this.anim||(this.anim={start:this.pos,time:+new Date,distance:t,factor:e,destination:this.to});var i=+new Date-this.anim.time,n=this.config.duration*this.anim.factor;if(i>n||this.anim.destination!=this.to)return this.anim=0,void(this.easing=this.easeout);this.pos=this.easing(null,i,this.anim.start,this.anim.distance,n)}this.setX()}},s}(),t.fn.galleria=function(e){var n=this.selector;return t(this).length?this.each(function(){t.data(this,"galleria")&&(t.data(this,"galleria").destroy(),t(this).find("*").hide()),t.data(this,"galleria",(new i).init(this,e))}):(t(function(){t(n).length?t(n).galleria(e):i.utils.wait({until:function(){return t(n).length},success:function(){t(n).galleria(e)},error:function(){i.raise('Init failed: Galleria could not find the element "'+n+'".')},timeout:5e3})}),this)},"object"==typeof module&&module&&"object"==typeof module.exports?module.exports=i:(e.Galleria=i,"function"==typeof define&&define.amd&&define("galleria",["jquery"],function(){return i}))}(jQuery,this);;
(function ($) {
	function jsonStreamFactory() {
		this.index = -1;
		this._jsonStream = jsonStream;
		this._allstream = new Object();

		this.createStream = function( options ) {
			this.index++;
			this._allstream[ this.index ] = new this._jsonStream( this, options );
			return this._allstream[ this.index ];
		}

		this.response = function( index, data ) {
			if ( this._allstream[ index ] instanceof jsonStream ) {
				this._allstream[ index ].response( data );
			}
		}

		this.destroy = function( index ) {
			delete this._allstream[ index ];
		}

		this.numInstance = function() {
			if ( typeof Object.keys != 'undefined' )
				return Object.keys( this._allstream ).length;
			var len = this._allstream.length ? --this._allstream.length : -1;
			for (var k in this._allstream )
				len++;
			return len;
		}
	}

	function jsonStream( factory, opt ) {
		this.factory = factory;
		this.opt = opt || {};
		this.key = this.factory.index;
		this.url = this.opt.url;

		this.iframe = null;
		this.loading = false;

		this._onsuccess = function( data ){ return data; }
		this._onfail = function( ){ return 'failed'; }

		this.init = function() {
			this.iframe = $('<iframe width="0" height="0" style="position: absolute; top: -1000px; overflow: hidden;"></iframe>');
			$( 'body' ).prepend( this.iframe );
			this.startStreaming()
		}

		this.response = function( respJson ) {
			var obj = JSON && JSON.parse( respJson ) || $.parseJSON( respJson );
			this._onsuccess( obj );
		}

		this.done = function( fn ) {
			this._onsuccess = fn;
		}

		this.startStreaming = function() {
			var that = this;
			if ( this.loading ) {
				this._onfail();
				return;
			}
			this.loading = true;
			this.url += "&_uniquekey=" + this.key + "&_jsont=" + new Date().getTime();
			$( this.iframe ).on('load', function(){ that.completed(); });
			$( this.iframe ).attr( 'src', this.url) ;
		}

		// evento che viene chiamato quando finisce lo streaming
		this.completed = function() {
			this.iframe.remove();
			this.loading = false;
			this.destroy();
		}

		this.fail = function( fn ) {
			this._onfail = fn;
		}

		this.destroy = function() {
			this.factory.destroy( this.key );
		}

		this.init();
	}
	_jsonStreamFactory = new jsonStreamFactory();
})(jQuery);